@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCS $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2.
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -42,6 +42,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <pqAnimationPanel.h>
 #include <pqAnimationViewWidget.h>
 #include <pqApplicationCore.h>
+#include <pqColorScaleToolbar.h>
 #include <pqComparativeVisPanel.h>
 //#include <pqLookmarkToolbar.h>
 #include <pqMainWindowCore.h>
@@ -92,7 +93,7 @@ public:
     ToolbarsMenu(0)
   {
   }
-  
+
   ~pqImplementation()
   {
     delete this->ViewMenu;
@@ -113,32 +114,56 @@ public:
   QLineEdit* CurrentTimeWidget;
   QSpinBox* CurrentTimeIndexWidget;
   QPointer<pqServer> ActiveServer;
+  QString DocumentationDir;
 };
 
 //////////////////////////////////////////////////////////////////////////////
 // MainWindow
-
+#include <QTreeView>
+#include "pqSILModel.h"
+#include "vtkGraphReader.h"
+#include "vtkExodusIIReaderParser.h"
+#include "vtkMutableDirectedGraph.h"
 MainWindow::MainWindow() :
   Implementation(new pqImplementation(this))
 {
+//  const char* filename = "/home/utkarsh/Kitware/Datasets/Exodus/DART_Examples/output/artifact.dta";
+//
+//  vtkExodusIIReaderParser* parser = vtkExodusIIReaderParser::New();
+//  parser->Go(filename);
+//  pqSILModel* model = new pqSILModel();
+//  model->update(parser->GetSIL());
+//  parser->Delete();
+//
+//  for (int cc=0; cc < 3; cc++)
+//    {
+//    QTreeView* treeView = new QTreeView();
+//    treeView->setModel(model);
+//    treeView->setRootIndex(model->hierarchies()[cc]);
+//    treeView->show(); 
+//    treeView->expandAll();
+//    }
+//
   this->Implementation->UI.setupUi(this);
 
   this->insertToolBarBreak(this->Implementation->UI.variableToolbar);
   this->insertToolBarBreak(this->Implementation->UI.commonFilters);
 
   this->setIconSize(QSize(24, 24));
-  
+
   this->Implementation->RecentFilesMenu = new
     pqRecentFilesMenu(*this->Implementation->UI.menuRecentFiles, this);
   QObject::connect(this->Implementation->RecentFilesMenu,
     SIGNAL(serverConnectFailed()),
     &this->Implementation->Core,
     SLOT(makeDefaultConnectionIfNoneExists()));
-  
-  this->Implementation->ViewMenu = 
+
+  this->Implementation->ViewMenu =
     new pqViewMenu(*this->Implementation->UI.menuView, this);
-  this->Implementation->ToolbarsMenu = 
+  this->Implementation->ToolbarsMenu =
     new pqViewMenu(*this->Implementation->UI.menuToolbars);
+  this->Implementation->Core.setDockWindowMenu(this->Implementation->ViewMenu);
+  this->Implementation->Core.setToolbarMenu(this->Implementation->ToolbarsMenu);
 
   this->setWindowTitle(
     QString("Kitware ParaView %1").arg(PARAVIEW_VERSION_FULL));
@@ -187,32 +212,32 @@ MainWindow::MainWindow() :
     this->Implementation->UI.actionFileSaveAnimation,
     SLOT(setEnabled(bool)));
 
-  connect(this->Implementation->UI.actionFileSaveGeometry, SIGNAL(triggered()), 
+  connect(this->Implementation->UI.actionFileSaveGeometry, SIGNAL(triggered()),
     &this->Implementation->Core, SLOT(onSaveGeometry()));
   connect(&this->Implementation->Core,
     SIGNAL(enableFileSaveGeometry(bool)),
     this->Implementation->UI.actionFileSaveGeometry,
     SLOT(setEnabled(bool)));
 
-  connect(this->Implementation->UI.actionFileExit, SIGNAL(triggered()), 
-    pqApplicationCore::instance(), SLOT(quit()));  
+  connect(this->Implementation->UI.actionFileExit, SIGNAL(triggered()),
+    pqApplicationCore::instance(), SLOT(quit()));
 
   pqUndoStack* undoStack = this->Implementation->Core.getApplicationUndoStack();
 
   connect(this->Implementation->UI.actionEditUndo,
     SIGNAL(triggered()), undoStack, SLOT(undo()));
   connect(undoStack,
-    SIGNAL(canUndoChanged(bool)), 
+    SIGNAL(canUndoChanged(bool)),
     this->Implementation->UI.actionEditUndo, SLOT(setEnabled(bool)));
   connect(undoStack,
     SIGNAL(undoLabelChanged(const QString&)), this, SLOT(onUndoLabel(const QString&)));
 
   connect(this->Implementation->UI.actionEditViewSettings,
     SIGNAL(triggered()), &this->Implementation->Core, SLOT(onEditViewSettings()));
-  
+
   connect(this->Implementation->UI.actionEditSettings,
     SIGNAL(triggered()), &this->Implementation->Core, SLOT(onEditSettings()));
-    
+
   connect(this->Implementation->UI.actionEditRedo,
     SIGNAL(triggered()), undoStack, SLOT(redo()));
   connect(undoStack,
@@ -230,7 +255,7 @@ MainWindow::MainWindow() :
     SIGNAL(cameraUndoLabel(const QString&)),
     this,
     SLOT(onCameraUndoLabel(const QString&)));
-    
+
   connect(this->Implementation->UI.actionEditCameraRedo,
     SIGNAL(triggered()), &this->Implementation->Core, SLOT(onEditCameraRedo()));
   connect(&this->Implementation->Core,
@@ -293,7 +318,7 @@ MainWindow::MainWindow() :
 
   connect(this->Implementation->UI.actionToolsManageCustomFilters,
     SIGNAL(triggered()), &this->Implementation->Core, SLOT(onToolsManageCustomFilters()));
-  
+
   connect(this->Implementation->UI.actionToolsManageLinks,
     SIGNAL(triggered()), &this->Implementation->Core, SLOT(onToolsManageLinks()));
 
@@ -328,29 +353,28 @@ MainWindow::MainWindow() :
     SIGNAL(triggered()), this, SLOT(onHelpHelp()));
 
   connect(this->Implementation->UI.actionHelpEnableTooltips,
-    SIGNAL(triggered(bool)), &this->Implementation->Core, SLOT(onHelpEnableTooltips(bool)));
-  this->Implementation->Core.onHelpEnableTooltips(
-    this->Implementation->UI.actionHelpEnableTooltips->isChecked());
+    SIGNAL(triggered(bool)), 
+    &this->Implementation->Core, SLOT(onHelpEnableTooltips(bool)));
+  connect(&this->Implementation->Core, SIGNAL(enableTooltips(bool)),
+    this->Implementation->UI.actionHelpEnableTooltips,
+    SLOT(setChecked(bool)));
 
-  connect(this->Implementation->UI.actionVCRPlay, SIGNAL(triggered()), 
+  connect(this->Implementation->UI.actionVCRPlay, SIGNAL(triggered()),
     &this->Implementation->Core.VCRController(), SLOT(onPlay()));
-  
-  connect(this->Implementation->UI.actionVCRPause, SIGNAL(triggered()), 
-    &this->Implementation->Core.VCRController(), SLOT(onPause()));
-  
+
   connect(this->Implementation->UI.actionVCRFirstFrame,
     SIGNAL(triggered()), &this->Implementation->Core.VCRController(), SLOT(onFirstFrame()));
-    
+
   connect(this->Implementation->UI.actionVCRPreviousFrame,
     SIGNAL(triggered()), &this->Implementation->Core.VCRController(), SLOT(onPreviousFrame()));
-    
+
   connect(this->Implementation->UI.actionVCRNextFrame,
     SIGNAL(triggered()), &this->Implementation->Core.VCRController(), SLOT(onNextFrame()));
-    
+
   connect(this->Implementation->UI.actionVCRLastFrame,
     SIGNAL(triggered()), &this->Implementation->Core.VCRController(), SLOT(onLastFrame()));
- 
-  connect(this->Implementation->UI.actionVCRLoop, SIGNAL(toggled(bool)), 
+
+  connect(this->Implementation->UI.actionVCRLoop, SIGNAL(toggled(bool)),
     &this->Implementation->Core.VCRController(), SLOT(onLoop(bool)));
 
   pqVCRController* vcrcontroller = &this->Implementation->Core.VCRController();
@@ -372,19 +396,21 @@ MainWindow::MainWindow() :
     this->Implementation->UI.actionVCRLoop, SLOT(setChecked(bool)));
   connect(vcrcontroller, SIGNAL(playing(bool)),
     this, SLOT(onPlaying(bool)));
-  connect(vcrcontroller, SIGNAL(playing(bool)),
-    this->Implementation->UI.actionVCRPause, SLOT(setEnabled(bool)));
-  this->Implementation->UI.actionVCRPause->setVisible(false);
 
-  pqProgressManager* progress_manager = 
+  pqProgressManager* progress_manager =
     pqApplicationCore::instance()->getProgressManager();
   progress_manager->addNonBlockableObject(
     this->Implementation->UI.VCRToolbar);
-  
-  // Create Selection Shortcut.
-  QShortcut *s=new QShortcut(QKeySequence(tr("S")),&this->Implementation->Core.multiViewManager());
-  QObject::connect(s, SIGNAL(activated()),
-    this, SLOT(onSelectionShortcut()));
+
+  QShortcut *ctrlSpace = new QShortcut(Qt::CTRL + Qt::Key_Space,
+    &this->Implementation->Core.multiViewManager());
+  QObject::connect(ctrlSpace, SIGNAL(activated()),
+    this, SLOT(onQuickLaunchShortcut()));
+
+  QShortcut *altSpace = new QShortcut(Qt::ALT + Qt::Key_Space,
+    &this->Implementation->Core.multiViewManager());
+  QObject::connect(altSpace, SIGNAL(activated()),
+    this, SLOT(onQuickLaunchShortcut()));
 
   connect(
     &this->Implementation->Core, SIGNAL(enableFileSaveScreenshot(bool)),
@@ -400,14 +426,14 @@ MainWindow::MainWindow() :
     this->Implementation->UI.actionPositiveY, SLOT(setEnabled(bool)));
   connect(
     &this->Implementation->Core, SIGNAL(enableFileSaveScreenshot(bool)),
-    this->Implementation->UI.actionNegativeY, SLOT(setEnabled(bool)));  
+    this->Implementation->UI.actionNegativeY, SLOT(setEnabled(bool)));
   connect(
     &this->Implementation->Core, SIGNAL(enableFileSaveScreenshot(bool)),
     this->Implementation->UI.actionPositiveZ, SLOT(setEnabled(bool)));
   connect(
     &this->Implementation->Core, SIGNAL(enableFileSaveScreenshot(bool)),
     this->Implementation->UI.actionNegativeZ, SLOT(setEnabled(bool)));
-  
+
   connect(
     this->Implementation->UI.actionResetCamera, SIGNAL(triggered()),
     &this->Implementation->Core, SLOT(resetCamera()));
@@ -422,7 +448,7 @@ MainWindow::MainWindow() :
     &this->Implementation->Core, SLOT(resetViewDirectionPosY()));
   connect(
     this->Implementation->UI.actionNegativeY, SIGNAL(triggered()),
-    &this->Implementation->Core, SLOT(resetViewDirectionNegY()));  
+    &this->Implementation->Core, SLOT(resetViewDirectionNegY()));
   connect(
     this->Implementation->UI.actionPositiveZ, SIGNAL(triggered()),
     &this->Implementation->Core, SLOT(resetViewDirectionPosZ()));
@@ -439,6 +465,7 @@ MainWindow::MainWindow() :
  // modeGroup->addAction(this->Implementation->UI.actionSelect_Thresholds);
   modeGroup->addAction(this->Implementation->UI.actionSelect_Frustum);
   modeGroup->addAction(this->Implementation->UI.actionSelectFrustumPoints);
+  modeGroup->addAction(this->Implementation->UI.actionSelect_Block);
 
   this->Implementation->Core.setupVariableToolbar(
     this->Implementation->UI.variableToolbar);
@@ -446,7 +473,7 @@ MainWindow::MainWindow() :
     {
     if (a == this->Implementation->UI.actionScalarBarVisibility)
       {
-      // actionScalarBarVisibility's enabled state is managed by 
+      // actionScalarBarVisibility's enabled state is managed by
       // pqScalarBarVisibilityAdaptor.
       continue;
       }
@@ -498,16 +525,22 @@ MainWindow::MainWindow() :
   pqProxyTabWidget* const proxyTab =
     this->Implementation->Core.setupProxyTabWidget(
       this->Implementation->UI.objectInspectorDock);
-      
+  
   QObject::connect(
     proxyTab->getObjectInspector(),
-    SIGNAL(preaccept()), 
+    SIGNAL(helpRequested(QString)),
+    this,
+    SLOT(showHelpForProxy(QString)));
+
+  QObject::connect(
+    proxyTab->getObjectInspector(),
+    SIGNAL(preaccept()),
     this,
     SLOT(onPreAccept()));
 
   QObject::connect(
     proxyTab->getObjectInspector(),
-    SIGNAL(postaccept()), 
+    SIGNAL(postaccept()),
     this,
     SLOT(onPostAccept()));
 
@@ -519,31 +552,31 @@ MainWindow::MainWindow() :
 
   this->Implementation->Core.setupStatisticsView(
     this->Implementation->UI.statisticsViewDock);
-    
+
 
   this->Implementation->Core.setupSelectionInspector(
     this->Implementation->UI.selectionInspectorDock);
-    
+
   this->Implementation->Core.setupLookmarkBrowser(
     this->Implementation->UI.lookmarkBrowserDock);
 
   this->Implementation->Core.setupLookmarkInspector(
     this->Implementation->UI.lookmarkInspectorDock);
 
-  pqAnimationPanel* animation_panel = 
+  pqAnimationPanel* animation_panel =
     this->Implementation->Core.setupAnimationPanel(
     this->Implementation->UI.animationPanelDock);
   animation_panel->setCurrentTimeToolbar(
     this->Implementation->UI.currentTimeToolbar);
 
-  pqComparativeVisPanel* cv_panel = 
+  pqComparativeVisPanel* cv_panel =
     new pqComparativeVisPanel(
     this->Implementation->UI.comparativePanelDock);
   this->Implementation->UI.comparativePanelDock->setWidget(cv_panel);
 
   this->Implementation->Core.setupAnimationView(
     this->Implementation->UI.animationViewDock);
-  
+
   // Setup the view menu ...
   this->Implementation->ToolbarsMenu->addWidget(
     this->Implementation->UI.variableToolbar,
@@ -553,6 +586,10 @@ MainWindow::MainWindow() :
     this->Implementation->UI.cameraToolbar,
     this->Implementation->UI.cameraToolbar->windowTitle());
 
+  this->Implementation->ToolbarsMenu->addWidget(
+    this->Implementation->UI.axesToolbar,
+    this->Implementation->UI.axesToolbar->windowTitle());
+
   this->Implementation->ToolbarsMenu->addWidget(
     this->Implementation->UI.commonFilters,
     this->Implementation->UI.commonFilters->windowTitle());
@@ -593,7 +630,7 @@ MainWindow::MainWindow() :
   this->Implementation->ViewMenu->addWidget(
     this->Implementation->UI.animationPanelDock,
     this->Implementation->UI.animationPanelDock->windowTitle());
-  
+
   this->Implementation->ViewMenu->addWidget(
     this->Implementation->UI.animationViewDock,
     this->Implementation->UI.animationViewDock->windowTitle());
@@ -625,7 +662,7 @@ MainWindow::MainWindow() :
   this->Implementation->ViewMenu->addWidget(
     this->Implementation->UI.statisticsViewDock,
     this->Implementation->UI.statisticsViewDock->windowTitle());
-    
+
   // Setup the multiview render window ...
   this->setCentralWidget(&this->Implementation->Core.multiViewManager());
 
@@ -658,7 +695,7 @@ MainWindow::MainWindow() :
 
   // Now that we're ready, initialize everything ...
   this->Implementation->Core.initializeStates();
-  
+
   this->Implementation->UI.actionEditUndo->setEnabled(
     undoStack->canUndo());
   this->Implementation->UI.actionEditRedo->setEnabled(
@@ -673,17 +710,23 @@ MainWindow::MainWindow() :
     SIGNAL(representationChanged(pqDataRepresentation*, pqView*)),
     sbva, SLOT(setActiveRepresentation(pqDataRepresentation*)));
 
+  // Set up the color map tool buttons.
+  pqColorScaleToolbar *colorScale =
+      this->Implementation->Core.getColorScaleEditorManager();
+  colorScale->setColorAction(this->Implementation->UI.actionEditColorMap);
+  colorScale->setRescaleAction(this->Implementation->UI.actionResetRange);
+
   // Set up Center Axes toolbar.
   QObject::connect(
     this->Implementation->UI.actionShowCenterAxes, SIGNAL(toggled(bool)),
     &this->Implementation->Core, SLOT(setCenterAxesVisibility(bool)));
   QObject::connect(
     this->Implementation->UI.actionResetCenter, SIGNAL(triggered()),
-    &this->Implementation->Core, 
+    &this->Implementation->Core,
     SLOT(resetCenterOfRotationToCenterOfCurrentData()));
   QObject::connect(
     this->Implementation->UI.actionPickCenter, SIGNAL(toggled(bool)),
-    &this->Implementation->Core, 
+    &this->Implementation->Core,
     SLOT(pickCenterOfRotation(bool)));
 
   QObject::connect(
@@ -698,7 +741,7 @@ MainWindow::MainWindow() :
   QObject::connect(
     &this->Implementation->Core, SIGNAL(pickingCenter(bool)),
     this->Implementation->UI.actionPickCenter, SLOT(setChecked(bool)));
-  
+
   connect(this->Implementation->UI.actionManage_Plugins,
     SIGNAL(triggered()), &this->Implementation->Core, SLOT(onManagePlugins()));
 
@@ -707,41 +750,53 @@ MainWindow::MainWindow() :
   QObject::connect(
     this->Implementation->UI.actionMoveMode, SIGNAL(triggered()),
     this->Implementation->Core.renderViewSelectionHelper(), SLOT(endSelection()));
-  
+
   // 3d Selection Modes
   QObject::connect(
-    this->Implementation->Core.renderViewSelectionHelper(), SIGNAL(enabled(bool)), 
+    this->Implementation->Core.renderViewSelectionHelper(), 
+    SIGNAL(enableSurfaceSelection(bool)),
     this->Implementation->UI.actionSelectionMode, SLOT(setEnabled(bool)));
   QObject::connect(
-    this->Implementation->Core.renderViewSelectionHelper(), SIGNAL(enabled(bool)), 
+    this->Implementation->Core.renderViewSelectionHelper(), 
+    SIGNAL(enableSurfacePointsSelection(bool)),
     this->Implementation->UI.actionSelectSurfacePoints, SLOT(setEnabled(bool)));
   QObject::connect(
-    this->Implementation->Core.renderViewSelectionHelper(), SIGNAL(enabled(bool)), 
+    this->Implementation->Core.renderViewSelectionHelper(), 
+    SIGNAL(enableFrustumSelection(bool)),
     this->Implementation->UI.actionSelect_Frustum, SLOT(setEnabled(bool)));
   QObject::connect(
-    this->Implementation->Core.renderViewSelectionHelper(), SIGNAL(enabled(bool)), 
+    this->Implementation->Core.renderViewSelectionHelper(), 
+    SIGNAL(enableFrustumPointSelection(bool)),
     this->Implementation->UI.actionSelectFrustumPoints, SLOT(setEnabled(bool)));
-
-
-  QObject::connect(
-    this->Implementation->UI.actionSelectionMode, SIGNAL(triggered()), 
-    this->Implementation->Core.renderViewSelectionHelper(), SLOT(beginSelection()));  
-  QObject::connect(
-    this->Implementation->UI.actionSelectSurfacePoints, SIGNAL(triggered()), 
-    this->Implementation->Core.renderViewSelectionHelper(), SLOT(beginSurfacePointsSelection()));  
-  QObject::connect(
-    this->Implementation->UI.actionSelect_Frustum, SIGNAL(triggered()), 
-    this->Implementation->Core.renderViewSelectionHelper(), SLOT(beginFrustumSelection()));
-  QObject::connect(
-    this->Implementation->UI.actionSelectFrustumPoints, SIGNAL(triggered()), 
-    this->Implementation->Core.renderViewSelectionHelper(), SLOT(beginFrustumPointsSelection()));
-
   QObject::connect(
     this->Implementation->Core.renderViewSelectionHelper(), 
+    SIGNAL(enableBlockSelection(bool)),
+    this->Implementation->UI.actionSelect_Block, SLOT(setEnabled(bool)));
+
+
+  QObject::connect(
+    this->Implementation->UI.actionSelectionMode, SIGNAL(triggered()),
+    this->Implementation->Core.renderViewSelectionHelper(), SLOT(beginSurfaceSelection()));
+  QObject::connect(
+    this->Implementation->UI.actionSelectSurfacePoints, SIGNAL(triggered()),
+    this->Implementation->Core.renderViewSelectionHelper(), SLOT(beginSurfacePointsSelection()));
+  QObject::connect(
+    this->Implementation->UI.actionSelect_Frustum, SIGNAL(triggered()),
+    this->Implementation->Core.renderViewSelectionHelper(), SLOT(beginFrustumSelection()));
+  QObject::connect(
+    this->Implementation->UI.actionSelectFrustumPoints, SIGNAL(triggered()),
+    this->Implementation->Core.renderViewSelectionHelper(), SLOT(beginFrustumPointsSelection()));
+  QObject::connect(
+    this->Implementation->UI.actionSelect_Block, SIGNAL(triggered()),
+    this->Implementation->Core.renderViewSelectionHelper(), 
+    SLOT(beginBlockSelection()));
+
+  QObject::connect(
+    this->Implementation->Core.renderViewSelectionHelper(),
     SIGNAL(selectionModeChanged(int)),
     this, SLOT(onSelectionModeChanged(int)));
   QObject::connect(
-    this->Implementation->Core.renderViewSelectionHelper(), 
+    this->Implementation->Core.renderViewSelectionHelper(),
     SIGNAL(interactionModeChanged(bool)),
     this->Implementation->UI.actionMoveMode, SLOT(setChecked(bool)));
 
@@ -751,6 +806,13 @@ MainWindow::MainWindow() :
     SIGNAL(selectionFinished(int, int, int, int)),
     this->Implementation->Core.renderViewSelectionHelper(), SLOT(endSelection()));
 
+  QObject::connect(
+    this->Implementation->UI.actionExport, SIGNAL(triggered()),
+    &this->Implementation->Core, SLOT(onFileExport()));
+  QObject::connect(
+    &this->Implementation->Core, SIGNAL(enableExport(bool)),
+    this->Implementation->UI.actionExport, SLOT(setEnabled(bool)));
+
   // Restore the state of the window ...
   pqApplicationCore::instance()->settings()->restoreState("MainWindow", *this);
 
@@ -788,7 +850,7 @@ bool MainWindow::compareView(const QString& ReferenceImage,
   return this->Implementation->Core.compareView(
     ReferenceImage, Threshold, Output, TempDirectory);
 }
-  
+
 //-----------------------------------------------------------------------------
 void MainWindow::onUndoLabel(const QString& label)
 {
@@ -881,16 +943,13 @@ QString Locate(const QString& appName)
   return app_dir + QDir::separator() + appName;
 }
 
+
 //-----------------------------------------------------------------------------
-void MainWindow::onHelpHelp()
+void MainWindow::makeAssistant()
 {
   if(this->Implementation->AssistantClient)
     {
-    if(!this->Implementation->AssistantClient->isOpen())
-      {
-      this->Implementation->AssistantClient->openAssistant();
-      }
-      return;
+    return;
     }
 
   QString assistantExe;
@@ -907,7 +966,7 @@ void MainWindow::onHelpHelp()
   const char* binDir = "/";
   const char* binDir1 = "/";
 #endif
-  
+
   QString helper = QCoreApplication::applicationDirPath() +
     binDir + QString("pqClientDocFinder.txt");
   if(!QFile::exists(helper))
@@ -920,8 +979,11 @@ void MainWindow::onHelpHelp()
     QFile file(helper);
     if(file.open(QIODevice::ReadOnly))
       {
-      assistantExe = file.readLine().trimmed() + assistantName;
+      assistantExe = file.readLine().trimmed();
       profileFile = file.readLine().trimmed();
+      // CMake escapes spaces, we need to unescape those.
+      assistantExe.replace("\\ ", " ");
+      profileFile.replace("\\ ", " ");
       }
     }
 
@@ -936,14 +998,23 @@ void MainWindow::onHelpHelp()
     assistantExe = assistant;
     */
     }
+#if defined (__APPLE__)
+  // trim upto .app in case of Mac
+  int suffixIndex = assistantExe.indexOf(QRegExp(".app/Contents/MacOS/assistant$"));
+  if (suffixIndex != -1)
+    {
+    assistantExe = assistantExe.left(suffixIndex);
+    }
+#endif
 
-  this->Implementation->AssistantClient = 
+
+  this->Implementation->AssistantClient =
     new QAssistantClient(assistantExe, this);
   QObject::connect(this->Implementation->AssistantClient,
                    SIGNAL(error(const QString&)),
                    this,
                    SLOT(assistantError(const QString&)));
-  
+
   QStringList args;
   args.append(QString("-profile"));
 
@@ -971,15 +1042,27 @@ void MainWindow::onHelpHelp()
       this, "Help error", "Couldn't find"
       " pqClient.adp.\nTry setting the PARAVIEW_HELP environment variable which"
       " points to that file");
-    
+
     delete this->Implementation->AssistantClient;
     return;
     }
-  
+
+  QFileInfo fi(profileFile);
+  this->Implementation->DocumentationDir = fi.absolutePath();
+
   args.append(profileFile);
-  
+
   this->Implementation->AssistantClient->setArguments(args);
-  this->Implementation->AssistantClient->openAssistant();
+}
+
+//-----------------------------------------------------------------------------
+void MainWindow::onHelpHelp()
+{
+  this->makeAssistant();
+  if(this->Implementation->AssistantClient)
+    {
+    this->Implementation->AssistantClient->openAssistant();
+    }
 }
 
 //-----------------------------------------------------------------------------
@@ -988,12 +1071,6 @@ void MainWindow::assistantError(const QString& error)
   qCritical(error.toAscii().data());
 }
 
-//-----------------------------------------------------------------------------
-void MainWindow::onSelectionShortcut()
-{
-  this->Implementation->Core.renderViewSelectionHelper()->beginSelection();
-}
-
 //-----------------------------------------------------------------------------
 void MainWindow::onSelectionModeChanged(int mode)
 {
@@ -1015,6 +1092,10 @@ void MainWindow::onSelectionModeChanged(int mode)
       {
       this->Implementation->UI.actionSelectFrustumPoints->setChecked(true);
       }
+    else if (mode == pqRubberBandHelper::BLOCKS)
+      {
+      this->Implementation->UI.actionSelect_Block->setChecked(true);
+      }
     else // INTERACT
       {
       this->Implementation->UI.actionMoveMode->setChecked(true);
@@ -1077,9 +1158,29 @@ QVariant MainWindow::findToolBarActionsNotInMenus()
 //-----------------------------------------------------------------------------
 void MainWindow::onPlaying(bool playing)
 {
-  this->Implementation->UI.actionVCRPlay->setVisible(!playing);
-  this->Implementation->UI.actionVCRPause->setVisible(playing);
+  if(playing)
+    {
+    disconnect(this->Implementation->UI.actionVCRPlay, SIGNAL(triggered()),
+      &this->Implementation->Core.VCRController(), SLOT(onPlay()));
+    connect(this->Implementation->UI.actionVCRPlay, SIGNAL(triggered()),
+      &this->Implementation->Core.VCRController(), SLOT(onPause()));
+    this->Implementation->UI.actionVCRPlay->setIcon(
+      QIcon(":/pqWidgets/Icons/pqVcrPause24.png"));
+    this->Implementation->UI.actionVCRPlay->setText("Pa&use");
+    }
+  else
+    {
+    connect(this->Implementation->UI.actionVCRPlay, SIGNAL(triggered()),
+      &this->Implementation->Core.VCRController(), SLOT(onPlay()));
+    disconnect(this->Implementation->UI.actionVCRPlay, SIGNAL(triggered()),
+      &this->Implementation->Core.VCRController(), SLOT(onPause()));
+    this->Implementation->UI.actionVCRPlay->setIcon(
+      QIcon(":/pqWidgets/Icons/pqVcrPlay24.png"));
+    this->Implementation->UI.actionVCRPlay->setText("&Play");
+    }
+
   this->Implementation->Core.setSelectiveEnabledState(!playing);
+
 }
 
 //-----------------------------------------------------------------------------
@@ -1093,7 +1194,7 @@ void MainWindow::onAddCameraLink()
     }
   else
     {
-    QMessageBox::information(this, "Add Camera Link", 
+    QMessageBox::information(this, "Add Camera Link",
                              "No render module is active");
     }
 }
@@ -1117,3 +1218,25 @@ void MainWindow::setTimeRanges(double start, double end)
     QString("Last Frame (%1)").arg(end, 0, 'g'));
 }
 
+//-----------------------------------------------------------------------------
+void MainWindow::onQuickLaunchShortcut()
+{
+  this->Implementation->Core.quickLaunch();
+}
+
+//-----------------------------------------------------------------------------
+void MainWindow::showHelpForProxy(const QString& proxy)
+{
+  // make sure assistant is ready
+  this->makeAssistant();
+
+  if(this->Implementation->AssistantClient)
+    {
+    this->Implementation->AssistantClient->openAssistant();
+    QString page("%1/Documentation/%2.html");
+    page = page.arg(this->Implementation->DocumentationDir);
+    page = page.arg(proxy);
+    this->Implementation->AssistantClient->showPage(page);
+    }
+}
+
