@@ -19,6 +19,7 @@
 #include "pvTestDriverConfig.h"
 
 #include <vtksys/SystemTools.hxx>
+#include <vtksys/String.hxx>
 
 #if !defined(_WIN32) || defined(__CYGWIN__)
 # include <unistd.h>
@@ -122,6 +123,9 @@ void pvTestDriver::CollectConfiguredOptions()
   cerr << "Error VTK_MPI_NUMPROC_FLAG must be defined to run test if MPI is on.\n";
   return;
 # endif
+#ifdef VTK_MPI_PRENUMPROC_FLAGS
+  this->SeparateArguments(VTK_MPI_PRENUMPROC_FLAGS, this->MPIPreNumProcFlags);
+#endif
 # ifdef VTK_MPI_PREFLAGS
   this->SeparateArguments(VTK_MPI_PREFLAGS, this->MPIPreFlags);
 # endif
@@ -261,6 +265,13 @@ pvTestDriver::CreateCommandLine(vtksys_stl::vector<const char*>& commandLine,
   if(this->MPIRun.size() && type != CLIENT)
     {
     commandLine.push_back(this->MPIRun.c_str());
+    if (!this->TestTiledDisplay)
+      {
+      for (unsigned int i = 0; i < this->MPIPreNumProcFlags.size(); ++i)
+        {
+        commandLine.push_back(this->MPIPreNumProcFlags[i].c_str());
+        }
+      }
     commandLine.push_back(this->MPINumProcessFlag.c_str());
     commandLine.push_back(numProc);
     if (!this->TestTiledDisplay)
@@ -320,6 +331,9 @@ pvTestDriver::CreateCommandLine(vtksys_stl::vector<const char*>& commandLine,
     if(this->ReverseConnection && type != CLIENT)
       {
       commandLine.push_back("-rc");
+#ifdef PV_TEST_CLIENT
+      commandLine.push_back("-ch=" PV_TEST_CLIENT);
+#endif
       }
 
     if (!this->TestTiledDisplay)
@@ -456,7 +470,7 @@ int pvTestDriver::OutputStringHasError(const char* pname, vtkstd::string& output
   };
 
   const char* nonErrors[] = {
-    "Memcheck, a memory error detector for x86-linux",  //valgrind
+    "Memcheck, a memory error detector",  //valgrind
     "error in locking authority file",  //Ice-T
     "WARNING: Far depth failed sanity check, resetting.", //Ice-T
     0
@@ -510,6 +524,25 @@ int pvTestDriver::OutputStringHasError(const char* pname, vtkstd::string& output
 //----------------------------------------------------------------------------
 int pvTestDriver::Main(int argc, char* argv[])
 {
+
+#ifdef PV_TEST_INIT_COMMAND
+  // run user-specified commands before initialization.
+  // For example: "killall -9 rsh paraview;"
+  if(strlen(PV_TEST_INIT_COMMAND) > 0)
+    {
+    vtkstd::vector<vtksys::String> commands = vtksys::SystemTools::SplitString(
+      PV_TEST_INIT_COMMAND, ';');
+    for (unsigned int cc=0; cc < commands.size(); cc++)
+      {
+      vtkstd::string command = commands[cc];
+      if (command.size() > 0)
+        {
+        system(command.c_str());
+        }
+      }
+    }
+#endif
+
   // temporary hack
   // when pvserver supports -rc option then this can be removed
   // until then, ReverseConnection must be set before CollectConfiguredOptions
@@ -613,6 +646,17 @@ int pvTestDriver::Main(int argc, char* argv[])
       cerr << "CONNECT_TO_RS_DS_REMOTE_RENDER_SCRIPT must be specified."
            << endl;
 
+      VTK_CLEAN_PROCESSES;
+      return 1;
+#endif
+      }
+    else if (this->ReverseConnection)
+      {
+#ifdef CONNECT_TO_RS_DS_REVERSE_SCRIPT
+      vtkstd::string temp = CONNECT_TO_RS_DS_REVERSE_SCRIPT;
+      this->ClientPostFlags.push_back("--run-test-init=" + temp);
+#else
+      cerr << "CONNECT_TO_RS_DS_REVERSE_SCRIPT must be specified." << endl;
       VTK_CLEAN_PROCESSES;
       return 1;
 #endif
@@ -641,6 +685,17 @@ int pvTestDriver::Main(int argc, char* argv[])
            << endl;
       VTK_CLEAN_PROCESSES;
       return 1;
+#endif
+      }
+    else if (this->ReverseConnection)
+      {
+#ifdef CONNECT_TO_SERVER_REVERSE_SCRIPT
+      vtkstd::string temp = CONNECT_TO_SERVER_REVERSE_SCRIPT;
+      this->ClientPostFlags.push_back("--run-test-init=" + temp);
+#else
+      cerr << "CONNECT_TO_SERVER_SCRIPT must be specified." << endl;
+      VTK_CLEAN_PROCESSES;
+      return 1;
 #endif
       }
     else
