@@ -31,7 +31,6 @@
 
 #include <QBrush>
 #include <QDir>
-#include <QDomDocument>
 #include <QFileInfo>
 #include <QHBoxLayout>
 #include <QMap>
@@ -55,10 +54,11 @@
 #include <vtkSMInputProperty.h>
 #include <vtkSMProxyManager.h>
 #include <vtkSMSourceProxy.h>
-#include <vtkSMCompoundProxy.h>
 #include <vtkSphereSource.h>
 #include <vtkStreamingDemandDrivenPipeline.h>
 #include <vtkUnstructuredGrid.h>
+#include <vtkPVXMLParser.h>
+#include <vtkPVXMLElement.h>
 
 #include <vtkExodusReader.h>
 #include <vtkProcessModule.h>
@@ -143,7 +143,6 @@ struct LineChartAdapter::pqImplementation
     title->setPalette(titleColors);
     
     pqChartArea *area = this->Chart.getChartArea();
-    area->createAxis(pqChartAxis::Bottom);
     pqChartAxis *xAxis = area->getAxis(pqChartAxis::Bottom);
     pqChartAxisOptions *axisOptions = xAxis->getOptions();
     axisOptions->setGridColorType(pqChartAxisOptions::Specified);
@@ -156,7 +155,6 @@ struct LineChartAdapter::pqImplementation
     title->setFont(bold);
     this->Chart.setAxisTitle(pqChartAxis::Bottom, title);
     
-    area->createAxis(pqChartAxis::Left);
     pqChartAxis *yAxis = area->getAxis(pqChartAxis::Left);
     axisOptions = yAxis->getOptions();
     axisOptions->setGridColorType(pqChartAxisOptions::Specified);
@@ -169,7 +167,6 @@ struct LineChartAdapter::pqImplementation
     this->Chart.setAxisTitle(pqChartAxis::Left, title);
 
     this->LineChart = new pqLineChart(area);
-    this->LineChart->setAxes(xAxis, yAxis);
     this->LineChart->getOptions()->getGenerator()->setColorScheme(
       pqChartSeriesOptionsGenerator::WildFlower);
     area->addLayer(this->LineChart);
@@ -197,18 +194,6 @@ struct LineChartAdapter::pqImplementation
   {
     this->SourceProxy = Proxy;
     this->ExodusVariableName = QString();
-
-    // TODO: hack -- figure out how compound proxies really fit in
-    vtkSMCompoundProxy* cp = vtkSMCompoundProxy::SafeDownCast(Proxy);
-    if(cp)
-      {
-        Proxy = NULL;
-        for(int i=cp->GetNumberOfProxies(); Proxy == NULL && i>0; i--)
-          {
-          Proxy = vtkSMSourceProxy::SafeDownCast(cp->GetProxy(i-1));
-          }
-      }
-
     this->onInputChanged();
   }
   
@@ -1204,38 +1189,40 @@ void LineChartAdapter::loadSetup(const QStringList& files)
     {
     QFileInfo setup_file_info(files[i]);
     QFile file(files[i]);
-    QDomDocument xml_document;
-    xml_document.setContent(&file, false);
-
-    QDomElement xml_setup = xml_document.documentElement();
-    if(xml_setup.nodeName() != "setup")
+    QByteArray dat = file.readAll();
+    vtkSmartPointer<vtkPVXMLParser> parser = 
+      vtkSmartPointer<vtkPVXMLParser>::New();
+    parser->Parse(dat.data());
+    vtkPVXMLElement* element = parser->GetRootElement();
+    if(QString(element->GetName()) != "setup")
       {
       QMessageBox::warning(0, "Dobran-O-Viz Error:", files[i] + " is not a Dobran-O-Viz setup file");
       continue;
       }
 
-    for(QDomNode xml_file = xml_setup.firstChild(); !xml_file.isNull(); xml_file = xml_file.nextSibling())
+    int num = element->GetNumberOfNestedElements();
+    for(int j=0; j<num; j++)
       {
-      if(!xml_file.isElement())
-        continue;
+      vtkPVXMLElement* xml_file = element->GetNestedElement(j);
 
-      QFileInfo file_info(xml_file.toElement().text());
+      QString filename = xml_file->GetCharacterData();
+      QFileInfo file_info(filename);
       if(file_info.isRelative())
-        file_info = setup_file_info.absoluteDir().absoluteFilePath(xml_file.toElement().text());
+        file_info = setup_file_info.absoluteDir().absoluteFilePath(filename);
       
-      if(xml_file.nodeName() == "experimental")
+      if(QString(xml_file->GetName()) == "experimental")
         {
         loadExperimentalData(file_info.absoluteFilePath());
         }
-      else if(xml_file.nodeName() == "experimental_uncertainty")
+      else if(QString(xml_file->GetName()) == "experimental_uncertainty")
         {
         loadExperimentalUncertainty(file_info.absoluteFilePath());
         }
-      else if(xml_file.nodeName() == "simulation_uncertainty")
+      else if(QString(xml_file->GetName()) == "simulation_uncertainty")
         {
         loadSimulationUncertainty(file_info.absoluteFilePath());
         }
-      else if(xml_file.nodeName() == "experiment_simulation_map")
+      else if(QString(xml_file->GetName()) == "experiment_simulation_map")
         {
         loadExperimentSimulationMap(file_info.absoluteFilePath());
         }
