@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqChartAxis.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -55,7 +55,7 @@ public:
   pqChartAxisItem();
   ~pqChartAxisItem() {}
 
-  int Pixel;
+  float Pixel;
   int Width;
 };
 
@@ -223,7 +223,7 @@ void pqChartAxis::setParallelAxis(const pqChartAxis *across)
   this->Across = across;
 }
 
-void pqChartAxis::setContentsScpace(const pqChartContentsSpace *contents)
+void pqChartAxis::setContentsSpace(const pqChartContentsSpace *contents)
 {
   this->Zoom = contents;
 }
@@ -273,6 +273,12 @@ void pqChartAxis::setBestFitRange(const pqChartValue &min,
     this->Internal->Minimum = min;
     this->Internal->Maximum = max;
     }
+
+  // Make sure the min and max are the same type.
+  if(this->Internal->Minimum.getType() != this->Internal->Maximum.getType())
+    {
+    this->Internal->Minimum.convertTo(this->Internal->Maximum.getType());
+    }
 }
 
 bool pqChartAxis::isMaxExtraPadded() const
@@ -624,12 +630,13 @@ void pqChartAxis::layoutAxis(const QRect &area)
     valueChanged = this->Scale->setValueRange(pqChartValue(), pqChartValue());
     }
 
-  //if((valueChanged || this->Internal->ScaleChanged) &&
-  //    this->Scale->getScaleType() == pqChartPixelScale::Logarithmic &&
-  //    !this->Scale->isLogScaleAvailable())
-  //  {
-  //  qWarning() << "Warning: Invalid range for a logarithmic scale.";
-  //  }
+  if((valueChanged || this->Internal->ScaleChanged) &&
+      this->Scale->getScaleType() == pqChartPixelScale::Logarithmic &&
+      !this->Scale->isLogScaleAvailable())
+    {
+    qWarning() << "Warning: Invalid range for a logarithmic scale."
+               << "Please select a range greater than zero.";
+    }
 
   // Signal the chart layers if the pixel-value map changed.
   if(pixelChanged || valueChanged || this->Internal->ScaleChanged)
@@ -650,7 +657,7 @@ void pqChartAxis::layoutAxis(const QRect &area)
       // Use the pixel-value scale to determine the pixel location for
       // the label.
       this->Model->getLabel(i, value);
-      (*iter)->Pixel = this->Scale->getPixelFor(value);
+      (*iter)->Pixel = this->Scale->getPixelF(value);
       }
 
     // If there is not space for all the labels, set up the skip count.
@@ -687,8 +694,8 @@ void pqChartAxis::layoutAxis(const QRect &area)
           }
 
         needed = 4 * count;
-        pixelRange = this->Internal->Items[0]->Pixel;
-        int pixel = this->Internal->Items[count]->Pixel;
+        pixelRange = (int)this->Internal->Items[0]->Pixel;
+        int pixel = (int)this->Internal->Items[count]->Pixel;
         if(pixel < pixelRange)
           {
           pixelRange = pixelRange - pixel;
@@ -868,6 +875,8 @@ void pqChartAxis::drawAxis(QPainter &painter, const QRect &area) const
     }
 
   // Draw the axis line.
+  painter.save();
+  painter.setRenderHint(QPainter::Antialiasing, true);
   painter.setPen(this->Options->getAxisColor());
   if(this->Location == pqChartAxis::Left)
     {
@@ -893,14 +902,15 @@ void pqChartAxis::drawAxis(QPainter &painter, const QRect &area) const
   // Only draw the labels if they are visible.
   if(!this->Options->areLabelsVisible() || this->Internal->SpaceTooSmall)
     {
+    painter.restore();
     return;
     }
 
   // Set up the constant values based on the axis location.
-  int x = 0;
-  int y = 0;
-  int tick = 0;
-  int tickSmall = 0;
+  float x = 0;
+  float y = 0;
+  float tick = 0;
+  float tickSmall = 0;
   if(this->Location == pqChartAxis::Left)
     {
     x = this->Internal->Bounds.right();
@@ -953,11 +963,11 @@ void pqChartAxis::drawAxis(QPainter &painter, const QRect &area) const
         }
 
       // Make sure the label is inside the axis bounds.
-      if(y > this->Internal->Bounds.bottom())
+      if((int)y > this->Internal->Bounds.bottom())
         {
         continue;
         }
-      else if(y < this->Internal->Bounds.top())
+      else if((int)y < this->Internal->Bounds.top())
         {
         break;
         }
@@ -968,7 +978,7 @@ void pqChartAxis::drawAxis(QPainter &painter, const QRect &area) const
       if(this->Internal->Skip == 1 || skipIndex == 0)
         {
         painter.setPen(this->Options->getAxisColor());
-        painter.drawLine(tick, y, x, y);
+        painter.drawLine(QPointF(tick, y), QPointF(x, y));
         this->Model->getLabel(i, value);
         label = value.getString(this->Options->getPrecision(),
             this->Options->getNotation());
@@ -976,19 +986,20 @@ void pqChartAxis::drawAxis(QPainter &painter, const QRect &area) const
         y += halfAscent;
         if(this->Location == pqChartAxis::Left)
           {
-          painter.drawText(tick - (*iter)->Width -
-              this->Internal->TickLabelSpacing, y, label);
+          painter.drawText(QPointF(tick - (*iter)->Width -
+              this->Internal->TickLabelSpacing, y), label);
           }
         else
           {
-          painter.drawText(tick + this->Internal->TickLabelSpacing, y, label);
+          painter.drawText(
+              QPointF(tick + this->Internal->TickLabelSpacing, y), label);
           }
         }
       else if(this->Internal->TickSkip == 1 ||
           skipIndex % this->Internal->TickSkip == 0)
         {
         painter.setPen(this->Options->getAxisColor());
-        painter.drawLine(tickSmall, y, x, y);
+        painter.drawLine(QPointF(tickSmall, y), QPointF(x, y));
         }
       }
     else
@@ -1001,11 +1012,11 @@ void pqChartAxis::drawAxis(QPainter &painter, const QRect &area) const
         }
 
       // Make sure the label is inside the axis bounds.
-      if(x < this->Internal->Bounds.left())
+      if((int)x < this->Internal->Bounds.left())
         {
         continue;
         }
-      else if(x > this->Internal->Bounds.right())
+      else if((int)x > this->Internal->Bounds.right())
         {
         break;
         }
@@ -1016,7 +1027,7 @@ void pqChartAxis::drawAxis(QPainter &painter, const QRect &area) const
       if(this->Internal->Skip == 1 || skipIndex == 0)
         {
         painter.setPen(this->Options->getAxisColor());
-        painter.drawLine(x, tick, x, y);
+        painter.drawLine(QPointF(x, tick), QPointF(x, y));
         this->Model->getLabel(i, value);
         label = value.getString(this->Options->getPrecision(),
             this->Options->getNotation());
@@ -1024,23 +1035,25 @@ void pqChartAxis::drawAxis(QPainter &painter, const QRect &area) const
         x -= (*iter)->Width / 2;
         if(this->Location == pqChartAxis::Top)
           {
-          painter.drawText(
-              x, tick - this->Internal->TickLabelSpacing - fontDescent, label);
+          painter.drawText(QPointF(x,
+              tick - this->Internal->TickLabelSpacing - fontDescent), label);
           }
         else
           {
-          painter.drawText(
-              x, tick + this->Internal->TickLabelSpacing + fontAscent, label);
+          painter.drawText(QPointF(x,
+              tick + this->Internal->TickLabelSpacing + fontAscent), label);
           }
         }
       else if(this->Internal->TickSkip == 1 ||
           skipIndex % this->Internal->TickSkip == 0)
         {
         painter.setPen(this->Options->getAxisColor());
-        painter.drawLine(x, tickSmall, x, y);
+        painter.drawLine(QPointF(x, tickSmall), QPointF(x, y));
         }
       }
     }
+
+  painter.restore();
 }
 
 void pqChartAxis::getBounds(QRect &bounds) const
@@ -1069,11 +1082,11 @@ bool pqChartAxis::isLabelTickVisible(int index) const
   return false;
 }
 
-int pqChartAxis::getLabelLocation(int index) const
+float pqChartAxis::getLabelLocation(int index) const
 {
   if(index >= 0 && index < this->Internal->Items.size())
     {
-    int pixel = this->Internal->Items[index]->Pixel;
+    float pixel = this->Internal->Items[index]->Pixel;
     if(this->Zoom)
       {
       if(this->Location == pqChartAxis::Top ||
