@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqChartMouseSelection.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -636,8 +636,8 @@ void pqChartMouseSelection::mouseMoveDragMove(const QPoint &point)
       pqChartValue offset, offset2;
       pqChartAxis *xAxis = this->Histogram->Chart->getXAxis();
       const pqChartPixelScale *xScale = xAxis->getPixelValueScale();
-      xScale->getValueFor(point.x(), offset);
-      xScale->getValueFor(this->Histogram->LastValueX, offset2);
+      xScale->getValue(point.x(), offset);
+      xScale->getValue(this->Histogram->LastValueX, offset2);
       offset -= offset2;
       if(offset != 0)
         {
@@ -647,7 +647,7 @@ void pqChartMouseSelection::mouseMoveDragMove(const QPoint &point)
         if(range.getFirst() == range.getSecond())
           {
           range.moveRange(offset);
-          this->Histogram->LastValueX = xScale->getPixelFor(range.getFirst());
+          this->Histogram->LastValueX = xScale->getPixel(range.getFirst());
           }
         else
           {
