@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqColorMapWidget.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -50,6 +50,11 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <QSize>
 #include <QTimer>
 
+#include <math.h>
+
+#ifndef M_PI
+#define M_PI 3.1415926535897932384626
+#endif
 
 class pqColorMapWidgetInternal
 {
@@ -78,6 +83,43 @@ public:
   bool PointMoved;            ///< True if point was moved.
 };
 
+//=============================================================================
+// Given two angular orientations, returns the smallest angle between the two.
+inline double pqColorMapWidgetAngleDiff(double a1, double a2)
+{
+  double adiff = a1 - a2;
+  if (adiff < 0.0) adiff = -adiff;
+  while (adiff >= M_PI) adiff -= M_PI;
+  return adiff;
+}
+
+// For the case when interpolating from a saturated color to an unsaturated
+// color, find a hue for the unsaturated color that makes sense.
+inline double pqColorMapWidgetAdjustHue(double satM, double satS, double satH,
+                                        double unsatM)
+{
+  if (satM >= unsatM - 0.1)
+    {
+    // The best we can do is hold hue constant.
+    return satH;
+    }
+  else
+    {
+    // This equation is designed to make the perceptual change of the
+    // interpolation to be close to constant.
+    double hueSpin = (  satS*sqrt(unsatM*unsatM - satM*satM)
+                      / (satM*sin(satS)) );
+    // Spin hue away from 0 except in purple hues.
+    if (satH > -0.3*M_PI)
+      {
+      return satH + hueSpin;
+      }
+    else
+      {
+      return satH - hueSpin;
+      }
+    }
+}
 
 //-----------------------------------------------------------------------------
 pqColorMapWidgetInternal::pqColorMapWidgetInternal()
@@ -368,7 +410,7 @@ void pqColorMapWidget::mouseMoveEvent(QMouseEvent *e)
         this->Internal->PointMoved = true;
 
         pqChartValue value;
-        this->Internal->PixelMap.getValueFor(pixel, value);
+        this->Internal->PixelMap.getValue(pixel, value);
         this->Model->setPointValue(this->Internal->PointIndex, value);
 
         this->generateGradient();
@@ -429,7 +471,7 @@ void pqColorMapWidget::mouseReleaseEvent(QMouseEvent *e)
       // Add a point to the list. Use the mouse position to determine
       // the value and color.
       pqChartValue value;
-      this->Internal->PixelMap.getValueFor(e->x(), value);
+      this->Internal->PixelMap.getValue(e->x(), value);
       QImage image = this->DisplayImage->toImage();
       QColor color = image.pixel(e->x() - this->Internal->ImageArea.left(), 0);
       this->Model->addPoint(value, color);
@@ -603,7 +645,7 @@ void pqColorMapWidget::updatePointValue(int index, const pqChartValue &value)
       index >= 0 && index < this->Internal->Items.size())
     {
     // Update the pixel position of the point.
-    this->Internal->Items[index] = this->Internal->PixelMap.getPixelFor(value);
+    this->Internal->Items[index] = this->Internal->PixelMap.getPixel(value);
 
     if(index == 0 || index == this->Internal->Items.size() - 1)
       {
@@ -634,7 +676,7 @@ void pqColorMapWidget::layoutPoints()
     for(int i = 0; iter != this->Internal->Items.end(); ++iter, ++i)
       {
       this->Model->getPointValue(i, value);
-      *iter = this->Internal->PixelMap.getPixelFor(value);
+      *iter = this->Internal->PixelMap.getPixel(value);
       }
     }
 }
@@ -750,6 +792,61 @@ void pqColorMapWidget::generateGradient()
             color.setRgbF(red, green, blue);
             painter.setPen(color);
             }
+          else if(this->Model->getColorSpace()==pqColorMapModel::DivergingSpace)
+            {
+            double M_next, s_next, h_next, M_previous, s_previous, h_previous;
+            pqColorMapModel::RGBToMsh(next.redF(), next.greenF(), next.blueF(),
+                                      &M_next, &s_next, &h_next);
+            pqColorMapModel::RGBToMsh(
+                           previous.redF(), previous.greenF(), previous.blueF(),
+                           &M_previous, &s_previous, &h_previous);
+
+            double interp = (double)(px - x1)/w;
+
+            // If the endpoints are distinct saturated colors, then place white
+            // in between them.
+            if (   (s_next > 0.05) && (s_previous > 0.05)
+                && (pqColorMapWidgetAngleDiff(h_next, h_previous) > 0.33*M_PI) )
+              {
+              // Insert the white midpoint by setting one end to white and
+              // adjusting the interpolation value.
+              if (interp < 0.5)
+                {
+                M_next = 95.0;  s_next = 0.0;  h_next = 0.0;
+                interp = 2.0*interp;
+                }
+              else
+                {
+                M_previous = 95.0;  s_previous = 0.0; h_previous = 0.0;
+                interp = 2.0*interp - 1.0;
+                }
+              }
+
+            // If one color has no saturation, thin its hue is invalid.  In this
+            // case, we want to set it to something logical so that the
+            // interpolation of hue makes sense.
+            if ((s_previous < 0.05) && (s_next > 0.05))
+              {
+              h_previous = pqColorMapWidgetAdjustHue(M_next, s_next, h_next,
+                                                     M_previous);
+              }
+            else if ((s_next < 0.01) && (s_previous > 0.01))
+              {
+              h_next = pqColorMapWidgetAdjustHue(
+                                             M_previous, s_previous, h_previous,
+                                             M_next);
+              }
+
+            double M = (1-interp)*M_previous + interp*M_next;
+            double s = (1-interp)*s_previous + interp*s_next;
+            double h = (1-interp)*h_previous + interp*h_next;
+
+            double red, green, blue;
+            pqColorMapModel::MshToRGB(M, s, h, &red, &green, &blue);
+            QColor color;
+            color.setRgbF(red, green, blue);
+            painter.setPen(color);
+            }
 
           painter.drawLine(px, 0, px, imageHeight);
           }
