@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqHistogramChart.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -35,8 +35,9 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include "pqHistogramChart.h"
 
-#include "pqChartAxisModel.h"
+#include "pqChartArea.h"
 #include "pqChartAxis.h"
+#include "pqChartAxisModel.h"
 #include "pqChartContentsSpace.h"
 #include "pqHistogramChartOptions.h"
 #include "pqHistogramModel.h"
@@ -44,6 +45,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <QPainter>
 #include <QRect>
+#include <QRectF>
 #include <QtDebug>
 #include <QVector>
 
@@ -54,8 +56,8 @@ public:
   pqHistogramChartInternal();
   ~pqHistogramChartInternal() {}
 
-  QVector<QRect> Items;      ///< The list of histogram bars.
-  QVector<QRect> Highlights; ///< The list of highlighted ranges.
+  QVector<QRectF> Items;      ///< The list of histogram bars.
+  QVector<QRectF> Highlights; ///< The list of highlighted ranges.
   QRect Bounds;              ///< Stores the chart bounds.
   QRect Contents;            ///< Stores the contents area.
 };
@@ -74,8 +76,7 @@ pqHistogramChart::pqHistogramChart(QObject *parentObject)
 {
   this->Internal = new pqHistogramChartInternal();
   this->Options = new pqHistogramChartOptions(this);
-  this->XAxis = 0;
-  this->YAxis = 0;
+  this->Axes = pqHistogramChart::BottomLeft;
   this->Model = 0;
   this->Selection = new pqHistogramSelectionModel(this);
   this->InModelChange = false;
@@ -97,24 +98,53 @@ pqHistogramChart::~pqHistogramChart()
   delete this->Selection;
 }
 
-void pqHistogramChart::setAxes(pqChartAxis *xAxis, pqChartAxis *yAxis)
+void pqHistogramChart::setChartAxes(ChartAxes axes)
 {
-  if(xAxis->getLocation() == pqChartAxis::Left ||
-      xAxis->getLocation() == pqChartAxis::Right)
+  if(this->Axes != axes)
     {
-    qCritical() << "Error: The x-axis must be a horizontal axis.";
-    return;
+    this->Axes = axes;
+    emit this->rangeChanged();
+    emit this->layoutNeeded();
+    }
+}
+
+pqChartAxis *pqHistogramChart::getXAxis() const
+{
+  pqChartArea *chartArea = this->getChartArea();
+  if(!chartArea)
+    {
+    return 0;
     }
 
-  if(yAxis->getLocation() == pqChartAxis::Top ||
-      yAxis->getLocation() == pqChartAxis::Bottom)
+  if(this->Axes == pqHistogramChart::BottomLeft ||
+      this->Axes == pqHistogramChart::BottomRight)
     {
-    qCritical() << "Error: The y-axis must be a vertical axis.";
-    return;
+    return chartArea->getAxis(pqChartAxis::Bottom);
+    }
+  else
+    {
+    return chartArea->getAxis(pqChartAxis::Top);
+    }
+}
+
+pqChartAxis *pqHistogramChart::getYAxis() const
+{
+  pqChartArea *chartArea = this->getChartArea();
+  if(!chartArea)
+    {
+    return 0;
     }
 
-  this->XAxis = xAxis;
-  this->YAxis = yAxis;
+  if(this->Axes == pqHistogramChart::BottomLeft ||
+      this->Axes == pqHistogramChart::TopLeft)
+    {
+    return chartArea->getAxis(pqChartAxis::Left);
+    }
+  else
+    {
+    return chartArea->getAxis(pqChartAxis::Right);
+    }
+  return 0;
 }
 
 void pqHistogramChart::setModel(pqHistogramModel *model)
@@ -172,7 +202,7 @@ int pqHistogramChart::getBinAt(int x, int y,
   if(this->Internal->Contents.isValid() &&
       this->Internal->Contents.contains(x, y))
     {
-    QVector<QRect>::Iterator iter = this->Internal->Items.begin();
+    QVector<QRectF>::Iterator iter = this->Internal->Items.begin();
     for(int i = 0; iter != this->Internal->Items.end(); iter++, i++)
       {
       if(mode == pqHistogramChart::BinRange && iter->isValid() &&
@@ -192,12 +222,7 @@ int pqHistogramChart::getBinAt(int x, int y,
 
 bool pqHistogramChart::getValueAt(int x, int y, pqChartValue &value) const
 {
-  if(!this->XAxis)
-    {
-    return false;
-    }
-
-  const pqChartPixelScale *scale = this->XAxis->getPixelValueScale();
+  const pqChartPixelScale *scale = this->getXAxis()->getPixelValueScale();
   if(this->Internal->Contents.isValid() && scale->isValid() &&
       this->Internal->Contents.contains(x, y))
     {
@@ -224,7 +249,7 @@ bool pqHistogramChart::getValueAt(int x, int y, pqChartValue &value) const
         }
       }
 
-    scale->getValueFor(x, value);
+    scale->getValue(x, value);
     return true;
     }
 
@@ -234,12 +259,7 @@ bool pqHistogramChart::getValueAt(int x, int y, pqChartValue &value) const
 bool pqHistogramChart::getValueRangeAt(int x, int y,
     pqHistogramSelection &range) const
 {
-  if(!this->XAxis)
-    {
-    return false;
-    }
-
-  const pqChartPixelScale *scale = this->XAxis->getPixelValueScale();
+  const pqChartPixelScale *scale = this->getXAxis()->getPixelValueScale();
   if(this->Internal->Contents.isValid() && scale->isValid() &&
     this->Internal->Contents.contains(x, y))
     {
@@ -270,7 +290,7 @@ bool pqHistogramChart::getValueRangeAt(int x, int y,
 
       // Search through the current selection list.
       pqChartValue value;
-      scale->getValueFor(x, value);
+      scale->getValue(x, value);
       pqHistogramSelectionList::ConstIterator iter = list.begin();
       for( ; iter != list.end(); ++iter)
         {
@@ -301,7 +321,7 @@ void pqHistogramChart::getBinsIn(const QRect &area,
       area.intersects(this->Internal->Contents))
     {
     pqChartValue i((int)0);
-    QVector<QRect>::Iterator iter = this->Internal->Items.begin();
+    QVector<QRectF>::Iterator iter = this->Internal->Items.begin();
     for( ; iter != this->Internal->Items.end(); ++iter, ++i)
       {
       if(area.right() < iter->left())
@@ -338,12 +358,12 @@ void pqHistogramChart::getBinsIn(const QRect &area,
 void pqHistogramChart::getValuesIn(const QRect &area,
     pqHistogramSelectionList &list) const
 {
-  if(!this->XAxis || !area.isValid() || !this->Internal->Contents.isValid())
+  if(!area.isValid() || !this->Internal->Contents.isValid())
     {
     return;
     }
 
-  const pqChartPixelScale *scale = this->XAxis->getPixelValueScale();
+  const pqChartPixelScale *scale = this->getXAxis()->getPixelValueScale();
   if(!scale->isValid() || !area.intersects(this->Internal->Contents))
     {
     return;
@@ -390,8 +410,8 @@ void pqHistogramChart::getSelectionArea(const pqHistogramSelectionList &list,
     if(leftBin >= 0 && leftBin < this->Internal->Items.size() &&
         rightBin >= 0 && rightBin < this->Internal->Items.size())
       {
-      area.setLeft(this->Internal->Items[leftBin].left());
-      area.setRight(this->Internal->Items[rightBin].right());
+      area.setLeft((int)this->Internal->Items[leftBin].left());
+      area.setRight((int)this->Internal->Items[rightBin].right());
       }
     else
       {
@@ -400,19 +420,14 @@ void pqHistogramChart::getSelectionArea(const pqHistogramSelectionList &list,
     }
   else
     {
-    if(!this->XAxis)
-      {
-      return;
-      }
-
-    const pqChartPixelScale *scale = this->XAxis->getPixelValueScale();
+    const pqChartPixelScale *scale = this->getXAxis()->getPixelValueScale();
     if(!scale->isValid())
       {
       return;
       }
 
-    area.setLeft(scale->getPixelFor(first.getFirst()));
-    area.setRight(scale->getPixelFor(last.getSecond()));
+    area.setLeft(scale->getPixel(first.getFirst()));
+    area.setRight(scale->getPixel(last.getSecond()));
     }
 
   const pqChartContentsSpace *zoomPan = this->getContentsSpace();
@@ -432,12 +447,12 @@ bool pqHistogramChart::getAxisRange(const pqChartAxis *axis,
 {
   if(this->Model && this->Model->getNumberOfBins() > 0)
     {
-    if(axis == this->XAxis)
+    if(axis == this->getXAxis())
       {
       this->Model->getRangeX(min, max);
       return true;
       }
-    else if(axis == this->YAxis)
+    else if(axis == this->getYAxis())
       {
       this->Model->getRangeY(min, max);
       if(axis->getPixelValueScale()->getScaleType() ==
@@ -507,7 +522,7 @@ bool pqHistogramChart::isAxisControlPreferred(
     const pqChartAxis *axis) const
 {
   return this->Model != 0 && this->Model->getNumberOfBins() > 0 &&
-      this->XAxis != 0 && this->XAxis == axis;
+      axis == this->getXAxis();
 }
 
 void pqHistogramChart::generateAxisLabels(pqChartAxis *axis)
@@ -535,14 +550,14 @@ void pqHistogramChart::generateAxisLabels(pqChartAxis *axis)
 
 void pqHistogramChart::layoutChart(const QRect &area)
 {
-  if(!this->Model || !this->XAxis || !this->YAxis || !area.isValid())
+  if(!this->Model || !this->getChartArea() || !area.isValid())
     {
     return;
     }
 
   // Make sure the axes are valid.
-  const pqChartPixelScale *xScale = this->XAxis->getPixelValueScale();
-  const pqChartPixelScale *yScale = this->YAxis->getPixelValueScale();
+  const pqChartPixelScale *xScale = this->getXAxis()->getPixelValueScale();
+  const pqChartPixelScale *yScale = this->getYAxis()->getPixelValueScale();
   if(!xScale->isValid() || !yScale->isValid())
     {
     return;
@@ -570,13 +585,13 @@ void pqHistogramChart::layoutChart(const QRect &area)
 
   // Set up the size for each bar in the histogram.
   pqChartValue value, min, max;
-  int bottom = yScale->getMinPixel();
+  float bottom = yScale->getMinPixel();
   bool reversed = false;
   if(yScale->isZeroInRange())
     {
     pqChartValue zero(0);
     zero.convertTo(yScale->getMaxValue().getType());
-    bottom = yScale->getPixelFor(zero);
+    bottom = yScale->getPixelF(zero);
     }
   else
     {
@@ -588,21 +603,21 @@ void pqHistogramChart::layoutChart(const QRect &area)
       }
     }
 
-  QVector<QRect>::Iterator iter = this->Internal->Items.begin();
+  QVector<QRectF>::Iterator iter = this->Internal->Items.begin();
   for(int i = 0; iter != this->Internal->Items.end(); ++iter, ++i)
     {
     this->Model->getBinValue(i, value);
     this->Model->getBinRange(i, min, max);
-    iter->setLeft(xScale->getPixelFor(min));
-    iter->setRight(xScale->getPixelFor(max));
+    iter->setLeft(xScale->getPixelF(min));
+    iter->setRight(xScale->getPixelF(max));
     if(reversed || value < 0)
       {
       iter->setTop(bottom);
-      iter->setBottom(yScale->getPixelFor(value));
+      iter->setBottom(yScale->getPixelF(value));
       }
     else
       {
-      iter->setTop(yScale->getPixelFor(value));
+      iter->setTop(yScale->getPixelF(value));
       iter->setBottom(bottom);
       }
     }
@@ -620,6 +635,7 @@ void pqHistogramChart::drawBackground(QPainter &painter, const QRect &area)
 
   // Translate the painter and the area to paint to contents space.
   painter.save();
+  painter.setRenderHint(QPainter::Antialiasing, true);
   QRect clipArea = area.intersect(this->Internal->Bounds);
   QRect toPaint = area;
   const pqChartContentsSpace *zoomPan = this->getContentsSpace();
@@ -635,7 +651,7 @@ void pqHistogramChart::drawBackground(QPainter &painter, const QRect &area)
   painter.setClipRect(clipArea);
 
   // Draw in the selection if there is one.
-  QVector<QRect>::Iterator highlight = this->Internal->Highlights.begin();
+  QVector<QRectF>::Iterator highlight = this->Internal->Highlights.begin();
   for( ; highlight != this->Internal->Highlights.end(); ++highlight)
     {
     if(highlight->intersects(toPaint))
@@ -657,6 +673,7 @@ void pqHistogramChart::drawChart(QPainter &painter, const QRect &area)
 
   // Translate the painter and the area to paint to contents space.
   painter.save();
+  painter.setRenderHint(QPainter::Antialiasing, true);
   QRect clipArea = area.intersect(this->Internal->Bounds);
   QRect toPaint = area;
   const pqChartContentsSpace *zoomPan = this->getContentsSpace();
@@ -675,8 +692,8 @@ void pqHistogramChart::drawChart(QPainter &painter, const QRect &area)
   int i = 0;
   bool areaFound = false;
   int total = this->Model->getNumberOfBins();
-  QVector<QRect>::Iterator highlight = this->Internal->Highlights.begin();
-  QVector<QRect>::Iterator iter = this->Internal->Items.begin();
+  QVector<QRectF>::Iterator highlight = this->Internal->Highlights.begin();
+  QVector<QRectF>::Iterator iter = this->Internal->Items.begin();
   for( ; iter != this->Internal->Items.end(); ++iter, ++i)
     {
     // Make sure the bounding rectangle is known.
@@ -699,70 +716,55 @@ void pqHistogramChart::drawChart(QPainter &painter, const QRect &area)
           bin = this->Options->getColorScheme()->getColor(i, total);
           }
 
-        painter.fillRect(iter->x(), iter->y(), iter->width() - 1,
-            iter->height() - 1, bin);
-
-        // Draw in the highlighted portion of the bar if it is selected
-        // and fill highlighting is being used.
-        if(this->Options->getHighlightStyle() == pqHistogramChartOptions::Fill)
-          {
-          for( ; highlight != this->Internal->Highlights.end(); ++highlight)
-            {
-            if(iter->right() < highlight->left())
-              {
-              break;
-              }
-            else if(iter->left() <= highlight->right())
-              {
-              painter.fillRect(iter->intersect(*highlight), bin.light(170));
-              if(iter->right() <= highlight->right())
-                {
-                break;
-                }
-              }
-            }
-          }
-
-        // Draw in the outline last to make sure it is on top.
+        // Set the outline color.
+        QPen outlinePen = painter.pen();
+        outlinePen.setJoinStyle(Qt::MiterJoin);
+        outlinePen.setWidth(1);
         if(this->Options->getOutlineStyle() == pqHistogramChartOptions::Darker)
           {
-          painter.setPen(bin.dark());
+          outlinePen.setColor(bin.dark());
           }
         else
           {
-          painter.setPen(Qt::black);
+          outlinePen.setColor(Qt::black);
           }
 
-        painter.drawRect(iter->x(), iter->y(), iter->width() - 1,
-            iter->height() - 1);
+        painter.setPen(outlinePen);
+        painter.setBrush(bin);
+        QRectF binRect = *iter;
+        painter.drawRect(binRect);
 
-        // If the bar is selected and outline highlighting is used, draw
-        // a lighter and thicker outline around the bar.
-        if(this->Options->getHighlightStyle() ==
-            pqHistogramChartOptions::Outline)
+        // Set up the painter to draw the highlights.
+        if(this->Options->getHighlightStyle() == pqHistogramChartOptions::Fill)
           {
-          for( ; highlight != this->Internal->Highlights.end(); ++highlight)
+          painter.setBrush(bin.light(170));
+          }
+        else
+          {
+          outlinePen.setWidth(2);
+          outlinePen.setColor(bin.light(170));
+          painter.setPen(outlinePen);
+          painter.setBrush(QBrush());
+          binRect.adjust(1, 1, 0, 0);
+          }
+
+        // Draw in the highlighted portion of the bin rectangle.
+        for( ; highlight != this->Internal->Highlights.end(); ++highlight)
+          {
+          if(iter->right() < highlight->left())
             {
-            if(iter->right() < highlight->left())
+            break;
+            }
+          else if(iter->left() <= highlight->right())
+            {
+            painter.save();
+            painter.setClipRect(highlight->intersect(iter->adjusted(0, 0, 1, 1)));
+            painter.drawRect(binRect);
+            painter.restore();
+            if(iter->right() <= highlight->right())
               {
               break;
               }
-            else if(iter->left() <= highlight->right())
-              {
-              painter.setPen(bin.light(170));
-              QRect inter = iter->intersect(*highlight);
-              inter.setWidth(inter.width() - 1);
-              inter.setHeight(inter.height() - 1);
-              painter.drawRect(inter);
-              inter.translate(1, 1);
-              inter.setWidth(inter.width() - 2);
-              inter.setHeight(inter.height() - 2);
-              painter.drawRect(inter);
-              if(iter->right() <= highlight->right())
-                {
-                break;
-                }
-              }
             }
           }
         }
@@ -775,13 +777,13 @@ void pqHistogramChart::drawChart(QPainter &painter, const QRect &area)
 
   // Draw in the selection outline.
   painter.setPen(QColor(60, 90, 135));
+  painter.setBrush(QBrush());
   highlight = this->Internal->Highlights.begin();
   for( ; highlight != this->Internal->Highlights.end(); ++highlight)
     {
     if(highlight->intersects(toPaint))
       {
-      painter.drawRect(highlight->x(), highlight->y(), highlight->width() - 1,
-          highlight->height() - 1);
+      painter.drawRect(*highlight);
       }
     }
 
@@ -852,7 +854,7 @@ void pqHistogramChart::finishBinRemoval()
 
 void pqHistogramChart::updateHighlights()
 {
-  if(!this->InModelChange && this->Internal->Contents.isValid() && this->XAxis)
+  if(!this->InModelChange && this->Internal->Contents.isValid())
     {
     this->layoutSelection();
     emit this->repaintNeeded();
@@ -862,7 +864,7 @@ void pqHistogramChart::updateHighlights()
 void pqHistogramChart::layoutSelection()
 {
   // Make sure the axes are valid.
-  const pqChartPixelScale *xScale = this->XAxis->getPixelValueScale();
+  const pqChartPixelScale *xScale = this->getXAxis()->getPixelValueScale();
   if(!xScale->isValid())
     {
     return;
@@ -878,7 +880,7 @@ void pqHistogramChart::layoutSelection()
     this->Internal->Highlights.resize(list.size());
     }
 
-  QVector<QRect>::Iterator highlight = this->Internal->Highlights.begin();
+  QVector<QRectF>::Iterator highlight = this->Internal->Highlights.begin();
   pqHistogramSelectionList::ConstIterator iter = list.begin();
   for( ; iter != list.end(); ++iter, ++highlight)
     {
@@ -888,8 +890,8 @@ void pqHistogramChart::layoutSelection()
     highlight->setBottom(this->Internal->Contents.bottom());
     if(iter->getType() == pqHistogramSelection::Value)
       {
-      highlight->setLeft(xScale->getPixelFor(iter->getFirst()));
-      highlight->setRight(xScale->getPixelFor(iter->getSecond()));
+      highlight->setLeft(xScale->getPixelF(iter->getFirst()));
+      highlight->setRight(xScale->getPixelF(iter->getSecond()));
       }
     else
       {
