@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqLineChart.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -35,6 +35,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include "pqLineChart.h"
 
+#include "pqChartArea.h"
 #include "pqChartAxis.h"
 #include "pqChartContentsSpace.h"
 #include "pqChartCoordinate.h"
@@ -74,7 +75,7 @@ public:
 
   virtual void clear() {this->Sequence.clear();}
 
-  QPolygon Sequence;
+  QPolygonF Sequence;
 };
 
 
@@ -86,7 +87,7 @@ public:
 
   virtual void clear() {this->Sequence.clear();}
 
-  QList<QPolygon> Sequence;
+  QList<QPolygonF> Sequence;
 };
 
 
@@ -96,9 +97,9 @@ public:
   pqLineChartSeriesErrorDataItem();
   ~pqLineChartSeriesErrorDataItem() {}
 
-  int X;
-  int Upper;
-  int Lower;
+  float X;
+  float Upper;
+  float Lower;
 };
 
 
@@ -111,7 +112,7 @@ public:
   virtual void clear() {this->Sequence.clear();}
 
   QVector<pqLineChartSeriesErrorDataItem> Sequence;
-  int Width;
+  float Width;
 };
 
 
@@ -198,8 +199,6 @@ pqLineChart::pqLineChart(QObject *parentObject)
   this->Internal = new pqLineChartInternal();
   this->Options = new pqLineChartOptions(this);
   this->Model = 0;
-  this->XAxis = 0;
-  this->YAxis = 0;
   this->NeedsLayout = false;
 
   this->connect(this->Options, SIGNAL(optionsChanged()),
@@ -212,53 +211,6 @@ pqLineChart::~pqLineChart()
   delete this->Internal;
 }
 
-void pqLineChart::setAxes(pqChartAxis *xAxis, pqChartAxis *yAxis)
-{
-  if(xAxis == this->XAxis && yAxis == this->YAxis)
-    {
-    return;
-    }
-
-  if(xAxis->getLocation() == pqChartAxis::Left ||
-      xAxis->getLocation() == pqChartAxis::Right)
-    {
-    qCritical() << "Error: The x-axis must be a horizontal axis.";
-    return;
-    }
-
-  if(yAxis->getLocation() == pqChartAxis::Top ||
-      yAxis->getLocation() == pqChartAxis::Bottom)
-    {
-    qCritical() << "Error: The y-axis must be a vertical axis.";
-    return;
-    }
-
-  if(this->XAxis)
-    {
-    this->disconnect(this->XAxis, 0, this, 0);
-    }
-
-  if(this->YAxis)
-    {
-    this->disconnect(this->YAxis, 0, this, 0);
-    }
-
-  this->XAxis = xAxis;
-  this->YAxis = yAxis;
-  this->NeedsLayout = true;
-  if(this->XAxis)
-    {
-    this->connect(this->XAxis, SIGNAL(pixelScaleChanged()),
-        this, SLOT(handleRangeChange()));
-    }
-
-  if(this->YAxis)
-    {
-    this->connect(this->YAxis, SIGNAL(pixelScaleChanged()),
-        this, SLOT(handleRangeChange()));
-    }
-}
-
 void pqLineChart::setModel(pqLineChartModel *model)
 {
   if(this->Model == model)
@@ -300,6 +252,9 @@ void pqLineChart::setModel(pqLineChartModel *model)
         this, SLOT(finishSeriesRemoval(int, int)));
     this->connect(this->Model, SIGNAL(seriesMoved(int, int)),
         this, SLOT(handleSeriesMoved(int, int)));
+    this->connect(this->Model,
+        SIGNAL(seriesChartAxesChanged(const pqLineChartSeries *)),
+        this, SLOT(handleSeriesAxesChanged(const pqLineChartSeries *)));
     this->connect(this->Model, SIGNAL(seriesReset(const pqLineChartSeries *)),
         this, SLOT(handleSeriesReset(const pqLineChartSeries *)));
     this->connect(this->Model,
@@ -361,16 +316,7 @@ bool pqLineChart::getAxisRange(const pqChartAxis *axis,
 {
   if(this->Model && this->Model->getNumberOfSeries() > 0)
     {
-    if(axis == this->XAxis)
-      {
-      this->Model->getRangeX(min, max);
-      return true;
-      }
-    else if(axis == this->YAxis)
-      {
-      this->Model->getRangeY(min, max);
-      return true;
-      }
+    return this->Model->getAxisRange(axis, min, max);
     }
 
   return false;
@@ -378,15 +324,8 @@ bool pqLineChart::getAxisRange(const pqChartAxis *axis,
 
 void pqLineChart::layoutChart(const QRect &area)
 {
-  if(!this->Model || !this->XAxis || !this->YAxis || !area.isValid())
-    {
-    return;
-    }
-
-  // Make sure the axes are valid.
-  const pqChartPixelScale *xScale = this->XAxis->getPixelValueScale();
-  const pqChartPixelScale *yScale = this->YAxis->getPixelValueScale();
-  if(!xScale->isValid() || !yScale->isValid())
+  pqChartArea *chartArea = this->getChartArea();
+  if(!this->Model || !chartArea || !area.isValid())
     {
     return;
     }
@@ -408,6 +347,8 @@ void pqLineChart::layoutChart(const QRect &area)
   // Layout each of the series. Only, layout the series if it has
   // changed or if either of the axis scales have changed.
   int i = 0;
+  const pqChartPixelScale *xScale = 0;
+  const pqChartPixelScale *yScale = 0;
   QList<pqLineChartSeriesItem *>::Iterator iter = this->Internal->Series.begin();
   for( ; iter != this->Internal->Series.end(); ++iter)
     {
@@ -448,6 +389,35 @@ void pqLineChart::layoutChart(const QRect &area)
         continue;
         }
 
+      // Get the pixel scales for the series axes.
+      pqLineChartSeries::ChartAxes axes = (*iter)->Series->getChartAxes();
+      if(axes == pqLineChartSeries::BottomLeft ||
+          axes == pqLineChartSeries::BottomRight)
+        {
+        xScale = chartArea->getAxis(pqChartAxis::Bottom)->getPixelValueScale();
+        }
+      else
+        {
+        xScale = chartArea->getAxis(pqChartAxis::Top)->getPixelValueScale();
+        }
+
+      if(axes == pqLineChartSeries::BottomLeft ||
+          axes == pqLineChartSeries::TopLeft)
+        {
+        yScale = chartArea->getAxis(pqChartAxis::Left)->getPixelValueScale();
+        }
+      else
+        {
+        yScale = chartArea->getAxis(pqChartAxis::Right)->getPixelValueScale();
+        }
+
+      // Make sure the axes are valid.
+      if(!xScale->isValid() || !yScale->isValid())
+        {
+        qWarning("Series pixel scales are invalid.");
+        continue;
+        }
+
       QList<pqLineChartSeriesItemData *>::Iterator sequence =
           (*iter)->Sequences.begin();
       for(i = 0; sequence != (*iter)->Sequences.end(); ++sequence, ++i)
@@ -461,7 +431,7 @@ void pqLineChart::layoutChart(const QRect &area)
         pqLineChartSeriesErrorData *error =
             dynamic_cast<pqLineChartSeriesErrorData *>(*sequence);
 
-        QPolygon *polygon = 0;
+        QPolygonF *polygon = 0;
         int total = (*iter)->Series->getNumberOfPoints(i);
         for(int j = 0; j < total; j++)
           {
@@ -471,8 +441,8 @@ void pqLineChart::layoutChart(const QRect &area)
 
           // Use the axis scale to translate the series' coordinates
           // into chart coordinates.
-          QPoint point(xScale->getPixelFor(coord.X),
-              yScale->getPixelFor(coord.Y));
+          QPointF point(xScale->getPixelF(coord.X),
+              yScale->getPixelF(coord.Y));
 
           // Store the pixel coordinates for the point.
           if(line)
@@ -481,7 +451,7 @@ void pqLineChart::layoutChart(const QRect &area)
             // break them up into manageable chunks.
             if(!polygon || j % 100 == 0 || isBreak)
               {
-              line->Sequence.append(QPolygon());
+              line->Sequence.append(QPolygonF());
               polygon = &line->Sequence.last();
               polygon->reserve(101);
 
@@ -509,7 +479,7 @@ void pqLineChart::layoutChart(const QRect &area)
               // Get the error width for the sequence.
               pqChartValue width;
               (*iter)->Series->getErrorWidth(i, width);
-              error->Width = xScale->getPixelFor(coord.X + width);
+              error->Width = xScale->getPixelF(coord.X + width);
               error->Width -= point.x();
               }
 
@@ -519,8 +489,8 @@ void pqLineChart::layoutChart(const QRect &area)
             // Get the error bounds for the point.
             pqChartValue upper, lower;
             (*iter)->Series->getErrorBounds(i, j, upper, lower);
-            errorData.Upper = yScale->getPixelFor(upper);
-            errorData.Lower = yScale->getPixelFor(lower);
+            errorData.Upper = yScale->getPixelF(upper);
+            errorData.Lower = yScale->getPixelF(lower);
 
             error->Sequence.append(errorData);
             }
@@ -597,11 +567,11 @@ void pqLineChart::drawChart(QPainter &painter, const QRect &area)
           }
 
         // Translate the painter for each point.
-        QPolygon::Iterator jter = points->Sequence.begin();
+        QPolygonF::Iterator jter = points->Sequence.begin();
         for( ; jter != points->Sequence.end(); ++jter)
           {
           // Make sure the point is in the clip area.
-          if(clipArea.contains(*jter))
+          if(clipArea.contains((int)jter->x(), (int)jter->y()))
             {
             painter.save();
             painter.translate(*jter);
@@ -620,7 +590,7 @@ void pqLineChart::drawChart(QPainter &painter, const QRect &area)
           {
           // Draw each polyline segment of the line. The line is broken
           // up to avoid a Qt bug when the polyline is large.
-          QList<QPolygon>::Iterator jter = line->Sequence.begin();
+          QList<QPolygonF>::Iterator jter = line->Sequence.begin();
           for( ; jter != line->Sequence.end(); ++jter)
             {
             painter.drawPolyline(*jter);
@@ -635,15 +605,16 @@ void pqLineChart::drawChart(QPainter &painter, const QRect &area)
             {
             // Draw a line from the upper bounds to the lower bounds
             // through the point.
-            painter.drawLine(jter->X, jter->Upper, jter->X, jter->Lower);
+            painter.drawLine(QPointF(jter->X, jter->Upper), 
+                QPointF(jter->X, jter->Lower));
 
             // Draw lines to cap the error bounds.
             if(error->Width > 0)
               {
-              painter.drawLine(jter->X - error->Width, jter->Upper,
-                  jter->X + error->Width, jter->Upper);
-              painter.drawLine(jter->X - error->Width, jter->Lower,
-                  jter->X + error->Width, jter->Lower);
+              painter.drawLine(QPointF(jter->X - error->Width, jter->Upper),
+                  QPointF(jter->X + error->Width, jter->Upper));
+              painter.drawLine(QPointF(jter->X - error->Width, jter->Lower),
+                  QPointF(jter->X + error->Width, jter->Lower));
               }
             }
           }
@@ -656,6 +627,43 @@ void pqLineChart::drawChart(QPainter &painter, const QRect &area)
   painter.restore();
 }
 
+void pqLineChart::setChartArea(pqChartArea *area)
+{
+  pqChartArea *oldArea = this->getChartArea();
+  if(area == oldArea)
+    {
+    return;
+    }
+
+  if(oldArea)
+    {
+    // Disconnect from the axis signals.
+    this->disconnect(oldArea->getAxis(pqChartAxis::Left), 0, this, 0);
+    this->disconnect(oldArea->getAxis(pqChartAxis::Bottom), 0, this, 0);
+    this->disconnect(oldArea->getAxis(pqChartAxis::Right), 0, this, 0);
+    this->disconnect(oldArea->getAxis(pqChartAxis::Top), 0, this, 0);
+    }
+
+  pqChartLayer::setChartArea(area);
+  this->NeedsLayout = true;
+  if(area)
+    {
+    // Listen for axis scale changes.
+    this->connect(
+        area->getAxis(pqChartAxis::Left), SIGNAL(pixelScaleChanged()),
+        this, SLOT(handleRangeChange()));
+    this->connect(
+        area->getAxis(pqChartAxis::Bottom), SIGNAL(pixelScaleChanged()),
+        this, SLOT(handleRangeChange()));
+    this->connect(
+        area->getAxis(pqChartAxis::Right), SIGNAL(pixelScaleChanged()),
+        this, SLOT(handleRangeChange()));
+    this->connect(
+        area->getAxis(pqChartAxis::Top), SIGNAL(pixelScaleChanged()),
+        this, SLOT(handleRangeChange()));
+    }
+}
+
 void pqLineChart::handleModelReset()
 {
   // Clean up the previous layout data.
@@ -719,6 +727,13 @@ void pqLineChart::handleSeriesMoved(int current, int index)
   emit this->repaintNeeded();
 }
 
+void pqLineChart::handleSeriesAxesChanged(const pqLineChartSeries *series)
+{
+  pqLineChartSeriesItem *item = this->getItem(series);
+  item->NeedsLayout = true;
+  emit this->layoutNeeded();
+}
+
 void pqLineChart::handleSeriesReset(const pqLineChartSeries *series)
 {
   // Find the given series' layout item.
