@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqAnimationManager.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -33,6 +33,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "ui_pqAbortAnimation.h"
 #include "ui_pqAnimationSettings.h"
 
+#include "vtkMath.h"
 #include "vtkProcessModule.h"
 #include "vtkPVServerInformation.h"
 #include "vtkSMAnimationSceneGeometryWriter.h"
@@ -62,11 +63,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqServerManagerModel.h"
 #include "pqSMAdaptor.h"
 #include "pqView.h"
-
-static inline int pqCeil(double val)
-{
-  return static_cast<int>(val+0.5);
-}
+#include "pqViewManager.h"
 
 #define SEQUENCE 0
 #define REALTIME 1
@@ -77,7 +74,7 @@ class pqAnimationManager::pqInternals
 {
 public:
   QPointer<pqServer> ActiveServer;
-  QPointer<QWidget> ViewWidget;
+  QPointer<pqViewManager> ViewWidget;
   typedef QMap<pqServer*, QPointer<pqAnimationScene> > SceneMap;
   SceneMap Scenes;
   Ui::Dialog* AnimationSettingsDialog;
@@ -111,7 +108,7 @@ pqAnimationManager::~pqAnimationManager()
 }
 
 //-----------------------------------------------------------------------------
-void pqAnimationManager::setViewWidget(QWidget* w)
+void pqAnimationManager::setViewWidget(pqViewManager* w)
 {
   this->Internals->ViewWidget = w;
 }
@@ -283,6 +280,31 @@ void pqAnimationManager::updateGUI()
     }
 }
 
+#define PADDING_COMPENSATION QSize(16, 16);
+inline void enforceMultiple4(QSize& newSize)
+{
+  QSize requested_newSize = newSize;
+  int &width = newSize.rwidth();
+  int &height = newSize.rheight();
+  if ((width % 4) > 0)
+    {
+    width -= width % 4;
+    }
+  if ((height % 4) > 0)
+    {
+    height -= height % 4;
+    }
+
+  if (requested_newSize != newSize)
+    {
+    QMessageBox::warning(NULL, "Resolution Changed",
+      QString("The requested resolution has been changed from (%1, %2)\n").arg(
+        requested_newSize.width()).arg(requested_newSize.height()) + 
+      QString("to (%1, %2) to match format specifications.").arg(
+        newSize.width()).arg(newSize.height()));
+    }
+}
+
 //-----------------------------------------------------------------------------
 bool pqAnimationManager::saveAnimation()
 {
@@ -291,6 +313,8 @@ bool pqAnimationManager::saveAnimation()
     {
     return false;
     }
+  // Ensure that GUI is up-to-date so that we get correct sizes.
+  pqEventDispatcher::processEventsAndWait(1);
   vtkSMAnimationSceneProxy* sceneProxy = scene->getAnimationSceneProxy();
 
   QDialog dialog;
@@ -303,7 +327,10 @@ bool pqAnimationManager::saveAnimation()
     this->Internals->ActiveServer->isRemote());
 
   // Set current size of the window.
-  QSize viewSize = scene->getViewSize();
+  QSize viewSize = this->Internals->ViewWidget->clientSize();
+  // to avoid some unpredicable padding issues, I am reducing the size by a few
+  // pixels.
+  viewSize -= PADDING_COMPENSATION;
   dialogUI.spinBoxHeight->setValue(viewSize.height());
   dialogUI.spinBoxWidth->setValue(viewSize.width());
 
@@ -403,11 +430,19 @@ bool pqAnimationManager::saveAnimation()
   filters +="JPEG images (*.jpg);;TIFF images (*.tif);;PNG images (*.png);;";
   filters +="All files(*)";
 
+  QWidget* parent_window = qobject_cast<QWidget*>(this->parent());
+  if (!parent_window)
+    {
+    // QApplication::activateWindow() is set up differntly on Macs resulting in
+    // ambiguities in test playback. Hence we use it as the last resort.
+    parent_window = QApplication::activeWindow();
+    }
+
   // Create a server dialog is disconnect-and-save is true, else create a client
   // dialog.
   pqFileDialog *file_dialog = new pqFileDialog(
     disconnect_and_save?  scene->getServer() : 0,
-    QApplication::activeWindow(),
+    parent_window,
     tr("Save Animation"), QString(), filters);
   file_dialog->setObjectName("FileSaveAnimationDialog");
   file_dialog->setFileMode(pqFileDialog::AnyFile);
@@ -462,7 +497,8 @@ bool pqAnimationManager::saveAnimation()
     dialogUI.spinBoxHeight->value());
 
   // Enforce any view size conditions (such a multiple of 4). 
-  int magnification = this->updateViewSizes(newSize, viewSize);
+  ::enforceMultiple4(newSize); 
+  int magnification = this->Internals->ViewWidget->prepareForCapture(newSize);
  
   if (disconnect_and_save)
     {
@@ -498,7 +534,7 @@ bool pqAnimationManager::saveAnimation()
     reviver->Delete();
     emit this->endNonUndoableChanges();
     pqApplicationCore::instance()->getObjectBuilder()->removeServer(server);
-    this->restoreViewSizes();
+    this->Internals->ViewWidget->finishedCapture();
     emit this->disconnectServer();
     return status;
     }
@@ -549,73 +585,11 @@ bool pqAnimationManager::saveAnimation()
     break;
     }
   sceneProxy->UpdateVTKObjects();
-
-  this->restoreViewSizes();
+  this->Internals->ViewWidget->finishedCapture();
   emit this->endNonUndoableChanges();
   return status;
 }
 
-//-----------------------------------------------------------------------------
-int pqAnimationManager::updateViewSizes(QSize newSize, QSize currentSize)
-{
-  QSize requested_newSize = newSize;
-  int &width = newSize.rwidth();
-  int &height = newSize.rheight();
-  if ((width % 4) > 0)
-    {
-    width -= width % 4;
-    }
-  if ((height % 4) > 0)
-    {
-    height -= height % 4;
-    }
-
-  if (requested_newSize != newSize)
-    {
-    QMessageBox::warning(NULL, "Resolution Changed",
-      QString("The requested resolution has been changed from (%1, %2)\n").arg(
-        requested_newSize.width()).arg(requested_newSize.height()) + 
-      QString("to (%1, %2) to match format specifications.").arg(
-        newSize.width()).arg(newSize.height()));
-    }
-
-  int magnification = 1;
-
-  // If newSize > currentSize, then magnification is involved.
-  int temp = pqCeil(newSize.width()/static_cast<double>(currentSize.width()));
-  magnification = (temp> magnification)? temp: magnification;
-
-  temp = pqCeil(newSize.height()/static_cast<double>(currentSize.height()));
-  magnification = (temp > magnification)? temp : magnification;
-
-  newSize = newSize/magnification;
-
-  if (!this->Internals->ViewWidget)
-    {
-    qDebug() << "ViewWidget must be set to the parent of all views.";
-    }
-  else
-    {
-    this->Internals->OldSize = this->Internals->ViewWidget->size();
-    this->Internals->OldMaxSize = this->Internals->ViewWidget->maximumSize();
-    this->Internals->ViewWidget->setMaximumSize(newSize);
-    this->Internals->ViewWidget->resize(newSize);
-    pqEventDispatcher::processEventsAndWait(1);
-    }
-
-  return magnification;
-}
-
-//-----------------------------------------------------------------------------
-void pqAnimationManager::restoreViewSizes()
-{
-  if (this->Internals->ViewWidget)
-    {
-    this->Internals->ViewWidget->setMaximumSize(this->Internals->OldMaxSize);
-    this->Internals->ViewWidget->resize(this->Internals->OldSize);
-    }
-}
-
 //-----------------------------------------------------------------------------
 bool pqAnimationManager::saveGeometry(const QString& filename, 
   pqView* view)
