@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqAnimationPanel.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -54,7 +54,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqAnimationManager.h"
 #include "pqAnimationScene.h"
 #include "pqApplicationCore.h"
-#include "pqDoubleSpinBoxDomain.h"
 #include "pqKeyFrameTimeValidator.h"
 #include "pqOutputPort.h"
 #include "pqPipelineSource.h"
@@ -111,7 +110,6 @@ pqAnimationPanel::pqAnimationPanel(QWidget* _parent) : QWidget(_parent)
   vboxlayout->setObjectName("vboxLayout");
 
   QWidget* container = new QWidget(this);
-  container->setObjectName("scrollWidget");
   container->setSizePolicy(QSizePolicy::MinimumExpanding,
     QSizePolicy::MinimumExpanding);
 
@@ -197,6 +195,7 @@ pqAnimationPanel::pqAnimationPanel(QWidget* _parent) : QWidget(_parent)
 
   this->Internal->ValueAdaptor = new pqSignalAdaptorKeyFrameValue(
     this->Internal->valueFrameLarge, this->Internal->valueFrame);
+  this->Internal->valueFrame->setObjectName("SignalAdaptorKeyFrameValue");
 
   this->Internal->TypeAdaptor = new pqSignalAdaptorKeyFrameType(
     this->Internal->keyFrameWidget, 
@@ -436,6 +435,8 @@ void pqAnimationPanel::onScenePlayModeChanged()
       {
       this->Internal->ToolbarCurrentTimeIndexWidget->setEnabled(true);
       }
+    this->onTimeChanged(
+      this->Internal->ActiveScene->getAnimationTime());
     }
 }
 
@@ -1025,6 +1026,7 @@ void pqAnimationPanel::updateToolbarCurrentTime(const QString& str)
   if (this->Internal->ToolbarCurrentTimeWidget->text() != str)
     {
     this->Internal->ToolbarCurrentTimeWidget->setText(str);
+    this->Internal->ToolbarCurrentTimeWidget->setCursorPosition(0);
     }
 }
 
