@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqAnimationViewWidget.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -166,9 +166,20 @@ public:
             {
             vtkSMProperty* prop =
               pqproxy->getProxy()->GetProperty(key.toAscii().data());
-            QString pp = prop->GetXMLLabel();
             QString n = pqproxy->getSMName();
-            name = QString("%1 - %2 - %3").arg(n).arg(pp).arg(p);
+            if (prop)
+              {
+              QString pp = prop->GetXMLLabel();
+              name = QString("%1 - %2 - %3").arg(n).arg(pp).arg(p);
+              }
+            else
+              {
+              name = QString("%1 - %2").arg(n).arg(p);
+              if (helpers.size() > 0)
+                {
+                name = QString("%1 [%2]").arg(name).arg(idx);
+                }
+              }
             }
           }
         }
@@ -207,6 +218,19 @@ public:
       }
     return num;
     }
+
+  QList<QVariant> ticks()
+    {
+    vtkSMProxy* pxy = this->Scene->getProxy();
+    QString mode =
+      pqSMAdaptor::getEnumerationProperty(pxy->GetProperty("PlayMode")).toString();
+    if(mode == "Snap To TimeSteps")
+      {
+      return pqSMAdaptor::getMultipleElementProperty(
+        pxy->GetProperty("TimeSteps"));
+      }
+    return QList<QVariant>();
+    }
 };
 
 //-----------------------------------------------------------------------------
@@ -489,7 +513,7 @@ void pqAnimationViewWidget::updateSceneTimeRange()
 void pqAnimationViewWidget::updateSceneTime()
 {
   double time =
-    this->Internal->Scene->getAnimationSceneProxy()->GetAnimationTime();
+    this->Internal->Scene->getAnimationTime();
 
   pqAnimationModel* animModel =
     this->Internal->AnimationWidget->animationModel();
@@ -624,7 +648,7 @@ void pqAnimationViewWidget::updatePlayMode()
     }
   else if(mode == "Snap To TimeSteps")
     {
-    animModel->setMode(pqAnimationModel::Sequence);
+    animModel->setMode(pqAnimationModel::Custom);
     this->Internal->Duration->setEnabled(false);
     this->Internal->DurationLabel->setEnabled(false);
     this->Internal->StartTime->setEnabled(false);
@@ -642,8 +666,21 @@ void pqAnimationViewWidget::updateTicks()
 {
   pqAnimationModel* animModel =
     this->Internal->AnimationWidget->animationModel();
-  int num = this->Internal->numberOfTicks();
-  animModel->setTicks(num);
+  if (animModel->mode() == pqAnimationModel::Custom)
+    {
+    QList<QVariant> ticks = this->Internal->ticks();
+    double *dticks = new double[ticks.size()+1];
+    for (int cc=0; cc < ticks.size(); cc++)
+      {
+      dticks[cc] = ticks[cc].toDouble();
+      }
+    animModel->setTickMarks(ticks.size(), dticks);
+    delete [] dticks;
+    }
+  else
+    {
+    animModel->setTicks(this->Internal->numberOfTicks());
+    }
 }
 
 void pqAnimationViewWidget::deleteTrack(pqAnimationTrack* track)
@@ -780,7 +817,7 @@ void pqAnimationViewWidget::createTrack()
     vtkSMProperty* prop = curProxy->GetProperty(pname.toAscii().data());
     QList<QVariant> mins;
     QList<QVariant> maxs;
-    if(pindex == -1)
+    if(pindex == -1 && prop)
       {
       QList<QList<QVariant> > domains =
         pqSMAdaptor::getMultipleElementPropertyDomain(prop);
