@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqColorPresetManager.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -522,6 +522,10 @@ void pqColorPresetManager::importColorMap(vtkPVXMLElement *element)
     {
     colorMap.setColorSpace(pqColorMapModel::WrappedHsvSpace);
     }
+  else if(space == "Diverging")
+    {
+    colorMap.setColorSpace(pqColorMapModel::DivergingSpace);
+    }
   else
     {
     colorMap.setColorSpace(pqColorMapModel::HsvSpace);
@@ -607,7 +611,7 @@ void pqColorPresetManager::exportColorMap(const QModelIndex &index,
     }
 
   // Get the color space and points from the color map model.
-  const char *spaceNames[] = {"RGB", "HSV", "Wrapped", "Lab"};
+  const char *spaceNames[] = {"RGB", "HSV", "Wrapped", "Lab", "Diverging"};
   const pqColorMapModel *colorMap = this->Model->getColorMap(index.row());
   if(colorMap)
     {
