@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqCustomFilterDefinitionWizard.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -38,8 +38,9 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include "pqCustomFilterDefinitionModel.h"
 #include "pqCustomFilterManagerModel.h"
-#include "pqPipelineSource.h"
 #include "pqFlatTreeView.h"
+#include "pqOutputPort.h"
+#include "pqPipelineSource.h"
 
 #include <QHeaderView>
 #include <QMessageBox>
@@ -49,7 +50,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <QTreeWidgetItem>
 
 #include "vtkPVXMLElement.h"
-#include "vtkSMCompoundProxy.h"
+#include "vtkSMCompoundSourceProxy.h"
 #include "vtkSMInputProperty.h"
 #include "vtkSMPropertyIterator.h"
 #include "vtkSMProxy.h"
@@ -61,13 +62,21 @@ class pqCustomFilterDefinitionWizardForm :
     public Ui::pqCustomFilterDefinitionWizard
 {
 public:
-  QStringList ListNames; ///< Used to make sure names are unique.
+  QStringList ExposedPropertyNames; ///< Used to ensure all exposed 
+                                    ///  property names are unique.
+ 
+  QStringList ExposedPortNames;     ///< Used to ensure all exposed 
+                                    ///  port names are unique.
+  QStringList ToExposeNames;        ///< Used to ensure that same thing
+             /// (either property or port) is not exposed twice.
+
   // Maps a proxy name with a map of <propertyLabel, propertyName>
   // pairs for each of its properties. This assumes that labels of
   // a proxy's properties are unique.
   QMap< QString, QMap<QString,QString> > LabelToNamePropertyMap; 
 };
 
+//-----------------------------------------------------------------------------
 pqCustomFilterDefinitionWizard::pqCustomFilterDefinitionWizard(
     pqCustomFilterDefinitionModel *model, QWidget *widgetParent)
   : QDialog(widgetParent)
@@ -168,8 +177,14 @@ pqCustomFilterDefinitionWizard::pqCustomFilterDefinitionWizard(
   QObject::connect(this->Form->CustomFilterName,
       SIGNAL(textEdited(const QString &)),
       this, SLOT(clearNameOverwrite(const QString &)));
+
+  // When combo selection changes, update new label to be same as old.
+  QObject::connect(this->Form->PropertyCombo,
+    SIGNAL(currentIndexChanged(const QString&)),
+    this->Form->PropertyName, SLOT(setText(const QString&)));
 }
 
+//-----------------------------------------------------------------------------
 pqCustomFilterDefinitionWizard::~pqCustomFilterDefinitionWizard()
 {
   if(this->Form)
@@ -184,7 +199,7 @@ pqCustomFilterDefinitionWizard::~pqCustomFilterDefinitionWizard()
     }
 }
 
-
+//-----------------------------------------------------------------------------
 QString pqCustomFilterDefinitionWizard::getCustomFilterName() const
 {
   if(this->Filter)
@@ -195,6 +210,7 @@ QString pqCustomFilterDefinitionWizard::getCustomFilterName() const
   return QString();
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::createCustomFilter()
 {
   if(this->Filter != 0 || this->Form->CustomFilterName->text().isEmpty())
@@ -204,13 +220,20 @@ void pqCustomFilterDefinitionWizard::createCustomFilter()
 
   // Create the compound proxy. Add all the proxies to it.
   pqPipelineSource *source = 0;
-  this->Filter = vtkSMCompoundProxy::New();
+  this->Filter = vtkSMCompoundSourceProxy::New();
+  bool first = true;
   QModelIndex index = this->Model->getNextIndex(QModelIndex());
   while(index.isValid())
     {
     source = this->Model->getSourceFor(index);
     if(source)
       {
+      if (first)
+        {
+        this->Filter->SetConnectionID(source->getProxy()->GetConnectionID());
+        this->Filter->SetServers(source->getProxy()->GetServers());
+        first = false;
+        }
       this->Filter->AddProxy(source->getSMName().toAscii().data(),
           source->getProxy());
       }
@@ -221,8 +244,8 @@ void pqCustomFilterDefinitionWizard::createCustomFilter()
   // Expose the input properties.
   int i = 0;
   QTreeWidgetItem *item = 0;
-  int total = this->Form->InputPorts->topLevelItemCount();
-  for( ; i < total; i++)
+  int numInputs = this->Form->InputPorts->topLevelItemCount();
+  for( ; i < numInputs; i++)
     {
     item = this->Form->InputPorts->topLevelItem(i);
     this->Filter->ExposeProperty(item->text(0).toAscii().data(),
@@ -231,12 +254,19 @@ void pqCustomFilterDefinitionWizard::createCustomFilter()
     }
 
   // Set the output proxies.
-  total = this->Form->OutputPorts->topLevelItemCount();
+  int total = this->Form->OutputPorts->topLevelItemCount();
   for(i = 0; i < total; i++)
     {
     item = this->Form->OutputPorts->topLevelItem(i);
-    this->Filter->SetConsumableProxy(item->text(0).toAscii().data());
-    break; // TEMP: Change when multiple outputs supported.
+    pqOutputPort* port = qobject_cast<pqOutputPort*>(
+      item->data(0, Qt::UserRole).value<QObject*>());
+    if (port)
+      {
+      this->Filter->ExposeOutputPort(
+        port->getSource()->getSMName().toAscii().data(),
+        port->getPortNumber(),
+        item->text(1).toAscii().data());
+      }
     }
 
   // Expose the properties.
@@ -255,8 +285,16 @@ void pqCustomFilterDefinitionWizard::createCustomFilter()
   // Register the compound proxy definition with the server manager.
   vtkPVXMLElement *root = this->Filter->SaveDefinition(0);
   vtkSMProxyManager *proxyManager = vtkSMProxyManager::GetProxyManager();
-  proxyManager->RegisterCompoundProxyDefinition(
+  if (numInputs > 0)
+    {
+    proxyManager->RegisterCustomProxyDefinition("filters",
       this->Form->CustomFilterName->text().toAscii().data(), root);
+    }
+  else
+    {
+    proxyManager->RegisterCustomProxyDefinition("sources",
+      this->Form->CustomFilterName->text().toAscii().data(), root);
+    }
   root->Delete();
 }
 
@@ -319,10 +357,10 @@ bool pqCustomFilterDefinitionWizard::validateCustomFilterName()
 
   // Make sure the name is unique.
   vtkSMProxyManager *proxyManager = vtkSMProxyManager::GetProxyManager();
-  if(!this->OverwriteOK)
+  if (!this->OverwriteOK)
     {
-    if(proxyManager->GetCompoundProxyDefinition(filterName.toAscii().data()) 
-      || proxyManager->GetProxy("filters_prototypes",filterName.toAscii().data()))
+    if (proxyManager->GetProxyDefinition("filters", filterName.toAscii().data()) || 
+      proxyManager->GetProxyDefinition("sources", filterName.toAscii().data()))
       {
       QMessageBox::warning(this, "Duplicate Name",
           "This filter name already exists.\n"
@@ -335,6 +373,7 @@ bool pqCustomFilterDefinitionWizard::validateCustomFilterName()
   return true;
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::setupDefaultInputOutput()
 {
   if(this->Model->rowCount() == 1)
@@ -379,15 +418,17 @@ void pqCustomFilterDefinitionWizard::setupDefaultInputOutput()
           this->Form->LabelToNamePropertyMap[source->getSMName()][propertyName] = 
               propertyName;
           this->Form->InputPorts->setCurrentItem(item);
-          this->Form->ListNames.append("Input");
+          this->Form->ExposedPropertyNames.append("Input");
+          this->Form->ToExposeNames.append(
+            QString("INPUT:%1.%2").arg(item->text(0)).arg(item->text(1)));
           }
         }
       }
 
     // Set up the default output property.
-    while(this->Model->hasChildren(index))
+    while (this->Model->hasChildren(index))
       {
-      if(this->Model->rowCount(index) == 1)
+      if(this->Model->rowCount(index) > 0)
         {
         index = this->Model->index(0, 0, index);
         }
@@ -401,18 +442,12 @@ void pqCustomFilterDefinitionWizard::setupDefaultInputOutput()
     pqPipelineSource *output = this->Model->getSourceFor(index);
     if(output)
       {
-      // Add the exposed output port to the list.
-      QStringList list;
-      list.append(output->getSMName());
-      list.append("Output");
-      QTreeWidgetItem *item = new QTreeWidgetItem(this->Form->OutputPorts,
-          list);
-      this->Form->OutputPorts->setCurrentItem(item);
-      this->Form->ListNames.append("Output");
+      this->addOutputInternal(output->getOutputPort(0), "Output");
       }
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::navigateBack()
 {
   if(this->CurrentPage > 0)
@@ -433,6 +468,7 @@ void pqCustomFilterDefinitionWizard::navigateBack()
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::navigateNext()
 {
   if(this->CurrentPage < 3)
@@ -458,6 +494,7 @@ void pqCustomFilterDefinitionWizard::navigateNext()
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::finishWizard()
 {
   // Make sure the name has been entered and is unique.
@@ -467,11 +504,13 @@ void pqCustomFilterDefinitionWizard::finishWizard()
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::clearNameOverwrite(const QString &)
 {
   this->OverwriteOK = false;
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::updateInputForm(const QModelIndex &current,
     const QModelIndex &)
 {
@@ -510,13 +549,33 @@ void pqCustomFilterDefinitionWizard::updateInputForm(const QModelIndex &current,
     }
 }
 
-void pqCustomFilterDefinitionWizard::updateOutputForm(const QModelIndex &,
+//-----------------------------------------------------------------------------
+void pqCustomFilterDefinitionWizard::updateOutputForm(const QModelIndex &current,
     const QModelIndex &)
 {
   // Clear the output name field.
   this->Form->OutputName->setText("");
+  this->Form->OutputCombo->clear();
+
+  // * Get the proxy from the index. 
+  // * Find the names for all output ports for that proxy.
+  // * Populate the OutputCombo with available options.
+  pqPipelineSource *source = this->Model->getSourceFor(current);
+  if(source)
+    {
+    vtkSMSourceProxy *proxy = vtkSMSourceProxy::SafeDownCast(source->getProxy());
+    if(proxy)
+      {
+      unsigned int numPorts = proxy->GetNumberOfOutputPorts();
+      for (unsigned int cc=0; cc < numPorts; cc++)
+        {
+        this->Form->OutputCombo->addItem(proxy->GetOutputPortName(cc));
+        }
+      }
+    }
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::updatePropertyForm(
     const QModelIndex &current, const QModelIndex &)
 {
@@ -557,6 +616,7 @@ void pqCustomFilterDefinitionWizard::updatePropertyForm(
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::addInput()
 {
   // Validate the entry. Make sure there is an object and a property
@@ -593,7 +653,7 @@ void pqCustomFilterDefinitionWizard::addInput()
     return;
     }
 
-  if(this->Form->ListNames.contains(name))
+  if(this->Form->ExposedPropertyNames.contains(name))
     {
     QMessageBox::warning(this, "Duplicate Name",
         "Another input already has the name entered.\n"
@@ -604,6 +664,16 @@ void pqCustomFilterDefinitionWizard::addInput()
     return;
     }
 
+  QString key = QString("INPUT:%1.%2").arg(source->getSMName()).arg(
+    this->Form->InputCombo->itemText(propertyIndex));
+  if (this->Form->ToExposeNames.contains(key))
+    {
+    QMessageBox::warning(this, "Duplicate Input",
+      "The selected Input has already been added.",
+      QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton);
+    return;
+    }
+
   // Add the exposed input port to the list.
   QStringList list;
   list.append(source->getSMName());
@@ -611,9 +681,11 @@ void pqCustomFilterDefinitionWizard::addInput()
   list.append(name);
   QTreeWidgetItem *item = new QTreeWidgetItem(this->Form->InputPorts, list);
   this->Form->InputPorts->setCurrentItem(item);
-  this->Form->ListNames.append(name);
+  this->Form->ExposedPropertyNames.append(name);
+  this->Form->ToExposeNames.append(key);
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::removeInput()
 {
   // Remove the selected row from the list.
@@ -621,7 +693,10 @@ void pqCustomFilterDefinitionWizard::removeInput()
   if(item)
     {
     int row = this->Form->InputPorts->indexOfTopLevelItem(item) - 1;
-    this->Form->ListNames.removeAll(item->text(2));
+    this->Form->ExposedPropertyNames.removeAll(item->text(2));
+    QString key = QString("INPUT:%1.%2").arg(item->text(0)).arg(
+      item->text(1));
+    this->Form->ToExposeNames.removeAll(key);
     delete item;
     if(row < 0)
       {
@@ -640,6 +715,7 @@ void pqCustomFilterDefinitionWizard::removeInput()
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::moveInputUp()
 {
   // Move the selected row up one if possible.
@@ -656,6 +732,7 @@ void pqCustomFilterDefinitionWizard::moveInputUp()
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::moveInputDown()
 {
   // Move the selected row down one if possible.
@@ -672,6 +749,7 @@ void pqCustomFilterDefinitionWizard::moveInputDown()
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::addOutput()
 {
   // Validate the entry. Make sure there is an object and a property
@@ -698,7 +776,7 @@ void pqCustomFilterDefinitionWizard::addOutput()
     return;
     }
 
-  if(this->Form->ListNames.contains(name))
+  if(this->Form->ExposedPortNames.contains(name))
     {
     QMessageBox::warning(this, "Duplicate Name",
         "Another output already has the name entered.\n"
@@ -709,24 +787,61 @@ void pqCustomFilterDefinitionWizard::addOutput()
     return;
     }
 
-  // TEMP: Change when multiple outputs supported.
-  if(this->Form->OutputPorts->topLevelItemCount() > 0)
+  // Validate port name.
+  QString portname = this->Form->OutputCombo->currentText();
+  pqOutputPort* port = source->getOutputPort(portname);
+  if (!port)
     {
-    QMessageBox::information(this, "Temporary Limitation",
-        "Only one output port is currently supported.\n"
-        "The first item in the output port list will be used.",
+    QMessageBox::warning(this, "No Output Port Selected",
+        "No output port was selected or selected output port does not exist.\n"
+        "Please select a output port from the \"Output Port\" combo box.",
         QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton);
+    this->Form->OutputCombo->setFocus();
+    return;
     }
 
-  // Add the exposed output port to the list.
-  QStringList list;
-  list.append(source->getSMName());
-  list.append(name);
-  QTreeWidgetItem *item = new QTreeWidgetItem(this->Form->OutputPorts, list);
-  this->Form->OutputPorts->setCurrentItem(item);
-  this->Form->ListNames.append(name);
+  // Verify that the selected port hasn't already been exposed. 
+  // We don't want to export the same output port twice.
+  QString key = QString("OUTPUT:%1 (%2)").arg(source->getSMName()).arg(port->getPortNumber());
+  if (this->Form->ToExposeNames.contains(key))
+    {
+    QMessageBox::warning(this, "Duplicate Output",
+      "Selected output port has already been exposed.",
+        QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton);
+    this->Form->OutputCombo->setFocus();
+    return;
+    }
+
+  this->addOutputInternal(port, name);
 }
 
+//-----------------------------------------------------------------------------
+void pqCustomFilterDefinitionWizard::addOutputInternal(
+  pqOutputPort* port, const QString& name)
+{
+  pqPipelineSource* source = port->getSource();
+  QString key = QString("OUTPUT:%1 (%2)").arg(source->getSMName()).
+    arg(port->getPortNumber());
+  // Add the exposed output port to the list.
+  QStringList list;
+  if (source->getNumberOfOutputPorts() > 1)
+    {
+    list.append(QString("%1 (%2)").arg(source->getSMName()).
+      arg(port->getPortNumber()));
+    }
+  else
+    {
+    list.append(source->getSMName());
+    }
+  list.append(name);
+  QTreeWidgetItem *item = new QTreeWidgetItem(this->Form->OutputPorts, list);
+  item->setData(0, Qt::UserRole, QVariant::fromValue<QObject*>(port));
+  this->Form->OutputPorts->setCurrentItem(item);
+  this->Form->ExposedPortNames.append(name);
+  this->Form->ToExposeNames.append(key);
+}
+
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::removeOutput()
 {
   // Remove the selected row from the list.
@@ -734,7 +849,13 @@ void pqCustomFilterDefinitionWizard::removeOutput()
   if(item)
     {
     int row = this->Form->OutputPorts->indexOfTopLevelItem(item) - 1;
-    this->Form->ListNames.removeAll(item->text(1));
+    this->Form->ExposedPortNames.removeAll(item->text(1));
+    pqOutputPort* port = qobject_cast<pqOutputPort*>(
+      item->data(0, Qt::UserRole).value<QObject*>());
+    pqPipelineSource* source = port->getSource();
+    QString key = QString("OUTPUT:%1 (%2)").arg(source->getSMName()).
+      arg(port->getPortNumber());
+    this->Form->ToExposeNames.removeAll(key);
     delete item;
     if(row < 0)
       {
@@ -753,6 +874,7 @@ void pqCustomFilterDefinitionWizard::removeOutput()
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::moveOutputUp()
 {
   // Move the selected row up one if possible.
@@ -769,6 +891,7 @@ void pqCustomFilterDefinitionWizard::moveOutputUp()
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::moveOutputDown()
 {
   // Move the selected row down one if possible.
@@ -785,6 +908,7 @@ void pqCustomFilterDefinitionWizard::moveOutputDown()
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::addProperty()
 {
   // Validate the entry. Make sure there is an object and a property
@@ -821,7 +945,7 @@ void pqCustomFilterDefinitionWizard::addProperty()
     return;
     }
 
-  if(this->Form->ListNames.contains(name))
+  if(this->Form->ExposedPropertyNames.contains(name))
     {
     QMessageBox::warning(this, "Duplicate Name",
         "Another property already has the name entered.\n"
@@ -832,6 +956,18 @@ void pqCustomFilterDefinitionWizard::addProperty()
     return;
     }
 
+  QString key = QString("INPUT:%1.%2").arg(source->getSMName()).arg(
+    this->Form->PropertyCombo->itemText(propertyIndex));
+  if (this->Form->ToExposeNames.contains(key))
+    {
+    QMessageBox::warning(this, "Duplicate Property",
+        "The selected property have already been exposed.\n"
+        "Please select another property.",
+        QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton);
+    this->Form->PropertyCombo->setFocus();
+    return;
+    }
+
   // Add the exposed property to the list.
   QStringList list;
   list.append(source->getSMName());
@@ -839,9 +975,11 @@ void pqCustomFilterDefinitionWizard::addProperty()
   list.append(name);
   QTreeWidgetItem *item = new QTreeWidgetItem(this->Form->PropertyList, list);
   this->Form->PropertyList->setCurrentItem(item);
-  this->Form->ListNames.append(name);
+  this->Form->ExposedPropertyNames.append(name);
+  this->Form->ToExposeNames.append(key);
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::removeProperty()
 {
   // Remove the selected row from the list.
@@ -849,7 +987,10 @@ void pqCustomFilterDefinitionWizard::removeProperty()
   if(item)
     {
     int row = this->Form->PropertyList->indexOfTopLevelItem(item) - 1;
-    this->Form->ListNames.removeAll(item->text(2));
+    this->Form->ExposedPropertyNames.removeAll(item->text(2));
+    QString key = QString("INPUT:%1.%2").arg(item->text(0)).arg(
+      item->text(1));
+    this->Form->ToExposeNames.removeAll(key);
     delete item;
     if(row < 0)
       {
@@ -868,6 +1009,7 @@ void pqCustomFilterDefinitionWizard::removeProperty()
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::movePropertyUp()
 {
   // Move the selected row up one if possible.
@@ -884,6 +1026,7 @@ void pqCustomFilterDefinitionWizard::movePropertyUp()
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::movePropertyDown()
 {
   // Move the selected row down one if possible.
@@ -900,6 +1043,7 @@ void pqCustomFilterDefinitionWizard::movePropertyDown()
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::updateInputButtons(
     const QModelIndex &current, const QModelIndex &)
 {
@@ -909,6 +1053,7 @@ void pqCustomFilterDefinitionWizard::updateInputButtons(
   this->Form->InputDownButton->setEnabled(indexIsValid);
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::updateOutputButtons(
     const QModelIndex &current, const QModelIndex &)
 {
@@ -918,6 +1063,7 @@ void pqCustomFilterDefinitionWizard::updateOutputButtons(
   this->Form->OutputDownButton->setEnabled(indexIsValid);
 }
 
+//-----------------------------------------------------------------------------
 void pqCustomFilterDefinitionWizard::updatePropertyButtons(
     const QModelIndex &current, const QModelIndex &)
 {
