@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqDisplayProxyEditor.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -35,8 +35,11 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "ui_pqDisplayProxyEditor.h"
 
 // Qt includes
+#include <QDoubleValidator>
 #include <QFileInfo>
 #include <QIcon>
+#include <QIntValidator>
+#include <QKeyEvent>
 #include <QMetaType>
 #include <QPointer>
 #include <QtDebug>
@@ -44,32 +47,38 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 // ParaView Server Manager includes
 #include "vtkEventQtSlotConnect.h"
-#include "vtkMaterialLibrary.h"
 #include "vtkLabeledDataMapper.h"
+#include "vtkMaterialLibrary.h"
 #include "vtkPVArrayInformation.h"
-#include "vtkPVDataSetAttributesInformation.h"
 #include "vtkPVDataInformation.h"
+#include "vtkPVDataSetAttributesInformation.h"
 #include "vtkSMLookupTableProxy.h"
 #include "vtkSMProperty.h"
 #include "vtkSMProxyProperty.h"
 #include "vtkSMPVRepresentationProxy.h"
 #include "vtkSMRenderViewProxy.h"
 #include "vtkSMSourceProxy.h"
+#include "vtkSMCompositeTreeDomain.h"
+#include "vtkSMIntVectorProperty.h"
 
 // ParaView widget includes
 #include "pqSignalAdaptors.h"
 
 // ParaView client includes
 #include "pqApplicationCore.h"
-#include "pqColorScaleEditor.h"
-#include "pqSMAdaptor.h"
-#include "pqPropertyLinks.h"
+#include "pqColorScaleToolbar.h"
+#include "pqCubeAxesEditorDialog.h"
+#include "pqFileDialog.h"
+#include "pqOutputPort.h"
 #include "pqPipelineRepresentation.h"
 #include "pqPipelineSource.h"
+#include "pqPropertyLinks.h"
 #include "pqRenderView.h"
-#include "pqFileDialog.h"
 #include "pqScalarsToColors.h"
-
+#include "pqSignalAdaptorCompositeTreeWidget.h"
+#include "pqSMAdaptor.h"
+#include "pqUndoStack.h"
+#include "pqWidgetRangeDomain.h"
 
 class pqDisplayProxyEditorInternal : public Ui::pqDisplayProxyEditor
 {
@@ -78,14 +87,19 @@ public:
     {
     this->Links = new pqPropertyLinks;
     this->InterpolationAdaptor = 0;
-    this->ColorAdaptor = 0;
     this->EdgeColorAdaptor = 0;
+    this->SliceDirectionAdaptor = 0;
+    this->SliceDomain = 0;
+    this->SelectedMapperAdaptor = 0;
+    this->CompositeTreeAdaptor = 0;
     }
 
   ~pqDisplayProxyEditorInternal()
     {
     delete this->Links;
     delete this->InterpolationAdaptor;
+    delete this->SliceDirectionAdaptor;
+    delete this->SliceDomain;
     }
 
   pqPropertyLinks* Links;
@@ -93,8 +107,11 @@ public:
   // The representation whose properties are being edited.
   QPointer<pqPipelineRepresentation> Representation;
   pqSignalAdaptorComboBox* InterpolationAdaptor;
-  pqSignalAdaptorColor*    ColorAdaptor;
   pqSignalAdaptorColor*    EdgeColorAdaptor;
+  pqSignalAdaptorComboBox* SliceDirectionAdaptor;
+  pqSignalAdaptorComboBox* SelectedMapperAdaptor;
+  pqWidgetRangeDomain* SliceDomain;
+  pqSignalAdaptorCompositeTreeWidget* CompositeTreeAdaptor;
 
   // map of <material labels, material files>
   static QMap<QString, QString> MaterialMap;
@@ -109,8 +126,6 @@ pqDisplayProxyEditor::pqDisplayProxyEditor(pqPipelineRepresentation* repr, QWidg
 {
   this->Internal = new pqDisplayProxyEditorInternal;
   this->Internal->setupUi(this);
-  this->Internal->StyleMaterial->hide();  // FIXME
-  this->Internal->StyleMaterialLabel->hide();  // FIXME
   this->setupGUIConnections();
 
   // setting a repr proxy will enable this
@@ -118,8 +133,14 @@ pqDisplayProxyEditor::pqDisplayProxyEditor(pqPipelineRepresentation* repr, QWidg
 
   this->setRepresentation(repr);
 
-  QObject::connect(this->Internal->Links, SIGNAL(qtWidgetChanged()),
+  QObject::connect(this->Internal->Links, SIGNAL(smPropertyChanged()),
     this, SLOT(updateAllViews()));
+  QObject::connect(this->Internal->EditCubeAxes, SIGNAL(clicked(bool)),
+    this, SLOT(editCubeAxes()));
+  QObject::connect(this->Internal->compositeTree, SIGNAL(itemSelectionChanged()),
+    this, SLOT(volumeBlockSelected()));
+
+  this->DisableSpecularOnScalarColoring = true;
 }
 
 //-----------------------------------------------------------------------------
@@ -129,6 +150,7 @@ pqDisplayProxyEditor::~pqDisplayProxyEditor()
   delete this->Internal;
 }
 
+
 //-----------------------------------------------------------------------------
 /// set the proxy to repr display properties for
 void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr) 
@@ -138,6 +160,11 @@ void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr)
     return;
     }
 
+  delete this->Internal->SliceDomain;
+  this->Internal->SliceDomain = 0;
+  delete this->Internal->CompositeTreeAdaptor;
+  this->Internal->CompositeTreeAdaptor = 0;
+
   vtkSMProxy* reprProxy = (repr)? repr->getProxy() : NULL;
   if(this->Internal->Representation)
     {
@@ -151,125 +178,187 @@ void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr)
     this->setEnabled(false);
     return;
     }
-  else
-    {
-    this->setEnabled(true);
-    }
+
+  this->setEnabled(true);
   
   // The slots are already connected but we do not want them to execute
   // while we are initializing the GUI
   this->DisableSlots = 1;
-  
+
   // setup for visibility
   this->Internal->Links->addPropertyLink(this->Internal->ViewData,
     "checked", SIGNAL(stateChanged(int)),
     reprProxy, reprProxy->GetProperty("Visibility"));
 
-  // setup for choosing color
-  this->Internal->Links->addPropertyLink(this->Internal->ColorAdaptor,
-    "color", SIGNAL(colorChanged(const QVariant&)),
-    reprProxy, reprProxy->GetProperty("AmbientColor"));
-  this->Internal->Links->addPropertyLink(this->Internal->ColorAdaptor,
-    "color", SIGNAL(colorChanged(const QVariant&)),
-    reprProxy, reprProxy->GetProperty("DiffuseColor"));
+  vtkSMProperty* prop = 0;
 
-  // setup for specular lighting
-  QObject::connect(this->Internal->SpecularWhite, SIGNAL(toggled(bool)),
-                   this, SIGNAL(specularColorChanged()));
-  QObject::connect(this->Internal->ColorAdaptor,
-                   SIGNAL(colorChanged(const QVariant&)),
-                   this, SIGNAL(specularColorChanged()));
-  this->Internal->Links->addPropertyLink(this->Internal->SpecularIntensity,
-    "value", SIGNAL(editingFinished()),
-    reprProxy, reprProxy->GetProperty("Specular"));
-  this->Internal->Links->addPropertyLink(this,
-    "specularColor", SIGNAL(specularColorChanged()),
-    reprProxy, reprProxy->GetProperty("SpecularColor"));
-  this->Internal->Links->addPropertyLink(this->Internal->SpecularPower,
-    "value", SIGNAL(editingFinished()),
-    reprProxy, reprProxy->GetProperty("SpecularPower"));
-  QObject::connect(this->Internal->SpecularIntensity, SIGNAL(editingFinished()),
-                   this, SLOT(updateAllViews()));
-  QObject::connect(this, SIGNAL(specularColorChanged()),
-                   this, SLOT(updateAllViews()));
-  QObject::connect(this->Internal->SpecularPower, SIGNAL(editingFinished()),
-                   this, SLOT(updateAllViews()));
+  // setup cube axes visibility.
+  if ((prop = reprProxy->GetProperty("CubeAxesVisibility")) != 0)
+    {
+     QObject::connect(this->Internal->ShowCubeAxes, SIGNAL(toggled(bool)),
+       this, SLOT(cubeAxesVisibilityChanged()));
+    this->Internal->AnnotationGroup->show();
+    }
+  else
+    {
+    this->Internal->AnnotationGroup->hide();
+    }
+
+  // setup for choosing color
+  if (reprProxy->GetProperty("DiffuseColor"))
+    {
+    QList<QVariant> curColor = pqSMAdaptor::getMultipleElementProperty(
+      reprProxy->GetProperty("DiffuseColor"));
+
+    bool prev = this->Internal->ColorActorColor->blockSignals(true);
+    this->Internal->ColorActorColor->setChosenColor(
+      QColor(qRound(curColor[0].toDouble()*255),
+        qRound(curColor[1].toDouble()*255),
+        qRound(curColor[2].toDouble()*255), 255));
+    this->Internal->ColorActorColor->blockSignals(prev);
+
+    // setup for specular lighting
+    QObject::connect(this->Internal->SpecularWhite, SIGNAL(toggled(bool)),
+      this, SIGNAL(specularColorChanged()));
+    this->Internal->Links->addPropertyLink(this->Internal->SpecularIntensity,
+      "value", SIGNAL(editingFinished()),
+      reprProxy, reprProxy->GetProperty("Specular"));
+    this->Internal->Links->addPropertyLink(this,
+      "specularColor", SIGNAL(specularColorChanged()),
+      reprProxy, reprProxy->GetProperty("SpecularColor"));
+    this->Internal->Links->addPropertyLink(this->Internal->SpecularPower,
+      "value", SIGNAL(editingFinished()),
+      reprProxy, reprProxy->GetProperty("SpecularPower"));
+    QObject::connect(this->Internal->SpecularIntensity, SIGNAL(editingFinished()),
+      this, SLOT(updateAllViews()));
+    QObject::connect(this, SIGNAL(specularColorChanged()),
+      this, SLOT(updateAllViews()));
+    QObject::connect(this->Internal->SpecularPower, SIGNAL(editingFinished()),
+      this, SLOT(updateAllViews()));
+    }
   
   // setup for interpolation
   this->Internal->StyleInterpolation->clear();
-  vtkSMProperty* Property = reprProxy->GetProperty("Interpolation");
-  Property->UpdateDependentDomains();
-  QList<QVariant> items = pqSMAdaptor::getEnumerationPropertyDomain(
-    Property);
-  foreach(QVariant item, items)
+  if ((prop = reprProxy->GetProperty("Interpolation")) != 0)
     {
-    this->Internal->StyleInterpolation->addItem(item.toString());
+    prop->UpdateDependentDomains();
+    QList<QVariant> items = pqSMAdaptor::getEnumerationPropertyDomain(prop);
+    foreach(QVariant item, items)
+      {
+      this->Internal->StyleInterpolation->addItem(item.toString());
+      }
+    this->Internal->Links->addPropertyLink(this->Internal->InterpolationAdaptor,
+      "currentText", SIGNAL(currentTextChanged(const QString&)),
+      reprProxy, prop);
+    this->Internal->StyleInterpolation->setEnabled(true);
+    }
+  else
+    {
+    this->Internal->StyleInterpolation->setEnabled(false);
     }
-  this->Internal->Links->addPropertyLink(this->Internal->InterpolationAdaptor,
-    "currentText", SIGNAL(currentTextChanged(const QString&)),
-    reprProxy, reprProxy->GetProperty("Interpolation"));
 
   // setup for point size
-  this->Internal->Links->addPropertyLink(this->Internal->StylePointSize,
-    "value", SIGNAL(editingFinished()),
-    reprProxy, reprProxy->GetProperty("PointSize"));
+  if ((prop = reprProxy->GetProperty("PointSize")) !=0)
+    {
+    this->Internal->Links->addPropertyLink(this->Internal->StylePointSize,
+      "value", SIGNAL(editingFinished()),
+      reprProxy, reprProxy->GetProperty("PointSize"));
+    this->Internal->StylePointSize->setEnabled(true);
+    }
+  else
+    {
+    this->Internal->StylePointSize->setEnabled(false);
+    }
 
   // setup for line width
-  this->Internal->Links->addPropertyLink(this->Internal->StyleLineWidth,
-    "value", SIGNAL(editingFinished()),
-    reprProxy, reprProxy->GetProperty("LineWidth"));
+  if ((prop = reprProxy->GetProperty("LineWidth")) != 0)
+    {
+    this->Internal->Links->addPropertyLink(this->Internal->StyleLineWidth,
+      "value", SIGNAL(editingFinished()),
+      reprProxy, reprProxy->GetProperty("LineWidth"));
+    this->Internal->StyleLineWidth->setEnabled(true);
+    }
+  else
+    {
+    this->Internal->StyleLineWidth->setEnabled(false);
+    }
 
   // setup for translate
   this->Internal->Links->addPropertyLink(this->Internal->TranslateX,
-    "value", SIGNAL(editingFinished()),
+    "text", SIGNAL(editingFinished()),
     reprProxy, reprProxy->GetProperty("Position"), 0);
+  QDoubleValidator* validator = new QDoubleValidator(this);
+  this->Internal->TranslateX->setValidator(validator);
 
   this->Internal->Links->addPropertyLink(this->Internal->TranslateY,
-    "value", SIGNAL(editingFinished()),
+    "text", SIGNAL(editingFinished()),
     reprProxy, reprProxy->GetProperty("Position"), 1);
+  validator = new QDoubleValidator(this);
+  this->Internal->TranslateY->setValidator(validator);
   
   this->Internal->Links->addPropertyLink(this->Internal->TranslateZ,
-    "value", SIGNAL(editingFinished()),
+    "text", SIGNAL(editingFinished()),
     reprProxy, reprProxy->GetProperty("Position"), 2);
+  validator = new QDoubleValidator(this);
+  this->Internal->TranslateZ->setValidator(validator);
+
 
   // setup for scale
   this->Internal->Links->addPropertyLink(this->Internal->ScaleX,
-    "value", SIGNAL(editingFinished()),
+    "text", SIGNAL(editingFinished()),
     reprProxy, reprProxy->GetProperty("Scale"), 0);
+  validator = new QDoubleValidator(this);
+  this->Internal->ScaleX->setValidator(validator);
 
   this->Internal->Links->addPropertyLink(this->Internal->ScaleY,
-    "value", SIGNAL(editingFinished()),
+    "text", SIGNAL(editingFinished()),
     reprProxy, reprProxy->GetProperty("Scale"), 1);
+  validator = new QDoubleValidator(this);
+  this->Internal->ScaleY->setValidator(validator);
 
   this->Internal->Links->addPropertyLink(this->Internal->ScaleZ,
-    "value", SIGNAL(editingFinished()),
+    "text", SIGNAL(editingFinished()),
     reprProxy, reprProxy->GetProperty("Scale"), 2);
+  validator = new QDoubleValidator(this);
+  this->Internal->ScaleZ->setValidator(validator);
 
   // setup for orientation
   this->Internal->Links->addPropertyLink(this->Internal->OrientationX,
-    "value", SIGNAL(editingFinished()),
+    "text", SIGNAL(editingFinished()),
     reprProxy, reprProxy->GetProperty("Orientation"), 0);
+  validator = new QDoubleValidator(this);
+  this->Internal->OrientationX->setValidator(validator);
 
   this->Internal->Links->addPropertyLink(this->Internal->OrientationY,
-    "value", SIGNAL(editingFinished()),
+    "text", SIGNAL(editingFinished()),
     reprProxy, reprProxy->GetProperty("Orientation"), 1);
+  validator = new QDoubleValidator(this);
+  this->Internal->OrientationY->setValidator(validator);
 
   this->Internal->Links->addPropertyLink(this->Internal->OrientationZ,
-    "value", SIGNAL(editingFinished()),
+    "text", SIGNAL(editingFinished()),
     reprProxy, reprProxy->GetProperty("Orientation"), 2);
+  validator = new QDoubleValidator(this);
+  this->Internal->OrientationZ->setValidator(validator);
 
   // setup for origin
   this->Internal->Links->addPropertyLink(this->Internal->OriginX,
-    "value", SIGNAL(editingFinished()),
+    "text", SIGNAL(editingFinished()),
     reprProxy, reprProxy->GetProperty("Origin"), 0);
+  validator = new QDoubleValidator(this);
+  this->Internal->OriginX->setValidator(validator);
 
   this->Internal->Links->addPropertyLink(this->Internal->OriginY,
-    "value", SIGNAL(editingFinished()),
+    "text", SIGNAL(editingFinished()),
     reprProxy, reprProxy->GetProperty("Origin"), 1);
+  validator = new QDoubleValidator(this);
+  this->Internal->OriginY->setValidator(validator);
 
   this->Internal->Links->addPropertyLink(this->Internal->OriginZ,
-    "value", SIGNAL(editingFinished()),
+    "text", SIGNAL(editingFinished()),
     reprProxy, reprProxy->GetProperty("Origin"), 2);
+  validator = new QDoubleValidator(this);
+  this->Internal->OriginZ->setValidator(validator);
 
   // setup for opacity
   this->Internal->Links->addPropertyLink(this->Internal->Opacity,
@@ -282,9 +371,12 @@ void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr)
     reprProxy, reprProxy->GetProperty("MapScalars"));
 
   // setup for InterpolateScalarsBeforeMapping
-  this->Internal->Links->addPropertyLink(
-    this->Internal->ColorInterpolateColors, "checked", SIGNAL(stateChanged(int)),
-    reprProxy, reprProxy->GetProperty("InterpolateScalarsBeforeMapping"));
+  if (reprProxy->GetProperty("InterpolateScalarsBeforeMapping"))
+    {
+    this->Internal->Links->addPropertyLink(
+      this->Internal->ColorInterpolateColors, "checked", SIGNAL(stateChanged(int)),
+      reprProxy, reprProxy->GetProperty("InterpolateScalarsBeforeMapping"));
+    }
 
   this->Internal->ColorBy->setRepresentation(repr);
   QObject::connect(this->Internal->ColorBy,
@@ -300,19 +392,63 @@ void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr)
     SIGNAL(currentTextChanged(const QString&)),
     this, SLOT(updateEnableState()), Qt::QueuedConnection);
 
-  /*
-  if (reprProxy->GetProperty("ScalarOpacityUnitDistance"))
-    {
-    this->Internal->Links->addPropertyLink(
-      this->Internal->ScalarOpacityUnitDistance, "value",
-      SIGNAL(editingFinished()),
-      reprProxy, reprProxy->GetProperty("ScalarOpacityUnitDistance"));
-    }
-    */
+  this->Internal->Texture->setRepresentation(repr);
 
-  this->Internal->Links->addPropertyLink(this->Internal->EdgeColorAdaptor,
-    "color", SIGNAL(colorChanged(const QVariant&)),
-    reprProxy, reprProxy->GetProperty("EdgeColor"));
+  if ( (prop = reprProxy->GetProperty("EdgeColor")) != 0)
+    {
+    this->Internal->Links->addPropertyLink(this->Internal->EdgeColorAdaptor,
+      "color", SIGNAL(colorChanged(const QVariant&)),
+      reprProxy, prop);
+    }
+
+  if (reprProxy->GetProperty("Slice"))
+    {
+    this->Internal->SliceDomain = new pqWidgetRangeDomain(
+      this->Internal->Slice, "minimum", "maximum",
+      reprProxy->GetProperty("Slice"), 0);
+
+    this->Internal->Links->addPropertyLink(this->Internal->Slice,
+      "value", SIGNAL(valueChanged(int)),
+      reprProxy, reprProxy->GetProperty("Slice"));
+
+    QList<QVariant> sliceModes = 
+      pqSMAdaptor::getEnumerationPropertyDomain(
+        reprProxy->GetProperty("SliceMode"));
+    foreach(QVariant item, sliceModes)
+      {
+      this->Internal->SliceDirection->addItem(item.toString());
+      }
+    this->Internal->Links->addPropertyLink(
+      this->Internal->SliceDirectionAdaptor,
+      "currentText", SIGNAL(currentTextChanged(const QString&)),
+      reprProxy, reprProxy->GetProperty("SliceMode"));
+    }
+
+  if (reprProxy->GetProperty("ExtractedBlockIndex"))
+    {
+    this->Internal->CompositeTreeAdaptor = 
+      new pqSignalAdaptorCompositeTreeWidget(
+        this->Internal->compositeTree,
+        this->Internal->Representation->getOutputPortFromInput()->getOutputPortProxy(),
+        vtkSMCompositeTreeDomain::NONE,
+        pqSignalAdaptorCompositeTreeWidget::INDEX_MODE_FLAT, false, true);
+    }
+
+  if (reprProxy->GetProperty("SelectedMapperIndex"))
+    {
+    QList<QVariant> mapperNames = 
+      pqSMAdaptor::getEnumerationPropertyDomain(
+        reprProxy->GetProperty("SelectedMapperIndex"));
+    foreach(QVariant item, mapperNames)
+      {
+      this->Internal->SelectedMapperIndex->addItem(item.toString());
+      }
+    this->Internal->Links->addPropertyLink(
+      this->Internal->SelectedMapperAdaptor,
+      "currentText", SIGNAL(currentTextChanged(const QString&)),
+      reprProxy, reprProxy->GetProperty("SelectedMapperIndex"));
+    }
+    
 
 #if 0                                       //FIXME 
   // material
@@ -357,63 +493,6 @@ void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr)
 //-----------------------------------------------------------------------------
 void pqDisplayProxyEditor::setupGUIConnections()
 {
-  // We are usinging Queues slot execution where ever possible,
-  // This ensures that the updateAllViews() slot is called 
-  // only after the vtkSMProperty has been changed by the pqPropertyLinks.
-  QObject::connect(
-    this->Internal->ViewData, SIGNAL(stateChanged(int)),
-    this, SLOT(updateAllViews()));
-  QObject::connect(
-    this->Internal->ColorInterpolateColors, SIGNAL(stateChanged(int)),
-    this, SLOT(updateAllViews()));
-  QObject::connect(
-    this->Internal->ColorMapScalars, SIGNAL(stateChanged(int)),
-    this, SLOT(updateAllViews()));
-  QObject::connect(
-    this->Internal->StylePointSize,  SIGNAL(editingFinished()), 
-    this, SLOT(updateAllViews()));
-  QObject::connect(
-    this->Internal->StyleLineWidth, SIGNAL(editingFinished()),
-    this, SLOT(updateAllViews()));
-  QObject::connect(
-    this->Internal->TranslateX, SIGNAL(editingFinished()),
-    this, SLOT(updateAllViews()));
-  QObject::connect(
-    this->Internal->TranslateY, SIGNAL(editingFinished()),
-    this, SLOT(updateAllViews()));
-  QObject::connect(
-    this->Internal->TranslateZ, SIGNAL(editingFinished()),
-    this, SLOT(updateAllViews()));
-  QObject::connect(
-    this->Internal->ScaleX, SIGNAL(editingFinished()),
-    this, SLOT(updateAllViews()));
-  QObject::connect(
-    this->Internal->ScaleY, SIGNAL(editingFinished()),
-    this, SLOT(updateAllViews()));
-  QObject::connect(
-    this->Internal->ScaleZ, SIGNAL(editingFinished()),
-    this, SLOT(updateAllViews()));
-  QObject::connect(
-    this->Internal->OrientationX, SIGNAL(editingFinished()),
-    this, SLOT(updateAllViews()));
-  QObject::connect(
-    this->Internal->OrientationY, SIGNAL(editingFinished()),
-    this, SLOT(updateAllViews()));
-  QObject::connect(
-    this->Internal->OrientationZ, SIGNAL(editingFinished()),
-    this, SLOT(updateAllViews()));
-  QObject::connect(
-    this->Internal->OriginX, SIGNAL(editingFinished()),
-    this, SLOT(updateAllViews()));
-  QObject::connect(
-    this->Internal->OriginY, SIGNAL(editingFinished()),
-    this, SLOT(updateAllViews()));
-  QObject::connect(
-    this->Internal->OriginZ, SIGNAL(editingFinished()),
-    this, SLOT(updateAllViews()));
-  QObject::connect(
-    this->Internal->Opacity, SIGNAL(editingFinished()),
-    this, SLOT(updateAllViews()));
   QObject::connect(
     this->Internal->ViewZoomToData, SIGNAL(clicked(bool)), 
     this, SLOT(zoomToData()));
@@ -434,24 +513,29 @@ void pqDisplayProxyEditor::setupGUIConnections()
     this->Internal->StyleInterpolation);
   this->Internal->InterpolationAdaptor->setObjectName(
     "StyleInterpolationAdapator");
-  QObject::connect(this->Internal->InterpolationAdaptor, 
-    SIGNAL(currentTextChanged(const QString&)), this, SLOT(updateAllViews()));
-    
-  this->Internal->ColorAdaptor = new pqSignalAdaptorColor(
-                            this->Internal->ColorActorColor,
-                            "chosenColor",
-                            SIGNAL(chosenColorChanged(const QColor&)), false);
+ 
+  QObject::connect(this->Internal->ColorActorColor,
+    SIGNAL(chosenColorChanged(const QColor&)),
+    this, SLOT(setSolidColor(const QColor&)));
+
   this->Internal->EdgeColorAdaptor = new pqSignalAdaptorColor(
     this->Internal->EdgeColor, "chosenColor",
     SIGNAL(chosenColorChanged(const QColor&)), false);
 
-  QObject::connect(
-    this->Internal->ColorActorColor, SIGNAL(chosenColorChanged(const QColor&)),
-    this, SLOT(updateAllViews()));
-  
   QObject::connect(this->Internal->StyleMaterial, SIGNAL(currentIndexChanged(int)),
                    this, SLOT(updateMaterial(int)));
 
+  this->Internal->SliceDirectionAdaptor = new pqSignalAdaptorComboBox(
+    this->Internal->SliceDirection);
+  QObject::connect(this->Internal->SliceDirectionAdaptor,
+    SIGNAL(currentTextChanged(const QString&)),
+    this, SLOT(sliceDirectionChanged()), Qt::QueuedConnection);
+
+  this->Internal->SelectedMapperAdaptor = new pqSignalAdaptorComboBox(
+    this->Internal->SelectedMapperIndex);
+  QObject::connect(this->Internal->SelectedMapperAdaptor,
+    SIGNAL(currentTextChanged(const QString&)),
+    this, SLOT(selectedMapperChanged()), Qt::QueuedConnection);
 }
 
 //-----------------------------------------------------------------------------
@@ -466,7 +550,10 @@ void pqDisplayProxyEditor::updateEnableState()
     }
   else
     {
-    this->Internal->LightingGroup->setEnabled(false);
+    if (this->DisableSpecularOnScalarColoring)
+      {
+      this->Internal->LightingGroup->setEnabled(false);
+      }
     this->Internal->ColorInterpolateColors->setEnabled(true);
     this->Internal->ColorButtonStack->setCurrentWidget(
         this->Internal->ColorMapPage);
@@ -474,18 +561,35 @@ void pqDisplayProxyEditor::updateEnableState()
 
   int reprType = this->Internal->Representation->getRepresentationType();
   
-  //this->Internal->ScalarOpacityUnitDistance->setEnabled(
-  //  reprType == vtkSMPVRepresentationProxy::VOLUME);
   this->Internal->EdgeStyleGroup->setEnabled(
     reprType == vtkSMPVRepresentationProxy::SURFACE_WITH_EDGES);
 
+  this->Internal->SliceGroup->setEnabled(
+    reprType == vtkSMPVRepresentationProxy::SLICE);
+  if (reprType == vtkSMPVRepresentationProxy::SLICE)
+    {
+    // every time the user switches to Slice mode we update the domain for the
+    // slider since the domain depends on the input to the image mapper which
+    // may have changed.
+    QTimer::singleShot(0, this, SLOT(sliceDirectionChanged()));
+    }
+
+  this->Internal->compositeTree->setVisible(
+   this->Internal->CompositeTreeAdaptor &&
+   (reprType == vtkSMPVRepresentationProxy::VOLUME));
+
+  this->Internal->SelectedMapperIndex->setEnabled(
+    reprType == vtkSMPVRepresentationProxy::VOLUME
+    && this->Internal->Representation->getProxy()->GetProperty("SelectedMapperIndex"));
+
   vtkSMDataRepresentationProxy* display = 
     this->Internal->Representation->getRepresentationProxy();
   if (display)
     {
     QVariant scalarMode = pqSMAdaptor::getEnumerationProperty(
       display->GetProperty("ColorAttributeType"));
-    vtkPVDataInformation* geomInfo = display->GetFullResDataInformation();
+    vtkPVDataInformation* geomInfo = 
+      display->GetRepresentedDataInformation(/*update=*/false);
     vtkPVDataSetAttributesInformation* attrInfo;
     if (scalarMode == "POINT_DATA")
       {
@@ -501,7 +605,8 @@ void pqDisplayProxyEditor::updateEnableState()
     if (arrayInfo && arrayInfo->GetDataType() == VTK_UNSIGNED_CHAR)
       {
       // Number of component restriction.
-      if (arrayInfo->GetNumberOfComponents() == 3)
+      // Upto 4 component unsigned chars can be direcly mapped.
+      if (arrayInfo->GetNumberOfComponents() <= 4)
         {
         // One component causes more trouble than it is worth.
         this->Internal->ColorMapScalars->setEnabled(true);
@@ -517,15 +622,13 @@ void pqDisplayProxyEditor::updateEnableState()
 //-----------------------------------------------------------------------------
 void pqDisplayProxyEditor::openColorMapEditor()
 {
-  if(this->Internal->Representation.isNull())
+  // Get the color scale editor from the application core's registry.
+  pqColorScaleToolbar *colorScale = qobject_cast<pqColorScaleToolbar *>(
+      pqApplicationCore::instance()->manager("COLOR_SCALE_EDITOR"));
+  if(colorScale)
     {
-    return;
+    colorScale->editColorMap(this->Internal->Representation);
     }
-
-  // Create a color map editor and set the display.
-  pqColorScaleEditor colorScale(this);
-  colorScale.setRepresentation(this->Internal->Representation);
-  colorScale.exec();
 }
 
 //-----------------------------------------------------------------------------
@@ -549,7 +652,7 @@ void pqDisplayProxyEditor::zoomToData()
     }
 
   double bounds[6];
-  this->Internal->Representation->getDataBounds(bounds);
+  this->Internal->Representation->getRepresentationProxy()->GetBounds(bounds);
   if (bounds[0]<=bounds[1] && bounds[2]<=bounds[3] && bounds[4]<=bounds[5])
     {
     pqRenderView* renModule = qobject_cast<pqRenderView*>(
@@ -563,6 +666,7 @@ void pqDisplayProxyEditor::zoomToData()
     }
 }
 
+//-----------------------------------------------------------------------------
 // TODO:  get rid of me !!  as soon as vtkSMDisplayProxy can tell us when new
 // arrays are added.
 void pqDisplayProxyEditor::reloadGUI()
@@ -571,6 +675,7 @@ void pqDisplayProxyEditor::reloadGUI()
 }
 
 
+//-----------------------------------------------------------------------------
 QVariant pqDisplayProxyEditor::specularColor() const
 {
   if(this->Internal->SpecularWhite->isChecked())
@@ -587,6 +692,7 @@ QVariant pqDisplayProxyEditor::specularColor() const
        proxy->GetProperty("DiffuseColor"));
 }
 
+//-----------------------------------------------------------------------------
 void pqDisplayProxyEditor::setSpecularColor(QVariant specColor)
 {
   QList<QVariant> whiteLight;
@@ -606,6 +712,7 @@ void pqDisplayProxyEditor::setSpecularColor(QVariant specColor)
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqDisplayProxyEditor::updateMaterial(int vtkNotUsed(idx))
 {
   // FIXME: when we enable materials.
@@ -651,3 +758,100 @@ void pqDisplayProxyEditor::updateMaterial(int vtkNotUsed(idx))
     }
 #endif
 }
+//-----------------------------------------------------------------------------
+void pqDisplayProxyEditor::cubeAxesVisibilityChanged()
+{
+  vtkSMProxy* reprProxy = (this->Internal->Representation)? this->Internal->Representation->getProxy() : NULL;
+  vtkSMProperty* prop = 0;
+
+  // setup cube axes visibility.
+  if ((prop = reprProxy->GetProperty("CubeAxesVisibility")) != 0)
+    {
+    pqSMAdaptor::setElementProperty(prop, this->Internal->ShowCubeAxes->isChecked());
+    reprProxy->UpdateVTKObjects();
+    }
+  this->updateAllViews();
+}
+//-----------------------------------------------------------------------------
+void pqDisplayProxyEditor::editCubeAxes()
+{
+  pqCubeAxesEditorDialog dialog(this);
+  dialog.setRepresentationProxy(this->Internal->Representation->getProxy());
+  dialog.exec();
+}
+//----------------------------------------------------------------------------
+bool pqDisplayProxyEditor::isCubeAxesVisible()
+{
+  return this->Internal->ShowCubeAxes->isChecked();
+}
+
+//-----------------------------------------------------------------------------
+void pqDisplayProxyEditor::sliceDirectionChanged()
+{
+  if (this->Internal->Representation)
+    {
+    vtkSMProxy* reprProxy = this->Internal->Representation->getProxy();
+    vtkSMProperty* prop = reprProxy->GetProperty("SliceMode");
+    if (prop)
+      {
+      prop->UpdateDependentDomains();
+      }
+    }
+}
+
+//-----------------------------------------------------------------------------
+/// Handle user selection of block for volume rendering
+void pqDisplayProxyEditor::volumeBlockSelected()
+{
+  if (this->Internal->CompositeTreeAdaptor
+      && this->Internal->Representation)
+    {
+    bool valid = false;
+    unsigned int selectedIndex = 
+      this->Internal->CompositeTreeAdaptor->getCurrentFlatIndex(&valid);
+    if (valid && selectedIndex > 0) 
+      {
+      vtkSMDataRepresentationProxy* repr =
+        this->Internal->Representation->getRepresentationProxy();
+      pqSMAdaptor::setElementProperty(
+        repr->GetProperty("ExtractedBlockIndex"), selectedIndex);
+      repr->UpdateVTKObjects();
+      this->Internal->Representation->renderViewEventually();
+      this->Internal->ColorBy->reloadGUI();
+      }
+    }
+}
+
+//-----------------------------------------------------------------------------
+// Called when the GUI selection for the solid color changes.
+void pqDisplayProxyEditor::setSolidColor(const QColor& color)
+{
+  QList<QVariant> val;
+  val.push_back(color.red()/255.0);
+  val.push_back(color.green()/255.0);
+  val.push_back(color.blue()/255.0);
+
+  pqUndoStack* stack = pqApplicationCore::instance()->getUndoStack();
+  if (stack)
+    {
+    stack->beginUndoSet("Change Solid Color");
+    }
+  pqSMAdaptor::setMultipleElementProperty(
+    this->Internal->Representation->getProxy()->GetProperty("AmbientColor"), val);
+  pqSMAdaptor::setMultipleElementProperty(
+    this->Internal->Representation->getProxy()->GetProperty("DiffuseColor"), val);
+
+  // If specular while if off, then we want to update the specular color as
+  // well.
+  emit this->specularColorChanged();
+  if (stack)
+    {
+    stack->endUndoSet();
+    }
+}
+
+//-----------------------------------------------------------------------------
+void pqDisplayProxyEditor::selectedMapperChanged()
+{
+
+}
