@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqExodusIIPanel.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -29,8 +29,8 @@ NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 =========================================================================*/
-
 #include "pqExodusIIPanel.h"
+#include "ui_pqExodusIIPanel.h"
 
 // Qt includes
 #include <QAction>
@@ -42,16 +42,23 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <QVariant>
 #include <QVector>
 #include <QMap>
+#include <QHeaderView>
 
 // VTK includes
 
 // ParaView Server Manager includes
-#include "vtkPVDataSetAttributesInformation.h"
+#include "vtkEventQtSlotConnect.h"
+#include "vtkGraph.h"
+#include "vtkProcessModule.h"
 #include "vtkPVArrayInformation.h"
 #include "vtkPVDataInformation.h"
-#include "vtkSMSourceProxy.h"
+#include "vtkPVDataSetAttributesInformation.h"
+#include "vtkPVSILInformation.h"
+#include "vtkSmartPointer.h"
 #include "vtkSMProperty.h"
 #include "vtkSMProxyManager.h"
+#include "vtkSMSourceProxy.h"
+#include "vtkSMPropertyHelper.h"
 
 // ParaView includes
 #include "pqPropertyManager.h"
@@ -59,35 +66,54 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqServer.h"
 #include "pqTimeKeeper.h"
 #include "pqSMAdaptor.h"
-#include "pqTreeWidgetCheckHelper.h"
+#include "pqTreeWidgetSelectionHelper.h"
 #include "pqTreeWidgetItemObject.h"
-#include "ui_pqExodusIIPanel.h"
 #include "vtkSMDoubleVectorProperty.h"
+#include "pqSILModel.h"
+#include "pqProxySILModel.h"
+#include "pqTreeViewSelectionHelper.h"
 
 class pqExodusIIPanel::pqUI : public QObject, public Ui::ExodusIIPanel 
 {
 public:
-  pqUI(pqExodusIIPanel* p) : QObject(p)
+  /// Specialization of pqSILModel is add support for icons on the 
+  /// "Blocks".
+  class pqExodusIISILModel : public pqSILModel
   {
-    // make a clone of the ExodusIIReader proxy
-    // NOTE: I added the ExodusIIReaderHelper since the new exodus reader
-    //  supports different types of blocks (faces and edges in addition
-    //  to element blocks) so BlockArray___ properties become 
-    //  ElementBlockArray____.
-    // we'll use the clone to help us with the ElementBlock/Material/Hierarchy array
-    // status (keep them in sync and auto check/uncheck hierarchically related 
-    // items in the hierarchy view).
-    vtkSMProxyManager* pm = vtkSMProxy::GetProxyManager();
-    ExodusHelper.TakeReference(pm->NewProxy("misc", "ExodusIIReaderHelper"));
-    ExodusHelper->InitializeAndCopyFromProxy(p->proxy());
-  }
-  vtkSmartPointer<vtkSMProxy> ExodusHelper;
+public:
+  typedef pqSILModel Superclass;
+  pqExodusIISILModel(QObject* p = 0) : Superclass(p) {}
+  QVariant data(const QModelIndex &idx, int role) const
+    {
+    if (role == Qt::DecorationRole && idx.isValid())
+      {
+      vtkIdType vertexId = static_cast<vtkIdType>(idx.internalId());
+      // If this vertex is a block, then show the ELEM_BLOCK icon.
+      if (this->SIL->GetOutDegree(vertexId) == 0)
+        {
+        return QVariant(QPixmap(":/pqWidgets/Icons/pqCellCenterData16.png"));
+        }
+      }
+    return this->Superclass::data(idx, role);
+    }
+  };
+
+public:
+  pqUI(pqExodusIIPanel* p) : QObject(p) 
+   {
+   this->VTKConnect = vtkSmartPointer<vtkEventQtSlotConnect>::New();
+   this->SILUpdateStamp = -1;
+   }
+
+  pqExodusIISILModel SILModel;
   QVector<double> TimestepValues;
   QMap<QTreeWidgetItem*, QString> TreeItemToPropMap;
+  vtkSmartPointer<vtkEventQtSlotConnect> VTKConnect;
+  int SILUpdateStamp;
 };
 
 pqExodusIIPanel::pqExodusIIPanel(pqProxy* object_proxy, QWidget* p) :
-  pqNamedObjectPanel(object_proxy, p)
+  Superclass(object_proxy, p)
 {
   this->UI = new pqUI(this);
   this->UI->setupUi(this);
@@ -95,27 +121,80 @@ pqExodusIIPanel::pqExodusIIPanel(pqProxy* object_proxy, QWidget* p) :
   this->DisplItem = 0;
   
   this->UI->XMLFileName->setServer(this->referenceProxy()->getServer());
+
+  this->UI->VTKConnect->Connect(
+    this->referenceProxy()->getProxy(),
+    vtkCommand::UpdateInformationEvent,
+    this, SLOT(updateSIL()));
   
+  pqProxySILModel* proxyModel = new pqProxySILModel("Blocks", &this->UI->SILModel);
+  proxyModel->setSourceModel(&this->UI->SILModel);
+  this->UI->Blocks->setModel(proxyModel);
+  this->UI->Blocks->header()->setClickable(true);
+  QObject::connect(this->UI->Blocks->header(), SIGNAL(sectionClicked(int)),
+    proxyModel, SLOT(toggleRootCheckState()), Qt::QueuedConnection);
+
+  proxyModel = new pqProxySILModel("Assemblies", &this->UI->SILModel);
+  proxyModel->setSourceModel(&this->UI->SILModel);
+  this->UI->Assemblies->setModel(proxyModel);
+  this->UI->Assemblies->header()->setClickable(true);
+  QObject::connect(this->UI->Assemblies->header(), SIGNAL(sectionClicked(int)),
+    proxyModel, SLOT(toggleRootCheckState()), Qt::QueuedConnection);
+
+  proxyModel = new pqProxySILModel("Materials", &this->UI->SILModel);
+  proxyModel->setSourceModel(&this->UI->SILModel);
+  this->UI->Materials->setModel(proxyModel);
+  this->UI->Materials->header()->setClickable(true);
+  QObject::connect(this->UI->Materials->header(), SIGNAL(sectionClicked(int)),
+    proxyModel, SLOT(toggleRootCheckState()), Qt::QueuedConnection);
+
+  this->updateSIL();
+
+  this->UI->Blocks->header()->setStretchLastSection(true);
+  this->UI->Assemblies->header()->setStretchLastSection(true);
+  this->UI->Materials->header()->setStretchLastSection(true);
+
+
   this->linkServerManagerProperties();
+
+  QList<pqTreeWidget*> treeWidgets = this->findChildren<pqTreeWidget*>();
+  foreach (pqTreeWidget* tree, treeWidgets)
+    {
+    new pqTreeWidgetSelectionHelper(tree);
+    }
+
+  QList<pqTreeView*> treeViews = this->findChildren<pqTreeView*>();
+  foreach (pqTreeView* tree, treeViews)
+    {
+    new pqTreeViewSelectionHelper(tree);
+    }
 }
 
 pqExodusIIPanel::~pqExodusIIPanel()
 {
 }
 
-void pqExodusIIPanel::reset()
+void pqExodusIIPanel::updateSIL()
 {
-  // push original values for block status back
-  // onto the vtkExodusReader, as the ExodusHelper
-  // might have played with them
-  vtkSMProxy* pxy = this->proxy();
-  pxy->UpdateProperty("EdgeBlockArrayStatus", 1);
-  pxy->UpdateProperty("ElementBlockArrayStatus", 1);
-  pxy->UpdateProperty("FaceBlockArrayStatus", 1);
-  pxy->UpdateProperty("MaterialArrayStatus", 1);
-  pxy->UpdateProperty("HierarchyArrayStatus", 1);
+  vtkSMProxy* reader = this->referenceProxy()->getProxy();
+  reader->UpdatePropertyInformation(reader->GetProperty("SILUpdateStamp"));
 
-  pqNamedObjectPanel::reset();
+  int stamp = vtkSMPropertyHelper(reader, "SILUpdateStamp").GetAsInt();
+  if (stamp != this->UI->SILUpdateStamp)
+    {
+    this->UI->SILUpdateStamp = stamp;
+    vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
+    vtkPVSILInformation* info = vtkPVSILInformation::New();
+    pm->GatherInformation(reader->GetConnectionID(),
+      vtkProcessModule::DATA_SERVER, info,
+      reader->GetID());
+    this->UI->SILModel.update(info->GetSIL());
+
+    this->UI->Blocks->expandAll();
+    this->UI->Assemblies->expandAll();
+    this->UI->Materials->expandAll();
+    info->Delete();
+    }
 }
 
 void pqExodusIIPanel::addSelectionsToTreeWidget(const QString& prop, 
@@ -187,13 +266,17 @@ void pqExodusIIPanel::addSelectionToTreeWidget(const QString& name,
 void pqExodusIIPanel::linkServerManagerProperties()
 {
 
+  this->propertyManager()->registerLink(
+    this->UI->Blocks->model(), "values", SIGNAL(valuesChanged()),
+    this->proxy(),
+    this->proxy()->GetProperty("ElementBlockArrayStatus"));
+
   // parent class hooks up some of our widgets in the ui
-  pqNamedObjectPanel::linkServerManagerProperties();
+  this->Superclass::linkServerManagerProperties();
 
   this->DisplItem = 0;
 
   // we hook up the node/element variables
-  new pqTreeWidgetCheckHelper(this->UI->Variables, 0, this);
 
   // do block id, global element id
   this->addSelectionToTreeWidget("Object Ids", "ObjectId", this->UI->Variables,
@@ -201,6 +284,10 @@ void pqExodusIIPanel::linkServerManagerProperties()
   
   this->addSelectionToTreeWidget("Global Element Ids", "GlobalElementId", this->UI->Variables,
                    PM_ELEMBLK, "GenerateGlobalElementIdArray");
+
+  // integer array indicating file id (number in file name or position in sequence)
+  this->addSelectionToTreeWidget("File Ids", "FileId", this->UI->Variables,
+                   PM_ELEMBLK, "GenerateFileIdArray");
   
   // do the cell variables
   this->addSelectionsToTreeWidget("ElementResultArrayStatus",
@@ -273,17 +360,14 @@ void pqExodusIIPanel::linkServerManagerProperties()
                                   this->UI->Variables, PM_GLOBAL);
 
   // we hook up the sideset/nodeset 
-  QTreeWidget* SetsTree = this->UI->Sets;
-  new pqTreeWidgetCheckHelper(SetsTree, 0, this);
+  //QTreeWidget* SetsTree = this->UI->Sets;
 
 
   // blocks
   this->addSelectionsToTreeWidget("EdgeBlockArrayStatus",
-                                  this->UI->BlockArrayStatus, PM_EDGEBLK);
+                                  this->UI->EdgeBlockArrays, PM_EDGEBLK);
   this->addSelectionsToTreeWidget("FaceBlockArrayStatus",
-                                  this->UI->BlockArrayStatus, PM_FACEBLK);
-  this->addSelectionsToTreeWidget("ElementBlockArrayStatus",
-                                  this->UI->BlockArrayStatus, PM_ELEMBLK);
+                                  this->UI->FaceBlockArrays, PM_FACEBLK);
 
   // sets
   this->addSelectionsToTreeWidget("SideSetArrayStatus",
@@ -350,18 +434,9 @@ void pqExodusIIPanel::linkServerManagerProperties()
   QObject::connect(this->UI->ModeSelectSpinBox, SIGNAL(valueChanged(int)),
                    this, SLOT(modeChanged(int)));
 
-  QObject::connect(this->UI->BlockArrayStatus, 
-    SIGNAL(itemChanged(QTreeWidgetItem*, int)),
-    this, SLOT(blockItemChanged(QTreeWidgetItem*)));
-  QObject::connect(this->UI->HierarchyArrayStatus,
-    SIGNAL(itemChanged(QTreeWidgetItem*, int)),
-    this, SLOT(hierarchyItemChanged(QTreeWidgetItem*)));
-  QObject::connect(this->UI->MaterialArrayStatus,
-    SIGNAL(itemChanged(QTreeWidgetItem*, int)),
-    this, SLOT(materialItemChanged(QTreeWidgetItem*)));
-
   QObject::connect(this->UI->Refresh,
     SIGNAL(pressed()), this, SLOT(onRefresh()));
+
 }
   
 void pqExodusIIPanel::applyDisplacements(int state)
@@ -433,101 +508,6 @@ void pqExodusIIPanel::modeChanged(int value)
     }
 }
 
-void pqExodusIIPanel::blockItemChanged(QTreeWidgetItem* item)
-{
-  // Use our map to find which property name this tree item belongs to, 
-  // since there are multiple types of blocks (element, edge, face).
-  // Right now we only support element blocks.
-  if(this->UI->TreeItemToPropMap[item] == "ElementBlockArrayStatus")
-    {
-    this->selectionItemChanged(item, "ElementBlockArrayStatus");
-    }
-}
-
-void pqExodusIIPanel::hierarchyItemChanged(QTreeWidgetItem* item)
-{
-  this->selectionItemChanged(item, "HierarchyArrayStatus");
-}
-
-void pqExodusIIPanel::materialItemChanged(QTreeWidgetItem* item)
-{
-  this->selectionItemChanged(item, "MaterialArrayStatus");
-}
-
-void pqExodusIIPanel::selectionItemChanged(QTreeWidgetItem* item,
-                                         const QString& propName)
-{
-
-  vtkSMProxy* pxy = this->UI->ExodusHelper;
-
-  vtkSMProperty* blockInfo[3];
-  vtkSMProperty* blockStatus[3];
-  int i;
-  
-  //blockInfo[0] = pxy->GetProperty("EdgeBlockArrayInfo");
-  blockInfo[0] = pxy->GetProperty("ElementBlockArrayInfo");
-  //blockInfo[2] = pxy->GetProperty("FaceBlockArrayInfo");
-  blockInfo[1] = pxy->GetProperty("HierarchyArrayInfo");
-  blockInfo[2] = pxy->GetProperty("MaterialArrayInfo");
-  
-  //blockStatus[0] = pxy->GetProperty("EdgeBlockArrayStatus");
-  blockStatus[0] = pxy->GetProperty("ElementBlockArrayStatus");
-  //blockStatus[2] = pxy->GetProperty("FaceBlockArrayStatus");
-  blockStatus[1] = pxy->GetProperty("HierarchyArrayStatus");
-  blockStatus[2] = pxy->GetProperty("MaterialArrayStatus");
-
-  QList< QList< QVariant > > values;
-
-  pqTreeWidgetItemObject* itemObject;
-  itemObject = static_cast<pqTreeWidgetItemObject*>(item);
-  vtkSMProperty* prop = NULL;
-  prop = pxy->GetProperty(propName.toAscii().data());
-  
-  // clear out any old stuff
-  for(i=0; i<3; i++)
-    {
-    pqSMAdaptor::setSelectionProperty(blockStatus[i], values);
-    }
-
-  // set only the single selection the user changed
-  values.append(QList<QVariant>());
-  values[0].append(itemObject->text(0));
-  values[0].append(itemObject->isChecked());
-  // send change down to the vtkExodusReader
-  pqSMAdaptor::setSelectionProperty(prop, values);
-  pxy->UpdateProperty(propName.toAscii().data());
-
-  // get the new selections back
-  for(i=0; i<3; i++)
-    {
-    pxy->UpdatePropertyInformation(blockInfo[i]);
-    blockStatus[i]->Copy(blockInfo[i]);
-    }
-
-  QTreeWidget* widgets[3] =
-    {
-    this->UI->BlockArrayStatus,
-    this->UI->HierarchyArrayStatus,
-    this->UI->MaterialArrayStatus
-    };
-
-  for(i=0; i<3; i++)
-    {
-    values = pqSMAdaptor::getSelectionProperty(blockStatus[i]);
-    for(int j=0; j<values.size(); j++)
-      {
-      pqTreeWidgetItemObject* treeItemObject;
-      treeItemObject = static_cast<pqTreeWidgetItemObject*>(
-                       widgets[i]->topLevelItem(j));
-      if(treeItemObject)
-        {
-        treeItemObject->setChecked(values[j][1].toBool());
-        }
-      }
-    }
-
-}
-
 void pqExodusIIPanel::onRefresh()
 {
   vtkSMSourceProxy* sp = vtkSMSourceProxy::SafeDownCast(this->proxy());
