@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqExodusPanel.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -61,7 +61,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqServer.h"
 #include "pqSMAdaptor.h"
 #include "pqTimeKeeper.h"
-#include "pqTreeWidgetCheckHelper.h"
+#include "pqTreeWidgetSelectionHelper.h"
 #include "pqTreeWidgetItemObject.h"
 
 class pqExodusPanel::pqUI : public QObject, public Ui::ExodusPanel 
@@ -79,6 +79,10 @@ public:
   }
   vtkSmartPointer<vtkSMProxy> ExodusHelper;
   QVector<double> TimestepValues;
+
+  QSet<QTreeWidgetItem*> PendingChangedItems;
+  QString PendingChangedItemsPropertyName;
+  QTimer PendingChangedItemsTimer;
 };
 
 pqExodusPanel::pqExodusPanel(pqProxy* object_proxy, QWidget* p) :
@@ -86,7 +90,11 @@ pqExodusPanel::pqExodusPanel(pqProxy* object_proxy, QWidget* p) :
 {
   this->UI = new pqUI(this);
   this->UI->setupUi(this);
-  
+
+  this->UI->PendingChangedItemsTimer.setInterval(10/*milliseconds*/);
+  QObject::connect(&this->UI->PendingChangedItemsTimer, SIGNAL(timeout()),
+    this, SLOT(updatePendingChangedItems()));
+
   QObject::connect(this->UI->DisplayType, SIGNAL(currentChanged(int)), 
                    this, SIGNAL(displayTypeChanged()));
   
@@ -151,7 +159,7 @@ void pqExodusPanel::linkServerManagerProperties()
 
   // we hook up the node/element variables
   QTreeWidget* VariablesTree = this->UI->Variables;
-  new pqTreeWidgetCheckHelper(VariablesTree, 0, this);
+  new pqTreeWidgetSelectionHelper(VariablesTree);
   pqTreeWidgetItemObject* item;
   QList<QString> strs;
   QString varName;
@@ -261,7 +269,7 @@ void pqExodusPanel::linkServerManagerProperties()
 
   // we hook up the sideset/nodeset 
   QTreeWidget* SetsTree = this->UI->Sets;
-  new pqTreeWidgetCheckHelper(SetsTree, 0, this);
+  new pqTreeWidgetSelectionHelper(SetsTree);
 
   // do the sidesets
   vtkSMProperty* SideProperty = this->proxy()->GetProperty("SideSetArrayStatus");
@@ -305,7 +313,7 @@ void pqExodusPanel::linkServerManagerProperties()
   QAction* a;
   
   QTreeWidget* BlockTree = this->UI->BlockArrayStatus;
-  new pqTreeWidgetCheckHelper(BlockTree, 0, this);
+  new pqTreeWidgetSelectionHelper(BlockTree);
   a = new QAction("All Blocks On", BlockTree);
   a->setObjectName("BlocksOn");
   QObject::connect(a, SIGNAL(triggered(bool)), BlockTree, SLOT(allOn()));
@@ -573,19 +581,126 @@ void pqExodusPanel::reset()
 
 void pqExodusPanel::blockItemChanged(QTreeWidgetItem* item)
 {
-  this->selectionItemChanged(item, "BlockArrayStatus");
+  if (!this->UI->PendingChangedItemsPropertyName.isEmpty() &&
+    this->UI->PendingChangedItemsPropertyName != "BlockArrayStatus")
+    {
+    this->updatePendingChangedItems();
+    }
+
+  this->UI->PendingChangedItemsPropertyName = "BlockArrayStatus";
+  this->UI->PendingChangedItems.insert(item);
+  this->UI->PendingChangedItemsTimer.start();
+
+  //this->selectionItemChanged(item, "BlockArrayStatus");
 }
 
 void pqExodusPanel::hierarchyItemChanged(QTreeWidgetItem* item)
 {
-  this->selectionItemChanged(item, "HierarchyArrayStatus");
+  if (!this->UI->PendingChangedItemsPropertyName.isEmpty() &&
+    this->UI->PendingChangedItemsPropertyName != "HierarchyArrayStatus")
+    {
+    this->updatePendingChangedItems();
+    }
+
+  this->UI->PendingChangedItemsPropertyName = "HierarchyArrayStatus";
+  this->UI->PendingChangedItems.insert(item);
+  this->UI->PendingChangedItemsTimer.start();
+//  this->selectionItemChanged(item, "HierarchyArrayStatus");
 }
 
 void pqExodusPanel::materialItemChanged(QTreeWidgetItem* item)
 {
-  this->selectionItemChanged(item, "MaterialArrayStatus");
+  if (!this->UI->PendingChangedItemsPropertyName.isEmpty() &&
+    this->UI->PendingChangedItemsPropertyName != "MaterialArrayStatus")
+    {
+    this->updatePendingChangedItems();
+    }
+
+  this->UI->PendingChangedItemsPropertyName = "MaterialArrayStatus";
+  this->UI->PendingChangedItems.insert(item);
+  this->UI->PendingChangedItemsTimer.start();
+//  this->selectionItemChanged(item, "MaterialArrayStatus");
 }
 
+//-----------------------------------------------------------------------------
+void pqExodusPanel::updatePendingChangedItems()
+{
+  QString propName = this->UI->PendingChangedItemsPropertyName;
+  if (propName.isEmpty() || this->UI->PendingChangedItems.size() == 0)
+    {
+    return;
+    }
+
+  vtkSMProxy* pxy = this->UI->ExodusHelper;
+  vtkSMProperty* blockInfo[3];
+  vtkSMProperty* blockStatus[3];
+  int i;
+
+  blockInfo[0] = pxy->GetProperty("BlockArrayInfo");
+  blockInfo[1] = pxy->GetProperty("HierarchyArrayInfo");
+  blockInfo[2] = pxy->GetProperty("MaterialArrayInfo");
+
+  blockStatus[0] = pxy->GetProperty("BlockArrayStatus");
+  blockStatus[1] = pxy->GetProperty("HierarchyArrayStatus");
+  blockStatus[2] = pxy->GetProperty("MaterialArrayStatus");
+
+
+  vtkSMProperty* prop = NULL;
+  prop = pxy->GetProperty(propName.toAscii().data());
+
+  QList< QList< QVariant > > values;
+  // clear out any old stuff
+  for(i=0; i<3; i++)
+    {
+    pqSMAdaptor::setSelectionProperty(blockStatus[i], values);
+    }
+
+  foreach (QTreeWidgetItem* item, this->UI->PendingChangedItems)
+    {
+    // set only the selections that the user changed
+    pqTreeWidgetItemObject* itemObject = 
+      static_cast<pqTreeWidgetItemObject*>(item);
+    int index = values.size();
+    values.append(QList<QVariant>());
+    values[index].append(itemObject->text(0));
+    values[index].append(itemObject->isChecked());
+    }
+
+  // send change down to the vtkExodusReader
+  pqSMAdaptor::setSelectionProperty(prop, values);
+  pxy->UpdateProperty(propName.toAscii().data());
+
+  // get the new selections back
+  for(i=0; i<3; i++)
+    {
+    pxy->UpdatePropertyInformation(blockInfo[i]);
+    blockStatus[i]->Copy(blockInfo[i]);
+    }
+
+  QTreeWidget* widgets[3] =
+    {
+    this->UI->BlockArrayStatus,
+    this->UI->HierarchyArrayStatus,
+    this->UI->MaterialArrayStatus
+    };
+
+  for(i=0; i<3; i++)
+    {
+    values = pqSMAdaptor::getSelectionProperty(blockStatus[i]);
+    for(int j=0; j<values.size(); j++)
+      {
+      pqTreeWidgetItemObject* treeItemObject;
+      treeItemObject = static_cast<pqTreeWidgetItemObject*>(
+        widgets[i]->topLevelItem(j));
+      treeItemObject->setChecked(values[j][1].toBool());
+      }
+    }
+
+  this->UI->PendingChangedItems.clear();
+  this->UI->PendingChangedItemsPropertyName.clear();
+}
+
+//-----------------------------------------------------------------------------
 void pqExodusPanel::selectionItemChanged(QTreeWidgetItem* item,
                                          const QString& propName)
 {
