@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqFilterInputDialog.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -256,16 +256,14 @@ pqFilterInputDialog::pqFilterInputDialog(QWidget *widgetParent)
   this->Sources = new pqFlatTreeView(this);
   this->Sources->setObjectName("Sources");
   this->Sources->getHeader()->hide();
-  this->Sources->setMaximumWidth(150);
-  this->Sources->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
+  this->Sources->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
   baseLayout->addWidget(this->Sources, 1, 1, 2, 1);
 
   // Create the preview pane and add it to the right.
   this->Preview = new pqFlatTreeView(this);
   this->Preview->setObjectName("Preview");
   this->Preview->getHeader()->hide();
-  this->Preview->setMaximumWidth(150);
-  this->Preview->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
+  this->Preview->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
   baseLayout->addWidget(this->Preview, 1, 3, 2, 1);
 
   // Add the separator and the dialog buttons.
