@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqGlobalRenderViewOptions.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -41,43 +41,15 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "vtkType.h"
 
 #include "pqApplicationCore.h"
-#include "pqServerManagerModel.h"
-#include "pqViewModuleInterface.h"
-#include "pqSettings.h"
-#include "pqRenderView.h"
+#include "pqPipelineRepresentation.h"  
 #include "pqPluginManager.h"
-  
+#include "pqRenderView.h"
+#include "pqTwoDRenderView.h"
+#include "pqServerManagerModel.h"
+#include "pqSettings.h"
+#include "pqViewModuleInterface.h"
 
-struct Manip
-{
-  int Mouse;
-  int Shift;
-  int Control;
-  QByteArray Name;
-
-  Manip &operator=(const Manip &other)
-    {
-    this->Mouse = other.Mouse;
-    this->Shift = other.Shift;
-    this->Control = other.Control;
-    this->Name = other.Name;
-    return *this;
-    }
-};
-
-// set up with default names
-static Manip DefaultManips[9] =
-  {
-    { 1, 0, 0, "Rotate"},
-    { 2, 0, 0, "Pan"},
-    { 3, 0, 0, "Zoom"},
-    { 1, 1, 0, "Roll"},
-    { 2, 1, 0, "Rotate"},
-    { 3, 1, 0, "Pan"},
-    { 1, 0, 1, "Zoom"},
-    { 2, 0, 1, "Rotate"},
-    { 3, 0, 1, "Zoom"},
-  };
+typedef pqRenderView::ManipulatorType Manip;
 
 class pqGlobalRenderViewOptions::pqInternal 
   : public Ui::pqGlobalRenderViewOptions
@@ -85,6 +57,8 @@ class pqGlobalRenderViewOptions::pqInternal
 public:
   QList<QComboBox*> CameraControl3DComboBoxList;
   QList<QString> CameraControl3DComboItemList;
+  QList<QComboBox*> CameraControl2DComboBoxList;
+  QList<QString> CameraControl2DComboItemList;
 
   void updateLODThresholdLabel(int value)
     {
@@ -109,6 +83,13 @@ public:
     this->compositeThresholdLabel->setText(
       QVariant(value/10.0).toString() + " MBytes");
     }
+
+  void updateTileDisplayCompositeThresholdLabel(int value)
+    {
+    this->tileDisplayCompositeThresholdLabel->setText(
+      QVariant(value/10.0).toString() + " MBytes");
+    }
+
   void updateSubsamplingRateLabel(int value)
     {
     this->subsamplingRateLabel->setText(QVariant(value).toString() 
@@ -151,11 +132,6 @@ pqGlobalRenderViewOptions::pqGlobalRenderViewOptions(QWidget *widgetParent)
   this->Internal = new pqInternal;
   this->Internal->setupUi(this);
   
-  // outline not used yet
-  this->Internal->label_3->hide();
-  this->Internal->outlineThresholdLabel->hide();
-  this->Internal->outlineThreshold->hide();
-
   this->init();
 }
 
@@ -171,7 +147,7 @@ void pqGlobalRenderViewOptions::init()
       << this->Internal->comboBoxCamera3D_4 << this->Internal->comboBoxCamera3D_5
       << this->Internal->comboBoxCamera3D_6 << this->Internal->comboBoxCamera3D_7
       << this->Internal->comboBoxCamera3D_8 << this->Internal->comboBoxCamera3D_9;
-  
+ 
   this->Internal->CameraControl3DComboItemList //<< "FlyIn" << "FlyOut" << "Move"
      << "Pan" << "Roll" << "Rotate" << "Zoom";
   
@@ -183,6 +159,23 @@ void pqGlobalRenderViewOptions::init()
       }
     }
 
+  this->Internal->CameraControl2DComboBoxList << this->Internal->comboBoxCamera2D
+      << this->Internal->comboBoxCamera2D_2 << this->Internal->comboBoxCamera2D_3
+      << this->Internal->comboBoxCamera2D_4 << this->Internal->comboBoxCamera2D_5
+      << this->Internal->comboBoxCamera2D_6 << this->Internal->comboBoxCamera2D_7
+      << this->Internal->comboBoxCamera2D_8 << this->Internal->comboBoxCamera2D_9;
+
+  this->Internal->CameraControl2DComboItemList //<< "FlyIn" << "FlyOut" << "Move"
+     << "Pan" << "Zoom";
+
+  for ( int cc = 0; cc < this->Internal->CameraControl2DComboBoxList.size(); cc++ )
+    {
+    foreach(QString name, this->Internal->CameraControl2DComboItemList)
+      {
+      this->Internal->CameraControl2DComboBoxList.at(cc)->addItem(name);
+      }
+    }
+
   // start fresh
   this->resetChanges();
 
@@ -198,6 +191,9 @@ void pqGlobalRenderViewOptions::init()
   QObject::connect(this->Internal->compositeThreshold,
     SIGNAL(valueChanged(int)), this, SLOT(compositeThresholdSliderChanged(int)));
 
+  QObject::connect(this->Internal->tileDisplayCompositeThreshold,
+    SIGNAL(valueChanged(int)), this, SLOT(tileDisplayCompositeThresholdSliderChanged(int)));
+
   QObject::connect(this->Internal->subsamplingRate,
     SIGNAL(valueChanged(int)), this, SLOT(subsamplingRateSliderChanged(int)));
 
@@ -225,7 +221,15 @@ void pqGlobalRenderViewOptions::init()
   QObject::connect(this->Internal->triangleStrips,
                   SIGNAL(toggled(bool)),
                   this, SIGNAL(changesAvailable()));
-  
+
+  QObject::connect(this->Internal->depthPeeling,
+                  SIGNAL(toggled(bool)),
+                  this, SIGNAL(changesAvailable()));
+
+  QObject::connect(this->Internal->useOffscreenRenderingForScreenshots,
+                  SIGNAL(toggled(bool)),
+                  this, SIGNAL(changesAvailable()));
+
   QObject::connect(this->Internal->renderingInterrupts,
                   SIGNAL(toggled(bool)),
                   this, SIGNAL(changesAvailable()));
@@ -254,16 +258,33 @@ void pqGlobalRenderViewOptions::init()
                   SIGNAL(toggled(bool)),
                   this, SIGNAL(changesAvailable()));
 
+  QObject::connect(this->Internal->enableTileDisplayCompositing,
+                  SIGNAL(toggled(bool)),
+                  this, SIGNAL(changesAvailable()));
+
   for ( int cc = 0; cc < this->Internal->CameraControl3DComboBoxList.size(); cc++ )
     {
     QObject::connect(this->Internal->CameraControl3DComboBoxList[cc],
                     SIGNAL(currentIndexChanged(int)),
                     this, SIGNAL(changesAvailable()));
     }
+
+  for ( int cc = 0; cc < this->Internal->CameraControl2DComboBoxList.size(); cc++ )
+    {
+    QObject::connect(this->Internal->CameraControl2DComboBoxList[cc],
+                    SIGNAL(currentIndexChanged(int)),
+                    this, SIGNAL(changesAvailable()));
+    }
   
   QObject::connect(this->Internal->resetCameraDefault,
     SIGNAL(clicked()), this, SLOT(resetDefaultCameraManipulators()));
-  
+
+#if defined(__APPLE__)
+  // Offscreen rendering is not needed on Mac (and it doesn't work on some
+  // cards.
+  this->Internal->useOffscreenRenderingForScreenshots->setEnabled(false);
+  this->Internal->useOffscreenRenderingForScreenshots->setChecked(false);
+#endif
 }
 
 void pqGlobalRenderViewOptions::setPage(const QString &page)
@@ -286,6 +307,7 @@ void pqGlobalRenderViewOptions::setPage(const QString &page)
     }
 }
 
+//-----------------------------------------------------------------------------
 QStringList pqGlobalRenderViewOptions::getPageList()
 {
   QStringList pages("Render View");
@@ -296,11 +318,16 @@ QStringList pqGlobalRenderViewOptions::getPageList()
     }
   return pages;
 }
-  
+ 
+//-----------------------------------------------------------------------------
 void pqGlobalRenderViewOptions::applyChanges()
 {
   pqSettings* settings = pqApplicationCore::instance()->settings();
 
+  // This needs to be out of beginGroup()/endGroup().
+  pqPipelineRepresentation::setUnstructuredGridOutlineThreshold(
+    this->Internal->outlineThreshold->value()/10.0);
+
   settings->beginGroup("renderModule");
   
   if (this->Internal->enableLOD->isChecked())
@@ -318,7 +345,13 @@ void pqGlobalRenderViewOptions::applyChanges()
   
   settings->setValue("UseTriangleStrips",
     this->Internal->triangleStrips->isChecked());
-  
+
+  settings->setValue("DepthPeeling",
+    this->Internal->depthPeeling->isChecked());
+
+  settings->setValue("UseOffscreenRenderingForScreenshots",
+    this->Internal->useOffscreenRenderingForScreenshots->isChecked());
+
   settings->setValue("RenderInterruptsEnabled",
     this->Internal->renderingInterrupts->isChecked());
   
@@ -333,6 +366,16 @@ void pqGlobalRenderViewOptions::applyChanges()
     settings->setValue("RemoteRenderThreshold", VTK_DOUBLE_MAX);
     }
 
+  if (this->Internal->enableTileDisplayCompositing->isChecked())
+    {
+    settings->setValue("TileDisplayCompositeThreshold", 
+      this->Internal->tileDisplayCompositeThreshold->value() / 10.0);
+    }
+  else
+    {
+    settings->setValue("TileDisplayCompositeThreshold", VTK_DOUBLE_MAX);
+    }
+
   settings->setValue("DisableOrderedCompositing",
         this->Internal->orderedCompositing->isChecked());
 
@@ -378,9 +421,10 @@ void pqGlobalRenderViewOptions::applyChanges()
   
   // save out camera manipulators
   Manip manips[9];
+  const Manip* default3DManips = pqRenderView::getDefaultManipulatorTypes();
   for(int i=0; i<9; i++)
     {
-    manips[i] = DefaultManips[i];
+    manips[i] = default3DManips[i];
     manips[i].Name =
       this->Internal->CameraControl3DComboBoxList[i]->currentText().toAscii();
     }
@@ -399,6 +443,28 @@ void pqGlobalRenderViewOptions::applyChanges()
 
   settings->endGroup();
 
+  // Now save out 2D camera manipulators (these are saved in a different group).
+  settings->beginGroup("renderModule2D");
+  const Manip* default2DManips = pqTwoDRenderView::getDefaultManipulatorTypes();
+  for(int i=0; i<9; i++)
+    {
+    manips[i] = default2DManips[i];
+    manips[i].Name =
+      this->Internal->CameraControl2DComboBoxList[i]->currentText().toAscii();
+    }
+  
+  strs.clear();
+  for(int i=0; i<9; i++)
+    {
+    strs << QString("Manipulator%1Mouse%2Shift%3Control%4Name%5")
+                    .arg(i+1)
+                    .arg(manips[i].Mouse)
+                    .arg(manips[i].Shift)
+                    .arg(manips[i].Control)
+                    .arg(QString(manips[i].Name));
+    }
+  settings->setValue("InteractorStyle/CameraManipulators", strs);
+  settings->endGroup();
 
   // loop through render views and apply new settings
   QList<pqRenderView*> views =
@@ -412,12 +478,18 @@ void pqGlobalRenderViewOptions::applyChanges()
 
 }
 
+//-----------------------------------------------------------------------------
 void pqGlobalRenderViewOptions::resetChanges()
 {
   pqSettings* settings = pqApplicationCore::instance()->settings();
 
-  settings->beginGroup("renderModule");
+  // This needs to be out of beginGroup()/endGroup().
+  this->Internal->outlineThreshold->setValue(
+    static_cast<int>(
+      pqPipelineRepresentation::getUnstructuredGridOutlineThreshold()*10));
+  this->Internal->updateOutlineThresholdLabel(this->Internal->outlineThreshold->value());
 
+  settings->beginGroup("renderModule");
   QVariant val = settings->value("LODThreshold", 5);
   if(val.toDouble() >= VTK_LARGE_FLOAT)
     {
@@ -437,14 +509,24 @@ void pqGlobalRenderViewOptions::resetChanges()
   
   val = settings->value("UseImmediateMode", true);
   this->Internal->immediateModeRendering->setChecked(val.toBool());
-  
+
   val = settings->value("UseTriangleStrips", false);
   this->Internal->triangleStrips->setChecked(val.toBool());
-  
+
+  val = settings->value("DepthPeeling", true);
+  this->Internal->depthPeeling->setChecked(val.toBool());
+
+  val = settings->value("UseOffscreenRenderingForScreenshots", true);
+  if (getenv("PV_NO_OFFSCREEN_SCREENSHOTS"))
+    {
+    val = false;
+    }
+  this->Internal->useOffscreenRenderingForScreenshots->setChecked(val.toBool());
+
   val = settings->value("RenderInterruptsEnabled", false);
   this->Internal->renderingInterrupts->setChecked(val.toBool());
 
-  
+
   //SquirtLevel"), 3);
   val = settings->value("RemoteRenderThreshold", 3);
   if(val.toDouble() >= VTK_LARGE_FLOAT)
@@ -458,6 +540,21 @@ void pqGlobalRenderViewOptions::resetChanges()
     this->Internal->compositeThreshold->setValue(static_cast<int>(val.toDouble()*10));
     this->Internal->updateCompositeThresholdLabel(this->Internal->compositeThreshold->value());
     }
+
+  val = settings->value("TileDisplayCompositeThreshold", 3);
+  if (val.toDouble() >= VTK_LARGE_FLOAT)
+    {
+    this->Internal->enableTileDisplayCompositing->setCheckState(Qt::Unchecked);
+    this->Internal->updateTileDisplayCompositeThresholdLabel(
+      this->Internal->tileDisplayCompositeThreshold->value());
+    }
+  else
+    {
+    this->Internal->enableTileDisplayCompositing->setCheckState(Qt::Checked);
+    this->Internal->tileDisplayCompositeThreshold->setValue(static_cast<int>(val.toDouble()*10));
+    this->Internal->updateTileDisplayCompositeThresholdLabel(
+      this->Internal->tileDisplayCompositeThreshold->value());
+    }
   
   
   val = settings->value("DisableOrderedCompositing", false);
@@ -521,9 +618,10 @@ void pqGlobalRenderViewOptions::resetChanges()
   val = settings->value("InteractorStyle/CameraManipulators");
 
   Manip manips[9];
+  const Manip* default3DManips = pqRenderView::getDefaultManipulatorTypes();
   for(int k = 0; k < 9; k++)
     {
-    manips[k] = DefaultManips[k];
+    manips[k] = default3DManips[k];
     }
 
   if(val.isValid())
@@ -560,6 +658,49 @@ void pqGlobalRenderViewOptions::resetChanges()
     }
 
   settings->endGroup();
+
+  settings->beginGroup("renderModule2D");
+  val = settings->value("InteractorStyle/CameraManipulators");
+
+  const Manip* default2DManips = pqTwoDRenderView::getDefaultManipulatorTypes();
+  for(int k = 0; k < 9; k++)
+    {
+    manips[k] = default2DManips[k];
+    }
+
+  if(val.isValid())
+    {
+    QStringList strs = val.toStringList();
+    int num = strs.count();
+    for(int i=0; i<num; i++)
+      {
+      Manip tmp;
+      tmp.Name.resize(strs[i].size());
+      int manip_index;
+      if(5 == sscanf(strs[i].toAscii().data(),
+        "Manipulator%dMouse%dShift%dControl%dName%s",
+        &manip_index, &tmp.Mouse, &tmp.Shift, &tmp.Control, tmp.Name.data()))
+        {
+        for(int j=0; j<9; j++)
+          {
+          if(manips[j].Mouse == tmp.Mouse &&
+             manips[j].Shift == tmp.Shift &&
+             manips[j].Control == tmp.Control)
+            {
+            manips[j].Name = tmp.Name.data();
+            break;
+            }
+          }
+        }
+      }
+    }
+
+  for(int i=0; i<9; i++)
+    {
+    int idx = this->Internal->CameraControl2DComboItemList.indexOf(manips[i].Name);
+    this->Internal->CameraControl2DComboBoxList[i]->setCurrentIndex(idx);
+    }
+  settings->endGroup();
   
 }
 
@@ -591,6 +732,13 @@ void pqGlobalRenderViewOptions::compositeThresholdSliderChanged(int value)
   emit this->changesAvailable();
 }
 
+//-----------------------------------------------------------------------------
+void pqGlobalRenderViewOptions::tileDisplayCompositeThresholdSliderChanged(int value)
+{
+  this->Internal->updateTileDisplayCompositeThresholdLabel(value);
+  emit this->changesAvailable();
+}
+
 //-----------------------------------------------------------------------------
 void pqGlobalRenderViewOptions::subsamplingRateSliderChanged(int value)
 {
@@ -622,10 +770,19 @@ void pqGlobalRenderViewOptions::clientCollectSliderChanged(int value)
 //-----------------------------------------------------------------------------
 void pqGlobalRenderViewOptions::resetDefaultCameraManipulators()
 {
+  const Manip* default3DManips = pqRenderView::getDefaultManipulatorTypes();
   for(int i=0; i<9; i++)
     {
-    int idx = this->Internal->CameraControl3DComboItemList.indexOf(DefaultManips[i].Name);
+    int idx = this->Internal->CameraControl3DComboItemList.indexOf(
+      default3DManips[i].Name);
     this->Internal->CameraControl3DComboBoxList[i]->setCurrentIndex(idx);
     }
-}
 
+  const Manip* default2DManips = pqTwoDRenderView::getDefaultManipulatorTypes();
+  for(int i=0; i<9; i++)
+    {
+    int idx = this->Internal->CameraControl2DComboItemList.indexOf(
+      default2DManips[i].Name);
+    this->Internal->CameraControl2DComboBoxList[i]->setCurrentIndex(idx);
+    }
+}
