@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqLookmarkStateLoader.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -36,7 +36,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "vtkSMProxy.h"
 #include "vtkSMProxyManager.h"
 #include "vtkSmartPointer.h"
-#include "vtkSMCompoundProxy.h"
 
 #include <QPointer>
 #include <QList>
@@ -104,7 +103,7 @@ public:
 //-----------------------------------------------------------------------------
 
 vtkStandardNewMacro(pqLookmarkStateLoader);
-vtkCxxRevisionMacro(pqLookmarkStateLoader, "$Revision: 1.18 $");
+vtkCxxRevisionMacro(pqLookmarkStateLoader, "$Revision: 1.23 $");
 //-----------------------------------------------------------------------------
 pqLookmarkStateLoader::pqLookmarkStateLoader()
 {
@@ -445,7 +444,7 @@ void pqLookmarkStateLoader::RegisterProxy(int id, vtkSMProxy* proxy)
     return;
     }
 
-  return this->Superclass::RegisterProxy(id, proxy);
+  this->Superclass::RegisterProxy(id, proxy);
 }
 
 
@@ -528,7 +527,8 @@ int pqLookmarkStateLoader::LoadProxyState(vtkPVXMLElement* proxyElement,
       }
     }
   else if (strcmp(proxyElement->GetName(), "Proxy")==0 && 
-      strcmp(proxyElement->GetAttribute("type"), "RenderView")==0 )
+      (proxy->IsA("vtkSMRenderViewProxy") ||
+       strcmp(proxyElement->GetAttribute("type"), "ClientGraphView")==0 ) )
     {
     unsigned int max = proxyElement->GetNumberOfNestedElements();
     QString name;
@@ -544,10 +544,15 @@ int pqLookmarkStateLoader::LoadProxyState(vtkPVXMLElement* proxyElement,
         {
         toRemove.push_back(element);
         }
+      else if (strcmp(element->GetName(), "Property") == 0 &&
+        (name == "GUISize" || name == "ViewPosition" || name == "ViewSize"))
+        {
+        toRemove.push_back(element);
+        }
       else if (element->GetName() == QString("Property") &&
         element->GetAttribute("name") == QString("Representations"))
         {
-        // remove unused displays from the render module's displays proxyproperty
+        // remove unused displays from the view's displays proxyproperty
         QStringList ids = this->Internal->IdsOfProxyElementsToIgnore;
         displaysToRemove.clear();
         for(int k=0;k<ids.size();k++)
