@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqNamedWidgets.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -47,29 +47,33 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <QTextEdit>
 #include <QTreeWidget>
 #include <QLabel>
+#include <QMetaObject>
+#include <QMetaProperty>
 
 // VTK includes
 
 // ParaView Server Manager includes
+#include "vtkCollection.h"
+#include "vtkPVXMLElement.h"
+#include "vtkSmartPointer.h"
+#include "vtkSMCompositeTreeDomain.h"
+#include "vtkSMDomainIterator.h"
 #include "vtkSMEnumerationDomain.h"
+#include "vtkSMOrderedPropertyIterator.h"
 #include "vtkSMProperty.h"
 #include "vtkSMPropertyIterator.h"
 #include "vtkSMStringListDomain.h"
-#include "vtkPVXMLElement.h"
-#include "vtkSMOrderedPropertyIterator.h"
-#include "vtkSMDomainIterator.h"
-#include "vtkSMVectorProperty.h"
-#include "vtkSmartPointer.h"
-#include "vtkCollection.h"
+#include "vtkSMIntVectorProperty.h"
 
 // ParaView includes
 #include "pq3DWidget.h"
 #include "pqApplicationCore.h"
 #include "pqCollapsedGroup.h"
 #include "pqComboBoxDomain.h"
-#include "pqDoubleRangeWidgetDomain.h"
+#include "pqSignalAdaptorCompositeTreeWidget.h"
 #include "pqDoubleRangeWidget.h"
-#include "pqDoubleSpinBoxDomain.h"
+#include "pqIntRangeWidget.h"
+#include "pqWidgetRangeDomain.h"
 #include "pqFieldSelectionAdaptor.h"
 #include "pqFileChooserWidget.h"
 #include "pqListWidgetItemObject.h"
@@ -81,16 +85,17 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqServerManagerObserver.h"
 #include "pqSignalAdaptorSelectionTreeWidget.h"
 #include "pqSignalAdaptors.h"
-#include "pqSliderDomain.h"
 #include "pqSMAdaptor.h"
 #include "pqSMProxy.h"
 #include "pqSMSignalAdaptors.h"
-#include "pqSpinBoxDomain.h"
-#include "pqTreeWidgetCheckHelper.h"
 #include "pqTreeWidget.h"
 #include "pqTreeWidgetItemObject.h"
+#include "pqTreeWidgetSelectionHelper.h"
 
-void pqNamedWidgets::link(QWidget* parent, pqSMProxy proxy, pqPropertyManager* property_manager)
+//-----------------------------------------------------------------------------
+void pqNamedWidgets::link(QWidget* parent, pqSMProxy proxy, 
+  pqPropertyManager* property_manager,
+  const QStringList* exceptions/*=0*/)
 {
   if(!parent || !proxy || !property_manager)
     {
@@ -100,16 +105,17 @@ void pqNamedWidgets::link(QWidget* parent, pqSMProxy proxy, pqPropertyManager* p
   vtkSMPropertyIterator *iter = proxy->NewPropertyIterator();
   for(iter->Begin(); !iter->IsAtEnd(); iter->Next())
     {
-    vtkSMProperty* SMProperty = iter->GetProperty();
-    if(SMProperty->GetInformationOnly())
+    // all property names with special characters are changed
+    QString propertyName = iter->GetKey();
+    if (exceptions && exceptions->contains(propertyName))
       {
       continue;
       }
-
-    // all property names with special characters are changed
-    QString propertyName = iter->GetKey();
     propertyName.replace(':', '_');
-    propertyName.replace(' ', '_');
+    
+    // escape regex chars
+    propertyName.replace(')', "\\)");
+    propertyName.replace('(', "\\(");
 
     const QString regex = QString("^%1$|^%1_.*$").arg(propertyName);
     QList<QObject*> foundObjects = parent->findChildren<QObject*>(QRegExp(regex));
@@ -124,6 +130,7 @@ void pqNamedWidgets::link(QWidget* parent, pqSMProxy proxy, pqPropertyManager* p
 
 }
 
+//-----------------------------------------------------------------------------
 void pqNamedWidgets::linkObject(QObject* object, pqSMProxy proxy,
                          const QString& property, pqPropertyManager* property_manager)
 {
@@ -154,50 +161,11 @@ void pqNamedWidgets::linkObject(QObject* object, pqSMProxy proxy,
         }
       if(index != -1)
         {
-        QLineEdit* le = qobject_cast<QLineEdit*>(propertyWidget);
-        QSlider* sl = qobject_cast<QSlider*>(propertyWidget);
-        QDoubleSpinBox* doubleSpinBox = qobject_cast<QDoubleSpinBox*>(object);
-        pqDoubleRangeWidget* doubleRange = qobject_cast<pqDoubleRangeWidget*>(object);
-        QSpinBox* spinBox = qobject_cast<QSpinBox*>(object);
-        if(le)
+        QString userProperty, userSignal;
+        if(pqNamedWidgets::propertyInformation(propertyWidget, userProperty, userSignal))
           {
-          property_manager->registerLink(
-            le, "text", SIGNAL(textChanged(const QString&)),
-            proxy, SMProperty, index);
-          }
-        else if(sl)
-          {
-          pqSliderDomain* d0 = new
-            pqSliderDomain(sl, SMProperty, index);
-          d0->setObjectName("SliderDomain");
-          property_manager->registerLink(
-            sl, "value", SIGNAL(valueChanged(int)),
-            proxy, SMProperty, index);
-          }
-        else if(doubleSpinBox)
-          {
-          pqDoubleSpinBoxDomain* d0 = new
-            pqDoubleSpinBoxDomain(doubleSpinBox, SMProperty, index);
-          d0->setObjectName("DoubleSpinBoxDomain");
-          property_manager->registerLink(doubleSpinBox, "value", SIGNAL(valueChanged(double)),
-                                             proxy, SMProperty, index);
-          }
-        else if(doubleRange)
-          {
-          pqDoubleRangeWidgetDomain* d0 = new
-            pqDoubleRangeWidgetDomain(doubleRange, SMProperty, index);
-          d0->setObjectName("DoubleRangeWidgetDomain");
-          property_manager->registerLink(doubleRange, "value", SIGNAL(valueChanged(double)),
-                                             proxy, SMProperty, index);
-          }
-        else if(spinBox)
-          {
-          pqSpinBoxDomain* d0 = new
-            pqSpinBoxDomain(spinBox, SMProperty, index);
-          d0->setObjectName("SpinBoxDomain");
-          property_manager->registerLink(spinBox, "value",
-                                         SIGNAL(valueChanged(int)),
-                                         proxy, SMProperty, index);
+          pqNamedWidgets::linkObject(propertyWidget, userProperty, userSignal,
+            proxy, SMProperty, index, property_manager);
           }
         }
       }
@@ -205,18 +173,8 @@ void pqNamedWidgets::linkObject(QObject* object, pqSMProxy proxy,
   else if(pt == pqSMAdaptor::ENUMERATION)
     {
     // enumerations can be combo boxes or check boxes
-    QCheckBox* checkBox = qobject_cast<QCheckBox*>(object);
     QComboBox* comboBox = qobject_cast<QComboBox*>(object);
-    QLineEdit* lineEdit = qobject_cast<QLineEdit*>(object);
-    pqTreeWidget* treeWidget = 
-      qobject_cast<pqTreeWidget*>(object);
-    if(checkBox)
-      {
-      property_manager->registerLink(
-        checkBox, "checked", SIGNAL(toggled(bool)),
-        proxy, SMProperty);
-      }
-    else if(comboBox)
+    if(comboBox)
       {
       pqComboBoxDomain* d0 = new pqComboBoxDomain(comboBox, SMProperty);
       d0->setObjectName("ComboBoxDomain");
@@ -228,15 +186,14 @@ void pqNamedWidgets::linkObject(QObject* object, pqSMProxy proxy,
         adaptor, "currentText", SIGNAL(currentTextChanged(const QString&)),
         proxy, SMProperty);
       }
-    else if(lineEdit)
+    else
       {
-      property_manager->registerLink(
-        lineEdit, "text", SIGNAL(textChanged(const QString&)),
-        proxy, SMProperty);
-      }
-    else if (treeWidget)
-      {
-      Q_ASSERT("invalid tree widget for enumeration\n" == 0);
+      QString userProperty, userSignal;
+      if(pqNamedWidgets::propertyInformation(object, userProperty, userSignal))
+        {
+        pqNamedWidgets::linkObject(object, userProperty, userSignal,
+          proxy, SMProperty, -1, property_manager);
+        }
       }
     }
   else if(pt == pqSMAdaptor::SELECTION)
@@ -326,15 +283,11 @@ void pqNamedWidgets::linkObject(QObject* object, pqSMProxy proxy,
 
       }
     }
-  else if(pt == pqSMAdaptor::SINGLE_ELEMENT)
+  else if(pt == pqSMAdaptor::SINGLE_ELEMENT || pt == pqSMAdaptor::FILE_LIST)
     {
     QComboBox* comboBox = qobject_cast<QComboBox*>(object);
-    QLineEdit* lineEdit = qobject_cast<QLineEdit*>(object);
     QTextEdit* textEdit = qobject_cast<QTextEdit*>(object);
-    QSlider* slider = qobject_cast<QSlider*>(object);
-    QSpinBox* spinBox = qobject_cast<QSpinBox*>(object);
-    QDoubleSpinBox* doubleSpinBox = qobject_cast<QDoubleSpinBox*>(object);
-    pqDoubleRangeWidget* doubleRange = qobject_cast<pqDoubleRangeWidget*>(object);
+
     if(comboBox)
       {
       // these combo boxes tend to be true/false combos
@@ -348,12 +301,6 @@ void pqNamedWidgets::linkObject(QObject* object, pqSMProxy proxy,
         adaptor, "currentText", SIGNAL(currentTextChanged(const QString&)),
         proxy, SMProperty);
       }
-    else if(lineEdit)
-      {
-      property_manager->registerLink(
-        lineEdit, "text", SIGNAL(textChanged(const QString&)),
-        proxy, SMProperty);
-      }
     else if(textEdit)
       {
       pqSignalAdaptorTextEdit *adaptor = 
@@ -363,60 +310,14 @@ void pqNamedWidgets::linkObject(QObject* object, pqSMProxy proxy,
         adaptor, "text", SIGNAL(textChanged()),
         proxy, SMProperty);
       }
-    else if(slider)
+    else
       {
-      pqSliderDomain* d0 = new
-        pqSliderDomain(slider, SMProperty);
-      d0->setObjectName("SliderDomain");
-      property_manager->registerLink(
-        slider, "value", SIGNAL(valueChanged(int)),
-        proxy, SMProperty);
-      }
-    else if(doubleSpinBox)
-      {
-      pqDoubleSpinBoxDomain* d0 = new
-        pqDoubleSpinBoxDomain(doubleSpinBox, SMProperty);
-      d0->setObjectName("DoubleSpinBoxDomain");
-      property_manager->registerLink(
-        doubleSpinBox, "value", SIGNAL(valueChanged(double)),
-        proxy, SMProperty);
-      }
-    else if(doubleRange)
-      {
-      pqDoubleRangeWidgetDomain* d0 = new
-        pqDoubleRangeWidgetDomain(doubleRange, SMProperty);
-      d0->setObjectName("DoubleRangeWidgetDomain");
-      property_manager->registerLink(
-        doubleRange, "value", SIGNAL(valueChanged(double)),
-        proxy, SMProperty);
-      }
-    else if(spinBox)
-      {
-      pqSpinBoxDomain* d0 = new
-        pqSpinBoxDomain(spinBox, SMProperty);
-      d0->setObjectName("SpinBoxDomain");
-
-      property_manager->registerLink(
-        spinBox, "value", SIGNAL(valueChanged(int)),
-        proxy, SMProperty);
-      }
-    }
-  else if(pt == pqSMAdaptor::FILE_LIST)
-    {
-    QLineEdit* lineEdit = qobject_cast<QLineEdit*>(object);
-    pqFileChooserWidget* chooser = 
-           qobject_cast<pqFileChooserWidget*>(object);
-    if(lineEdit)
-      {
-      property_manager->registerLink(
-        lineEdit, "text", SIGNAL(textChanged(const QString&)),
-        proxy, SMProperty);
-      }
-    else if(chooser)
-      {
-      property_manager->registerLink(
-        chooser, "Filename", SIGNAL(filenameChanged(const QString&)),
-        proxy, SMProperty);
+      QString userProperty, userSignal;
+      if(pqNamedWidgets::propertyInformation(object, userProperty, userSignal))
+        {
+        pqNamedWidgets::linkObject(object, userProperty, userSignal,
+          proxy, SMProperty, -1, property_manager);
+        }
       }
     }
   else if(pt == pqSMAdaptor::FIELD_SELECTION)
@@ -447,7 +348,7 @@ void pqNamedWidgets::linkObject(QObject* object, pqSMProxy proxy,
           new pqSignalAdaptorComboBox(comboBox);
         adaptor->setObjectName("ComboBoxAdaptor");
         property_manager->registerLink(
-          adaptor, "currentText", SIGNAL(currentTextChanged(const QString&)),
+          adaptor, "currentData", SIGNAL(currentTextChanged(const QString&)),
           proxy, SMProperty, 1);  // 1 means link scalar for field selection
         }
       else
@@ -465,9 +366,26 @@ void pqNamedWidgets::linkObject(QObject* object, pqSMProxy proxy,
         }
       }
     }
+  else if (pt == pqSMAdaptor::COMPOSITE_TREE)
+    {
+    QTreeWidget* tree = qobject_cast<QTreeWidget*>(object);
+    if (tree)
+      {
+      pqSignalAdaptorCompositeTreeWidget* treeAdaptor = 
+        new pqSignalAdaptorCompositeTreeWidget(tree,
+          vtkSMIntVectorProperty::SafeDownCast(SMProperty));
+      pqTreeWidgetSelectionHelper* helper = 
+        new pqTreeWidgetSelectionHelper(tree);
+      helper->setObjectName("CompositeTreeSelectionHelper");
+      treeAdaptor->setObjectName("CompositeTreeAdaptor");
+      property_manager->registerLink(
+        treeAdaptor, "values", SIGNAL(valuesChanged()),
+        proxy, SMProperty);
+      }
+    }
 }
 
-
+//-----------------------------------------------------------------------------
 void pqNamedWidgets::unlink(QWidget* parent, pqSMProxy proxy, pqPropertyManager* property_manager)
 {
   if(!parent || !proxy || !property_manager)
@@ -478,16 +396,14 @@ void pqNamedWidgets::unlink(QWidget* parent, pqSMProxy proxy, pqPropertyManager*
   vtkSMPropertyIterator *iter = proxy->NewPropertyIterator();
   for(iter->Begin(); !iter->IsAtEnd(); iter->Next())
     {
-    vtkSMProperty* SMProperty = iter->GetProperty();
-    if(SMProperty->GetInformationOnly())
-      {
-      continue;
-      }
-
     // all property names with special characters are changed
     QString propertyName = iter->GetKey();
     propertyName.replace(':', '_');
-    propertyName.replace(' ', '_');
+
+    // escape regex chars
+    propertyName.replace(')', "\\)");
+    propertyName.replace('(', "\\(");
+
     const QString regex = QString("^%1$|^%1_.*$").arg(propertyName);
     QList<QObject*> foundObjects = parent->findChildren<QObject*>(QRegExp(regex));
     for(int i=0; i<foundObjects.size(); i++)
@@ -501,6 +417,7 @@ void pqNamedWidgets::unlink(QWidget* parent, pqSMProxy proxy, pqPropertyManager*
   proxy->UpdateVTKObjects();
 }
 
+//-----------------------------------------------------------------------------
 void pqNamedWidgets::unlinkObject(QObject* object, pqSMProxy proxy,
                            const QString& property, pqPropertyManager* property_manager)
 {
@@ -529,67 +446,11 @@ void pqNamedWidgets::unlinkObject(QObject* object, pqSMProxy proxy,
         }
       if(index != -1)
         {
-        QLineEdit* le = qobject_cast<QLineEdit*>(propertyWidget);
-        QSlider* sl = qobject_cast<QSlider*>(propertyWidget);
-        QDoubleSpinBox* doubleSpinBox = qobject_cast<QDoubleSpinBox*>(propertyWidget);
-        pqDoubleRangeWidget* doubleRange = qobject_cast<pqDoubleRangeWidget*>(propertyWidget);
-        QSpinBox* spinBox = qobject_cast<QSpinBox*>(propertyWidget);
-        if(le)
+        QString userProperty, userSignal;
+        if(pqNamedWidgets::propertyInformation(propertyWidget, userProperty, userSignal))
           {
-          property_manager->unregisterLink(
-            le, "text", SIGNAL(textChanged(const QString&)),
-            proxy, SMProperty, index);
-          }
-        else if(sl)
-          {
-          pqSliderDomain* d0 = 
-            sl->findChild<pqSliderDomain*>("SliderDomain");
-          if(d0)
-            {
-            delete d0;
-            }
-          property_manager->unregisterLink(
-            sl, "value", SIGNAL(valueChanged(int)),
-            proxy, SMProperty, index);
-          }
-        else if(doubleSpinBox)
-          {
-          pqDoubleSpinBoxDomain* d0 = 
-            doubleSpinBox->findChild<pqDoubleSpinBoxDomain*>("DoubleSpinBoxDomain");
-          if(d0)
-            {
-            delete d0;
-            }
-          property_manager->unregisterLink(doubleSpinBox, 
-                                              "value", 
-                                              SIGNAL(valueChanged(double)),
-                                              proxy, SMProperty);
-          }
-        else if(doubleRange)
-          {
-          pqDoubleRangeWidgetDomain* d0 = 
-            doubleRange->findChild<pqDoubleRangeWidgetDomain*>("DoubleRangeWidgetDomain");
-          if(d0)
-            {
-            delete d0;
-            }
-          property_manager->unregisterLink(doubleRange, 
-                                              "value", 
-                                              SIGNAL(valueChanged(double)),
-                                              proxy, SMProperty);
-          }
-        else if(spinBox)
-          {
-          pqSpinBoxDomain* d0 = 
-            spinBox->findChild<pqSpinBoxDomain*>("SpinBoxDomain");
-          if(d0)
-            {
-            delete d0;
-            }
-          property_manager->unregisterLink(spinBox, 
-                                           "value", 
-                                           SIGNAL(valueChanged(int)),
-                                           proxy, SMProperty);
+          pqNamedWidgets::unlinkObject(propertyWidget, userProperty, userSignal,
+            proxy, SMProperty, index, property_manager);
           }
         }
       }
@@ -597,18 +458,8 @@ void pqNamedWidgets::unlinkObject(QObject* object, pqSMProxy proxy,
   else if(pt == pqSMAdaptor::ENUMERATION)
     {
     // enumerations can be combo boxes or check boxes
-    QCheckBox* checkBox = qobject_cast<QCheckBox*>(object);
     QComboBox* comboBox = qobject_cast<QComboBox*>(object);
-    QLineEdit* lineEdit = qobject_cast<QLineEdit*>(object);
-    pqTreeWidget* treeWidget = 
-      qobject_cast<pqTreeWidget*>(object);
-    if(checkBox)
-      {
-      property_manager->unregisterLink(
-        checkBox, "checked", SIGNAL(toggled(bool)),
-        proxy, SMProperty);
-      }
-    else if(comboBox)
+    if(comboBox)
       {
       pqComboBoxDomain* d0 =
         comboBox->findChild<pqComboBoxDomain*>("ComboBoxDomain");
@@ -627,15 +478,14 @@ void pqNamedWidgets::unlinkObject(QObject* object, pqSMProxy proxy,
         delete adaptor;
         }
       }
-    else if(lineEdit)
+    else
       {
-      property_manager->unregisterLink(
-        lineEdit, "text", SIGNAL(textChanged(const QString&)),
-        proxy, SMProperty);
-      }
-    else if (treeWidget)
-      {
-      Q_ASSERT("invalid tree widget for enumeration\n" == 0);
+      QString userProperty, userSignal;
+      if(pqNamedWidgets::propertyInformation(object, userProperty, userSignal))
+        {
+        pqNamedWidgets::unlinkObject(object, userProperty, userSignal,
+          proxy, SMProperty, -1, property_manager);
+        }
       }
     }
   else if(pt == pqSMAdaptor::SELECTION)
@@ -681,14 +531,10 @@ void pqNamedWidgets::unlinkObject(QObject* object, pqSMProxy proxy,
       delete comboAdaptor;
       }
     }
-  else if(pt == pqSMAdaptor::SINGLE_ELEMENT)
+  else if(pt == pqSMAdaptor::SINGLE_ELEMENT || pt == pqSMAdaptor::FILE_LIST)
     {
     QComboBox* comboBox = qobject_cast<QComboBox*>(object);
-    QLineEdit* lineEdit = qobject_cast<QLineEdit*>(object);
-    QSlider* slider = qobject_cast<QSlider*>(object);
-    QDoubleSpinBox* doubleSpinBox = qobject_cast<QDoubleSpinBox*>(object);
-    pqDoubleRangeWidget* doubleRange = qobject_cast<pqDoubleRangeWidget*>(object);
-    QSpinBox* spinBox = qobject_cast<QSpinBox*>(object);
+    QTextEdit* textEdit = qobject_cast<QTextEdit*>(object);
     if(comboBox)
       {
       pqComboBoxDomain* d0 =
@@ -705,77 +551,41 @@ void pqNamedWidgets::unlinkObject(QObject* object, pqSMProxy proxy,
         proxy, SMProperty);
       delete adaptor;
       }
-    else if(lineEdit)
+    else if(textEdit)
       {
+      pqSignalAdaptorTextEdit* adaptor = 
+        textEdit->findChild<pqSignalAdaptorTextEdit*>("TextEditAdaptor");
       property_manager->unregisterLink(
-        lineEdit, "text", SIGNAL(textChanged(const QString&)),
+        adaptor, "text", SIGNAL(textChanged()),
         proxy, SMProperty);
       }
-    else if(slider)
+    else
       {
-      pqSliderDomain* d0 = 
-        slider->findChild<pqSliderDomain*>("SliderDomain");
-      if(d0)
+      QString userProperty, userSignal;
+      if(pqNamedWidgets::propertyInformation(object, userProperty, userSignal))
         {
-        delete d0;
+        pqNamedWidgets::unlinkObject(object, userProperty, userSignal,
+          proxy, SMProperty, -1, property_manager);
         }
-      property_manager->unregisterLink(
-        slider, "value", SIGNAL(valueChanged(int)),
-        proxy, SMProperty);
-      }
-    else if(doubleSpinBox)
-      {
-      pqDoubleSpinBoxDomain* d0 = 
-        doubleSpinBox->findChild<pqDoubleSpinBoxDomain*>("DoubleSpinBoxDomain");
-      if(d0)
-        {
-        delete d0;
-        }
-      property_manager->unregisterLink(
-        doubleSpinBox, "value", SIGNAL(valueChanged(double)),
-        proxy, SMProperty);
-      }
-    else if(doubleRange)
-      {
-      pqDoubleRangeWidgetDomain* d0 = 
-        doubleRange->findChild<pqDoubleRangeWidgetDomain*>("DoubleRangeWidgetDomain");
-      if(d0)
-        {
-        delete d0;
-        }
-      property_manager->unregisterLink(
-        doubleRange, "value", SIGNAL(valueChanged(double)),
-        proxy, SMProperty);
-      }
-    else if(spinBox)
-      {
-      pqSpinBoxDomain* d0 = 
-        spinBox->findChild<pqSpinBoxDomain*>("SpinBoxDomain");
-      if(d0)
-        {
-        delete d0;
-        }
-      property_manager->unregisterLink(
-        spinBox, "value", SIGNAL(valueChanged(int)),
-        proxy, SMProperty);
       }
     }
-  else if(pt == pqSMAdaptor::FILE_LIST)
+  else if (pt == pqSMAdaptor::COMPOSITE_TREE)
     {
-    QLineEdit* lineEdit = qobject_cast<QLineEdit*>(object);
-    pqFileChooserWidget* chooser = 
-           qobject_cast<pqFileChooserWidget*>(object);
-    if(lineEdit)
+    QTreeWidget* tree = qobject_cast<QTreeWidget*>(object);
+    if (tree)
       {
+      pqSignalAdaptorCompositeTreeWidget* treeAdaptor = 
+        tree->findChild<pqSignalAdaptorCompositeTreeWidget*>(
+          "CompositeTreeAdaptor");
       property_manager->unregisterLink(
-        lineEdit, "text", SIGNAL(textChanged(const QString&)),
-        proxy, SMProperty);
-      }
-    else if(chooser)
-      {
-      property_manager->unregisterLink(
-        chooser, "Filename", SIGNAL(filenameChanged(const QString&)),
+        treeAdaptor, "values", SIGNAL(valuesChanged()),
         proxy, SMProperty);
+      delete treeAdaptor;
+
+      pqTreeWidgetSelectionHelper* helper = 
+        tree->findChild<pqTreeWidgetSelectionHelper*>(
+          "CompositeTreeSelectionHelper");
+      delete helper;
       }
     }
   else if(pt == pqSMAdaptor::FIELD_SELECTION)
@@ -957,21 +767,23 @@ static void setupValidator(QLineEdit* lineEdit, QVariant::Type type)
     }
 }
 
+//-----------------------------------------------------------------------------
 static QLabel* createPanelLabel(QWidget* parent, QString text, QString pname)
 {
   QLabel* label = new QLabel(parent);
-  label->setObjectName(pname+QString("_label"));
+  label->setObjectName(QString("_labelFor")+pname);
   label->setText(text);
   label->setWordWrap(true);
   return label;
 }
 
+//-----------------------------------------------------------------------------
 void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
                                                vtkSMProxy* pxy)
 {
   int rowCount = 0;
   int skippedFirstFileProperty = 0;
-  bool isCompoundProxy = pxy->IsA("vtkSMCompoundProxy");
+  bool isCompoundProxy = pxy->IsA("vtkSMCompoundSourceProxy");
 
   // query for proxy properties, and create widgets
   vtkSMOrderedPropertyIterator *iter = vtkSMOrderedPropertyIterator::New();
@@ -1007,13 +819,14 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
   for(iter->Begin(); !iter->IsAtEnd(); iter->Next())
     {
     vtkSMProperty* SMProperty = iter->GetProperty();
+    bool informationOnly = SMProperty->GetInformationOnly();
+
     QString propertyName = iter->GetKey();
     if (propertiesToHide.contains(propertyName))
       {
       continue;
       }
     propertyName.replace(':', '_');
-    propertyName.replace(' ', '_');
 
     // When this proxy represents a custom filter, we want to use the property 
     //  names the user provided rather than the built in property labels:
@@ -1028,11 +841,12 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
       }
 
     // skip information properties
-    if(SMProperty->GetInformationOnly() || SMProperty->GetIsInternal())
+    if((!propertiesToShow.contains(propertyName) && informationOnly) || 
+        SMProperty->GetIsInternal())
       {
       continue;
       }
-
+    
     if (!propertiesToShow.contains(propertyName))
       {
       if (SMProperty->IsA("vtkSMStringVectorProperty") && !skippedFirstFileProperty)
@@ -1078,6 +892,10 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
       {
       // create a combo box with list of proxies
       QComboBox* combo = new QComboBox(panelLayout->parentWidget());
+      if(informationOnly)
+        {
+        combo->setEnabled(false);
+        }
       combo->setObjectName(propertyName);
       QLabel* label = createPanelLabel(panelLayout->parentWidget(),
                                        propertyLabel,
@@ -1097,6 +915,10 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
         new pqProxySelectionWidget(pxy, iter->GetKey(),
                                    propertyLabel,
                                    panelLayout->parentWidget());
+      if(informationOnly)
+        {
+        w->setEnabled(false);
+        }
       w->setObjectName(propertyName);
       panelLayout->addWidget(w, rowCount, 0, 1, 2);
       rowCount++;
@@ -1110,6 +932,10 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
         QCheckBox* check;
         check = new QCheckBox(propertyLabel, 
                               panelLayout->parentWidget());
+        if(informationOnly)
+          {
+          check->setEnabled(false);
+          }
         check->setObjectName(propertyName);
         panelLayout->addWidget(check, rowCount, 0, 1, 2);
         rowCount++;
@@ -1123,13 +949,19 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
         if (vp && vp->GetRepeatCommand())
           {
           QTreeWidget* tw = new pqTreeWidget(panelLayout->parentWidget());
+          if(informationOnly)
+            {
+            tw->setEnabled(false);
+            }
           tw->setColumnCount(1);
           tw->setRootIsDecorated(false);
           QTreeWidgetItem* h = new QTreeWidgetItem();
           h->setData(0, Qt::DisplayRole, propertyLabel);
           tw->setHeaderItem(h);
           tw->setObjectName(propertyName);
-          new pqTreeWidgetCheckHelper(tw, 0, tw);
+          pqTreeWidgetSelectionHelper* helper = 
+            new pqTreeWidgetSelectionHelper(tw);
+          helper->setObjectName(QString("%1Helper").arg(propertyName));
           panelLayout->addWidget(tw, rowCount, 0, 1, 2);
           rowCount++;
           }
@@ -1137,6 +969,10 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
           {
           // combo box with strings
           QComboBox* combo = new QComboBox(panelLayout->parentWidget());
+          if(informationOnly)
+            {
+            combo->setEnabled(false);
+            }
           combo->setObjectName(propertyName);
           QLabel* label = createPanelLabel(panelLayout->parentWidget(),
                                            propertyLabel,
@@ -1173,13 +1009,19 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
         QList<QList<QVariant> > items;
         items = pqSMAdaptor::getSelectionProperty(SMProperty);
         QTreeWidget* tw = new pqTreeWidget(panelLayout->parentWidget());
+        if(informationOnly)
+          {
+          tw->setEnabled(false);
+          }
         tw->setColumnCount(1);
         tw->setRootIsDecorated(false);
         QTreeWidgetItem* h = new QTreeWidgetItem();
         h->setData(0, Qt::DisplayRole, header);
         tw->setHeaderItem(h);
         tw->setObjectName(name);
-        new pqTreeWidgetCheckHelper(tw, 0, tw);
+        pqTreeWidgetSelectionHelper* helper = 
+          new pqTreeWidgetSelectionHelper(tw);
+        helper->setObjectName(QString("%1Helper").arg(propertyName));
         panelLayout->addWidget(tw, rowCount, 0, 1, 2);
         rowCount++;
         }
@@ -1193,6 +1035,10 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
         {
         // combo box with strings
         QComboBox* combo = new QComboBox(panelLayout->parentWidget());
+        if(informationOnly)
+          {
+          combo->setEnabled(false);
+          }
         combo->setObjectName(propertyName);
         QLabel* label = createPanelLabel(panelLayout->parentWidget(),
                                          propertyLabel,
@@ -1209,21 +1055,19 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
         QLabel* label = createPanelLabel(panelLayout->parentWidget(),
                                          propertyLabel,
                                          propertyName);
-        QSlider* slider;
-        slider = new QSlider(Qt::Horizontal, panelLayout->parentWidget());
-        slider->setObjectName(QString(propertyName) + "_Slider");
-        slider->setRange(propertyDomain[0].toInt(), propertyDomain[1].toInt());
 
-        QLineEdit* lineEdit = new QLineEdit(panelLayout->parentWidget());
-        lineEdit->setObjectName(propertyName);
-        setupValidator(lineEdit, elem_property.type()); 
+        pqIntRangeWidget* range;
+        range = new pqIntRangeWidget(panelLayout->parentWidget());
+        if(informationOnly)
+          {
+          range->setEnabled(false);
+          }
+        range->setObjectName(propertyName);
+        range->setMinimum(propertyDomain[0].toInt());
+        range->setMaximum(propertyDomain[1].toInt());
         panelLayout->addWidget(label, rowCount, 0, 1, 1);
-        QHBoxLayout* hlayout = new QHBoxLayout;
-        hlayout->addWidget(slider);
-        hlayout->addWidget(lineEdit);
-        panelLayout->addLayout(hlayout, rowCount, 1, 1, 1);
-        slider->show();
-        lineEdit->show();
+        panelLayout->addWidget(range, rowCount, 1, 1, 1);
+        range->show();
         rowCount++;
         }
       else if(elem_property.type() == QVariant::Double && 
@@ -1231,23 +1075,15 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
               propertyDomain[0].isValid() && propertyDomain[1].isValid() &&
               domainsTypes.contains("vtkSMDoubleRangeDomain"))
         {
-        /*
-        double range[2];
-        range[0] = propertyDomain[0].toDouble();
-        range[1] = propertyDomain[1].toDouble();
-        */
         QLabel* label = createPanelLabel(panelLayout->parentWidget(),
                                          propertyLabel,
                                          propertyName);
-        /*
-        QDoubleSpinBox* spinBox;
-        spinBox = new QDoubleSpinBox(panelLayout->parentWidget());
-        spinBox->setObjectName(propertyName);
-        spinBox->setRange(range[0], range[1]);
-        spinBox->setSingleStep((range[1] - range[0]) / 20.0);
-        */
         pqDoubleRangeWidget* range;
         range = new pqDoubleRangeWidget(panelLayout->parentWidget());
+        if(informationOnly)
+          {
+          range->setEnabled(false);
+          }
         range->setObjectName(propertyName);
         range->setMinimum(propertyDomain[0].toDouble());
         range->setMaximum(propertyDomain[1].toDouble());
@@ -1277,6 +1113,10 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
         if (!multiLineString)
           {
           QLineEdit* lineEdit = new QLineEdit(panelLayout->parentWidget());
+          if(informationOnly)
+            {
+            lineEdit->setEnabled(false);
+            }
           lineEdit->setObjectName(propertyName);
           setupValidator(lineEdit, elem_property.type()); 
           
@@ -1291,6 +1131,12 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
         else
           {
           QTextEdit *textEdit = new QTextEdit(panelLayout->parentWidget());
+          QFont textFont("Courier");
+          textEdit->setFont(textFont);
+          if(informationOnly)
+            {
+            textEdit->setEnabled(false);
+            }
           textEdit->setObjectName(propertyName);
           textEdit->setAcceptRichText(false);
           // The default tab stop is too big
@@ -1354,6 +1200,10 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
            domain.size() && domain[i].size() */)  // link not supported yet
           {
           QComboBox* combo = new QComboBox(panelLayout->parentWidget());
+          if(informationOnly)
+            {
+            combo->setEnabled(false);
+            }
           combo->setObjectName(QString("%1_%2").arg(propertyName).arg(i));
           item = new QWidgetItem(combo);
           }
@@ -1366,6 +1216,10 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
           range[1] = domain[i][1].toInt();
           QSpinBox* spinBox;
           spinBox = new QSpinBox(panelLayout->parentWidget());
+          if(informationOnly)
+            {
+            spinBox->setEnabled(false);
+            }
           spinBox->setObjectName(QString("%1_%2").arg(propertyName).arg(i));
           spinBox->setRange(range[0], range[1]);
           item = new QWidgetItem(spinBox);
@@ -1374,18 +1228,12 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
                 domain.size() && domain[i].size() == 2 &&
                 domain[i][0].isValid() && domain[i][1].isValid())
           {
-          /*
-          double range[2];
-          range[0] = domain[i][0].toDouble();
-          range[1] = domain[i][1].toDouble();
-          QDoubleSpinBox* spinBox;
-          spinBox = new QDoubleSpinBox(panelLayout->parentWidget());
-          spinBox->setObjectName(QString("%1_%2").arg(propertyName).arg(i));
-          spinBox->setRange(range[0], range[1]);
-          spinBox->setSingleStep((range[1] - range[0]) / 20.0);
-          */
           pqDoubleRangeWidget* range;
           range = new pqDoubleRangeWidget(panelLayout->parentWidget());
+          if(informationOnly)
+            {
+            range->setEnabled(false);
+            }
           range->setObjectName(QString("%1_%2").arg(propertyName).arg(i));
           range->setMinimum(domain[i][0].toDouble());
           range->setMaximum(domain[i][1].toDouble());
@@ -1394,6 +1242,10 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
         else
           {
           QLineEdit* lineEdit = new QLineEdit(panelLayout->parentWidget());
+          if(informationOnly)
+            {
+            lineEdit->setEnabled(false);
+            }
           lineEdit->setObjectName(QString("%1_%2").arg(propertyName).arg(i));
           setupValidator(lineEdit, v.type()); 
           item = new QWidgetItem(lineEdit);
@@ -1437,6 +1289,10 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
       {
       pqFileChooserWidget* chooser;
       chooser = new pqFileChooserWidget(panelLayout->parentWidget());
+      if(informationOnly)
+        {
+        chooser->setEnabled(false);
+        }
       pqServerManagerModel* m =
         pqApplicationCore::instance()->getServerManagerModel();
       chooser->setServer(m->findServer(pxy->GetConnectionID()));
@@ -1455,18 +1311,122 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
                                        "Scalars",
                                        propertyName);
       QComboBox* combo = new QComboBox(panelLayout->parentWidget());
+      if(informationOnly)
+        {
+        combo->setEnabled(false);
+        }
       combo->setObjectName(QString(propertyName));
       panelLayout->addWidget(label, rowCount, 0, 1, 1);
       panelLayout->addWidget(combo, rowCount, 1, 1, 1);
       rowCount++;
       }
+    else if (pt == pqSMAdaptor::COMPOSITE_TREE)
+      {
+      QTreeWidget* tree = new QTreeWidget(panelLayout->parentWidget());
+      if(informationOnly)
+        {
+        tree->setEnabled(false);
+        }
+      tree->setObjectName(propertyName);
+        
+      QTreeWidgetItem* header = new QTreeWidgetItem();
+      header->setData(0, Qt::DisplayRole, propertyLabel);
+      tree->setHeaderItem(header);
+      panelLayout->addWidget(tree, rowCount, 0, 1, 2); 
+      rowCount++;
+      }
     }
   iter->Delete();
-  panelLayout->addItem(new QSpacerItem(0,0,
-                                       QSizePolicy::Expanding,
-                                       QSizePolicy::Expanding), 
-                       rowCount, 0, 1, 2);
+  panelLayout->setRowStretch(rowCount, 1);
   panelLayout->invalidate();
 }
 
+//-----------------------------------------------------------------------------
+bool pqNamedWidgets::propertyInformation(QObject* object, 
+    QString& property, QString& signal)
+{
+  if(!object)
+    {
+    return false;
+    }
+
+  const QMetaObject* mo = object->metaObject();
+  QMetaProperty UserProperty = mo->userProperty();
+
+  if(UserProperty.isValid())
+    {
+    QString propertyName = UserProperty.name();
+    QString signalName;
+    signalName = QString("%1Changed").arg(propertyName);
+    int numMethods = mo->methodCount();
+    int signalIndex = -1;
+    for(int i=0; signalIndex == -1 && i<numMethods; i++)
+      {
+      if(mo->method(i).methodType() == QMetaMethod::Signal)
+        {
+        if(QString(mo->method(i).signature()).startsWith(signalName))
+          {
+          signalIndex = i;
+          }
+        }
+      }
+    if(signalIndex != -1)
+      {
+      QString theSignal = SIGNAL(%1);
+      signal = theSignal.arg(mo->method(signalIndex).signature());
+      property = propertyName;
+      return true;
+      }
+    }
+    
+  QAbstractButton* btn = qobject_cast<QAbstractButton*>(object);
+  if(btn && btn->isCheckable())
+     {
+     property = "checked";
+     signal = SIGNAL(toggled(bool));
+     return true;
+     }
+
+  return false;
+}
+
+//-----------------------------------------------------------------------------
+void pqNamedWidgets::linkObject(QObject* o, const QString& property,
+                       const QString& signal, pqSMProxy proxy,
+                       vtkSMProperty* smProperty, int index,
+                       pqPropertyManager* pm)
+{
+  pm->registerLink(o, property.toAscii().data(), 
+    signal.toAscii().data(), proxy, smProperty, index);
+
+  // if the widget has min or max property, hook it up too
+  if(o->metaObject()->indexOfProperty("minimum") != -1 ||
+     o->metaObject()->indexOfProperty("maximum") != -1)
+    {
+    QWidget* w = qobject_cast<QWidget*>(o);
+    if(w)
+      {
+      pqWidgetRangeDomain* d0 = new
+        pqWidgetRangeDomain(w, "minimum", "maximum", smProperty, index);
+      d0->setObjectName("WidgetRangeDomain");
+      }
+    }
+}
+
+//-----------------------------------------------------------------------------
+void pqNamedWidgets::unlinkObject(QObject* o, const QString& property,
+                       const QString& signal, pqSMProxy proxy,
+                       vtkSMProperty* smProperty, int index,
+                       pqPropertyManager* pm)
+{
+  pqWidgetRangeDomain* d0 = 
+    o->findChild<pqWidgetRangeDomain*>("WidgetRangeDomain");
+  if(d0)
+    {
+    delete d0;
+    }
+  
+  pm->unregisterLink(o, property.toAscii().data(), 
+    signal.toAscii().data(), proxy, smProperty, index);
+}
 
