@@ -3,13 +3,13 @@
    Program:   ParaView
    Module:    $RCSfile: pqPipelineModel.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -55,7 +55,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <QString>
 #include <QtDebug>
 
-#include "vtkSMCompoundProxy.h"
 #include "vtkSMProxy.h"
 
 class pqPipelineModelSource;
@@ -491,19 +490,22 @@ pqPipelineModelItem::VisibleState pqPipelineModelOutput::computeVisibleState(
   // If no view is present, it implies that a suitable type of view
   // will be created.
   pqPipelineModelItem::VisibleState eye = pqPipelineModelItem::NotVisible;
-  if(view)
+  pqApplicationCore* core = pqApplicationCore::instance();
+  pqDisplayPolicy* policy = core->getDisplayPolicy();
+  if (policy)
     {
-    pqDataRepresentation *repr = port->getRepresentation(view);
-    if(repr && repr->isVisible())
+    switch (policy->getVisibility(view, port))
       {
+    case pqDisplayPolicy::Visible:
       eye = pqPipelineModelItem::Visible;
-      }
-    else if(repr || view->canDisplay(port))
-      {
+      break;
+
+    case pqDisplayPolicy::Hidden:
       eye = pqPipelineModelItem::NotVisible;
-      }
-    else
-      {
+      break;
+
+    case pqDisplayPolicy::NotApplicable:
+    default:
       eye = pqPipelineModelItem::NotAllowed;
       }
     }
@@ -550,7 +552,6 @@ pqPipelineModelItem::IconType pqPipelineModelSource::getIconType() const
     return INDETERMINATE;
     }
 
-  /*
   // This has only 1 output port.
   pqApplicationCore* core = pqApplicationCore::instance();
   pqDisplayPolicy* policy = core->getDisplayPolicy();
@@ -567,7 +568,7 @@ pqPipelineModelItem::IconType pqPipelineModelSource::getIconType() const
       return LINECHART;
       }
     }
-    */
+  
   return GEOMETRY;
 }
 
@@ -638,8 +639,9 @@ pqPipelineModelOutputPort::pqPipelineModelOutputPort(
 
 QString pqPipelineModelOutputPort::getName() const
 {
-  // TODO: Let the user rename the output port.
-  return QString("Output%1").arg(this->Port);
+  pqOutputPort* opPort = qobject_cast<pqOutputPort*>(
+    this->getObject());
+  return opPort->getPortName();
 }
 
 pqServerManagerModelItem *pqPipelineModelOutputPort::getObject() const 
@@ -665,7 +667,6 @@ void pqPipelineModelOutputPort::updateVisibleState(pqView *view)
 }
 pqPipelineModelItem::IconType pqPipelineModelOutputPort::getIconType() const
 {
-  /*
   pqApplicationCore* core = pqApplicationCore::instance();
   pqDisplayPolicy* policy = core->getDisplayPolicy();
   if (policy)
@@ -681,7 +682,7 @@ pqPipelineModelItem::IconType pqPipelineModelOutputPort::getIconType() const
       return LINECHART;
       }
     }
-    */
+  
   return GEOMETRY;
 }
 
@@ -1300,7 +1301,7 @@ void pqPipelineModel::addSource(pqPipelineSource *source)
   // the source's xml group name.
   pqPipelineModelSource *item = 0;
   vtkSMProxy *proxy = source->getProxy();
-  if(vtkSMCompoundProxy::SafeDownCast(proxy) != 0)
+  if(proxy->IsA("vtkSMCompoundSourceProxy"))
     {
     item = new pqPipelineModelFilter(server, source,
         pqPipelineModel::CustomFilter);
@@ -2027,15 +2028,13 @@ void pqPipelineModel::initializePixmaps()
     this->PixmapList[pqPipelineModelItem::LINK].load(
       ":/pqWidgets/Icons/pqLinkBack16.png");
     this->PixmapList[pqPipelineModelItem::GEOMETRY].load(
-      ":/pqWidgets/Icons/pqBundle16.png");
+      ":/pqWidgets/Icons/pq3DView16.png");
     this->PixmapList[pqPipelineModelItem::BARCHART].load(
-    //  ":/pqWidgets/Icons/pqBundle16.png");
-        ":/pqWidgets/Icons/pqFilter16.png");
+        ":/pqWidgets/Icons/pqHistogram16.png");
     this->PixmapList[pqPipelineModelItem::LINECHART].load(
-    //  ":/pqWidgets/Icons/pqBundle16.png");
-       ":/pqWidgets/Icons/pqFilter16.png");
+       ":/pqWidgets/Icons/pqLineChart16.png");
     this->PixmapList[pqPipelineModelItem::INDETERMINATE].load(
-      ":/pqWidgets/Icons/pqSource16.png");
+      ":/pqWidgets/Icons/pq3DView16.png");
     this->PixmapList[pqPipelineModelInternal::EYEBALL].load(
       ":/pqWidgets/Icons/pqEyeball16.png");
     this->PixmapList[pqPipelineModelInternal::EYEBALL_GRAY].load(
