@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqViewManager.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -33,12 +33,17 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "ui_pqEmptyView.h"
 
 // VTK includes.
+#include "vtkErrorCode.h"
+#include "vtkImageData.h"
+#include "vtkImageIterator.h"
 #include "vtkPVConfig.h"
 #include "vtkPVXMLElement.h"
+#include "vtkSMAnimationSceneImageWriter.h"
 #include "vtkSmartPointer.h"
 #include "vtkSMIntVectorProperty.h"
 #include "vtkSMProxy.h"
 #include "vtkSMStateLoaderBase.h"
+#include "vtkSMUtilities.h"
 
 // Qt includes.
 #include <QAction>
@@ -56,12 +61,16 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <QtDebug>
 #include <QUuid>
 #include <QApplication>
+#include <QScrollArea>
 
 
 // ParaView includes.
+#include "pqActiveViewOptionsManager.h"
 #include "pqApplicationCore.h"
 #include "pqCloseViewUndoElement.h"
 #include "pqComparativeRenderView.h"
+#include "pqEventDispatcher.h"
+#include "pqImageUtil.h"
 #include "pqMultiViewFrame.h"
 #include "pqObjectBuilder.h"
 #include "pqPluginManager.h"
@@ -69,6 +78,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqServerManagerModel.h"
 #include "pqSplitViewUndoElement.h"
 #include "pqUndoStack.h"
+#include "pqViewFrameActionGroupInterface.h"
 #include "pqViewModuleInterface.h"
 #include "pqXMLUtil.h"
 
@@ -91,6 +101,7 @@ public:
   QPointer<pqServer> ActiveServer;
   QPointer<pqView> ActiveView;
   QPointer<pqUndoStack> UndoStack;
+  QPointer<pqActiveViewOptionsManager> ViewOptionsManager;
   QMenu ConvertMenu;
   QSignalMapper* LookmarkSignalMapper;
 
@@ -112,6 +123,10 @@ public:
   // * unregister view
   // * close frame
   vtkSmartPointer<vtkSMUndoElement> CloseFrameUndoElement;
+
+  // Used by prepareForCapture and finishedCapture.
+  QSize SavedMaxSize;
+  QSize SavedSize;
 };
 
 //-----------------------------------------------------------------------------
@@ -176,6 +191,12 @@ pqViewManager::~pqViewManager()
   delete this->Internal;
 }
 
+//-----------------------------------------------------------------------------
+void pqViewManager::setViewOptionsManager(pqActiveViewOptionsManager* mgr)
+{
+  this->Internal->ViewOptionsManager = mgr;
+}
+
 //-----------------------------------------------------------------------------
 void pqViewManager::buildConvertMenu()
 {
@@ -319,11 +340,22 @@ void pqViewManager::onFrameAdded(pqMultiViewFrame* frame)
 
   // Setup the UI shown when no view is present in the frame.
   QWidget* emptyFrame = frame->emptyMainWidget();
+
+  QScrollArea* scrollArea = new QScrollArea(emptyFrame);
+  scrollArea->setFrameShape(QFrame::NoFrame);
+  scrollArea->setWidgetResizable(true);
+  QFrame* frame2 = new QFrame(scrollArea);
+  scrollArea->setWidget(frame2);
   Ui::EmptyView ui;
-  ui.setupUi(emptyFrame);
+  ui.setupUi(frame2);
+
+  QVBoxLayout* vlayout = new QVBoxLayout(emptyFrame);
+  vlayout->addWidget(scrollArea);
 
   this->buildConvertMenu();
 
+  ui.ConvertActionsFrame->layout()->setSpacing(0);
+
   // Add buttons for all conversion actions.
   QList<QAction*> convertActions = 
     this->Internal->ConvertMenu.actions();
@@ -423,6 +455,17 @@ void pqViewManager::onPreFrameRemoved(pqMultiViewFrame* frame)
   state->Delete();
 }
 
+//-----------------------------------------------------------------------------
+void pqViewManager::reset(QList<QWidget*> &removed)
+{
+  foreach (pqMultiViewFrame* frame, this->Internal->PendingFrames)
+    {
+    this->onFrameRemoved(frame);
+    }
+
+  this->Superclass::reset(removed);
+}
+
 //-----------------------------------------------------------------------------
 void pqViewManager::connect(pqMultiViewFrame* frame, pqView* view)
 {
@@ -444,9 +487,34 @@ void pqViewManager::connect(pqMultiViewFrame* frame, pqView* view)
     frame->setMainWidget(NULL);
     }
 
+  // Search for view frame action group plugins and allow them to decide whether
+  // to add their actions to this view type's frame or not.
+  QObjectList ifaces =
+    pqApplicationCore::instance()->getPluginManager()->interfaces();
+  foreach(QObject* iface, ifaces)
+    {
+    pqViewFrameActionGroupInterface* agi = qobject_cast<pqViewFrameActionGroupInterface*>(iface);
+    if(agi)
+      {
+      agi->connect(frame, view);
+      }
+    }
+
   pqRenderView* const render_module = 
     qobject_cast<pqRenderView*>(view);
   if(render_module)
+    {
+    QAction* cameraAction = new QAction(QIcon(":/pqWidgets/Icons/pqEditCamera16.png"), 
+      "Adjust Camera", 
+      this);
+    cameraAction->setObjectName("CameraButton");
+    frame->addTitlebarAction(cameraAction);
+    cameraAction->setEnabled(true);
+    QObject::connect(cameraAction, SIGNAL(triggered()), 
+      this, SLOT(onCameraTriggered()));
+    }
+
+  if(view->supportsLookmarks())
     {
     QAction* lookmarkAction = new QAction(QIcon(":/pqWidgets/Icons/pqLookmark16.png"), 
       "Lookmark", 
@@ -458,24 +526,20 @@ void pqViewManager::connect(pqMultiViewFrame* frame, pqView* view)
     this->Internal->LookmarkSignalMapper->setMapping(lookmarkAction, frame);
     QObject::connect(lookmarkAction, SIGNAL(triggered(bool)), 
       this->Internal->LookmarkSignalMapper, SLOT(map()));
-
-    QAction* cameraAction = new QAction(QIcon(":/pqWidgets/Icons/pqEditCamera16.png"), 
-      "Adjust Camera", 
-      this);
-    cameraAction->setObjectName("CameraButton");
-    frame->addTitlebarAction(cameraAction);
-    cameraAction->setEnabled(true);
-    QObject::connect(cameraAction, SIGNAL(triggered()), 
-      this, SLOT(onCameraTriggered()));
     }
 
   QAction* optionsAction = new QAction(
     QIcon(":/pqWidgets/Icons/pqOptions16.png"), "Edit View Options", this);
   optionsAction->setObjectName("OptionsButton");
-  optionsAction->setEnabled(true);
+  optionsAction->setEnabled(false);
+  if (this->Internal->ViewOptionsManager && 
+    this->Internal->ViewOptionsManager->canShowOptions(view))
+    {
+    optionsAction->setEnabled(true);
+    }
   frame->addTitlebarAction(optionsAction);
   QObject::connect(optionsAction, SIGNAL(triggered()), 
-    this, SIGNAL(viewOptionsRequested()));
+    this, SLOT(onViewOptionsRequested()));
 
   if (view->supportsUndo())
     {
@@ -507,8 +571,6 @@ void pqViewManager::connect(pqMultiViewFrame* frame, pqView* view)
       backAction, SLOT(setEnabled(bool)));
     }
 
-
-
   this->Internal->Frames.insert(frame, view);
 }
 
@@ -535,12 +597,6 @@ void pqViewManager::disconnect(pqMultiViewFrame* frame, pqView* view)
     qobject_cast<pqRenderView*>(view);
   if(render_module)
     {
-    QAction *lookmarkAction= frame->getAction("LookmarkButton");
-    if(lookmarkAction)
-      {
-      frame->removeTitlebarAction(lookmarkAction);
-      delete lookmarkAction;
-      }
     QAction *cameraAction= frame->getAction("CameraButton");
     if(cameraAction)
       {
@@ -549,6 +605,16 @@ void pqViewManager::disconnect(pqMultiViewFrame* frame, pqView* view)
       }
     }
 
+  if(view->supportsLookmarks())
+    {
+    QAction *lookmarkAction= frame->getAction("LookmarkButton");
+    if(lookmarkAction)
+      {
+      frame->removeTitlebarAction(lookmarkAction);
+      delete lookmarkAction;
+      }
+    }
+
   QAction *optionsAction= frame->getAction("OptionsButton");
   if(optionsAction)
     {
@@ -574,6 +640,20 @@ void pqViewManager::disconnect(pqMultiViewFrame* frame, pqView* view)
       }
     }
 
+  // Search for view frame action group plugins and have them remove their 
+  // actions for this view's frame if need be.
+  QObjectList ifaces =
+    pqApplicationCore::instance()->getPluginManager()->interfaces();
+  foreach(QObject* iface, ifaces)
+    {
+    pqViewFrameActionGroupInterface* agi = 
+        qobject_cast<pqViewFrameActionGroupInterface*>(iface);
+    if(agi)
+      {
+      agi->disconnect(frame, view);
+      }
+    }
+
   this->Internal->PendingFrames.push_back(frame);
 }
 
@@ -881,9 +961,12 @@ void pqViewManager::updateViewPositions()
   
   foreach(pqView* view, this->Internal->Frames)
     {
-    QRect bounds = view->getWidget()->rect();
-    bounds.moveTo(view->getWidget()->mapToGlobal(QPoint(0,0)));
-    totalBounds |= bounds;
+    if (view->getWidget()->isVisible())
+      {
+      QRect bounds = view->getWidget()->rect();
+      bounds.moveTo(view->getWidget()->mapToGlobal(QPoint(0,0)));
+      totalBounds |= bounds;
+      }
     }
 
   /// GUISize, ViewSize and ViewPosition properties are managed
@@ -894,6 +977,10 @@ void pqViewManager::updateViewPositions()
   // Now we loop thorough all view modules and set the GUISize/ViewPosition.
   foreach(pqView* view, this->Internal->Frames)
     {
+    if (!view->getWidget()->isVisible())
+      {
+      continue;
+      }
     vtkSMIntVectorProperty* prop = 0;
 
     // set size containing all views
@@ -1184,7 +1271,102 @@ void pqViewManager::onCameraTriggered()
   emit this->triggerCameraAdjustment(this->Internal->ActiveView);
 }
 
+#define PADDING_COMPENSATION QSize(16, 16);
+//-----------------------------------------------------------------------------
+int pqViewManager::prepareForCapture(const QSize& fullsize)
+{
+  this->hideDecorations();
+  // update GUI
+  // FIXME: It would be nice if we could block renders right now, there's not need to
+  // render the views, we just need to update the GUI.
+  pqEventDispatcher::processEventsAndWait(1);
 
+  // This method cannot set the size of the widget to be greater than the
+  // current size, irrespective of the image size the user requested.
+  QSize currentSize = this->clientSize();
+  // to avoid some unpredicable padding issues, I am reducing the size by a few
+  // pixels.
+  currentSize -= PADDING_COMPENSATION;
 
+  int magnification = pqView::computeMagnification(fullsize, currentSize);
 
+  QSize oldSize = this->size();
+  QSize oldMaxSize = this->maximumSize();
 
+  // currentSize is the client area size for the view widget which may be
+  // smaller than actual size of the widget. The difference is padding.
+  // pqMultiView::computeSize() returns the widget size required to achieve
+  // the client size requested.
+  currentSize = this->computeSize(currentSize);
+  this->setMaximumSize(currentSize);
+  this->resize(currentSize);
+  pqEventDispatcher::processEventsAndWait(1);
+
+  this->Internal->SavedSize = oldSize;
+  this->Internal->SavedMaxSize = oldMaxSize;
+  return magnification;
+}
+
+//-----------------------------------------------------------------------------
+void pqViewManager::finishedCapture()
+{
+  this->setMaximumSize(this->Internal->SavedMaxSize);
+  this->resize(this->Internal->SavedSize);
+  this->showDecorations();
+}
+
+//-----------------------------------------------------------------------------
+vtkImageData* pqViewManager::captureImage(int _width, int _height)
+{
+  int magnification = this->prepareForCapture(QSize(_width, _height));
+
+  // Create full image data.
+  vtkImageData* fullImage = vtkImageData::New();
+  fullImage->SetDimensions(_width, _height, 1);
+  fullImage->SetScalarTypeToUnsignedChar();
+  fullImage->SetNumberOfScalarComponents(3);
+  fullImage->AllocateScalars();
+  unsigned char rgb[3] = {0, 0, 0};
+  vtkImageIterator<unsigned char> it(fullImage, fullImage->GetExtent());
+  while (!it.IsAtEnd())
+    {
+    unsigned char* span = it.BeginSpan();
+    unsigned char* spanEnd = it.EndSpan();
+    while (spanEnd != span)
+      {
+      *span = rgb[0]; span++;
+      *span = rgb[1]; span++;
+      *span = rgb[2]; span++;
+      }
+    it.NextSpan();
+    }
+
+  // Now iterate over each of the views and merge the rendered image into the
+  // final fullImage.
+  // TODO: handle the case when one view is maximized (generally speaking handle
+  // case when views are not visible).
+  foreach (pqView* view, this->Internal->Frames)
+    {
+    if (view)
+      {
+      vtkImageData* image = view->captureImage(magnification);
+      if (image)
+        {
+        vtkSMAnimationSceneImageWriter::Merge(fullImage, image);
+        image->Delete();
+        }
+      }
+    }
+
+  this->finishedCapture();
+  return fullImage;
+}
+
+//-----------------------------------------------------------------------------
+void pqViewManager::onViewOptionsRequested()
+{
+  if (this->Internal->ViewOptionsManager)
+    {
+    this->Internal->ViewOptionsManager->showOptions();
+    }
+}
