@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqXYPlotDisplayProxyEditor.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -52,11 +52,11 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqComboBoxDomain.h"
 #include "pqLineChartRepresentation.h"
 #include "pqPropertyLinks.h"
+#include "pqSignalAdaptorCompositeTreeWidget.h"
 #include "pqSignalAdaptors.h"
 #include "pqSMAdaptor.h"
-#include "pqTreeWidgetCheckHelper.h"
 #include "pqTreeWidgetItemObject.h"
-
+#include "pqView.h"
 
 //-----------------------------------------------------------------------------
 class pqLineSeriesEditorModelItem
@@ -207,6 +207,7 @@ public:
     this->AttributeModeAdaptor = 0;
     this->Model = 0;
     this->InChange = false;
+    this->CompositeIndexAdaptor = 0;
     }
 
   ~pqInternal()
@@ -215,6 +216,7 @@ public:
     delete this->XAxisComponentAdaptor;
     delete this->XAxisArrayDomain;
     delete this->AttributeModeAdaptor;
+    delete this->CompositeIndexAdaptor;
     }
 
   pqPropertyLinks Links;
@@ -222,6 +224,7 @@ public:
   pqSignalAdaptorComboBox* XAxisComponentAdaptor;
   pqSignalAdaptorComboBox* AttributeModeAdaptor;
   pqComboBoxDomain* XAxisArrayDomain;
+  pqSignalAdaptorCompositeTreeWidget* CompositeIndexAdaptor;
 
   QPointer<pqLineChartRepresentation> Display;
   pqLineSeriesEditorModel *Model;
@@ -254,7 +257,7 @@ pqLineSeriesEditorModel::pqLineSeriesEditorModel(
   : pqCheckableHeaderModel(parentObject), Items(), Display(display)
 {
   // Set up the column headers.
-  this->insertHeaderSections(Qt::Horizontal, 0, 2);
+  this->insertHeaderSections(Qt::Horizontal, 0, 1);
   this->setCheckable(0, Qt::Horizontal, true);
   this->setCheckState(0, Qt::Horizontal, Qt::Unchecked);
 
@@ -280,7 +283,7 @@ int pqLineSeriesEditorModel::rowCount(const QModelIndex &parentIndex) const
 
 int pqLineSeriesEditorModel::columnCount(const QModelIndex &) const
 {
-  return 3;
+  return 2;
 }
 
 bool pqLineSeriesEditorModel::hasChildren(const QModelIndex &parentIndex) const
@@ -291,7 +294,7 @@ bool pqLineSeriesEditorModel::hasChildren(const QModelIndex &parentIndex) const
 QModelIndex pqLineSeriesEditorModel::index(int row, int column,
     const QModelIndex &parentIndex) const
 {
-  if(!parentIndex.isValid() && column >= 0 && column < 3 && row >= 0 &&
+  if(!parentIndex.isValid() && column >= 0 && column < 2 && row >= 0 &&
       row < this->Items.size())
     {
     return this->createIndex(row, column);
@@ -315,48 +318,15 @@ QVariant pqLineSeriesEditorModel::data(const QModelIndex &idx, int role) const
       {
       if(idx.column() == 0)
         {
-        return QVariant(item->Variable);
+        QString arrayName = item->Variable;
+        this->Display->addComponentLabel(arrayName, item->Component,
+            item->ComponentCount);
+        return QVariant(arrayName);
         }
       else if(idx.column() == 1)
         {
         return QVariant(item->LegendName);
         }
-      else if(idx.column() == 2)
-        {
-        int component = item->Component;
-        if(role == Qt::DisplayRole && item->ComponentCount > 1)
-          {
-          if(component == -2)
-            {
-            return QVariant(QString("Distance"));
-            }
-          else if(component == -1)
-            {
-            return QVariant(QString("Magnitude"));
-            }
-          else if(item->ComponentCount == 3)
-            {
-            if(item->Component == 0)
-              {
-              return QVariant(QString("X"));
-              }
-            else if(item->Component == 1)
-              {
-              return QVariant(QString("Y"));
-              }
-            else if(item->Component == 2)
-              {
-              return QVariant(QString("Z"));
-              }
-            }
-
-          return QVariant(QString::number(component));
-          }
-        else if(role == Qt::EditRole)
-          {
-          return QVariant(component);
-          }
-        }
       }
     else if(role == Qt::CheckStateRole)
       {
@@ -372,16 +342,6 @@ QVariant pqLineSeriesEditorModel::data(const QModelIndex &idx, int role) const
         return QVariant(item->LineColor);
         }
       }
-    else if(role == Qt::UserRole)
-      {
-      if(idx.column() == 2 && item->ComponentCount > 1)
-        {
-        QList<QVariant> range;
-        range.append(QVariant(-1));
-        range.append(QVariant(item->ComponentCount));
-        return QVariant(range);
-        }
-      }
     }
 
   return QVariant();
@@ -400,10 +360,6 @@ Qt::ItemFlags pqLineSeriesEditorModel::flags(const QModelIndex &idx) const
       {
       result |= Qt::ItemIsEditable;
       }
-    else if(idx.column() == 2 && this->Items[idx.row()]->ComponentCount > 1)
-      {
-      result |= Qt::ItemIsEditable;
-      }
     }
 
   return result;
@@ -425,8 +381,7 @@ bool pqLineSeriesEditorModel::setData(const QModelIndex &idx,
         if(name != item->LegendName)
           {
           item->LegendName = name;
-          int series = this->Display->getSeriesIndex(item->Variable);
-          this->Display->setSeriesLabel(series, item->LegendName);
+          this->Display->setSeriesLabel(idx.row(), item->LegendName);
           this->Display->renderViewEventually();
           emit this->dataChanged(idx, idx);
           }
@@ -451,26 +406,12 @@ bool pqLineSeriesEditorModel::setData(const QModelIndex &idx,
 
       if(result && changed)
         {
-        int series = this->Display->getSeriesIndex(item->Variable);
-        this->Display->setSeriesEnabled(series, item->Enabled);
+        this->Display->setSeriesEnabled(idx.row(), item->Enabled);
         this->Display->renderViewEventually();
         emit this->dataChanged(idx, idx);
         this->updateCheckState(0, Qt::Horizontal);
         }
       }
-    else if(idx.column() == 2 && role == Qt::EditRole)
-      {
-      result = true;
-      int component = value.toInt();
-      if(component != item->Component)
-        {
-        item->Component = component;
-        int series = this->Display->getSeriesIndex(item->Variable);
-        this->Display->setSeriesComponent(series, component);
-        this->Display->renderViewEventually();
-        emit this->dataChanged(idx, idx);
-        }
-      }
     }
 
   return result;
@@ -489,10 +430,6 @@ QVariant pqLineSeriesEditorModel::headerData(int section,
       {
       return QVariant(QString("Legend Name"));
       }
-    else if(section == 2)
-      {
-      return QVariant(QString("Component"));
-      }
     }
   else
     {
@@ -706,7 +643,6 @@ pqXYPlotDisplayProxyEditor::pqXYPlotDisplayProxyEditor(pqRepresentation* display
       new pqLineSeriesEditorDelegate(this));
   this->Internal->Model = new pqLineSeriesEditorModel(0, this);
   this->Internal->SeriesList->setModel(this->Internal->Model);
-  //this->Internal->SeriesList->header()->setResizeMode(QHeaderView::Stretch);
 
   QObject::connect(
     this->Internal->SeriesList, SIGNAL(activated(const QModelIndex &)),
@@ -792,6 +728,9 @@ void pqXYPlotDisplayProxyEditor::setDisplay(pqRepresentation* disp)
   this->Internal->Model->reloadSeries();
   delete this->Internal->XAxisArrayDomain;
   this->Internal->XAxisArrayDomain = 0;
+  delete this->Internal->CompositeIndexAdaptor;
+  this->Internal->CompositeIndexAdaptor = 0;
+
   if(this->Internal->Display)
     {
     this->disconnect(this->Internal->Display, 0, this, 0);
@@ -839,6 +778,39 @@ void pqXYPlotDisplayProxyEditor::setDisplay(pqRepresentation* disp)
       this->Internal->Display, SIGNAL(styleChanged(int, Qt::PenStyle)),
       this, SLOT(updateItemStyle(int, Qt::PenStyle)));
 
+  // Set up the CompositeIndexAdaptor 
+  this->Internal->CompositeIndexAdaptor = new pqSignalAdaptorCompositeTreeWidget(
+    this->Internal->CompositeIndex, 
+    vtkSMIntVectorProperty::SafeDownCast(
+      proxy->GetProperty("CompositeDataSetIndex")),
+    /*autoUpdateVisibility=*/true);
+
+  this->Internal->Links.addPropertyLink(this->Internal->CompositeIndexAdaptor,
+    "values", SIGNAL(valuesChanged()),
+    proxy, proxy->GetProperty("CompositeDataSetIndex"));
+  QObject::connect(this->Internal->CompositeIndexAdaptor, SIGNAL(valuesChanged()),
+    this, SLOT(updateAllViews()), Qt::QueuedConnection);
+
+  // Although plot type is a view property, I am showing it here for sake of
+  // simplicity. It's going to change soon anyways.
+  vtkSMProxy* viewProxy = display->getView()->getProxy();
+  if (viewProxy)
+    {
+    new pqComboBoxDomain(
+      this->Internal->PlotType, viewProxy->GetProperty("Type"));
+
+    pqSignalAdaptorComboBox* adaptor = new pqSignalAdaptorComboBox(this->Internal->PlotType);
+    this->Internal->Links.addPropertyLink(adaptor,
+      "currentText", SIGNAL(currentTextChanged(const QString&)),
+      viewProxy, viewProxy->GetProperty("Type"));
+    QObject::connect(adaptor, SIGNAL(currentTextChanged(const QString&)),
+      this, SLOT(updateAllViews()), Qt::QueuedConnection);
+    }
+  else
+    {
+    this->Internal->PlotType->setEnabled(false);
+    }
+
   this->reloadSeries();
 }
 
