@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqFileDialogModel.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -34,10 +34,10 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <vtkstd/algorithm>
 
-#include <QFileIconProvider>
 #include <QStyle>
 #include <QDir>
 #include <QApplication>
+#include <QMessageBox>
 
 #include <pqServer.h>
 #include <vtkClientServerStream.h>
@@ -52,6 +52,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <vtkSMProxyManager.h>
 #include <vtkSMStringVectorProperty.h>
 #include <vtkStringList.h>
+#include <vtkDirectory.h>
 
 #include "pqSMAdaptor.h"
 
@@ -66,19 +67,13 @@ public:
   }
 
   pqFileDialogModelFileInfo(const QString& l, const QString& filepath, 
-           const bool isdir, const QList<pqFileDialogModelFileInfo>& g =
+           vtkPVFileInformation::FileTypes t, const QList<pqFileDialogModelFileInfo>& g =
            QList<pqFileDialogModelFileInfo>()) :
     Label(l),
     FilePath(filepath),
-    IsDir(isdir),
-    IsLink(false),
+    Type(t),
     Group(g)
   {
-  if(this->Label.endsWith(".lnk"))
-    {
-    this->IsLink = true;
-    this->Label.chop(4);
-    }
   }
 
   const QString& label() const
@@ -91,16 +86,11 @@ public:
     return this->FilePath;
   }
   
-  bool isDir() const
+  vtkPVFileInformation::FileTypes type() const
   {
-    return this->IsDir;
+    return this->Type;
   }
   
-  bool isLink() const
-  {
-    return this->IsLink;
-  }
-
   bool isGroup() const
   {
     return !this->Group.empty();
@@ -114,62 +104,88 @@ public:
 private:
   QString Label;
   QString FilePath;
-  bool IsDir;
-  bool IsLink;
+  vtkPVFileInformation::FileTypes Type;
   QList<pqFileDialogModelFileInfo> Group;
 };
 
 /////////////////////////////////////////////////////////////////////
 // Icons
 
-class pqFileDialogModelIconProvider : protected QFileIconProvider
+pqFileDialogModelIconProvider::pqFileDialogModelIconProvider()
 {
-public:
-  enum IconType { Drive, Folder, File, FolderLink, FileLink };
-  pqFileDialogModelIconProvider()
-    {
-    QStyle* style = QApplication::style();
-    this->FolderLinkIcon = style->standardIcon(QStyle::SP_DirLinkIcon);
-    this->FileLinkIcon = style->standardIcon(QStyle::SP_FileLinkIcon);
-    }
+  QStyle* style = QApplication::style();
+  this->FolderLinkIcon = style->standardIcon(QStyle::SP_DirLinkIcon);
+  this->FileLinkIcon = style->standardIcon(QStyle::SP_FileLinkIcon);
+  this->DomainIcon.addPixmap(QPixmap(":/pqCore/Icons/pqDomain16.png"));
+  this->NetworkIcon.addPixmap(QPixmap(":/pqCore/Icons/pqNetwork16.png"));
+}
 
-  QIcon icon(IconType t) const
+QIcon pqFileDialogModelIconProvider::icon(IconType t) const
+{
+  switch(t)
     {
-    switch(t)
-      {
-      case Drive:
-        return QFileIconProvider::icon(QFileIconProvider::Drive);
-      case Folder:
-        return QFileIconProvider::icon(QFileIconProvider::Folder);
-      case File:
-        return QFileIconProvider::icon(QFileIconProvider::File);
-      case FolderLink:
-        return this->FolderLinkIcon;
-      case FileLink:
-        return this->FileLinkIcon;
-      }
-    return QIcon();
+    case Computer:
+      return QFileIconProvider::icon(QFileIconProvider::Computer);
+    case Drive:
+      return QFileIconProvider::icon(QFileIconProvider::Drive);
+    case Folder:
+      return QFileIconProvider::icon(QFileIconProvider::Folder);
+    case File:
+      return QFileIconProvider::icon(QFileIconProvider::File);
+    case FolderLink:
+      return this->FolderLinkIcon;
+    case FileLink:
+      return this->FileLinkIcon;
+    case NetworkFolder:
+      return QFileIconProvider::icon(QFileIconProvider::Network);
+    case NetworkRoot:
+      return this->NetworkIcon;
+    case NetworkDomain:
+      return this->DomainIcon;
     }
+  return QIcon();
+}
 
-  QIcon icon(const pqFileDialogModelFileInfo& info) const
+QIcon pqFileDialogModelIconProvider::icon(vtkPVFileInformation::FileTypes f) const
+{
+  if(f == vtkPVFileInformation::DIRECTORY_LINK)
     {
-    if(info.isDir())
-      return icon(info.isLink() ? FolderLink : Folder);
-    return icon(info.isLink() ? FileLink : File);
+    return icon(pqFileDialogModelIconProvider::FolderLink);
     }
-  QIcon icon(const QFileInfo& info) const
+  else if(f == vtkPVFileInformation::SINGLE_FILE_LINK)
     {
-    return QFileIconProvider::icon(info);
+    return icon(pqFileDialogModelIconProvider::FileLink);
     }
-  QIcon icon(QFileIconProvider::IconType ico) const
+  else if(f == vtkPVFileInformation::NETWORK_SHARE)
     {
-    return QFileIconProvider::icon(ico);
+    return icon(pqFileDialogModelIconProvider::NetworkFolder);
     }
-
-protected:
-  QIcon FolderLinkIcon;
-  QIcon FileLinkIcon;
-};
+  else if(f == vtkPVFileInformation::NETWORK_SERVER)
+    {
+    return icon(pqFileDialogModelIconProvider::Computer);
+    }
+  else if(f == vtkPVFileInformation::NETWORK_DOMAIN)
+    {
+    return icon(pqFileDialogModelIconProvider::NetworkDomain);
+    }
+  else if(f == vtkPVFileInformation::NETWORK_ROOT)
+    {
+    return icon(pqFileDialogModelIconProvider::NetworkRoot);
+    }
+  else if(vtkPVFileInformation::IsDirectory(f))
+    {
+    return icon(pqFileDialogModelIconProvider::Folder);
+    }
+  return icon(pqFileDialogModelIconProvider::File);
+}
+QIcon pqFileDialogModelIconProvider::icon(const QFileInfo& info) const
+{
+  return QFileIconProvider::icon(info);
+}
+QIcon pqFileDialogModelIconProvider::icon(QFileIconProvider::IconType ico) const
+{
+  return QFileIconProvider::icon(ico);
+}
 
 Q_GLOBAL_STATIC(pqFileDialogModelIconProvider, Icons);
 
@@ -245,7 +261,7 @@ public:
     this->FileInformation.TakeReference(vtkPVFileInformation::New());
 
     // current path
-    vtkPVFileInformation* info = this->GetData(false, ".", false);
+    vtkPVFileInformation* info = this->GetData(false, "", ".", false);
     this->CurrentPath = info->GetFullPath();
   }
   
@@ -257,8 +273,7 @@ public:
   /// and points slashes in the correct direction for the server
   const QString cleanPath(const QString& Path)
   {
-    QString result = QDir::cleanPath(Path);
-    result.replace('/', this->Separator);
+    QString result = QDir::cleanPath(QDir::fromNativeSeparators(Path));
     return result.trimmed();
   }
 
@@ -267,7 +282,7 @@ public:
                                 const QString& path,
                                 bool specialDirs)
     {
-    return this->GetData(dirListing, "", path, specialDirs);
+    return this->GetData(dirListing, this->CurrentPath, path, specialDirs);
     }
 
   /// query the file system for information
@@ -332,13 +347,15 @@ public:
         {
         continue;
         }
-      if (info->GetType() == vtkPVFileInformation::DIRECTORY)
+      if (vtkPVFileInformation::IsDirectory(info->GetType()))
         {
-        dirs.push_back(pqFileDialogModelFileInfo(info->GetName(), info->GetFullPath(), true));
+        dirs.push_back(pqFileDialogModelFileInfo(info->GetName(), info->GetFullPath(), 
+            static_cast<vtkPVFileInformation::FileTypes>(info->GetType())));
         }
-      else if (info->GetType() == vtkPVFileInformation::SINGLE_FILE)
+      else if (info->GetType() != vtkPVFileInformation::FILE_GROUP)
         {
-        files.push_back(pqFileDialogModelFileInfo(info->GetName(), info->GetFullPath(), false));
+        files.push_back(pqFileDialogModelFileInfo(info->GetName(), info->GetFullPath(),
+            static_cast<vtkPVFileInformation::FileTypes>(info->GetType())));
         }
       else if (info->GetType() == vtkPVFileInformation::FILE_GROUP)
         {
@@ -351,10 +368,10 @@ public:
           vtkPVFileInformation* child = vtkPVFileInformation::SafeDownCast(
             childIter->GetCurrentObject());
           groupFiles.push_back(pqFileDialogModelFileInfo(child->GetName(), child->GetFullPath(),
-                                   false));
+            static_cast<vtkPVFileInformation::FileTypes>(child->GetType())));
           }
-        files.push_back(pqFileDialogModelFileInfo(info->GetName(), groupFiles[0].filePath(), false,
-            groupFiles));
+        files.push_back(pqFileDialogModelFileInfo(info->GetName(), groupFiles[0].filePath(),
+          vtkPVFileInformation::SINGLE_FILE, groupFiles));
         }
       }
 
@@ -413,7 +430,7 @@ public:
       return false;
 
     pqFileDialogModelFileInfo& file = this->FileList[Index.row()];
-    return file.isDir();
+    return vtkPVFileInformation::IsDirectory(file.type());
     }
 
   bool isRemote()
@@ -434,6 +451,27 @@ public:
   /// Caches information about the set of files within the current path.
   QVector<pqFileDialogModelFileInfo> FileList;  // adjacent memory occupation for QModelIndex
 
+  const pqFileDialogModelFileInfo* infoForIndex(const QModelIndex& idx) const
+    {
+    if(idx.isValid() && 
+       NULL == idx.internalPointer() &&
+       idx.row() >= 0 && 
+       idx.row() < this->FileList.size())
+      {
+      return &this->FileList[idx.row()];
+      }
+    else if(idx.isValid() && idx.internalPointer())
+      {
+      pqFileDialogModelFileInfo* ptr = reinterpret_cast<pqFileDialogModelFileInfo*>(idx.internalPointer());
+      const QList<pqFileDialogModelFileInfo>& grp = ptr->group();
+      if(idx.row() >= 0 && idx.row() < grp.size())
+        {
+        return &grp[idx.row()];
+        }
+      }
+    return NULL;
+    }
+
 private:   
   // server vs. local implementation private
   pqServer* Server;
@@ -483,8 +521,7 @@ QString pqFileDialogModel::absoluteFilePath(const QString& path)
     }
 
   vtkPVFileInformation* info;
-  info = this->Implementation->GetData(false,
-    this->getCurrentPath(), path, false);
+  info = this->Implementation->GetData(false, path, false);
   return this->Implementation->cleanPath(info->GetFullPath());
 }
 
@@ -525,7 +562,7 @@ bool pqFileDialogModel::fileExists(const QString& file, QString& fullpath)
   return false;
 }
 
-bool pqFileDialogModel::makeDirEntry(const QString& dirName)
+bool pqFileDialogModel::mkdir(const QString& dirName)
 {
   QString path;
   QString dirPath = this->absoluteFilePath(dirName);
@@ -534,65 +571,43 @@ bool pqFileDialogModel::makeDirEntry(const QString& dirName)
     return false;
     }
 
-  this->Implementation->FileList.push_back(pqFileDialogModelFileInfo(dirName, dirPath, true));
-  this->reset();
+  bool ret = false;
 
-  return true;
-}
-
-bool pqFileDialogModel::removeDirEntry(const QString& dirName)
-{
-  QString dirPath = this->absoluteFilePath(dirName);
-  int i=0;
-  for(i=0; i<this->Implementation->FileList.size(); i++)
+  if (this->Implementation->isRemote())
     {
-    if(this->Implementation->FileList[i].isDir() && this->Implementation->FileList[i].filePath()==dirPath)
-      {
-      break;
-      }
-    }
-  
-  if(i<this->Implementation->FileList.size())
-    {
-    this->Implementation->FileList.remove(i);
-    this->reset();
-    }
+    // File system is on remote server.
+    vtkIdType conn = this->Implementation->getServer()->GetConnectionID();
+    vtkTypeUInt32 servers = this->Implementation->isRemote() ? 
+                              vtkProcessModule::DATA_SERVER :
+                              vtkProcessModule::CLIENT;
 
-  return true;
-}
-
-bool pqFileDialogModel::makeDir(const QString& dirName)
-{
-  QString path;
-  QString dirPath = this->absoluteFilePath(dirName);
-  if(this->dirExists(dirPath,path))
-    {
-    return false;
-    }
-
-  if(this->Implementation->getServer())
-    {
     vtkProcessModule *pm = vtkProcessModule::GetProcessModule();
     vtkClientServerStream stream;
     vtkClientServerID dirID = pm->NewStreamObject("vtkDirectory", stream);
     stream << vtkClientServerStream::Invoke
-          << dirID << "MakeDirectory" << dirPath.toAscii().data()
-          << vtkClientServerStream::End;
-    pm->SendStream(
-      this->Implementation->getServer()->GetConnectionID(),
-      vtkProcessModule::DATA_SERVER, stream);
+           << dirID << "MakeDirectory" 
+           << dirPath.toAscii().data()
+           << vtkClientServerStream::End;
+    pm->SendStream(conn, servers, stream);
+
+    vtkClientServerStream result = pm->GetLastResult(conn, servers);
+    if(result.GetNumberOfMessages() == 1 && 
+       result.GetNumberOfArguments(0) == 1)
+      {
+      int tmp;
+      if(result.GetArgument(0, 0, &tmp) && tmp)
+        {
+        ret = true;
+        }
+      }
+    
     pm->DeleteStreamObject(dirID, stream);
-    pm->SendStream(
-      this->Implementation->getServer()->GetConnectionID(),
-      vtkProcessModule::DATA_SERVER, stream);
+    pm->SendStream(conn, servers, stream);
     }
   else
     {
-    QDir dir(this->getCurrentPath());
-    if(!dir.mkdir(dirName))
-      {
-      return false;
-      }
+    // File system is local.
+    ret = (vtkDirectory::MakeDirectory(dirPath.toAscii().data()) != 0);
     }
 
   QString cPath = this->Implementation->cleanPath(this->getCurrentPath());
@@ -602,7 +617,143 @@ bool pqFileDialogModel::makeDir(const QString& dirName)
 
   this->reset();
 
-  return true;
+  return ret;
+}
+
+bool pqFileDialogModel::rmdir(const QString& dirName)
+{
+  QString path;
+  QString dirPath = this->absoluteFilePath(dirName);
+  if(!this->dirExists(dirPath,path))
+    {
+    return false;
+    }
+  
+  bool ret = false;
+  
+  if (this->Implementation->isRemote())
+    {
+    vtkIdType conn = this->Implementation->getServer()->GetConnectionID();
+    vtkTypeUInt32 servers = this->Implementation->isRemote() ? 
+                           vtkProcessModule::DATA_SERVER :
+                           vtkProcessModule::CLIENT;
+
+    vtkProcessModule *pm = vtkProcessModule::GetProcessModule();
+    vtkClientServerStream stream;
+    vtkClientServerID dirID = pm->NewStreamObject("vtkDirectory", stream);
+    stream << vtkClientServerStream::Invoke
+          << dirID << "DeleteDirectory" 
+          << dirPath.toAscii().data()
+          << vtkClientServerStream::End;
+    pm->SendStream(conn, servers, stream);
+    
+    vtkClientServerStream result = pm->GetLastResult(conn, servers);
+    if(result.GetNumberOfMessages() == 1 && 
+       result.GetNumberOfArguments(0) == 1)
+      {
+      int tmp;
+      if(result.GetArgument(0, 0, &tmp) && tmp)
+        {
+        ret = true;
+        }
+      }
+
+    pm->DeleteStreamObject(dirID, stream);
+    pm->SendStream(conn, servers, stream);
+    }
+  else
+    {
+    // File system is local.
+    ret = (vtkDirectory::DeleteDirectory(dirPath.toAscii().data()) != 0);
+    }
+  
+  QString cPath = this->Implementation->cleanPath(this->getCurrentPath());
+  vtkPVFileInformation* info;
+  info = this->Implementation->GetData(true, cPath, false);
+  this->Implementation->Update(cPath, info);
+
+  this->reset();
+  
+  return ret;
+}
+
+bool pqFileDialogModel::rename(const QString& oldname, const QString& newname)
+{
+  QString oldPath = this->absoluteFilePath(oldname);
+  QString newPath = this->absoluteFilePath(newname);
+
+  if(oldPath == newPath)
+    {
+    return true;
+    }
+
+  vtkPVFileInformation* info;
+  info = this->Implementation->GetData(false, oldPath, false);
+
+  int oldType = info->GetType();
+  
+  if(oldType != vtkPVFileInformation::SINGLE_FILE && 
+    !vtkPVFileInformation::IsDirectory(oldType))
+    {
+    return false;
+    }
+  
+  // don't replace file/dir
+  info = this->Implementation->GetData(false, newPath, false);
+  if(info->GetType() == oldType)
+    {
+    QString message("Cannot rename to %1, which already exists");
+    message = message.arg(newname);
+    QMessageBox::warning(NULL, "Error renaming", message);
+    return false;
+    }
+
+  bool ret = false;
+
+  if (this->Implementation->isRemote())
+    {
+    vtkIdType conn = this->Implementation->getServer()->GetConnectionID();
+    vtkTypeUInt32 servers = this->Implementation->isRemote() ? 
+                              vtkProcessModule::DATA_SERVER :
+                              vtkProcessModule::CLIENT;
+
+    vtkProcessModule *pm = vtkProcessModule::GetProcessModule();
+    vtkClientServerStream stream;
+    vtkClientServerID dirID = pm->NewStreamObject("vtkDirectory", stream);
+    stream << vtkClientServerStream::Invoke
+           << dirID << "Rename" 
+           << oldPath.toAscii().data()
+           << newPath.toAscii().data()
+           << vtkClientServerStream::End;
+    pm->SendStream(conn, servers, stream);
+  
+    vtkClientServerStream result = pm->GetLastResult(conn, servers);
+    if(result.GetNumberOfMessages() == 1 && 
+       result.GetNumberOfArguments(0) == 1)
+      {
+      int tmp;
+      if(result.GetArgument(0, 0, &tmp) && tmp)
+        {
+        ret = true;
+        }
+      }
+
+    pm->DeleteStreamObject(dirID, stream);
+    pm->SendStream(conn, servers, stream);
+    }
+  else
+    {
+    ret = (vtkDirectory::Rename(oldPath.toAscii().data(),
+                                newPath.toAscii().data()) != 0);
+    }
+
+  QString cPath = this->Implementation->cleanPath(this->getCurrentPath());
+  info = this->Implementation->GetData(true, cPath, false);
+  this->Implementation->Update(cPath, info);
+    
+  this->reset();
+
+  return ret;
 }
 
 bool pqFileDialogModel::dirExists(const QString& path, QString& fullpath)
@@ -612,14 +763,12 @@ bool pqFileDialogModel::dirExists(const QString& path, QString& fullpath)
   info = this->Implementation->GetData(false, dir, false);
   
   // try again for shortcuts
-  if(info->GetType() != vtkPVFileInformation::DIRECTORY &&
-     info->GetType() != vtkPVFileInformation::DRIVE)
+  if(!vtkPVFileInformation::IsDirectory(info->GetType()))
     {
     info = this->Implementation->GetData(false, dir + ".lnk", false);
     }
 
-  if(info->GetType() == vtkPVFileInformation::DIRECTORY ||
-     info->GetType() == vtkPVFileInformation::DRIVE)
+  if(vtkPVFileInformation::IsDirectory(info->GetType()))
     {
     fullpath = info->GetFullPath();
     return true;
@@ -639,24 +788,8 @@ int pqFileDialogModel::columnCount(const QModelIndex& /*idx*/) const
 
 QVariant pqFileDialogModel::data(const QModelIndex & idx, int role) const
 {
-  const pqFileDialogModelFileInfo* file = NULL;
-
-  if(idx.isValid() && 
-     NULL == idx.internalPointer() &&
-     idx.row() >= 0 && 
-     idx.row() < this->Implementation->FileList.size())
-    {
-    file = &this->Implementation->FileList[idx.row()];
-    }
-  else if(idx.isValid() && idx.internalPointer())
-    {
-    pqFileDialogModelFileInfo* ptr = reinterpret_cast<pqFileDialogModelFileInfo*>(idx.internalPointer());
-    const QList<pqFileDialogModelFileInfo>& grp = ptr->group();
-    if(idx.row() >= 0 && idx.row() < grp.size())
-      {
-      file = &grp[idx.row()];
-      }
-    }
+  const pqFileDialogModelFileInfo* file =
+    this->Implementation->infoForIndex(idx);
 
   if(!file)
     {
@@ -664,13 +797,13 @@ QVariant pqFileDialogModel::data(const QModelIndex & idx, int role) const
     return QVariant();
     }
 
-  if(role == Qt::DisplayRole && idx.column() == 0)
+  if((role == Qt::DisplayRole || role == Qt::EditRole) && idx.column() == 0)
     {
     return file->label();
     }
   else if(role == Qt::DecorationRole && idx.column() == 0)
     {
-    return Icons()->icon(*file);
+    return Icons()->icon(file->type());
     }
 
   return QVariant();
@@ -754,3 +887,35 @@ QVariant pqFileDialogModel::headerData(int section,
   return QVariant();
 }
 
+bool pqFileDialogModel::setData(const QModelIndex& idx, const QVariant& value, int role)
+{
+  if(role != Qt::DisplayRole && role != Qt::EditRole)
+    {
+    return false;
+    }
+
+  const pqFileDialogModelFileInfo* file =
+    this->Implementation->infoForIndex(idx);
+
+  if(!file)
+    {
+    return false;
+    }
+
+  QString name = value.toString();
+  return this->rename(file->filePath(), name);
+}
+  
+Qt::ItemFlags pqFileDialogModel::flags(const QModelIndex& idx) const
+{
+  Qt::ItemFlags ret = Qt::ItemIsSelectable | Qt::ItemIsEnabled;
+  const pqFileDialogModelFileInfo* file =
+    this->Implementation->infoForIndex(idx);
+  if(file && !file->isGroup())
+    {
+    ret |= Qt::ItemIsEditable;
+    }
+  return ret;
+}
+
+
