@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqObjectBuilder.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -33,8 +33,8 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include "vtkProcessModuleConnectionManager.h"
 #include "vtkProcessModule.h"
+#include "vtkPVXMLElement.h"
 #include "vtkSmartPointer.h"
-#include "vtkSMCompoundProxy.h"
 #include "vtkSMDomain.h"
 #include "vtkSMDomainIterator.h"
 #include "vtkSMInputProperty.h"
@@ -50,8 +50,8 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include "pqAnimationScene.h"
 #include "pqApplicationCore.h"
-#include "pqComparativeRenderView.h"
 #include "pqDataRepresentation.h"
+#include "pqDisplayPolicy.h"
 #include "pqNameCount.h"
 #include "pqOutputPort.h"
 #include "pqPipelineFilter.h"
@@ -66,6 +66,16 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqView.h"
 #include "pqViewModuleInterface.h"
 
+inline QString pqObjectBuilderGetName(vtkSMProxy* proxy,
+  pqNameCount *nameGenerator)
+{
+  QString label = 
+    proxy->GetXMLLabel()? proxy->GetXMLLabel() : proxy->GetXMLName();
+  label.remove(' ');
+  return QString("%1%2").arg(label).arg(
+    nameGenerator->GetCountAndIncrement(label));
+}
+
 //-----------------------------------------------------------------------------
 pqObjectBuilder::pqObjectBuilder(QObject* _parent/*=0*/) :QObject(_parent)
 {
@@ -78,7 +88,6 @@ pqObjectBuilder::~pqObjectBuilder()
   delete this->NameGenerator;
 }
 
-
 //-----------------------------------------------------------------------------
 pqPipelineSource* pqObjectBuilder::createSource(const QString& sm_group,
     const QString& sm_name, pqServer* server)
@@ -168,40 +177,13 @@ pqPipelineSource* pqObjectBuilder::createFilter(
   return filter;
 }
 
-//-----------------------------------------------------------------------------
-pqPipelineSource* pqObjectBuilder::createCustomFilter(const QString& sm_name,
-    pqServer* server, pqPipelineSource* input/*=0*/)
+inline QString pqObjectBuilderGetPath(const QString& filename, bool use_dir)
 {
-  vtkSMProxy* proxy = 
-    this->createProxyInternal(QString(), sm_name, server, "sources");
-  if (!proxy)
+  if (use_dir)
     {
-    return 0;
+    return QFileInfo(filename).path();
     }
-
-  pqPipelineSource* filter = pqApplicationCore::instance()->
-    getServerManagerModel()->findItem<pqPipelineSource*>(proxy);
-  if (!filter)
-    {
-    qDebug() << "Failed to locate pqPipelineSource for the created custom filter proxy "
-      << sm_name;
-    return 0;
-    }
-
-  vtkSMProperty* inputProperty = proxy->GetProperty("Input");
-  if (inputProperty && input)
-    {
-    pqSMAdaptor::setProxyProperty(inputProperty, input->getProxy());
-    proxy->UpdateVTKObjects();
-    inputProperty->UpdateDependentDomains();
-    }
-
-  // Set default property values.
-  filter->setDefaultPropertyValues();
-  filter->setModifiedState(pqProxy::UNINITIALIZED);
-  emit this->customFilterCreated(filter);
-  emit this->proxyCreated(filter);
-  return filter;
+  return filename;
 }
 
 //-----------------------------------------------------------------------------
@@ -213,11 +195,31 @@ pqPipelineSource* pqObjectBuilder::createReader(const QString& sm_group,
     return 0;
     }
 
-  QFileInfo fileInfo(files[0]);
+  unsigned int numFiles = files.size();
+  QString reg_name = QFileInfo(files[0]).fileName();
+
+  if (numFiles > 1)
+    {
+    // Find the largest prefix that matches all filenames, and then append a '*'
+    // to signify that it is a collection of files.  If they all start with
+    // something different, just give up and add the '*' anyway.
+    for (unsigned int i = 1; i < numFiles; i++)
+      {
+      QString nextFile = QFileInfo(files[i]).fileName();
+      if (nextFile.startsWith(reg_name)) continue;
+      QString commonPrefix = reg_name;
+      do
+        {
+        commonPrefix.chop(1);
+        } while (!nextFile.startsWith(commonPrefix) && !commonPrefix.isEmpty());
+      if (commonPrefix.isEmpty()) break;
+      reg_name = commonPrefix;
+      }
+    reg_name += '*';
+    }
 
   vtkSMProxy* proxy = 
-    this->createProxyInternal(sm_group, sm_name, server, "sources", 
-      fileInfo.fileName());
+    this->createProxyInternal(sm_group, sm_name, server, "sources", reg_name);
   if (!proxy)
     {
     return 0;
@@ -243,17 +245,25 @@ pqPipelineSource* pqObjectBuilder::createReader(const QString& sm_group,
       return 0;
       }
 
-    unsigned int numElems = files.size();
-    if (numElems == 1 || !prop->GetRepeatCommand())
+    // If there's a hint on the property indicating that this property expects a
+    // directory name, then, we will set the directory name on it.
+    bool use_dir = false;
+    if (prop->GetHints() && prop->GetHints()->FindNestedElementByName("UseDirectoryName"))
       {
-      pqSMAdaptor::setElementProperty(prop, files[0]);
+      use_dir = true;
+      }
+
+    if (numFiles == 1 || !prop->GetRepeatCommand())
+      {
+      pqSMAdaptor::setElementProperty(prop, 
+        pqObjectBuilderGetPath(files[0], use_dir));
       }
     else
       {
       QList<QVariant> values;
       foreach (QString file, files)
         {
-        values.push_back(file);
+        values.push_back(pqObjectBuilderGetPath(file, use_dir));
         }
       pqSMAdaptor::setMultipleElementProperty(prop, values);
       }
@@ -264,6 +274,7 @@ pqPipelineSource* pqObjectBuilder::createReader(const QString& sm_group,
   reader->setModifiedState(pqProxy::UNINITIALIZED);
 
   emit this->readerCreated(reader, files[0]);
+  emit this->readerCreated(reader, files);
   emit this->proxyCreated(reader);
   return reader;
 }
@@ -276,7 +287,7 @@ void pqObjectBuilder::destroy(pqPipelineSource* source)
     return;
     }
 
-  if (source->getNumberOfConsumers())
+  if (source->getAllConsumers().size() > 0)
     {
     qDebug() << "Cannot remove source with consumers.";
     return;
@@ -287,20 +298,28 @@ void pqObjectBuilder::destroy(pqPipelineSource* source)
   // * remove inputs.
   // TODO: this step should not be necessary, but it currently
   // is :(. Needs some looking into.
-  vtkSMProxyProperty* pp = vtkSMProxyProperty::SafeDownCast(
-    source->getProxy()->GetProperty("Input"));
-  if (pp)
+  vtkSmartPointer<vtkSMPropertyIterator> piter;
+  piter.TakeReference(source->getProxy()->NewPropertyIterator());
+  for(piter->Begin(); !piter->IsAtEnd(); piter->Next())
     {
-    pp->RemoveAllProxies();
+    vtkSMProxyProperty* pp = vtkSMProxyProperty::SafeDownCast(
+      piter->GetProperty());
+    if (pp)
+      {
+      pp->RemoveAllProxies();
+      }
     }
 
   // * remove all representations.
-  QList<pqDataRepresentation*> reprs = source->getRepresentations(0);
-  foreach (pqDataRepresentation* repr, reprs)
+  for (int cc=0; cc < source->getNumberOfOutputPorts(); cc++)
     {
-    if (repr)
+    QList<pqDataRepresentation*> reprs = source->getRepresentations(cc, NULL);
+    foreach (pqDataRepresentation* repr, reprs)
       {
-      this->destroy(repr);
+      if (repr)
+        {
+        this->destroy(repr);
+        }
       }
     }
 
@@ -320,28 +339,16 @@ pqView* pqObjectBuilder::createView(const QString& type,
 
   vtkSMProxyManager* pxm = vtkSMProxyManager::GetProxyManager();
   vtkSMProxy* proxy= 0;
-  if(type == pqRenderView::renderViewType())
+
+  QObjectList ifaces =
+    pqApplicationCore::instance()->getPluginManager()->interfaces();
+  foreach(QObject* iface, ifaces)
     {
-    proxy = server->newRenderView();
-    }
-  else if (type == pqComparativeRenderView::comparativeRenderViewType())
-    {
-    QString xmlname = server->getRenderViewXMLName();
-    xmlname = "Comparative" + xmlname;
-    proxy = pxm->NewProxy("views", xmlname.toAscii().data());
-    }
-  else
-    {
-    QObjectList ifaces =
-      pqApplicationCore::instance()->getPluginManager()->interfaces();
-    foreach(QObject* iface, ifaces)
+    pqViewModuleInterface* vmi = qobject_cast<pqViewModuleInterface*>(iface);
+    if(vmi && vmi->viewTypes().contains(type))
       {
-      pqViewModuleInterface* vmi = qobject_cast<pqViewModuleInterface*>(iface);
-      if(vmi && vmi->viewTypes().contains(type))
-        {
-        proxy = vmi->createViewProxy(type);
-        break;
-        }
+      proxy = vmi->createViewProxy(type, server);
+      break;
       }
     }
 
@@ -353,8 +360,7 @@ pqView* pqObjectBuilder::createView(const QString& type,
 
   proxy->SetConnectionID(server->GetConnectionID());
 
-  QString name = QString("%1%2").arg(proxy->GetXMLName()).arg(
-    this->NameGenerator->GetCountAndIncrement(proxy->GetXMLName()));
+  QString name = ::pqObjectBuilderGetName(proxy, this->NameGenerator);
   pxm->RegisterProxy("views", name.toAscii().data(), proxy);
   proxy->Delete();
 
@@ -408,7 +414,8 @@ void pqObjectBuilder::destroy(pqView* view)
 
 //-----------------------------------------------------------------------------
 pqDataRepresentation* pqObjectBuilder::createDataRepresentation(
-  pqOutputPort* opPort, pqView* view)
+  pqOutputPort* opPort, pqView* view,
+  const QString &representationType)
 {
   if (!opPort || !view)
     {
@@ -428,8 +435,14 @@ pqDataRepresentation* pqObjectBuilder::createDataRepresentation(
 
   // HACK to create correct representation for text sources/filters.
   QString srcProxyName = source->getProxy()->GetXMLName();
-  if ( (srcProxyName == "TextSource" || srcProxyName == "TimeToTextConvertor"
-      || srcProxyName == "TimeToTextConvertorSource") && 
+  if (representationType != "")
+    {
+    reprProxy = vtkSMObject::GetProxyManager()->NewProxy(
+      "representations", representationType.toAscii().data());
+    }
+  else if ( (srcProxyName == "TextSource" || 
+             srcProxyName == "TimeToTextConvertor"
+             || srcProxyName == "TimeToTextConvertorSource") && 
     qobject_cast<pqRenderView*>(view))
     {
     reprProxy = vtkSMObject::GetProxyManager()->NewProxy(
@@ -462,6 +475,12 @@ pqDataRepresentation* pqObjectBuilder::createDataRepresentation(
   // Set the reprProxy's input.
   pqSMAdaptor::setInputProperty(reprProxy->GetProperty("Input"), 
     source->getProxy(), opPort->getPortNumber());
+  // Let application ignore default and hide display of filters if they must.
+  if (pqApplicationCore::instance()->getDisplayPolicy()->getHideByDefault())
+    {
+    pqSMAdaptor::setElementProperty(reprProxy->GetProperty("Visibility"),
+                                    0);
+    }
   reprProxy->UpdateVTKObjects();
 
   // Add the reprProxy to render module.
@@ -474,6 +493,9 @@ pqDataRepresentation* pqObjectBuilder::createDataRepresentation(
     findItem<pqDataRepresentation*>(reprProxy);
   if (repr)
     {
+    // inherit properties from the representation for the input if applicable.
+    this->initializeInheritedProperties(repr);
+
     repr->setDefaultPropertyValues();
 
     emit this->dataRepresentationCreated(repr);
@@ -618,7 +640,7 @@ void pqObjectBuilder::destroySources(pqServer* server)
     {
     for(int i=0; i<sources.size(); i++)
       {
-      if(sources[i]->getNumberOfConsumers() == 0)
+      if(sources[i]->getAllConsumers().size() == 0)
         {
         builder->destroy(sources[i]);
         sources[i] = NULL;
@@ -680,25 +702,16 @@ vtkSMProxy* pqObjectBuilder::createProxyInternal(const QString& sm_group,
     qDebug() << "server cannot be null";
     return 0;
     }
-
-  QString actual_regname = reg_name;
-  if (reg_name.isEmpty())
+  if (sm_group.isEmpty() || sm_name.isEmpty())
     {
-    actual_regname = QString("%1%2").arg(sm_name).arg(
-      this->NameGenerator->GetCountAndIncrement(sm_name));
+    qCritical() << "Group name and proxy name must be non empty.";
+    return 0;
     }
 
   vtkSMProxyManager* pxm = vtkSMObject::GetProxyManager();
   vtkSmartPointer<vtkSMProxy> proxy;
-  if (sm_group.isEmpty())
-    {
-    proxy.TakeReference(pxm->NewCompoundProxy(sm_name.toAscii().data()));
-    }
-  else
-    {
-    proxy.TakeReference(
-      pxm->NewProxy(sm_group.toAscii().data(), sm_name.toAscii().data()));
-    }
+  proxy.TakeReference(
+    pxm->NewProxy(sm_group.toAscii().data(), sm_name.toAscii().data()));
 
   if (!proxy.GetPointer())
     {
@@ -707,6 +720,12 @@ vtkSMProxy* pqObjectBuilder::createProxyInternal(const QString& sm_group,
     }
   proxy->SetConnectionID(server->GetConnectionID());
 
+  QString actual_regname = reg_name;
+  if (reg_name.isEmpty())
+    {
+    actual_regname = ::pqObjectBuilderGetName(proxy, this->NameGenerator);
+    }
+
   pxm->RegisterProxy(reg_group.toAscii().data(), 
     actual_regname.toAscii().data(), proxy);
   return proxy;
@@ -734,7 +753,7 @@ QString pqObjectBuilder::getFileNamePropertyName(vtkSMProxy* proxy) const
   while(!piter->IsAtEnd())
     {
     vtkSMProperty* prop = piter->GetProperty();
-    if (prop->IsA("vtkSMStringVectorProperty"))
+    if (prop && prop->IsA("vtkSMStringVectorProperty"))
       {
       vtkSmartPointer<vtkSMDomainIterator> diter;
       diter.TakeReference(prop->NewDomainIterator());
@@ -842,3 +861,38 @@ void pqObjectBuilder::removeServer(pqServer* server)
   core->getServerManagerModel()->endRemoveServer();
 }
 
+//-----------------------------------------------------------------------------
+void pqObjectBuilder::initializeInheritedProperties(pqDataRepresentation* repr)
+{
+ vtkPVXMLElement* hints = repr->getProxy()->GetHints();
+ hints = hints? hints->FindNestedElementByName("InheritRepresentationProperties") : 0;
+ if (hints == 0)
+   {
+   return;
+   }
+
+ pqDataRepresentation* input_repr = 
+   repr->getRepresentationForUpstreamSource();
+ if (!input_repr)
+   {
+   return;
+   }
+
+ vtkSMProxy* reprProxy = repr->getProxy();
+ vtkSMProxy* inputReprProxy = input_repr->getProxy();
+  unsigned int num_children = hints->GetNumberOfNestedElements();
+  for (unsigned int cc=0; cc < num_children; cc++)
+    {
+    vtkPVXMLElement* child = hints->GetNestedElement(cc);
+    if (child && child->GetName() && strcmp(child->GetName(), "Property") == 0)
+      {
+      const char* propname = child->GetAttribute("name");
+      if (propname && reprProxy->GetProperty(propname) &&
+        inputReprProxy->GetProperty(propname))
+        {
+        reprProxy->GetProperty(propname)->Copy(
+          inputReprProxy->GetProperty(propname));
+        }
+      }
+    }
+}
