@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqPipelineRepresentation.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -39,17 +39,19 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 // ParaView Server Manager includes.
 #include "vtkCommand.h"
 #include "vtkEventQtSlotConnect.h"
+#include "vtkMath.h"
 #include "vtkProcessModule.h"
+#include "vtkProperty.h"
 #include "vtkPVArrayInformation.h"
 #include "vtkPVDataSetAttributesInformation.h"
 #include "vtkPVGeometryInformation.h"
 #include "vtkSmartPointer.h" 
-#include "vtkSMCompoundProxy.h"
 #include "vtkSMDoubleVectorProperty.h"
 #include "vtkSMProxyManager.h"
 #include "vtkSMProxyProperty.h"
 #include "vtkSMPVRepresentationProxy.h"
 #include "vtkSMSourceProxy.h"
+#include "vtkSMSurfaceRepresentationProxy.h"
 
 // Qt includes.
 #include <QList>
@@ -67,33 +69,38 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqPipelineSource.h"
 #include "pqRenderView.h"
 #include "pqScalarBarRepresentation.h"
+#include "pqScalarOpacityFunction.h"
 #include "pqScalarsToColors.h"
 #include "pqServer.h"
 #include "pqServerManagerModel.h"
+#include "pqSettings.h"
 #include "pqSMAdaptor.h"
+#include "pqDisplayPolicy.h"
 
 //-----------------------------------------------------------------------------
 class pqPipelineRepresentation::pqInternal
 {
 public:
-  vtkSmartPointer<vtkSMPVRepresentationProxy> RepresentationProxy;
+  vtkSmartPointer<vtkSMPropRepresentationProxy> RepresentationProxy;
   vtkSmartPointer<vtkEventQtSlotConnect> VTKConnect;
+  pqScalarOpacityFunction *Opacity;
 
   pqInternal()
     {
     this->VTKConnect = vtkSmartPointer<vtkEventQtSlotConnect>::New();
+    this->Opacity = 0;
     }
 
   // Convenience method to get array information.
   vtkPVArrayInformation* getArrayInformation(
-    const char* arrayname, int fieldType)
+    const char* arrayname, int fieldType, vtkPVDataInformation* argInfo=0)
     {
     if (!arrayname || !arrayname[0] || !this->RepresentationProxy)
       {
       return 0; 
       }
     vtkSMDataRepresentationProxy* repr = this->RepresentationProxy;
-    vtkPVDataInformation* dataInfo = repr->GetFullResDataInformation();
+    vtkPVDataInformation* dataInfo = argInfo? argInfo: repr->GetRepresentedDataInformation();
     if(!dataInfo)
       {
       return 0;
@@ -120,12 +127,18 @@ public:
 pqPipelineRepresentation::pqPipelineRepresentation(
   const QString& group,
   const QString& name,
-  vtkSMPVRepresentationProxy* display,
+  vtkSMProxy* display,
   pqServer* server, QObject* p/*=null*/):
   Superclass(group, name, display, server, p)
 {
   this->Internal = new pqPipelineRepresentation::pqInternal();
-  this->Internal->RepresentationProxy = display;
+  this->Internal->RepresentationProxy
+    = vtkSMPropRepresentationProxy::SafeDownCast(display);
+
+  if (!this->Internal->RepresentationProxy)
+    {
+    qFatal("Display given is not a vtkSMPropRepresentationProxy.");
+    }
 
   // If any of these properties change, we know that the coloring for the
   // representation has been affected.
@@ -162,19 +175,38 @@ pqPipelineRepresentation::~pqPipelineRepresentation()
 }
 
 //-----------------------------------------------------------------------------
-vtkSMPVRepresentationProxy* pqPipelineRepresentation::getRepresentationProxy() const
+vtkSMPropRepresentationProxy* pqPipelineRepresentation::getRepresentationProxy() const
 {
   return this->Internal->RepresentationProxy;
 }
 
 //-----------------------------------------------------------------------------
-vtkSMProxy* pqPipelineRepresentation::getScalarOpacityFunctionProxy() const
+vtkSMProxy* pqPipelineRepresentation::getScalarOpacityFunctionProxy()
 {
   // We may want to create a new proxy is none exists.
   return pqSMAdaptor::getProxyProperty(
     this->getProxy()->GetProperty("ScalarOpacityFunction"));
 }
 
+//-----------------------------------------------------------------------------
+pqScalarOpacityFunction* pqPipelineRepresentation::getScalarOpacityFunction()
+{
+  if(this->getRepresentationType() == vtkSMPVRepresentationProxy::VOLUME)
+    {
+    if(!this->Internal->Opacity)
+      {
+      // TODO: Add the opacity function to the server manager model.
+      this->Internal->Opacity = new pqScalarOpacityFunction(
+        "piecewise_functions", "PiecewiseFunction",
+        this->getScalarOpacityFunctionProxy(), this->getServer(), this);
+      }
+
+    return this->Internal->Opacity;
+    }
+
+  return 0;
+}
+
 //-----------------------------------------------------------------------------
 void pqPipelineRepresentation::createHelperProxies()
 {
@@ -206,7 +238,9 @@ void pqPipelineRepresentation::setDefaultPropertyValues()
   // its messing up with the scalar coloring.
   // this->Superclass::setDefaultPropertyValues();
 
-  if (!this->isVisible())
+  if (!this->isVisible() &&
+      !pqApplicationCore::instance()->getDisplayPolicy()->getHideByDefault()
+      )
     {
     // don't worry about invisible displays.
     return;
@@ -214,7 +248,7 @@ void pqPipelineRepresentation::setDefaultPropertyValues()
 
   this->createHelperProxies();
 
-  vtkSMPVRepresentationProxy* repr = this->getRepresentationProxy();
+  vtkSMPropRepresentationProxy* repr = this->getRepresentationProxy();
   if (!repr)
     {
     return;
@@ -242,37 +276,48 @@ void pqPipelineRepresentation::setDefaultPropertyValues()
   vtkPVArrayInformation* arrayInfo;
 
   // Get the time that this representation is going to use.
-  double time = pqSMAdaptor::getElementProperty(
-    repr->GetProperty("UpdateTime")).toDouble();
-
   vtkPVDataInformation* dataInfo = 0;
 
-  vtkSMProxy* inputProxy = this->getInput()->getProxy();
-  if (inputProxy->IsA("vtkSMCompoundProxy"))
-    {
-    inputProxy = vtkSMCompoundProxy::SafeDownCast(inputProxy)->GetConsumableProxy();
-    }
-  vtkSMSourceProxy* inputSourceProxy = vtkSMSourceProxy::SafeDownCast(inputProxy);
-  inputSourceProxy->UpdatePipeline(time);
-
   dataInfo = this->getOutputPortFromInput()->getDataInformation(true);
 
   // get data set type
   // and set the default representation
-  if(dataInfo)
+  if (dataInfo && repr->IsA("vtkSMPVRepresentationProxy"))
     {
     int dataSetType = dataInfo->GetDataSetType();
     if(dataSetType == VTK_POLY_DATA ||
-       dataSetType == VTK_UNSTRUCTURED_GRID ||
        dataSetType == VTK_HYPER_OCTREE ||
        dataSetType == VTK_GENERIC_DATA_SET)
       {
       pqSMAdaptor::setEnumerationProperty(repr->GetProperty("Representation"),
         "Surface");
       }
+    else if (dataSetType == VTK_UNSTRUCTURED_GRID)
+      {
+      if (static_cast<double>(dataInfo->GetNumberOfCells()) >= 
+        pqPipelineRepresentation::getUnstructuredGridOutlineThreshold()*1000000.0)
+        {
+        pqSMAdaptor::setEnumerationProperty(repr->GetProperty("Representation"),
+          "Outline");
+        }
+      }
+    else if (dataSetType == VTK_IMAGE_DATA)
+      {
+      // Use slice representation by default for 2D image data.
+      int* ext = dataInfo->GetExtent();
+      if (ext[0] == ext[1] || ext[2] == ext[3] || ext[4] == ext[5])
+        {
+        pqSMAdaptor::setEnumerationProperty(repr->GetProperty("Representation"),
+          "Slice");
+        }
+      else
+        {
+        pqSMAdaptor::setEnumerationProperty(repr->GetProperty("Representation"),
+          "Outline");
+        }
+      }
     else if(dataSetType == VTK_RECTILINEAR_GRID ||
-       dataSetType == VTK_STRUCTURED_GRID ||
-       dataSetType == VTK_IMAGE_DATA)
+       dataSetType == VTK_STRUCTURED_GRID)
       {
       int* ext = dataInfo->GetExtent();
       if (ext[0] == ext[1] || ext[2] == ext[3] || ext[4] == ext[5])
@@ -296,17 +341,21 @@ void pqPipelineRepresentation::setDefaultPropertyValues()
     {
     double bounds[6];
     dataInfo->GetBounds(bounds);
-    double diameter =
-      sqrt( (bounds[1] - bounds[0]) * (bounds[1] - bounds[0]) +
-        (bounds[3] - bounds[2]) * (bounds[3] - bounds[2]) +
-        (bounds[5] - bounds[4]) * (bounds[5] - bounds[4]) );
-
-    int numCells = dataInfo->GetNumberOfCells();
-    double linearNumCells = pow( (double) numCells, (1.0/3.0) );
-    double unitDistance = diameter;
-    if (linearNumCells != 0.0)
+    double unitDistance = 1.0;
+    if(vtkMath::AreBoundsInitialized(bounds))
       {
-      unitDistance = diameter / linearNumCells;
+      double diameter =
+        sqrt( (bounds[1] - bounds[0]) * (bounds[1] - bounds[0]) +
+          (bounds[3] - bounds[2]) * (bounds[3] - bounds[2]) +
+          (bounds[5] - bounds[4]) * (bounds[5] - bounds[4]) );
+
+      int numCells = dataInfo->GetNumberOfCells();
+      double linearNumCells = pow( (double) numCells, (1.0/3.0) );
+      unitDistance = diameter;
+      if (linearNumCells != 0.0)
+        {
+        unitDistance = diameter / linearNumCells;
+        }
       }
     pqSMAdaptor::setElementProperty(
       repr->GetProperty("ScalarOpacityUnitDistance"),
@@ -314,28 +363,22 @@ void pqPipelineRepresentation::setDefaultPropertyValues()
     }
   repr->UpdateVTKObjects();
 
-  // Update the representation, so that we can obtain updated data information.
-  repr->Update();
-  geomInfo = repr->GetFullResDataInformation();
+  geomInfo = repr->GetRepresentedDataInformation(/*update=*/true);
 
   // Locate input display.
-  pqPipelineFilter* myInput = qobject_cast<pqPipelineFilter*>(this->getInput());
-  pqPipelineRepresentation* upstreamDisplay = 0;
-  if (myInput && myInput->getInputCount() > 0)
+  pqPipelineRepresentation* upstreamDisplay = 
+    qobject_cast<pqPipelineRepresentation*>(
+      this->getRepresentationForUpstreamSource());
+  if (upstreamDisplay)
     {
-    pqPipelineSource* myInputsInput = myInput->getInput(0);
-    upstreamDisplay = qobject_cast<pqPipelineRepresentation*>(
-      myInputsInput->getRepresentation(0, this->getView()));
-    if (upstreamDisplay)
-      {
-      inGeomInfo = upstreamDisplay->getRepresentationProxy()->GetFullResDataInformation();
-      }
+    inGeomInfo = upstreamDisplay->getRepresentationProxy()->
+      GetRepresentedDataInformation();
     }
 
   // Look for a new point array.
   // I do not think the logic is exactly as describerd in this methods
   // comment.  I believe this method only looks at "Scalars".
-  if(geomInfo)
+  if (geomInfo)
     {
     attrInfo = geomInfo->GetPointDataInformation();
     inAttrInfo = inGeomInfo? inGeomInfo->GetPointDataInformation() : 0;
@@ -349,7 +392,7 @@ void pqPipelineRepresentation::setDefaultPropertyValues()
     }
     
   // Check for new cell scalars.
-  if(geomInfo)
+  if (geomInfo)
     {
     attrInfo = geomInfo->GetCellDataInformation();
     inAttrInfo = inGeomInfo? inGeomInfo->GetCellDataInformation() : 0;
@@ -387,16 +430,32 @@ void pqPipelineRepresentation::setDefaultPropertyValues()
       return;
       }
     }
+  
+  QList<QString> myColorFields = this->getColorFields();
 
   // Try to inherit the same array selected by the input.
   if (upstreamDisplay)
     {
-    // propagate solid color.
-    repr->GetProperty("Color")->Copy(
-      upstreamDisplay->getProxy()->GetProperty("Color"));
+    const QString &upstreamColorField = upstreamDisplay->getColorField(false);
+    if (myColorFields.contains(upstreamColorField))
+      {
+      this->setColorField(upstreamColorField);
+      return;
+      }
+    }
 
-    this->setColorField(upstreamDisplay->getColorField(false));
-    return;
+  // We are going to set the default color mode to use solid color i.e. not use
+  // scalar coloring at all. However, for some representations (eg. slice/volume)
+  // this is an error, we have to color by some array. Since no active scalar
+  // were choosen, we simply use the first color array available. (If no arrays
+  // are available, then error will be raised anyways).
+  if (!myColorFields.contains(pqPipelineRepresentation::solidColor()))
+    {
+    if (myColorFields.size() > 0)
+      {
+      this->setColorField(myColorFields[0]);
+      return;
+      }
     }
 
   // Color by property.
@@ -415,7 +474,7 @@ int pqPipelineRepresentation::getNumberOfComponents(
 //-----------------------------------------------------------------------------
 void pqPipelineRepresentation::colorByArray(const char* arrayname, int fieldtype)
 {
-  vtkSMPVRepresentationProxy* repr = this->getRepresentationProxy();
+  vtkSMPropRepresentationProxy* repr = this->getRepresentationProxy();
   if (!repr)
     {
     return;
@@ -521,7 +580,54 @@ void pqPipelineRepresentation::colorByArray(const char* arrayname, int fieldtype
 //-----------------------------------------------------------------------------
 int pqPipelineRepresentation::getRepresentationType() const
 {
-  return this->getRepresentationProxy()->GetRepresentation();
+  vtkSMProxy* repr = this->getRepresentationProxy();
+  vtkSMPVRepresentationProxy* pvRepr = 
+    vtkSMPVRepresentationProxy::SafeDownCast(repr);
+  if (pvRepr)
+    {
+    return pvRepr->GetRepresentation();
+    }
+
+  const char* xmlname = repr->GetXMLName();
+  if (strcmp(xmlname, "SurfaceRepresentation") == 0)
+    {
+    int reprType = 
+      pqSMAdaptor::getElementProperty(repr->GetProperty("Representation")).toInt();
+    switch (reprType)
+      {
+    case VTK_POINTS:
+      return vtkSMPVRepresentationProxy::POINTS;
+
+    case VTK_WIREFRAME:
+      return vtkSMPVRepresentationProxy::WIREFRAME;
+
+    case VTK_SURFACE_WITH_EDGES:
+      return vtkSMPVRepresentationProxy::SURFACE_WITH_EDGES;
+
+    case VTK_SURFACE:
+    default:
+      return vtkSMPVRepresentationProxy::SURFACE;
+      }
+    }
+  
+  if (strcmp(xmlname, "OutlineRepresentation") == 0)
+    {
+    return vtkSMPVRepresentationProxy::OUTLINE;
+    }
+
+  if (strcmp(xmlname, "UnstructuredGridVolumeRepresentation") == 0 ||
+    strcmp(xmlname, "UniformGridVolumeRepresentation") == 0)
+    {
+    return vtkSMPVRepresentationProxy::VOLUME;
+    }
+
+  if (strcmp(xmlname, "ImageSliceRepresentation") == 0)
+    {
+    return vtkSMPVRepresentationProxy::SLICE;
+    }
+
+  qCritical() << "pqPipelineRepresentation created for a incorrect proxy : " << xmlname;
+  return 0;
 }
 
 //-----------------------------------------------------------------------------
@@ -530,7 +636,15 @@ double pqPipelineRepresentation::getOpacity() const
   vtkSMProperty* prop = this->getProxy()->GetProperty("Opacity");
   return (prop? pqSMAdaptor::getElementProperty(prop).toDouble() : 1.0);
 }
+//-----------------------------------------------------------------------------
+void pqPipelineRepresentation::setColor(double R,double G,double B)
+{
+  pqSMAdaptor::setMultipleElementProperty(this->getProxy()->GetProperty("Color"),0,R);
+  pqSMAdaptor::setMultipleElementProperty(this->getProxy()->GetProperty("Color"),1,G);
+  pqSMAdaptor::setMultipleElementProperty(this->getProxy()->GetProperty("Color"),2,B);
+  this->getProxy()->UpdateVTKObjects();
 
+}
 //-----------------------------------------------------------------------------
 void pqPipelineRepresentation::resetLookupTableScalarRange()
 {
@@ -543,7 +657,11 @@ void pqPipelineRepresentation::resetLookupTableScalarRange()
     lut->setScalarRange(range.first, range.second);
 
     // scalar opacity is treated as slave to the lookup table.
-    this->setScalarOpacityRange(range.first, range.second);
+    pqScalarOpacityFunction* opacity = this->getScalarOpacityFunction();
+    if(opacity)
+      {
+      opacity->setScalarRange(range.first, range.second);
+      }
     }
 }
 
@@ -566,74 +684,17 @@ void pqPipelineRepresentation::updateLookupTableScalarRange()
   lut->setWholeScalarRange(range.first, range.second);
 
   // Adjust opacity function range.
-  vtkSMProxy* opacityFunction = this->getScalarOpacityFunctionProxy();
-  if (opacityFunction && !lut->getScalarRangeLock() &&
-    this->getRepresentationType() == vtkSMPVRepresentationProxy::VOLUME)
+  pqScalarOpacityFunction* opacityFunction = this->getScalarOpacityFunction();
+  if (opacityFunction && !lut->getScalarRangeLock())
     {
     QPair<double, double> adjusted_range = lut->getScalarRange();
 
     // Opacity function always follows the LUT scalar range.
     // scalar opacity is treated as slave to the lookup table.
-    this->setScalarOpacityRange(adjusted_range.first, adjusted_range.second);
+    opacityFunction->setScalarRange(adjusted_range.first, adjusted_range.second);
     }
 }
 
-//-----------------------------------------------------------------------------
-void pqPipelineRepresentation::setScalarOpacityRange(double min, double max)
-{
-  // A far more better way would be to create a new pqProxy subclass for
-  // the piecewise function.
-  vtkSMProxy* opacityFunction = this->getScalarOpacityFunctionProxy();
-  if (!opacityFunction)
-    {
-    return;
-    }
-
-  vtkSMDoubleVectorProperty* dvp = vtkSMDoubleVectorProperty::SafeDownCast(
-    opacityFunction->GetProperty("Points"));
-
-  QList<QVariant> controlPoints = pqSMAdaptor::getMultipleElementProperty(dvp);
-  if (controlPoints.size() == 0)
-    {
-    return;
-    }
-
-  int max_index = dvp->GetNumberOfElementsPerCommand() * (
-    (controlPoints.size()-1)/ dvp->GetNumberOfElementsPerCommand());
-  QPair<double, double> current_range(controlPoints[0].toDouble(),
-    controlPoints[max_index].toDouble());
-
-  // Adjust vtkPiecewiseFunction points to the new range.
-  double dold = (current_range.second - current_range.first);
-  dold = (dold > 0) ? dold : 1;
-
-  double dnew = (max -min);
-
-  if (dnew > 0)
-    {
-    double scale = dnew/dold;
-    for (int cc=0; cc < controlPoints.size(); 
-         cc+= dvp->GetNumberOfElementsPerCommand())
-      {
-      controlPoints[cc] = 
-        scale * (controlPoints[cc].toDouble()-current_range.first) + min;
-      }
-    }
-  else
-    {
-    // allowing an opacity transfer function with a scalar range of 0.
-    // In this case, the piecewise function only contains the endpoints.
-    controlPoints.clear();
-    controlPoints.push_back(min);
-    controlPoints.push_back(0);
-    controlPoints.push_back(max);
-    controlPoints.push_back(1);
-    }
-
-  pqSMAdaptor::setMultipleElementProperty(dvp, controlPoints);
-  opacityFunction->UpdateVTKObjects();
-}
-
 //-----------------------------------------------------------------------------
 void pqPipelineRepresentation::getColorArray(
   vtkPVDataSetAttributesInformation* attrInfo,
@@ -666,7 +727,7 @@ void pqPipelineRepresentation::getColorArray(
 //-----------------------------------------------------------------------------
 QList<QString> pqPipelineRepresentation::getColorFields()
 {
-  vtkSMPVRepresentationProxy* repr = this->getRepresentationProxy();
+  vtkSMPropRepresentationProxy* repr = this->getRepresentationProxy();
 
   QList<QString> ret;
   if(!repr)
@@ -676,14 +737,15 @@ QList<QString> pqPipelineRepresentation::getColorFields()
 
   int representation = this->getRepresentationType();
 
-  if (representation != vtkSMPVRepresentationProxy::VOLUME)
+  if (representation != vtkSMPVRepresentationProxy::VOLUME && 
+    representation != vtkSMPVRepresentationProxy::SLICE)
     {
     // Actor color is one way to color this part.
     // Not applicable when volume rendering.
     ret.append(pqPipelineRepresentation::solidColor());
     }
 
-  vtkPVDataInformation* geomInfo = repr->GetFullResDataInformation();
+  vtkPVDataInformation* geomInfo = repr->GetRepresentedDataInformation();
   if(!geomInfo)
     {
     return ret;
@@ -714,6 +776,13 @@ QList<QString> pqPipelineRepresentation::getColorFields()
       for(int i=0; i<cellinfo->GetNumberOfArrays(); i++)
         {
         vtkPVArrayInformation* info = cellinfo->GetArrayInformation(i);
+        if (representation == vtkSMPVRepresentationProxy::VOLUME &&
+          info->GetNumberOfComponents() != 1)
+          {
+          // Skip vectors when volumerendering.
+          continue;
+          }
+
         QString name = info->GetName();
         name += " (cell)";
         ret.append(name);
@@ -729,6 +798,12 @@ QList<QString> pqPipelineRepresentation::getColorFields()
     for(int i=0; i<pointinfo->GetNumberOfArrays(); i++)
       {
       vtkPVArrayInformation* info = pointinfo->GetArrayInformation(i);
+      if (representation == vtkSMPVRepresentationProxy::VOLUME &&
+        info->GetNumberOfComponents() != 1)
+        {
+        // Skip vectors when volumerendering.
+        continue;
+        }
       QString name = info->GetName();
       name += " (point)";
       ret.append(name);
@@ -762,6 +837,14 @@ int pqPipelineRepresentation::getColorFieldNumberOfComponents(const QString& arr
     fieldType);
 }
 
+//-----------------------------------------------------------------------------
+bool pqPipelineRepresentation::isPartial(const QString& array, int fieldType) const
+{
+  vtkPVArrayInformation* info = this->Internal->getArrayInformation(
+    array.toAscii().data(), fieldType, this->getInputDataInformation());
+  return (info? (info->GetIsPartial()==1) : false);
+}
+
 //-----------------------------------------------------------------------------
 QPair<double, double> 
 pqPipelineRepresentation::getColorFieldRange(const QString& array, int component)
@@ -786,17 +869,37 @@ pqPipelineRepresentation::getColorFieldRange(const QString& array, int component
     fieldType = vtkSMDataRepresentationProxy::POINT_DATA;
     }
 
-  vtkPVArrayInformation* info = this->Internal->getArrayInformation(
+  vtkPVArrayInformation* representedInfo = this->Internal->getArrayInformation(
     field.toAscii().data(), fieldType);
-  if(info)
+
+  vtkPVDataInformation* inputInformation = this->getInputDataInformation();
+  vtkPVArrayInformation* inputInfo = this->Internal->getArrayInformation(
+    field.toAscii().data(), fieldType, inputInformation);
+
+  // Try to use full input data range is possible. Sometimes, the data array is
+  // only provided by some pre-processing filter added by the representation
+  // (and is not present in the original input), in that case we use the range
+  // provided by the representation.
+  if (inputInfo)
     {
-    if (component <info->GetNumberOfComponents())
+    if (component < inputInfo->GetNumberOfComponents())
       {
       double range[2];
-      info->GetComponentRange(component, range);
+      inputInfo->GetComponentRange(component, range);
       return QPair<double,double>(range[0], range[1]);
       }
     }
+
+  if (representedInfo)
+    {
+    if (component <representedInfo->GetNumberOfComponents())
+      {
+      double range[2];
+      representedInfo->GetComponentRange(component, range);
+      return QPair<double,double>(range[0], range[1]);
+      }
+    }
+
   return ret;
 }
 
@@ -825,7 +928,7 @@ QPair<double, double> pqPipelineRepresentation::getColorFieldRange()
 //-----------------------------------------------------------------------------
 void pqPipelineRepresentation::setColorField(const QString& value)
 {
-  vtkSMPVRepresentationProxy* repr = this->getRepresentationProxy();
+  vtkSMPropRepresentationProxy* repr = this->getRepresentationProxy();
 
   if(!repr)
     {
@@ -859,7 +962,7 @@ void pqPipelineRepresentation::setColorField(const QString& value)
 //-----------------------------------------------------------------------------
 QString pqPipelineRepresentation::getColorField(bool raw)
 {
-  vtkSMPVRepresentationProxy* repr = this->getRepresentationProxy();
+  vtkSMPropRepresentationProxy* repr = this->getRepresentationProxy();
   if (!repr)
     {
     return pqPipelineRepresentation::solidColor();
@@ -893,21 +996,23 @@ QString pqPipelineRepresentation::getColorField(bool raw)
 //-----------------------------------------------------------------------------
 bool pqPipelineRepresentation::getDataBounds(double bounds[6])
 {
-  vtkSMPVRepresentationProxy* repr = 
+  vtkSMPropRepresentationProxy* repr = 
     this->getRepresentationProxy();
 
-  if(!repr || !repr->GetFullResDataInformation())
+  vtkPVDataInformation* info = repr? 
+    repr->GetRepresentedDataInformation() : 0;
+  if(!info)
     {
     return false;
     }
-  repr->GetFullResDataInformation()->GetBounds(bounds);
+  info->GetBounds(bounds);
   return true;
 }
 
 //-----------------------------------------------------------------------------
 void pqPipelineRepresentation::setRepresentation(int representation)
 {
-  vtkSMPVRepresentationProxy* repr = this->getRepresentationProxy();
+  vtkSMPropRepresentationProxy* repr = this->getRepresentationProxy();
   pqSMAdaptor::setElementProperty(
     repr->GetProperty("Representation"), representation);
   repr->UpdateVTKObjects();
@@ -917,19 +1022,20 @@ void pqPipelineRepresentation::setRepresentation(int representation)
 //-----------------------------------------------------------------------------
 void pqPipelineRepresentation::onRepresentationChanged()
 {
- vtkSMPVRepresentationProxy* repr = this->getRepresentationProxy();
+  vtkSMPropRepresentationProxy* repr = this->getRepresentationProxy();
+  if (!repr)
+    {
+    return;
+    }
 
-  if(!repr || 
-    this->getRepresentationType() != vtkSMPVRepresentationProxy::VOLUME)
+  int reprType = this->getRepresentationType();
+  if (reprType != vtkSMPVRepresentationProxy::VOLUME  &&
+    reprType != vtkSMPVRepresentationProxy::SLICE)
     {
     // Nothing to do here.
     return;
     }
 
-  // This is essential, since otherwise the geometry filter does not execute
-  // and we won't get the correct arrays available.
-  repr->Update();
-
   // Representation is Volume, is color array set?
   QList<QString> colorFields = this->getColorFields();
   if (colorFields.size() == 0)
@@ -995,3 +1101,42 @@ void pqPipelineRepresentation::updateScalarBarVisibility(bool visible)
       }
     }
 }
+
+//-----------------------------------------------------------------------------
+const char* pqPipelineRepresentation::UNSTRUCTURED_GRID_OUTLINE_THRESHOLD()
+{
+  return "/representation/UnstructuredGridOutlineThreshold";
+}
+
+//-----------------------------------------------------------------------------
+void pqPipelineRepresentation::setUnstructuredGridOutlineThreshold(double numcells)
+{
+  pqApplicationCore* core = pqApplicationCore::instance();
+  pqSettings* settings = core->settings();
+  if (settings)
+    {
+    settings->setValue(
+      pqPipelineRepresentation::UNSTRUCTURED_GRID_OUTLINE_THRESHOLD(), 
+      QVariant(numcells));
+    }
+}
+
+//-----------------------------------------------------------------------------
+double pqPipelineRepresentation::getUnstructuredGridOutlineThreshold()
+{
+  pqApplicationCore* core = pqApplicationCore::instance();
+  pqSettings* settings = core->settings();
+  if (settings && settings->contains(
+      pqPipelineRepresentation::UNSTRUCTURED_GRID_OUTLINE_THRESHOLD()))
+    {
+    bool ok;
+    double numcells = settings->value(
+      pqPipelineRepresentation::UNSTRUCTURED_GRID_OUTLINE_THRESHOLD()).toDouble(&ok);
+    if (ok)
+      {
+      return numcells;
+      }
+    }
+
+  return 0.5; //  1/2 million cells.
+}
