@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqPipelineSource.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -37,12 +37,12 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 // ParaView Server Manager.
 #include "vtkClientServerStream.h"
+#include "vtkEventQtSlotConnect.h"
 #include "vtkProcessModule.h"
 #include "vtkPVDataInformation.h"
-#include "vtkPVNumberOfOutputsInformation.h"
 #include "vtkPVXMLElement.h"
 #include "vtkSmartPointer.h"
-#include "vtkSMCompoundProxy.h"
+#include "vtkSMPropertyHelper.h"
 #include "vtkSMPropertyIterator.h"
 #include "vtkSMPropertyLink.h"
 #include "vtkSMProxyListDomain.h"
@@ -90,22 +90,15 @@ pqPipelineSource::pqPipelineSource(const QString& name, vtkSMProxy* proxy,
 : pqProxy("sources", name, proxy, server, _parent)
 {
   this->Internal = new pqPipelineSourceInternal(name, proxy);
-  vtkSMCompoundProxy* compoundProxy = vtkSMCompoundProxy::SafeDownCast(this->getProxy());
-
-  vtkSMSourceProxy* source = compoundProxy? 
-    vtkSMSourceProxy::SafeDownCast(compoundProxy->GetConsumableProxy()):
-    vtkSMSourceProxy::SafeDownCast(this->getProxy());
+  vtkSMSourceProxy* source = vtkSMSourceProxy::SafeDownCast(this->getProxy());
   if (source)
     {
     // Obtain information about number of output ports.
-    vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
-    vtkPVNumberOfOutputsInformation* info =
-      vtkPVNumberOfOutputsInformation::New();
-    vtkClientServerStream stream;
-    pm->GatherInformation(
-      source->GetConnectionID(), source->GetServers(), info, source->GetID());
-    int numports = info->GetNumberOfOutputs();
-    info->Delete();
+    // Number of output ports is valid even if the output port proxies haven't
+    // been created (so long as CreateVTKObjects() has been called on the
+    // proxy).
+    source->GetID(); // causes CreateVTKObjects() to be called.
+    int numports = source->GetNumberOfOutputPorts();
 
     for (int cc=0; cc < numports; cc++)
       {
@@ -137,6 +130,8 @@ pqPipelineSource::pqPipelineSource(const QString& name, vtkSMProxy* proxy,
 
       this->Internal->OutputPorts.push_back(op);
       }
+    this->getConnector()->Connect(source, vtkCommand::UpdateDataEvent,
+        this, SLOT(dataUpdated()));
     }
 }
 
@@ -150,6 +145,20 @@ pqPipelineSource::~pqPipelineSource()
   delete this->Internal;
 }
 
+//-----------------------------------------------------------------------------
+void pqPipelineSource::updatePipeline()
+{
+  pqTimeKeeper* timekeeper = this->getServer()->getTimeKeeper();
+  double time = timekeeper->getTime();
+  vtkSMSourceProxy::SafeDownCast(this->getProxy())->UpdatePipeline(time);
+}
+
+//-----------------------------------------------------------------------------
+void pqPipelineSource::dataUpdated()
+{
+  emit this->dataUpdated(this);
+}
+
 //-----------------------------------------------------------------------------
 void pqPipelineSource::prePortConnectionAdded(
   pqOutputPort* op, pqPipelineSource* cons)
@@ -350,6 +359,8 @@ void pqPipelineSource::setDefaultPropertyValues()
     // initialized correctly eg. FileName in case of readers,
     // all Input/Source properties in case filters etc.
     sp->UpdatePipelineInformation();
+
+    sp->CreateOutputPorts();
     }
 
   this->Superclass::setDefaultPropertyValues();
@@ -368,6 +379,30 @@ void pqPipelineSource::setDefaultPropertyValues()
       }
     diter->Delete();
     }
+
+  this->createAnimationHelpersIfNeeded();
+}
+
+//-----------------------------------------------------------------------------
+void pqPipelineSource::createAnimationHelpersIfNeeded()
+{
+  QList<vtkSMProxy*> helpers = this->getHelperProxies("RepresentationAnimationHelper");
+  if (helpers.size() == 0)
+    {
+    // Create animation helper which assists in animating display properties.
+    vtkSMProxyManager* pxm = vtkSMProxyManager::GetProxyManager();
+    int numPorts = this->getNumberOfOutputPorts();
+    for (int cc=0; cc < numPorts; cc++)
+      {
+      vtkSMProxy* helper = pxm->NewProxy("misc", "RepresentationAnimationHelper");
+      helper->SetConnectionID(this->getProxy()->GetConnectionID());
+      helper->UpdateVTKObjects();
+      vtkSMPropertyHelper(helper, "Source").Add(this->getProxy());
+      helper->UpdateVTKObjects();
+      this->addHelperProxy("RepresentationAnimationHelper", helper);
+      helper->Delete();
+      }
+    }
 }
 
 //-----------------------------------------------------------------------------
@@ -383,6 +418,19 @@ pqOutputPort* pqPipelineSource::getOutputPort(int outputport) const
   return this->Internal->OutputPorts[outputport];
 }
 
+//-----------------------------------------------------------------------------
+pqOutputPort* pqPipelineSource::getOutputPort(const QString& name) const
+{
+  vtkSMSourceProxy* source = vtkSMSourceProxy::SafeDownCast(this->getProxy());
+  unsigned int index = source->GetOutputPortIndex(name.toAscii().data());
+  if (index != VTK_UNSIGNED_INT_MAX)
+    {
+    return this->getOutputPort(static_cast<int>(index));
+    }
+
+  return 0;
+}
+
 //-----------------------------------------------------------------------------
 int pqPipelineSource::getNumberOfConsumers(int outputport) const
 {
@@ -408,6 +456,24 @@ pqPipelineSource *pqPipelineSource::getConsumer(int outputport, int index) const
   return this->Internal->OutputPorts[outputport]->getConsumer(index);
 }
 
+//-----------------------------------------------------------------------------
+QList<pqPipelineSource*> pqPipelineSource::getAllConsumers() const
+{
+  QList<pqPipelineSource*> consumers;
+  foreach (pqOutputPort* port, this->Internal->OutputPorts)
+    {
+    QList<pqPipelineSource*> portConsumers = port->getConsumers();
+    for (int cc=0; cc < portConsumers.size(); cc++)
+      {
+      if (!consumers.contains(portConsumers[cc]))
+        {
+        consumers.push_back(portConsumers[cc]);
+        }
+      }
+    }
+  return consumers;
+}
+
 //-----------------------------------------------------------------------------
 void pqPipelineSource::onRepresentationVisibilityChanged()
 {
