@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqPlotView.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -63,6 +63,8 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqChartMouseSelection.h"
 #include "pqChartTitle.h"
 #include "pqChartWidget.h"
+#include "pqEventDispatcher.h"
+#include "pqImageUtil.h"
 #include "pqLineChartRepresentation.h"
 #include "pqOutputPort.h"
 #include "pqPipelineSource.h"
@@ -71,8 +73,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqRepresentation.h"
 #include "pqServer.h"
 #include "pqSMAdaptor.h"
-#include "pqEventDispatcher.h"
-
 
 class pqPlotViewInternal
 {
@@ -150,15 +150,9 @@ pqPlotView::pqPlotView(const QString& type,
 {
   this->Internal = new pqPlotViewInternal();
 
-  // Create the chart widget.
+  // Create the chart widget and get the chart area.
   this->Internal->Chart = new pqChartWidget();
-
-  // Get the chart area and set up the axes.
   pqChartArea *chartArea = this->Internal->Chart->getChartArea();
-  chartArea->createAxis(pqChartAxis::Left);
-  chartArea->createAxis(pqChartAxis::Bottom);
-  chartArea->createAxis(pqChartAxis::Right);
-  chartArea->createAxis(pqChartAxis::Top);
 
   // Set up the chart legend.
   this->Internal->LegendModel = new pqChartLegendModel(this);
@@ -389,19 +383,31 @@ void pqPlotView::forceRender()
 void pqPlotView::renderInternal()
 {
   this->Internal->RenderRequestPending = false;
-  if(this->Internal->Histogram)
+
+  vtkSMProxy *proxy = this->getProxy();
+  if (this->Internal->Histogram)
     {
     this->Internal->Histogram->update();
     }
 
-  if(this->Internal->LineChart)
+  if (this->Internal->LineChart)
     {
-    this->Internal->LineChart->update();
+    pqLineChartSeries::SequenceType type = 
+      static_cast<pqLineChartSeries::SequenceType>(
+        pqSMAdaptor::getElementProperty(proxy->GetProperty("Type")).toInt());
+    if (type != this->Internal->LineChart->getSequenceType())
+      {
+      this->Internal->LineChart->setSequenceType(type);
+      this->Internal->LineChart->update(true);
+      }
+    else
+      {
+      this->Internal->LineChart->update(false);
+      }
     }
 
   // Update the chart legend.
   QList<QVariant> values;
-  vtkSMProxy *proxy = this->getProxy();
   this->Internal->ShowLegend = pqSMAdaptor::getElementProperty(
       proxy->GetProperty("ShowLegend")).toInt() != 0;
   if((this->Internal->LegendModel->getNumberOfEntries() == 0 ||
@@ -438,45 +444,55 @@ void pqPlotView::renderInternal()
   this->updateAxisOptions();
 }
 
+//-----------------------------------------------------------------------------
+vtkImageData* pqPlotView::captureImage(const QSize& newsize)
+{
+  QWidget* plot_widget = this->getWidget();
+  QSize curSize = plot_widget->size();
+  if (newsize.isValid())
+    {
+    plot_widget->resize(newsize);
+    }
+  vtkImageData* img = this->captureImage(1);
+  if (newsize.isValid())
+    {
+    plot_widget->resize(curSize);
+    }
+  return img;
+}
+
 //-----------------------------------------------------------------------------
 vtkImageData* pqPlotView::captureImage(int magnification)
 {
+  QWidget* plot_widget = this->getWidget();
+  QSize curSize = plot_widget->size();
+  QSize newSize = curSize*magnification;
+  if (magnification > 1)
+    {
+    // Magnify.
+    plot_widget->resize(newSize);
+    }
+
   // vtkSMRenderViewProxy::CaptureWindow() ensures that render is called on the
   // view. Hence, we must explicitly call render here to be consistent.
   this->forceRender();
-  
+
   // Since charts don't update the view immediatetly, we need to process all the
   // Qt::QueuedConnection slots which render the plots before capturing the
   // image.
   pqEventDispatcher::processEventsAndWait(0);
 
-  QPixmap grabbedPixMap = QPixmap::grabWidget(this->getWidget());
-  grabbedPixMap = grabbedPixMap.scaled(grabbedPixMap.size().width()*magnification,
-    grabbedPixMap.size().height()*magnification);
+  QPixmap grabbedPixMap = QPixmap::grabWidget(plot_widget);
+
+  if (magnification > 1)
+    {
+    // Restore size.
+    plot_widget->resize(curSize);
+    }
 
   // Now we need to convert this pixmap to vtkImageData.
-  QImage image = grabbedPixMap.toImage();
-
   vtkImageData* vtkimage = vtkImageData::New();
-  vtkimage->SetScalarTypeToUnsignedChar();
-  vtkimage->SetNumberOfScalarComponents(3);
-  vtkimage->SetDimensions(image.size().width(), image.size().height(), 1);
-  vtkimage->AllocateScalars();
-
-  QSize imgSize = image.size();
-
-  unsigned char* data = static_cast<unsigned char*>(vtkimage->GetScalarPointer());
-  for (int y=0; y < imgSize.height(); y++)
-    {
-    int index=(imgSize.height()-y-1) * imgSize.width()*3;
-    for (int x=0; x< imgSize.width(); x++)
-      {
-      QRgb color = image.pixel(x, y);
-      data[index++] = qRed(color);
-      data[index++] = qGreen(color);
-      data[index++] = qBlue(color);
-      }
-    }
+  pqImageUtil::toImageData(grabbedPixMap.toImage(), vtkimage);
 
   // Update image extents based on window position.
   int *position = this->getViewProxy()->GetViewPosition();
@@ -487,17 +503,18 @@ vtkImageData* pqPlotView::captureImage(int magnification)
     extents[cc] += position[cc/2]*magnification;
     }
   vtkimage->SetExtent(extents);
-
   return vtkimage;
 }
 
 //-----------------------------------------------------------------------------
-bool pqPlotView::saveImage(int width, int height, 
-    const QString& filename)
+bool pqPlotView::saveImage(int width, int height, const QString& filename)
 {
+  QWidget* plot_widget = this->getWidget();
+  QSize curSize;
   if (width != 0 && height != 0)
     {
-    this->getWidget()->resize(width, height);
+    curSize = plot_widget->size();
+    plot_widget->resize(width, height);
     }
 
   if (QFileInfo(filename).suffix().toLower() == "pdf")
@@ -527,12 +544,24 @@ bool pqPlotView::saveImage(int width, int height,
       }
     else
       {
+      if (curSize.isValid())
+        {
+        plot_widget->resize(curSize);
+        }
       return false;
       }
+    if (curSize.isValid())
+      {
+      plot_widget->resize(curSize);
+      }
     return true;
     }
 
-  QPixmap grabbedPixMap = QPixmap::grabWidget(this->getWidget());
+  QPixmap grabbedPixMap = QPixmap::grabWidget(plot_widget);
+  if (curSize.isValid())
+    {
+    plot_widget->resize(curSize);
+    }
   return grabbedPixMap.save(filename);
 }
 
@@ -591,7 +620,7 @@ bool pqPlotView::canDisplay(pqOutputPort* opPort) const
   if(!opPort|| !source ||
      opPort->getServer()->GetConnectionID() !=
      this->getServer()->GetConnectionID() || !sourceProxy ||
-     sourceProxy->GetNumberOfParts()==0)
+     sourceProxy->GetOutputPortsCreated()==0)
     {
     return false;
     }
@@ -718,6 +747,7 @@ void pqPlotView::updateTitles()
     this->Internal->Chart->setTitle(this->Internal->Title);
     }
 
+  emit this->beginSetTitleText(this, titleText);
   this->Internal->Title->setText(titleText);
   QList<QVariant> values = pqSMAdaptor::getMultipleElementProperty(
       proxy->GetProperty("ChartTitleFont"));
@@ -784,6 +814,7 @@ void pqPlotView::updateTitles()
       this->Internal->Chart->setAxisTitle(axes[i], this->Internal->AxisTitles[i]);
       }
 
+    emit this->beginSetTitleText(this, titleText);
     this->Internal->AxisTitles[i]->setText(titleText);
     }
 
