@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqPlotViewLineChart.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -103,10 +103,12 @@ public:
   pqPlotViewLineChartInternal();
   ~pqPlotViewLineChartInternal() {}
 
-  QPointer<pqLineChart> Layer[4];
-  pqLineChartModel *Model[4];
+  QPointer<pqLineChart> Layer;
+  pqLineChartModel *Model;
   QMap<vtkSMProxy *, pqPlotViewLineChartItem *> Representations;
   pqChartLegendModel *Legend;
+  pqLineChartSeries::ChartAxes IndexAxes[4];
+  pqLineChartSeries::SequenceType Type;
 };
 
 
@@ -167,15 +169,16 @@ bool pqPlotViewLineChartItem::setDataType(int dataType)
 pqPlotViewLineChartInternal::pqPlotViewLineChartInternal()
   : Representations()
 {
-  this->Layer[pqPlotViewLineChart::BottomLeft] = 0;
-  this->Layer[pqPlotViewLineChart::BottomRight] = 0;
-  this->Layer[pqPlotViewLineChart::TopLeft] = 0;
-  this->Layer[pqPlotViewLineChart::TopRight] = 0;
-  this->Model[pqPlotViewLineChart::BottomLeft] = 0;
-  this->Model[pqPlotViewLineChart::BottomRight] = 0;
-  this->Model[pqPlotViewLineChart::TopLeft] = 0;
-  this->Model[pqPlotViewLineChart::TopRight] = 0;
+  this->Layer = 0;
+  this->Model = 0;
   this->Legend = 0;
+  this->Type = pqLineChartSeries::Line;
+
+  // Set up the index to series chart axes map.
+  this->IndexAxes[0] = pqLineChartSeries::BottomLeft;
+  this->IndexAxes[1] = pqLineChartSeries::BottomRight;
+  this->IndexAxes[2] = pqLineChartSeries::TopLeft;
+  this->IndexAxes[3] = pqLineChartSeries::TopRight;
 }
 
 
@@ -205,10 +208,23 @@ pqPlotViewLineChart::~pqPlotViewLineChart()
   delete this->Internal;
 }
 
+//----------------------------------------------------------------------------
+void pqPlotViewLineChart::setSequenceType(pqLineChartSeries::SequenceType type)
+{
+  this->Internal->Type = type;
+}
+
+//----------------------------------------------------------------------------
+pqLineChartSeries::SequenceType pqPlotViewLineChart::getSequenceType() const
+{
+  return this->Internal->Type;
+}
+
+//----------------------------------------------------------------------------
 void pqPlotViewLineChart::initialize(pqChartArea *chartArea,
     pqChartLegendModel *legend)
 {
-  if(this->Internal->Model[pqPlotViewLineChart::BottomLeft])
+  if(this->Internal->Model)
     {
     return;
     }
@@ -216,57 +232,22 @@ void pqPlotViewLineChart::initialize(pqChartArea *chartArea,
   // Save the legend model for later.
   this->Internal->Legend = legend;
 
-  // Add a line chart layer for each of the possible axes
-  // configurations. Add the layers in reverse order to keep the
-  // bottom-left chart on top.
-  int i = 3;
-  for( ; i >= 0; i--)
-    {
-    this->Internal->Layer[i] = new pqLineChart(chartArea);
-    if(i == pqPlotViewLineChart::BottomLeft)
-      {
-      this->Internal->Layer[i]->setAxes(
-          chartArea->getAxis(pqChartAxis::Bottom),
-          chartArea->getAxis(pqChartAxis::Left));
-      }
-    else if(i == pqPlotViewLineChart::BottomRight)
-      {
-      this->Internal->Layer[i]->setAxes(
-          chartArea->getAxis(pqChartAxis::Bottom),
-          chartArea->getAxis(pqChartAxis::Right));
-      }
-    else if(i == pqPlotViewLineChart::TopLeft)
-      {
-      this->Internal->Layer[i]->setAxes(chartArea->getAxis(pqChartAxis::Top),
-          chartArea->getAxis(pqChartAxis::Left));
-      }
-    else if(i == pqPlotViewLineChart::TopRight)
-      {
-      this->Internal->Layer[i]->setAxes(chartArea->getAxis(pqChartAxis::Top),
-          chartArea->getAxis(pqChartAxis::Right));
-      }
+  // Add a line chart layer to the chart.
+  this->Internal->Layer = new pqLineChart(chartArea);
+  this->Internal->Model = new pqLineChartModel(this);
+  this->Internal->Layer->setModel(this->Internal->Model);
+  chartArea->addLayer(this->Internal->Layer);
 
-    this->Internal->Model[i] = new pqLineChartModel(this);
-    this->Internal->Layer[i]->setModel(
-        this->Internal->Model[i]);
-    chartArea->addLayer(this->Internal->Layer[i]);
-    }
-
-  // Set up the color options for the line charts. They should all
-  // share the same series color manager.
+  // Set up the color options for the line charts.
   pqChartSeriesColorManager *manager =
-      this->Internal->Layer[0]->getOptions()->getSeriesColorManager();
+      this->Internal->Layer->getOptions()->getSeriesColorManager();
   manager->getGenerator()->setColorScheme(
       pqChartSeriesOptionsGenerator::Spectrum);
-  for(i = 1; i < 4; i++)
-    {
-    this->Internal->Layer[i]->getOptions()->setSeriesColorManager(manager);
-    }
 }
 
 void pqPlotViewLineChart::update(bool force)
 {
-  if(!this->Internal->Model[pqPlotViewLineChart::BottomLeft])
+  if(!this->Internal->Model)
     {
     return;
     }
@@ -297,7 +278,8 @@ void pqPlotViewLineChart::update(bool force)
       }
 
     // First, remove or update the current model series.
-    QStringList displayNames;
+    QMap<QString, QList<int> > displayList;
+    QMap<QString, QList<int> >::Iterator kter;
     QList<pqPlotViewLineChartSeries>::Iterator series =
         (*jter)->Series.begin();
     while(series != (*jter)->Series.end())
@@ -305,7 +287,7 @@ void pqPlotViewLineChart::update(bool force)
       // Remove the series if the data type has changed, the display
       // is not visible, or the series is not enabled.
       int index = (*jter)->Representation->getSeriesIndex(
-          series->RepresentationName);
+          series->RepresentationName, series->YComponent);
       if(typeChanged || !isVisible ||
           !(*jter)->Representation->isSeriesEnabled(index))
         {
@@ -317,16 +299,14 @@ void pqPlotViewLineChart::update(bool force)
           series->LegendId = 0;
           }
 
-        this->Internal->Model[series->Chart]->removeSeries(series->Model);
+        this->Internal->Model->removeSeries(series->Model);
         delete series->Model;
         series = (*jter)->Series.erase(series);
         }
       else
         {
         yArray = 0;
-        int component = (*jter)->Representation->getSeriesComponent(index);
-        bool componentChanged = component != series->YComponent;
-        if(arrayChanged || componentChanged)
+        if(arrayChanged)
           {
           yArray = (*jter)->Representation->getYArray(index);
           if(!yArray)
@@ -345,7 +325,7 @@ void pqPlotViewLineChart::update(bool force)
               }
 
             // Remove the series if the x or y array are null.
-            this->Internal->Model[series->Chart]->removeSeries(series->Model);
+            this->Internal->Model->removeSeries(series->Model);
             delete series->Model;
             series = (*jter)->Series.erase(series);
             continue;
@@ -354,25 +334,30 @@ void pqPlotViewLineChart::update(bool force)
 
         // Move the series if needed.
         int axesIndex = (*jter)->Representation->getSeriesAxesIndex(index);
-        if(axesIndex != series->Chart)
+        if(axesIndex < 0 || axesIndex >= 4)
           {
-          this->Internal->Model[series->Chart]->removeSeries(series->Model);
-          series->Chart = axesIndex;
-          this->Internal->Model[series->Chart]->appendSeries(series->Model);
+          axesIndex = 0;
           }
 
-        if(arrayChanged || componentChanged)
+        if(axesIndex != series->Chart)
+          {
+          series->Chart = axesIndex;
+          series->Model->setChartAxes(this->Internal->IndexAxes[series->Chart]);
+          }
+
+        if(arrayChanged)
           {
           // Update the arrays for the series.
-          series->YComponent = component;
+          series->YComponent =
+              (*jter)->Representation->getSeriesComponent(index);
           series->Model->setDataArrays(xArray, yArray, maskArray, 0,
               series->YComponent);
           }
 
         // Update the options for the series.
         pqLineChartSeriesOptions *options =
-            this->Internal->Layer[series->Chart]->getOptions()->getSeriesOptions(
-            this->Internal->Model[series->Chart]->getIndexOf(series->Model));
+            this->Internal->Layer->getOptions()->getSeriesOptions(
+            this->Internal->Model->getIndexOf(series->Model));
         QPen seriesPen;
         options->getPen(seriesPen);
         QColor color;
@@ -412,7 +397,22 @@ void pqPlotViewLineChart::update(bool force)
           series->LegendId = 0;
           }
 
-        displayNames.append(series->RepresentationName);
+        // Add the array name to the list.
+        kter = displayList.find(series->RepresentationName);
+        if(kter != displayList.end())
+          {
+          kter->append(series->YComponent);
+          }
+        else
+          {
+          QList<int> components;
+          components.append(series->YComponent);
+          displayList.insert(series->RepresentationName, components);
+          }
+
+        // Set the series type.
+        series->Model->setSequenceType(this->Internal->Type); 
+
         ++series;
         }
       }
@@ -429,9 +429,14 @@ void pqPlotViewLineChart::update(bool force)
           {
           QString name;
           (*jter)->Representation->getSeriesName(i, name);
-          if(displayNames.contains(name))
+          int yComponent = (*jter)->Representation->getSeriesComponent(i);
+          kter = displayList.find(name);
+          if(kter != displayList.end())
             {
-            continue;
+            if(kter->contains(yComponent))
+              {
+              continue;
+              }
             }
 
           yArray = (*jter)->Representation->getYArray(i);
@@ -471,19 +476,28 @@ void pqPlotViewLineChart::update(bool force)
             }
 
           // Set the model arrays.
-          plot->YComponent = (*jter)->Representation->getSeriesComponent(i);
+          plot->YComponent = yComponent;
           plot->Model->setDataArrays(xArray, yArray, maskArray, 0,
               plot->YComponent);
 
           // Add the line chart series to the line chart model.
           plot->Chart = (*jter)->Representation->getSeriesAxesIndex(i);
-          int index = this->Internal->Model[plot->Chart]->getNumberOfSeries();
-          this->Internal->Model[plot->Chart]->appendSeries(plot->Model);
+          if(plot->Chart < 0 || plot->Chart >= 3)
+            {
+            plot->Chart = 0;
+            }
+
+          plot->Model->setChartAxes(this->Internal->IndexAxes[plot->Chart]);
+          int index = this->Internal->Model->getNumberOfSeries();
+          this->Internal->Model->appendSeries(plot->Model);
+
+          // Set the series type.
+          plot->Model->setSequenceType(this->Internal->Type); 
 
           // Update the series options.
           bool changedOptions = false;
           pqLineChartSeriesOptions *options =
-              this->Internal->Layer[plot->Chart]->getOptions()->getSeriesOptions(index);
+              this->Internal->Layer->getOptions()->getSeriesOptions(index);
           QPen seriesPen;
           options->getPen(seriesPen);
           if((*jter)->Representation->isSeriesColorSet(i))
@@ -573,7 +587,7 @@ void pqPlotViewLineChart::removeRepresentation(
         series->LegendId = 0;
         }
 
-      this->Internal->Model[series->Chart]->removeSeries(series->Model);
+      this->Internal->Model->removeSeries(series->Model);
       delete series->Model;
       }
 
@@ -583,11 +597,7 @@ void pqPlotViewLineChart::removeRepresentation(
 
 void pqPlotViewLineChart::removeAllRepresentations()
 {
-  for(int i = 0; i < 4; i++)
-    {
-    this->Internal->Model[i]->removeAll();
-    }
-
+  this->Internal->Model->removeAll();
   QMap<vtkSMProxy *, pqPlotViewLineChartItem *>::Iterator iter =
       this->Internal->Representations.begin();
   for( ; iter != this->Internal->Representations.end(); ++iter)
