@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqPropertyLinks.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -66,8 +66,6 @@ public:
     {
     }
 
-  pqSMAdaptor::PropertyType Type;
-
   vtkSMProxy* Proxy;
   vtkSMProperty* Property;
   int Index;
@@ -123,7 +121,6 @@ pqPropertyLinksConnection::pqPropertyLinksConnection(
   this->Internal->Index = idx;
   this->Internal->QtObject = qobject;
   this->Internal->QtProperty = qproperty;
-  this->Internal->Type = pqSMAdaptor::getPropertyType(this->Internal->Property);
   this->Internal->OutOfSync = false;
 }
 
@@ -177,7 +174,7 @@ void pqPropertyLinksConnection::smLinkedPropertyChanged()
     QVariant old;
     old = this->Internal->QtObject->property(this->Internal->QtProperty);
     QVariant prop;
-    switch(this->Internal->Type)
+    switch(pqSMAdaptor::getPropertyType(this->Internal->Property))
       {
     case pqSMAdaptor::PROXY:
     case pqSMAdaptor::PROXYSELECTION:
@@ -250,6 +247,7 @@ void pqPropertyLinksConnection::smLinkedPropertyChanged()
         }
       break;
     case pqSMAdaptor::MULTIPLE_ELEMENTS:
+    case pqSMAdaptor::COMPOSITE_TREE:
         {
         if(this->Internal->Index == -1)
           {
@@ -320,7 +318,7 @@ void pqPropertyLinksConnection::qtLinkedPropertyChanged()
     // get the property of the object
     QVariant prop;
     prop = this->Internal->QtObject->property(this->Internal->QtProperty);
-    switch(this->Internal->Type)
+    switch(pqSMAdaptor::getPropertyType(this->Internal->Property))
       {
     case pqSMAdaptor::PROXY:
     case pqSMAdaptor::PROXYSELECTION:
@@ -442,6 +440,7 @@ void pqPropertyLinksConnection::qtLinkedPropertyChanged()
         }
       break;
     case pqSMAdaptor::MULTIPLE_ELEMENTS:
+    case pqSMAdaptor::COMPOSITE_TREE:
         {
         if(this->Internal->Index == -1)
           {
