@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqSMAdaptor.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -67,7 +67,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "vtkSMVectorProperty.h"
 #include "vtkSMExtentDomain.h"
 #include "vtkSMFileListDomain.h"
-
+#include "vtkSMCompositeTreeDomain.h"
 // ParaView includes
 #include "pqSMProxy.h"
 
@@ -119,6 +119,7 @@ pqSMAdaptor::PropertyType pqSMAdaptor::getPropertyType(vtkSMProperty* Property)
     vtkSMProxyGroupDomain* proxyGroupDomain = NULL;
     vtkSMFileListDomain* fileListDomain = NULL;
     vtkSMStringListDomain* stringListDomain = NULL;
+    vtkSMCompositeTreeDomain* compositeTreeDomain = NULL;
     
     vtkSMDomainIterator* iter = Property->NewDomainIterator();
     for(iter->Begin(); !iter->IsAtEnd(); iter->Next())
@@ -147,6 +148,10 @@ pqSMAdaptor::PropertyType pqSMAdaptor::getPropertyType(vtkSMProperty* Property)
         {
         stringListDomain = vtkSMStringListDomain::SafeDownCast(iter->GetDomain());
         }
+      if (!compositeTreeDomain)
+        {
+        compositeTreeDomain = vtkSMCompositeTreeDomain::SafeDownCast(iter->GetDomain());
+        }
       }
     iter->Delete();
 
@@ -154,6 +159,10 @@ pqSMAdaptor::PropertyType pqSMAdaptor::getPropertyType(vtkSMProperty* Property)
       {
       type = pqSMAdaptor::FILE_LIST;
       }
+    else if (compositeTreeDomain)
+      {
+      type = pqSMAdaptor::COMPOSITE_TREE;
+      }
     else if(stringListRangeDomain || 
       (VectorProperty && VectorProperty->GetRepeatCommand() && 
        (stringListDomain || enumerationDomain)))
@@ -464,6 +473,25 @@ QList<QVariant> pqSMAdaptor::getSelectionProperty(vtkSMProperty* Property,
             }
           }
         }
+
+      vtkSMStringVectorProperty* infoSP = vtkSMStringVectorProperty::SafeDownCast(
+        StringProperty->GetInformationProperty());
+      if (!value.isValid() && infoSP)
+        {
+        // check if the information property is giving us the status for the
+        // array selection.
+
+        numElements = infoSP->GetNumberOfElements();
+        for(int i=0; (i+1)<numElements; i+=2)
+          {
+          if(StringName == infoSP->GetElement(i))
+            {
+            value = infoSP->GetElement(i+1);
+            break;
+            }
+          }
+        }
+
       // make up a zero
       if(!value.isValid())
         {
@@ -1878,6 +1906,7 @@ void pqSMAdaptor::setUncheckedFieldSelectionScalar(vtkSMProperty* prop,
     }
 }
 
+//-----------------------------------------------------------------------------
 QList<QString> pqSMAdaptor::getFieldSelectionScalarDomain(vtkSMProperty* prop)
 {
   QList<QString> types;
@@ -1902,6 +1931,32 @@ QList<QString> pqSMAdaptor::getFieldSelectionScalarDomain(vtkSMProperty* prop)
   return types;
 }
 
+//-----------------------------------------------------------------------------
+QList<QPair<QString, bool> > pqSMAdaptor::getFieldSelectionScalarDomainWithPartialArrays(
+  vtkSMProperty* prop)
+{
+  QList<QPair<QString, bool> > types;
+  if (!prop)
+    {
+    return types;
+    }
+
+  vtkSMStringVectorProperty* Property =
+    vtkSMStringVectorProperty::SafeDownCast(prop);
+  vtkSMArrayListDomain* domain = prop ?
+    vtkSMArrayListDomain::SafeDownCast(prop->GetDomain("array_list")) : 0;
+  
+  if(Property && domain)
+    {
+    int numEntries = domain->GetNumberOfStrings();
+    for(int i=0; i<numEntries; i++)
+      {
+      types.append(QPair<QString, bool>(domain->GetString(i), domain->IsArrayPartial(i)!=0));
+      }
+    }
+  return types;
+}
+
 //-----------------------------------------------------------------------------
 QList<QString> pqSMAdaptor::getDomainTypes(vtkSMProperty* property)
 {
