@@ -3,13 +3,13 @@
    Program: ParaView
    Module:    $RCSfile: pqSpreadSheetViewModel.cxx,v $
 
-   Copyright (c) 2005,2006 Sandia Corporation, Kitware Inc.
+   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.1. 
+   under the terms of the ParaView license version 1.2. 
 
-   See License_v1.1.txt for the full ParaView license.
+   See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
    Kitware Inc.
    28 Corporate Drive
@@ -32,12 +32,16 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqSpreadSheetViewModel.h"
 
 // Server Manager Includes.
+#include "vtkEventQtSlotConnect.h"
+#include "vtkIdList.h"
 #include "vtkIdTypeArray.h"
 #include "vtkIndexBasedBlockFilter.h"
 #include "vtkInformation.h"
+#include "vtkPVArrayInformation.h"
+#include "vtkPVCompositeDataInformation.h"
+#include "vtkPVCompositeDataInformationIterator.h"
 #include "vtkPVDataInformation.h"
 #include "vtkPVDataSetAttributesInformation.h"
-#include "vtkPVArrayInformation.h"
 #include "vtkSelection.h"
 #include "vtkSmartPointer.h"
 #include "vtkSMInputProperty.h"
@@ -45,6 +49,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "vtkSMSpreadSheetRepresentationProxy.h"
 #include "vtkStdString.h"
 #include "vtkTable.h"
+#include "vtkUnsignedIntArray.h"
 #include "vtkVariant.h"
 
 // Qt Includes.
@@ -56,10 +61,12 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 // ParaView Includes.
 #include "pqSMAdaptor.h"
 #include "pqDataRepresentation.h"
+#include "pqOutputPort.h"
+#include "pqPipelineSource.h"
 
-static uint qHash(QPair<vtkIdType, vtkIdType> pair)
+static uint qHash(pqSpreadSheetViewModel::vtkIndex index)
 {
-  return qHash(pair.second);
+  return qHash(index.Tuple[2]);
 }
 
 
@@ -70,7 +77,10 @@ public:
   SelectionModel(svmodel)
   {
   this->ActiveBlockNumber = 0;
+  this->Dirty = true;
+  this->VTKConnect = vtkSmartPointer<vtkEventQtSlotConnect>::New();
   }
+
   QPointer<pqDataRepresentation> DataRepresentation;
   vtkSmartPointer<vtkSMSpreadSheetRepresentationProxy> Representation;
   int NumberOfColumns;
@@ -119,6 +129,9 @@ public:
 
   QTimer SelectionTimer;
   QSet<vtkIdType> PendingSelectionBlocks;
+  vtkSmartPointer<vtkEventQtSlotConnect> VTKConnect;
+
+  bool Dirty;
 };
 
 //-----------------------------------------------------------------------------
@@ -143,12 +156,6 @@ pqSpreadSheetViewModel::~pqSpreadSheetViewModel()
   delete this->Internal;
 }
 
-//-----------------------------------------------------------------------------
-QItemSelectionModel* pqSpreadSheetViewModel::selectionModel() const
-{
-  return &this->Internal->SelectionModel;
-}
-
 //-----------------------------------------------------------------------------
 int pqSpreadSheetViewModel::rowCount(const QModelIndex&) const
 {
@@ -181,10 +188,25 @@ void pqSpreadSheetViewModel::setRepresentationProxy(
 {
   if (this->Internal->Representation.GetPointer() != repr)
     {
+    this->Internal->VTKConnect->Disconnect();
     this->Internal->Representation = repr;
+    this->Internal->Dirty = true;
+    if (repr)
+      {
+      // when repr updates, the view is dirty.
+      this->Internal->VTKConnect->Connect(repr, vtkCommand::UpdateDataEvent,
+        this, SLOT(markDirty()));
+      }
     }
 }
 
+//-----------------------------------------------------------------------------
+void pqSpreadSheetViewModel::markDirty()
+{
+  // cout << "markDirty" << endl;
+  this->Internal->Dirty = true;
+}
+
 //-----------------------------------------------------------------------------
 vtkSMSpreadSheetRepresentationProxy* pqSpreadSheetViewModel::
 getRepresentationProxy() const
@@ -202,9 +224,19 @@ int pqSpreadSheetViewModel::getFieldType() const
   return -1;
 }
 
+//-----------------------------------------------------------------------------
+void pqSpreadSheetViewModel::update()
+{
+  if (this->Internal->Dirty)
+    {
+    this->forceUpdate();
+    }
+}
+
 //-----------------------------------------------------------------------------
 void pqSpreadSheetViewModel::forceUpdate()
 {
+  this->Internal->Dirty = false;
   // Note that this method is called after the representation has already been
   // updated.
   int old_rows = this->Internal->NumberOfRows;
@@ -217,35 +249,7 @@ void pqSpreadSheetViewModel::forceUpdate()
     {
     vtkTable* table = vtkTable::SafeDownCast(
       repr->GetOutput(this->Internal->ActiveBlockNumber));
-    int field_type = this->Internal->getFieldType(); 
-
-    vtkSMInputProperty* ip = vtkSMInputProperty::SafeDownCast(
-      repr->GetProperty("Input"));
-    vtkSMSourceProxy* inputProxy = vtkSMSourceProxy::SafeDownCast(
-      ip->GetProxy(0));
-    int port = ip->GetOutputPortForConnection(0);
-
-    vtkPVDataInformation* info = inputProxy?
-      inputProxy->GetDataInformation(port) : 0;
-    if (info)
-      {
-      if (field_type == vtkIndexBasedBlockFilter::FIELD && 
-          info->GetFieldDataInformation())
-        {
-        // Since arrays in the field data can be variables lengths, find the
-        // length of the longest one:
-        this->Internal->NumberOfRows = 
-            info->GetFieldDataInformation()->GetMaximumNumberOfTuples();
-        }
-      else if (field_type == vtkIndexBasedBlockFilter::POINT)
-        {
-        this->Internal->NumberOfRows = info->GetNumberOfPoints();
-        }
-      else if (field_type == vtkIndexBasedBlockFilter::CELL)
-        {
-        this->Internal->NumberOfRows = info->GetNumberOfCells();
-        }
-      }
+    this->Internal->NumberOfRows = repr->GetMaximumNumberOfItems();
     this->Internal->NumberOfColumns = table? table->GetNumberOfColumns()  :0;
     if (this->Internal->NumberOfColumns == 0 && this->Internal->ActiveBlockNumber != 0)
       {
@@ -254,10 +258,20 @@ void pqSpreadSheetViewModel::forceUpdate()
       this->Internal->ActiveBlockNumber = 0;
       this->forceUpdate();
       }
+    
+    // When SelectionOnly is true, the delivered data has an extra
+    // "vtkOriginalIndices" column that needs to be hidden since it does not
+    // make any sense to the user.
+    if (this->Internal->NumberOfColumns && repr->GetSelectionOnly())
+      {
+      this->Internal->NumberOfColumns--;
+      }
     }
 
   this->Internal->SelectionModel.clear();
-  if (old_rows == this->Internal->NumberOfColumns &&
+  emit this->selectionChanged(this->Internal->SelectionModel.selection());
+
+  if (old_rows == this->Internal->NumberOfRows &&
     old_columns == this->Internal->NumberOfColumns)
     {
     this->Internal->SelectionTimer.start();
@@ -279,13 +293,25 @@ void pqSpreadSheetViewModel::updateSelectionForBlock(vtkIdType blockNumber)
   if (repr && 
     this->Internal->getFieldType() != vtkIndexBasedBlockFilter::FIELD)
     {
-    vtkSelection* selection = repr->GetSelectionOutput(blockNumber);
-    // This selection has information about ids that are currently selected.
-    // We now need to create a Qt selection list of indices for the items in
-    // the vtk selection.
-    QItemSelection qtSelection = this->convertToQtSelection(selection);;
-    this->Internal->SelectionModel.select(qtSelection, 
-      QItemSelectionModel::Select|QItemSelectionModel::Rows);
+    // If we are showing only the selected items, then there's not point in
+    // highlighting the selected items, since all items are selected. So we
+    // don't do any selection highlighting if SelectionOnly is true.
+    if (repr->GetSelectionOnly())
+      {
+      this->Internal->SelectionModel.clear();
+      }
+    else
+      {
+      vtkSelection* selection = repr->GetSelectionOutput(blockNumber);
+      // This selection has information about ids that are currently selected.
+      // We now need to create a Qt selection list of indices for the items in
+      // the vtk selection.
+      QItemSelection qtSelection = this->convertToQtSelection(selection);
+      this->Internal->SelectionModel.select(qtSelection,
+        QItemSelectionModel::Select|QItemSelectionModel::Rows|
+        QItemSelectionModel::Clear);
+      }
+    emit this->selectionChanged(this->Internal->SelectionModel.selection());
     }
 }
 
@@ -400,6 +426,11 @@ QVariant pqSpreadSheetViewModel::data(
       {
       vtkVariant value = table->GetValue(blockOffset, column);
       QString str = value.ToString().c_str();
+      if (value.IsChar() || value.IsUnsignedChar() || value.IsSignedChar())
+        {
+        // Don't show ASCII characted for char arrays.
+        str = QString::number(value.ToInt());
+        }
       str.replace(" ", "\t");
       return str;
       }
@@ -444,39 +475,146 @@ QVariant pqSpreadSheetViewModel::headerData (int section, Qt::Orientation orient
         title = (this->Internal->getFieldType() == vtkIndexBasedBlockFilter::POINT)?
           "Point ID" : "Cell ID";
         }
+      else if (title == "vtkOriginalCellIds" && repr->GetSelectionOnly())
+        {
+        title = "Cell ID";
+        }
+      else if (title == "vtkOriginalPointIds" && repr->GetSelectionOnly())
+        {
+        title = "Point ID";
+        }
+      else if (title == "vtkCompositeIndexArray")
+        {
+        title = "Block Number";
+        }
 
       return QVariant(title);
       }
     }
+  else if (orientation == Qt::Vertical && repr && role == Qt::DisplayRole)
+    {
+    // Row number to start from 0.
+    QVariant rowNo = this->Superclass::headerData(section, orientation, role);
+    return QVariant(rowNo.toUInt()-1);
+    }
 
   return this->Superclass::headerData(section, orientation, role);
 }
 
 //-----------------------------------------------------------------------------
-QModelIndex pqSpreadSheetViewModel::indexFor(int pid, vtkIdType vtkindex)
+QModelIndex pqSpreadSheetViewModel::indexFor(vtkSelection* vtkselection, vtkIdType vtkindex)
 {
+  vtkSMSpreadSheetRepresentationProxy* repr = 
+    this->Internal->Representation;
+
   // Find the qt index for a row with given process id and original id. 
   vtkTable* activeBlock = vtkTable::SafeDownCast(
     this->Internal->Representation->GetOutput(this->Internal->ActiveBlockNumber));
 
+  const char* column_name = "vtkOriginalIndices";
+  if (repr->GetSelectionOnly())
+    {
+    column_name = (this->Internal->getFieldType() == vtkIndexBasedBlockFilter::POINT)?
+      "vtkOriginalPointIds" : "vtkOriginalCellIds";
+    }
   vtkIdTypeArray* indexcolumn = vtkIdTypeArray::SafeDownCast(
-    activeBlock->GetColumnByName("vtkOriginalIndices"));
+    activeBlock->GetColumnByName(column_name));
+  if (!indexcolumn)
+    {
+    qDebug() << "indexcolumn missing" ;
+    return QModelIndex();
+    }
 
   vtkIdTypeArray* pidcolumn = vtkIdTypeArray::SafeDownCast(
     activeBlock->GetColumnByName("vtkOriginalProcessIds"));
 
-  for (vtkIdType cc=0; cc < indexcolumn->GetNumberOfTuples(); cc++)
+  vtkUnsignedIntArray* compositeIndexColumn = vtkUnsignedIntArray::SafeDownCast(
+    activeBlock->GetColumnByName("vtkCompositeIndexArray"));
+
+  // Get the list of vtkOriginalIndices that match the given vtkindex.
+  vtkIdList* ids = vtkIdList::New();
+  indexcolumn->LookupValue(vtkindex, ids);
+ 
+  if (vtkselection->GetProperties()->Has(vtkSelection::PROCESS_ID()) && pidcolumn)
     {
-    if (indexcolumn->GetValue(cc) == vtkindex)
+    int pid = vtkselection->GetProperties()->Get(vtkSelection::PROCESS_ID());
+    if (pid != -1)
       {
-      if (pid == -1 || !pidcolumn || pidcolumn->GetValue(cc) == pid)
+      // remove those ids from the "ids" list that don't have the same process ID
+      // as the selection.
+      for (vtkIdType cc=0; cc < ids->GetNumberOfIds();)
         {
-        return this->createIndex(this->Internal->computeRowIndex(cc), 0);
+        vtkIdType id = ids->GetId(static_cast<int>(cc));
+        if (pidcolumn->GetValue(id) != pid)
+          {
+          ids->DeleteId(id);
+          }
+        else
+          {
+          cc++;
+          }
         }
       }
     }
 
-  return QModelIndex();
+  if (vtkselection->GetProperties()->Has(vtkSelection::HIERARCHICAL_LEVEL()) &&
+      vtkselection->GetProperties()->Has(vtkSelection::HIERARCHICAL_INDEX()) &&
+      compositeIndexColumn && compositeIndexColumn->GetNumberOfComponents() == 2)
+    {
+    unsigned int hid = static_cast<unsigned int>(
+      vtkselection->GetProperties()->Get(vtkSelection::HIERARCHICAL_INDEX()));
+    unsigned int hlevel = static_cast<unsigned int>(
+      vtkselection->GetProperties()->Get(vtkSelection::HIERARCHICAL_LEVEL()));
+    // remove those ids from the "ids" list that don't have the same process ID
+    // as the selection.
+    for (vtkIdType cc=0; cc < ids->GetNumberOfIds();)
+      {
+      vtkIdType id = ids->GetId(static_cast<int>(cc));
+      unsigned int val[2];
+      compositeIndexColumn->GetTupleValue(id, val);
+      if (val[0] != hlevel || val[1] != hid)
+        {
+        ids->DeleteId(id);
+        }
+      else
+        {
+        cc++;
+        }
+      }
+    }
+  else if (vtkselection->GetProperties()->Has(vtkSelection::COMPOSITE_INDEX()) && 
+    compositeIndexColumn)
+    {
+    unsigned int cid = static_cast<unsigned int>(
+      vtkselection->GetProperties()->Get(vtkSelection::COMPOSITE_INDEX()));
+    // remove those ids from the "ids" list that don't have the same process ID
+    // as the selection.
+    for (vtkIdType cc=0; cc < ids->GetNumberOfIds();)
+      {
+      vtkIdType id = ids->GetId(static_cast<int>(cc));
+      if (compositeIndexColumn->GetValue(id) != cid)
+        {
+        ids->DeleteId(id);
+        }
+      else
+        {
+        cc++;
+        }
+      }
+    }
+
+  QModelIndex idx;
+  if (ids->GetNumberOfIds() > 0)
+    {
+    if (ids->GetNumberOfIds() > 1)
+      {
+      qCritical() << "Multiple ids match the same selection index. Probably a BUG.";
+      }
+    idx = this->createIndex(this->Internal->computeRowIndex(ids->GetId(0)), 0);
+    }
+
+  ids->Delete();
+  return idx;
 }
 
 //-----------------------------------------------------------------------------
@@ -503,15 +641,12 @@ QItemSelection pqSpreadSheetViewModel::convertToQtSelection(vtkSelection* vtksel
     // Iterate over all indices in the vtk selection, 
     // Determine the qt model index for each and then add that to the
     // qt selection.
-    int pid = vtkselection->GetProperties()->Has(vtkSelection::PROCESS_ID())?
-      vtkselection->GetProperties()->Get(vtkSelection::PROCESS_ID()) : -1;
     vtkIdTypeArray *indices = vtkIdTypeArray::SafeDownCast(
       vtkselection->GetSelectionList());
     for (vtkIdType cc=0; indices && cc < indices->GetNumberOfTuples(); cc++)
       {
       vtkIdType idx = indices->GetValue(cc);
-      // cout << "Selection (" << pid << ", " << index << ") " << endl;
-      QModelIndex qtIndex = this->indexFor(pid, idx);
+      QModelIndex qtIndex = this->indexFor(vtkselection, idx);
       if (qtIndex.isValid())
         {
         // cout << "Selecting: " << qtIndex.row() << endl;
@@ -520,36 +655,85 @@ QItemSelection pqSpreadSheetViewModel::convertToQtSelection(vtkSelection* vtksel
       }
     return qSel;
     }
+  else if (vtkselection->GetContentType() == vtkSelection::BLOCKS)
+    {
+    QItemSelection qSel;
+    vtkUnsignedIntArray* blocks = vtkUnsignedIntArray::SafeDownCast(
+      vtkselection->GetSelectionList());
+    if (blocks && blocks->GetNumberOfTuples() > 0)
+      {
+      qSel.select(this->createIndex(0, 0),
+        this->createIndex(this->rowCount()-1, 0));
+      }
+    return qSel;
+    }
   qCritical() << "Unknown selection object.";
   return QItemSelection();
 
 }
 
 //-----------------------------------------------------------------------------
-QSet<QPair<vtkIdType, vtkIdType> > pqSpreadSheetViewModel::getVTKIndices(
+QSet<pqSpreadSheetViewModel::vtkIndex> pqSpreadSheetViewModel::getVTKIndices(
   const QModelIndexList& indexes)
 {
-  QSet<QPair<vtkIdType, vtkIdType> > vtkindices;
+  // each variant in the vtkindices is 3 tuple
+  // - (-1, pid, id) or
+  // - (cid, pid, id) or
+  // - (hlevel, hindex, id)
+  QSet<vtkIndex> vtkindices;
 
   vtkSMSpreadSheetRepresentationProxy* repr =
     this->getRepresentationProxy();
-  if (repr)
+  if (!repr)
     {
-    foreach (QModelIndex idx, indexes)
-      {
-      int row = idx.row();
-      vtkIdType blockNumber = this->Internal->computeBlockNumber(row);
-      vtkIdType blockOffset = this->Internal->computeBlockOffset(row);
+    return vtkindices;
+    }
 
-      this->Internal->ActiveBlockNumber = blockNumber;
-      vtkTable* table = vtkTable::SafeDownCast(repr->GetOutput(blockNumber));
-      if (table)
+  foreach (QModelIndex idx, indexes)
+    {
+    int row = idx.row();
+    vtkIdType blockNumber = this->Internal->computeBlockNumber(row);
+    vtkIdType blockOffset = this->Internal->computeBlockOffset(row);
+
+    this->Internal->ActiveBlockNumber = blockNumber;
+    vtkTable* table = vtkTable::SafeDownCast(repr->GetOutput(blockNumber));
+    if (table)
+      {
+      vtkIndex value;
+
+      vtkVariant processId = table->GetValueByName(blockOffset, "vtkOriginalProcessIds");
+
+      const char* column_name = "vtkOriginalIndices";
+      if (repr->GetSelectionOnly())
         {
-        vtkVariant processId = table->GetValueByName(blockOffset, "vtkOriginalProcessIds");
-        vtkVariant vtkindex = table->GetValueByName(blockOffset, "vtkOriginalIndices");
-        int pid = processId.IsValid()? processId.ToInt() : 0;
-        vtkindices.insert(QPair<vtkIdType, vtkIdType>(pid, vtkindex.ToLong()));
+        column_name = (this->Internal->getFieldType() == vtkIndexBasedBlockFilter::POINT)?
+          "vtkOriginalPointIds" : "vtkOriginalCellIds";
         }
+
+      int pid = processId.IsValid()? processId.ToInt() : -1;
+      value.Tuple[1] = pid;
+
+      vtkUnsignedIntArray* compositeIndexColumn = vtkUnsignedIntArray::SafeDownCast( 
+        table->GetColumnByName("vtkCompositeIndexArray"));
+      if (compositeIndexColumn)
+        {
+        if (compositeIndexColumn->GetNumberOfComponents() == 2)
+          {
+          // using hierarchical indexing.
+          unsigned int val[3];
+          compositeIndexColumn->GetTupleValue(blockOffset, val);
+          value.Tuple[0] = static_cast<vtkIdType>(val[0]);
+          value.Tuple[1] = static_cast<vtkIdType>(val[1]);
+          }
+        else
+          {
+          value.Tuple[0] = compositeIndexColumn->GetValue(blockOffset);
+          }
+        }
+
+      vtkVariant vtkindex = table->GetValueByName(blockOffset, column_name);
+      value.Tuple[2] = static_cast<vtkIdType>(vtkindex.ToLongLong());
+      vtkindices.insert(value);
       }
     }
   return vtkindices;
@@ -614,3 +798,57 @@ bool pqSpreadSheetViewModel::isDataValid( const QModelIndex &idx) const
 
   return false;
 }
+
+//-----------------------------------------------------------------------------
+void pqSpreadSheetViewModel::resetCompositeDataSetIndex()
+{
+  vtkSMProxy* reprProxy = this->getRepresentationProxy();
+  int cur_index = pqSMAdaptor::getElementProperty(
+    reprProxy->GetProperty("CompositeDataSetIndex")).toInt();
+
+  pqOutputPort* input_port = this->getRepresentation()->getOutputPortFromInput();
+  vtkSMSourceProxy* inputProxy = vtkSMSourceProxy::SafeDownCast(
+    input_port->getSource()->getProxy());
+
+  vtkSMSourceProxy* extractSelection = inputProxy->GetSelectionOutput(
+    input_port->getPortNumber());
+  if (!extractSelection)
+    {
+    return;
+    }
+
+  vtkPVDataInformation* mbInfo = extractSelection->GetDataInformation();
+  if (!mbInfo || !mbInfo->GetCompositeDataClassName())
+    {
+    return;
+    }
+
+  vtkPVDataInformation* blockInfo = mbInfo->GetDataInformationForCompositeIndex(cur_index);
+  if (blockInfo && blockInfo->GetNumberOfPoints() > 0)
+    {
+    return;
+    }
+
+
+  // find first index with non-empty points.
+  vtkPVCompositeDataInformationIterator* iter = vtkPVCompositeDataInformationIterator::New();
+  iter->SetDataInformation(mbInfo);
+  for (iter->InitTraversal(); !iter->IsDoneWithTraversal(); iter->GoToNextItem())
+    {
+    vtkPVDataInformation* curInfo = iter->GetCurrentDataInformation();
+    if (!curInfo || curInfo->GetCompositeDataClassName() != 0)
+      {
+      continue;
+      }
+    if (curInfo->GetDataSetType() != -1 && curInfo->GetNumberOfPoints() > 0)
+      {
+      cur_index = static_cast<int>(iter->GetCurrentFlatIndex());
+      break;
+      }
+    }
+  iter->Delete();
+
+  pqSMAdaptor::setElementProperty(
+    reprProxy->GetProperty("CompositeDataSetIndex"), cur_index);
+  reprProxy->UpdateVTKObjects();
+}
