@@ -23,8 +23,10 @@
 #include "vtkCompositeDataIterator.h"
 #include "vtkCompositeDataSet.h"
 #include "vtkDataArray.h"
+#include "vtkDataObjectTypes.h"
 #include "vtkDataSet.h"
 #include "vtkGenericDataSet.h"
+#include "vtkGraph.h"
 #include "vtkImageData.h"
 #include "vtkInformation.h"
 #include "vtkMultiProcessController.h"
@@ -40,11 +42,12 @@
 #include "vtkStructuredGrid.h"
 #include "vtkTable.h"
 #include "vtkUniformGrid.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 
 #include <vtkstd/vector>
 
 vtkStandardNewMacro(vtkPVDataInformation);
-vtkCxxRevisionMacro(vtkPVDataInformation, "$Revision: 1.38 $");
+vtkCxxRevisionMacro(vtkPVDataInformation, "$Revision: 1.57 $");
 
 //----------------------------------------------------------------------------
 vtkPVDataInformation::vtkPVDataInformation()
@@ -63,16 +66,19 @@ vtkPVDataInformation::vtkPVDataInformation()
   this->PointDataInformation = vtkPVDataSetAttributesInformation::New();
   this->CellDataInformation = vtkPVDataSetAttributesInformation::New();
   this->FieldDataInformation = vtkPVDataSetAttributesInformation::New();
-
+  this->VertexDataInformation = vtkPVDataSetAttributesInformation::New();
+  this->EdgeDataInformation = vtkPVDataSetAttributesInformation::New();
+  this->RowDataInformation = vtkPVDataSetAttributesInformation::New();
   this->CompositeDataInformation = vtkPVCompositeDataInformation::New();
-
   this->PointArrayInformation = vtkPVArrayInformation::New();
-  
-  this->Name = 0;
+
   this->DataClassName = 0;
   this->CompositeDataClassName = 0;
   this->NumberOfDataSets = 0;
-  this->NameSetToDefault = 0;
+  this->TimeSpan[0] = VTK_DOUBLE_MAX;
+  this->TimeSpan[1] = -VTK_DOUBLE_MAX;
+  this->HasTime = 0;
+  this->Time = 0.0;
 }
 
 //----------------------------------------------------------------------------
@@ -84,12 +90,16 @@ vtkPVDataInformation::~vtkPVDataInformation()
   this->CellDataInformation = NULL;
   this->FieldDataInformation->Delete();
   this->FieldDataInformation = NULL;
+  this->VertexDataInformation->Delete();
+  this->VertexDataInformation = NULL;
+  this->EdgeDataInformation->Delete();
+  this->EdgeDataInformation = NULL;
+  this->RowDataInformation->Delete();
+  this->RowDataInformation = NULL;
   this->CompositeDataInformation->Delete();
   this->CompositeDataInformation = NULL;
   this->PointArrayInformation->Delete();
   this->PointArrayInformation = NULL;
-  
-  this->SetName(0);
   this->SetDataClassName(0);
   this->SetCompositeDataClassName(0);
 }
@@ -114,10 +124,17 @@ void vtkPVDataInformation::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "Extent: " << this->Extent[0] << ", " << this->Extent[1]
      << ", " << this->Extent[2] << ", " << this->Extent[3]
      << ", " << this->Extent[4] << ", " << this->Extent[5] << endl;
+
   os << indent << "PointDataInformation " << endl;
   this->PointDataInformation->PrintSelf(os, i2);
   os << indent << "CellDataInformation " << endl;
   this->CellDataInformation->PrintSelf(os, i2);
+  os << indent << "VertexDataInformation" << endl;
+  this->VertexDataInformation->PrintSelf(os, i2);
+  os << indent << "EdgeDataInformation" << endl;
+  this->EdgeDataInformation->PrintSelf(os, i2);
+  os << indent << "RowDataInformation" << endl;
+  this->RowDataInformation->PrintSelf(os, i2);
   os << indent << "FieldDataInformation " << endl;
   this->FieldDataInformation->PrintSelf(os, i2);
   os << indent << "CompositeDataInformation " << endl;
@@ -125,19 +142,14 @@ void vtkPVDataInformation::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "PointArrayInformation " << endl;
   this->PointArrayInformation->PrintSelf(os, i2);
 
-  if (this->Name)
-    {
-    os << indent << "Name: " << this->Name << endl;
-    }
-  else
-    {
-    os << indent << "Name: NULL\n";
-    }
-
-  os << indent << "DataClassName: " 
+  os << indent << "DataClassName: "
      << (this->DataClassName?this->DataClassName:"(none)") << endl;
-  os << indent << "CompositeDataClassName: " 
+  os << indent << "CompositeDataClassName: "
      << (this->CompositeDataClassName?this->CompositeDataClassName:"(none)") << endl;
+
+  os << indent << "TimeSpan: "
+     << this->TimeSpan[0] << ", " << this->TimeSpan[1]
+     << endl;
 }
 
 //----------------------------------------------------------------------------
@@ -157,17 +169,23 @@ void vtkPVDataInformation::Initialize()
   this->Extent[1] = this->Extent[3] = this->Extent[5] = -VTK_LARGE_INTEGER;
   this->PointDataInformation->Initialize();
   this->CellDataInformation->Initialize();
+  this->VertexDataInformation->Initialize();
+  this->EdgeDataInformation->Initialize();
+  this->RowDataInformation->Initialize();
   this->FieldDataInformation->Initialize();
   this->CompositeDataInformation->Initialize();
   this->PointArrayInformation->Initialize();
-  
-  this->SetName(0);
   this->SetDataClassName(0);
   this->SetCompositeDataClassName(0);
+  this->TimeSpan[0] = VTK_DOUBLE_MAX;
+  this->TimeSpan[1] = -VTK_DOUBLE_MAX;
+  this->HasTime = 0;
+  this->Time = 0.0;
 }
 
 //----------------------------------------------------------------------------
-void vtkPVDataInformation::DeepCopy(vtkPVDataInformation *dataInfo)
+void vtkPVDataInformation::DeepCopy(vtkPVDataInformation *dataInfo,
+  bool copyCompositeInformation/*=true*/)
 {
   int idx;
   double *bounds;
@@ -200,54 +218,111 @@ void vtkPVDataInformation::DeepCopy(vtkPVDataInformation *dataInfo)
   // Copy attribute information.
   this->PointDataInformation->DeepCopy(dataInfo->GetPointDataInformation());
   this->CellDataInformation->DeepCopy(dataInfo->GetCellDataInformation());
+  this->VertexDataInformation->DeepCopy(dataInfo->GetVertexDataInformation());
+  this->EdgeDataInformation->DeepCopy(dataInfo->GetEdgeDataInformation());
+  this->RowDataInformation->DeepCopy(dataInfo->GetRowDataInformation());
   this->FieldDataInformation->DeepCopy(dataInfo->GetFieldDataInformation());
-  this->CompositeDataInformation->AddInformation(
-    dataInfo->GetCompositeDataInformation());
+  if (copyCompositeInformation)
+    {
+    this->CompositeDataInformation->AddInformation(
+      dataInfo->GetCompositeDataInformation());
+    }
   this->PointArrayInformation->AddInformation(
     dataInfo->GetPointArrayInformation());
 
-  this->SetName(dataInfo->GetName());
+  double *timespan;
+  timespan = dataInfo->GetTimeSpan();
+  this->TimeSpan[0] = timespan[0];
+  this->TimeSpan[1] = timespan[1];
 }
 
 //----------------------------------------------------------------------------
-int vtkPVDataInformation::AddFromCompositeDataSet(vtkCompositeDataSet* data)
+void vtkPVDataInformation::AddFromMultiPieceDataSet(vtkCompositeDataSet* data)
 {
-  int numDataSets = 0;
   vtkCompositeDataIterator* iter = data->NewIterator();
-  iter->GoToFirstItem();
-  while (!iter->IsDoneWithTraversal())
+  for(iter->InitTraversal(); !iter->IsDoneWithTraversal(); iter->GoToNextItem())
     {
-    numDataSets++;
     vtkDataObject* dobj = iter->GetCurrentDataObject();
-    vtkPVDataInformation* dinf = vtkPVDataInformation::New();
-    dinf->CopyFromObject(dobj);
-    dinf->SetDataClassName(dobj->GetClassName());
-    dinf->DataSetType = dobj->GetDataObjectType();
-    this->AddInformation(dinf, 1);
-    dinf->Delete();
-
-    iter->GoToNextItem();
+    if (dobj)
+      {
+      vtkPVDataInformation* dinf = vtkPVDataInformation::New();
+      dinf->CopyFromObject(dobj);
+      dinf->SetDataClassName(dobj->GetClassName());
+      dinf->DataSetType = dobj->GetDataObjectType();
+      this->AddInformation(dinf, /*addingParts=*/ 1);
+      dinf->Delete();
+      }
     }
   iter->Delete();
-
-  return numDataSets;
 }
 
 //----------------------------------------------------------------------------
-void vtkPVDataInformation::CopyFromCompositeDataSet(vtkCompositeDataSet* data,
-                                                    int recurse)
+void vtkPVDataInformation::CopyFromCompositeDataSet(vtkCompositeDataSet* data)
 {
   this->Initialize();
+  this->CompositeDataInformation->CopyFromObject(data);
 
-  int numDataSets = this->AddFromCompositeDataSet(data);
-
-  if (recurse)
+  unsigned int numDataSets = this->CompositeDataInformation->GetNumberOfChildren();
+  if (this->CompositeDataInformation->GetDataIsMultiPiece())
     {
-    this->CompositeDataInformation->CopyFromObject(data);
+    // For vtkMultiPieceDataSet, the vtkPVCompositeDataInformation does not
+    // give us individual piece information, we collect that explicitly.
+    this->AddFromMultiPieceDataSet(data);
+    }
+  else
+    {
+    for (unsigned int cc=0; cc < numDataSets; cc++)
+      {
+      vtkPVDataInformation* childInfo =
+        this->CompositeDataInformation->GetDataInformation(cc);
+      if (childInfo)
+        {
+        this->AddInformation(childInfo, /*addingParts=*/ 1);
+        }
+      }
     }
   this->SetCompositeDataClassName(data->GetClassName());
   this->CompositeDataSetType = data->GetDataObjectType();
-  this->NumberOfDataSets = numDataSets;
+
+  if (this->DataSetType == -1)
+    {
+    // This is a composite dataset with no non-empty leaf node. Set some data
+    // type (Look at BUG #7144).
+    this->SetDataClassName("vtkDataSet");
+    this->DataSetType = VTK_DATA_SET;
+    }
+
+
+  // AddInformation should have updated NumberOfDataSets correctly to count
+  // number of non-zero datasets. We don't need to fix it here.
+  // this->NumberOfDataSets = numDataSets;
+}
+
+//----------------------------------------------------------------------------
+void vtkPVDataInformation::CopyCommonMetaData(vtkDataObject* data)
+{
+  // Gather some common stuff
+  vtkInformation *pinfo = data->GetPipelineInformation();
+  if (!pinfo)
+    {
+    return;
+    }
+
+  if (pinfo->Has(vtkStreamingDemandDrivenPipeline::TIME_RANGE()))
+    {
+    double *times = pinfo->Get(vtkStreamingDemandDrivenPipeline::TIME_RANGE());
+    this->TimeSpan[0] = times[0];
+    this->TimeSpan[1] = times[1];
+    }
+
+  vtkInformation *dinfo = data->GetInformation();
+  if (dinfo->Has(vtkDataObject::DATA_TIME_STEPS()) &&
+    dinfo->Length(vtkDataObject::DATA_TIME_STEPS()) == 1)
+    {
+    double time = dinfo->Get(vtkDataObject::DATA_TIME_STEPS())[0];
+    this->Time = time;
+    this->HasTime = 1;
+    }
 }
 
 //----------------------------------------------------------------------------
@@ -262,15 +337,6 @@ void vtkPVDataInformation::CopyFromDataSet(vtkDataSet* data)
 
   this->NumberOfDataSets = 1;
 
-  // Look for a name stored in Field Data.
-  vtkDataArray *nameArray = data->GetFieldData()->GetArray("Name");
-  if (nameArray)
-    {
-    char* str = static_cast<char*>(nameArray->GetVoidPointer(0));
-    this->SetName(str);
-    }
-
-
   switch ( this->DataSetType )
     {
     case VTK_IMAGE_DATA:
@@ -328,7 +394,7 @@ void vtkPVDataInformation::CopyFromDataSet(vtkDataSet* data)
     *tmpFile << "\t" << this->NumberOfPoints << " points" << endl;
     *tmpFile << "\t" << this->NumberOfCells << " cells" << endl;
     }
-  
+
   bds = data->GetBounds();
   for (idx = 0; idx < 6; ++idx)
     {
@@ -355,6 +421,7 @@ void vtkPVDataInformation::CopyFromDataSet(vtkDataSet* data)
     this->FieldDataInformation->CopyFromFieldData(fd);
     }
 }
+
 //----------------------------------------------------------------------------
 void vtkPVDataInformation::CopyFromGenericDataSet(vtkGenericDataSet *data)
 {
@@ -365,15 +432,6 @@ void vtkPVDataInformation::CopyFromGenericDataSet(vtkGenericDataSet *data)
   this->DataSetType = data->GetDataObjectType();
 
   this->NumberOfDataSets = 1;
-
-  // Look for a name stored in Field Data.
-  vtkDataArray *nameArray = data->GetFieldData()->GetArray("Name");
-  if (nameArray)
-    {
-    char* str = static_cast<char*>(nameArray->GetVoidPointer(0));
-    this->SetName(str);
-    }
-
   this->NumberOfPoints = data->GetNumberOfPoints();
   if (!this->NumberOfPoints)
     {
@@ -413,7 +471,7 @@ void vtkPVDataInformation::CopyFromSelection(vtkSelection* data)
   this->SetDataClassName(data->GetClassName());
   this->DataSetType = data->GetDataObjectType();
   this->NumberOfDataSets = 1;
-  this->Bounds[0] = this->Bounds[1] = this->Bounds[2] 
+  this->Bounds[0] = this->Bounds[1] = this->Bounds[2]
     = this->Bounds[3] = this->Bounds[4] = this->Bounds[5] = 0;
 
   this->MemorySize = data->GetActualMemorySize();
@@ -424,13 +482,34 @@ void vtkPVDataInformation::CopyFromSelection(vtkSelection* data)
   this->PointDataInformation->CopyFromFieldData(data->GetFieldData());
 }
 
+//----------------------------------------------------------------------------
+void vtkPVDataInformation::CopyFromGraph(vtkGraph* data)
+{
+  this->SetDataClassName(data->GetClassName());
+  this->DataSetType = data->GetDataObjectType();
+  this->NumberOfDataSets = 1;
+  this->Bounds[0] = this->Bounds[1] = this->Bounds[2]
+    = this->Bounds[3] = this->Bounds[4] = this->Bounds[5] = 0;
+
+  if(data->GetPoints())
+    data->GetPoints()->GetBounds(this->Bounds);
+
+  this->MemorySize = data->GetActualMemorySize();
+  this->NumberOfCells = data->GetNumberOfEdges();
+  this->NumberOfPoints = data->GetNumberOfVertices();
+  this->NumberOfRows = 0;
+
+  this->VertexDataInformation->CopyFromFieldData(data->GetVertexData());
+  this->EdgeDataInformation->CopyFromFieldData(data->GetEdgeData());
+}
+
 //----------------------------------------------------------------------------
 void vtkPVDataInformation::CopyFromTable(vtkTable* data)
 {
   this->SetDataClassName(data->GetClassName());
   this->DataSetType = data->GetDataObjectType();
   this->NumberOfDataSets = 1;
-  this->Bounds[0] = this->Bounds[1] = this->Bounds[2] 
+  this->Bounds[0] = this->Bounds[1] = this->Bounds[2]
     = this->Bounds[3] = this->Bounds[4] = this->Bounds[5] = 0;
 
   this->MemorySize = data->GetActualMemorySize();
@@ -438,8 +517,7 @@ void vtkPVDataInformation::CopyFromTable(vtkTable* data)
   this->NumberOfPoints = 0;
   this->NumberOfRows = data->GetNumberOfRows();
 
-  // Copy Point Data information
-  this->PointDataInformation->CopyFromFieldData(data->GetFieldData());
+  this->RowDataInformation->CopyFromFieldData(data->GetRowData());
 }
 
 //----------------------------------------------------------------------------
@@ -461,15 +539,22 @@ void vtkPVDataInformation::CopyFromObject(vtkObject* object)
 
   if (!dobj)
     {
-    vtkErrorMacro("Could not cast object to a known data set: " 
+    vtkErrorMacro("Could not cast object to a known data set: "
                   << (object?object->GetClassName():"(null"));
     return;
     }
 
+  if (strcmp(dobj->GetProducerPort()->GetProducer()->GetClassName(), "vtkPVNullSource") == 0)
+    {
+    // Don't gather any data information from the hypothetical null source.
+    return;
+    }
+
   vtkCompositeDataSet* cds = vtkCompositeDataSet::SafeDownCast(dobj);
   if (cds)
     {
     this->CopyFromCompositeDataSet(cds);
+    this->CopyCommonMetaData(dobj);
     return;
     }
 
@@ -477,6 +562,7 @@ void vtkPVDataInformation::CopyFromObject(vtkObject* object)
   if (ds)
     {
     this->CopyFromDataSet(ds);
+    this->CopyCommonMetaData(dobj);
     return;
     }
 
@@ -484,6 +570,15 @@ void vtkPVDataInformation::CopyFromObject(vtkObject* object)
   if (ads)
     {
     this->CopyFromGenericDataSet(ads);
+    this->CopyCommonMetaData(dobj);
+    return;
+    }
+
+  vtkGraph* graph = vtkGraph::SafeDownCast(dobj);
+  if( graph)
+    {
+    this->CopyFromGraph(graph);
+    this->CopyCommonMetaData(dobj);
     return;
     }
 
@@ -491,6 +586,7 @@ void vtkPVDataInformation::CopyFromObject(vtkObject* object)
   if (table)
     {
     this->CopyFromTable(table);
+    this->CopyCommonMetaData(dobj);
     return;
     }
 
@@ -498,12 +594,17 @@ void vtkPVDataInformation::CopyFromObject(vtkObject* object)
   if (selection)
     {
     this->CopyFromSelection(selection);
+    this->CopyCommonMetaData(dobj);
     return;
     }
 
-  vtkErrorMacro("Could not cast object to a known data set: " 
-                << (dobj?dobj->GetClassName():"(null"));
+  // Because custom applications may implement their own data
+  // object types, this isn't an error condition - just
+  // display the name of the data object and return quietly.
+  this->SetDataClassName(dobj->GetClassName());
+  this->CopyCommonMetaData(dobj);
 }
+
 //----------------------------------------------------------------------------
 void vtkPVDataInformation::AddInformation(vtkPVInformation* pvi)
 {
@@ -526,22 +627,25 @@ void vtkPVDataInformation::AddInformation(
     return;
     }
 
-  this->SetCompositeDataClassName(info->GetCompositeDataClassName());
-  this->CompositeDataSetType = info->CompositeDataSetType;
-
-  this->CompositeDataInformation->AddInformation(
-    info->CompositeDataInformation);
+  if (!addingParts)
+    {
+    this->SetCompositeDataClassName(info->GetCompositeDataClassName());
+    this->CompositeDataSetType = info->CompositeDataSetType;
+    this->CompositeDataInformation->AddInformation(
+      info->CompositeDataInformation);
+    }
 
   if (info->NumberOfDataSets == 0)
     {
     return;
     }
 
-  if (this->NumberOfPoints == 0 && 
-      this->NumberOfCells == 0 && 
+  if (this->NumberOfPoints == 0 &&
+      this->NumberOfCells == 0 &&
       this->NumberOfDataSets == 0)
-    { // Just copy the other array information.
-    this->DeepCopy(info);
+    {
+    // Just copy the other array information.
+    this->DeepCopy(info, !addingParts);
     return;
     }
 
@@ -588,7 +692,7 @@ void vtkPVDataInformation::AddInformation(
   this->NumberOfCells += info->GetNumberOfCells();
   this->MemorySize += info->GetMemorySize();
   this->NumberOfRows += info->GetNumberOfRows();
-  
+
   switch ( this->DataSetType )
     {
     case VTK_POLY_DATA:
@@ -600,7 +704,7 @@ void vtkPVDataInformation::AddInformation(
     // Adding data information of parts
     this->NumberOfDataSets += info->GetNumberOfDataSets();
     }
-  else 
+  else
     {
     // Adding data information of 1 part across processors
     if (this->GetCompositeDataClassName())
@@ -651,116 +755,28 @@ void vtkPVDataInformation::AddInformation(
   this->PointArrayInformation->AddInformation(info->GetPointArrayInformation());
   this->PointDataInformation->AddInformation(info->GetPointDataInformation());
   this->CellDataInformation->AddInformation(info->GetCellDataInformation());
+  this->VertexDataInformation->AddInformation(info->GetVertexDataInformation());
+  this->EdgeDataInformation->AddInformation(info->GetEdgeDataInformation());
+  this->RowDataInformation->AddInformation(info->GetRowDataInformation());
   this->FieldDataInformation->AddInformation(info->GetFieldDataInformation());
 //  this->GenericAttributesInformation->AddInformation(info->GetGenericAttributesInformation());
 
-  if (this->Name == NULL)
+  double *times = info->GetTimeSpan();
+  if (times[0] < this->TimeSpan[0])
     {
-    this->SetName(info->GetName());
+    this->TimeSpan[0] = times[0];
     }
-}
-
-//----------------------------------------------------------------------------
-void vtkPVDataInformation::SetName(const char* name)
-{
-  if (!this->Name && !name) { return;}
-  if ( this->Name && name && (!strcmp(this->Name, name))) 
-    { 
-    return;
-    }
-  if (this->Name) { delete [] this->Name; }
-  if (name) 
-    { 
-    this->Name = new char[strlen(name)+1];
-    strcpy(this->Name, name);
-    } 
-  else 
-    { 
-    this->Name = 0; 
-    } 
-  this->Modified(); 
-  this->NameSetToDefault = 0;
-}
-
-//----------------------------------------------------------------------------
-const char* vtkPVDataInformation::GetName()
-{
-  if (!this->Name || this->Name[0] == '\0' || this->NameSetToDefault)
+  if (times[1] > this->TimeSpan[1])
     {
-    if (this->Name)
-      {
-      delete[] this->Name;
-      }
-    char str[256];
-    if (this->GetDataSetType() == VTK_POLY_DATA)
-      {
-      long nc = this->GetNumberOfCells();
-      sprintf(str, "Polygonal: %ld cells", nc);
-      }
-    else if (this->GetDataSetType() == VTK_UNSTRUCTURED_GRID)
-      {
-      long nc = this->GetNumberOfCells();
-      sprintf(str, "Unstructured Grid: %ld cells", nc);
-      }
-    else if (this->GetDataSetType() == VTK_IMAGE_DATA)
-      {
-      int *ext = this->GetExtent();
-      if (ext)
-        {
-        sprintf(str, "Uniform Rectilinear: extent (%d, %d) (%d, %d) (%d, %d)", 
-                ext[0], ext[1], ext[2], ext[3], ext[4], ext[5]);
-        }
-      else
-        {
-        sprintf(str, "Uniform Rectilinear: invalid extent");
-        }
-      }
-    else if (this->GetDataSetType() == VTK_UNIFORM_GRID)
-      {
-      int *ext = this->GetExtent();
-      sprintf(str, 
-              "Uniform Rectilinear (with blanking): "
-              "extent (%d, %d) (%d, %d) (%d, %d)", 
-              ext[0], ext[1], ext[2], ext[3], ext[4], ext[5]);
-      }
-    else if (this->GetDataSetType() == VTK_RECTILINEAR_GRID)
-      {
-      int *ext = this->GetExtent();
-      if (ext)
-        {
-        sprintf(str, 
-                "Nonuniform Rectilinear: extent (%d, %d) (%d, %d) (%d, %d)", 
-                ext[0], ext[1], ext[2], ext[3], ext[4], ext[5]);
-        }
-      else
-        {
-        sprintf(str, "Nonuniform Rectilinear: invalid extent");
-        }
-      }
-    else if (this->GetDataSetType() == VTK_STRUCTURED_GRID)
-      {
-      int *ext = this->GetExtent();
-      if (ext)
-        {
-        sprintf(str, "Curvilinear: extent (%d, %d) (%d, %d) (%d, %d)", 
-                ext[0], ext[1], ext[2], ext[3], ext[4], ext[5]);
-        }
-      else
-        {
-        sprintf(str, "Curvilinear: invalid extent");
-        }
-      }
-    else
-      {
-      sprintf(str, "Part of unknown type");
-      }
-    this->Name = new char[256];
-    strncpy(this->Name, str, 256);
-    this->NameSetToDefault = 1;
+    this->TimeSpan[1] = times[1];
     }
-  return this->Name;
-}
 
+  if (!this->HasTime && info->GetHasTime())
+    {
+    this->Time = info->GetTime();
+    this->HasTime = 1;
+    }
+}
 
 //----------------------------------------------------------------------------
 const char* vtkPVDataInformation::GetPrettyDataTypeString()
@@ -817,63 +833,25 @@ const char* vtkPVDataInformation::GetPrettyDataTypeString()
       return "Graph";
     case VTK_TREE:
       return "Tree";
+    case VTK_SELECTION:
+      return "Selection";
+    case VTK_DIRECTED_GRAPH:
+      return "Directed Graph";
+    case VTK_UNDIRECTED_GRAPH:
+      return "Undirected Graph";
+    case VTK_MULTIPIECE_DATA_SET:
+      return "Multi-piece Dataset";
+    case VTK_DIRECTED_ACYCLIC_GRAPH:
+      return "Directed Acyclic Graph";
     }
-  
+
   return "UnknownType";
 }
 
 //----------------------------------------------------------------------------
 const char* vtkPVDataInformation::GetDataSetTypeAsString()
 {
-  switch(this->DataSetType)
-    {
-    case VTK_POLY_DATA:
-      return "vtkPolyData";
-    case VTK_STRUCTURED_POINTS:
-      return "vtkStructuredPoints";
-    case VTK_STRUCTURED_GRID:
-      return "vtkStructuredGrid";
-    case VTK_RECTILINEAR_GRID:
-      return "vtkRectilinearGrid";
-    case VTK_UNSTRUCTURED_GRID:
-      return "vtkUnstructuredGrid";
-    case VTK_PIECEWISE_FUNCTION:
-      return "vtkPiecewiseFunction";
-    case VTK_IMAGE_DATA:
-      return "vtkImageData";
-    case VTK_DATA_OBJECT:
-      return "vtkDataObject";
-    case VTK_DATA_SET:
-      return "vtkDataSet";
-    case VTK_POINT_SET:
-      return "vtkPointSet";
-    case VTK_UNIFORM_GRID:
-      return "vtkUniformGrid";
-    case VTK_COMPOSITE_DATA_SET:
-      return "vtkCompositeDataSet";
-    case VTK_MULTIGROUP_DATA_SET:
-      return "vtkMultigroupDataSet";
-    case VTK_MULTIBLOCK_DATA_SET:
-      return "vtkMultiblockDataSet";
-    case VTK_HIERARCHICAL_DATA_SET:
-      return "vtkHierarchicalDataSet";
-    case VTK_HIERARCHICAL_BOX_DATA_SET:
-      return "vtkHierarchicalBoxDataSet";
-    case VTK_GENERIC_DATA_SET:
-      return "vtkGenericDataSet";
-    case VTK_HYPER_OCTREE:
-      return "vtkHyperOctree";
-    case VTK_TEMPORAL_DATA_SET:
-      return "vtkTemporalDataSet";
-    case VTK_TABLE:
-      return "vtkTable";
-    case VTK_GRAPH:
-      return "vtkGraph";
-    case VTK_TREE:
-      return "vtkTree";
-    }
-  
-  return "UnknownType";
+  return vtkDataObjectTypes::GetClassNameFromTypeId(this->DataSetType);
 }
 
 //----------------------------------------------------------------------------
@@ -923,13 +901,47 @@ int vtkPVDataInformation::DataSetTypeIsA(const char* type)
   return 0;
 }
 
+//----------------------------------------------------------------------------
+int vtkPVDataInformation::IsDataStructured()
+{
+  switch (this->DataSetType)
+    {
+    case VTK_IMAGE_DATA:
+    case VTK_STRUCTURED_GRID:
+    case VTK_RECTILINEAR_GRID:
+    case VTK_UNIFORM_GRID:
+      return 1;
+    }
+  return 0;
+}
+
+//----------------------------------------------------------------------------
+vtkPVDataInformation*
+vtkPVDataInformation::GetDataInformationForCompositeIndex(int index)
+{
+  return this->GetDataInformationForCompositeIndex(&index);
+}
+
+//----------------------------------------------------------------------------
+vtkPVDataInformation*
+vtkPVDataInformation::GetDataInformationForCompositeIndex(int* index)
+{
+  if (*index == 0)
+    {
+    (*index)--;
+    return this;
+    }
+
+  (*index)--;
+  return this->CompositeDataInformation->GetDataInformationForCompositeIndex(index);
+}
+
 //----------------------------------------------------------------------------
 void vtkPVDataInformation::CopyToStream(vtkClientServerStream* css)
 {
   css->Reset();
   *css << vtkClientServerStream::Reply;
-  *css << this->Name
-       << this->DataClassName
+  *css << this->DataClassName
        << this->DataSetType
        << this->NumberOfDataSets
        << this->NumberOfPoints
@@ -937,6 +949,8 @@ void vtkPVDataInformation::CopyToStream(vtkClientServerStream* css)
        << this->NumberOfRows
        << this->MemorySize
        << this->PolygonCount
+       << this->Time 
+       << this->HasTime
        << vtkClientServerStream::InsertArray(this->Bounds, 6)
        << vtkClientServerStream::InsertArray(this->Extent, 6);
 
@@ -960,6 +974,24 @@ void vtkPVDataInformation::CopyToStream(vtkClientServerStream* css)
   dcss.GetData(&data, &length);
   *css << vtkClientServerStream::InsertArray(data, length);
 
+  dcss.Reset();
+
+  this->VertexDataInformation->CopyToStream(&dcss);
+  dcss.GetData(&data, &length);
+  *css << vtkClientServerStream::InsertArray(data, length);
+
+  dcss.Reset();
+
+  this->EdgeDataInformation->CopyToStream(&dcss);
+  dcss.GetData(&data, &length);
+  *css << vtkClientServerStream::InsertArray(data, length);
+
+  dcss.Reset();
+
+  this->RowDataInformation->CopyToStream(&dcss);
+  dcss.GetData(&data, &length);
+  *css << vtkClientServerStream::InsertArray(data, length);
+
   *css << this->CompositeDataClassName;
   *css << this->CompositeDataSetType;
 
@@ -975,67 +1007,94 @@ void vtkPVDataInformation::CopyToStream(vtkClientServerStream* css)
   dcss.GetData(&data, &length);
   *css << vtkClientServerStream::InsertArray(data, length);
 
+  *css << vtkClientServerStream::InsertArray(this->TimeSpan, 2);
+
   *css << vtkClientServerStream::End;
 }
 
+// Macros used to make it easy to insert/remove entries when serializing
+// to/from a stream.
+
+#define CSS_ARGUMENT_BEGIN() \
+  {\
+  int __vtk__css_argument_int_counter = 0
+
+#define CSS_GET_NEXT_ARGUMENT(css, msg, var)\
+  css->GetArgument(msg, __vtk__css_argument_int_counter++, var)
+
+#define CSS_GET_NEXT_ARGUMENT2(css, msg, var, len)\
+  css->GetArgument(msg, __vtk__css_argument_int_counter++, var, len)
+
+#define CSS_GET_CUR_INDEX() __vtk__css_argument_int_counter
+
+#define CSS_ARGUMENT_END() \
+  }
+
+
 //----------------------------------------------------------------------------
 void vtkPVDataInformation::CopyFromStream(const vtkClientServerStream* css)
 {
-  const char* name = 0;
-  if(!css->GetArgument(0, 0, &name))
-    {
-    vtkErrorMacro("Error parsing name of data.");
-    return;
-    }
-  this->SetName(name);
+  CSS_ARGUMENT_BEGIN();
+
   const char* dataclassname = 0;
-  if(!css->GetArgument(0, 1, &dataclassname))
+  if (!CSS_GET_NEXT_ARGUMENT(css, 0, &dataclassname))
     {
     vtkErrorMacro("Error parsing class name of data.");
     return;
     }
   this->SetDataClassName(dataclassname);
-  if(!css->GetArgument(0, 2, &this->DataSetType))
+
+  if (!CSS_GET_NEXT_ARGUMENT(css, 0, &this->DataSetType))
     {
     vtkErrorMacro("Error parsing data set type.");
     return;
     }
-  if(!css->GetArgument(0, 3, &this->NumberOfDataSets))
+  if (!CSS_GET_NEXT_ARGUMENT(css, 0, &this->NumberOfDataSets))
     {
     vtkErrorMacro("Error parsing number of datasets.");
     return;
     }
-  if(!css->GetArgument(0, 4, &this->NumberOfPoints))
+  if (!CSS_GET_NEXT_ARGUMENT(css, 0, &this->NumberOfPoints))
     {
     vtkErrorMacro("Error parsing number of points.");
     return;
     }
-  if(!css->GetArgument(0, 5, &this->NumberOfCells))
+  if (!CSS_GET_NEXT_ARGUMENT(css, 0, &this->NumberOfCells))
     {
     vtkErrorMacro("Error parsing number of cells.");
     return;
     }
-  if(!css->GetArgument(0, 6, &this->NumberOfRows))
+  if (!CSS_GET_NEXT_ARGUMENT(css, 0, &this->NumberOfRows))
     {
     vtkErrorMacro("Error parsing number of cells.");
     return;
     }
-  if(!css->GetArgument(0, 7, &this->MemorySize))
+  if(!CSS_GET_NEXT_ARGUMENT(css, 0, &this->MemorySize))
     {
     vtkErrorMacro("Error parsing memory size.");
     return;
     }
-  if(!css->GetArgument(0, 8, &this->PolygonCount))
+  if(!CSS_GET_NEXT_ARGUMENT(css, 0, &this->PolygonCount))
     {
     vtkErrorMacro("Error parsing memory size.");
     return;
     }
-  if(!css->GetArgument(0, 9, this->Bounds, 6))
+  if(!CSS_GET_NEXT_ARGUMENT(css, 0, &this->Time))
+    {
+    vtkErrorMacro("Error parsing Time.");
+    return;
+    }
+  if(!CSS_GET_NEXT_ARGUMENT(css, 0, &this->HasTime))
+    {
+    vtkErrorMacro("Error parsing has-time.");
+    return;
+    }
+  if(!CSS_GET_NEXT_ARGUMENT2(css, 0, this->Bounds, 6))
     {
     vtkErrorMacro("Error parsing bounds.");
     return;
     }
-  if(!css->GetArgument(0, 10, this->Extent, 6))
+  if(!CSS_GET_NEXT_ARGUMENT2(css, 0,  this->Extent, 6))
     {
     vtkErrorMacro("Error parsing extent.");
     return;
@@ -1046,72 +1105,123 @@ void vtkPVDataInformation::CopyFromStream(const vtkClientServerStream* css)
   vtkClientServerStream dcss;
 
   // Point array information.
-  if(!css->GetArgumentLength(0, 11, &length))
+  if(!css->GetArgumentLength(0, CSS_GET_CUR_INDEX(), &length))
     {
     vtkErrorMacro("Error parsing length of point data information.");
     return;
     }
   data.resize(length);
-  if(!css->GetArgument(0, 11, &*data.begin(), length))
+  if(!css->GetArgument(0, CSS_GET_CUR_INDEX(), &*data.begin(), length))
     {
     vtkErrorMacro("Error parsing point data information.");
     return;
     }
   dcss.SetData(&*data.begin(), length);
   this->PointArrayInformation->CopyFromStream(&dcss);
+  CSS_GET_CUR_INDEX()++;
 
   // Point data array information.
-  if(!css->GetArgumentLength(0, 12, &length))
+  if(!css->GetArgumentLength(0, CSS_GET_CUR_INDEX(), &length))
     {
     vtkErrorMacro("Error parsing length of point data information.");
     return;
     }
   data.resize(length);
-  if(!css->GetArgument(0, 12, &*data.begin(), length))
+  if(!css->GetArgument(0, CSS_GET_CUR_INDEX(), &*data.begin(), length))
     {
     vtkErrorMacro("Error parsing point data information.");
     return;
     }
   dcss.SetData(&*data.begin(), length);
   this->PointDataInformation->CopyFromStream(&dcss);
+  CSS_GET_CUR_INDEX()++;
 
   // Cell data array information.
-  if(!css->GetArgumentLength(0, 13, &length))
+  if(!css->GetArgumentLength(0, CSS_GET_CUR_INDEX(), &length))
     {
     vtkErrorMacro("Error parsing length of cell data information.");
     return;
     }
   data.resize(length);
-  if(!css->GetArgument(0, 13, &*data.begin(), length))
+  if(!css->GetArgument(0, CSS_GET_CUR_INDEX(), &*data.begin(), length))
     {
     vtkErrorMacro("Error parsing cell data information.");
     return;
     }
   dcss.SetData(&*data.begin(), length);
   this->CellDataInformation->CopyFromStream(&dcss);
+  CSS_GET_CUR_INDEX()++;
+
+  // Vertex data array information.
+  if(!css->GetArgumentLength(0, CSS_GET_CUR_INDEX(), &length))
+    {
+    vtkErrorMacro("Error parsing length of cell data information.");
+    return;
+    }
+  data.resize(length);
+  if(!css->GetArgument(0, CSS_GET_CUR_INDEX(), &*data.begin(), length))
+    {
+    vtkErrorMacro("Error parsing cell data information.");
+    return;
+    }
+  dcss.SetData(&*data.begin(), length);
+  this->VertexDataInformation->CopyFromStream(&dcss);
+  CSS_GET_CUR_INDEX()++;
+
+  // Edge data array information.
+  if(!css->GetArgumentLength(0, CSS_GET_CUR_INDEX(), &length))
+    {
+    vtkErrorMacro("Error parsing length of cell data information.");
+    return;
+    }
+  data.resize(length);
+  if(!css->GetArgument(0, CSS_GET_CUR_INDEX(), &*data.begin(), length))
+    {
+    vtkErrorMacro("Error parsing cell data information.");
+    return;
+    }
+  dcss.SetData(&*data.begin(), length);
+  this->EdgeDataInformation->CopyFromStream(&dcss);
+  CSS_GET_CUR_INDEX()++;
+
+  // Row data array information.
+  if(!css->GetArgumentLength(0, CSS_GET_CUR_INDEX(), &length))
+    {
+    vtkErrorMacro("Error parsing length of cell data information.");
+    return;
+    }
+  data.resize(length);
+  if(!css->GetArgument(0, CSS_GET_CUR_INDEX(), &*data.begin(), length))
+    {
+    vtkErrorMacro("Error parsing cell data information.");
+    return;
+    }
+  dcss.SetData(&*data.begin(), length);
+  this->RowDataInformation->CopyFromStream(&dcss);
+  CSS_GET_CUR_INDEX()++;
 
   const char* compositedataclassname = 0;
-  if(!css->GetArgument(0, 14, &compositedataclassname))
+  if(!CSS_GET_NEXT_ARGUMENT(css, 0, &compositedataclassname))
     {
     vtkErrorMacro("Error parsing class name of data.");
     return;
     }
   this->SetCompositeDataClassName(compositedataclassname);
 
-  if(!css->GetArgument(0, 15, &this->CompositeDataSetType))
+  if(!CSS_GET_NEXT_ARGUMENT(css, 0, &this->CompositeDataSetType))
     {
     vtkErrorMacro("Error parsing data set type.");
     return;
     }
 
   // Composite data information.
-  if(!css->GetArgumentLength(0, 16, &length))
+  if(!css->GetArgumentLength(0, CSS_GET_CUR_INDEX(), &length))
     {
     vtkErrorMacro("Error parsing length of cell data information.");
     return;
     }
   data.resize(length);
-  if(!css->GetArgument(0, 16, &*data.begin(), length))
+  if(!css->GetArgument(0, CSS_GET_CUR_INDEX(), &*data.begin(), length))
     {
     vtkErrorMacro("Error parsing cell data information.");
     return;
@@ -1125,20 +1235,30 @@ void vtkPVDataInformation::CopyFromStream(const vtkClientServerStream* css)
     {
     this->CompositeDataInformation->Initialize();
     }
+  CSS_GET_CUR_INDEX()++;
 
   // Field data array information.
-  if(!css->GetArgumentLength(0, 17, &length))
+  if(!css->GetArgumentLength(0, CSS_GET_CUR_INDEX(), &length))
     {
     vtkErrorMacro("Error parsing length of field data information.");
     return;
     }
 
   data.resize(length);
-  if(!css->GetArgument(0, 17, &*data.begin(), length))
+  if(!css->GetArgument(0, CSS_GET_CUR_INDEX(), &*data.begin(), length))
     {
     vtkErrorMacro("Error parsing field data information.");
     return;
     }
   dcss.SetData(&*data.begin(), length);
   this->FieldDataInformation->CopyFromStream(&dcss);
+  CSS_GET_CUR_INDEX()++;
+
+  if(!CSS_GET_NEXT_ARGUMENT2(css, 0, this->TimeSpan, 2))
+    {
+    vtkErrorMacro("Error parsing timespan.");
+    return;
+    }
+
+  CSS_ARGUMENT_END();
 }
