@@ -23,6 +23,7 @@
 
 #if defined(_WIN32)
 # define _WIN32_IE 0x0400  // special folder support
+# define _WIN32_WINNT 0x0400  // shared folder support
 # include <windows.h>   // FindFirstFile, FindNextFile, FindClose, ...
 # include <direct.h>    // _getcwd
 # include <shlobj.h>    // SHGetFolderPath
@@ -50,7 +51,7 @@
 #include <vtkstd/string>
 
 vtkStandardNewMacro(vtkPVFileInformation);
-vtkCxxRevisionMacro(vtkPVFileInformation, "$Revision: 1.20 $");
+vtkCxxRevisionMacro(vtkPVFileInformation, "$Revision: 1.31 $");
 
 inline void vtkPVFileInformationAddTerminatingSlash(vtkstd::string& name)
 {
@@ -59,21 +60,250 @@ inline void vtkPVFileInformationAddTerminatingSlash(vtkstd::string& name)
     char last = *(name.end()-1);
     if (last != '/' && last != '\\')
       {
+#if defined(_WIN32)
+      name += "\\";
+#else
       name += "/";
+#endif
       }
     }
 }
 
 #if defined(_WIN32)
-static bool vtkPVFileInformationResolveLink(const vtkstd::string& fname,
-                                                      vtkstd::string& result,
+
+static vtkstd::string WindowsNetworkRoot = "Windows Network";
+
+// assumes native back-slashes
+static bool IsUncPath(const vtkstd::string& path)
+{
+  if(path.size() >= 2 && path[0] == '\\' && path[1] == '\\')
+    {
+    return true;
+    }
+  return false;
+}
+
+static bool IsNetworkPath(const vtkstd::string& path)
+{
+  if(path.compare(0, WindowsNetworkRoot.size(), WindowsNetworkRoot) == 0)
+    {
+    return true;
+    }
+  return false;
+}
+
+// returns true if path is valid, false otherwise
+// returns subdirs, if any 
+
+static bool getNetworkSubdirs(const vtkstd::string& name,
+                              DWORD DisplayType,
+                              vtkstd::vector<vtkstd::string>& subdirs)
+{
+  HANDLE han=0;
+  NETRESOURCEA rc;
+  rc.dwScope = RESOURCE_GLOBALNET;
+  rc.dwType = RESOURCETYPE_ANY;  // TODO, restrict this to disk at server level?
+  rc.dwDisplayType = DisplayType;
+  rc.dwUsage = RESOURCEUSAGE_CONTAINER;
+  if(DisplayType == RESOURCEDISPLAYTYPE_NETWORK)
+    {
+    rc.dwUsage |= RESOURCEUSAGE_RESERVED;  // wonder why this is needed?
+    }
+  rc.lpLocalName = NULL;
+  rc.lpProvider = NULL;
+  rc.lpComment = NULL;
+  rc.lpRemoteName = const_cast<char*>(name.c_str());
+
+  DWORD ret = WNetOpenEnumA(RESOURCE_GLOBALNET,
+    RESOURCETYPE_ANY, 0, &rc, &han);
+
+  if(NO_ERROR == ret)
+    {
+    DWORD count = 10;
+    NETRESOURCE res[10];
+    DWORD bufsize = sizeof(NETRESOURCE) * count;
+    do
+      {
+      ret = WNetEnumResourceA(han, &count, res, &bufsize);
+      if(ret == NO_ERROR || ret == ERROR_MORE_DATA)
+        {
+        for(DWORD i=0; i<count; i++)
+          {
+          vtkstd::string subdir = res[i].lpRemoteName;
+          if(subdir.compare(0, name.size(), name) == 0)
+            {
+            subdir = subdir.c_str() + name.size() + 1;
+            }
+          subdirs.push_back(subdir);
+          }
+        }
+      } while(ret == ERROR_MORE_DATA);
+    }
+  return NO_ERROR == ret;
+}
+
+
+static bool getNetworkSubdirs(const vtkstd::string& path,
+                              vtkstd::vector<vtkstd::string>& subdirs,
+                              int& type)
+{
+  if(!IsNetworkPath(path))
+    {
+    return false;
+    }
+
+  // path follows this convention
+  // Windows Network\Domain\Server\Share
+  // this doesn't return subdirectories of "Share", use normal
+  // win32 API for that
+
+  static const int MaxTokens = 4;
+
+  vtkstd::vector<vtksys::String> pathtokens;
+  pathtokens = vtksys::SystemTools::SplitString(path.c_str()+1, '\\');
+
+  static DWORD DisplayType[MaxTokens] =
+    {
+    RESOURCEDISPLAYTYPE_NETWORK, 
+    RESOURCEDISPLAYTYPE_DOMAIN,
+    RESOURCEDISPLAYTYPE_SERVER,
+    RESOURCEDISPLAYTYPE_SHARE
+    };
+
+  int tokenIndex = pathtokens.size()-1;
+
+  if(tokenIndex >= MaxTokens)
+    return false;
+
+  vtkstd::string name = pathtokens[tokenIndex];
+
+  if(tokenIndex == 0)
+    {
+    type = vtkPVFileInformation::NETWORK_DOMAIN;
+    }
+  else if(tokenIndex == 1)
+    {
+    type = vtkPVFileInformation::NETWORK_SERVER;
+    }
+  else if(tokenIndex == 2)
+    {
+    type = vtkPVFileInformation::NETWORK_SHARE;
+    name = "\\\\" + name;
+    }
+
+  return getNetworkSubdirs(name, DisplayType[tokenIndex], subdirs);
+}
+
+static bool getUncSharesOnServer(const vtkstd::string& server,
+                          vtkstd::vector<vtkstd::string>& ret)
+{
+  return getNetworkSubdirs(vtkstd::string("\\\\") + server, 
+                         RESOURCEDISPLAYTYPE_SERVER, ret);
+}
+
+#endif
+
+static int vtkPVFileInformationGetType(const char* path)
+{
+  int type = vtkPVFileInformation::INVALID;
+
+  vtkstd::string realpath = path;
+  
+  if(vtksys::SystemTools::FileExists(realpath.c_str()))
+    {
+    type = vtkPVFileInformation::SINGLE_FILE;
+    }
+  if(vtksys::SystemTools::FileIsDirectory(realpath.c_str()))
+    {
+    type = vtkPVFileInformation::DIRECTORY;
+    }
+#if defined(_WIN32)
+  // doing stat on root of devices doesn't work
+
+  // is it the root of a drive?
+  if (realpath[0] && realpath[1] == ':' && (realpath[2] == '\0' || 
+      (realpath[2] == '\\' && realpath[3] == '\0')))
+    {
+    // Path may be drive letter.
+    DWORD n = GetLogicalDrives();
+    int which = tolower(realpath[0]) - 'a';
+    if(n & (1 << which))
+      {
+      type = vtkPVFileInformation::DRIVE;
+      }
+    }
+
+  if(IsNetworkPath(realpath))
+    {
+    // this code doesn't give out anything with 
+    // "Windows Network\..." unless its a directory.
+    // that may change
+    vtkstd::vector<vtksys::String> pathtokens;
+    pathtokens = vtksys::SystemTools::SplitString(realpath.c_str(), '\\');
+    if(pathtokens.size() == 1)
+      {
+      type = vtkPVFileInformation::NETWORK_ROOT;
+      }
+    else if(pathtokens.size() == 2)
+      {
+      type = vtkPVFileInformation::NETWORK_DOMAIN;
+      }
+    else if(pathtokens.size() == 3)
+      {
+      type = vtkPVFileInformation::NETWORK_SERVER;
+      }
+    else
+      {
+      type = vtkPVFileInformation::NETWORK_SHARE;
+      }
+    }
+
+  // is it the root of a shared folder?
+  if(IsUncPath(realpath))
+    {
+    vtkstd::vector<vtksys::String> parts =
+      vtksys::SystemTools::SplitString(realpath.c_str()+2, '\\', true);
+    if(parts.empty())
+      {
+      // global network
+      type = vtkPVFileInformation::DIRECTORY;
+      }
+    else
+      {
+      vtkstd::vector<vtkstd::string> shares;
+      bool ret = getUncSharesOnServer(parts[0], shares);
+      if(parts.size() == 1 && ret)
+        {
+        // server exists
+        type = vtkPVFileInformation::NETWORK_SERVER;
+        }
+      else if(parts.size() == 2 && ret)
+        {
+        for(unsigned int i = 0; i<shares.size(); i++)
+          {
+          if(parts[1] == shares[i])
+            {
+            // share on server exists
+            type = vtkPVFileInformation::DIRECTORY;
+            break;
+            }
+          }
+        }
+      }
+    }
+#endif
+  return type;
+}
+
+#if defined(_WIN32)
+static vtkstd::string vtkPVFileInformationResolveLink(const vtkstd::string& fname,
                                                       WIN32_FIND_DATA& wfd)
 {
   IShellLink* shellLink;
   HRESULT hr;
   char Link[MAX_PATH];
   bool coInit = false;
-  bool success = false;
+  vtkstd::string result;
 
   hr = ::CoCreateInstance(CLSID_ShellLink, NULL, 
                           CLSCTX_INPROC_SERVER, IID_IShellLink,
@@ -99,7 +329,6 @@ static bool vtkPVFileInformationResolveLink(const vtkstd::string& fname,
         if(shellLink->GetPath(Link, MAX_PATH, &wfd, SLGP_UNCPRIORITY) == NOERROR)
           {
           result = Link;
-          success = true;
           }
         ppf->Release();
         }
@@ -111,11 +340,25 @@ static bool vtkPVFileInformationResolveLink(const vtkstd::string& fname,
     CoUninitialize();
     }
 
-  return success;
+  return result;
 
 }
 #endif
 
+vtkstd::string MakeAbsolutePath(const vtkstd::string& path,
+                            const vtkstd::string& working_dir)
+{
+  vtkstd::string ret = path;
+#if defined(WIN32)
+  if(!IsUncPath(path) && !IsNetworkPath(path))
+#endif
+    {
+    ret = vtksys::SystemTools::CollapseFullPath(path.c_str(),
+      working_dir.c_str());
+    }
+  return ret;
+}
+
 
 //-----------------------------------------------------------------------------
 class vtkPVFileInformationSet : 
@@ -142,6 +385,14 @@ vtkPVFileInformation::~vtkPVFileInformation()
   this->SetFullPath(0);
 }
 
+bool vtkPVFileInformation::IsDirectory(int t)
+{
+  return t == DIRECTORY || t == DIRECTORY_LINK || 
+         t == DRIVE || t == NETWORK_ROOT ||
+         t == NETWORK_DOMAIN || t == NETWORK_SERVER ||
+         t == NETWORK_SHARE;
+}
+
 //-----------------------------------------------------------------------------
 void vtkPVFileInformation::CopyFromObject(vtkObject* object)
 {
@@ -171,71 +422,52 @@ void vtkPVFileInformation::CopyFromObject(vtkObject* object)
     working_directory = helper->GetWorkingDirectory();
     }
 
-  vtkstd::string path = vtksys::SystemTools::CollapseFullPath(helper->GetPath(),
-    working_directory.c_str());
-  
+  vtkstd::string path = MakeAbsolutePath(helper->GetPath(), working_directory);
+
   this->SetName(helper->GetPath());
+  bool isLink = false;
+
 #if defined(_WIN32)
   vtkstd::string::size_type idx;
   for(idx = path.find('/', 0);
-      idx != vtkstd::string::npos;
-      idx = path.find('/', idx))
+        idx != vtkstd::string::npos;
+        idx = path.find('/', idx))
     {
     path.replace(idx, 1, 1, '\\');
     }
-  size_t len = path.size();
-  WIN32_FIND_DATA data;
-  if(len > 4 && strncmp(path.c_str()+len-4, ".lnk", 4) == 0)
-  {
-  vtkPVFileInformationResolveLink(path, path, data);
-  }
+  
+  int len = path.size();
+  if(len > 4 && path.compare(len-4, 4, ".lnk") == 0)
+    {
+    WIN32_FIND_DATA data;
+    path = vtkPVFileInformationResolveLink(path, data);
+    isLink = true;
+    }
 #endif
+
   this->SetFullPath(path.c_str());
 
-  if (!vtksys::SystemTools::FileExists(this->FullPath))
+  this->Type = vtkPVFileInformationGetType(this->FullPath);
+  if(isLink && this->Type == SINGLE_FILE)
     {
-    return;
+    this->Type = SINGLE_FILE_LINK;
+    }
+  else if(isLink && this->Type == DIRECTORY)
+    {
+    this->Type = DIRECTORY_LINK;
     }
 
-  bool is_directory = vtksys::SystemTools::FileIsDirectory(this->FullPath);
-  this->Type = (is_directory)? DIRECTORY : SINGLE_FILE;
+  if (this->IsDirectory(this->Type) && helper->GetDirectoryListing())
+    {
+    // Since we want a directory listing, we now to platform specific listing 
+    // with intelligent pattern matching hee-haa.
 #if defined(_WIN32)
-  if (!is_directory)
-    {
-    // Path may be drive letter.
-    char strings[1024];
-    DWORD n = GetLogicalDriveStrings(1024, strings);
-    char* start = strings;
-    char* end = start;
-    for(;end != strings+n; ++end)
-      {
-      if(!*end)
-        {
-        if (stricmp(start,this->FullPath) == 0)
-          {
-          is_directory = true;
-          this->Type = DRIVE;
-          break;
-          }
-        start = end+1;
-        }
-      }
-    }
-#endif
-
-  if (!helper->GetDirectoryListing() || !is_directory)
-    {
-    return;
-    }
-
-  // Since we want a directory listing, we now to platform specific listing 
-  // with intelligent pattern matching hee-haa.
-
-#if defined(_WIN32)
-  this->GetWindowsDirectoryListing();
+    this->GetWindowsDirectoryListing();
 #else
-  this->GetDirectoryListing();
+    this->GetDirectoryListing();
 #endif
+    }
+
 }
 
 //-----------------------------------------------------------------------------
@@ -246,7 +478,7 @@ void vtkPVFileInformation::GetSpecialDirectories()
   // Return favorite directories ...
 
   TCHAR szPath[MAX_PATH];
-
+  
   if(SUCCEEDED(SHGetSpecialFolderPath(NULL, szPath, CSIDL_PERSONAL, false)))
     {
     vtkSmartPointer<vtkPVFileInformation> info =
@@ -278,124 +510,143 @@ void vtkPVFileInformation::GetSpecialDirectories()
     }
 
   // Return drive letters ...
-  char strings[1024];
-  DWORD n = GetLogicalDriveStrings(1024, strings);
-  char* start = strings;
-  char* end = start;
-  for(;end != strings+n; ++end)
+  DWORD n = GetLogicalDrives();
+  for(int i=0; i<sizeof(DWORD)*8; i++)
     {
-    if(!*end)
+    if(n & (1<<i))
       {
+      vtkstd::string driveLetter;
+      driveLetter += 'A' + i;
+      driveLetter += ":\\";
       vtkSmartPointer<vtkPVFileInformation> info =
         vtkSmartPointer<vtkPVFileInformation>::New();
-      info->SetFullPath(start);
-      info->SetName(start);
+      info->SetFullPath(driveLetter.c_str());
+      info->SetName(driveLetter.c_str());
       info->Type = DRIVE;
       this->Contents->AddItem(info);
-      start = end+1;
       }
     }
+  
+  vtkSmartPointer<vtkPVFileInformation> info =
+      vtkSmartPointer<vtkPVFileInformation>::New();
+  info->SetFullPath(WindowsNetworkRoot.c_str());
+  info->SetName("Windows Network");
+  info->Type = NETWORK_ROOT;
+  this->Contents->AddItem(info);
 
 #else // _WIN32
 #if defined (__APPLE__ )
-    //-------- Get the List of Mounted Volumes from the System
-    int idx = 1;
-    HFSUniStr255 hfsname;
-    FSRef ref;
-    while(noErr == FSGetVolumeInfo(kFSInvalidVolumeRefNum, idx++, NULL,
-                                   kFSVolInfoNone, NULL, &hfsname, &ref))
+  //-------- Get the List of Mounted Volumes from the System
+  
+  int idx = 1;
+  HFSUniStr255 hfsname;
+  FSRef ref;
+  while (noErr == FSGetVolumeInfo(kFSInvalidVolumeRefNum, idx++, NULL,
+      kFSVolInfoNone, NULL, &hfsname, &ref))
     {
-      CFURLRef resolvedUrl = CFURLCreateFromFSRef(NULL, &ref);
-      if(resolvedUrl)
+    CFURLRef resolvedUrl = CFURLCreateFromFSRef(NULL, &ref);
+    if (resolvedUrl)
       {
-        CFStringRef url;
-        url = CFURLCopyFileSystemPath(resolvedUrl, kCFURLPOSIXPathStyle);
+      CFStringRef url;
+      url = CFURLCopyFileSystemPath(resolvedUrl, kCFURLPOSIXPathStyle);
+      if(url)
+        {
         CFStringRef cfname = CFStringCreateWithCharacters(kCFAllocatorDefault,
-                                                          hfsname.unicode, hfsname.length);
+            hfsname.unicode, hfsname.length);
 
         CFIndex pathSize = CFStringGetLength(url)+1;
         vtkstd::vector<char> pathChars(pathSize, 0);
-        OSStatus pathStatus = CFStringGetCString(url,&pathChars[0],pathSize, 
-                                                  kCFStringEncodingASCII);
+        OSStatus pathStatus = CFStringGetCString(url, &pathChars[0], pathSize,
+            kCFStringEncodingASCII);
 
         pathSize = CFStringGetLength(cfname)+1;
         vtkstd::vector<char> nameChars(pathSize, 0);
-        OSStatus nameStatus = CFStringGetCString(cfname, &nameChars[0], pathSize, 
-                                                  kCFStringEncodingASCII);
+        OSStatus nameStatus = CFStringGetCString(cfname, &nameChars[0], pathSize,
+            kCFStringEncodingASCII);
 
         if (pathStatus && nameStatus)
-        {
-          vtkSmartPointer<vtkPVFileInformation> info = 
-                          vtkSmartPointer<vtkPVFileInformation>::New();
-          info->SetFullPath( &(pathChars.front() ) );
-          info->SetName( &(nameChars.front() ) );
+          {
+          vtkSmartPointer<vtkPVFileInformation> info = vtkSmartPointer<
+              vtkPVFileInformation>::New();
+          info->SetFullPath( &(pathChars.front() ));
+          info->SetName( &(nameChars.front() ));
           info->Type = DRIVE;
           this->Contents->AddItem(info);
-        }
+          }
         CFRelease(cfname);
-        CFRelease(resolvedUrl);
+        }
+      CFRelease(resolvedUrl);
       }
     }
-    //-- Read the com.apple.sidebar.plist file to get the user's list of directories
-    CFPropertyListRef p = CFPreferencesCopyAppValue(CFSTR("useritems"),
-                                                    CFSTR("com.apple.sidebarlists"));
-    if(p && CFDictionaryGetTypeID() == CFGetTypeID(p))
+  //-- Read the com.apple.sidebar.plist file to get the user's list of directories
+  CFPropertyListRef p = CFPreferencesCopyAppValue(CFSTR("useritems"),
+      CFSTR("com.apple.sidebarlists"));
+  if (p && CFDictionaryGetTypeID() == CFGetTypeID(p))
     {
-      CFArrayRef r = (CFArrayRef)(CFDictionaryGetValue((CFDictionaryRef)p, CFSTR("CustomListItems")));
-      if(r && CFArrayGetTypeID() == CFGetTypeID(r))
+    CFArrayRef r = (CFArrayRef)(CFDictionaryGetValue((CFDictionaryRef)p,
+        CFSTR("CustomListItems")));
+    if (r && CFArrayGetTypeID() == CFGetTypeID(r))
       {
-        int count = CFArrayGetCount(r);
-        for(int i=0; i<count; i++)
+      int count = CFArrayGetCount(r);
+      for (int i=0; i<count; i++)
         {
-          CFDictionaryRef dr = (CFDictionaryRef)CFArrayGetValueAtIndex(r, i);
-          if(dr && CFDictionaryGetTypeID() == CFGetTypeID(dr))
+        CFDictionaryRef dr = (CFDictionaryRef)CFArrayGetValueAtIndex(r, i);
+        if (dr && CFDictionaryGetTypeID() == CFGetTypeID(dr))
           {
-            CFStringRef name;
-            CFStringRef url;
-            CFDataRef alias;
-            if(CFDictionaryGetValueIfPresent(dr, CFSTR("Name"), (const void**)&name) &&
-               CFDictionaryGetValueIfPresent(dr, CFSTR("Alias"), (const void**)&alias) &&
-               name && alias &&
-               CFStringGetTypeID() == CFGetTypeID(name) &&
-               CFDataGetTypeID() == CFGetTypeID(alias) )
+          CFStringRef name = 0;
+          CFStringRef url = 0;
+          CFDataRef alias;
+          if (CFDictionaryGetValueIfPresent(dr, CFSTR("Name"),
+              (const void**)&name) && CFDictionaryGetValueIfPresent(dr,
+              CFSTR("Alias"), (const void**)&alias) && name && alias
+              && CFStringGetTypeID() == CFGetTypeID(name) && CFDataGetTypeID()
+              == CFGetTypeID(alias) )
             {
-              CFIndex dataSize = CFDataGetLength(alias);
-              AliasHandle tAliasHdl = (AliasHandle) NewHandle( dataSize );
-              if(tAliasHdl)
+            CFIndex dataSize = CFDataGetLength(alias);
+            AliasHandle tAliasHdl = (AliasHandle) NewHandle(dataSize);
+            if (tAliasHdl)
               {
-                CFDataGetBytes( alias, CFRangeMake( 0, dataSize ), ( UInt8*) *tAliasHdl );
-                FSRef tFSRef;
-                Boolean changed;
-                if(noErr == FSResolveAlias(NULL, tAliasHdl, &tFSRef, &changed))
+              CFDataGetBytes(alias, CFRangeMake( 0, dataSize),
+                  ( UInt8*) *tAliasHdl );
+              FSRef tFSRef;
+              Boolean changed;
+              if (noErr == FSResolveAlias(NULL, tAliasHdl, &tFSRef, &changed))
                 {
-                  CFURLRef resolvedUrl = CFURLCreateFromFSRef(NULL, &tFSRef);
-                  if(resolvedUrl)
+                CFURLRef resolvedUrl = CFURLCreateFromFSRef(NULL, &tFSRef);
+                if (resolvedUrl)
                   {
-                    url = CFURLCopyFileSystemPath(resolvedUrl, kCFURLPOSIXPathStyle);
-                    CFRelease(resolvedUrl);
+                  url = CFURLCopyFileSystemPath(resolvedUrl,
+                      kCFURLPOSIXPathStyle);
+                  CFRelease(resolvedUrl);
                   }
                 }
-                DisposeHandle((Handle)tAliasHdl);
+              DisposeHandle((Handle)tAliasHdl);
               }
-              // now put the name and path into a FileInfo Object
-              CFIndex pathSize = CFStringGetLength(url)+1;
-              vtkstd::vector<char> pathChars(pathSize, 0);
-              OSStatus pathStatus = CFStringGetCString(url,&pathChars[0],pathSize, 
-                                                        kCFStringEncodingASCII);
 
-              pathSize = CFStringGetLength(name)+1;
-              vtkstd::vector<char> nameChars(pathSize, 0);
-              OSStatus nameStatus = CFStringGetCString(name, &nameChars[0], pathSize, 
-                                                        kCFStringEncodingASCII);
-
-              if (pathStatus && nameStatus)
+            if(!url || !name)
               {
-                vtkSmartPointer<vtkPVFileInformation> info = 
-                                vtkSmartPointer<vtkPVFileInformation>::New();
-                info->SetFullPath( &(pathChars.front() ) );
-                info->SetName( &(nameChars.front() ) );
-                info->Type = DIRECTORY;
-                this->Contents->AddItem(info);
+              continue;
+              }
+
+            // now put the name and path into a FileInfo Object
+            CFIndex pathSize = CFStringGetLength(url)+1;
+            vtkstd::vector<char> pathChars(pathSize, 0);
+            OSStatus pathStatus = CFStringGetCString(url, &pathChars[0],
+                pathSize, kCFStringEncodingASCII);
+
+            pathSize = CFStringGetLength(name)+1;
+            vtkstd::vector<char> nameChars(pathSize, 0);
+            OSStatus nameStatus = CFStringGetCString(name, &nameChars[0],
+                pathSize, kCFStringEncodingASCII);
+
+            if (pathStatus && nameStatus)
+              {
+              vtkSmartPointer<vtkPVFileInformation> info = vtkSmartPointer<
+                  vtkPVFileInformation>::New();
+              info->SetFullPath( &(pathChars.front() ));
+              info->SetName( &(nameChars.front() ));
+              info->Type = DIRECTORY;
+              this->Contents->AddItem(info);
               }
             }
           }
@@ -422,6 +673,77 @@ void vtkPVFileInformation::GetWindowsDirectoryListing()
 #if defined(_WIN32)
   vtkPVFileInformationSet info_set;
 
+  if(IsNetworkPath(this->FullPath))
+    {
+    vtkstd::vector<vtkstd::string> shares;
+    int type;
+    if(getNetworkSubdirs(this->FullPath, shares, type))
+      {
+      for(unsigned int i=0; i<shares.size(); i++)
+        {
+        vtkPVFileInformation* info = vtkPVFileInformation::New();
+        info->SetName(shares[i].c_str());
+        vtkstd::string fullpath = 
+          vtksys::SystemTools::CollapseFullPath(shares[i].c_str(), this->FullPath);
+        info->SetFullPath(fullpath.c_str());
+        info->Type = type;
+        info->FastFileTypeDetection = this->FastFileTypeDetection;
+        info_set.insert(info);
+        info->Delete();
+        }
+      }
+    this->OrganizeCollection(info_set);
+
+    for (vtkPVFileInformationSet::iterator iter = info_set.begin();
+      iter != info_set.end(); ++iter)
+      {
+      this->Contents->AddItem(*iter);
+      }
+    return;
+    }
+
+  if(IsUncPath(this->FullPath))
+    {
+    bool didListing = false;
+    vtkstd::vector<vtksys::String> parts =
+      vtksys::SystemTools::SplitString(this->FullPath+2, '\\', true);
+
+    if(parts.size() == 1)
+      {
+      // get list of all shares on server
+      vtkstd::vector<vtkstd::string> shares;
+      if(getUncSharesOnServer(parts[0], shares))
+        {
+        for(unsigned int i=0; i<shares.size(); i++)
+          {
+          vtkPVFileInformation* info = vtkPVFileInformation::New();
+          info->SetName(shares[i].c_str());
+          vtkstd::string fullpath = "\\\\" + parts[0] + "\\" + shares[i];
+          info->SetFullPath(fullpath.c_str());
+          info->Type = NETWORK_SHARE;
+          info->FastFileTypeDetection = this->FastFileTypeDetection;
+          info_set.insert(info);
+          info->Delete();
+          }
+        }
+      didListing = true;
+      }
+
+    if(didListing)
+      {
+      this->OrganizeCollection(info_set);
+
+      for (vtkPVFileInformationSet::iterator iter = info_set.begin();
+        iter != info_set.end(); ++iter)
+        {
+        this->Contents->AddItem(*iter);
+        }
+      return;
+      }
+    // fall through for normal file listing that works after shares are
+    // known
+    }
+
   // Search for all files in the given directory.
   vtkstd::string prefix = this->FullPath;
   vtkPVFileInformationAddTerminatingSlash(prefix);
@@ -449,11 +771,13 @@ void vtkPVFileInformation::GetWindowsDirectoryListing()
       continue;
     vtkstd::string fullpath = prefix + filename;
     size_t len = filename.size();
-    bool success = true;
+    bool isLink = false;
 
     if(len > 4 && strncmp(filename.c_str()+len-4, ".lnk", 4) == 0)
       {
-      success = vtkPVFileInformationResolveLink(fullpath, fullpath, data);
+      fullpath = vtkPVFileInformationResolveLink(fullpath, data);
+      filename.resize(filename.size() - 4);
+      isLink = true;
       }
 
     DWORD isdir = data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY;
@@ -463,8 +787,12 @@ void vtkPVFileInformation::GetWindowsDirectoryListing()
                    !(data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY));
 
     FileTypes type = isdir ? DIRECTORY : SINGLE_FILE;
+    if(isLink)
+      {
+      type = type == DIRECTORY ? DIRECTORY_LINK : SINGLE_FILE_LINK;
+      }
 
-    if(success && (isdir || isfile))
+    if(isdir || isfile)
       {
       vtkPVFileInformation* infoD = vtkPVFileInformation::New();
       infoD->SetName(filename.c_str());
@@ -668,7 +996,7 @@ void vtkPVFileInformation::OrganizeCollection(vtkPVFileInformationSet& info_set)
     {
     vtkPVFileInformation* obj = *iter;
 
-    if (obj->Type != FILE_GROUP && obj->Type != DIRECTORY)
+    if (obj->Type != FILE_GROUP && !IsDirectory(obj->Type))
       {
       bool match = false;
       vtkstd::string groupName;
