@@ -18,13 +18,19 @@
 #include "vtkObjectFactory.h"
 #include "vtkSmartPointer.h"
 
-vtkCxxRevisionMacro(vtkPVXMLElement, "$Revision: 1.19 $");
+vtkCxxRevisionMacro(vtkPVXMLElement, "$Revision: 1.22 $");
 vtkStandardNewMacro(vtkPVXMLElement);
 
 #include <vtkstd/string>
 #include <vtkstd/vector>
 #include <vtksys/ios/sstream>
 
+#if defined(_WIN32) && !defined(__CYGWIN__)
+# define SNPRINTF _snprintf
+#else
+# define SNPRINTF snprintf
+#endif
+
 struct vtkPVXMLElementInternals
 {
   vtkstd::vector<vtkstd::string> AttributeNames;
@@ -117,50 +123,6 @@ void vtkPVXMLElement::AddAttribute(const char* attrName,
   this->Internal->AttributeValues.push_back(attrValue);
 }
 
-//----------------------------------------------------------------------------
-void vtkPVXMLElement::AddSanitizedAttribute(const char* attrName,
-                                            const char* attrValue)
-{
-  if (!attrName || !attrValue)
-    {
-    return;
-    }
-  
-  this->Internal->AttributeNames.push_back(attrName);
-
-  //escape any characters that are not allowed in XML
-  vtkstd::string sanitized = "";
-  const int numtok = 9;
-  const char escapees[numtok][3] = 
-    {{"&"},{"\'"},{">"},{"<"},{"\""},{"\r\n"},{"\r"},{"\n"},{"\t"}};
-  const char replacees[numtok][13] = 
-    {{"&amp;"},{"&apos;"},{"&gt;"},{"&lt;"},{"&quot;"},{"&#x0D;&#x0A;"},{"&#x0D;"},{"&#x0A;"},{"&#x09;"}};
-  int len = strlen(attrValue);
-  const char *ptr = attrValue;
-  for (int i = 0; i < len; i++)
-    {
-    bool replaced = false;
-    for (int j = 0; j < numtok; j++)
-      {
-      int szof = strlen(escapees[j]);
-      if (!strncmp(ptr, escapees[j], szof))
-        {
-        sanitized += replacees[j];
-        ptr += szof;
-        replaced = true;
-        break;
-        }
-      }
-    if (!replaced)
-      {
-      char c = *ptr;
-      sanitized += c;
-      ptr++;
-      }    
-    }
-  this->Internal->AttributeValues.push_back(sanitized.c_str());
-}
-
 //----------------------------------------------------------------------------
 void vtkPVXMLElement::SetAttribute(const char* attrName,
   const char* attrValue)
@@ -264,58 +226,6 @@ const char* vtkPVXMLElement::GetAttribute(const char* name)
   return 0;
 }
 
-//----------------------------------------------------------------------------
-char* vtkPVXMLElement::GetSanitizedAttribute(const char* name)
-{
-  unsigned int numAttributes = this->Internal->AttributeNames.size();
-  unsigned int a;
-  for(a=0; a < numAttributes; ++a)
-    {
-    if(strcmp(this->Internal->AttributeNames[a].c_str(), name) == 0)
-      {
-      const char *value = this->Internal->AttributeValues[a].c_str();
-      if (value)
-        {
-        //un-escape any characters that are not allowed in XML
-        vtkstd::string sanitized = "";
-        const int numtok = 9;
-        const char escapees[numtok][3] = 
-          {{"&"},{"\'"},{">"},{"<"},{"\""},{"\r\n"},{"\r"},{"\n"},{"\t"}};
-        const char replacees[numtok][13] = 
-          {{"&amp;"},{"&apos;"},{"&gt;"},{"&lt;"},{"&quot;"},{"&#x0D;&#x0A;"},{"&#x0D;"},{"&#x0A;"},{"&#x09;"}};
-        int len = strlen(value);
-        const char *ptr = value;
-        for (int i = 0; i < len; i++)
-          {
-          bool replaced = false;
-          for (int j = 0; j < numtok; j++)
-            {
-            int szof = strlen(replacees[j]);
-            if (!strncmp(ptr, replacees[j], szof))
-              {
-              sanitized += escapees[j];
-              ptr += szof;
-              replaced = true;
-              break;
-              }
-            }
-          if (!replaced)
-            {
-            char c = *ptr;
-            sanitized += c;
-            ptr++;
-            }    
-          }       
-        char *retval = new char[strlen(sanitized.c_str())+1];
-        strcpy(retval, sanitized.c_str());
-        return retval;
-        }
-      return 0;
-      }
-    }
-  return 0;
-}
-
 //----------------------------------------------------------------------------
 const char* vtkPVXMLElement::GetCharacterData()
 {
@@ -338,8 +248,13 @@ void vtkPVXMLElement::PrintXML(ostream& os, vtkIndent indent)
     {
     const char* aName = this->Internal->AttributeNames[i].c_str();
     const char* aValue = this->Internal->AttributeValues[i].c_str();
+
+    // we always print the encoded value. The expat parser processes encoded
+    // values when reading them, hence we don't need any decoding when reading
+    // the values back.
+    const vtkStdString& sanitizedValue = vtkPVXMLElement::Encode(aValue);
     os << " " << (aName?aName:"NoName")
-       << "=\"" << (aValue?aValue:"NoValue") << "\"";
+       << "=\"" << (aValue?sanitizedValue.c_str():"NoValue") << "\"";
     }
   unsigned int numberOfNestedElements = this->Internal->NestedElements.size();
   if(numberOfNestedElements > 0)
@@ -600,6 +515,47 @@ void vtkPVXMLElement::GetElementsByName(const char* name, vtkCollection* element
     }
 }
 
+
+//----------------------------------------------------------------------------
+vtkStdString vtkPVXMLElement::Encode(const char* plaintext)
+{
+  //escape any characters that are not allowed in XML
+  vtkStdString sanitized = "";
+  if (!plaintext)
+    {
+    return sanitized;
+    }
+
+  const char toescape[] = { '&', '\'', '<', '>', '\"', '\r', '\n', '\t', 0};
+
+  size_t pt_length = strlen(plaintext);
+  for (size_t cc = 0; cc < pt_length; cc++)
+    {
+    const char* escape_char = toescape;
+    for ( ; *escape_char != 0; escape_char++)
+      {
+      if (plaintext[cc] == *escape_char)
+        {
+        break;
+        }
+      }
+
+    if (*escape_char)
+      {
+      char temp[20];
+      SNPRINTF(temp, 20, "&#x%x;", static_cast<int>(*escape_char));
+      sanitized += temp;
+      }
+    else
+      {
+      sanitized += plaintext[cc];
+      }
+    }
+
+  return sanitized;
+}
+
+
 #if defined(VTK_USE_64BIT_IDS)
 //----------------------------------------------------------------------------
 int vtkPVXMLElement::GetCharacterDataAsVector(int length, vtkIdType* data)
@@ -607,3 +563,87 @@ int vtkPVXMLElement::GetCharacterDataAsVector(int length, vtkIdType* data)
   return vtkPVXMLVectorAttributeParse(this->GetCharacterData(), length, data);
 }
 #endif
+
+void vtkPVXMLElement::Merge(vtkPVXMLElement* element, const char* attributeName)
+{
+  if(!element || 0 != strcmp(this->GetName(), element->GetName()))
+    {
+    return;
+    }
+  if(attributeName)
+    {
+    const char* attr1 = this->GetAttribute(attributeName);
+    const char* attr2 = element->GetAttribute(attributeName);
+    if(attr1 && attr2 && 0 != strcmp(attr1, attr2))
+      {
+      return;
+      }
+    }
+
+  // override character data if there is some
+  if(!element->Internal->CharacterData.empty())
+    {
+    this->Internal->CharacterData = element->Internal->CharacterData;
+    }
+  
+  // add attributes from element to this, or override attribute values on this
+  unsigned int numAttributes = element->Internal->AttributeNames.size();
+  unsigned int numAttributes2 = this->Internal->AttributeNames.size();
+
+  for(unsigned int i=0; i < numAttributes; ++i)
+    {
+    bool found = false;
+    for(unsigned int j=0; !found && j < numAttributes2; ++j)
+      {
+      if(element->Internal->AttributeNames[i] ==
+        this->Internal->AttributeNames[j])
+        {
+        this->Internal->AttributeValues[j] =
+          element->Internal->AttributeValues[i];
+        found = true;
+        }
+      }
+    // if not found, add it
+    if(!found)
+      {
+      this->AddAttribute(element->Internal->AttributeNames[i].c_str(),
+                         element->Internal->AttributeValues[i].c_str());
+      }
+    }
+
+  // now recursively merge the children with the same names
+
+  vtkPVXMLElementInternals::VectorOfElements::iterator iter;
+  vtkPVXMLElementInternals::VectorOfElements::iterator iter2;
+
+  for(iter = element->Internal->NestedElements.begin();
+      iter != element->Internal->NestedElements.end(); ++iter)
+    {
+    bool found = false;
+    for(iter2 = this->Internal->NestedElements.begin();
+        iter2 != this->Internal->NestedElements.end(); ++iter2)
+      {
+      const char* attr1 = attributeName ? this->GetAttribute(attributeName) : NULL;
+      const char* attr2 = attributeName ? element->GetAttribute(attributeName) : NULL;
+      if(0 == strcmp((*iter)->Name, (*iter2)->Name) &&
+        (!attributeName || (!attr1 || !attr2 || 0 == strcmp(attr1, attr2))))
+        {
+        (*iter2)->Merge(*iter, attributeName);
+        found = true;
+        }
+      }
+    // if not found, add it
+    if(!found)
+      {
+      vtkSmartPointer<vtkPVXMLElement> newElement = 
+        vtkSmartPointer<vtkPVXMLElement>::New();
+      newElement->SetName((*iter)->GetName());
+      newElement->SetId((*iter)->GetId());
+      newElement->Internal->AttributeNames = (*iter)->Internal->AttributeNames;
+      newElement->Internal->AttributeValues = (*iter)->Internal->AttributeValues;
+      this->AddNestedElement(newElement);
+      newElement->Merge(*iter, attributeName);
+      }
+    }
+}
+
