@@ -26,7 +26,6 @@
 #include "vtkDataObject.h"
 #include "vtkInstantiator.h"
 #include "vtkKWProcessStatistics.h"
-#include "vtkMapper.h"
 #include "vtkMultiProcessController.h"
 #include "vtkMultiThreader.h"
 #include "vtkObjectFactory.h"
@@ -46,15 +45,22 @@
 #include "vtkStdString.h"
 #include "vtkStringList.h"
 #include "vtkTimerLog.h"
+#include "vtkToolkits.h" // For VTK_USE_MPI
+
+#ifdef VTK_USE_MPI
+# include "vtkMPIController.h"
+#endif
 
 #include <vtkstd/map>
 #include <vtkstd/new>
+#include <vtkstd/vector>
 
 #include <vtksys/RegularExpression.hxx>
 #include <vtksys/SystemTools.hxx>
 #include <vtksys/ios/sstream>
 
 vtkProcessModule* vtkProcessModule::ProcessModule = 0;
+ostream* vtkProcessModule::DebugLogStream = 0;
 
 //*****************************************************************************
 class vtkProcessModuleInternals
@@ -73,6 +79,31 @@ public:
   vtkTypeUInt32 ProgressServersFlag;
 };
 
+//*****************************************************************************
+class vtkProcessModule::vtkInterpreterInitializationCallbackVector : 
+  public vtkstd::vector<vtkProcessModule::InterpreterInitializationCallback>
+{
+};
+
+//*****************************************************************************
+// Used to clean up the vtkProcessModule::InitializationCallbacks vector.
+class vtkProcessModuleCleanup
+{
+public:
+  inline void Use()
+    {
+    }
+  ~vtkProcessModuleCleanup()
+    {
+    delete vtkProcessModule::InitializationCallbacks;
+    vtkProcessModule::InitializationCallbacks = 0;
+    }
+
+};
+
+vtkProcessModule::vtkInterpreterInitializationCallbackVector* vtkProcessModule::InitializationCallbacks = 0;
+static vtkProcessModuleCleanup vtkProcessModuleCleanupGlobal;
+
 //*****************************************************************************
 class vtkProcessModuleObserver : public vtkCommand
 {
@@ -105,15 +136,15 @@ protected:
 
 
 vtkStandardNewMacro(vtkProcessModule);
-vtkCxxRevisionMacro(vtkProcessModule, "$Revision: 1.76 $");
+vtkCxxRevisionMacro(vtkProcessModule, "$Revision: 1.86.2.1 $");
 vtkCxxSetObjectMacro(vtkProcessModule, ActiveRemoteConnection, vtkRemoteConnection);
 vtkCxxSetObjectMacro(vtkProcessModule, GUIHelper, vtkProcessModuleGUIHelper);
 
-int vtkProcessModule::StreamBlockFlag = 0;
-
 //-----------------------------------------------------------------------------
 vtkProcessModule::vtkProcessModule()
 {
+  vtkProcessModuleCleanupGlobal.Use();
+
   this->Internals = new vtkProcessModuleInternals;
   this->ConnectionManager = 0;
   this->Interpreter = 0;
@@ -143,6 +174,7 @@ vtkProcessModule::vtkProcessModule()
   this->ExceptionRaised = 0;
   
   this->MemoryInformation = vtkKWProcessStatistics::New();
+
   this->ServerInformation = vtkPVServerInformation::New();
 
   this->UseMPI = 1;
@@ -155,14 +187,11 @@ vtkProcessModule::vtkProcessModule()
   this->LastProgress = -1;
   this->LastProgressName = 0;
 
-
-  // setting this here since this must be initialized on all processes to produce
-  // correct result.
-  // This is essential to ensure that edge-visibility works correctly.
-  vtkMapper::SetResolveCoincidentTopologyToPolygonOffset();
-  // We offset lines/vertices.
-  vtkMapper::SetResolveCoincidentTopologyPolygonOffsetFaces(0);
-  vtkMapper::SetResolveCoincidentTopologyPolygonOffsetParameters(-1.0, -1.0);
+#ifdef VTK_USE_MPI
+  // ParaView uses Ssend for all Trigger RMI calls. This helps in overcoming
+  // bufferring issues with send on some implementations.
+  vtkMPIController::SetUseSsendForRMI(1);
+#endif
 }
 
 //-----------------------------------------------------------------------------
@@ -295,7 +324,6 @@ int vtkProcessModule::Start(int argc, char** argv)
   this->ConnectionManager->AddObserver(vtkCommand::ConnectionClosedEvent,
     this->Observer);
 
-
   int myId;
   // This call blocks on the Satellite nodes (never on root node).
   if (this->ConnectionManager->Initialize(argc, argv, 
@@ -327,6 +355,12 @@ int vtkProcessModule::Start(int argc, char** argv)
     return this->StartClient(argc, argv);
     }
 
+  // if (this->GetPartitionId()==0)
+  //  {
+  //  ofstream *fp = new ofstream("/tmp/pvdebug.log");
+  //  this->InitializeDebugLog(*fp);
+  //  }
+
   // Running in server mode.
   return this->StartServer(0);
 }
@@ -376,6 +410,11 @@ int vtkProcessModule::StartClient(int argc, char** argv)
     // established by now. Disable any new connection requests.
     this->DisableNewConnections=true;
     }
+
+  // fill up ServerInformation with local information. ServerInformation is only used
+  // when there's no remote server.
+  this->ServerInformation->CopyFromObject(this);
+
   // if the PM supports multiple connections, its the responsibility of the GUI
   // to connect to the server.
   return this->GUIHelper->RunGUIStart(argc, argv, 1, 0);
@@ -567,6 +606,12 @@ void vtkProcessModule::StopAcceptingConnections(int id)
   this->ConnectionManager->StopAcceptingConnections(id);
 }
 
+//-----------------------------------------------------------------------------
+bool vtkProcessModule::IsAcceptingConnections()
+{
+  return this->ConnectionManager->IsAcceptingConnections();
+}
+
 //-----------------------------------------------------------------------------
 vtkIdType vtkProcessModule::ConnectToRemote(const char* servername, int port)
 {
@@ -886,6 +931,24 @@ void vtkProcessModule::Finalize()
   this->FinalizeInterpreter();
 }
 
+//-----------------------------------------------------------------------------
+void vtkProcessModule::InitializeInterpreter(InterpreterInitializationCallback callback)
+{
+  vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
+  if (pm && pm->GetInterpreter())
+    {
+    (*callback)(pm->GetInterpreter());
+    }
+  else
+    {
+    if (!vtkProcessModule::InitializationCallbacks)
+      {
+      vtkProcessModule::InitializationCallbacks = new vtkInterpreterInitializationCallbackVector();
+      }
+    vtkProcessModule::InitializationCallbacks->push_back(callback);
+    }
+}
+
 //-----------------------------------------------------------------------------
 void vtkProcessModule::InitializeInterpreter()
 {
@@ -947,6 +1010,21 @@ void vtkProcessModule::InitializeInterpreter()
     << this->GetProcessModuleID() << this
     << vtkClientServerStream::End;
   this->Interpreter->ProcessStream(css); 
+
+  // If any initialization callbacks were registered, call them.
+  if (this->InitializationCallbacks)
+    {
+    vtkInterpreterInitializationCallbackVector& callbacks = 
+      (*this->InitializationCallbacks);
+   vtkInterpreterInitializationCallbackVector::iterator iter;
+   for (iter = callbacks.begin(); iter != callbacks.end(); ++iter)
+     {
+     if (*iter)
+       {
+       (*(*iter))(this->GetInterpreter());
+       }
+     }
+    }
 }
 
 //-----------------------------------------------------------------------------
@@ -1466,51 +1544,77 @@ void vtkProcessModule::LogEndEvent(const char* str)
       << " KB" << endl;
     }
 }
+
+//----------------------------------------------------------------------------
+void vtkProcessModule::SetLogBufferLength(vtkIdType connectionID,
+                                          vtkTypeUInt32 servers,
+                                          int length)
+{
+  vtkClientServerStream stream;
+  stream << vtkClientServerStream::Invoke
+         << this->GetProcessModuleID()
+         << "SetLogBufferLength"
+         << length
+         << vtkClientServerStream::End;
+  this->SendStream(connectionID, servers, stream);
+}
+
 //----------------------------------------------------------------------------
 void vtkProcessModule::SetLogBufferLength(int length)
 {
   vtkTimerLog::SetMaxEntries(length);
 }
 
+//----------------------------------------------------------------------------
+void vtkProcessModule::ResetLog(vtkIdType connectionID,
+                                vtkTypeUInt32 servers)
+{
+  vtkClientServerStream stream;
+  stream << vtkClientServerStream::Invoke
+         << this->GetProcessModuleID()
+         << "ResetLog"
+         << vtkClientServerStream::End;
+  this->SendStream(connectionID, servers, stream);
+}
+
 //----------------------------------------------------------------------------
 void vtkProcessModule::ResetLog()
 {
   vtkTimerLog::ResetLog();
 }
 
+//----------------------------------------------------------------------------
+void vtkProcessModule::SetEnableLog(vtkIdType connectionID,
+                                    vtkTypeUInt32 servers,
+                                    int flag)
+{
+  vtkClientServerStream stream;
+  stream << vtkClientServerStream::Invoke
+         << this->GetProcessModuleID()
+         << "SetEnableLog"
+         << flag
+         << vtkClientServerStream::End;
+  this->SendStream(connectionID, servers, stream);
+}
+
 //----------------------------------------------------------------------------
 void vtkProcessModule::SetEnableLog(int flag)
 {
   vtkTimerLog::SetLogging(flag);
 }
 
-//-----------------------------------------------------------------------------
-void vtkProcessModule::SetStreamBlock(int val)
+//----------------------------------------------------------------------------
+void vtkProcessModule::SetLogThreshold(vtkIdType connectionID, 
+                                       vtkTypeUInt32 servers,
+                                       double threshold)
 {
-  if (this->StreamBlockFlag == val)
-    {
-    return;
-    }
-  this->SetStreamBlockFlag(val);
   vtkClientServerStream stream;
   stream << vtkClientServerStream::Invoke
          << this->GetProcessModuleID()
-         << "SetStreamBlockFlag" << val << vtkClientServerStream::End;
-  this->SendStream(
-    vtkProcessModuleConnectionManager::GetRootServerConnectionID(), 
-    vtkProcessModule::DATA_SERVER, stream);
-}
-
-//-----------------------------------------------------------------------------
-int vtkProcessModule::GetStreamBlock()
-{
-  return vtkProcessModule::StreamBlockFlag;
-}
-
-//-----------------------------------------------------------------------------
-void vtkProcessModule::SetStreamBlockFlag(int val)
-{
-  this->StreamBlockFlag = val;
+         << "SetLogThreshold"
+         << threshold
+         << vtkClientServerStream::End;
+  this->SendStream(connectionID, servers, stream);
 }
 
 //============================================================================
@@ -1766,6 +1870,31 @@ int vtkProcessModule::GetNumberOfConnections()
   return this->ConnectionManager->GetNumberOfConnections();
 }
 
+//-----------------------------------------------------------------------------
+void vtkProcessModule::InitializeDebugLog(ostream& ref)
+{
+  if (vtkProcessModule::DebugLogStream)
+    {
+    vtkGenericWarningMacro("Debug log already initialized.");
+    }
+  vtkProcessModule::DebugLogStream = &ref;
+}
+
+//-----------------------------------------------------------------------------
+void vtkProcessModule::DebugLog(const char* msg)
+{
+  if (vtkProcessModule::DebugLogStream)
+    {
+    *vtkProcessModule::DebugLogStream << msg << endl;
+    }
+}
+
+//-----------------------------------------------------------------------------
+void vtkProcessModule::FinalizeDebugLog()
+{
+  vtkProcessModule::DebugLogStream = 0;
+}
+
 //-----------------------------------------------------------------------------
 void vtkProcessModule::PrintSelf(ostream& os, vtkIndent indent)
 {
@@ -1779,6 +1908,8 @@ void vtkProcessModule::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "UseMPI: " << this->UseMPI << endl;
   os << indent << "SendStreamToClientOnly: " 
     << this->SendStreamToClientOnly << endl;
+  os << indent 
+    << (this->LastProgressName? this->LastProgressName : "(none)") << endl;
  
   os << indent << "Interpreter: " ;
   if (this->Interpreter)
