@@ -34,6 +34,7 @@ class vtkClientServerStream;
 class vtkDataObject;
 class vtkKWProcessStatistics;
 class vtkMultiProcessController;
+class vtkProcessModuleCleanup;
 class vtkProcessModuleConnection;
 class vtkProcessModuleConnectionManager;
 class vtkProcessModuleGUIHelper;
@@ -59,8 +60,7 @@ public:
 //BTX
   // Description: 
   // These flags are used to specify destination servers for the
-  // SendStream function. Note that the new interface no longer
-  // support the server flag CLIENT.
+  // SendStream function.
   enum ServerFlags
     {
     DATA_SERVER = 0x01,
@@ -142,6 +142,12 @@ public:
   virtual void GatherInformation(vtkIdType connectionID,
     vtkTypeUInt32 serverFlags, vtkPVInformation* info, vtkClientServerID id);
 //ETX
+  virtual void GatherInformation(vtkIdType connectionID,
+    vtkTypeUInt32 serverFlags, vtkPVInformation* info, int id)
+  {
+    this->GatherInformation(connectionID, serverFlags, info, 
+                            vtkClientServerID(id));
+  }
   
   // Description:
   // Start the process modules. It will create the application
@@ -266,6 +272,12 @@ public:
   // Get the vtkClientServerID used for the ProcessModule.
   vtkClientServerID GetProcessModuleID();
 //ETX
+  int GetProcessModuleIDAsInt()
+  {
+  vtkClientServerID id = this->GetProcessModuleID();
+  return id.ID;
+  }
+
   // Description:
   // Get/Set the global process module.
   static vtkProcessModule* GetProcessModule();
@@ -328,34 +340,29 @@ public:
   virtual void CreateLogFile();
 
   // Description:
-  // For loggin from Tcl start and end execute events.  We do not have c
-  // pointers to all filters.
+  // For logging start and end execute events.
   void LogStartEvent(const char* str);
   void LogEndEvent(const char* str);
 
   // Description:
-  // More timer log access methods.  Static methods are not accessible 
-  // from tcl.  We need a timer object on all procs.
+  // More timer log access methods. 
   void SetLogBufferLength(int length);
+  void SetLogBufferLength(vtkIdType connectionID,
+                          vtkTypeUInt32 servers,
+                          int length);
   void ResetLog();
+  void ResetLog(vtkIdType connectionID, vtkTypeUInt32 servers);
   void SetEnableLog(int flag);
+  void SetEnableLog(vtkIdType connectionID, vtkTypeUInt32 servers,
+                    int flag);
 
   // Description:
   // Time threshold for event (start-end) when getting the log with indents.
-  // We do not have a timer object on all procs.  Statics do not work with Tcl.
   vtkSetMacro(LogThreshold, double);
+  void SetLogThreshold(vtkIdType connectionID, vtkTypeUInt32 servers,
+                       double threshold);
   vtkGetMacro(LogThreshold, double);
 
-//BTX
-  // Description:
-  // I am experimenting with streaming. This turns streaming on and off.
-  // When this value is zero, pipelines do not update.
-  // When the flag is turned on, then the pipeline streams.
-  void SetStreamBlock(int val);
-  static int GetStreamBlock();
-//ETX
-  void SetStreamBlockFlag(int val);
-
   // Description:
   // We need to get the data path for the demo on the server.
   const char* GetPath(const char* tag, const char* relativePath, const char* file);
@@ -513,6 +520,12 @@ public:
   void StopAcceptingConnections(int id);
   void StopAcceptingAllConnections();
 
+  // Description:
+  // Returns if the process module is correctly accepting connections on any
+  // port. This typically means its waiting for some remote process to connect
+  // to it.
+  bool IsAcceptingConnections();
+
   // Description:
   // This creates a new SelfConnection. This is experimental feature.
   vtkIdType ConnectToSelf();
@@ -595,7 +608,26 @@ public:
   vtkGetStringMacro(LastProgressName);
 
 //BTX
+  // Description:
+  // Initialize the log file.
+  static void InitializeDebugLog(ostream& ref);
 
+  // Description:
+  // Used to dump debug messages.
+  static void DebugLog(const char*);
+
+  // Description:
+  // Finalize debug log.
+  static void FinalizeDebugLog();
+
+  typedef void (*InterpreterInitializationCallback)(vtkClientServerInterpreter*);
+
+  // Description:
+  // Use this method to initialize the interpreter. This register the callback
+  // if the process module hasn't been created yet, so that the callbacks get
+  // invoked once the process module is created. This is generally necessary for
+  // plugins.
+  static void InitializeInterpreter(InterpreterInitializationCallback callback);
 protected:
   vtkProcessModule();
   ~vtkProcessModule();
@@ -723,8 +755,6 @@ protected:
   // the servers. This is useful for reiviving a server manager.
   int SendStreamToClientOnly;
 
-  static int StreamBlockFlag;
-
   int UseMPI;
 
   vtkCacheSizeKeeper* CacheSizeKeeper;
@@ -733,6 +763,11 @@ private:
   void operator=(const vtkProcessModule&); // Not implemented.
 
   vtkIdType LastConnectionID;
+  static ostream* DebugLogStream;
+
+  friend class vtkProcessModuleCleanup;
+  class vtkInterpreterInitializationCallbackVector;
+  static vtkInterpreterInitializationCallbackVector* InitializationCallbacks;
 //ETX
 };
 
