@@ -72,7 +72,7 @@ protected:
 
 //-----------------------------------------------------------------------------
 vtkStandardNewMacro(vtkProcessModuleConnectionManager);
-vtkCxxRevisionMacro(vtkProcessModuleConnectionManager, "$Revision: 1.22 $");
+vtkCxxRevisionMacro(vtkProcessModuleConnectionManager, "$Revision: 1.23 $");
 
 //-----------------------------------------------------------------------------
 vtkProcessModuleConnectionManager::vtkProcessModuleConnectionManager()
@@ -213,6 +213,7 @@ void vtkProcessModuleConnectionManager::StopAcceptingConnections(int id)
   ss->CloseSocket();
   this->Internals->IntToServerSocketMap.erase(iter);
 }
+
 //-----------------------------------------------------------------------------
 void vtkProcessModuleConnectionManager::StopAcceptingAllConnections()
 {
@@ -227,6 +228,12 @@ void vtkProcessModuleConnectionManager::StopAcceptingAllConnections()
   this->Internals->IntToServerSocketMap.clear();
 }
 
+//-----------------------------------------------------------------------------
+bool vtkProcessModuleConnectionManager::IsAcceptingConnections()
+{
+  return (this->Internals->IntToServerSocketMap.size() > 0);
+}
+
 //-----------------------------------------------------------------------------
 vtkIdType vtkProcessModuleConnectionManager::OpenSelfConnection()
 {
