@@ -35,7 +35,7 @@
 
 
 vtkStandardNewMacro(vtkServerConnection);
-vtkCxxRevisionMacro(vtkServerConnection, "$Revision: 1.14 $");
+vtkCxxRevisionMacro(vtkServerConnection, "$Revision: 1.15 $");
 //-----------------------------------------------------------------------------
 vtkServerConnection::vtkServerConnection()
 {
@@ -348,7 +348,7 @@ void vtkServerConnection::GatherInformationFromController(vtkSocketController* c
     vtkRemoteConnection::ROOT_INFORMATION_LENGTH_TAG);
   if (length2 <= 0)
     {
-    vtkErrorMacro("Server could failed to gather information.");
+    vtkErrorMacro("Server failed to gather information.");
     return;
     }
   unsigned char* data2 = new unsigned char[length2];
