@@ -25,32 +25,38 @@
 #include "vtkDataSetSurfaceFilter.h"
 #include "vtkFloatArray.h"
 #include "vtkGarbageCollector.h"
+#include "vtkGenericDataSet.h"
+#include "vtkGenericGeometryFilter.h"
 #include "vtkGeometryFilter.h"
+#include "vtkHierarchicalBoxDataIterator.h"
+#include "vtkHyperOctree.h"
+#include "vtkHyperOctreeSurfaceFilter.h"
 #include "vtkImageData.h"
 #include "vtkInformation.h"
 #include "vtkInformationVector.h"
-#include "vtkMultiGroupDataSet.h"
+#include "vtkCompositeDataSet.h"
 #include "vtkMultiProcessController.h"
 #include "vtkObjectFactory.h"
 #include "vtkOutlineSource.h"
 #include "vtkPointData.h"
 #include "vtkPolyData.h"
 #include "vtkPolygon.h"
+#include "vtkPVTrivialProducer.h"
 #include "vtkRectilinearGrid.h"
 #include "vtkRectilinearGridOutlineFilter.h"
+#include "vtkSmartPointer.h"
 #include "vtkStreamingDemandDrivenPipeline.h"
 #include "vtkStripper.h"
 #include "vtkStructuredGrid.h"
 #include "vtkStructuredGridOutlineFilter.h"
 #include "vtkUnsignedCharArray.h"
 #include "vtkUnstructuredGrid.h"
-#include "vtkGenericDataSet.h"
-#include "vtkGenericGeometryFilter.h"
-#include "vtkHyperOctree.h"
-#include "vtkHyperOctreeSurfaceFilter.h"
+#include "vtkUnsignedIntArray.h"
 
+#include <vtkstd/map>
+#include <vtkstd/string>
 
-vtkCxxRevisionMacro(vtkPVGeometryFilter, "$Revision: 1.71 $");
+vtkCxxRevisionMacro(vtkPVGeometryFilter, "$Revision: 1.85 $");
 vtkStandardNewMacro(vtkPVGeometryFilter);
 
 vtkCxxSetObjectMacro(vtkPVGeometryFilter, Controller, vtkMultiProcessController);
@@ -77,8 +83,7 @@ vtkPVGeometryFilter::vtkPVGeometryFilter ()
 
   this->OutlineSource = vtkOutlineSource::New();
 
-  this->GenerateGroupScalars = 0;
-  this->CurrentGroup = 0;
+  this->CompositeIndex = 0;
 
   this->PassThroughCellIds = 1;
   this->PassThroughPointIds = 1;
@@ -160,6 +165,154 @@ int vtkPVGeometryFilter::CheckAttributes(vtkDataObject* input)
   return 0;
 }
 
+typedef vtkstd::map<vtkstd::string, vtkDataSetAttributes*> vtkArrayMap;
+
+//----------------------------------------------------------------------------
+static void vtkBuildArrayMap(vtkDataSetAttributes* dsa, vtkArrayMap& arrayMap)
+{
+  int numArrays = dsa->GetNumberOfArrays();
+  for (int kk=0; kk < numArrays; kk++)
+    {
+    vtkDataArray* array = dsa->GetArray(kk);
+
+    if (!array || !array->GetName())
+      {
+      continue;
+      }
+
+    if (array == dsa->GetNormals() || array == dsa->GetTCoords() ||
+      array == dsa->GetGlobalIds() || array == dsa->GetPedigreeIds())
+      {
+      // Normals, Tcoords, GIDs, PIDs are not filled up with filling partial arrays.
+      continue;
+      }
+
+    if (arrayMap.find(array->GetName()) == arrayMap.end())
+      {
+      arrayMap[array->GetName()] = dsa;
+      }
+    }
+}
+
+//----------------------------------------------------------------------------
+static void vtkFillPartialArrays(vtkDataSetAttributes* dsa, 
+  vtkArrayMap& arrayMap, vtkIdType count)
+{
+  vtkArrayMap::iterator iter;
+  for (iter = arrayMap.begin(); iter != arrayMap.end(); ++iter)
+    {
+    if (dsa->GetArray(iter->first.c_str()) == NULL)
+      {
+      vtkDataArray* srcArray = iter->second->GetArray(iter->first.c_str());
+      vtkDataArray* clone = srcArray->NewInstance();
+      clone->SetName(srcArray->GetName());
+      clone->SetNumberOfComponents(srcArray->GetNumberOfComponents());
+      clone->SetNumberOfTuples(count);
+      for (int cc=0; cc < clone->GetNumberOfComponents(); cc++)
+        {
+        clone->FillComponent(cc, 0.0);
+        }
+      dsa->AddArray(clone);
+      clone->Delete();
+      }
+    }
+}
+
+//----------------------------------------------------------------------------
+vtkCompositeDataSet* vtkPVGeometryFilter::FillPartialArrays(
+  vtkCompositeDataSet* input)
+{
+  if (!input)
+    {
+    return NULL;
+    }
+
+  vtkArrayMap arrayMapPD;
+  vtkArrayMap arrayMapCD;
+
+
+  vtkCompositeDataSet* output = input->NewInstance();
+  output->CopyStructure(input);
+  vtkCompositeDataIterator* iter = input->NewIterator();
+
+  // build arrayMap with representative arrays for each array name.
+  // As far as active attributes are concerned, we only fill in the scalars or
+  // vectors. For appendPolyData to work correctly, it is essential that the
+  // active scalars/vectors are the same array on all the inputs. To avoid
+  // contention, we choose the first available scalars/vectors.
+  vtkstd::string activePScalars, activeCScalars, activePVectors, activeCVectors;
+  
+  for (iter->InitTraversal(); !iter->IsDoneWithTraversal(); iter->GoToNextItem())
+    {
+    vtkDataSet* ds = vtkDataSet::SafeDownCast(iter->GetCurrentDataObject());
+    if (ds)
+      {
+      if (activePScalars=="" && ds->GetPointData()->GetScalars() &&
+        ds->GetPointData()->GetScalars()->GetName())
+        {
+        activePScalars = ds->GetPointData()->GetScalars()->GetName();
+        }
+      if (activeCScalars=="" && ds->GetCellData()->GetScalars() &&
+        ds->GetCellData()->GetScalars()->GetName())
+        {
+        activeCScalars = ds->GetCellData()->GetScalars()->GetName();
+        }
+
+      if (activePVectors=="" && ds->GetPointData()->GetVectors() &&
+        ds->GetPointData()->GetVectors()->GetName())
+        {
+        activePVectors = ds->GetPointData()->GetVectors()->GetName();
+        }
+      if (activeCVectors=="" && ds->GetCellData()->GetVectors() &&
+        ds->GetCellData()->GetVectors()->GetName())
+        {
+        activeCVectors = ds->GetCellData()->GetVectors()->GetName();
+        }
+
+      ::vtkBuildArrayMap(ds->GetPointData(), arrayMapPD);
+      ::vtkBuildArrayMap(ds->GetCellData(), arrayMapCD);
+      }
+
+    vtkDataSet* clone = ds->NewInstance();
+    clone->ShallowCopy(ds);
+    output->SetDataSet(iter, clone);
+    clone->Delete();
+    }
+  iter->Delete();
+
+  // Now fill partial arrays.
+  iter = output->NewIterator();
+  for (iter->InitTraversal(); !iter->IsDoneWithTraversal(); iter->GoToNextItem())
+    {
+    vtkDataSet* ds = vtkDataSet::SafeDownCast(iter->GetCurrentDataObject());
+    if (ds)
+      {
+      ::vtkFillPartialArrays(ds->GetPointData(), arrayMapPD, ds->GetNumberOfPoints());
+      ::vtkFillPartialArrays(ds->GetCellData(), arrayMapCD, ds->GetNumberOfCells());
+
+      if (activePScalars != "")
+        {
+        ds->GetPointData()->SetActiveScalars(activePScalars.c_str());
+        }
+      if (activeCScalars != "")
+        {
+        ds->GetCellData()->SetActiveScalars(activeCScalars.c_str());
+        }
+      if (activePVectors != "")
+        {
+        ds->GetPointData()->SetActiveVectors(activePVectors.c_str());
+        }
+      if (activeCVectors != "")
+        {
+        ds->GetCellData()->SetActiveVectors(activeCVectors.c_str());
+        }
+      }
+    }
+
+  iter->Delete();
+  return output;
+}
+
 //----------------------------------------------------------------------------
 int vtkPVGeometryFilter::RequestInformation(
   vtkInformation*, vtkInformationVector**, vtkInformationVector* outputVector)
@@ -240,9 +393,9 @@ int vtkPVGeometryFilter::RequestData(vtkInformation* request,
   vtkInformation* inInfo = inputVector[0]->GetInformationObject(0);
 
   vtkDataObject* inputDobj = inInfo->Get(vtkDataObject::DATA_OBJECT());
-  vtkMultiGroupDataSet *mgInput = 
-    vtkMultiGroupDataSet::SafeDownCast(inputDobj);
-  if (mgInput) 
+  vtkCompositeDataSet *compInput = 
+    vtkCompositeDataSet::SafeDownCast(inputDobj);
+  if (compInput) 
     {
     return this->RequestCompositeData(request, inputVector, outputVector);
     }
@@ -283,7 +436,7 @@ int vtkPVGeometryFilter::RequestCompositeData(vtkInformation*,
     info->Get(vtkDataObject::DATA_OBJECT()));
   if (!output) {return 0;}
 
-  vtkMultiGroupDataSet *mgInput = vtkMultiGroupDataSet::SafeDownCast(
+  vtkCompositeDataSet *mgInput = vtkCompositeDataSet::SafeDownCast(
     inInfo->Get(vtkDataObject::DATA_OBJECT()));
   if (!mgInput)
     {
@@ -292,19 +445,21 @@ int vtkPVGeometryFilter::RequestCompositeData(vtkInformation*,
     return 0;
     }
 
-  if (this->CheckAttributes(mgInput))
+  // fills partial array.
+  vtkSmartPointer<vtkCompositeDataSet> constructuredInput;
+  constructuredInput.TakeReference(this->FillPartialArrays(mgInput));
+
+  if (this->CheckAttributes(constructuredInput))
     {
     return 0;
     }
 
   vtkAppendPolyData* append = vtkAppendPolyData::New();
-  this->GenerateGroupScalars = 1;
   int numInputs = 0;
 
   int retVal = 0;
-  if (this->ExecuteCompositeDataSet(mgInput, append, numInputs, 1))
+  if (this->ExecuteCompositeDataSet(constructuredInput, append, numInputs))
     {
-    this->GenerateGroupScalars = 0;
     if (numInputs > 0)
       {
       append->Update();
@@ -318,67 +473,82 @@ int vtkPVGeometryFilter::RequestCompositeData(vtkInformation*,
 }
 
 //----------------------------------------------------------------------------
-int vtkPVGeometryFilter::ExecuteCompositeDataSet(
-  vtkMultiGroupDataSet* mgInput, 
-  vtkAppendPolyData* append, 
-  int& numInputs,
-  int updateGroup)
+void vtkPVGeometryFilter::AddCompositeIndex(vtkPolyData* pd, unsigned int index)
 {
-  unsigned int numGroups = mgInput->GetNumberOfGroups();
+  vtkUnsignedIntArray* cindex = vtkUnsignedIntArray::New();
+  cindex->SetNumberOfComponents(1);
+  cindex->SetNumberOfTuples(pd->GetNumberOfCells());
+  cindex->FillComponent(0, index);
+  cindex->SetName("vtkCompositeIndex");
+  pd->GetCellData()->AddArray(cindex);
+  cindex->Delete();
+}
+
+//----------------------------------------------------------------------------
+void vtkPVGeometryFilter::AddHierarchicalIndex(vtkPolyData* pd, 
+  unsigned int level, unsigned int index)
+{
+  vtkUnsignedIntArray* dslevel = vtkUnsignedIntArray::New();
+  dslevel->SetNumberOfTuples(pd->GetNumberOfCells());
+  dslevel->FillComponent(0, level);
+  dslevel->SetName("vtkAMRLevel");
+  pd->GetCellData()->AddArray(dslevel);
+  dslevel->Delete();
+
+  vtkUnsignedIntArray* dsindex = vtkUnsignedIntArray::New();
+  dsindex->SetNumberOfTuples(pd->GetNumberOfCells());
+  dsindex->FillComponent(0, index);
+  dsindex->SetName("vtkAMRIndex");
+  pd->GetCellData()->AddArray(dsindex);
+  dsindex->Delete();
+}
+
+//----------------------------------------------------------------------------
+int vtkPVGeometryFilter::ExecuteCompositeDataSet(
+  vtkCompositeDataSet* mgInput, 
+  vtkAppendPolyData* append, 
+  int& numInputs)
+{
+  vtkSmartPointer<vtkCompositeDataIterator> iter;
+  iter.TakeReference(mgInput->NewIterator());
+
+  vtkHierarchicalBoxDataIterator* hdIter = 
+    vtkHierarchicalBoxDataIterator::SafeDownCast(iter);
 
-  unsigned int group;
-  unsigned int dataset;
   unsigned int totNumBlocks=0;
-  for (group=0; group<numGroups; group++)
+  for (iter->InitTraversal(); !iter->IsDoneWithTraversal(); iter->GoToNextItem())
     {
-    unsigned int numDataSets = mgInput->GetNumberOfDataSets(group);
-    for (dataset=0; dataset<numDataSets; dataset++)
-      {
-      if (mgInput->GetDataSet(group, dataset))
-        {
-        totNumBlocks++;
-        }
-      }
+    // iter skips empty blocks automatically.
+    totNumBlocks++;
     }
-  
-  for (group=0; group<numGroups; group++)
+ 
+  unsigned int group = 0;
+  for (iter->InitTraversal(); !iter->IsDoneWithTraversal(); iter->GoToNextItem(), group++)
     {
-    unsigned int numDataSets = mgInput->GetNumberOfDataSets(group);
-    if (updateGroup)
-      {
-      this->CurrentGroup = group;
-      }
-    for (dataset=0; dataset<numDataSets; dataset++)
-      {
-      vtkDataObject* block = mgInput->GetDataSet(group, dataset);
-      vtkDataSet* ds = vtkDataSet::SafeDownCast(block);
-      if (ds)
-        {
-        vtkPolyData* tmpOut = vtkPolyData::New();
-        this->ExecuteBlock(ds, tmpOut, 0);
-        append->AddInput(tmpOut);
-        // Call FastDelete() instead of Delete() to avoid garbage
-        // collection checks. This improves the preformance significantly
-        tmpOut->FastDelete();
-        numInputs++;
+    this->CompositeIndex = iter->GetCurrentFlatIndex();
+    vtkDataObject* block = iter->GetCurrentDataObject();
+    
+    vtkPolyData* tmpOut = vtkPolyData::New();
+    this->ExecuteBlock(block, tmpOut, 0);
 
-        this->UpdateProgress(static_cast<float>(numInputs)/totNumBlocks);
-        }
-      else
-        {
-        vtkMultiGroupDataSet* mgds = vtkMultiGroupDataSet::SafeDownCast(block);
-        if (mgds)
-          {
-          // Do not show the group ids of the sub-composite datasets.
-          // Set updateGroup to 0.
-          if (!this->ExecuteCompositeDataSet(mgds, append, numInputs, 0))
-            {
-            return 0;
-            }
-          }
-        }
+    if (hdIter)
+      {
+      this->AddHierarchicalIndex(tmpOut, 
+        hdIter->GetCurrentLevel(), hdIter->GetCurrentIndex());
       }
+    else
+      {
+      this->AddCompositeIndex(tmpOut, iter->GetCurrentFlatIndex());
+      }
+
+    append->AddInput(tmpOut);
+    // Call FastDelete() instead of Delete() to avoid garbage
+    // collection checks. This improves the preformance significantly
+    tmpOut->FastDelete();
+    numInputs++;
+    this->UpdateProgress(static_cast<float>(numInputs)/totNumBlocks);
     }
+
   return 1;
 }
 
@@ -555,20 +725,6 @@ void vtkPVGeometryFilter::DataSetExecute(
     
       output->SetPoints(this->OutlineSource->GetOutput()->GetPoints());
       output->SetLines(this->OutlineSource->GetOutput()->GetLines());
-
-      if (this->GenerateGroupScalars)
-        {
-        vtkFloatArray* newArray = vtkFloatArray::New();
-        vtkIdType numCells = output->GetNumberOfCells();
-        newArray->SetNumberOfTuples(numCells);
-        for(vtkIdType cellId=0; cellId<numCells; cellId++)
-          {
-          newArray->SetValue(cellId, this->CurrentGroup);
-          }
-        newArray->SetName("GroupScalars");
-        output->GetCellData()->SetScalars(newArray);
-        newArray->Delete();
-        }
       }
     }
 }
@@ -671,10 +827,25 @@ void vtkPVGeometryFilter::GenericDataSetExecute(
 void vtkPVGeometryFilter::DataSetSurfaceExecute(vtkDataSet* input, 
                                                 vtkPolyData* output)
 {
-  vtkDataSet* ds = input->NewInstance();
-  ds->ShallowCopy(input);
-  this->DataSetSurfaceFilter->SetInput(ds);
-  ds->Delete();
+  vtkDataSet* clone = input->NewInstance();
+
+  // If we directly pass clone to the input of the DataSetSurfaceFilter, then
+  // for structured datasets, the information about whole extent of the data is
+  // lost. This is so, because vtkTrivialProducer sets that the whole
+  // extent for the output to be same as the extent of the data. This results is
+  // false boundary surfaces for such data when number of processes > 1. To
+  // overcome that issue, we are creating a new produces which still reports the
+  // "real" whole extent, however, has a special extent translator that never
+  // results in a request for the whole extent, only that provided by the data.
+  vtkPVTrivialProducer* producer = vtkPVTrivialProducer::New();
+  producer->SetWholeExtent(input->GetWholeExtent());
+  producer->SetOutput(clone);
+
+  clone->ShallowCopy(input);
+  this->DataSetSurfaceFilter->SetInputConnection(producer->GetOutputPort());
+  
+  producer->Delete();
+  clone->Delete();
 
   // Observe the progress of the internal filter.
   this->DataSetSurfaceFilter->AddObserver(vtkCommand::ProgressEvent, 
@@ -682,6 +853,7 @@ void vtkPVGeometryFilter::DataSetSurfaceExecute(vtkDataSet* input,
   this->DataSetSurfaceFilter->Update();
   // The internal filter is finished.  Remove the observer.
   this->DataSetSurfaceFilter->RemoveObserver(this->InternalProgressObserver);
+  this->DataSetSurfaceFilter->SetInput(0);
 
   output->ShallowCopy(this->DataSetSurfaceFilter->GetOutput());
 }
@@ -741,20 +913,6 @@ void vtkPVGeometryFilter::ImageDataExecute(vtkImageData *input,
     output->SetPoints(outline->GetOutput()->GetPoints());
     output->SetLines(outline->GetOutput()->GetLines());
     outline->Delete();
-
-    if (this->GenerateGroupScalars)
-      {
-      vtkFloatArray* newArray = vtkFloatArray::New();
-      vtkIdType numCells = output->GetNumberOfCells();
-      newArray->SetNumberOfTuples(numCells);
-      for(vtkIdType cellId=0; cellId<numCells; cellId++)
-        {
-        newArray->SetValue(cellId, this->CurrentGroup);
-        }
-      newArray->SetName("GroupScalars");
-      output->GetCellData()->SetScalars(newArray);
-      newArray->Delete();
-      }
     }
   else
     {
@@ -877,7 +1035,6 @@ void vtkPVGeometryFilter::PolyDataExecute(
     else
       {
       out->ShallowCopy(input);
-      out->RemoveGhostCells(1);
       if (this->PassThroughCellIds)
         {
         vtkIdTypeArray *originalCellIds = vtkIdTypeArray::New();
@@ -910,6 +1067,7 @@ void vtkPVGeometryFilter::PolyDataExecute(
         originalPointIds->Delete();
         originalPointIds = NULL;
         }
+      out->RemoveGhostCells(1);
       }
     return;
     }
