@@ -29,7 +29,7 @@
 
 #include <vtkstd/list>
 
-vtkCxxRevisionMacro(vtkTransferFunctionEditorWidgetSimple1D, "$Revision: 1.33.4.1 $");
+vtkCxxRevisionMacro(vtkTransferFunctionEditorWidgetSimple1D, "$Revision: 1.35 $");
 vtkStandardNewMacro(vtkTransferFunctionEditorWidgetSimple1D);
 
 // The vtkNodeList is a PIMPLed list<T>.
@@ -1328,8 +1328,9 @@ double vtkTransferFunctionEditorWidgetSimple1D::GetElementScalar(
 //----------------------------------------------------------------------------
 void vtkTransferFunctionEditorWidgetSimple1D::SetColorSpace(int space)
 {
-  if (space < 0 || space > 3)
+  if (space < 0 || space > 4)
     {
+    vtkWarningMacro("Invalid color space.");
     return;
     }
 
@@ -1349,6 +1350,9 @@ void vtkTransferFunctionEditorWidgetSimple1D::SetColorSpace(int space)
     case 3:
       this->ColorFunction->SetColorSpaceToLab();
       break;
+    case 4:
+      this->ColorFunction->SetColorSpaceToDiverging();
+      break;
     }
 }
 
