@@ -16,7 +16,7 @@
 
 #include "vtkCallbackCommand.h"
 #include "vtkCharArray.h"
-#include "vtkDataObject.h"
+#include "vtkDataSet.h"
 #include "vtkFieldData.h"
 #include "vtkInstantiator.h"
 #include "vtkMultiBlockDataSet.h"
@@ -32,7 +32,7 @@
 #include <vtkstd/map>
 #include <vtkstd/algorithm>
 
-vtkCxxRevisionMacro(vtkXMLCollectionReader, "$Revision: 1.18 $");
+vtkCxxRevisionMacro(vtkXMLCollectionReader, "$Revision: 1.20 $");
 vtkStandardNewMacro(vtkXMLCollectionReader);
 
 //----------------------------------------------------------------------------
@@ -82,6 +82,8 @@ vtkXMLCollectionReader::vtkXMLCollectionReader()
 
   this->InternalForceMultiBlock = false;
   this->ForceOutputTypeToMultiBlock = 0;
+  
+  this->CurrentOutput = -1;
 }
 
 //----------------------------------------------------------------------------
@@ -253,18 +255,7 @@ int vtkXMLCollectionReader::FillOutputPortInformation(int, vtkInformation *info)
 //----------------------------------------------------------------------------
 void vtkXMLCollectionReader::SetupEmptyOutput()
 {
-  vtkExecutive* exec = this->GetExecutive();
-  vtkInformation* info = exec->GetOutputInformation(0);
-
-  vtkDataObject* doOutput = 
-    info->Get(vtkDataObject::DATA_OBJECT());
-  vtkMultiGroupDataSet* hb = 
-    vtkMultiGroupDataSet::SafeDownCast(doOutput);
-  if (!hb)
-    {
-    return;
-    }
-  hb->Initialize();
+  this->GetCurrentOutput()->Initialize();
 }
 
 //----------------------------------------------------------------------------
@@ -483,7 +474,7 @@ void vtkXMLCollectionReader::ReadXMLDataImpl()
   int n = static_cast<int>(this->Internal->RestrictedDataSets.size());
   this->Internal->Readers.resize(n);
 
-  vtkInformation* outInfo = this->GetExecutive()->GetOutputInformation(0);
+  vtkInformation* outInfo = this->GetCurrentOutputInformation();
   int updatePiece = outInfo->Get(
     vtkStreamingDemandDrivenPipeline::UPDATE_PIECE_NUMBER());
   int updateNumPieces = outInfo->Get(
@@ -515,6 +506,7 @@ void vtkXMLCollectionReader::ReadXMLDataImpl()
                     "unless the output is forced to be multi-block");
       return;
       }
+    this->CurrentOutput = 0;
     this->ReadAFile(0,
                     updatePiece,
                     updateNumPieces,
@@ -529,6 +521,15 @@ void vtkXMLCollectionReader::ReadXMLDataImpl()
     output->SetNumberOfBlocks(nBlocks);
     for(int i=0; i < nBlocks; ++i)
       {
+      vtkMultiBlockDataSet* block = vtkMultiBlockDataSet::SafeDownCast(
+        output->GetBlock(i));
+      if (!block)
+        {
+        block = vtkMultiBlockDataSet::New();
+        output->SetBlock(i, block);
+        block->Delete();
+        }
+
       this->CurrentOutput = i;
       vtkDataObject* actualOutput = this->SetupOutput(filePath.c_str(), i);
       this->ReadAFile(i, 
@@ -536,8 +537,8 @@ void vtkXMLCollectionReader::ReadXMLDataImpl()
                       updateNumPieces,
                       updateGhostLevels,
                       actualOutput);
-      output->SetNumberOfDataSets(i, updateNumPieces);
-      output->SetDataSet(i, updatePiece, actualOutput);
+      block->SetNumberOfBlocks(updateNumPieces);
+      block->SetBlock(updatePiece, actualOutput);
       actualOutput->Delete();
       }
     }
@@ -751,8 +752,8 @@ vtkXMLCollectionReaderInternals::ReaderList[] =
   {"vti", "vtkXMLImageDataReader"},
   {"vtr", "vtkXMLRectilinearGridReader"},
   {"vtmb", "vtkXMLMultiBlockDataReader"},
-  {"vtmg", "vtkXMLMultiGroupDataReader"},
-  {"vthd", "vtkXMLHierarchicalDataReader"},
+  {"vtmg", "vtkXMLMultiGroupDataReader"}, // legacy reader - produces vtkMultiBlockDataSet.
+  {"vthd", "vtkXMLHierarchicalDataReader"}, // legacy reader - produces vtkMultiBlockDataSet.
   {"vthb", "vtkXMLHierarchicalBoxDataReader"},
   {"vts", "vtkXMLStructuredGridReader"},
   {"pvtp", "vtkXMLPPolyDataReader"},
