@@ -25,12 +25,15 @@
 #include "vtkSMPropertyIterator.h"
 #include "vtkSMProxyLink.h"
 #include "vtkSMProxyManager.h"
+#include "vtkSMPropertyHelper.h"
 #include "vtkSMRepresentationProxy.h"
+#include "vtkSMProxyProperty.h"
 
 #include <vtkstd/vector>
 #include <vtkstd/map>
 #include <vtkstd/set>
 #include <vtkstd/string>
+#include <vtksys/ios/sstream>
 
 class vtkSMComparativeViewProxy::vtkInternal
 {
@@ -64,12 +67,13 @@ public:
 
   unsigned int ActiveIndexX;
   unsigned int ActiveIndexY;
+  vtkstd::string SuggestedViewType;
 };
 
 //----------------------------------------------------------------------------
 
 vtkStandardNewMacro(vtkSMComparativeViewProxy);
-vtkCxxRevisionMacro(vtkSMComparativeViewProxy, "$Revision: 1.18 $");
+vtkCxxRevisionMacro(vtkSMComparativeViewProxy, "$Revision: 1.26 $");
 
 //----------------------------------------------------------------------------
 vtkSMComparativeViewProxy::vtkSMComparativeViewProxy()
@@ -85,6 +89,8 @@ vtkSMComparativeViewProxy::vtkSMComparativeViewProxy()
   this->Spacing[0] = this->Spacing[1] = 1;
   this->AnimationSceneX = 0;
   this->AnimationSceneY = 0;
+  this->ShowTimeSteps = 1;
+  this->ViewUpdateMode = UPDATE_MODE_ROOT;
 
   this->SceneOutdated = true;
 
@@ -204,6 +210,7 @@ void vtkSMComparativeViewProxy::Build(int dx, int dy)
 //----------------------------------------------------------------------------
 void vtkSMComparativeViewProxy::UpdateViewLayout()
 {
+
   int width = 
     (this->ViewSize[0] - (this->Dimensions[0]-1)*this->Spacing[0])/this->Dimensions[0];
   int height = 
@@ -225,8 +232,11 @@ void vtkSMComparativeViewProxy::UpdateViewLayout()
 
       ivp = vtkSMIntVectorProperty::SafeDownCast(
         view->GetProperty("ViewSize"));
-      ivp->SetElement(0, width);
-      ivp->SetElement(1, height);
+      if (ivp) // Not all view classes have a ViewSize property
+        {
+        ivp->SetElement(0, width);
+        ivp->SetElement(1, height);
+        }
 
       ivp = vtkSMIntVectorProperty::SafeDownCast(
         view->GetProperty("GUISize"));
@@ -241,7 +251,14 @@ void vtkSMComparativeViewProxy::UpdateViewLayout()
 //----------------------------------------------------------------------------
 vtkSMViewProxy* vtkSMComparativeViewProxy::GetRootView()
 {
-  return this->Internal->Views[0];
+  if (this->Internal->Views.size())
+    {
+    return this->Internal->Views[0];
+    }
+  else
+    {
+    return 0;
+    }
 }
 
 //----------------------------------------------------------------------------
@@ -258,16 +275,22 @@ static void vtkCopyClone(vtkSMProxy* source, vtkSMProxy* clone,
   for (; !iterSource->IsAtEnd() && !iterDest->IsAtEnd();
     iterSource->Next(), iterDest->Next())
     {
+
+    // Skip the property if it is in the exceptions list.
     if (exceptions && 
       exceptions->find(iterSource->GetKey()) != exceptions->end())
       {
       continue;
       }
     vtkSMProperty* destProp = iterDest->GetProperty();
+
+    // Skip the property if it is information only or is internal.
     if (destProp->GetInformationOnly() || destProp->GetIsInternal())
       {
       continue;
       }
+
+    // Copy the property from source to dest
     destProp->Copy(iterSource->GetProperty());
     }
 }
@@ -275,6 +298,7 @@ static void vtkCopyClone(vtkSMProxy* source, vtkSMProxy* clone,
 //----------------------------------------------------------------------------
 void vtkSMComparativeViewProxy::AddNewView()
 {
+
   vtkSMViewProxy* rootView = this->GetRootView();
 
   vtkSMProxyManager* pxm = vtkSMProxyManager::GetProxyManager();
@@ -313,12 +337,11 @@ void vtkSMComparativeViewProxy::AddNewView()
 
     vtkSMRepresentationProxy* newRepr = vtkSMRepresentationProxy::SafeDownCast(
       pxm->NewProxy(repr->GetXMLGroup(), repr->GetXMLName()));
-    vtkCopyClone(repr, newRepr); // create a clone.
-    newRepr->UpdateVTKObjects();
+    vtkCopyClone(repr, newRepr); // create a clone
+    newRepr->UpdateVTKObjects(); // create objects
+    data.Link->AddLinkedProxy(newRepr, vtkSMLink::OUTPUT); // link properties
+    newView->AddRepresentation(newRepr);  // add representation to view
 
-    data.Link->AddLinkedProxy(newRepr, vtkSMLink::OUTPUT);
-    newView->AddRepresentation(newRepr);
-      
     // Now update data structure to include this view/repr clone.
     data.Clones[newView] = newRepr;
     newRepr->Delete();
@@ -334,7 +357,7 @@ void vtkSMComparativeViewProxy::RemoveView(vtkSMViewProxy* view)
     return;
     }
 
-  // Remove all representations in this new.
+  // Remove all representations in this view.
   vtkInternal::MapOfReprClones::iterator reprIter;
   for (reprIter = this->Internal->RepresentationClones.begin();
     reprIter != this->Internal->RepresentationClones.end(); ++reprIter)
@@ -371,43 +394,67 @@ void vtkSMComparativeViewProxy::RemoveView(vtkSMViewProxy* view)
 //----------------------------------------------------------------------------
 void vtkSMComparativeViewProxy::AddRepresentation(vtkSMRepresentationProxy* repr)
 {
-  if (repr)
+
+  if (!repr)
     {
-    // Add repr to RootView. Create clones of repr and add them to all other
-    // views.
-    vtkSMViewProxy* rootView = this->GetRootView();
-    rootView->AddRepresentation(repr);
-
-    // We need to save information about the clones we create for this
-    // representation so that we can clean them up later.
-    vtkInternal::RepresentationData data;
-    vtkSMProxyLink* reprLink = vtkSMProxyLink::New();
-    data.Link.TakeReference(reprLink);
-
-    reprLink->AddException("UpdateTime");
-    reprLink->AddLinkedProxy(repr, vtkSMLink::INPUT);
-
-    vtkSMProxyManager* pxm = vtkSMProxyManager::GetProxyManager();
-    vtkInternal::VectorOfViews::iterator iter = this->Internal->Views.begin();
-    iter++; // skip root view.
-    for (; iter != this->Internal->Views.end(); ++iter)
-      {
-      vtkSMRepresentationProxy* newRepr = vtkSMRepresentationProxy::SafeDownCast(
-        pxm->NewProxy(repr->GetXMLGroup(), repr->GetXMLName()));
-      vtkCopyClone(repr, newRepr); // create a clone.
-      newRepr->UpdateVTKObjects();
-
-      reprLink->AddLinkedProxy(newRepr, vtkSMLink::OUTPUT);
-      iter->GetPointer()->AddRepresentation(newRepr);
-      
-      // Now update data structure to include this view/repr clone.
-      data.Clones[iter->GetPointer()] = newRepr;
-      newRepr->Delete();
-      }
-
-    this->Internal->RepresentationClones[repr] = data;
+    return;
     }
 
+  // Add representation to the root view
+  vtkSMViewProxy* rootView = this->GetRootView();
+  rootView->AddRepresentation(repr);
+
+  // Create clones of representation and add them to all other views.
+  // We will save information about the clones we create
+  // so that we can clean them up later.
+  vtkInternal::RepresentationData data;
+
+  // We'll link the clones' properties to the original
+  // representation using a proxy link.  The "UpdateTime"
+  // property will not be linked however.
+  vtkSMProxyLink* reprLink = vtkSMProxyLink::New();
+  data.Link.TakeReference(reprLink);
+  reprLink->AddLinkedProxy(repr, vtkSMLink::INPUT);
+  reprLink->AddException("UpdateTime");
+
+  // Need the proxy manager to create new representations
+  vtkSMProxyManager* pxm = vtkSMProxyManager::GetProxyManager();
+
+
+  // For each view
+  vtkInternal::VectorOfViews::iterator iter = this->Internal->Views.begin();
+  iter++; // skip root view.
+  for (; iter != this->Internal->Views.end(); ++iter)
+    {
+
+    // Create a new representation
+    vtkSMRepresentationProxy* newRepr = vtkSMRepresentationProxy::SafeDownCast(
+      pxm->NewProxy(repr->GetXMLGroup(), repr->GetXMLName()));
+
+    // Made the new representation a clone
+    vtkCopyClone(repr, newRepr);
+    newRepr->UpdateVTKObjects();
+    reprLink->AddLinkedProxy(newRepr, vtkSMLink::OUTPUT);
+
+    // Add the cloned representation to this view
+    iter->GetPointer()->AddRepresentation(newRepr);
+    
+    // Add the cloned representation to the RepresentationData struct
+    // The clone is added to a map where its view is the key.
+    data.Clones[iter->GetPointer()] = newRepr;
+
+    // Clean up this reference
+    newRepr->Delete();
+    }
+
+  // Add the RepresentationData struct to a map
+  // with the original representation as the key
+  this->Internal->RepresentationClones[repr] = data;
+
+  // Signal that representations have changed
+  this->InvokeEvent(vtkCommand::UserEvent);
+
+
   // Override superclass' AddRepresentation since  repr has already been added
   // to the rootView.
 }
@@ -446,6 +493,8 @@ void vtkSMComparativeViewProxy::RemoveRepresentation(vtkSMRepresentationProxy* r
   vtkSMViewProxy* rootView = this->GetRootView();
   rootView->RemoveRepresentation(repr);
 
+  // Signal that representations have changed
+  this->InvokeEvent(vtkCommand::UserEvent);
 
   // Override superclass' RemoveRepresentation since  repr was not added to this
   // view at all, we added it to (and removed from) the root view.
@@ -503,7 +552,7 @@ vtkSMRepresentationProxy* vtkSMComparativeViewProxy::CreateDefaultRepresentation
 }
 
 //----------------------------------------------------------------------------
-void vtkSMComparativeViewProxy::UpdateVisualization()
+void vtkSMComparativeViewProxy::UpdateVisualization(int force)
 {
   if (!this->AnimationSceneX && !this->AnimationSceneY)
     {
@@ -511,8 +560,12 @@ void vtkSMComparativeViewProxy::UpdateVisualization()
     return;
     }
 
-  if (!this->SceneOutdated)
+  if (!this->SceneOutdated && !force && this->ViewUpdateMode != UPDATE_MODE_ALL)
     {
+    if (this->ViewUpdateMode == UPDATE_MODE_ROOT)
+      {
+      this->UpdateRootView();
+      }
     // no need to rebuild.
     return;
     }
@@ -538,7 +591,10 @@ void vtkSMComparativeViewProxy::UpdateVisualization()
     this->UpdateFilmStripVisualization(this->AnimationSceneX);
     }
 
-  for (iter = this->Internal->Views.begin(); 
+    /*
+    // I'm not sure why this block of code was needed?
+    // It called SetUseCache(false) which caused bad behavior.
+    for (iter = this->Internal->Views.begin(); 
        iter != this->Internal->Views.end(); ++iter)
     {
     // Mark all representations as updated; this won't cause any real updation
@@ -546,14 +602,37 @@ void vtkSMComparativeViewProxy::UpdateVisualization()
     iter->GetPointer()->UpdateAllRepresentations();
     iter->GetPointer()->SetUseCache(false);
     }
-
+    */
   this->SceneOutdated = false;
 }
 
+//----------------------------------------------------------------------------
+void vtkSMComparativeViewProxy::UpdateRootView()
+{
+  this->AnimationSceneX->SetAnimationTime(0);
+  vtkSMViewProxy* view = this->Internal->Views[0];
+  if (this->GetShowTimeSteps())
+    {
+    view->SetViewUpdateTime(this->TimeRange[0]);
+    }
+  else
+    {
+    view->SetViewUpdateTime(this->GetViewUpdateTime());
+    }
+  view->SetCacheTime(view->GetCacheTime()+1.0);
+  view->StillRender();
+}
+
 //----------------------------------------------------------------------------
 void vtkSMComparativeViewProxy::UpdateFilmStripVisualization(
   vtkSMAnimationSceneProxy* scene)
 {
+
+  // Set EndTime to the number of views in comparative view.
+  // Think of this as the number of key frames in the animation.
+  // Next, for each frame in the CV, set that frame's time to an
+  // intermediate value between this->TimeRange[0] and this->TimeRange[1].
+
   vtkSMDoubleVectorProperty* dvp = vtkSMDoubleVectorProperty::SafeDownCast(
     scene->GetProperty("EndTime"));
   dvp->SetElement(0, this->Dimensions[0]*this->Dimensions[1]-1);
@@ -563,7 +642,7 @@ void vtkSMComparativeViewProxy::UpdateFilmStripVisualization(
   this->Internal->ActiveIndexY = 0;
 
   double increment =  (this->TimeRange[1] - this->TimeRange[0])/
-    (this->Dimensions[0]*this->Dimensions[1]);
+    (this->Dimensions[0]*this->Dimensions[1]-1);
 
   for (int view_index=0; 
     view_index < this->Dimensions[0]*this->Dimensions[1]; ++view_index)
@@ -572,8 +651,15 @@ void vtkSMComparativeViewProxy::UpdateFilmStripVisualization(
 
     vtkSMViewProxy* view = this->Internal->Views[view_index];
 
-    double time = this->TimeRange[0] + view_index*increment;
-    view->SetViewUpdateTime(time);
+    if (this->GetShowTimeSteps())
+      {
+      double time = this->TimeRange[0] + view_index*increment;
+      view->SetViewUpdateTime(time);
+      }
+    else
+      {
+      view->SetViewUpdateTime(this->GetViewUpdateTime());
+      }
 
     // HACK: This ensure that obsolete cache is never used when the CV is being
     // generated.
@@ -600,6 +686,9 @@ void vtkSMComparativeViewProxy::UpdateComparativeVisualization(
   dvp->SetElement(0, this->Dimensions[1]-1);
   sceneY->UpdateVTKObjects();
 
+  double increment =  (this->TimeRange[1] - this->TimeRange[0])/
+    (this->Dimensions[0]*this->Dimensions[1]-1);
+
   int view_index=0;
   for (int y=0; y < this->Dimensions[1]; y++)
     {
@@ -609,6 +698,16 @@ void vtkSMComparativeViewProxy::UpdateComparativeVisualization(
       sceneX->SetAnimationTime(x);
       vtkSMViewProxy* view = this->Internal->Views[view_index];
 
+      if (this->GetShowTimeSteps())
+        {
+        double time = this->TimeRange[0] + view_index*increment;
+        view->SetViewUpdateTime(time);
+        }
+      else
+        {
+        view->SetViewUpdateTime(this->GetViewUpdateTime());
+        }
+
       // HACK: This ensure that obsolete cache is never used when the CV is being
       // generated.
       view->SetCacheTime(view->GetCacheTime()+1.0);
@@ -625,6 +724,11 @@ void vtkSMComparativeViewProxy::UpdateComparativeVisualization(
 //----------------------------------------------------------------------------
 void vtkSMComparativeViewProxy::GetViews(vtkCollection* collection)
 {
+  if (!collection)
+    {
+    return;
+    }
+
   vtkInternal::VectorOfViews::iterator iter;
   for (iter = this->Internal->Views.begin(); 
        iter != this->Internal->Views.end(); ++iter)
@@ -633,6 +737,97 @@ void vtkSMComparativeViewProxy::GetViews(vtkCollection* collection)
     }
 }
 
+//----------------------------------------------------------------------------
+void vtkSMComparativeViewProxy::GetRepresentationsForView(vtkSMViewProxy* view, 
+      vtkCollection *collection)
+{
+  if (!collection)
+    {
+    return;
+    }
+
+  // Find all representations in this view.
+
+  // For each representation
+  vtkInternal::MapOfReprClones::iterator reprIter;
+  for (reprIter = this->Internal->RepresentationClones.begin();
+    reprIter != this->Internal->RepresentationClones.end(); ++reprIter)
+    {
+
+    if (view == this->GetRootView())
+      {
+      // If the view is the root view, then its representations
+      // are the keys of map we are iterating over
+      collection->AddItem(reprIter->first);
+      continue;
+      }
+
+    // The view is not the root view, so it could be one of the cloned views.
+    // Search the RepresentationData struct for a representation
+    // belonging to the cloned view.
+    vtkInternal::RepresentationData& data = reprIter->second;
+    vtkInternal::RepresentationData::MapOfViewToRepr::iterator cloneIter
+      = data.Clones.find(view);
+    if (cloneIter != data.Clones.end())
+      {
+      // A representation was found, so add it to the collection.
+      vtkSMRepresentationProxy* repr = cloneIter->second.GetPointer();
+      collection->AddItem(repr);
+      }
+    }
+
+}
+
+
+//-----------------------------------------------------------------------------
+bool vtkSMComparativeViewProxy::GetXPropertyAndElement(vtkSMProperty *& prop, int & element)
+{
+  vtkSMProxyProperty* pp = vtkSMProxyProperty::SafeDownCast(this->GetProperty("XCues"));
+  for (unsigned int i=0; i < pp->GetNumberOfProxies(); ++i)
+    {
+    vtkSMAnimationCueProxy* cue = vtkSMAnimationCueProxy::SafeDownCast(pp->GetProxy(i));
+    if (cue && vtkSMPropertyHelper(cue, "Enabled").GetAsInt())
+      {
+      element = cue->GetAnimatedElement();
+      prop = cue->GetAnimatedProperty();
+      return true;
+      }
+    }
+  return false;
+}
+
+//-----------------------------------------------------------------------------
+bool vtkSMComparativeViewProxy::GetYPropertyAndElement(vtkSMProperty *& prop, int & element)
+{
+  vtkSMProxyProperty* pp = vtkSMProxyProperty::SafeDownCast(this->GetProperty("YCues"));
+  for (unsigned int i=0; i < pp->GetNumberOfProxies(); ++i)
+    {
+    vtkSMAnimationCueProxy* cue = vtkSMAnimationCueProxy::SafeDownCast(pp->GetProxy(i));
+    if (cue && vtkSMPropertyHelper(cue, "Enabled").GetAsInt())
+      {
+      element = cue->GetAnimatedElement();
+      prop = cue->GetAnimatedProperty();
+      return true;
+      }
+    }
+  return false;
+}
+
+//----------------------------------------------------------------------------
+const char* vtkSMComparativeViewProxy::GetSuggestedViewType(vtkIdType connectionID)
+{
+  vtkSMViewProxy* rootView = vtkSMViewProxy::SafeDownCast(this->GetSubProxy("RootView"));
+  if (rootView)
+    {
+    vtksys_ios::ostringstream stream;
+    stream << "Comparative" << rootView->GetSuggestedViewType(connectionID);
+    this->Internal->SuggestedViewType = stream.str();
+    return this->Internal->SuggestedViewType.c_str();
+    }
+
+  return this->Superclass::GetSuggestedViewType(connectionID);
+}
+
 //----------------------------------------------------------------------------
 void vtkSMComparativeViewProxy::PrintSelf(ostream& os, vtkIndent indent)
 {
@@ -640,5 +835,8 @@ void vtkSMComparativeViewProxy::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "Dimensions: " << this->Dimensions[0] 
     << ", " << this->Dimensions[1] << endl;
   os << indent << "Mode: " << this->Mode << endl;
+  os << indent << "TimeRange: " 
+    << this->TimeRange[0] <<", " << this->TimeRange[1] << endl;
+  os << indent << "Spacing: " << this->Spacing << endl;
 }
 
