@@ -44,6 +44,7 @@ vtkSMDataLabelRepresentationProxy::vtkSMDataLabelRepresentationProxy()
   this->ActorProxy = 0;
   this->TextPropertyProxy = 0;
   this->GeometryIsValid = 0;
+  this->AppendProxy = 0;
   
   this->CellCenterFilter = 0;
   this->CellActorProxy = 0;
@@ -58,6 +59,7 @@ vtkSMDataLabelRepresentationProxy::~vtkSMDataLabelRepresentationProxy()
   this->UpdateSuppressorProxy = 0;
   this->MapperProxy = 0;
   this->ActorProxy = 0;
+  this->AppendProxy = 0;
   this->TextPropertyProxy = 0;
 
   this->CellCenterFilter = 0;
@@ -116,6 +118,7 @@ bool vtkSMDataLabelRepresentationProxy::BeginCreateVTKObjects()
     return false;
     }
  
+  this->AppendProxy = this->GetSubProxy("Append");
   this->CollectProxy = vtkSMSourceProxy::SafeDownCast(
     this->GetSubProxy("Collect"));
   this->UpdateSuppressorProxy = this->GetSubProxy("UpdateSuppressor");
@@ -123,8 +126,8 @@ bool vtkSMDataLabelRepresentationProxy::BeginCreateVTKObjects()
   this->ActorProxy = this->GetSubProxy("PointLabelProp2D");
   this->TextPropertyProxy =  this->GetSubProxy("PointLabelProperty");
 
-  if (!this->CollectProxy || !this->UpdateSuppressorProxy || !this->MapperProxy
-    || !this->ActorProxy || !this->TextPropertyProxy)
+  if (!this->AppendProxy || !this->CollectProxy || !this->UpdateSuppressorProxy ||
+    !this->MapperProxy || !this->ActorProxy || !this->TextPropertyProxy)
     {
     vtkErrorMacro("Not all required subproxies were defined.");
     return false;
@@ -143,6 +146,7 @@ bool vtkSMDataLabelRepresentationProxy::BeginCreateVTKObjects()
     return false;
     }
 
+  this->AppendProxy->SetServers(vtkProcessModule::DATA_SERVER);
   this->CollectProxy->SetServers(vtkProcessModule::CLIENT_AND_SERVERS);
 
   this->UpdateSuppressorProxy->SetServers(vtkProcessModule::CLIENT_AND_SERVERS);
@@ -190,8 +194,10 @@ bool vtkSMDataLabelRepresentationProxy::EndCreateVTKObjects()
   // There used to be a check to ensure that the data type of input is vtkDataSet.
   // I've taken that out since it would cause excution of the extract selection
   // filter. We can put that back if needed.
-  this->Connect(this->GetInputProxy(), this->CollectProxy, 
+  this->Connect(this->GetInputProxy(), this->AppendProxy, 
     "Input", this->OutputPort);
+
+  this->Connect(this->AppendProxy, this->CollectProxy);
   this->SetupPipeline();
   this->SetupDefaults();
 
@@ -209,7 +215,7 @@ void vtkSMDataLabelRepresentationProxy::SetupPipeline()
     this->CollectProxy->GetProperty("OutputDataType"));
   if (otype != NULL)
     {
-    otype->SetElement(0,4); //vtkUnstructuredGrid
+    otype->SetElement(0, VTK_UNSTRUCTURED_GRID); 
     }
   
   stream << vtkClientServerStream::Invoke
@@ -451,36 +457,40 @@ void vtkSMDataLabelRepresentationProxy::Update(
   this->GeometryIsValid = 1;
   this->UpdateSuppressorProxy->InvokeCommand("ForceUpdate");
   this->Superclass::Update(view);
+
+  // CellCenterFilter is connected after the update suppressor, so it is
+  // essential that we call UpdatePipeline() on it which ensures that the
+  // PostUpdateData() is called on the filters before the mapper.
+  this->CellCenterFilter->UpdatePipeline();
+
+  // This updates the domains for the choosing the correct array name on the
+  // mapper.
+  this->MapperProxy->GetProperty("Input")->UpdateDependentDomains();
+  this->CellMapperProxy->GetProperty("Input")->UpdateDependentDomains();
 //  this->InvokeEvent(vtkSMViewProxy::ForceUpdateEvent);
 }
 
 //-----------------------------------------------------------------------------
-void vtkSMDataLabelRepresentationProxy::SetUpdateTime(double time)
+void vtkSMDataLabelRepresentationProxy::SetUpdateTimeInternal(double time)
 {
-  if (!this->ObjectsCreated)
-    {
-    vtkErrorMacro("Objects not created!");
-    return;
-    }
-
   vtkSMDoubleVectorProperty* dvp = vtkSMDoubleVectorProperty::SafeDownCast(
     this->UpdateSuppressorProxy->GetProperty("UpdateTime"));
   dvp->SetElement(0, time);
-  // UpdateTime is immediate update, so no need to update.
+  this->UpdateSuppressorProxy->UpdateProperty("UpdateTime");
 
   // Go upstream to the reader and mark it modified.
   this->MarkUpstreamModified();
 }
 
 //----------------------------------------------------------------------------
-void vtkSMDataLabelRepresentationProxy::MarkModified(vtkSMProxy* modifiedProxy)
+void vtkSMDataLabelRepresentationProxy::MarkDirty(vtkSMProxy* modifiedProxy)
 {
   if (modifiedProxy != this)
     {
     this->GeometryIsValid = 0;
     }
 
-  this->Superclass::MarkModified(modifiedProxy);
+  this->Superclass::MarkDirty(modifiedProxy);
 }
 
 //-----------------------------------------------------------------------------
