@@ -182,7 +182,8 @@ vtkstd::string FilterDescription(vtkstd::string description)
     {
     description = regExp.match(1);
     }
-  vtksys::SystemTools::ReplaceString(description,"\n","<br/><br/>\n");
+  vtksys::SystemTools::ReplaceString(description,"\n\n","<br/><br/>\n");
+  vtksys::SystemTools::ReplaceString(description,"\n","<br/>\n");
   return description;
 }
 
@@ -559,7 +560,7 @@ bool WriteDomain(vtkSMDomain *dom, ostream &docFile)
     {
     vtkSMNumberOfPartsDomain *nopd =
       vtkSMNumberOfPartsDomain::SafeDownCast(dom);
-    switch (nopd->GetPartMultiplicity())
+    switch (nopd->GetOutputPortMultiplicity())
       {
       case vtkSMNumberOfPartsDomain::SINGLE:
         domainWritten = true;
@@ -844,7 +845,7 @@ void WriteProxies(vtkStringPairList *stringList, vtkStringPairList *labelList,
     proxy = manager->GetPrototypeProxy((*iter).second.c_str(), (*iter).first.c_str());
     if (!proxy)
       {
-      return;
+      continue;
       }
 
     vtksys_ios::ostringstream filename;
