@@ -29,14 +29,14 @@
 #include "vtkSMProxyManager.h"
 #include "vtkSMProxyProperty.h"
 #include "vtkSMRepresentationStrategyVector.h"
-#include "vtkSMSimpleParallelStrategy.h"
+#include "vtkSMUnstructuredDataParallelStrategy.h"
 #include "vtkSMSourceProxy.h"
 #include "vtkPVGenericRenderWindowInteractor.h"
 
 #include <vtkstd/vector>
 
 vtkStandardNewMacro(vtkSMIceTCompositeViewProxy);
-vtkCxxRevisionMacro(vtkSMIceTCompositeViewProxy, "$Revision: 1.19 $");
+vtkCxxRevisionMacro(vtkSMIceTCompositeViewProxy, "$Revision: 1.23 $");
 
 vtkInformationKeyMacro(vtkSMIceTCompositeViewProxy, KD_TREE, ObjectBase);
 //----------------------------------------------------------------------------
@@ -113,16 +113,12 @@ void vtkSMIceTCompositeViewProxy::InitializeForMultiView(vtkSMViewProxy* view)
     }
 
   this->SharedParallelRenderManagerID = 
-    otherView->SharedParallelRenderManagerID.IsNull()?
-    otherView->ParallelRenderManager->GetID():
-    otherView->SharedParallelRenderManagerID; 
+    otherView->ParallelRenderManager->GetID();
 
-  this->SharedMultiViewManagerID = otherView->SharedMultiViewManagerID.IsNull()?
-    (otherView->MultiViewManager? otherView->MultiViewManager->GetID() : 0) : 
-    otherView->SharedMultiViewManagerID; 
+  this->SharedMultiViewManagerID = otherView->MultiViewManager? 
+    otherView->MultiViewManager->GetID() : vtkClientServerID(0);
 
-  this->SharedRenderWindowID = otherView->SharedRenderWindowID.IsNull()?
-    otherView->RenderWindowProxy->GetID() : otherView->SharedRenderWindowID;
+  this->SharedRenderWindowID = otherView->RenderWindowProxy->GetID();
 
   this->Superclass::InitializeForMultiView(view);
 }
@@ -456,9 +452,6 @@ void vtkSMIceTCompositeViewProxy::BeginInteractiveRender()
       stream);
     }
 
-  // When BeginInteractiveRender() is called we are assured that
-  // UpdateAllRepresentations() has been called.
-
   // Give the superclass a chance to decide if it wants to use LOD or not.
   // Let the superclass decide if we are using compositing at all.
   this->Superclass::BeginInteractiveRender();
@@ -569,12 +562,15 @@ void vtkSMIceTCompositeViewProxy::UpdateOrderedCompositingPipeline()
       // Essential to update the representation, so that the
       // KdTree generator gets the updated data to use when generating the
       // KdTree.
+      // It is safe to call Update() on representations or strategies after the
+      // compositing/remote render decision has been made.
       strategyIter->GetPointer()->Update();
       }
     else
       {
-      vtkSMSimpleParallelStrategy* pstrategy = 
-        vtkSMSimpleParallelStrategy::SafeDownCast(strategyIter->GetPointer());
+      vtkSMUnstructuredDataParallelStrategy* pstrategy = 
+        vtkSMUnstructuredDataParallelStrategy::SafeDownCast(
+          strategyIter->GetPointer());
       if (pstrategy && pstrategy->GetDistributedSource())
         {
         ppProducers->AddProxy(pstrategy->GetDistributedSource());
