@@ -33,7 +33,7 @@
 #include "vtkSMPropertyInternals.h"
 
 vtkStandardNewMacro(vtkSMProperty);
-vtkCxxRevisionMacro(vtkSMProperty, "$Revision: 1.57 $");
+vtkCxxRevisionMacro(vtkSMProperty, "$Revision: 1.59 $");
 
 vtkCxxSetObjectMacro(vtkSMProperty, Proxy, vtkSMProxy);
 vtkCxxSetObjectMacro(vtkSMProperty, InformationHelper, vtkSMInformationHelper);
@@ -62,8 +62,10 @@ vtkSMProperty::vtkSMProperty()
   this->IsInternal = 1;
   this->Documentation = 0;
   this->Repeatable = 0;
-
+  
   this->Hints = 0;
+  this->BlockModifiedEvents = false;
+  this->PendingModifiedEvents = false;
 }
 
 //---------------------------------------------------------------------------
@@ -579,6 +581,7 @@ void vtkSMProperty::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "InformationProperty: " << this->InformationProperty << endl;
   os << indent << "Animateable: " << this->Animateable << endl;
   os << indent << "Internal: " << this->IsInternal << endl;
+  os << indent << "Repeatable: " << this->Repeatable << endl;
   os << indent << "Documentation: " << this->Documentation << endl;
   os << indent << "Hints: " ;
   if (this->Hints)
