@@ -38,10 +38,11 @@
 #include <vtksys/ios/sstream>
 
 vtkStandardNewMacro(vtkSMProxy);
-vtkCxxRevisionMacro(vtkSMProxy, "$Revision: 1.100 $");
+vtkCxxRevisionMacro(vtkSMProxy, "$Revision: 1.105 $");
 
 vtkCxxSetObjectMacro(vtkSMProxy, XMLElement, vtkPVXMLElement);
 vtkCxxSetObjectMacro(vtkSMProxy, Hints, vtkPVXMLElement);
+vtkCxxSetObjectMacro(vtkSMProxy, Deprecated, vtkPVXMLElement);
 
 //---------------------------------------------------------------------------
 // Observer for modified event of the property
@@ -134,7 +135,10 @@ vtkSMProxy::vtkSMProxy()
   this->Documentation = vtkSMDocumentation::New();
   this->InMarkModified = 0;
 
+  this->NeedsUpdate = true;
+  
   this->Hints = 0;
+  this->Deprecated = 0;
 }
 
 //---------------------------------------------------------------------------
@@ -175,6 +179,7 @@ vtkSMProxy::~vtkSMProxy()
     }
   this->Documentation->Delete();
   this->SetHints(0);
+  this->SetDeprecated(0);
 }
 
 //---------------------------------------------------------------------------
@@ -317,7 +322,7 @@ void vtkSMProxy::ReportReferences(vtkGarbageCollector* collector)
   /*
   // Report references held by this object that may be in a loop.
   // This includes the consumers.
-  vtkstd::vector<vtkSMProxyInternals::ConsumerInfo>::iterator i = 
+  vtkstd::vector<vtkSMProxyInternals::ConnectionInfo>::iterator i = 
     this->Internals->Consumers.begin();
   for(; i != this->Internals->Consumers.end(); i++)
     {
@@ -345,7 +350,7 @@ void vtkSMProxy::SetServers(vtkTypeUInt32 servers)
     this->Internals->SubProxies.begin();
   for( ; it2 != this->Internals->SubProxies.end(); it2++)
     {
-    it2->second.GetPointer()->SetServersSelf(servers);
+    it2->second.GetPointer()->SetServers(servers);
     }
 }
 
@@ -414,6 +419,12 @@ vtkClientServerID vtkSMProxy::GetID()
   return this->VTKObjectID;
 }
 
+//---------------------------------------------------------------------------
+vtkObjectBase* vtkSMProxy::GetClientSideObject()
+{
+  return vtkProcessModule::GetProcessModule()->GetObjectFromID(this->GetID());
+}
+
 //---------------------------------------------------------------------------
 const char* vtkSMProxy::GetPropertyName(vtkSMProperty* prop)
 {
@@ -631,7 +642,7 @@ void vtkSMProxy::AddPropertyToSelf(
 }
 
 //---------------------------------------------------------------------------
-void vtkSMProxy::UpdateProperty(const char* name, int force)
+bool vtkSMProxy::UpdateProperty(const char* name, int force)
 {
   // This will ensure that the SelfID is assigned properly.
   this->GetSelfID();
@@ -645,17 +656,19 @@ void vtkSMProxy::UpdateProperty(const char* name, int force)
       this->Internals->ExposedProperties.find(name);
     if (eiter == this->Internals->ExposedProperties.end())
       {
-      return;
+      return false;
       }
     const char* subproxy_name =  eiter->second.SubProxyName.c_str();
     const char* property_name = eiter->second.PropertyName.c_str();
     vtkSMProxy * sp = this->GetSubProxy(subproxy_name);
-    if (sp)
+    if (sp && sp->UpdateProperty(property_name, force))
       {
-      sp->UpdateProperty(property_name, force);
+      this->MarkModified(this);
+      return true;
       }
-    return;
+    return false;
     }
+    
   if (it->second.ModifiedFlag || force)
     {
     // In case this property is a self property and causes
@@ -671,11 +684,18 @@ void vtkSMProxy::UpdateProperty(const char* name, int force)
       {
       vtkClientServerStream str;
       prop->AppendCommandToStream(this, &str, this->GetSelfID());
-      pm->SendStream(this->ConnectionID, vtkProcessModule::CLIENT, str);
-
-      // Fire event to let everyone know that a property has been updated.
-      this->InvokeEvent(vtkCommand::UpdatePropertyEvent, (void*)name);
-      this->MarkModified(this);
+      if (str.GetNumberOfMessages() > 0)
+        {
+        pm->SendStream(this->ConnectionID, vtkProcessModule::CLIENT, str);
+  
+        // Fire event to let everyone know that a property has been updated.
+        this->InvokeEvent(vtkCommand::UpdatePropertyEvent, (void*)name);
+        if (!this->InUpdateVTKObjects) // if updating multiple properties
+          {                            // MarkModified() is called only once.
+          this->MarkModified(this);
+          }
+        return true;
+        }
       }
     else
       {
@@ -695,11 +715,17 @@ void vtkSMProxy::UpdateProperty(const char* name, int force)
 
           // Fire event to let everyone know that a property has been updated.
           this->InvokeEvent(vtkCommand::UpdatePropertyEvent, (void*)name);
-          this->MarkModified(this);
+          if (!this->InUpdateVTKObjects) // if updating multiple properties
+            {                            // MarkModified() is called only once.
+            this->MarkModified(this);
+            }
+          return true;  
           }
         }
       }
     }
+    
+    return false;
 }
 
 //---------------------------------------------------------------------------
@@ -959,6 +985,7 @@ void vtkSMProxy::UpdateVTKObjects()
     return;
     }
 
+  bool modified = false;
   if (old_SelfPropertiesModified)
     {
     for (it  = this->Internals->Properties.begin();
@@ -971,6 +998,7 @@ void vtkSMProxy::UpdateVTKObjects()
         this->UpdateProperty(it->first.c_str());
         }
       }
+    modified = true;    
     }
 
   this->InUpdateVTKObjects = 0;
@@ -989,9 +1017,15 @@ void vtkSMProxy::UpdateVTKObjects()
     this->Internals->SubProxies.begin();
   for( ; it2 != this->Internals->SubProxies.end(); it2++)
     {
+    modified = modified || it2->second.GetPointer()->ArePropertiesModified();
     it2->second.GetPointer()->UpdateVTKObjects();
     }
 
+  if (modified)
+    {
+    this->MarkModified(this);
+    }
+    
   this->InvokeEvent(vtkCommand::UpdateEvent, 0);
 }
 
@@ -1108,6 +1142,7 @@ void vtkSMProxy::CreateVTKObjects()
     }
   this->ObjectsCreated = 1;
   this->GetSelfID(); // this will ensure that the SelfID is assigned properly.
+  this->WarnIfDeprecated();
 
   if (this->VTKClassName && this->VTKClassName[0] != '\0')
     {
@@ -1132,6 +1167,23 @@ void vtkSMProxy::CreateVTKObjects()
     }
 }
 
+//---------------------------------------------------------------------------
+bool vtkSMProxy::WarnIfDeprecated()
+{
+  if (this->Deprecated)
+    {
+    vtkWarningMacro("Proxy (" << this->XMLGroup << ", " << this->XMLName 
+      << ")  has been deprecated in ParaView " <<
+      this->Deprecated->GetAttribute("deprecated_in") << 
+      " and will be removed by ParaView " <<
+      this->Deprecated->GetAttribute("to_remove_in") << ". " <<
+      (this->Deprecated->GetCharacterData()? 
+       this->Deprecated->GetCharacterData() : ""));
+    return true;
+    }
+  return false;
+}
+
 //---------------------------------------------------------------------------
 void vtkSMProxy::ReviveVTKObjects()
 {
@@ -1180,12 +1232,16 @@ void vtkSMProxy::ReviveVTKObjects()
            << vtkClientServerStream::End;
     pm->SendStream(this->ConnectionID, this->Servers, stream);
     }
-  vtkSMProxyInternals::ProxyMap::iterator it2 =
-    this->Internals->SubProxies.begin();
-  for( ; it2 != this->Internals->SubProxies.end(); it2++)
-    {
-    it2->second.GetPointer()->ReviveVTKObjects();
-    }
+
+  // * No need to iterate over subproxies, since a call to LoadRevivalState() on
+  // * subproxies will call ReviveVTKObjects() on the subproxies as well. (Look at
+  // * implementation of LoadRevivalState().
+  //vtkSMProxyInternals::ProxyMap::iterator it2 =
+  //  this->Internals->SubProxies.begin();
+  //for( ; it2 != this->Internals->SubProxies.end(); it2++)
+  //  {
+  //  it2->second.GetPointer()->ReviveVTKObjects();
+  //  }
 }
 
 //---------------------------------------------------------------------------
@@ -1274,7 +1330,6 @@ void vtkSMProxy::AddSubProxy(const char* name, vtkSMProxy* proxy)
 
   this->Internals->SubProxies[name] = proxy;
   
-  proxy->AddObserver(vtkCommand::ModifiedEvent, this->SubProxyObserver);
   proxy->AddObserver(vtkCommand::PropertyModifiedEvent, 
     this->SubProxyObserver);
   proxy->AddObserver(vtkCommand::UpdatePropertyEvent, this->SubProxyObserver);
@@ -1400,13 +1455,6 @@ void vtkSMProxy::ExecuteSubProxyEvent(vtkSMProxy* subproxy,
       this->InvokeEvent(vtkCommand::UpdatePropertyEvent, (void*)exposed_name);
       }
     }
-  else if (subproxy && event == vtkCommand::ModifiedEvent)
-    {
-    vtkSMProxy* modifiedProxy = vtkSMProxy::SafeDownCast(
-      reinterpret_cast<vtkObjectBase*>(data));
-    this->MarkConsumersAsModified(modifiedProxy);
-    this->InvokeEvent(vtkCommand::ModifiedEvent, modifiedProxy);
-    }
 
   // Note we are not throwing vtkCommand::UpdateEvent fired by subproxies.
   // Since doing so would imply that this proxy (as well as all its subproxies)
@@ -1418,7 +1466,7 @@ void vtkSMProxy::ExecuteSubProxyEvent(vtkSMProxy* subproxy,
 void vtkSMProxy::AddConsumer(vtkSMProperty* property, vtkSMProxy* proxy)
 {
   int found=0;
-  vtkstd::vector<vtkSMProxyInternals::ConsumerInfo>::iterator i = 
+  vtkstd::vector<vtkSMProxyInternals::ConnectionInfo>::iterator i = 
     this->Internals->Consumers.begin();
   for(; i != this->Internals->Consumers.end(); i++)
     {
@@ -1431,7 +1479,7 @@ void vtkSMProxy::AddConsumer(vtkSMProperty* property, vtkSMProxy* proxy)
 
   if (!found)
     {
-    vtkSMProxyInternals::ConsumerInfo info(property, proxy);
+    vtkSMProxyInternals::ConnectionInfo info(property, proxy);
     this->Internals->Consumers.push_back(info);
     }
   
@@ -1440,7 +1488,7 @@ void vtkSMProxy::AddConsumer(vtkSMProperty* property, vtkSMProxy* proxy)
 //---------------------------------------------------------------------------
 void vtkSMProxy::RemoveConsumer(vtkSMProperty* property, vtkSMProxy* proxy)
 {
-  vtkstd::vector<vtkSMProxyInternals::ConsumerInfo>::iterator i = 
+  vtkstd::vector<vtkSMProxyInternals::ConnectionInfo>::iterator i = 
     this->Internals->Consumers.begin();
   for(; i != this->Internals->Consumers.end(); i++)
     {
@@ -1477,6 +1525,80 @@ vtkSMProperty* vtkSMProxy::GetConsumerProperty(unsigned int idx)
   return this->Internals->Consumers[idx].Property;
 }
 
+//---------------------------------------------------------------------------
+void vtkSMProxy::AddProducer(vtkSMProperty* property, vtkSMProxy* proxy)
+{
+  int found=0;
+  vtkstd::vector<vtkSMProxyInternals::ConnectionInfo>::iterator i = 
+    this->Internals->Producers.begin();
+  for(; i != this->Internals->Producers.end(); i++)
+    {
+    if ( i->Property == property && i->Proxy == proxy )
+      {
+      found = 1;
+      break;
+      }
+    }
+
+  if (!found)
+    {
+    vtkSMProxyInternals::ConnectionInfo info(property, proxy);
+    this->Internals->Producers.push_back(info);
+    }
+  
+}
+
+//---------------------------------------------------------------------------
+void vtkSMProxy::RemoveProducer(vtkSMProperty* property, vtkSMProxy* proxy)
+{
+  vtkstd::vector<vtkSMProxyInternals::ConnectionInfo>::iterator i = 
+    this->Internals->Producers.begin();
+  for(; i != this->Internals->Producers.end(); i++)
+    {
+    if ( i->Property == property && i->Proxy == proxy )
+      {
+      this->Internals->Producers.erase(i);
+      break;
+      }
+    }
+}
+
+//---------------------------------------------------------------------------
+unsigned int vtkSMProxy::GetNumberOfProducers()
+{
+  return this->Internals->Producers.size();
+}
+
+//---------------------------------------------------------------------------
+vtkSMProxy* vtkSMProxy::GetProducerProxy(unsigned int idx)
+{
+  return this->Internals->Producers[idx].Proxy;
+}
+
+//---------------------------------------------------------------------------
+vtkSMProperty* vtkSMProxy::GetProducerProperty(unsigned int idx)
+{
+  return this->Internals->Producers[idx].Property;
+}
+
+//----------------------------------------------------------------------------
+void vtkSMProxy::PostUpdateData()
+{
+  unsigned int numProducers = this->GetNumberOfProducers();
+  for (unsigned int i=0; i<numProducers; i++)
+    {
+    if (this->GetVTKClassName() && this->GetProducerProxy(i)->GetVTKClassName())
+      {
+      }
+    if (this->GetProducerProxy(i)->NeedsUpdate)
+      {
+      this->GetProducerProxy(i)->PostUpdateData();
+      }
+    }
+  this->InvokeEvent(vtkCommand::UpdateDataEvent, 0);
+  this->NeedsUpdate = false;
+}
+
 //----------------------------------------------------------------------------
 void vtkSMProxy::MarkModified(vtkSMProxy* modifiedProxy)
 {
@@ -1500,13 +1622,25 @@ void vtkSMProxy::MarkModified(vtkSMProxy* modifiedProxy)
     {
     this->InMarkModified = 1;
     this->InvokeEvent(vtkCommand::ModifiedEvent, (void*)modifiedProxy);
-    this->MarkConsumersAsModified(modifiedProxy);
+    this->MarkDirty(modifiedProxy);
     this->InMarkModified = 0;
     }
 }
 
 //----------------------------------------------------------------------------
-void vtkSMProxy::MarkConsumersAsModified(vtkSMProxy* modifiedProxy)
+void vtkSMProxy::MarkDirty(vtkSMProxy* modifiedProxy)
+{
+  if (this->NeedsUpdate)
+    {
+    return;
+    }
+    
+  this->MarkConsumersAsDirty(modifiedProxy);
+  this->NeedsUpdate = true;
+}
+
+//----------------------------------------------------------------------------
+void vtkSMProxy::MarkConsumersAsDirty(vtkSMProxy* modifiedProxy)
 {
   unsigned int numConsumers = this->GetNumberOfConsumers();
   for (unsigned int i=0; i<numConsumers; i++)
@@ -1514,7 +1648,7 @@ void vtkSMProxy::MarkConsumersAsModified(vtkSMProxy* modifiedProxy)
     vtkSMProxy* cons = this->GetConsumerProxy(i);
     if (cons)
       {
-      cons->MarkModified(modifiedProxy);
+      cons->MarkDirty(modifiedProxy);
       }
     }
 }
@@ -1624,11 +1758,8 @@ vtkSMProperty* vtkSMProxy::NewProperty(const char* name,
 }
 
 //---------------------------------------------------------------------------
-int vtkSMProxy::ReadXMLAttributes(
-  vtkSMProxyManager* pm, vtkPVXMLElement* element)
+void vtkSMProxy::ReadCoreXMLAttributes(vtkPVXMLElement* element)
 {
-  this->SetXMLElement(element);
-
   const char* className = element->GetAttribute("class");
   if(className)
     {
@@ -1648,11 +1779,6 @@ int vtkSMProxy::ReadXMLAttributes(
     this->SetXMLLabel(xmllabel);
     }
 
-  if (!this->CreateProxyHierarchy(pm, element))
-    {
-    return 0;
-    }
-
   // Locate documentation.
   for (unsigned int cc=0; cc < element->GetNumberOfNestedElements(); ++cc)
     {
@@ -1665,6 +1791,25 @@ int vtkSMProxy::ReadXMLAttributes(
       {
       this->SetHints(subElem);
       }
+    else if (strcmp(subElem->GetName(), "Deprecated") == 0)
+      {
+      this->SetDeprecated(subElem);
+      }
+    }
+}
+
+//---------------------------------------------------------------------------
+int vtkSMProxy::ReadXMLAttributes(
+  vtkSMProxyManager* pm, vtkPVXMLElement* element)
+{
+  this->SetXMLElement(element);
+
+  // Read the common attributes.
+  this->ReadCoreXMLAttributes(element);
+
+  if (!this->CreateProxyHierarchy(pm, element))
+    {
+    return 0;
     }
   this->SetXMLElement(0);
   return 1;
@@ -1733,7 +1878,7 @@ int vtkSMProxy::CreateSubProxiesAndProperties(vtkSMProxyManager* pm,
             }
           else
             {
-            subproxy = pm->NewProxy(subElement, 0);
+            subproxy = pm->NewProxy(subElement, 0, 0);
             }
           if (!subproxy)
             {
@@ -1751,7 +1896,8 @@ int vtkSMProxy::CreateSubProxiesAndProperties(vtkSMProxyManager* pm,
     else
       {
       const char* name = propElement->GetAttribute("name");
-      if (name)
+      vtkstd::string tagName = propElement->GetName();
+      if (name && tagName.find("Property") == (tagName.size()-8))
         {
         this->NewProperty(name, propElement);
         }
@@ -1948,31 +2094,17 @@ int vtkSMProxy::LoadRevivalState(vtkPVXMLElement* revivalElem,
     const char* name = currentElement->GetName();
     if (name && strncmp(name, "VTKObjectID", strlen("VTKObjectID")) == 0)
       {
-      for (unsigned int i=0; 
-           i < currentElement->GetNumberOfNestedElements(); 
-           i++)
+      vtkClientServerID id;
+      int int_id;
+      if (currentElement->GetScalarAttribute("id", &int_id) && int_id)
         {
-        vtkPVXMLElement* element = currentElement->GetNestedElement(i);
-        if (element->GetName() && strcmp(element->GetName(), "Element") == 0)
-          {
-          vtkClientServerID id;
-          int int_id;
-          if (element->GetScalarAttribute("id", &int_id) && int_id)
-            {
-            this->VTKObjectID.ID = int_id;
-            }
-          else
-            {
-            // Some proxies may not have any vtk object they represent (such as
-            // all the animation proxies). It's not an error.
-            // vtkErrorMacro("Element with id attribute not found.");
-            }
-          }
-        else
-          {
-          vtkErrorMacro("Unexpected element, name: " 
-                        << (element->GetName()?element->GetName():"(null)"));
-          }
+        this->VTKObjectID.ID = int_id;
+        }
+      else
+        {
+        // Some proxies may not have any vtk object they represent (such as
+        // all the animation proxies). It's not an error.
+        // vtkErrorMacro("Element with id attribute not found.");
         }
       }
     else if (name && strcmp(name, "SubProxy") == 0)
@@ -2048,16 +2180,11 @@ vtkPVXMLElement* vtkSMProxy::SaveRevivalState(vtkPVXMLElement* root)
   // Save VTKObject ID as well.
   vtkPVXMLElement* idRoot = vtkPVXMLElement::New();
   idRoot->SetName("VTKObjectID");
+  idRoot->AddAttribute("id", 
+                     static_cast<unsigned int>(this->VTKObjectID.ID));
   proxyElement->AddNestedElement(idRoot);
   idRoot->Delete();
 
-  vtkPVXMLElement* elem = vtkPVXMLElement::New();
-  elem->SetName("Element");
-  elem->AddAttribute("id", 
-                     static_cast<unsigned int>(this->VTKObjectID.ID));
-  idRoot->AddNestedElement(elem);
-  elem->Delete();
-
   vtkSMProxyInternals::ProxyMap::iterator iter =
     this->Internals->SubProxies.begin();
   for (; iter != this->Internals->SubProxies.end(); ++iter)
