@@ -175,12 +175,12 @@ public:
   // Update the value of one property (pushed to the server) if it is
   // modified.  If the object has not been created, it will be created
   // first. If force is true, the property is pushed even if it not
-  // modified.
-  void UpdateProperty(const char* name)
+  // modified. Return true if UpdateProperty pushes the property value.
+  bool UpdateProperty(const char* name)
     {
-      this->UpdateProperty(name, 0);
+      return this->UpdateProperty(name, 0);
     }
-  void UpdateProperty(const char* name, int force);
+  bool UpdateProperty(const char* name, int force);
 
   // Description:
   // Convenience method equivalent to UpdateProperty(name, 1).
@@ -267,6 +267,21 @@ public:
   // a property (usually vtkSMProxyProperty)
   vtkSMProperty* GetConsumerProperty(unsigned int idx);
 
+  // Description:
+  // Returns the number of proxies this proxy depends on (uses or is
+  // connected to through the pipeline).
+  unsigned int GetNumberOfProducers();
+
+  // Description:
+  // Returns a proxy this proxy depends on, given index.
+  vtkSMProxy* GetProducerProxy(unsigned int idx);
+
+  // Description:
+  // Returns the property holding a producer proxy given an index. Note
+  // that this is a property of this proxy and it points to the producer
+  // proxy.
+  vtkSMProperty* GetProducerProperty(unsigned int idx);
+
   // Description:
   // Assigned by the XML parser. The name assigned in the XML
   // configuration. Can be used to figure out the origin of the
@@ -309,7 +324,7 @@ public:
 //BTX
   // Description:
   // Set server ids on self and sub-proxies.
-  void SetServers(vtkTypeUInt32 servers);
+  virtual void SetServers(vtkTypeUInt32 servers);
  
   // Description:
   // Return the servers.
@@ -351,14 +366,9 @@ public:
     int proxyPropertyCopyFlag);
   
   // Description:
-  // Calls MarkModified() on all consumers. Sub-classes
-  // should add their functionality and call this.
+  // Calls MarkDirty() and invokes ModifiedEvent.
   virtual void MarkModified(vtkSMProxy* modifiedProxy);
 
-  // Description:
-  // Calls MarkModified() on all consumers.
-  virtual void MarkConsumersAsModified(vtkSMProxy* modifiedProxy);
-
   // Description:
   // Returns the self ID as string. If the name was overwritten
   // with SetName(), it returns that instead.
@@ -409,10 +419,28 @@ public:
   virtual void InitializeAndCopyFromID(vtkClientServerID id);
 //ETX
 
+  // Description:
+  // Saves the state of the proxy. This state can be reloaded
+  // to create a new proxy that is identical the present state of this proxy.
+  virtual vtkPVXMLElement* SaveState(vtkPVXMLElement* root);
+
+  // Description:
+  // Dirty means this algorithm will execute during next update.
+  // This all marks all consumers as dirty.
+  virtual void MarkDirty(vtkSMProxy* modifiedProxy);
+
+  // Description:
+  // Returns the client side object associated with the VTKObjectID (if any).
+  vtkObjectBase* GetClientSideObject();
+
 protected:
   vtkSMProxy();
   ~vtkSMProxy();
 
+  // Description:
+  // Calls MarkDirty() on all consumers.
+  virtual void MarkConsumersAsDirty(vtkSMProxy* modifiedProxy);
+
 
   // Description:
   // Overloaded for garbage collection.
@@ -433,6 +461,7 @@ protected:
   // as such.
   friend class vtkSMCameraLink;
   friend class vtkSMCompoundProxy;
+  friend class vtkSMCompoundSourceProxy;
   friend class vtkSMInputProperty;
   friend class vtkSMOrderedPropertyIterator;
   friend class vtkSMPart;
@@ -618,6 +647,30 @@ protected:
   // Remove all consumers.
   void RemoveAllConsumers();
 
+  // Description:
+  // Called by an proxy/input property to add property, proxy pair
+  // to the list of producers.
+  void AddProducer(vtkSMProperty* property, vtkSMProxy* proxy);
+  
+  // Description:
+  // Remove the property,proxy pair from the list of producers.
+  void RemoveProducer(vtkSMProperty* property, vtkSMProxy* proxy);
+
+  // Description:
+  // This method is called after the algorithm(s) (if any) associated
+  // with this proxy execute. Subclasses overwrite this method to
+  // add necessary functionality.
+  virtual void PostUpdateData();
+
+  // Description:
+  // If a proxy is deprecated, prints a warning.
+  bool WarnIfDeprecated();
+  
+  // When an algorithm proxy is marked modified, NeedsUpdate is
+  // set to true. In PostUpdateData(), NeedsUpdate is set to false.
+  // This is used to keep track of data information validity.
+  bool NeedsUpdate;
+  
   // Description:
   // Creates a new proxy and initializes it by calling ReadXMLAttributes()
   // with the right XML element.
@@ -634,6 +687,11 @@ protected:
   // Read attributes from an XML element.
   virtual int ReadXMLAttributes(vtkSMProxyManager* pm, vtkPVXMLElement* element);
 
+  // Description:
+  // Used by ReadXMLAttributes() to read the very core attributes such as
+  // documentaion, hints, name etc.
+  void ReadCoreXMLAttributes(vtkPVXMLElement* element);
+
   // Description:
   // Handle events fired by subproxies.
   virtual void ExecuteSubProxyEvent(vtkSMProxy* o, unsigned long event, 
@@ -686,15 +744,11 @@ protected:
   int ArePropertiesModified(int selfOnly = 0);
 
   void SetHints(vtkPVXMLElement* hints);
+  void SetDeprecated(vtkPVXMLElement* deprecated);
 
   void SetXMLElement(vtkPVXMLElement* element);
   vtkPVXMLElement* XMLElement;
 
-  // Description:
-  // Saves the state of the proxy. This state can be reloaded
-  // to create a new proxy that is identical the present state of this proxy.
-  virtual vtkPVXMLElement* SaveState(vtkPVXMLElement* root);
-
   // Description:
   // This method saves state information about the proxy
   // which can be used to revive the proxy using server side objects
@@ -716,6 +770,7 @@ protected:
 
   vtkSMDocumentation* Documentation;
   vtkPVXMLElement* Hints;
+  vtkPVXMLElement* Deprecated;
 
   // Flag used to break consumer loops.
   int InMarkModified;
