@@ -34,15 +34,16 @@
 
 class vtkCollection;
 class vtkPVXMLElement;
-class vtkSMCompoundProxy;
+class vtkSMCompoundSourceProxy;
 class vtkSMDocumentation;
 class vtkSMLink;
 class vtkSMProperty;
 class vtkSMProxy;
+class vtkSMProxyManagerExtension;
 class vtkSMProxyManagerObserver;
+class vtkSMProxyManagerProxySet;
 class vtkSMStateLoader;
 class vtkStringList;
-class vtkSMProxyManagerProxySet;
 
 //BTX
 struct vtkSMProxyManagerInternals;
@@ -113,6 +114,14 @@ public:
   vtkSMProxy* GetProxy(vtkIdType connectionID, vtkClientServerID id);
   //ETX
 
+  // Description:
+  // Returns a proxy with the given self ID registered under the indicated
+  // group, if any.
+  vtkSMProxy* GetProxy(const char* groupname, int id);
+  //BTX
+  vtkSMProxy* GetProxy(const char* groupname, vtkClientServerID id);
+  //ETX
+  
   // Description:
   // Returns all proxies registered under the given group with the given name.
   // The collection is cleared before the proxies are added to it.
@@ -218,37 +227,42 @@ public:
   void UnRegisterAllLinks();
   
   // Description:
-  // Register a compound proxy definition with the proxy manager. This
-  // definition (represented as a tree of vtkPVXMLElement objects) can
-  // then be used to instantiate a copy of the compound proxy. See
-  // vtkSMCompoundProxy.h for details.
-  void RegisterCompoundProxyDefinition(const char* name, vtkPVXMLElement* top);
+  // Register a custom proxy definition with the proxy manager. 
+  // This can be a compound proxy definition (look at
+  // vtkSMCompoundSourceProxy.h) or a regular proxy definition.
+  // For all practical purposes, there's no difference between a proxy
+  // definition added using this method or by parsing a server manager
+  // configuration file.
+  void RegisterCustomProxyDefinition(
+    const char* group, const char* name, vtkPVXMLElement* top);
 
   // Description:
-  // Given its name, unregisters a compound proxy definition.
-  void UnRegisterCompoundProxyDefinition(const char* name);
+  // Given its name, unregisters a custom proxy definition.
+  // Note that this can only be used to remove definitions added using
+  // RegisterCustomProxyDefinition(), cannot be used to remove definitions
+  // loaded using vtkSMXMLParser.
+  void UnRegisterCustomProxyDefinition(const char* group, const char* name);
 
   // Description:
-  // Unregisters all registered compound proxy definitions.
-  void UnRegisterCompoundProxyDefinitions();
+  // Unregisters all registered custom proxy definitions.
+  // Note that this can only be used to remove definitions added using
+  // RegisterCustomProxyDefinition(), cannot be used to remove definitions
+  // loaded using vtkSMXMLParser.
+  void UnRegisterCustomProxyDefinitions();
 
   // Description:
-  // Returns a registered compound proxy definition.
-  vtkPVXMLElement* GetCompoundProxyDefinition(const char* name);
+  // Returns a registered proxy definition. 
+  vtkPVXMLElement* GetProxyDefinition(const char* group, const char* name);
 
   // Description:
-  // Creates a compound proxy from compound proxy definition.
-  vtkSMCompoundProxy* NewCompoundProxy(const char* name); 
+  // Load custom proxy definitions and register them.
+  void LoadCustomProxyDefinitions(const char* filename);
+  void LoadCustomProxyDefinitions(vtkPVXMLElement* root);
 
   // Description:
-  // Load compound proxy definitions and register them.
-  void LoadCompoundProxyDefinitions(const char* filename);
-  void LoadCompoundProxyDefinitions(vtkPVXMLElement* root);
-
-  // Description:
-  // Save registered compound proxy definitions.
-  void SaveCompoundProxyDefinitions(const char* filename);
-  void SaveCompoundProxyDefinitions(vtkPVXMLElement* root);
+  // Save registered custom proxy definitions.
+  void SaveCustomProxyDefinitions(const char* filename);
+  void SaveCustomProxyDefinitions(vtkPVXMLElement* root);
 
   // Description:
   // Loads the state of the server manager from XML.
@@ -403,6 +417,15 @@ public:
   // this method will return 1.
   int GetVersionPatch();
 
+  // Description:
+  // Register a proxy manager extension. Returns true if the registration is
+  // successful.
+  bool RegisterExtension(vtkSMProxyManagerExtension* ext);
+
+  // Description:
+  // Unregister a previously register extension.
+  void UnRegisterExtension(vtkSMProxyManagerExtension* ext);
+
 protected:
   vtkSMProxyManager();
   ~vtkSMProxyManager();
@@ -424,7 +447,8 @@ protected:
   // Description:
   // Given an XML element and group name create a proxy 
   // and all of it's properties.
-  vtkSMProxy* NewProxy(vtkPVXMLElement* element, const char* groupname);
+  vtkSMProxy* NewProxy(vtkPVXMLElement* element, 
+    const char* groupname, const char* proxyname);
 
   // Description:
   // Given the proxy name and group name, returns the XML element for
