@@ -32,7 +32,7 @@
 #include "vtkStdString.h"
 
 vtkStandardNewMacro(vtkSMProxyProperty);
-vtkCxxRevisionMacro(vtkSMProxyProperty, "$Revision: 1.49 $");
+vtkCxxRevisionMacro(vtkSMProxyProperty, "$Revision: 1.54 $");
 
 struct vtkSMProxyPropertyInternals
 {
@@ -51,7 +51,7 @@ vtkSMProxyProperty::vtkSMProxyProperty()
   this->RepeatCommand = 0;
   this->RemoveCommand = 0;
   this->IsInternal = 0;
-  this->AddConsumer = 1;
+  this->NullOnEmpty = 0;
 }
 
 //---------------------------------------------------------------------------
@@ -122,6 +122,7 @@ void vtkSMProxyProperty::AppendCommandToStreamWithRemoveCommand(
     vtkSMProxy* toAppend = iter1->GetPointer();
     this->AppendProxyToStream(toAppend, str, objectId, 1);
     toAppend->RemoveConsumer(this, cons);
+    cons->RemoveProducer(this, toAppend);
     }
 
   // Add the proxies to add.
@@ -133,10 +134,8 @@ void vtkSMProxyProperty::AppendCommandToStreamWithRemoveCommand(
     // Keep track of all proxies that point to this as a
     // consumer so that we can remove this from the consumer
     // list later if necessary.
-    if (this->AddConsumer)
-      {
-      toAppend->AddConsumer(this, cons);
-      }
+    toAppend->AddConsumer(this, cons);
+    cons->AddProducer(this, toAppend);
     this->AppendProxyToStream(toAppend, str, objectId, 0);
     }
  
@@ -184,12 +183,20 @@ void vtkSMProxyProperty::AppendCommandToStream(
       // consumer so that we can remove this from the consumer
       // list later if necessary
       this->AddPreviousProxy(proxy);
-      if (proxy && this->AddConsumer)
+      if (proxy)
         {
         proxy->AddConsumer(this, cons);
+        cons->AddProducer(this, proxy);
         }
       this->AppendProxyToStream(proxy, str, objectId, 0);
       }
+
+    // When no proxies are present in the property, we call the command on
+    // the server side object with NULL argument.
+    if (numProxies == 0 && !this->CleanCommand && this->NullOnEmpty)
+      {
+      this->AppendProxyToStream(NULL, str, objectId, 0);
+      }
     }
 }
 
@@ -269,6 +276,7 @@ void vtkSMProxyProperty::RemoveConsumerFromPreviousProxies(vtkSMProxy* cons)
     if (it->GetPointer())
       {
       it->GetPointer()->RemoveConsumer(this, cons);
+      cons->RemoveProducer(this, it->GetPointer());
       }
     }
 }
@@ -385,7 +393,8 @@ unsigned int vtkSMProxyProperty::RemoveProxy(vtkSMProxy* proxy, int modify)
 //---------------------------------------------------------------------------
 int vtkSMProxyProperty::SetProxy(unsigned int idx, vtkSMProxy* proxy)
 {
-  if (proxy == this->PPInternals->Proxies[idx])
+  if (this->PPInternals->Proxies.size() > idx &&
+      proxy == this->PPInternals->Proxies[idx])
     {
     return 1;
     }
@@ -401,6 +410,10 @@ int vtkSMProxyProperty::SetProxy(unsigned int idx, vtkSMProxy* proxy)
       }
     }
   this->RemoveAllUncheckedProxies();
+  if (this->PPInternals->Proxies.size() <= idx)
+    {
+    this->PPInternals->Proxies.resize(idx+1);
+    }
 
   this->PPInternals->Proxies[idx] = proxy;
   this->Modified();
@@ -453,6 +466,19 @@ void vtkSMProxyProperty::RemoveAllProxies(int modify)
     }
 }
 
+//---------------------------------------------------------------------------
+void vtkSMProxyProperty::SetNumberOfProxies(unsigned int num)
+{
+  if (num != 0)
+    {
+    this->PPInternals->Proxies.resize(num);
+    }
+  else
+    {
+    this->PPInternals->Proxies.clear();
+    }
+}
+
 //---------------------------------------------------------------------------
 unsigned int vtkSMProxyProperty::GetNumberOfProxies()
 {
@@ -502,12 +528,13 @@ int vtkSMProxyProperty::ReadXMLAttributes(vtkSMProxy* parent,
     this->SetRemoveCommand(remove_command);
     }
 
-  int add_consumer;
-  if (element->GetScalarAttribute("add_consumer", & add_consumer))
+  int null_on_empty;
+  if (element->GetScalarAttribute("null_on_empty", &null_on_empty))
     {
-    this->SetAddConsumer(add_consumer);
+    this->SetNullOnEmpty(null_on_empty);
     }
 
+
   if (this->RemoveCommand && !this->GetUpdateSelf())
     {
     vtkErrorMacro("Due to reference counting issue, remove command can "
