@@ -22,6 +22,7 @@
 #include "vtkCollection.h"
 #include "vtkCollectionIterator.h"
 #include "vtkCommand.h"
+#include "vtkDataArray.h"
 #include "vtkDoubleArray.h"
 #include "vtkErrorCode.h"
 #include "vtkExtractSelectedFrustum.h"
@@ -32,6 +33,7 @@
 #include "vtkInformationIntegerKey.h"
 #include "vtkInstantiator.h"
 #include "vtkObjectFactory.h"
+#include "vtkPointData.h"
 #include "vtkProcessModuleConnectionManager.h"
 #include "vtkProcessModule.h"
 #include "vtkPVClientServerIdCollectionInformation.h"
@@ -58,12 +60,10 @@
 #include "vtkSMSelectionHelper.h"
 #include "vtkSMSourceProxy.h"
 #include "vtkSMStringVectorProperty.h"
+#include "vtkSMUtilities.h"
 #include "vtkTimerLog.h"
 #include "vtkWindowToImageFilter.h"
 
-#include "vtkSMMultiProcessRenderView.h"
-#include "vtkProp3DCollection.h"
-
 #include <vtksys/SystemTools.hxx>
 #include <vtkstd/map>
 #include <vtkstd/set>
@@ -89,7 +89,7 @@ inline bool SetIntVectorProperty(vtkSMProxy* proxy, const char* pname,
 }
 
 //-----------------------------------------------------------------------------
-vtkCxxRevisionMacro(vtkSMRenderViewProxy, "$Revision: 1.50.2.1 $");
+vtkCxxRevisionMacro(vtkSMRenderViewProxy, "$Revision: 1.75 $");
 vtkStandardNewMacro(vtkSMRenderViewProxy);
 
 vtkInformationKeyMacro(vtkSMRenderViewProxy, LOD_RESOLUTION, Integer);
@@ -126,6 +126,7 @@ vtkSMRenderViewProxy::vtkSMRenderViewProxy()
   this->RenderTimer = vtkTimerLog::New();
   this->ResetPolygonsPerSecondResults();
   this->MeasurePolygonsPerSecond = 0;
+  this->UseOffscreenRenderingForScreenshots = 0;
 
   this->LODThreshold = 0.0;
 
@@ -135,6 +136,8 @@ vtkSMRenderViewProxy::vtkSMRenderViewProxy()
   this->SetLODResolution(50);
   this->Information->Set(USE_ORDERED_COMPOSITING(), 0);
   this->Information->Set(USE_COMPOSITING(), 0);
+
+  this->LightKitAdded = false;
 }
 
 //-----------------------------------------------------------------------------
@@ -173,7 +176,8 @@ vtkSMRepresentationStrategy* vtkSMRenderViewProxy::NewStrategyInternal(
   vtkSMProxyManager* pxm = vtkSMObject::GetProxyManager();
   vtkSMRepresentationStrategy* strategy = 0;
 
-  if ((dataType == VTK_POLY_DATA) || dataType == VTK_UNIFORM_GRID)
+  if (dataType == VTK_POLY_DATA || dataType == VTK_UNIFORM_GRID || 
+    dataType == VTK_IMAGE_DATA)
     {
     strategy = vtkSMRepresentationStrategy::SafeDownCast(
       pxm->NewProxy("strategies", "PolyDataStrategy"));
@@ -324,6 +328,7 @@ void vtkSMRenderViewProxy::EndCreateVTKObjects()
   this->Superclass::EndCreateVTKObjects();
 
   vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
+  vtkPVOptions* pvoptions = pm->GetOptions();
   
   // Set all the client side pointers.
   this->Renderer = vtkRenderer::SafeDownCast(
@@ -340,7 +345,10 @@ void vtkSMRenderViewProxy::EndCreateVTKObjects()
   // Set the helper for interaction.
   this->Interactor->SetPVRenderView(this->RenderViewHelper);
 
-  if (pm->GetOptions()->GetUseStereoRendering())
+  // Mark 2D renderer as non-interactive, since it's a slave to the 3D renderer.
+  this->Renderer2D->SetInteractive(0);
+
+  if (pvoptions->GetUseStereoRendering())
     {
     SetIntVectorProperty(this->RenderWindowProxy, "StereoCapableWindow", 1);
     SetIntVectorProperty(this->RenderWindowProxy, "StereoRender", 1);
@@ -376,7 +384,11 @@ void vtkSMRenderViewProxy::EndCreateVTKObjects()
   vtkCommand* observer = this->GetObserver();
   this->Renderer->AddObserver(
     vtkCommand::ResetCameraClippingRangeEvent, observer);
-  this->Renderer->AddObserver(vtkCommand::StartEvent, observer);
+
+  // Synchronize the renderers before any of the parallel render manager code
+  // takes control. Otherwise, interprocess communication may become
+  // interleaved because iceT as well as this observer make MPI calls.
+  this->RenderWindow->AddObserver(vtkCommand::StartEvent, observer, 1);
   this->RenderWindow->AddObserver(vtkCommand::AbortCheckEvent, 
     this->GetObserver());
 
@@ -397,14 +409,19 @@ void vtkSMRenderViewProxy::EndCreateVTKObjects()
     pm->SendStream(this->ConnectionID, 
       vtkProcessModule::RENDER_SERVER, stream);
 
-    // Ensure that the client always does onscreen rendering.
-    stream  << vtkClientServerStream::Invoke
-            << this->RenderWindowProxy->GetID()
-            << "SetOffScreenRendering"
-            << 0
-            << vtkClientServerStream::End;
-    pm->SendStream(this->ConnectionID, 
-      vtkProcessModule::CLIENT, stream);
+    // Ensure that the client always does onscreen rendering, unless we are
+    // running in batch mode and offscreen rendering is requested.
+    if (pvoptions->GetUseOffscreenRendering() == 0)
+      {
+      // pvoptions->GetUseOffscreenRendering() can be set only in batch mode.
+      stream  << vtkClientServerStream::Invoke
+              << this->RenderWindowProxy->GetID()
+              << "SetOffScreenRendering"
+              << 0
+              << vtkClientServerStream::End;
+      pm->SendStream(this->ConnectionID, 
+        vtkProcessModule::CLIENT, stream);
+      }
     }
 
   this->Interactor->Enable();
@@ -428,7 +445,7 @@ void vtkSMRenderViewProxy::ProcessEvents(vtkObject* caller, unsigned long eventI
     // renderer.
     this->ResetCameraClippingRange();
     }
-  else if (eventId == vtkCommand::StartEvent && caller == this->Renderer)
+  else if (eventId == vtkCommand::StartEvent && caller == this->RenderWindow)
     {
     // At the start of every render ensure that the 2D renderer and 3D renderer
     // have the same camera.
@@ -447,6 +464,13 @@ void vtkSMRenderViewProxy::SetUseLight(int enable)
     return;
     }
 
+  if ((enable==1) == this->LightKitAdded)
+    {
+    // nothing to do.
+    return;
+    }
+
+  this->LightKitAdded = enable==1;
   vtkClientServerStream stream;
   vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
   stream  << vtkClientServerStream::Invoke
@@ -473,6 +497,11 @@ int vtkSMRenderViewProxy::GetLODResolution()
 //-----------------------------------------------------------------------------
 void vtkSMRenderViewProxy::SetUseLOD(bool use_lod)
 {
+  if (this->GetUseLOD() != use_lod)
+    {
+    // we only need to invalidate the DisplayDataSize, but we do both for now.
+    this->InvalidateDataSizes();
+    }
   this->Information->Set(USE_LOD(), use_lod);
 }
 
@@ -494,29 +523,10 @@ void vtkSMRenderViewProxy::BeginInteractiveRender()
   vtkRenderWindow *renWin = this->GetRenderWindow(); 
   renWin->SetDesiredUpdateRate(5.0);
 
-  bool using_lod = this->GetUseLOD();
-
-  // Determine if we are using LOD or not.
-  if (this->GetLODDecision())
-    {
-    this->SetUseLOD(true);
-    if (!using_lod)
-      {
-      // We changed LOD decision, implying that the LOD pipelines for all
-      // representations aren't up-to-date, ensure that they are updated.
-      // This can be done by setting the ForceRepresentationUpdate to true
-      // as a result of which the superclass will update the representations once 
-      // again before performing render.
-      this->SetForceRepresentationUpdate(true);
-      }
-    }
-  else
-    {
-    this->SetUseLOD(false);
-    // Even if using_lod was true, we don't need to UpdateAllRepresentations
-    // since calling UpdateAllRepresentations when UseLOD is true updates the
-    // full-res pipeline anyways.
-    }
+  // Determine if we are using LOD or not
+  // This may partially update the representation pipelines to get correct data
+  // size information.
+  this->SetUseLOD(this->GetLODDecision());
 
   this->GetRenderer()->ResetCameraClippingRange();
   this->Superclass::BeginInteractiveRender();
@@ -539,10 +549,7 @@ void vtkSMRenderViewProxy::BeginStillRender()
   // Interactive renders get called through the PVInteractorStyles
   // which cal ResetCameraClippingRange on the Renderer.
   // We could convert them to call a method on the module directly ...
-  if ( ! vtkProcessModule::GetStreamBlock())
-    {
-    this->GetRenderer()->ResetCameraClippingRange();
-    }
+  this->GetRenderer()->ResetCameraClippingRange();
   renWindow->SetDesiredUpdateRate(0.002);
 
   this->SetUseLOD(false);
@@ -789,31 +796,11 @@ void vtkSMRenderViewProxy::ResetCamera(double bds[6])
 void vtkSMRenderViewProxy::ResetCameraClippingRange()
 {
   double bds[6];
-  double range1[2];
-  double range2[2];
   vtkRenderer* ren = this->GetRenderer();
-  // Get default clipping range.
-  // Includes 3D widgets but not all processes.
-  ren->GetActiveCamera()->GetClippingRange(range1);
 
   this->ComputeVisiblePropBounds(bds);
   ren->ResetCameraClippingRange(bds);
   
-  // Get part clipping range.
-  // Includes all process partitions, but not 3d Widgets.
-  ren->GetActiveCamera()->GetClippingRange(range2);
-
-  // Merge
-  if (range1[0] < range2[0])
-    {
-    range2[0] = range1[0];
-    }
-  if (range1[1] > range2[1])
-    {
-    range2[1] = range1[1];
-    }
-  // Includes all process partitions and 3D Widgets.
-  ren->GetActiveCamera()->SetClippingRange(range2);
   this->Modified();
 }
 
@@ -831,18 +818,16 @@ void vtkSMRenderViewProxy::ComputeVisiblePropBounds(double bds[6])
       iter->GetCurrentObject());
     if (repr && repr->GetVisibility())
       {
-      vtkPVDataInformation* info = repr->GetDisplayedDataInformation();
-      if (!info)
+      double tmp[6];
+      if (repr->GetBounds(tmp))
         {
-        continue;
+        if (tmp[0] < bds[0]) { bds[0] = tmp[0]; }  
+        if (tmp[1] > bds[1]) { bds[1] = tmp[1]; }  
+        if (tmp[2] < bds[2]) { bds[2] = tmp[2]; }  
+        if (tmp[3] > bds[3]) { bds[3] = tmp[3]; }  
+        if (tmp[4] < bds[4]) { bds[4] = tmp[4]; }  
+        if (tmp[5] > bds[5]) { bds[5] = tmp[5]; }  
         }
-      double *tmp = info->GetBounds();
-      if (tmp[0] < bds[0]) { bds[0] = tmp[0]; }  
-      if (tmp[1] > bds[1]) { bds[1] = tmp[1]; }  
-      if (tmp[2] < bds[2]) { bds[2] = tmp[2]; }  
-      if (tmp[3] > bds[3]) { bds[3] = tmp[3]; }  
-      if (tmp[4] < bds[4]) { bds[4] = tmp[4]; }  
-      if (tmp[5] > bds[5]) { bds[5] = tmp[5]; }  
       }
     }
 
@@ -1018,14 +1003,39 @@ vtkPVXMLElement* vtkSMRenderViewProxy::SaveState(vtkPVXMLElement* root)
   return this->Superclass::SaveState(root);
 }
 
+//-----------------------------------------------------------------------------
+void vtkSMRenderViewProxy::SetUseOffscreen(int offscreen)
+{
+  if (this->ObjectsCreated)
+    {
+    this->GetRenderWindow()->SetOffScreenRendering(offscreen);
+    }
+}
+
+//-----------------------------------------------------------------------------
+int vtkSMRenderViewProxy::GetUseOffscreen()
+{
+  if (this->ObjectsCreated)
+    {
+    return this->GetRenderWindow()->GetOffScreenRendering();
+    }
+  return 0;
+}
+
 //-----------------------------------------------------------------------------
 vtkImageData* vtkSMRenderViewProxy::CaptureWindow(int magnification)
 {
   // Offscreen rendering is not functioning properly on the mac.
   // Do not use it.
 #if !defined(__APPLE__)
-  this->GetRenderWindow()->SetOffScreenRendering(1);
+  int useOffscreenRenderingForScreenshots = this->UseOffscreenRenderingForScreenshots;
+  int prevOffscreen = this->GetRenderWindow()->GetOffScreenRendering();
+  if (useOffscreenRenderingForScreenshots && !prevOffscreen)
+    {
+    this->GetRenderWindow()->SetOffScreenRendering(1);
+    }
 #endif
+
   this->GetRenderWindow()->SwapBuffersOff();
   this->StillRender();
 
@@ -1042,8 +1052,42 @@ vtkImageData* vtkSMRenderViewProxy::CaptureWindow(int magnification)
 
   this->GetRenderWindow()->SwapBuffersOn();
   this->GetRenderWindow()->Frame();
+
 #if !defined(__APPLE__)
-  this->GetRenderWindow()->SetOffScreenRendering(0);
+  if (useOffscreenRenderingForScreenshots && !prevOffscreen)
+    {
+    this->GetRenderWindow()->SetOffScreenRendering(0);
+    }
+
+  if (useOffscreenRenderingForScreenshots)
+    {
+    vtkDataArray* scalars = capture->GetPointData()->GetScalars();
+    bool invalid_image = true;
+    for (int comp=0; comp < scalars->GetNumberOfComponents(); comp++)
+      {
+      double range[2];
+      scalars->GetRange(range, comp);
+      if (range[0] != 0.0 || range[1] != 0.0)
+        {
+        invalid_image = false;
+        break;
+        }
+      }
+
+    if (invalid_image)
+      {
+      // free up current image.
+      capture->Delete();
+      capture = 0;
+      vtkWarningMacro("Disabling offscreen rendering since empty image was detected.");
+      this->UseOffscreenRenderingForScreenshots = false;
+      if (prevOffscreen)
+        {
+        this->GetRenderWindow()->SetOffScreenRendering(0);
+        }
+      return this->CaptureWindow(magnification);
+      }
+    }
 #endif
 
   // Update image extents based on ViewPosition
@@ -1058,6 +1102,14 @@ vtkImageData* vtkSMRenderViewProxy::CaptureWindow(int magnification)
   return capture;
 }
 
+//-----------------------------------------------------------------------------
+int vtkSMRenderViewProxy::WriteImage(const char* filename, int magnification)
+{
+  vtkSmartPointer<vtkImageData> shot;
+  shot.TakeReference(this->CaptureWindow(magnification));
+  return vtkSMUtilities::SaveImage(shot, filename);
+}
+
 //-----------------------------------------------------------------------------
 int vtkSMRenderViewProxy::WriteImage(const char* filename,
   const char* writerName, int magnification)
@@ -1156,7 +1208,7 @@ vtkIdType vtkSMRenderViewProxy::GetTotalNumberOfPolygons()
       iter->GetCurrentObject());
     if (repr && repr->GetVisibility())
       {
-      vtkPVDataInformation* info = repr->GetDisplayedDataInformation();
+      vtkPVDataInformation* info = repr->GetRepresentedDataInformation(true);
       if (!info)
         {
         continue;
@@ -1179,45 +1231,55 @@ void vtkSMRenderViewProxy::ResetPolygonsPerSecondResults()
 }
 
 //-----------------------------------------------------------------------------
-int vtkSMRenderViewProxy::IsSelectionAvailable()
-{ 
-  //check if we are not supposed to turn compositing on (in parallel)
-  //the paraview gui uses a big number to say never composite
-  //it we are not supposed to turn it on, then disallow selections
-  double compThresh = 0;
-  vtkSMMultiProcessRenderView *me2 = 
-    vtkSMMultiProcessRenderView::SafeDownCast(this);
-  if (me2 != NULL)
+bool vtkSMRenderViewProxy::IsSelectionAvailable()
+{
+  const char* msg = this->IsSelectVisibleCellsAvailable();
+  if (msg)
     {
-    compThresh = me2->GetRemoteRenderThreshold();
-    if (compThresh > 100.0) //the highest setting in the paraview gui
-      {
-      return 0;
-      }
-
-    if (!me2->GetRemoteRenderAvailable())
-      {
-      // Cannot remote render.
-      return 0;
-      }
+    vtkErrorMacro(<< msg);
+    return false;
     }
 
+  return true;
+}
+
+//-----------------------------------------------------------------------------
+const char* vtkSMRenderViewProxy::IsSelectVisibleCellsAvailable()
+{ 
   //check if we don't have enough color depth to do color buffer selection
   //if we don't then disallow selection
   int rgba[4];
   vtkRenderWindow *rwin = this->GetRenderWindow();
   if (!rwin)
     {
-    return 0;
+    return "No render window available";
     }
+
   rwin->GetColorBufferSizes(rgba);
   if (rgba[0] < 8 || rgba[1] < 8 || rgba[2] < 8)
     {
-    return 0;
+    return "Selection not supported due to insufficient color depth.";
     }
 
   //yeah!
-  return 1;
+  return NULL;
+}
+
+//-----------------------------------------------------------------------------
+const char* vtkSMRenderViewProxy::IsSelectVisiblePointsAvailable()
+{
+  const char* msg = this->IsSelectVisibleCellsAvailable();
+  if (msg)
+    {
+    return msg;
+    }
+
+  vtkRenderWindow *rwin = this->GetRenderWindow();
+  if (!rwin || !rwin->GetStencilCapable())
+    {
+    return "Stencil buffers are not supported.";
+    }
+  return NULL;
 }
 
 //-----------------------------------------------------------------------------
@@ -1311,7 +1373,8 @@ bool vtkSMRenderViewProxy::SelectOnSurface(unsigned int x0,
   //   Will returns a surface selection in terms of cells selected on the 
   //   visible props from all representations.
   vtkSelection* surfaceSel = this->SelectVisibleCells(x0, y0, x1, y1, ofPoints);
-  
+  // cout << surfaceSel->GetNumberOfChildren() << endl;
+
   if (!multiple_selections)
     {
     vtkSMRenderViewProxyShrinkSelection(surfaceSel);
@@ -1356,7 +1419,7 @@ bool vtkSMRenderViewProxy::SelectFrustum(unsigned int x0,
                                          vtkCollection* selectedRepresentations,
                                          vtkCollection* selectionSources,
                                          vtkCollection* frustumSelections,
-                                         bool vtkNotUsed(multiple_selections),
+                                         bool multiple_selections,
                                          bool ofPoints/*=false*/)
 {
   int displayRectangle[4] = {x0, y0, x1, y1};
@@ -1460,7 +1523,7 @@ bool vtkSMRenderViewProxy::SelectFrustum(unsigned int x0,
       {
       continue;
       }
-    vtkPVDataInformation* datainfo = repr->GetDisplayedDataInformation();
+    vtkPVDataInformation* datainfo = repr->GetRepresentedDataInformation(true);
     if (!datainfo)
       {
       continue;
@@ -1485,7 +1548,10 @@ bool vtkSMRenderViewProxy::SelectFrustum(unsigned int x0,
       selectedRepresentations->AddItem(repr);
 
       selectionSource->Delete();
-      break;
+      if(!multiple_selections)
+        {
+        break;
+        }
       }
     }
 
@@ -1531,7 +1597,7 @@ vtkSelection* vtkSMRenderViewProxy::SelectVisibleCells(unsigned int x0,
       continue;
       }
 
-    vtkPVDataInformation *gi = repr->GetDisplayedDataInformation();
+    vtkPVDataInformation *gi = repr->GetRepresentedDataInformation(true);
     if (!gi)
       {
       continue;
@@ -1639,18 +1705,6 @@ vtkSelection* vtkSMRenderViewProxy::SelectVisibleCells(unsigned int x0,
     this->ForceTriStripUpdate = 0;
     }
 
-  double compThresh = 0.0;
-  //Force parallel compositing on for the selection render.
-  //TODO: intelligently code dataserver rank into originalcellids to
-  //make this ugly hack unecessary.
-  vtkSMMultiProcessRenderView *me2 = 
-    vtkSMMultiProcessRenderView::SafeDownCast(this);
-  if (me2 != NULL)
-    {
-    compThresh = me2->GetRemoteRenderThreshold();
-    me2->SetRemoteRenderThreshold(0.0);
-    }      
-
   if (ofPoints)
     {
     pti->SetDoVertices(1);
@@ -1704,12 +1758,6 @@ vtkSelection* vtkSMRenderViewProxy::SelectVisibleCells(unsigned int x0,
     this->SetUseTriangleStrips(1);
     }
 
-  //Force parallel compositing on for the selection render.
-  if (me2 != NULL)
-    {
-    me2->SetRemoteRenderThreshold(compThresh);
-    }      
-
   for (int i = 0; i < numlayers; i++)
     {
     vtkObject *anObj = rcol->GetItemAsObject(i);
@@ -1799,13 +1847,25 @@ vtkSMRepresentationProxy* vtkSMRenderViewProxy::CreateDefaultRepresentation(
       pxm->NewProxy("representations", "UniformGridRepresentation"));
     }
 
-  return vtkSMRepresentationProxy::SafeDownCast(
+  prototype = pxm->GetPrototypeProxy("representations",
+    "GeometryRepresentation");
+  pp = vtkSMInputProperty::SafeDownCast(
+    prototype->GetProperty("Input"));
+  pp->RemoveAllUncheckedProxies();
+  pp->AddUncheckedInputConnection(source, opport);
+  bool g = (pp->IsInDomains()>0);
+  pp->RemoveAllUncheckedProxies();
+  if (g)
+    {
+    return vtkSMRepresentationProxy::SafeDownCast(
       pxm->NewProxy("representations", "GeometryRepresentation"));
+    }
+
+  return 0;
 }
 
 //-----------------------------------------------------------------------------
-const char* vtkSMRenderViewProxy::GetSuggestedRenderViewType(
-  vtkIdType connectionID)
+const char* vtkSMRenderViewProxy::GetSuggestedViewType(vtkIdType connectionID)
 {
   vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
 
@@ -1863,6 +1923,8 @@ void vtkSMRenderViewProxy::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "RenderWindow: " << this->RenderWindow << endl;
   os << indent << "MeasurePolygonsPerSecond: " 
     << this->MeasurePolygonsPerSecond << endl;
+  os << indent << "UseOffscreenRenderingForScreenshots: "
+    << this->UseOffscreenRenderingForScreenshots << endl;
   os << indent << "AveragePolygonsPerSecond: " 
     << this->AveragePolygonsPerSecond << endl;
   os << indent << "MaximumPolygonsPerSecond: " 
