@@ -37,7 +37,7 @@
 #include "vtkSMStringVectorProperty.h"
 
 vtkStandardNewMacro(vtkSMUnstructuredGridVolumeRepresentationProxy);
-vtkCxxRevisionMacro(vtkSMUnstructuredGridVolumeRepresentationProxy, "$Revision: 1.13 $");
+vtkCxxRevisionMacro(vtkSMUnstructuredGridVolumeRepresentationProxy, "$Revision: 1.16 $");
 //----------------------------------------------------------------------------
 vtkSMUnstructuredGridVolumeRepresentationProxy::vtkSMUnstructuredGridVolumeRepresentationProxy()
 {
@@ -289,8 +289,8 @@ void vtkSMUnstructuredGridVolumeRepresentationProxy::DetermineVolumeSupport()
     this->VolumeFilter->GetProperty("Input")->GetDomain("input_type"));
   if (domain && domain->IsInDomain(this->GetInputProxy(), this->OutputPort))
     {
-
-    vtkPVDataInformation* datainfo = this->GetInputProxy()->GetDataInformation();
+    vtkPVDataInformation* datainfo = 
+      this->GetInputProxy()->GetDataInformation();
     if (datainfo->GetNumberOfCells() < 1000000)
       {
       this->SupportsZSweepMapper = 1;
@@ -634,17 +634,75 @@ vtkSMProxy* vtkSMUnstructuredGridVolumeRepresentationProxy::ConvertSelection(
   return selectionSource;
 }
 
+//----------------------------------------------------------------------------
+void vtkSMUnstructuredGridVolumeRepresentationProxy::SetSelectedMapperIndex(int index)
+{
+  this->SelectedMapperIndex = index;
+  switch (index)
+    {
+    case vtkSMUnstructuredGridVolumeRepresentationProxy::PROJECTED_TETRA_VOLUME_MAPPER:
+      this->SetVolumeMapperToPTCM();
+      break;
+    case vtkSMUnstructuredGridVolumeRepresentationProxy::HAVS_VOLUME_MAPPER:
+      this->SetVolumeMapperToHAVSCM();
+      break;
+    case vtkSMUnstructuredGridVolumeRepresentationProxy::ZSWEEP_VOLUME_MAPPER:
+      this->SetVolumeMapperToZSweepCM();
+      break;
+    case vtkSMUnstructuredGridVolumeRepresentationProxy::BUNYK_RAY_CAST_VOLUME_MAPPER:
+      this->SetVolumeMapperToBunykCM();
+      break;
+    default:
+      vtkDebugMacro("Unknown volume mapper index " << index);
+      break;
+    }
+}
+
+//----------------------------------------------------------------------------
+void vtkSMUnstructuredGridVolumeRepresentationProxy::SetSelectedMapperIndexIfSupported(int index)
+{
+
+  if (index == vtkSMUnstructuredGridVolumeRepresentationProxy::PROJECTED_TETRA_VOLUME_MAPPER)
+    {
+    this->SetVolumeMapperToPTCM();
+    }
+  else if (index == vtkSMUnstructuredGridVolumeRepresentationProxy::HAVS_VOLUME_MAPPER
+          && this->GetSupportsHAVSMapper())
+    {
+    this->SetVolumeMapperToHAVSCM();
+    }
+  else if (index == vtkSMUnstructuredGridVolumeRepresentationProxy::ZSWEEP_VOLUME_MAPPER
+           && this->GetSupportsZSweepMapper())
+    {
+    this->SetVolumeMapperToZSweepCM();
+    }
+  else if (index == vtkSMUnstructuredGridVolumeRepresentationProxy::BUNYK_RAY_CAST_VOLUME_MAPPER
+           && this->GetSupportsBunykMapper())
+    {
+    this->SetVolumeMapperToBunykCM();
+    }
+  else
+    {
+    vtkDebugMacro("Requested volume mapper index " << index << " is not supported.");
+    return;
+    }
+
+  // Only set the index if we were successful
+  this->SelectedMapperIndex = index;
+}
+
 //----------------------------------------------------------------------------
 void vtkSMUnstructuredGridVolumeRepresentationProxy::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os, indent);
-  os << indent << "VolumeFilterProxy: " << this->VolumeFilter << endl;
+  os << indent << "VolumeFilter: " << this->VolumeFilter << endl;
   os << indent << "VolumePropertyProxy: " << this->VolumeProperty << endl;
   os << indent << "VolumeActorProxy: " << this->VolumeActor << endl;
   os << indent << "SupportsHAVSMapper: " << this->SupportsHAVSMapper << endl;
   os << indent << "SupportsBunykMapper: " << this->SupportsBunykMapper << endl;
   os << indent << "SupportsZSweepMapper: " << this->SupportsZSweepMapper << endl;
   os << indent << "RenderViewExtensionsTested: " << this->RenderViewExtensionsTested << endl;
+  os << indent << "SelectedMapperIndex: " << this->SelectedMapperIndex << endl;
 }
 
 
