@@ -264,7 +264,7 @@ class SourceProxy(Proxy):
         vtkPVDataInformation"""
         if self.SMProxy:
             return DataInformation( \
-                self.SMProxy.GetDataInformation(idx, False), \
+                self.SMProxy.GetDataInformation(idx), \
                 self.SMProxy, idx)
                 
 class Property(object):
@@ -330,40 +330,45 @@ class VectorProperty(Property):
     val = property[2]
     property[1:3] = (1, 2)
     """
+    def ConvertValue(self, value):
+       """Converts value to type suitable for vtSMProperty::SetElement()"""
+       if self.SMProperty.IsA("vtkSMIntVectorProperty") and \
+         self.SMProperty.GetDomain("enum") and type(value) == str:
+           domain = self.SMProperty.GetDomain("enum")
+           if domain.HasEntryText(value):
+              return domain.GetEntryValueForText(value)
+       return value
+
     
     def __len__(self):
         """Returns the number of elements."""
         return self.SMProperty.GetNumberOfElements()
 
-    def __getitem__(self, idx):
-        """Given an index, return an element. Raises an IndexError
-        exception if the argument is out of bounds."""
-        if idx >= len(self):
-            raise exceptions.IndexError
-        return self.SMProperty.GetElement(idx)
-
     def __setitem__(self, idx, value):
-        """Given an index and a value, sets an element."""
-        self.SMProperty.SetElement(idx, value)
-        self._UpdateProperty()
-
-    def __getslice__(self, min, max):
-        """Returns the range [min, max) of elements. Raises an IndexError
-        exception if an argument is out of bounds."""
-        if min < 0 or min > len(self) or max < 0 or max > len(self):
-            raise exceptions.IndexError
-        retval = []
-        for i in range(min, max):
-            retval.append(self.SMProperty.GetElement(i))
-        return retval
-
-    def __setslice__(self, min, max, values):
-        "Given a list or tuple of values, sets a slice of values [min, max)."
-        if min < 0 or min > len(self) or max < 0 or max > len(self):
-            raise exceptions.IndexError
-        for i in range(min, max):
-            self.SMProperty.SetElement(i, values[i-min])
-        self._UpdateProperty()
+      """Given a list or tuple of values, sets a slice of values [min, max)"""
+      if isinstance(idx, slice):        
+        indices = idx.indices(len(self))         
+        for i, j in zip(range(*indices), value):  
+          self.SMProperty.SetElement(i, self.ConvertValue(j))                
+        self._UpdateProperty()    
+      elif idx >= len(self) or idx < 0:
+        raise exceptions.IndexError
+      else:        
+        self.SMProperty.SetElement(idx, self.ConvertValue(value))
+        self._UpdateProperty()                
+  
+    def __getitem__(self, idx):
+      """Returns the range [min, max) of elements. Raises an IndexError
+      exception if an argument is out of bounds."""
+      if isinstance(idx, slice):        
+        indices = idx.indices(len(self)) 
+        retVal = []
+        for i in range(*indices):
+          retVal.append(self.SMProperty.GetElement(i))
+        return retVal
+      elif idx >= len(self) or idx < 0:
+        raise exceptions.IndexError
+      return _getPyProxy(self.SMProperty.GetElement(idx))   
 
     def __getattr__(self, name):
         "Unknown attribute requests get forwarded to SMProperty."
@@ -375,7 +380,7 @@ class VectorProperty(Property):
         if property.GetRepeatable() or \
            property.GetRepeatCommand() or  \
            property.GetNumberOfElements() > 1:
-            return self.__getslice__(0, len(self))
+            return self[0:len(self)]
         elif property.GetNumberOfElements() == 1:
             return property.GetElement(0)
 
@@ -386,7 +391,7 @@ class VectorProperty(Property):
            not isinstance(values, list):
             values = (values,)
         for i in range(len(values)):
-            self.SMProperty.SetElement(i, values[i])
+            self.SMProperty.SetElement(i, self.ConvertValue(values[i]))
         self._UpdateProperty()
 
     def Clear(self):
@@ -407,51 +412,52 @@ class ProxyProperty(Property):
         """Returns the number of elements."""
         return self.SMProperty.GetNumberOfProxies()
 
-    def __getitem__(self, idx):
-        """Given an index, return an element. Raises an IndexError
-        exception if the argument is out of bounds."""
-        if idx >= len(self):
-            raise exceptions.IndexError
-        return _getPyProxy(self.SMProperty.GetProxy(idx))
-
     def __setitem__(self, idx, value):
-        """Given an index and a value, sets an element."""
+      """Given a list or tuple of values, sets a slice of values [min, max)"""
+      if isinstance(idx, slice):        
+        indices = idx.indices(len(self))         
+        for i, j in zip(range(*indices), value):  
+          self.SMProperty.SetProxy(i, j.SMProxy)      
+          
+        self._UpdateProperty()    
+      elif idx >= len(self) or idx < 0:
+        raise exceptions.IndexError
+      else:        
         self.SMProperty.SetProxy(idx, value.SMProxy)
-        self._UpdateProperty()
-
-    def __delitem__(self, idx):
-        """Removes the element idx"""
-        proxy = self[idx].SMProxy
-        self.SMProperty.RemoveProxy(proxy)
-        self._UpdateProperty()
-
-    def __getslice__(self, min, max):
-        """Returns the range [min, max) of elements. Raises an IndexError
-        exception if an argument is out of bounds."""
-        if min < 0 or min > len(self) or max < 0 or max > len(self):
-            raise exceptions.IndexError
-        retval = []
-        for i in range(min, max):
-            proxy = _getPyProxy(self.SMProperty.GetProxy(i))
-            retval.append(proxy)
-        return retval
-
-    def __setslice__(self, min, max, values):
-        "Given a list or tuple of values, sets a slice of values [min, max)."
-        if min < 0 or min > len(self) or max < 0 or max > len(self):
-            raise exceptions.IndexError
-        for i in range(min, max):
-            self.SMProperty.SetProxy(i, values[i-min].SMProxy)
-        self._UpdateProperty()
-
-    def __delslice__(self, min, max):
-        """Removes elements [min, max)"""
-        proxies = []
-        for i in range(min, max):
-            proxies.append(self[i].SMProxy)
-        for i in proxies:
-            self.SMProperty.RemoveProxy(i)
-        self._UpdateProperty()
+        self._UpdateProperty()            
+                    
+    def __delitem__(self,idx):
+      """Removes the element idx"""
+      if isinstance(idx, slice):        
+        indices = idx.indices(len(self))
+        # Collect the elements to delete to a new list first.
+        # Otherwise indices are screwed up during the actual
+        # remove loop.
+        toremove = []
+        for i in range(*indices):
+          toremove.append(self[i])
+        for i in toremove:
+          self.SMProperty.RemoveProxy(i.SMProxy)        
+        self._UpdateProperty()    
+      elif idx >= len(self) or idx < 0:
+        raise exceptions.IndexError
+      else:        
+        self.SMProperty.RemoveProxy(self[idx].SMProxy)
+        self._UpdateProperty()                
+  
+    def __getitem__(self, idx):
+      """Returns the range [min, max) of elements. Raises an IndexError
+      exception if an argument is out of bounds."""
+      if isinstance(idx, slice):        
+        indices = idx.indices(len(self)) 
+        retVal = []
+        for i in range(*indices):
+          retVal.append(_getPyProxy(self.SMProperty.GetProxy(i)))
+        return retVal
+      elif idx >= len(self) or idx < 0:
+        raise exceptions.IndexError
+      return _getPyProxy(self.SMProperty.GetProxy(idx))     
+                              
 
     def __getattr__(self, name):
         "Unknown attribute requests get forwarded to SMProperty."
@@ -466,7 +472,7 @@ class ProxyProperty(Property):
         "Returns all elements as either a list or a single value."
         property = self.SMProperty
         if property.GetRepeatable() or property.GetNumberOfProxies() > 1:
-            return self.__getslice__(0, len(self))
+            return self[0:len(self)]
         else:
             if property.GetNumberOfProxies() > 0:
                 return _getPyProxy(property.GetProxy(0))
@@ -504,14 +510,6 @@ class InputProperty(ProxyProperty):
     del property[1]
     """
 
-    def __getitem__(self, idx):
-        """Given an index, return an element as an OutputPort object.
-        Raises an IndexError exception if the argument is out of bounds."""
-        if idx >= len(self):
-            raise exceptions.IndexError
-        return OutputPort(_getPyProxy(self.SMProperty.GetProxy(idx)),\
-                          self.SMProperty.GetOutputPortForConnection(idx))
-
     def __getOutputPort(self, value):
         portidx = 0
         if isinstance(value, OutputPort):
@@ -524,35 +522,36 @@ class InputProperty(ProxyProperty):
         return OutputPort(value_proxy, portidx)
         
     def __setitem__(self, idx, value):
-        """Given an index and a value, sets an element. Accepts Proxy or
-        OutputPort objects."""
+      """Given a list or tuple of values, sets a slice of values [min, max)"""
+      if isinstance(idx, slice):        
+        indices = idx.indices(len(self))         
+        for i, j in zip(range(*indices), value):  
+          op = self.__getOutputPort(value[i-min])
+          self.SMProperty.SetInputConnection(i, op.Proxy, op.Port)                
+        self._UpdateProperty()    
+      elif idx >= len(self) or idx < 0:
+        raise exceptions.IndexError
+      else:        
         op = self.__getOutputPort(value)
         self.SMProperty.SetInputConnection(idx, op.Proxy, op.Port)
-        self._UpdateProperty()
-
-    def __getslice__(self, min, max):
-        """Returns the range [min, max) of elements as a list of OutputPort
-        objects. Raises an IndexError exception if an argument is out of
-        bounds."""
-        if min < 0 or min > len(self) or max < 0 or max > len(self):
-            raise exceptions.IndexError
-        retval = []
-        for i in range(min, max):
-            port = OutputPort(_getPyProxy(self.SMProperty.GetProxy(i)),\
+        self._UpdateProperty()    
+        
+    def __getitem__(self, idx):
+      """Returns the range [min, max) of elements. Raises an IndexError
+      exception if an argument is out of bounds."""
+      if isinstance(idx, slice):        
+        indices = idx.indices(len(self)) 
+        retVal = []
+        for i in range(*indices):
+          port = OutputPort(_getPyProxy(self.SMProperty.GetProxy(i)),\
                               self.SMProperty.GetOutputPortForConnection(i))
-            retval.append(port)
-        return retval
-
-    def __setslice__(self, min, max, values):
-        """Given a list or tuple of values, sets a slice of values [min, max).
-        Accepts Proxy or OutputPort objects."""
-        if min < 0 or min > len(self) or max < 0 or max > len(self):
-            raise exceptions.IndexError
-        for i in range(min, max):
-            op = self.__getOutputPort(value[i-min])
-            self.SMProperty.SetInputConnection(i, op.Proxy, op.Port)
-        self._UpdateProperty()
-
+          retval.append(port)
+        return retVal
+      elif idx >= len(self) or idx < 0:
+        raise exceptions.IndexError
+      return OutputPort(_getPyProxy(self.SMProperty.GetProxy(idx)),\
+                          self.SMProperty.GetOutputPortForConnection(idx))    
+                          
     def append(self, value):
         """Appends the given proxy to the property values.
         Accepts Proxy or OutputPort objects."""
@@ -565,7 +564,7 @@ class InputProperty(ProxyProperty):
         a single OutputPort object."""
         property = self.SMProperty
         if property.GetRepeatable() or property.GetNumberOfProxies() > 1:
-            return self.__getslice__(0, len(self))
+            return self[0:len(self)]
         else:
             if property.GetNumberOfProxies() > 0:
                 return OutputPort(_getPyProxy(property.GetProxy(0)),\
@@ -604,11 +603,14 @@ class DataInformation(object):
         self.Idx = idx
 
     def Update(self):
-        """Update the data information if necessary. Note that this
+        """****Deprecated**** There is no reason anymore to use this method
+        explicitly, it is called automatically when one gets any value from the
+        data information object.
+        Update the data information if necessary. Note that this
         does not cause execution of the underlying object. In certain
         cases, you may have to call UpdatePipeline() on the proxy."""
         if self.Proxy:
-            self.Proxy.GetDataInformation(self.Idx, False)
+            self.Proxy.GetDataInformation(self.Idx)
             
     def GetDataSetType(self):
         """Returns the dataset type as defined in vtkDataObjectTypes."""
@@ -1105,21 +1107,16 @@ def LoadState(filename, connection=None):
     if not connection:
         raise exceptions.RuntimeError, "Cannot load state without a connection"
     loader = vtkSMPQStateLoader()
-    rvname = vtkSMRenderViewProxy.GetSuggestedRenderViewType(connection.ID)
-    if rvname:
-        loader.SetRenderViewXMLName(rvname)
-        pm = ProxyManager()
-        pm.LoadState(filename, ActiveConnection.ID, loader)
-        views = GetRenderViews()
-        for view in views:
-            # Make sure that the client window size matches the
-            # ViewSize property. In paraview, the GUI takes care
-            # of this.
-            if view.GetClassName() == "vtkSMIceTDesktopRenderViewProxy":
-                view.GetRenderWindow().SetSize(view.ViewSize[0], \
-                                               view.ViewSize[1])
-    else:
-        raise exceptions.RuntimeError, "Could not load state because no appropriate render view was found."
+    pm = ProxyManager()
+    pm.LoadState(filename, ActiveConnection.ID, loader)
+    views = GetRenderViews()
+    for view in views:
+        # Make sure that the client window size matches the
+        # ViewSize property. In paraview, the GUI takes care
+        # of this.
+        if view.GetClassName() == "vtkSMIceTDesktopRenderViewProxy":
+            view.GetRenderWindow().SetSize(view.ViewSize[0], \
+                                           view.ViewSize[1])
     
 def Connect(ds_host=None, ds_port=11111, rs_host=None, rs_port=11111):
     """
@@ -1137,6 +1134,9 @@ def Connect(ds_host=None, ds_port=11111, rs_host=None, rs_port=11111):
       render server on rs_host: rs_port.
     """
     global ActiveConnection
+    global fromGUI
+    if fromGUI:
+        raise exceptions.RuntimeError, "Cannot create a connection through python. Use the GUI to setup the connection."
     if ds_host == None:
         connectionId = _connectSelf()
     elif rs_host == None:
@@ -1159,6 +1159,9 @@ def ReverseConnect(port=11111):
     The optional port specified the port to listen to.
     """
     global ActiveConnection
+    global fromGUI
+    if fromGUI:
+        raise exceptions.RuntimeError, "Cannot create a connection through python. Use the GUI to setup the connection."
     connectionId = _connectServer("Reverse connection", port, True)
     if not ActiveConnection:
         ActiveConnection = connectionId
@@ -1168,6 +1171,9 @@ def Disconnect(connection=None):
     """Disconnects the connection. Make sure to clear the proxy manager
     first."""
     global ActiveConnection
+    global fromGUI
+    if fromGUI:
+        raise exceptions.RuntimeError, "Cannot disconnect through python. Use the GUI to disconnect."
     if not connection or connection == ActiveConnection:
         connection = ActiveConnection
         ActiveConnection = None
@@ -1231,8 +1237,9 @@ def CreateRenderView(connection=None, **extraArgs):
     if not connection:
         raise exceptions.RuntimeError, "Cannot create render window without connection."
     pxm = ProxyManager()
-    proxy_xml_name = vtkSMRenderViewProxy.GetSuggestedRenderViewType(\
-        connection.ID)
+    prototype = pxm.GetPrototypeProxy("views", "RenderView")
+
+    proxy_xml_name = prototype.GetSuggestedViewType(connection.ID)
     ren_module = None
     if proxy_xml_name:
         ren_module = CreateProxy("views", proxy_xml_name, connection)
@@ -1250,18 +1257,27 @@ def CreateRepresentation(aProxy, view, **extraArgs):
     keyword arguments where the key is the name of the property.In addition
     registrationGroup and registrationName (optional) can be specified (as
     keyword arguments) to automatically register the proxy with the proxy
-    manager."""
+    manager.
+    
+    This method tries to create the best possible representation for the given
+    proxy in the given view. Additionally, the user can specify proxyName
+    (optional) to create a representation of a particular type."""
 
     global rendering
     if not aProxy:
         raise exceptions.RuntimeError, "proxy argument cannot be None."
     if not view:
         raise exceptions.RuntimeError, "render module argument cannot be None."
-    display = view.SMProxy.CreateDefaultRepresentation(aProxy.SMProxy, 0)
+    if "proxyName" in extraArgs:
+      display = CreateProxy("representations", extraArgs['proxyName'], None)
+      del extraArgs['proxyName']
+    else:
+      display = view.SMProxy.CreateDefaultRepresentation(aProxy.SMProxy, 0)
+      if display:
+        display.UnRegister(None)
     if not display:
         return None
     display.SetConnectionID(aProxy.GetConnectionID())
-    display.UnRegister(None)
     extraArgs['proxy'] = display
     proxy = rendering.__dict__[display.GetXMLName()](**extraArgs)
     proxy.Input = aProxy
@@ -1293,10 +1309,10 @@ def LoadPlugin(filename, connection=None):
             parser.Parse(xmlstring)
             parser.ProcessConfiguration(vtkSMObject.GetProxyManager())
             # Update the modules
-            _updateModules()
+            updateModules()
             
 
-def Fetch(input, arg1=None, arg2=None):
+def Fetch(input, arg1=None, arg2=None, idx=0):
     """ 
     A convenience method that moves data from the server to the client, 
     optionally performing some operation on the data as it moves.
@@ -1309,7 +1325,7 @@ def Fetch(input, arg1=None, arg2=None):
     In parallel runs an appropriate append Filter merges the
     data on each processor into one data object. The filter chosen will be 
     vtkAppendPolyData for vtkPolyData, vtkAppendRectilinearGrid for 
-    vtkRectilinearGrid, vtkMultiGroupDataGroupFilter for vtkCompositeData, 
+    vtkRectilinearGrid, vtkMultiBlockDataGroupFilter for vtkCompositeData, 
     and vtkAppendFilter for anything else.
     
     If arg1 is an integer then one particular processor's output is brought to
@@ -1322,31 +1338,35 @@ def Fetch(input, arg1=None, arg2=None):
     applied pre-gather and arg2 will be applied post-gather. In parallel
     runs the algorithm will be run on each processor to make intermediate
     results and then again on the root processor over all of the
-    intermediate results to create a global result.  """
+    intermediate results to create a global result.  
+    
+    Optional argument idx is used to specify the output port number to fetch the
+    data from. Default is port 0.
+    """
 
     import types
 
     #create the pipeline that reduces and transmits the data
     gvd = rendering.ClientDeliveryRepresentationBase()
-    gvd.AddInput(input, "DONTCARE") 
+    gvd.AddInput(0, input, idx, "DONTCARE")
   
     if arg1 == None:
         print "getting appended"
 
-        cdinfo = input.GetDataInformation().GetCompositeDataInformation()
-        if (cdinfo.GetDataIsComposite() or cdinfo.GetDataIsHierarchical()):
+        cdinfo = input.GetDataInformation(idx).GetCompositeDataInformation()
+        if cdinfo.GetDataIsComposite():
             print "use composite data append"
             gvd.SetReductionType(5)        
 
-        elif input.GetDataInformation().GetDataClassName() == "vtkPolyData":
+        elif input.GetDataInformation(idx).GetDataClassName() == "vtkPolyData":
             print "use append poly data filter"
             gvd.SetReductionType(1)        
 
-        elif input.GetDataInformation().GetDataClassName() == "vtkRectilinearGrid":
+        elif input.GetDataInformation(idx).GetDataClassName() == "vtkRectilinearGrid":
             print "use append rectilinear grid filter"
             gvd.SetReductionType(4)
 
-        elif input.GetDataInformation().IsA("vtkDataSet"):
+        elif input.GetDataInformation(idx).IsA("vtkDataSet"):
             print "use unstructured append filter"
             gvd.SetReductionType(2)
 
@@ -1455,8 +1475,14 @@ def _getPyProxy(smproxy):
 def _makeUpdateCameraMethod(rv):
     """ This internal method is used to create observer methods """
     rvref = rv
+    rvref().BlockUpdateCamera = False
     def UpdateCamera(obj, string):
-        rvref().SynchronizeCameraProperties()
+        if not rvref().BlockUpdateCamera:
+          # used to avoid some nasty recursion that occurs when interacting in
+          # the GUI.
+          rvref().BlockUpdateCamera = True
+          rvref().SynchronizeCameraProperties()
+          rvref().BlockUpdateCamera = False
     return UpdateCamera
 
 def _createInitialize(group, name):
@@ -1531,35 +1557,39 @@ def _printProgress(caller, event):
         currentAlgorithm = None
         currentProgress = 0
             
-def _updateModules():
+def updateModules():
     """Called when a plugin is loaded, this method updates
     the proxy class object in all known modules."""
-    global sources, filters, writers, rendering, animation
+    global sources, filters, writers, rendering, animation, implicit_functions
 
-    _createModule("sources", sources)
-    _createModule("filters", filters)
-    _createModule("writers", writers)
-    _createModule("representations", rendering)
-    _createModule("views", rendering)
-    _createModule("lookup_tables", rendering)
-    _createModule("animation", animation)
-    _createModule('animation_keyframes', animation)
+    createModule("sources", sources)
+    createModule("filters", filters)
+    createModule("writers", writers)
+    createModule("representations", rendering)
+    createModule("views", rendering)
+    createModule("lookup_tables", rendering)
+    createModule("textures", rendering)
+    createModule("animation", animation)
+    createModule('animation_keyframes', animation)
+    createModule('implicit_functions', implicit_functions)
     
 def _createModules():
     """Called when the module is loaded, this creates sub-
     modules for all know proxy groups."""
-    global sources, filters, writers, rendering, animation
+    global sources, filters, writers, rendering, animation, implicit_functions
 
-    sources = _createModule('sources')
-    filters = _createModule('filters')
-    writers = _createModule('writers')
-    rendering = _createModule('representations')
-    _createModule('views', rendering)
-    _createModule("lookup_tables", rendering)
-    animation = _createModule('animation')
-    _createModule('animation_keyframes', animation)
+    sources = createModule('sources')
+    filters = createModule('filters')
+    writers = createModule('writers')
+    rendering = createModule('representations')
+    createModule('views', rendering)
+    createModule("lookup_tables", rendering)
+    createModule("textures", rendering)
+    animation = createModule('animation')
+    createModule('animation_keyframes', animation)
+    implicit_functions = createModule('implicit_functions')
     
-def _createModule(groupName, mdl=None):
+def createModule(groupName, mdl=None):
     """Populates a module with proxy classes defined in the given group.
     If mdl is not specified, it also creates the module"""
     pxm = vtkSMObject.GetProxyManager()
@@ -1606,6 +1636,80 @@ def _createModule(groupName, mdl=None):
         mdl.__dict__[pname] = cobj
     return mdl
 
+
+def __determineGroup(proxy):
+    """Internal method"""
+    if not proxy:
+        return None
+    xmlgroup = proxy.GetXMLGroup()
+    xmlname = proxy.GetXMLName()
+    if xmlgroup == "sources":
+        return "sources"
+    elif xmlgroup == "filters":
+        return "sources"
+    elif xmlgroup == "views":
+        return "views"
+    elif xmlgroup == "representations":
+        if xmlname == "ScalarBarWidgetRepresentation":
+            return "scalar_bars"
+        return "representations";
+    elif xmlgroup == "lookup_tables":
+        return "lookup_tables"
+    return None
+
+__nameCounter = 0
+def __determineName(proxy, group):
+    global __nameCounter
+    __nameCounter += 1
+    return "%s%d" % (proxy.GetXMLName(), __nameCounter)
+
+def __getName(proxy, group):
+    pxm = ProxyManager()
+    if isinstance(proxy, Proxy):
+        proxy = proxy.SMProxy
+    return pxm.GetProxyName(group, proxy)
+
+def Register(proxy, **extraArgs):
+    """Registers a proxy with the proxy manager. If no 'registrationGroup' is
+    specified, then the group is inferred from the type of the proxy.
+    'registrationName' may be specified to register with a particular name
+    otherwise a default name will be created."""
+    # TODO: handle duplicate registration
+    if "registrationGroup" in extraArgs:
+        registrationGroup = extraArgs["registrationGroup"]
+    else:
+        registrationGroup = __determineGroup(proxy)
+        
+    if "registrationName" in extraArgs:
+        registrationName = extraArgs["registrationName"]
+    else:
+        registrationName = __determineName(proxy, registrationGroup)
+    if registrationGroup and registrationName:
+        pxm = ProxyManager()
+        pxm.RegisterProxy(registrationGroup, registrationName, proxy)
+    else:
+        raise exceptions.RuntimeError, "Registeration error."
+    return (registrationGroup, registrationName);
+
+def UnRegister(proxy, **extraArgs):
+    """UnRegisters proxies registered using Register()."""
+    if "registrationGroup" in extraArgs:
+        registrationGroup = extraArgs["registrationGroup"]
+    else:
+        registrationGroup = __determineGroup(proxy)
+        
+    if "registrationName" in extraArgs:
+        registrationName = extraArgs["registrationName"]
+    else:
+        registrationName = __getName(proxy, registrationGroup)
+
+    if registrationGroup and registrationName:
+        pxm = ProxyManager()
+        pxm.UnRegisterProxy(registrationGroup, registrationName, proxy)
+    else:
+        raise exceptions.RuntimeError, "UnRegisteration error."
+    return (registrationGroup, registrationName);
+
 def demo1():
     """This simple demonstration creates a sphere, renders it and delivers
     it to the client using Fetch. It returns a tuple of (data, render
@@ -1707,7 +1811,7 @@ def demo3():
     probes it with a line, delivers the result to the client using Fetch
     and plots it using pylab. This demo requires numpy and pylab installed.
     It returns a tuple of (data, render view)."""
-    import paraview.numeric
+    import paraview.numpy_support
     import pylab
     
     if not ActiveConnection:
@@ -1763,7 +1867,7 @@ def demo3():
     # Now deliver it to the client. Remember, this is for small data.
     data = Fetch(probe)
     # Convert it to a numpy array
-    data = paraview.numeric.getarray(data.GetPointData().GetArray(0))
+    data = paraview.numpy_support.vtk_to_numpy(data.GetPointData().GetArray(0))
     # Plot it using matplotlib
     pylab.plot(data)
     pylab.show()
