@@ -2,9 +2,9 @@
 /*                               XDMF                              */
 /*                   eXtensible Data Model and Format              */
 /*                                                                 */
-/*  Id : $Id: XdmfDOM.cxx,v 1.22 2007/07/13 14:31:56 dave.demarle Exp $  */
-/*  Date : $Date: 2007/07/13 14:31:56 $ */
-/*  Version : $Revision: 1.22 $ */
+/*  Id : $Id: XdmfDOM.cxx,v 1.24 2008-05-16 15:09:50 clarke Exp $  */
+/*  Date : $Date: 2008-05-16 15:09:50 $ */
+/*  Version : $Revision: 1.24 $ */
 /*                                                                 */
 /*  Author:                                                        */
 /*     Jerry A. Clarke                                             */
@@ -73,11 +73,13 @@ XdmfDOM::XdmfDOM(){
 }
 
 XdmfDOM::~XdmfDOM(){
+   XdmfDebug("Destroying DOM");
   if( ( this->Output != &cout ) && ( this->Output != &cerr ) ) {
     ofstream *OldOutput = ( ofstream *)this->Output;
     OldOutput->close();
   }
   if( this->Input != &cin ) {
+     XdmfDebug("Deleting Input");
     ifstream *OldInput = ( ifstream *)this->Input;
     OldInput->close();
     delete this->Input;
@@ -655,6 +657,14 @@ if(!Node) {
     Node = this->Tree;
 }
 if(!Node) return(0);
+/*
+cout << "For " << Attribute << " DOM Returning ";
+if((XdmfConstString)xmlGetProp(Node, (xmlChar *)Attribute)) {
+    cout << (XdmfConstString)xmlGetProp(Node, (xmlChar *)Attribute) << endl;
+    } else{
+        cout << " NULL " << endl;
+    }
+    */
 return((XdmfConstString)xmlGetProp(Node, (xmlChar *)Attribute));
 }
 
