@@ -2,9 +2,9 @@
 /*                               XDMF                              */
 /*                   eXtensible Data Model and Format              */
 /*                                                                 */
-/*  Id : $Id: XdmfDataDesc.cxx,v 1.1 2006/12/15 21:55:21 clarke Exp $  */
-/*  Date : $Date: 2006/12/15 21:55:21 $ */
-/*  Version : $Revision: 1.1 $ */
+/*  Id : $Id: XdmfDataDesc.cxx,v 1.6 2008-07-04 13:25:54 biddisco Exp $  */
+/*  Date : $Date: 2008-07-04 13:25:54 $ */
+/*  Version : $Revision: 1.6 $ */
 /*                                                                 */
 /*  Author:                                                        */
 /*     Jerry A. Clarke                                             */
@@ -24,6 +24,8 @@
 /*******************************************************************/
 #include "XdmfDataDesc.h"
 
+#include <stdlib.h>
+
 XdmfDataDesc::XdmfDataDesc() {
   H5dont_atexit();
   this->DataSpace = H5I_BADID;
@@ -32,6 +34,9 @@ XdmfDataDesc::XdmfDataDesc() {
   this->NextOffset = 0;
   this->Rank = 1;
   this->ShapeString = 0;
+#ifndef XDMF_NO_MPI
+  this->DsmBuffer = NULL;
+#endif
 }
 
 XdmfDataDesc::~XdmfDataDesc() {
@@ -344,10 +349,17 @@ for( i = 0 ; i < Length ; i++ ){
   HCoordinates[i] = Coordinates[i];
   }
 #if (H5_VERS_MAJOR>1)||((H5_VERS_MAJOR==1)&&((H5_VERS_MINOR>6)||((H5_VERS_MINOR==6)&&(H5_VERS_RELEASE>=4))))
+# if (H5_VERS_MAJOR>1)||((H5_VERS_MAJOR==1)&&((H5_VERS_MINOR>6)||((H5_VERS_MINOR==6)&&(H5_VERS_RELEASE>=7))))
+status = H5Sselect_elements( this->DataSpace,
+        H5S_SELECT_SET,
+         NElements,
+         ( const hsize_t *)HCoordinates);
+# else
 status = H5Sselect_elements( this->DataSpace,
         H5S_SELECT_SET,
          NElements,
          ( const hsize_t **)HCoordinates);
+# endif
 #else
 status = H5Sselect_elements( this->DataSpace,
         H5S_SELECT_SET,
@@ -364,9 +376,10 @@ XdmfInt64
 XdmfDataDesc::GetNumberOfElements( void ) {
 hsize_t    i, Dimensions[ XDMF_MAX_DIMENSION ];
 XdmfInt64  Nelements = 0;
-XdmfInt32  rank = H5Sget_simple_extent_ndims(this->DataSpace );
+XdmfInt32  rank;
 
-this->Rank = rank;
+if(this->DataSpace == H5I_BADID) return(0);
+this->Rank = rank = H5Sget_simple_extent_ndims(this->DataSpace );
 H5Sget_simple_extent_dims( this->DataSpace, Dimensions, NULL );
 if(rank) {
   Nelements = this->Dimension[0] = Dimensions[0];
