@@ -2,9 +2,9 @@
 /*                               XDMF                              */
 /*                   eXtensible Data Model and Format              */
 /*                                                                 */
-/*  Id : $Id: XdmfDataItem.cxx,v 1.12 2007/07/12 20:01:31 dave.demarle Exp $  */
-/*  Date : $Date: 2007/07/12 20:01:31 $ */
-/*  Version : $Revision: 1.12 $ */
+/*  Id : $Id: XdmfDataItem.cxx,v 1.21 2008-08-29 15:08:58 clarke Exp $  */
+/*  Date : $Date: 2008-08-29 15:08:58 $ */
+/*  Version : $Revision: 1.21 $ */
 /*                                                                 */
 /*  Author:                                                        */
 /*     Jerry A. Clarke                                             */
@@ -27,10 +27,14 @@
 #include "XdmfExpression.h"
 #include "XdmfArray.h"
 #include "XdmfDOM.h"
+#include "XdmfHDF.h"
 
 // Supported Xdmf Formats
 #include "XdmfValuesXML.h"
 #include "XdmfValuesHDF.h"
+#ifdef XDMF_USE_MYSQL
+#include "XdmfValuesMySQL.h"
+#endif
 
 #include <libxml/tree.h>
 
@@ -51,13 +55,22 @@ XdmfDataItem::XdmfDataItem() {
 
 XdmfDataItem::~XdmfDataItem() {
     XdmfDebug(".... Deleteing DataItem " << this);
-    XdmfDebug("Deleteing array");
-    if(this->Array && this->ArrayIsMine ) delete this->Array;
-    XdmfDebug("Done Deleteing array");
+    if(this->Array && this->ArrayIsMine ){
+        XdmfDebug("Deleteing array " << this->Array);
+        delete this->Array;
+        XdmfDebug("Done Deleteing array");
+    }
     if(this->DataDesc && this->DataDescIsMine) delete this->DataDesc;
     if(this->Values) delete this->Values;
 }
 
+XdmfInt32
+XdmfDataItem::Release(){
+    // Don't Delete it.
+    if(this->Array) this->Array->SetNumberOfElements(3);
+    return(XDMF_SUCCESS);
+}
+
 XdmfInt32
 XdmfDataItem::Insert( XdmfElement *Child){
     if(Child && (
@@ -106,6 +119,9 @@ XdmfDataItem::Copy(XdmfElement *Source){
     // this->SetDOM(ds->GetDOM());
     this->SetFormat(ds->GetFormat());
     this->SetHeavyDataSetName(ds->GetHeavyDataSetName());
+#ifndef XDMF_NO_MPI
+    this->SetDsmBuffer(ds->GetDsmBuffer());
+#endif
     this->DataDesc->CopyType(ds->GetDataDesc());
     this->DataDesc->CopyShape(ds->GetDataDesc());
     this->DataDesc->CopySelection(ds->GetDataDesc());
@@ -153,14 +169,6 @@ XdmfInt32 XdmfDataItem::UpdateInformationUniform(){
     XdmfConstString Value;
     XdmfInt32   Precision = 4;
 
-    Value = this->Get("Dimensions");
-    if(!Value) {
-        XdmfErrorMessage("Dimensions are not set in XML Element");
-        cout  << this->DOM->Serialize(this->Element) << endl;
-        return(XDMF_FAIL);
-    }
-    if(!this->DataDesc) this->DataDesc = new XdmfDataDesc();
-    this->DataDesc->SetShapeFromString(Value);
     Value = this->Get("Precision");
     if(Value) Precision = atoi(Value);
     Value = this->Get("NumberType");
@@ -196,6 +204,8 @@ XdmfInt32 XdmfDataItem::UpdateInformationUniform(){
         this->SetFormat(XDMF_FORMAT_HDF);
     } else if(XDMF_WORD_CMP(Value, "XML")){
         this->SetFormat(XDMF_FORMAT_XML);
+    } else if(XDMF_WORD_CMP(Value, "MYSQL")){
+        this->SetFormat(XDMF_FORMAT_MYSQL);
     }else if(Value){
         XdmfErrorMessage("Unsupported DataItem Format :" << Value);
         return(XDMF_FAIL);
@@ -259,6 +269,14 @@ XdmfInt32 XdmfDataItem::UpdateInformation(){
         XdmfDebug("Reference DataItem Copied Info from another ReferenceObject");
         return(XDMF_SUCCESS);
     }
+    Value = this->Get("Dimensions");
+    if(!Value) {
+        XdmfErrorMessage("Dimensions are not set in XML Element");
+        XdmfErrorMessage(this->DOM->Serialize(this->Element));
+        return(XDMF_FAIL);
+    }
+    if(!this->DataDesc) this->DataDesc = new XdmfDataDesc();
+    this->DataDesc->SetShapeFromString(Value);
     switch(this->ItemType){
         case XDMF_ITEM_UNIFORM :
             return(this->UpdateInformationUniform());
@@ -414,6 +432,20 @@ XdmfInt32 XdmfDataItem::UpdateFunction(){
     if(Value && ReturnArray){
         ReturnArray->ReformFromString(Value);
     }
+    // If only a portion of the DataItem was requested
+    // the XdmfValues did not reflect this selection since
+    // DataDesc was used to select HyperSlad | Coordinates | Function
+    if(this->DataDesc->GetSelectionType() != XDMF_SELECTALL){
+            XdmfArray   *Portion;
+            XdmfInt64  SelectionSize = this->DataDesc->GetSelectionSize();
+
+            Portion = ReturnArray->Clone();
+            ReturnArray->SetShape(1, &SelectionSize);
+            ReturnArray->SelectAll();
+            Portion->CopySelection(this->DataDesc);
+            CopyArray(Portion, ReturnArray);
+            delete Portion;
+    }
     while( NTmp ){
         NTmp--;
         XdmfDebug("Deleteing DataItem #" << NTmp );
@@ -444,21 +476,29 @@ XdmfInt32 XdmfDataItem::Update(){
         return(this->UpdateFunction());
     }
     if(this->Array->CopyType(this->DataDesc) != XDMF_SUCCESS) return(XDMF_FAIL);
-    if(this->Array->CopyShape(this->DataDesc) != XDMF_SUCCESS) return(XDMF_FAIL);
+    // if(this->Array->CopyShape(this->DataDesc) != XDMF_SUCCESS) return(XDMF_FAIL);
     // if(this->Array->CopySelection(this->DataDesc) != XDMF_SUCCESS) return(XDMF_FAIL);
     if(this->CheckValues(this->Format) != XDMF_SUCCESS){
         XdmfErrorMessage("Error Accessing Internal XdmfValues");
         return(XDMF_FAIL);
     }
     if(this->Values->GetDataDesc()->CopySelection(this->DataDesc) != XDMF_SUCCESS) return(XDMF_FAIL);
+    XdmfDebug("Resize Array");
+    if(this->Array->SetShapeFromSelection(this->DataDesc) != XDMF_SUCCESS) return(XDMF_FAIL);
     switch (this->Format) {
         case XDMF_FORMAT_HDF :
             this->Values->SetDebug(this->GetDebug());
+            // this->SetDsmBuffer(this->Values->GetDsmBuffer());
+            // cout << "Setting Values Dsm to " << this->DsmBuffer << endl;
+#ifndef XDMF_NO_MPI
+            this->Values->SetDsmBuffer(this->DsmBuffer);
+#endif
+            XdmfDebug("Reading Data");
             if(!((XdmfValuesHDF *)this->Values)->Read(this->Array)){
                 XdmfErrorMessage("Reading Values Failed");
                 return(XDMF_FAIL);
             }
-            this->SetHeavyDataSetName(Values->GetHeavyDataSetName());
+            this->SetHeavyDataSetName(this->Values->GetHeavyDataSetName());
             break;
         case XDMF_FORMAT_XML :
             this->Values->SetDebug(this->GetDebug());
@@ -467,6 +507,18 @@ XdmfInt32 XdmfDataItem::Update(){
                 return(XDMF_FAIL);
             }
             break;
+        case XDMF_FORMAT_MYSQL :
+            this->Values->SetDebug(this->GetDebug());
+#ifdef XDMF_USE_MYSQL
+            if(!((XdmfValuesMySQL *)this->Values)->Read(this->Array)){
+                XdmfErrorMessage("Reading Values Failed");
+                return(XDMF_FAIL);
+            }
+#else
+            XdmfErrorMessage("XdmfValuesMySQL not enabled in this Xdmf");
+            return(XDMF_FAIL);
+#endif
+            break;
         default :
             XdmfErrorMessage("Unsupported Data Format");
             return(XDMF_FAIL);
@@ -555,6 +607,9 @@ XdmfInt32 XdmfDataItem::Build(){
         case XDMF_FORMAT_HDF :
             XdmfDebug("Writing Values in HDF Format");
             Values->SetHeavyDataSetName(this->GetHeavyDataSetName());
+#ifndef XDMF_NO_MPI
+            Values->SetDsmBuffer(this->GetDsmBuffer());
+#endif
             if(((XdmfValuesHDF *)Values)->Write(this->Array) != XDMF_SUCCESS){
                 XdmfErrorMessage("Writing Values Failed");
                 return(XDMF_FAIL);
@@ -596,6 +651,14 @@ XdmfDataItem::CheckValues(XdmfInt32 aFormat){
             case XDMF_FORMAT_XML :
                 this->Values = (XdmfValues *)new XdmfValuesXML();
                 break;
+            case XDMF_FORMAT_MYSQL :
+#ifdef XDMF_USE_MYSQL
+                this->Values = (XdmfValues *)new XdmfValuesMySQL();
+#else
+                XdmfErrorMessage("MySQL not supported in this Xdmf");
+                return(XDMF_FAIL);
+#endif
+                break;
             default :
                 XdmfErrorMessage("Unsupported Data Format");
                 return(XDMF_FAIL);
