@@ -2,9 +2,9 @@
 /*                               XDMF                              */
 /*                   eXtensible Data Model and Format              */
 /*                                                                 */
-/*  Id : $Id: XdmfGrid.cxx,v 1.14 2007/07/12 20:01:31 dave.demarle Exp $  */
-/*  Date : $Date: 2007/07/12 20:01:31 $ */
-/*  Version : $Revision: 1.14 $ */
+/*  Id : $Id: XdmfGrid.cxx,v 1.25 2008-09-04 15:46:47 dave.demarle Exp $  */
+/*  Date : $Date: 2008-09-04 15:46:47 $ */
+/*  Version : $Revision: 1.25 $ */
 /*                                                                 */
 /*  Author:                                                        */
 /*     Jerry A. Clarke                                             */
@@ -30,6 +30,7 @@
 #include "XdmfTopology.h"
 #include "XdmfGeometry.h"
 #include "XdmfAttribute.h"
+#include "XdmfTime.h"
 
 XdmfGrid *HandleToXdmfGrid( XdmfString Source ){
   XdmfObject  *TempObj;
@@ -50,11 +51,14 @@ XdmfGrid::XdmfGrid() {
   this->GeometryIsMine = 1;
   this->Topology = new XdmfTopology;
   this->TopologyIsMine = 1;
+  this->Time = new XdmfTime;
+  this->TimeIsMine = 1;
   this->Attribute = (XdmfAttribute **)calloc(1, sizeof( XdmfAttribute * ));
   this->Children = (XdmfGrid **)calloc(1, sizeof( XdmfGrid * ));
   this->AssignedAttribute = NULL;
   this->NumberOfAttributes = 0;
   this->GridType = XDMF_GRID_UNSET;
+  this->CollectionType = XDMF_GRID_COLLECTION_UNSET;
   this->NumberOfChildren = 0;
   this->Debug = 0;
   }
@@ -63,6 +67,7 @@ XdmfGrid::~XdmfGrid() {
   XdmfInt32  Index;
   if( this->GeometryIsMine && this->Geometry ) delete this->Geometry;
   if( this->TopologyIsMine && this->Topology ) delete this->Topology;
+  if( this->TimeIsMine && this->Time ) delete this->Time;
   for ( Index = 0; Index < this->NumberOfAttributes; Index ++ )
     {
     delete this->Attribute[Index];
@@ -70,6 +75,17 @@ XdmfGrid::~XdmfGrid() {
   free(this->Attribute);
   }
 
+XdmfInt32
+XdmfGrid::Release(){
+  XdmfInt32  Index;
+  if( this->GeometryIsMine && this->Geometry ) this->Geometry->Release();
+  if( this->TopologyIsMine && this->Topology ) this->Topology->Release();
+  for ( Index = 0; Index < this->NumberOfAttributes; Index ++ )
+    {
+    this->Attribute[Index]->Release();
+    }
+   return(XDMF_SUCCESS);
+}
 XdmfInt32
 XdmfGrid::InsertTopology(){
     if(!this->Topology->GetElement()){
@@ -101,6 +117,7 @@ XdmfGrid::Insert( XdmfElement *Child){
         XDMF_WORD_CMP(Child->GetElementName(), "Topology") ||
         XDMF_WORD_CMP(Child->GetElementName(), "Attribute") ||
         XDMF_WORD_CMP(Child->GetElementName(), "DataItem") ||
+        XDMF_WORD_CMP(Child->GetElementName(), "Time") ||
         XDMF_WORD_CMP(Child->GetElementName(), "Information")
         )){
         XdmfInt32   status = XdmfElement::Insert(Child);
@@ -133,6 +150,22 @@ XdmfGrid::Build(){
         this->Geometry->Build();
     }else{
     }
+    if(this->BuildTime && this->Time){
+            if(this->Time->Build() != XDMF_SUCCESS) return(XDMF_FAIL);
+    }
+    return(XDMF_SUCCESS);
+}
+
+XdmfInt32
+XdmfGrid::SetCollectionTypeFromString(XdmfConstString aCollectionType){
+    if(XDMF_WORD_CMP(aCollectionType, "Temporal")){
+        this->SetCollectionType(XDMF_GRID_COLLECTION_TEMPORAL);
+    }else if(XDMF_WORD_CMP(aCollectionType, "Spatial")){
+        this->SetCollectionType(XDMF_GRID_COLLECTION_SPATIAL);
+    }else{
+        XdmfErrorMessage("Unknown Collection Type : " << aCollectionType);
+        return(XDMF_FAIL);
+    }
     return(XDMF_SUCCESS);
 }
 
@@ -153,6 +186,21 @@ XdmfGrid::SetGridTypeFromString(XdmfConstString aGridType){
     return(XDMF_SUCCESS);
 }
 
+XdmfConstString
+XdmfGrid::GetCollectionTypeAsString(){
+    if((this->GridType & XDMF_GRID_MASK) == XDMF_GRID_COLLECTION){
+        switch(this->CollectionType){
+            case XDMF_GRID_COLLECTION_TEMPORAL :
+                return("Temporal");
+            case XDMF_GRID_COLLECTION_SPATIAL :
+                return("Spatial");
+            default :
+                return("Unset");
+        }
+    }
+    return(0);
+}
+
 XdmfConstString
 XdmfGrid::GetGridTypeAsString(){
     if(this->GridType & XDMF_GRID_MASK){
@@ -195,6 +243,9 @@ XdmfGrid::AssignAttribute( XdmfAttribute *attribute ){
 XdmfInt32 Status = 0;
 
 if( attribute ){
+#ifndef XDMF_NO_MPI
+  if(!attribute->GetDsmBuffer()) attribute->SetDsmBuffer(this->DsmBuffer);
+#endif
   attribute->Update();
   // Status = attribute->SetBaseAttribute( this, this->BaseGrid );
   this->AssignedAttribute = attribute;
@@ -266,6 +317,14 @@ attribute = this->Get("GridType");
 if(!attribute) attribute = this->Get("Type");
 if( XDMF_WORD_CMP(attribute, "Collection") ){
     this->GridType = XDMF_GRID_COLLECTION;
+    attribute = this->Get("CollectionType");
+    if(attribute){
+        if( XDMF_WORD_CMP(attribute, "Temporal") ){
+            this->SetCollectionType(XDMF_GRID_COLLECTION_TEMPORAL);
+        }else if( XDMF_WORD_CMP(attribute, "Spatial") ){
+            this->SetCollectionType(XDMF_GRID_COLLECTION_SPATIAL);
+        }
+    }
 }else if( XDMF_WORD_CMP(attribute, "Subset") ){
     this->GridType = XDMF_GRID_SUBSET;
 }else if( XDMF_WORD_CMP(attribute, "Tree") ){
@@ -294,6 +353,17 @@ if( this->GridType & XDMF_GRID_MASK){
     nchild = this->DOM->FindNumberOfElements("Grid", this->Element);
     this->NumberOfChildren = nchild;
     this->Children = (XdmfGrid **)realloc(this->Children, nchild * sizeof(XdmfGrid *));
+    anElement = this->DOM->FindElement("Time", 0, this->Element);
+    if(anElement){
+        if(this->Time->SetDOM( this->DOM ) == XDMF_FAIL) return(XDMF_FAIL);
+        if(this->Time->SetElement(anElement) == XDMF_FAIL) return(XDMF_FAIL);
+        Status = this->Time->UpdateInformation();
+        if( Status == XDMF_FAIL ){
+            XdmfErrorMessage("Error Reading Time");
+            return( XDMF_FAIL );
+        }
+    }
+    anElement = this->DOM->FindElement("Geometry", 0, this->Element);
     for(i=0 ; i < nchild ; i++){
         node = this->DOM->FindElement("Grid", i, this->Element);
         if(!node) {
@@ -303,7 +373,11 @@ if( this->GridType & XDMF_GRID_MASK){
         this->Children[i] = new XdmfGrid;
         if(this->Children[i]->SetDOM(this->DOM) == XDMF_FAIL) return(XDMF_FAIL);
         if(this->Children[i]->SetElement(node) == XDMF_FAIL) return(XDMF_FAIL);
+        // cout << "Calling update info for child " << i << endl;
         if(this->Children[i]->UpdateInformation() == XDMF_FAIL) return(XDMF_FAIL);
+        if(this->Time->GetTimeType() != XDMF_TIME_UNSET){
+                if(this->Children[i]->GetTime()->SetTimeFromParent(this->Time, i) != XDMF_SUCCESS) return(XDMF_FAIL);
+        }
     }
     if((this->GridType & XDMF_GRID_MASK) == XDMF_GRID_SUBSET){
         // Selection is the First Element Under Grid
@@ -358,7 +432,7 @@ if( this->GridType & XDMF_GRID_MASK){
             di->SetElement(select);
             di->UpdateInformation();
             di->Update();
-//            cout << "UpdateInfo - Select Cells : " << di->GetArray()->GetValues() << endl;
+//          cout << "UpdateInfo - Select Cells : " << di->GetArray()->GetValues() << endl;
             shape = this->Topology->GetShapeDesc();
             shape->CopyShape(di->GetDataDesc());
             delete di;
@@ -369,6 +443,17 @@ if( this->GridType & XDMF_GRID_MASK){
 //    return(XDMF_SUCCESS);
 }else{
 // Handle Uniform Grid
+    // cout << "Update Uniform Grid" << endl;
+    anElement = this->DOM->FindElement("Time", 0, this->Element);
+    if(anElement){
+        if(this->Time->SetDOM( this->DOM ) == XDMF_FAIL) return(XDMF_FAIL);
+        if(this->Time->SetElement(anElement) == XDMF_FAIL) return(XDMF_FAIL);
+        Status = this->Time->UpdateInformation();
+        if( Status == XDMF_FAIL ){
+            XdmfErrorMessage("Error Reading Time");
+            return( XDMF_FAIL );
+        }
+    }
     anElement = this->DOM->FindElement("Topology", 0, this->Element);
     if(anElement){
         if(this->Topology->SetDOM( this->DOM ) == XDMF_FAIL) return(XDMF_FAIL);
@@ -435,6 +520,9 @@ if((this->GridType & XDMF_GRID_MASK) != XDMF_GRID_UNIFORM){
     XdmfInt32   i;
     // SubSet, Tree or Collection
     for(i=0; i < this->NumberOfChildren ; i++){
+#ifndef XDMF_NO_MPI
+        if(!this->Children[i]->GetDsmBuffer()) this->Children[i]->SetDsmBuffer(this->DsmBuffer);
+#endif
         if(this->Children[i]->Update() == XDMF_FAIL){
             XdmfErrorMessage("Error in Update() of Child Grid " << i);
             return(XDMF_FAIL);
@@ -505,10 +593,16 @@ if((this->GridType & XDMF_GRID_MASK) != XDMF_GRID_UNIFORM){
     }
     return(XDMF_SUCCESS);
 }
+#ifndef XDMF_NO_MPI
+if(!this->Topology->GetDsmBuffer()) this->Topology->SetDsmBuffer(this->DsmBuffer);
+#endif
 if(this->Topology->Update() == XDMF_FAIL){
     XdmfErrorMessage("Error in Update() of Topology");
     return(XDMF_FAIL);
 }
+#ifndef XDMF_NO_MPI
+if(!this->Geometry->GetDsmBuffer()) this->Geometry->SetDsmBuffer(this->DsmBuffer);
+#endif
 if(this->Geometry->Update() == XDMF_FAIL){
     XdmfErrorMessage("Error in Update() of Geometry");
     return(XDMF_FAIL);
@@ -535,3 +629,56 @@ XdmfGrid::IsUniform(){
     if(this->GridType & XDMF_GRID_MASK) return(XDMF_FALSE);
     return(XDMF_TRUE);
 }
+
+XdmfInt32
+XdmfGrid::FindGridsAtTime(XdmfTime *theTime, XdmfArray *ArrayToFill, XdmfFloat64 Epsilon, XdmfInt32 Append){
+    XdmfInt64   i, index = 0, nchild;
+
+    nchild = this->GetNumberOfChildren();
+    if(!nchild) return(XDMF_FALSE);
+    if(Append){
+        index = ArrayToFill->GetNumberOfElements();
+        ArrayToFill->SetNumberOfElements(index + nchild);
+    }else{
+        index = 0;
+        ArrayToFill->SetNumberOfElements(nchild);
+    }
+    for(i=0 ; i < this->GetNumberOfChildren() ; i++){
+        // cout << "IsValid(" << i << ") = " << this->GetChild(i)->GetTime()->IsValid(Time) << endl;
+        if(this->GetChild(i)->GetTime()->IsValid(theTime)){
+            ArrayToFill->SetValueFromInt64(index, i);
+            index++;
+        }
+    }
+    if(index){
+        // Squeeze
+        ArrayToFill->SetNumberOfElements(index);
+    }else{
+        return(XDMF_FAIL);
+    }
+return(XDMF_SUCCESS);
+}
+
+XdmfInt32
+XdmfGrid::FindGridsInTimeRange(XdmfFloat64 TimeMin, XdmfFloat64 TimeMax, XdmfArray *ArrayToFill){
+    XdmfInt64   i, index = 0, nchild;
+
+    nchild = this->GetNumberOfChildren();
+    if(!nchild) return(XDMF_FALSE);
+    ArrayToFill->SetNumberType(XDMF_INT64_TYPE);
+    ArrayToFill->SetNumberOfElements(nchild);
+    for(i=0 ; i < nchild ; i++){
+        // cout << "IsValid(" << i << ") = " << this->GetChild(i)->GetTime()->IsValid(TimeMin, TimeMax) << endl;
+        if(this->GetChild(i)->GetTime()->IsValid(TimeMin, TimeMax)){
+            ArrayToFill->SetValueFromInt64(index, i);
+            index++;
+        }
+    }
+    if(index){
+        // Squeeze
+        ArrayToFill->SetNumberOfElements(index);
+    }else{
+        return(XDMF_FAIL);
+    }
+return(XDMF_SUCCESS);
+}
