@@ -2,9 +2,9 @@
 /*                               XDMF                              */
 /*                   eXtensible Data Model and Format              */
 /*                                                                 */
-/*  Id : $Id: XdmfHDF.cxx,v 1.4 2007/07/10 17:55:41 dave.demarle Exp $  */
-/*  Date : $Date: 2007/07/10 17:55:41 $ */
-/*  Version : $Revision: 1.4 $ */
+/*  Id : $Id: XdmfHDF.cxx,v 1.8 2008-05-19 21:02:36 clarke Exp $  */
+/*  Date : $Date: 2008-05-19 21:02:36 $ */
+/*  Version : $Revision: 1.8 $ */
 /*                                                                 */
 /*  Author:                                                        */
 /*     Jerry A. Clarke                                             */
@@ -27,12 +27,6 @@
 #include "XdmfH5Driver.h"
 #endif
 
-#ifdef HAVE_NDGM
-extern "C" {
-#include "H5FDndgm.h"
-}
-#endif
-
 #ifdef WIN32
 #define XDMF_HDF5_SIZE_T        ssize_t
 #else
@@ -55,6 +49,21 @@ XdmfHDF::XdmfHDF() {
   this->NumberOfChildren = 0;
   this->Compression = 0;
   this->UseSerialFile = 0;
+  // We may have been compiled with Parallel IO support, but be run only on a single
+  // machine without mpiexec. Disable parallel if just one process.
+#if H5_HAVE_PARALLEL && ((H5_VERS_MAJOR>1)||((H5_VERS_MAJOR==1)&&(H5_VERS_MINOR>=6)))
+  int valid, nprocs=0;
+  MPI_Initialized(&valid);
+  if (valid) 
+    {
+    MPI_Comm_size(MPI_COMM_WORLD,&nprocs);
+    }
+  if (nprocs<=1) 
+    {
+    this->UseSerialFile = 1;
+    }
+#endif
+
 #ifndef XDMF_NO_MPI
   this->DsmBuffer = 0;
 #endif
@@ -600,7 +609,6 @@ if( DataSetName != NULL ) {
   *firstcolon = '\0';
   firstcolon++;
   if ( ( STRCASECMP( NewName, "FILE" ) == 0 ) ||
-    ( STRCASECMP( NewName, "NDGM" ) == 0 ) ||
     ( STRCASECMP( NewName, "GASS" ) == 0 ) ||
     ( STRCASECMP( NewName, "CORE" ) == 0 ) ||
     ( STRCASECMP( NewName, "DUMMY" ) == 0 ) ) {
@@ -664,8 +672,8 @@ XdmfDebug("Using Domain " << this->Domain );
       this->AccessPlist = H5Pcreate( H5P_FILE_ACCESS );
 //      H5Pset_fapl_core( this->AccessPlist, 1000000, 1 );
       H5Pset_fapl_core( this->AccessPlist, 1000000, 0 );
-#ifndef XDMF_NO_MPI
     } else if( STRCASECMP( this->Domain, "DSM" ) == 0 ) {
+#ifndef XDMF_NO_MPI
         XdmfDebug("Using DSM Interface");  
         if(!this->DsmBuffer){
             XdmfErrorMessage("Cannot Open a DSM HDF5 File Until DsmBuffer has been set");
@@ -680,20 +688,8 @@ XdmfDebug("Using Domain " << this->Domain );
       return( XDMF_FAIL );  
 #endif
     } else if( STRCASECMP( this->Domain, "NDGM" ) == 0 ) {
-#ifdef HAVE_NDGM
-      XdmfDebug("Using NDGM Interface");  
-      H5FD_ndgm_init();
-      this->AccessPlist = H5Pcreate( H5P_FILE_ACCESS );
-      XdmfDebug("Using NdgmHost " << this->GetNdgmHost() );
-      if( strlen( this->GetNdgmHost() ) < 2 ) {
-        H5Pset_fapl_ndgm( this->AccessPlist, 1000000, NULL);
-      } else {
-        H5Pset_fapl_ndgm( this->AccessPlist, 1000000, (char*)this->GetNdgmHost());
-      }
-#else
       XdmfErrorMessage("NDGM Interface is unavailable");
       return( XDMF_FAIL );  
-#endif
     } else if( STRCASECMP( this->Domain, "GASS" ) == 0 ) {
     } else {
 // Check for Parallel HDF5 ... MPI must already be initialized
@@ -756,20 +752,8 @@ if( this->File < 0 ){
       return( XDMF_FAIL );
 #endif
     } else if( STRCASECMP( this->Domain, "NDGM" ) == 0 ) {
-      XdmfDebug("Using NDGM Interface");  
-#ifdef HAVE_NDGM
-      H5FD_ndgm_init();
-      this->AccessPlist = H5Pcreate( H5P_FILE_ACCESS );
-      XdmfDebug("Using NdgmHost " << this->GetNdgmHost() );
-      if( strlen( this->GetNdgmHost() ) < 2 ) {
-        H5Pset_fapl_ndgm( this->AccessPlist, 1000000, NULL);
-      } else {
-        H5Pset_fapl_ndgm( this->AccessPlist, 1000000, (char*)this->GetNdgmHost() );
-      }
-#else
       XdmfErrorMessage("NDGM interface is unavailable");
       return( XDMF_FAIL );
-#endif
     } else if( STRCASECMP( this->Domain, "FILE" ) == 0 ) {
     }
     this->File = H5Fcreate(this->FileName,
