@@ -2,9 +2,9 @@
 /*                               XDMF                              */
 /*                   eXtensible Data Model and Format              */
 /*                                                                 */
-/*  Id : $Id: XdmfTopology.cxx,v 1.10 2007/07/12 16:59:50 dave.demarle Exp $  */
-/*  Date : $Date: 2007/07/12 16:59:50 $ */
-/*  Version : $Revision: 1.10 $ */
+/*  Id : $Id: XdmfTopology.cxx,v 1.17 2008-05-02 14:58:15 clarke Exp $  */
+/*  Date : $Date: 2008-05-02 14:58:15 $ */
+/*  Version : $Revision: 1.17 $ */
 /*                                                                 */
 /*  Author:                                                        */
 /*     Jerry A. Clarke                                             */
@@ -58,6 +58,15 @@ XdmfTopology::~XdmfTopology() {
   delete this->Shape;
   }
 
+XdmfInt32
+XdmfTopology::Release(){
+    if( this->ConnectivityIsMine && this->Connectivity ) delete this->Connectivity;
+    if(this->CellOffsets) delete this->CellOffsets;
+    this->Connectivity = NULL;
+    this->CellOffsets = NULL;
+    return(XDMF_SUCCESS);
+}
+
 XdmfInt32
 XdmfTopology::Build(){
     if(XdmfElement::Build() != XDMF_SUCCESS) return(XDMF_FAIL);
@@ -68,6 +77,12 @@ XdmfTopology::Build(){
     if( this->OrderIsDefault == 0 ){
         this->Set("Order", this->GetOrderAsString());
     } 
+    if( this->BaseOffset != 0 ){
+        char Offset[80];
+        ostrstream OffsetStream(Offset, 80);
+        OffsetStream << this->BaseOffset << ends;
+        this->Set("BaseOffset", Offset);
+    }
     if(this->Connectivity){
         XdmfDataItem    *di = NULL;
         XdmfXmlNode     node;
@@ -569,19 +584,24 @@ if( this->TopologyType == XDMF_NOTOPOLOGY ){
   }
 if( this->GetClass() == XDMF_UNSTRUCTURED ){
   // Read Connection
-  ConnectionElement = this->DOM->FindElement( NULL, 0, this->Element );
+  ConnectionElement = this->DOM->FindDataElement(0, this->Element );
   if( ConnectionElement ){
       XdmfDataItem  Connections;
   
+    // cout << "Connection DataItem = " << &Connections << endl;
     XdmfDebug("Reading Connections from DataItem");
     if(Connections.SetDOM(this->DOM) == XDMF_FAIL) return(XDMF_FAIL);
+#ifndef XDMF_NO_MPI
+    Connections.SetDsmBuffer(this->DsmBuffer);
+#endif
     if( this->ConnectivityIsMine && this->Connectivity ) delete this->Connectivity;
-    if(Connections.SetElement(ConnectionElement) == XDMF_FAIL) return(XDMF_FAIL);
+    if(Connections.SetElement(ConnectionElement, 0) == XDMF_FAIL) return(XDMF_FAIL);
     if(Connections.UpdateInformation() == XDMF_FAIL) return(XDMF_FAIL);
     if(Connections.Update() == XDMF_FAIL) return(XDMF_FAIL);
     // Steal the Array so it doesn't get deleted in the destructor of the DataItem
     this->Connectivity = Connections.GetArray();
     Connections.SetArrayIsMine(0);
+    this->ConnectivityIsMine = 1;
 
     if( this->BaseOffset ) {
       XdmfDebug("Adjusting due to BaseOffset");
@@ -593,6 +613,7 @@ if( this->GetClass() == XDMF_UNSTRUCTURED ){
       }
   } else {
     // Default Connectivity
+    XdmfDebug("Using Default Connectivity");
     XdmfInt64  Length;
     if( !this->Connectivity ){
       this->Connectivity = new XdmfArray;
@@ -602,6 +623,7 @@ if( this->GetClass() == XDMF_UNSTRUCTURED ){
     Length = this->GetShapeDesc()->GetNumberOfElements() * this->NodesPerElement;
     this->Connectivity->SetNumberOfElements( Length );
     this->Connectivity->Generate( 0 , Length - 1 );
+    XdmfDebug("Default Connectivity Set");
   }
   if( this->OrderIsDefault == 0 ){
     XdmfInt64  i,j;
