@@ -18,7 +18,7 @@
 #include "vtkMath.h"
 #include <assert.h>
 
-vtkCxxRevisionMacro(vtkLookupTable, "$Revision: 1.105 $");
+vtkCxxRevisionMacro(vtkLookupTable, "$Revision: 1.107 $");
 vtkStandardNewMacro(vtkLookupTable);
 
 // Construct with range=(0,1); and hsv ranges set up for rainbow color table 
@@ -50,6 +50,8 @@ vtkLookupTable::vtkLookupTable(int sze, int ext)
 
   this->Ramp = VTK_RAMP_SCURVE;
   this->Scale = VTK_SCALE_LINEAR;
+  
+  this->OpaqueFlag=1;
 }
 
 //----------------------------------------------------------------------------
@@ -59,6 +61,30 @@ vtkLookupTable::~vtkLookupTable()
   this->Table = NULL;
 }
 
+//----------------------------------------------------------------------------
+// Description:
+// Return true if all of the values defining the mapping have an opacity
+// equal to 1. Default implementation return true.
+int vtkLookupTable::IsOpaque()
+{
+  if(this->OpaqueFlagBuildTime<this->GetMTime())
+    {
+    int opaque=1;
+    int size=this->Table->GetNumberOfTuples();
+    int i=0;
+    unsigned char *ptr=this->Table->GetPointer(0);
+    while(opaque && i<size)
+      {
+      opaque=ptr[3]==255;
+      ptr+=4;
+      ++i;
+      }
+    this->OpaqueFlag=opaque;
+    this->OpaqueFlagBuildTime.Modified();
+    }
+  return this->OpaqueFlag;
+}
+
 //----------------------------------------------------------------------------
 // Scalar values greater than maximum range value are clamped to maximum
 // range value.
@@ -206,7 +232,7 @@ void vtkLookupTable::ForceBuild()
         }
         break;
       default:
-        assert("check: impossible case."); // reaching this line is a bug.
+        assert("check: impossible case." && 0); // reaching this line is a bug.
         break;
       }
     }
