@@ -35,12 +35,10 @@
 # endif
 #endif
 
-vtkCxxRevisionMacro(vtkMath, "$Revision: 1.124 $");
+vtkCxxRevisionMacro(vtkMath, "$Revision: 1.129 $");
 vtkStandardNewMacro(vtkMath);
 
 long vtkMath::Seed = 1177; // One authors home address
-static const double sqrt3 = sqrt( static_cast<double>(3.) );
-static const double inv3 = 1 / 3.;
 
 // Avoid aliasing optimization problems by using a union:
 union vtkIEEE754Bits {
@@ -116,6 +114,14 @@ void vtkMath::RandomSeed(long s)
   vtkMath::Random();
 }
 
+//----------------------------------------------------------------------------
+// Description:
+// Return the current seed used by the random number generator.
+long vtkMath::GetSeed()
+{
+  return vtkMath::Seed;
+}
+
 //----------------------------------------------------------------------------
 // The number of combinations of n objects from a pool of m objects (m>n).
 //
@@ -905,189 +911,6 @@ double* vtkMath::SolveCubic( double c0, double c1, double c2, double c3 )
   return roots;
 }
 
-extern "C" {
-  int vtkMathCompareRoots(const void* a, const void* b)
-  {
-    return (*((const double*)a)) < (*((const double*)b)) ? -1 : 1; 
-  }
-}
-
-// Algebraically extracts REAL roots of the quartic polynomial with 
-// REAL coefficients X^4 + c[0] X^3 + c[1] X^2 + c[2] X + c[3]
-// and stores them (when they exist) and their respective multiplicities
-// in the r and m arrays.
-int vtkMath::FerrariSolve( double* c, double* r, int* m )
-{
-  // step 0: eliminate trivial cases up to numerical noise
-  if ( fabs( c[3] ) < VTK_DBL_EPSILON )
-    {
-    if ( fabs( c[2] ) < VTK_DBL_EPSILON )
-      {
-      if ( fabs( c[1] ) < VTK_DBL_EPSILON )
-        {
-        if ( fabs( c[0] ) < VTK_DBL_EPSILON )
-          {
-          r[0] = 0.;
-          m[0] = 4;
-          return 1;
-          }
-        else
-          {
-          r[0] = - c[1];
-          m[0] = 1;
-          r[1] = 0.;
-          m[1] = 3;
-          return 2;
-          }
-        }
-      else
-        {
-        double cc[3];
-        cc[0] = 1.;
-        cc[1] = c[0];
-        cc[2] = c[1];
-        int nr = vtkMath::SolveQuadratic( cc, r, m );
-        r[nr] = 0.;
-        m[nr] = 2;
-        return nr + 1;
-        }
-      }
-    else
-      {
-      int nr = vtkMath::TartagliaCardanSolve( c - 1, r, m );
-      r[nr] = 0.;
-      m[nr] = 1;
-      return nr + 1;
-      }
-    }
-  if ( ( fabs( c[0] ) < VTK_DBL_EPSILON ) && ( fabs( c[2] ) < VTK_DBL_EPSILON ) )
-    {
-    if ( fabs( c[1] ) < VTK_DBL_EPSILON )
-      {
-      if ( c[3] < 0. ) return 0;
-      r[0] = sqrt( sqrt( c[3] ) );
-      m[0] = 4;
-      return 1;
-      }
-    double cc[3], cr[2];
-    int cm[2];
-    cc[0] = 1.;
-    cc[1] = c[1];
-    cc[2] = c[3];
-    int nr1 = vtkMath::SolveQuadratic( cc, cr, cm );
-    int nr = 0;
-    int i;
-    for ( i = 0; i < nr1; ++ i )
-      {
-      if ( fabs( cr[i] ) < VTK_DBL_EPSILON )
-        {
-        r[nr] = 0.;
-        m[nr ++] = 2 * cm[i];
-        }
-      else 
-        {
-        if ( cr[i] > VTK_DBL_EPSILON )
-          {
-          r[nr] = sqrt( cr[i] );
-          m[nr ++] = cm[i];
-          r[nr] = - sqrt( cr[i] );
-          m[nr ++] = cm[i];
-          }
-        }
-      }
-    return nr;
-    }
-  
-  // step 1: reduce to X^4 + aX^2 + bX + d
-  double p2d8 = c[0] * c[0] * .125 ;
-  double qd2 = c[1] * .5;
-  double a = c[1] - 3 * p2d8;
-  double b = c[0] * ( p2d8 - qd2 ) + c[2];
-  double d = p2d8 * ( qd2 - .75 * p2d8 ) - c[0] * c[2] * .25 + c[3];
-  // expedite the case when the reduced equation is biquadratic
-  if ( fabs( b ) < VTK_DBL_EPSILON )
-    {
-    double cc[3], cr[2];
-    int cm[2];
-    cc[0] = 1.;
-    cc[1] = a;
-    cc[2] = d;
-    int nr1 = vtkMath::SolveQuadratic( cc, cr, cm );
-    int nr = 0;
-    double shift = - c[0] * .25;
-    int i;
-    for ( i = 0; i < nr1; ++ i )
-      {
-      if ( fabs( cr[i] ) < VTK_DBL_EPSILON )
-        {
-        r[nr] = shift;
-        m[nr ++] = 2 * cm[i];
-        }
-      else 
-        {
-        if ( cr[i] > VTK_DBL_EPSILON )
-          {
-          r[nr] = sqrt( cr[i] ) + shift;
-          m[nr ++] = cm[i];
-          r[nr] = - sqrt( cr[i] ) + shift;
-          m[nr ++] = cm[i];
-          }
-        }
-      }
-    return nr;
-    }
-
-  // step 2: solve the companion cubic
-  double cc[4], cr[3];
-  double unsorted[8];
-  int cm[3];
-  cc[1] = 2. * a;
-  cc[2] = a * a - 4. * d;
-  cc[3] = - b * b;
-  int nr = vtkMath::TartagliaCardanSolve( cc, cr, cm );
-
-  // step 3: figure alpha^2
-  double alpha2 = cr[-- nr];
-  while ( alpha2 < 0. && nr ) alpha2 = cr[-- nr];
-
-  // step 4: solve the quadratics
-  cc[0] = 1.;
-  cc[1] = sqrt( alpha2 );
-  double rho = - b / cc[1];
-  cc[2] = ( a + alpha2 + rho ) * .5;
-  int nr1 = vtkMath::SolveQuadratic( cc, r, m );
-  cc[1] = - cc[1];
-  cc[2] -= rho;
-  nr = nr1 + vtkMath::SolveQuadratic( cc, r + nr1, m + nr1 );
-  if ( ! nr ) return 0;
-
-  // step 5: sort, filter and shift roots (if any)
-  int i;
-  for ( i = 0; i < nr; ++ i )
-    {
-    unsorted[2*i] = r[i];
-    unsorted[2*i + 1] = m[i];
-    }
-  qsort( unsorted, nr, 2*sizeof( double ), vtkMathCompareRoots );
-  r[0] = unsorted[0];
-  m[0] = static_cast<int>(unsorted[1]);
-  nr1 = 1;
-  for ( i = 1; i < nr; ++ i )
-    {
-    if ( unsorted[2*i] == unsorted[2*i - 2] )
-      {
-      m[i - 1] += static_cast<int>(unsorted[2*i + 1]);
-      continue;
-      }
-    r[nr1] = unsorted[2*i];
-    m[nr1++] = static_cast<int>(unsorted[2*i + 1]);
-    }
-  double shift = - c[0] * .25;
-  for ( i = 0; i < nr1; ++ i ) r[i] += shift;
-
-  return nr1;
-}
-
 //----------------------------------------------------------------------------
 // Solves a cubic equation when c0, c1, c2, And c3 Are REAL.  Solution
 // is motivated by Numerical Recipes In C 2nd Ed.  Roots and number of
@@ -1202,139 +1025,6 @@ int vtkMath::SolveCubic( double c0, double c1, double c2, double c3,
     }
 }
 
-//----------------------------------------------------------------------------
-// Algebraically extracts REAL roots of the cubic polynomial with 
-// REAL coefficients X^3 + c[1] X^2 + c[2] X + c[3]
-// and stores them (when they exist) and their respective multiplicities.
-// The main differences with SolveCubic are that (1) the polynomial must have
-// unit leading coefficient, (2) no information is returned regarding complex
-// roots, and (3) non-simple roots are stored only once -- this is a
-// specialized solver.
-// Returns the number of roots.
-// 
-int vtkMath::TartagliaCardanSolve( double* c, double* r, int* m )
-{
-  // step 0: eliminate trivial cases up to numerical noise
-  if ( fabs( c[3] ) < VTK_DBL_EPSILON )
-    {
-    r[0] = 0.;
-    if ( fabs( c[2] ) < VTK_DBL_EPSILON )
-      {
-      if ( fabs( c[1] ) < VTK_DBL_EPSILON )
-        {
-        m[0] = 3;
-        return 1;
-        }
-      else
-        {
-        m[0] = 2;
-        r[1] = - c[1];
-        m[1] = 1;
-        return 2;
-        }
-      }
-    else
-      {
-      m[0] = 1;
-      double delta = c[1] * c[1] - 4. * c[2];
-      if ( delta > VTK_DBL_EPSILON )
-        {
-        delta = sqrt( delta );
-        r[1] = ( - delta - c[1] ) * 0.5;
-        m[1] = 1;
-        r[2] = ( delta - c[1] ) * 0.5;
-        m[2] = 1;
-        return 3;
-        }
-      else
-        {
-        if ( delta < - VTK_DBL_EPSILON ) return 1;
-        r[1] = - c[1] * 0.5;
-        m[1] = 2;
-        return 2;
-        }
-      }
-    }
-
-  // step 1: reduce to X^3 + pX + q
-  double shift = - c[1] / 3.;
-  double a2 = c[1] * c[1];
-  double p = c[2] - a2 / 3.;
-  double q = c[1] * ( 2. * a2 / 9. - c[2] ) / 3. + c[3];
-
-  // step 2: compute the trivial real roots if p or q are 0
-  // case 2.1: p = 0: 1 triple real root
-  if ( fabs( p ) < VTK_DBL_EPSILON )
-    {
-    if ( fabs( q ) < VTK_DBL_EPSILON )
-      {
-      r[0] = + shift;
-      m[0] = 3;
-      return 1;
-      }
-    double x;
-    x = q < 0 ? pow( - q, inv3 ) : - pow( q, inv3 );
-    r[0] = x + shift;
-    m[0] = 3;
-    return 1;
-    }
-
-  // case 2.2: q = 0: 1 ( p > 0 ) or 3 ( p < 0 ) simple real root(s) 
-  if ( fabs( q ) < VTK_DBL_EPSILON )
-    {
-    r[0] = + shift;
-    m[0] = 1;
-    if ( p < 0 )
-      {
-      double x = sqrt( - p );
-      r[1] =  x + shift;
-      r[2] =  - x + shift;
-      m[1] = m[2] = 1;
-      return 3;
-      }
-    return 1;
-    }
-
-  // step 3: compute discriminant
-  double p_3 = p * inv3;
-  double q_2 = q * 0.5;
-  double D = p_3 * p_3 * p_3 + q_2 * q_2;
-
-  // step 4: compute roots depending on the discriminant
-  double u;
-  // 4.1: case D = 0: 1 simple and 1 double real roots
-  if ( fabs( D ) < VTK_DBL_EPSILON )
-    {
-    u = q > 0 ? - pow( q_2, inv3 ) : pow( - q_2, inv3 );
-    r[0] =  2. * u + shift;
-    m[0] = 1;
-    r[1] =  - u + shift;
-    m[1] = 2;
-    return 2;
-    }
-  // 4.2: case D > 0: 1 simple real root
-  if ( D > 0 )
-    {
-    u = sqrt( D ) - q_2;
-    u = u < 0 ? - pow( - u, inv3 ) : pow( u, inv3 );
-    r[0] = u - p_3 / u + shift;
-    m[0] = 1;
-    return 1;
-    }
-  // 5.3: case D < 0: 3 simple real roots
-  double smp_3 = sqrt( - p_3 );
-  double argu  = acos( q_2 / ( p_3 * smp_3 ) ) * inv3;
-  double x1 = cos( argu );
-  double x2 = sqrt3 * sqrt( 1. - x1 * x1 );
-  x1 *= smp_3;
-  x2 *= smp_3;
-  r[0] = 2. * x1 + shift;
-  r[1] = x2 - x1 + shift;
-  r[2] = r[1]  - 2. * x2;
-  m[0] = m[1] = m[2] = 1;
-  return 3;
-}
-
 //----------------------------------------------------------------------------
 // Solves a quadratic equation c1*t^2 + c2*t + c3 = 0 when c1, c2, and
 // c3 are REAL.  Solution is motivated by Numerical Recipes In C 2nd
@@ -3052,12 +2742,12 @@ void vtkMath::LabToXYZ(double L, double a, double b,
 
   if ( pow(var_Z,3) > 0.008856 ) var_Z = pow(var_Z,3);
   else var_Z = ( var_Z - 16.0 / 116.0 ) / 7.787;
-  const double ref_X =  95.047;
-  const double ref_Y = 100.000;
-  const double ref_Z = 108.883;
-  *x = ref_X * var_X;     //ref_X =  95.047  Observer= 2 deg Illuminant= D65
-  *y = ref_Y * var_Y;     //ref_Y = 100.000
-  *z = ref_Z * var_Z;     //ref_Z = 108.883
+  const double ref_X = 0.9505;
+  const double ref_Y = 1.000;
+  const double ref_Z = 1.089;
+  *x = ref_X * var_X;     //ref_X = 0.9505  Observer= 2 deg Illuminant= D65
+  *y = ref_Y * var_Y;     //ref_Y = 1.000
+  *z = ref_Z * var_Z;     //ref_Z = 1.089
 }
 
 //-----------------------------------------------------------------------------
@@ -3072,12 +2762,12 @@ double *vtkMath::LabToXYZ(const double lab[3])
 void vtkMath::XYZToLab(double x, double y, double z,
                        double *L, double *a, double *b)
 {
-  const double ref_X =  95.047;
-  const double ref_Y = 100.000;
-  const double ref_Z = 108.883;
-  double var_X = x / ref_X;  //ref_X =  95.047  Observer= 2 deg, Illuminant= D65
-  double var_Y = y / ref_Y;  //ref_Y = 100.000
-  double var_Z = z / ref_Z;  //ref_Z = 108.883
+  const double ref_X = 0.9505;
+  const double ref_Y = 1.000;
+  const double ref_Z = 1.089;
+  double var_X = x / ref_X;  //ref_X = 0.9505  Observer= 2 deg, Illuminant= D65
+  double var_Y = y / ref_Y;  //ref_Y = 1.000
+  double var_Z = z / ref_Z;  //ref_Z = 1.089
 
   if ( var_X > 0.008856 ) var_X = pow(var_X, 1.0/3.0);
   else                    var_X = ( 7.787 * var_X ) + ( 16.0 / 116.0 );
@@ -3103,29 +2793,30 @@ double *vtkMath::XYZToLab(const double xyz[3])
 void vtkMath::XYZToRGB(double x, double y, double z,
                        double *r, double *g, double *b)
 {
-  //double ref_X =  95.047;        //Observer = 2 deg Illuminant = D65
-  //double ref_Y = 100.000;
-  //double ref_Z = 108.883;
+  //double ref_X = 0.9505;        //Observer = 2 deg Illuminant = D65
+  //double ref_Y = 1.000;
+  //double ref_Z = 1.089;
  
-  double var_X = x / 100;        //X = From 0 to ref_X
-  double var_Y = y / 100;        //Y = From 0 to ref_Y
-  double var_Z = z / 100;        //Z = From 0 to ref_Z
- 
-  double var_R = var_X *  3.2406 + var_Y * -1.5372 + var_Z * -0.4986;
-  double var_G = var_X * -0.9689 + var_Y *  1.8758 + var_Z *  0.0415;
-  double var_B = var_X *  0.0557 + var_Y * -0.2040 + var_Z *  1.0570;
- 
-  if ( var_R > 0.0031308 ) var_R = 1.055 * ( pow(var_R, ( 1 / 2.4 )) ) - 0.055;
-  else var_R = 12.92 * var_R;
-  if ( var_G > 0.0031308 ) var_G = 1.055 * ( pow(var_G ,( 1 / 2.4 )) ) - 0.055;
-  else  var_G = 12.92 * var_G;
-  if ( var_B > 0.0031308 ) var_B = 1.055 * ( pow(var_B, ( 1 / 2.4 )) ) - 0.055;
-  else var_B = 12.92 * var_B;
-
-  *r = var_R;
-  *g = var_G;
-  *b = var_B;
+  *r = x *  3.2406 + y * -1.5372 + z * -0.4986;
+  *g = x * -0.9689 + y *  1.8758 + z *  0.0415;
+  *b = x *  0.0557 + y * -0.2040 + z *  1.0570;
 
+  // The following performs a "gamma correction" specified by the sRGB color
+  // space.  sRGB is defined by a cononical definition of a display monitor and
+  // has been standardized by the International Electrotechnical Commission (IEC
+  // 61966-2-1).  However, it is a non-linear color space, which means that it
+  // will invalidate the color computations done by OpenGL.  Furthermore, most
+  // OS or display drivers will do some gamma correction on there own, so
+  // displaying these colors directly usually results in overly bright images.
+  // Thus, the non-linear RGB values are inappropriate for VTK, so the code is
+  // commented out.  If someone needs sRGB values in the future, there should be
+  // a separate set of color conversion functions for that.
+//   if (*r > 0.0031308) *r = 1.055 * (pow(*r, ( 1 / 2.4 ))) - 0.055;
+//   else *r = 12.92 * (*r);
+//   if (*g > 0.0031308) *g = 1.055 * (pow(*g ,( 1 / 2.4 ))) - 0.055;
+//   else  *g = 12.92 * )*g);
+//   if (*b > 0.0031308) *b = 1.055 * (pow(*b, ( 1 / 2.4 ))) - 0.055;
+//   else *b = 12.92 * (*b);
   
   // Clip colors. ideally we would do something that is perceptually closest
   // (since we can see colors outside of the display gamut), but this seems to
@@ -3157,25 +2848,27 @@ double *vtkMath::XYZToRGB(const double xyz[3])
 void vtkMath::RGBToXYZ(double r, double g, double b,
                        double *x, double *y, double *z)
 {
-  double var_R;
-  double var_G;
-  double var_B;
-
-  if ( r > 0.04045 ) var_R = pow(( r + 0.055 ) / 1.055, 2.4);
-  else               var_R = r / 12.92;
-  if ( g > 0.04045 ) var_G = pow(( g + 0.055 ) / 1.055, 2.4);
-  else               var_G = g / 12.92;
-  if ( b > 0.04045 ) var_B = pow(( b + 0.055 ) / 1.055, 2.4);
-  else               var_B = b / 12.92;
-
-  var_R = var_R * 100;
-  var_G = var_G * 100;
-  var_B = var_B * 100;
+  // The following performs an inverse "gamma correction" specified by the sRGB
+  // color space.  sRGB is defined by a cononical definition of a display
+  // monitor and has been standardized by the International Electrotechnical
+  // Commission (IEC 61966-2-1).  However, it is a non-linear color space, which
+  // means that it will invalidate the color computations done by OpenGL.
+  // Furthermore, most OS or display drivers will do some gamma correction on
+  // there own, so displaying these colors directly usually results in overly
+  // bright images.  Thus, the non-linear RGB values are inappropriate for VTK,
+  // so the code is commented out.  If someone needs sRGB values in the future,
+  // there should be a separate set of color conversion functions for that.
+//   if ( r > 0.04045 ) r = pow(( r + 0.055 ) / 1.055, 2.4);
+//   else               r = r / 12.92;
+//   if ( g > 0.04045 ) g = pow(( g + 0.055 ) / 1.055, 2.4);
+//   else               g = g / 12.92;
+//   if ( b > 0.04045 ) b = pow(( b + 0.055 ) / 1.055, 2.4);
+//   else               b = b / 12.92;
 
   //Observer. = 2 deg, Illuminant = D65
-  *x = var_R * 0.4124 + var_G * 0.3576 + var_B * 0.1805;
-  *y = var_R * 0.2126 + var_G * 0.7152 + var_B * 0.0722;
-  *z = var_R * 0.0193 + var_G * 0.1192 + var_B * 0.9505;
+  *x = r * 0.4124 + g * 0.3576 + b * 0.1805;
+  *y = r * 0.2126 + g * 0.7152 + b * 0.0722;
+  *z = r * 0.0193 + g * 0.1192 + b * 0.9505;
 }
 
 //-----------------------------------------------------------------------------
@@ -3387,7 +3080,7 @@ int vtkMath::GetAdjustedScalarRange(
         }
       break;
     default:
-      assert("check: impossible case."); // reaching this line is a bug.
+      assert("check: impossible case." && 0); // reaching this line is a bug.
       break;
     }
 
