@@ -12,14 +12,14 @@
      PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================
-  Copyright 2005 Sandia Corporation.
+  Copyright 2007 Sandia Corporation.
   Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
   license for use of this work by or on behalf of the
   U.S. Government. Redistribution and use in source and binary forms, with
   or without modification, are permitted provided that this Notice and any
   statement of authorship are reproduced on all copies.
 
-  Contact: pppebay@ca.sandia.gov,dcthomp@sandia.gov,
+  Contact: pppebay@ca.sandia.gov,dcthomp@sandia.gov
 
 =========================================================================*/
 // .NAME vtkMath - performs common math operations
@@ -34,6 +34,7 @@
 #define __vtkMath_h
 
 #include "vtkObject.h"
+#include <assert.h> // assert() in inline implementations.
 
 #ifndef DBL_EPSILON
 #  define VTK_DBL_EPSILON    2.2204460492503131e-16
@@ -436,6 +437,10 @@ public:
   // repeatability of Random().
   static void RandomSeed(long s);  
 
+  // Description:
+  // Return the current seed used by the random number generator.
+  static long GetSeed();
+  
   // Description:
   // Generate random numbers between 0.0 and 1.0.
   // This is used to provide portability across different systems.
@@ -486,16 +491,6 @@ public:
   // Return array contains number of roots followed by roots themselves.
   static double* SolveLinear(double c0, double c1);
 
-  // Description:
-  // Algebraically extracts REAL roots of the quartic polynomial with 
-  // REAL coefficients X^4 + c[0] X^3 + c[1] X^2 + c[2] X + c[3]
-  // and stores them (when they exist) and their respective multiplicities
-  // in the \a r and \a m arrays, based on Ferrari's method.
-  // Some numerical noise is filtered by the use of relative tolerance
-  // instead of equality with 0.
-  // Returns the number of roots.
-  static int FerrariSolve( double* c, double* r, int* m );
-
   // Description:
   // Solves a cubic equation when c0, c1, c2, And c3 Are REAL.  Solution
   // is motivated by Numerical Recipes In C 2nd Ed.  Roots and number of
@@ -511,20 +506,6 @@ public:
   static int SolveCubic(double c0, double c1, double c2, double c3, 
                         double *r1, double *r2, double *r3, int *num_roots);
 
-  // Description:
-  // Algebraically extracts REAL roots of the cubic polynomial with 
-  // REAL coefficients X^3 + c[1] X^2 + c[2] X + c[3]
-  // and stores them (when they exist) and their respective multiplicities
-  // in the \a r and \a m arrays.
-  // The main differences with SolveCubic are that (1) the polynomial must have
-  // unit leading coefficient, (2) complex roots are discarded upfront, 
-  // (3) non-simple roots are stored only once, along with their respective
-  // multiplicities, and (4) some numerical noise is filtered by the use of 
-  // relative tolerance instead of equality with 0.
-  // Returns the number of roots.
-  // <i> In memoriam </i> Niccolo Tartaglia (1500 - 1559), unfairly forgotten.
-  static int TartagliaCardanSolve( double* c, double* r, int* m );
-
   // Description:
   // Solves a quadratic equation c1*t^2  + c2*t  + c3 = 0 when 
   // c1, c2, and c3 are REAL.
@@ -690,6 +671,14 @@ public:
   static void ClampValues(
     const double *values, int nb_values, const double range[2], double *clamped_values);
 
+  // Description:
+  // Clamp a value against a range and then normalized it between 0 and 1.
+  // If range[0]==range[1], the result is 0.
+  // \pre valid_range: range[0]<=range[1]
+  // \post valid_result: result>=0.0 && result<=1.0
+  static double ClampAndNormalizeValue(double value,
+                                       const double range[2]);
+
   // Description:
   // Return the scalar type that is most likely to have enough precision 
   // to store a given range of data once it has been scaled and shifted 
@@ -769,7 +758,7 @@ inline int vtkMath::Floor(double x)
   // round-to-nearest,even mode instead of round-to-nearest,+infinity
   return u.i[0] >> 1;
 #else
-  return (int)floor(x);
+  return static_cast<int>(floor(x));
 #endif
 }
 
@@ -959,4 +948,43 @@ inline void vtkMath::ClampValue(
     }
 }
 
+// ---------------------------------------------------------------------------
+inline double vtkMath::ClampAndNormalizeValue(double value,
+                                              const double range[2])
+{
+  assert("pre: valid_range" && range[0]<=range[1]);
+  
+  double result;
+  if(range[0]==range[1])
+    {
+      result=0.0;
+    }
+  else
+    {
+      // clamp
+      if(value<range[0])
+        {
+          result=range[0];
+        }
+      else
+        {
+          if(value>range[1])
+            {
+              result=range[1];
+            }
+          else
+            {
+              result=value;
+            }
+        }
+
+      // normalize
+      result=(result-range[0])/(range[1]-range[0]);
+    }
+
+  assert("post: valid_result" && result>=0.0 && result<=1.0);
+
+  return result;
+}
+
 #endif
