@@ -12,10 +12,29 @@
      PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
-/*----------------------------------------------------------------------------
- Copyright (c) Sandia Corporation
- See Copyright.txt or http://www.paraview.org/HTML/Copyright.html for details.
-----------------------------------------------------------------------------*/
+/*-------------------------------------------------------------------------
+  Copyright 2008 Sandia Corporation.
+  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+  the U.S. Government retains certain rights in this software.
+-------------------------------------------------------------------------*/
+#ifdef VTK_VARIANT_IMPL
+/*
+All code in this source file is conditionally compiled to work-around
+a build problem on the SGI with MIPSpro 7.4.4.  The prelinker loads
+vtkVariant.cxx while looking for the template definition of
+vtkVariant::ToNumeric<> declared in the vtkVariant.h header.  It wants
+the definition in order to instantiate it in vtkCharArray.o or some
+other object to which the instantiation has been assigned.  For some
+reason providing the explicit instantiation in vtkVariant.o is not
+enough to prevent the prelinker from assigning the instantiation to
+another object.  When vtkVariant.cxx is included in the translation
+unit by the prelinker, it causes additional instantiations of other
+templates, like vtkVariantStringToNumeric.  This sends the prelinker
+into an infinite loop of instantiation requests.  By placing all the
+code in this preprocessing condition, we hide it from the prelinker.
+The CMakeLists.txt file defines the macro to compile this code when
+really building the vtkVariant.o object.
+*/
 #include "vtkVariant.h"
 
 #include "vtkStdString.h"
@@ -30,16 +49,206 @@
 
 #include "vtksys/ios/sstream"
 
-// Implementation of vtkVariant's less than operation
-bool vtkVariantLessThan::operator()(vtkVariant s1, vtkVariant s2) const
+//----------------------------------------------------------------------------
+
+// Implementation of vtkVariant's
+// fast-but-potentially-counterintuitive < operation
+bool vtkVariantStrictWeakOrder::operator()(const vtkVariant& s1, const vtkVariant& s2) const
 {
-  if (s1.IsString() && s2.IsString())
+  // First sort on type if they are different
+  if (s1.Type != s2.Type)
     {
-    return s1.ToString() < s2.ToString();
+    return s1.Type < s2.Type;
+    }
+ 
+  // Next check for nulls
+  if (!(s1.Valid && s2.Valid))
+    {
+    if (!(s1.Valid || s2.Valid))
+      {
+      return false; // nulls are equal to one another
+      }
+    else if (!s1.Valid)
+      {
+      return true; // null is less than any valid value
+      }
+    else
+      {
+      return false;
+      }
+    }
+
+  switch (s1.Type)
+    {
+    case VTK_STRING:
+      return (*(s1.Data.String) < *(s2.Data.String));
+
+    case VTK_OBJECT:
+      return (s1.Data.VTKObject < s2.Data.VTKObject);
+
+    case VTK_CHAR:
+      return (s1.Data.Char < s2.Data.Char);
+
+    case VTK_SIGNED_CHAR:
+      return (s1.Data.SignedChar < s2.Data.SignedChar);
+
+    case VTK_UNSIGNED_CHAR:
+      return (s1.Data.UnsignedChar < s2.Data.UnsignedChar);
+      
+    case VTK_SHORT:
+      return (s1.Data.Short < s2.Data.Short);
+      
+    case VTK_UNSIGNED_SHORT:
+      return (s1.Data.UnsignedShort < s2.Data.UnsignedShort);
+
+    case VTK_INT:
+      return (s1.Data.Int < s2.Data.Int);
+
+    case VTK_UNSIGNED_INT:
+      return (s1.Data.UnsignedInt < s2.Data.UnsignedInt);
+
+    case VTK_LONG:
+      return (s1.Data.Long < s2.Data.Long);
+
+    case VTK_UNSIGNED_LONG:
+      return (s1.Data.UnsignedLong < s2.Data.UnsignedLong);
+
+#if defined(VTK_TYPE_USE___INT64)
+    case VTK___INT64:
+      return (s1.Data.__Int64 < s2.Data.__Int64);
+
+    case VTK_UNSIGNED___INT64:
+      return (s1.Data.Unsigned__Int64 < s2.Data.Unsigned__Int64);
+#endif
+
+#if defined(VTK_TYPE_USE_LONG_LONG)
+    case VTK_LONG_LONG:
+      return (s1.Data.LongLong < s2.Data.LongLong);
+
+    case VTK_UNSIGNED_LONG_LONG:
+      return (s1.Data.UnsignedLongLong < s2.Data.UnsignedLongLong);
+#endif
+
+    case VTK_FLOAT:
+      return (s1.Data.Float < s2.Data.Float);
+      
+    case VTK_DOUBLE:
+      return (s1.Data.Double < s2.Data.Double);
+
+    default:
+      cerr << "ERROR: Unhandled type " << s1.Type << " in vtkVariantStrictWeakOrder\n";
+      return false;
     }
-  return s1.ToDouble() < s2.ToDouble();
 }
 
+// ----------------------------------------------------------------------
+
+bool
+vtkVariantStrictEquality::operator()(const vtkVariant &s1, const vtkVariant &s2) const
+{
+  // First sort on type if they are different
+  if (s1.Type != s2.Type)
+    {
+    cerr << "Types differ: " << s1.Type << " and " << s2.Type << "\n";
+    return false;
+    }
+  
+  // Next check for nulls
+  if (!(s1.Valid && s2.Valid))
+    {
+    cerr << "Validity may differ: " << s1.Valid << " and " << s2.Valid << "\n";
+    return (s1.Valid == s2.Valid);
+    }
+
+  // At this point we know that both variants contain a valid value.
+  switch (s1.Type)
+    {
+    case VTK_STRING:
+    {
+    if (*(s1.Data.String) != *(s2.Data.String))
+      {
+      cerr << "Strings differ: '"
+           << *(s1.Data.String) << "' and '"
+           << *(s2.Data.String) << "'\n";
+      }
+      return (*(s1.Data.String) == *(s2.Data.String));
+    };
+
+    case VTK_OBJECT:
+      return (s1.Data.VTKObject == s2.Data.VTKObject);
+
+    case VTK_CHAR:
+      return (s1.Data.Char == s2.Data.Char);
+
+    case VTK_SIGNED_CHAR:
+      return (s1.Data.SignedChar == s2.Data.SignedChar);
+
+    case VTK_UNSIGNED_CHAR:
+      return (s1.Data.UnsignedChar == s2.Data.UnsignedChar);
+      
+    case VTK_SHORT:
+      return (s1.Data.Short == s2.Data.Short);
+      
+    case VTK_UNSIGNED_SHORT:
+      return (s1.Data.UnsignedShort == s2.Data.UnsignedShort);
+
+    case VTK_INT:
+      return (s1.Data.Int == s2.Data.Int);
+
+    case VTK_UNSIGNED_INT:
+      return (s1.Data.UnsignedInt == s2.Data.UnsignedInt);
+
+    case VTK_LONG:
+      return (s1.Data.Long == s2.Data.Long);
+
+    case VTK_UNSIGNED_LONG:
+      return (s1.Data.UnsignedLong == s2.Data.UnsignedLong);
+
+#if defined(VTK_TYPE_USE___INT64)
+    case VTK___INT64:
+      return (s1.Data.__Int64 == s2.Data.__Int64);
+
+    case VTK_UNSIGNED___INT64:
+      return (s1.Data.Unsigned__Int64 == s2.Data.Unsigned__Int64);
+#endif
+
+#if defined(VTK_TYPE_USE_LONG_LONG)
+    case VTK_LONG_LONG:
+      return (s1.Data.LongLong == s2.Data.LongLong);
+
+    case VTK_UNSIGNED_LONG_LONG:
+      return (s1.Data.UnsignedLongLong == s2.Data.UnsignedLongLong);
+#endif
+
+    case VTK_FLOAT:
+      return (s1.Data.Float == s2.Data.Float);
+      
+    case VTK_DOUBLE:
+      return (s1.Data.Double == s2.Data.Double);
+
+    default:
+      cerr << "ERROR: Unhandled type " << s1.Type << " in vtkVariantStrictEquality\n";
+      return false;
+    }
+}
+
+// ----------------------------------------------------------------------
+
+bool
+vtkVariantLessThan::operator()(const vtkVariant &v1, const vtkVariant &v2) const
+{
+  return v1.operator<(v2);
+}
+
+// ----------------------------------------------------------------------
+
+bool
+vtkVariantEqual::operator()(const vtkVariant &v1, const vtkVariant &v2) const
+{
+  return v1.operator==(v2);
+}
+
+//----------------------------------------------------------------------------
 vtkVariant::vtkVariant()
 {
   this->Valid = 0;
@@ -68,6 +277,12 @@ vtkVariant::vtkVariant(const vtkVariant & other)
 
 const vtkVariant & vtkVariant::operator= (const vtkVariant & other)
 {
+  // Short circuit if assigning to self:
+  if (this == &other)
+    {
+    return *this;
+    }
+
   // First delete current variant item.
   if (this->Valid)
     {
@@ -514,6 +729,101 @@ vtkStdString vtkVariant::ToString() const
   return vtkStdString();
 }
 
+vtkObjectBase* vtkVariant::ToVTKObject() const
+{
+  if (this->IsVTKObject())
+    {
+    return this->Data.VTKObject;
+    }
+  return 0;
+}
+
+vtkAbstractArray* vtkVariant::ToArray() const
+{
+  if (this->IsArray())
+    {
+    return vtkAbstractArray::SafeDownCast(this->Data.VTKObject);
+    }
+  return 0;
+}
+
+template <typename T>
+T vtkVariantStringToNumeric(vtkStdString str, bool* valid, T* vtkNotUsed(ignored) = 0)
+{
+  vtksys_ios::istringstream vstr(str);
+  T data;
+  vstr >> data;
+  // Check for a valid result
+  if (valid)
+    {
+    *valid =  ((vstr.rdstate() & ios::badbit) == 0
+      && (vstr.rdstate() & ios::failbit) == 0);
+    *valid = *valid && vstr.eof();
+    //*valid = (vstr.rdstate() == ios::goodbit);
+    }
+  return data;
+}
+
+//----------------------------------------------------------------------------
+// Definition of ToNumeric
+
+// Visual Studio 6 chokes with "error C2893: Failed to specialize function
+// template" when the definition of ToNumeric is included here and it
+// encounters the various callers of ToNumeric below. So for VS6 include the
+// the implementation of ToNumeric at the bottom of the file.
+//
+#if !defined(_MSC_VER) || (_MSC_VER >= 1300)
+# define VTK_VARIANT_TO_NUMERIC_CXX_INCLUDED
+# include "vtkVariantToNumeric.cxx"
+#endif
+
+//----------------------------------------------------------------------------
+// Explicit instantiations of ToNumeric
+
+// Visual Studio 6 chokes with "fatal error C1001: INTERNAL COMPILER ERROR"
+// when trying to use the vtkVariantToNumericInstantiateMacro, so don't...
+//
+#if defined(_MSC_VER) && (_MSC_VER < 1300)
+# define VTK_VARIANT_NO_INSTANTIATE
+#endif
+
+//----------------------------------------------------------------------------
+// Explicitly instantiate the ToNumeric member template to make sure
+// the symbols are exported from this object file.
+// This explicit instantiation exists to resolve VTK issue #5791.
+
+#if !defined(VTK_VARIANT_NO_INSTANTIATE)
+
+#define vtkVariantToNumericInstantiateMacro(x)                          \
+  template x vtkVariant::ToNumeric< x >(bool*, x*) const
+
+vtkVariantToNumericInstantiateMacro(char);
+vtkVariantToNumericInstantiateMacro(float);
+vtkVariantToNumericInstantiateMacro(double);
+vtkVariantToNumericInstantiateMacro(unsigned char);
+vtkVariantToNumericInstantiateMacro(signed char);
+vtkVariantToNumericInstantiateMacro(short);
+vtkVariantToNumericInstantiateMacro(unsigned short);
+vtkVariantToNumericInstantiateMacro(int);
+vtkVariantToNumericInstantiateMacro(unsigned int);
+vtkVariantToNumericInstantiateMacro(long);
+vtkVariantToNumericInstantiateMacro(unsigned long);
+
+#if defined(VTK_TYPE_USE___INT64)
+vtkVariantToNumericInstantiateMacro(__int64);
+vtkVariantToNumericInstantiateMacro(unsigned __int64);
+#endif
+
+#if defined(VTK_TYPE_USE_LONG_LONG)
+vtkVariantToNumericInstantiateMacro(long long);
+vtkVariantToNumericInstantiateMacro(unsigned long long);
+#endif
+
+#endif
+
+//----------------------------------------------------------------------------
+// Callers causing implicit instantiations of ToNumeric
+
 float vtkVariant::ToFloat(bool* valid) const
 {
   return this->ToNumeric(valid, static_cast<float *>(0));
@@ -603,144 +913,16 @@ vtkTypeUInt64 vtkVariant::ToTypeUInt64(bool* valid) const
   return this->ToNumeric(valid, static_cast<vtkTypeUInt64 *>(0));
 }
 
-vtkObjectBase* vtkVariant::ToVTKObject() const
+bool vtkVariant::IsEqual(const vtkVariant& other) const
 {
-  if (this->IsVTKObject())
-    {
-    return this->Data.VTKObject;
-    }
-  return 0;
+  return this->operator==(other);
 }
 
-vtkAbstractArray* vtkVariant::ToArray() const
-{
-  if (this->IsArray())
-    {
-    return vtkAbstractArray::SafeDownCast(this->Data.VTKObject);
-    }
-  return 0;
-}
+//----------------------------------------------------------------------------
+// Definition of ToNumeric if not already included above:
 
-template <typename T>
-T vtkVariantStringToNumeric(vtkStdString str, bool* valid, T* vtkNotUsed(ignored) = 0)
-{
-  vtksys_ios::istringstream vstr(str);
-  T data;
-  vstr >> data;
-  // Check for a valid result
-  if (valid)
-    {
-    *valid =  ((vstr.rdstate() & ios::badbit) == 0
-      && (vstr.rdstate() & ios::failbit) == 0);
-    *valid = *valid && vstr.eof();
-    //*valid = (vstr.rdstate() == ios::goodbit);
-    }
-  return data;
-}
-
-template <typename T>
-T vtkVariant::ToNumeric(bool* valid, T* vtkNotUsed(ignored)) const
-{
-  if (valid)
-    {
-    *valid = true;
-    }
-  if (this->IsString())
-    {
-    return vtkVariantStringToNumeric<T>(* this->Data.String, valid);
-    }
-  if (this->IsFloat())
-    {
-    return static_cast<T>(this->Data.Float);
-    }
-  if (this->IsDouble())
-    {
-    return static_cast<T>(this->Data.Double);
-    }
-  if (this->IsChar())
-    {
-    return static_cast<T>(this->Data.Char);
-    }
-  if (this->IsUnsignedChar())
-    {
-    return static_cast<T>(this->Data.UnsignedChar);
-    }
-  if (this->IsSignedChar())
-    {
-    return static_cast<T>(this->Data.SignedChar);
-    }
-  if (this->IsShort())
-    {
-    return static_cast<T>(this->Data.Short);
-    }
-  if (this->IsUnsignedShort())
-    {
-    return static_cast<T>(this->Data.UnsignedShort);
-    }
-  if (this->IsInt())
-    {
-    return static_cast<T>(this->Data.Int);
-    }
-  if (this->IsUnsignedInt())
-    {
-    return static_cast<T>(this->Data.UnsignedInt);
-    }
-  if (this->IsLong())
-    {
-    return static_cast<T>(this->Data.Long);
-    }
-  if (this->IsUnsignedLong())
-    {
-    return static_cast<T>(this->Data.UnsignedLong);
-    }
-#if defined(VTK_TYPE_USE___INT64)
-  if (this->Is__Int64())
-    {
-    return static_cast<T>(this->Data.__Int64);
-    }
-  if (this->IsUnsigned__Int64())
-    {
-    return static_cast<T>(static_cast<__int64>(this->Data.Unsigned__Int64));
-    }
+#if !defined(VTK_VARIANT_TO_NUMERIC_CXX_INCLUDED)
+# define VTK_VARIANT_TO_NUMERIC_CXX_INCLUDED
+# include "vtkVariantToNumeric.cxx"
 #endif
-#if defined(VTK_TYPE_USE_LONG_LONG)
-  if (this->IsLongLong())
-    {
-    return static_cast<T>(this->Data.LongLong);
-    }
-  if (this->IsUnsignedLongLong())
-    {
-    return static_cast<T>(this->Data.UnsignedLongLong);
-    }
 #endif
-  // For arrays, convert the first value to the appropriate type.
-  if (this->IsArray())
-    {
-    if (this->Data.VTKObject->IsA("vtkDataArray"))
-      {
-      // Note: This are not the best conversion.
-      //       We covert the first value to double, then
-      //       cast it back to the appropriate numeric type.
-      vtkDataArray* da = vtkDataArray::SafeDownCast(this->Data.VTKObject);
-      return static_cast<T>(da->GetTuple1(0));
-      }
-    if (this->Data.VTKObject->IsA("vtkVariantArray"))
-      {
-      // Note: This are not the best conversion.
-      //       We covert the first value to double, then
-      //       cast it back to the appropriate numeric type.
-      vtkVariantArray* va = vtkVariantArray::SafeDownCast(this->Data.VTKObject);
-      return static_cast<T>(va->GetValue(0).ToDouble());
-      }
-    if (this->Data.VTKObject->IsA("vtkStringArray"))
-      {
-      vtkStringArray* sa = vtkStringArray::SafeDownCast(this->Data.VTKObject);
-      return vtkVariantStringToNumeric<T>(sa->GetValue(0), valid);
-      }
-    }
-  if (valid)
-    {
-    *valid = false;
-    }
-  return static_cast<T>(0);
-}
