@@ -23,25 +23,30 @@
 #include "vtkErrorCode.h"
 #include "vtkFieldData.h"
 #include "vtkGarbageCollector.h"
+#include "vtkGraph.h"
 #include "vtkInformation.h"
+#include "vtkInformationExecutivePortKey.h"
+#include "vtkInformationExecutivePortVectorKey.h"
 #include "vtkInformationInformationVectorKey.h"
 #include "vtkInformationIntegerKey.h"
 #include "vtkInformationStringKey.h"
+#include "vtkInformationStringVectorKey.h"
 #include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkSmartPointer.h"
 #include "vtkStreamingDemandDrivenPipeline.h"
+#include "vtkTable.h"
 
 #include <vtkstd/set>
 #include <vtkstd/vector>
 
-vtkCxxRevisionMacro(vtkAlgorithm, "$Revision: 1.39 $");
+vtkCxxRevisionMacro(vtkAlgorithm, "$Revision: 1.44 $");
 vtkStandardNewMacro(vtkAlgorithm);
 
 vtkCxxSetObjectMacro(vtkAlgorithm,Information,vtkInformation);
 
-vtkInformationKeyMacro(vtkAlgorithm, INPUT_REQUIRED_DATA_TYPE, String);
+vtkInformationKeyMacro(vtkAlgorithm, INPUT_REQUIRED_DATA_TYPE, StringVector);
 vtkInformationKeyMacro(vtkAlgorithm, INPUT_IS_OPTIONAL, Integer);
 vtkInformationKeyMacro(vtkAlgorithm, INPUT_IS_REPEATABLE, Integer);
 vtkInformationKeyMacro(vtkAlgorithm, INPUT_REQUIRED_FIELDS, InformationVector);
@@ -378,6 +383,46 @@ vtkAbstractArray* vtkAlgorithm::GetInputAbstractArrayToProcess(
       return fd->GetAbstractArray(name);
       }
     
+    if (fieldAssoc == vtkDataObject::FIELD_ASSOCIATION_ROWS)
+      {
+      vtkTable *inputT = vtkTable::SafeDownCast(input);
+      if (!inputT)
+        {
+        vtkErrorMacro("Attempt to get row data from a non-table");
+        return NULL;
+        }
+      vtkFieldData *fd = inputT->GetRowData();
+      return fd->GetAbstractArray(name);
+      }
+
+    if (fieldAssoc == vtkDataObject::FIELD_ASSOCIATION_VERTICES ||
+        fieldAssoc == vtkDataObject::FIELD_ASSOCIATION_EDGES)
+      {
+      vtkGraph *inputG = vtkGraph::SafeDownCast(input);
+      if (!inputG)
+        {
+        vtkErrorMacro("Attempt to get vertex or edge data from a non-graph");
+        return NULL;
+        }
+      vtkFieldData *fd = 0;
+      if (fieldAssoc == vtkDataObject::FIELD_ASSOCIATION_VERTICES)
+        {
+        fd = inputG->GetVertexData();
+        }
+      else
+        {
+        fd = inputG->GetEdgeData();
+        }
+      return fd->GetAbstractArray(name);
+      }
+
+    if (vtkGraph::SafeDownCast(input) && 
+        fieldAssoc == vtkDataObject::FIELD_ASSOCIATION_POINTS)
+      {
+      return vtkGraph::SafeDownCast(input)->
+        GetVertexData()->GetAbstractArray(name);
+      }
+
     vtkDataSet *inputDS = vtkDataSet::SafeDownCast(input);
     if (!inputDS)
       {
@@ -573,9 +618,9 @@ void vtkAlgorithm::SetNumberOfOutputPorts(int n)
     vtkInformation* info = producer->GetOutputInformation(i);
 
     // Remove all consumers' references to this producer on this port.
-    vtkExecutive** consumers = info->GetExecutives(vtkExecutive::CONSUMERS());
-    int* consumerPorts = info->GetPorts(vtkExecutive::CONSUMERS());
-    int consumerCount = info->Length(vtkExecutive::CONSUMERS());
+    vtkExecutive** consumers = vtkExecutive::CONSUMERS()->GetExecutives(info);
+    int* consumerPorts = vtkExecutive::CONSUMERS()->GetPorts(info);
+    int consumerCount = vtkExecutive::CONSUMERS()->Length(info);
     for(int j=0; j < consumerCount; ++j)
       {
       vtkInformationVector* inputs =
@@ -584,7 +629,7 @@ void vtkAlgorithm::SetNumberOfOutputPorts(int n)
       }
 
     // Remove this producer's references to all consumers on this port.
-    info->Remove(vtkExecutive::CONSUMERS());
+    vtkExecutive::CONSUMERS()->Remove(info);
     }
 
   // Set the number of output port information objects.
@@ -818,7 +863,7 @@ void vtkAlgorithm::SetInputConnection(int port, vtkAlgorithmOutput* input)
   // Add this consumer to the new input's list of consumers.
   if(newInfo)
     {
-    newInfo->Append(vtkExecutive::CONSUMERS(), consumer, consumerPort);
+    vtkExecutive::CONSUMERS()->Append(newInfo, consumer, consumerPort);
     }
 
   // Remove this consumer from all old inputs' lists of consumers.
@@ -826,7 +871,7 @@ void vtkAlgorithm::SetInputConnection(int port, vtkAlgorithmOutput* input)
     {
     if(vtkInformation* oldInfo = inputs->GetInformationObject(i))
       {
-      oldInfo->Remove(vtkExecutive::CONSUMERS(), consumer, consumerPort);
+      vtkExecutive::CONSUMERS()->Remove(oldInfo, consumer, consumerPort);
       }
     }
 
@@ -883,7 +928,7 @@ void vtkAlgorithm::AddInputConnection(int port, vtkAlgorithmOutput* input)
   vtkInformation* newInfo = producer->GetOutputInformation(producerPort);
 
   // Add this consumer to the input's list of consumers.
-  newInfo->Append(vtkExecutive::CONSUMERS(), consumer, consumerPort);
+  vtkExecutive::CONSUMERS()->Append(newInfo, consumer, consumerPort);
 
   // Add the information object to the list of inputs.
   inputs->Append(newInfo);
@@ -926,7 +971,7 @@ void vtkAlgorithm::RemoveInputConnection(int port, vtkAlgorithmOutput* input)
   vtkInformation* oldInfo = producer->GetOutputInformation(producerPort);
 
   // Remove this consumer from the old input's list of consumers.
-  oldInfo->Remove(vtkExecutive::CONSUMERS(), consumer, consumerPort);
+  vtkExecutive::CONSUMERS()->Remove(oldInfo, consumer, consumerPort);
 
   // Remove the information object from the list of inputs.
   inputs->Remove(oldInfo);
@@ -978,13 +1023,13 @@ void vtkAlgorithm::SetNthInputConnection(int port, int index,
   // Add the consumer to the new input's list of consumers.
   if(newInfo)
     {
-    newInfo->Append(vtkExecutive::CONSUMERS(), consumer, consumerPort);
+    vtkExecutive::CONSUMERS()->Append(newInfo, consumer, consumerPort);
     }
 
   // Remove the consumer from the old input's list of consumers.
   if(oldInfo)
     {
-    oldInfo->Remove(vtkExecutive::CONSUMERS(), consumer, consumerPort);
+    vtkExecutive::CONSUMERS()->Remove(oldInfo, consumer, consumerPort);
     }
 
   // Store the information object in the vector of input connections.
@@ -1016,7 +1061,7 @@ void vtkAlgorithm::SetNumberOfInputConnections(int port, int n)
     // Remove each input's reference to this consumer.
     if(vtkInformation* oldInfo = inputs->GetInformationObject(i))
       {
-      oldInfo->Remove(vtkExecutive::CONSUMERS(), consumer, consumerPort);
+      vtkExecutive::CONSUMERS()->Remove(oldInfo, consumer, consumerPort);
       }
     }
 
@@ -1090,7 +1135,7 @@ vtkAlgorithmOutput* vtkAlgorithm::GetInputConnection(int port, int index)
     // it is a NULL input.
     vtkExecutive* producer;
     int producerPort;
-    info->Get(vtkExecutive::PRODUCER(),producer,producerPort);
+    vtkExecutive::PRODUCER()->Get(info,producer,producerPort);
     if(producer)
       {
       return producer->GetAlgorithm()->GetOutputPort(producerPort);
@@ -1285,3 +1330,17 @@ void vtkAlgorithm::SetProgressText(const char* ptext)
     do { *cp1++ = *cp2++; } while ( --n );
     }
 }
+
+//-------------------------------------------------------------
+double vtkAlgorithm::ComputePriority()
+{
+  vtkStreamingDemandDrivenPipeline *sddp =
+    vtkStreamingDemandDrivenPipeline::SafeDownCast
+      (this->GetExecutive());
+  if (!sddp)
+    {
+    return 1.0;
+    }
+  return sddp->ComputePriority(0);
+}
+
