@@ -41,6 +41,7 @@ class vtkInformation;
 class vtkInformationInformationVectorKey;
 class vtkInformationIntegerKey;
 class vtkInformationStringKey;
+class vtkInformationStringVectorKey;
 class vtkInformationVector;
 
 class VTK_FILTERING_EXPORT vtkAlgorithm : public vtkObject
@@ -183,7 +184,7 @@ public:
   static vtkInformationIntegerKey* INPUT_IS_OPTIONAL();
   static vtkInformationIntegerKey* INPUT_IS_REPEATABLE();
   static vtkInformationInformationVectorKey* INPUT_REQUIRED_FIELDS();
-  static vtkInformationStringKey* INPUT_REQUIRED_DATA_TYPE();
+  static vtkInformationStringVectorKey* INPUT_REQUIRED_DATA_TYPE();
   static vtkInformationInformationVectorKey* INPUT_ARRAYS_TO_PROCESS();
   static vtkInformationIntegerKey* INPUT_PORT();
   static vtkInformationIntegerKey* INPUT_CONNECTION();
@@ -355,6 +356,12 @@ public:
   // in CreateDefaultExecutive() using NewInstance().
   static void SetDefaultExecutivePrototype(vtkExecutive* proto);
 
+  // Description:
+  // Returns the priority of the piece described by the current update 
+  // extent. The priority is a number between 0.0 and 1.0 with 0 meaning 
+  // skippable (REQUEST_DATA not needed) and 1.0 meaning important. 
+  virtual double ComputePriority();
+
 protected:
   vtkAlgorithm();
   ~vtkAlgorithm();
