@@ -20,10 +20,12 @@
 #include <vtkstd/set>
 #include <vtkstd/algorithm>
 #include <vtkstd/iterator>
+#include <math.h>
 
-vtkCxxRevisionMacro(vtkColorTransferFunction, "$Revision: 1.71.4.1 $");
+vtkCxxRevisionMacro(vtkColorTransferFunction, "$Revision: 1.76 $");
 vtkStandardNewMacro(vtkColorTransferFunction);
 
+//=============================================================================
 class vtkCTFNode
 {
 public:
@@ -89,6 +91,131 @@ public:
   vtkCTFFindNodeOutOfRange    FindNodeOutOfRange;
 };
 
+//=============================================================================
+// Convert to and from a special polar version of CIELAB (useful for creating
+// continuous diverging color maps).
+inline void vtkColorTransferFunctionLabToMsh(const double lab[3], double msh[3])
+{
+  const double &L = lab[0];
+  const double &a = lab[1];
+  const double &b = lab[2];
+  double &M = msh[0];
+  double &s = msh[1];
+  double &h = msh[2];
+
+  M = sqrt(L*L + a*a + b*b);
+  s = (M > 0.001) ? acos(L/M) : 0.0;
+  h = (s > 0.001) ? atan2(b,a) : 0.0;
+}
+
+inline void vtkColorTransferFunctionMshToLab(const double msh[3], double lab[3])
+{
+  const double &M = msh[0];
+  const double &s = msh[1];
+  const double &h = msh[2];
+  double &L = lab[0];
+  double &a = lab[1];
+  double &b = lab[2];
+
+  L = M*cos(s);
+  a = M*sin(s)*cos(h);
+  b = M*sin(s)*sin(h);
+}
+
+// Given two angular orientations, returns the smallest angle between the two.
+inline double vtkColorTransferFunctionAngleDiff(double a1, double a2)
+{
+  double adiff = a1 - a2;
+  if (adiff < 0.0) adiff = -adiff;
+  while (adiff >= 2*vtkMath::DoublePi()) adiff -= 2*vtkMath::DoublePi();
+  if (adiff > vtkMath::DoublePi()) adiff = 2*vtkMath::DoublePi() - adiff;
+  return adiff;
+}
+
+// For the case when interpolating from a saturated color to an unsaturated
+// color, find a hue for the unsaturated color that makes sense.
+inline double vtkColorTransferFunctionAdjustHue(const double msh[3],
+                                                double unsatM)
+{
+  if (msh[0] >= unsatM - 0.1)
+    {
+    // The best we can do is hold hue constant.
+    return msh[2];
+    }
+  else
+    {
+    // This equation is designed to make the perceptual change of the
+    // interpolation to be close to constant.
+    double hueSpin = (  msh[1]*sqrt(unsatM*unsatM - msh[0]*msh[0])
+                      / (msh[0]*sin(msh[1])) );
+    // Spin hue away from 0 except in purple hues.
+    if (msh[2] > -0.3*vtkMath::DoublePi())
+      {
+      return msh[2] + hueSpin;
+      }
+    else
+      {
+      return msh[2] - hueSpin;
+      }
+    }
+}
+
+// Interpolate a diverging color map.
+inline void vtkColorTransferFunctionInterpolateDiverging(double s,
+                                                         const double rgb1[3],
+                                                         const double rgb2[3],
+                                                         double result[3])
+{
+  double lab1[3], lab2[3];
+  vtkMath::RGBToLab(rgb1, lab1);
+  vtkMath::RGBToLab(rgb2, lab2);
+
+  double msh1[3], msh2[3];
+  vtkColorTransferFunctionLabToMsh(lab1, msh1);
+  vtkColorTransferFunctionLabToMsh(lab2, msh2);
+
+  // If the endpoints are distinct saturated colors, then place white in between
+  // them.
+  if (   (msh1[1] > 0.05) && (msh2[1] > 0.05)
+      && (vtkColorTransferFunctionAngleDiff(msh1[2], msh2[2]) > 0.33*vtkMath::DoublePi()) )
+    {
+    // Insert the white midpoint by setting one end to white and adjusting the
+    // scalar value.
+    if (s < 0.5)
+      {
+      msh2[0] = 95.0;  msh2[1] = 0.0;  msh2[2] = 0.0;
+      s = 2.0*s;
+      }
+    else
+      {
+      msh1[0] = 95.0;  msh1[1] = 0.0;  msh1[2] = 0.0;
+      s = 2.0*s - 1.0;
+      }
+    }
+
+  // If one color has no saturation, then its hue value is invalid.  In this
+  // case, we want to set it to something logical so that the interpolation of
+  // hue makes sense.
+  if ((msh1[1] < 0.05) && (msh2[1] > 0.05))
+    {
+    msh1[2] = vtkColorTransferFunctionAdjustHue(msh2, msh1[0]);
+    }
+  else if ((msh2[1] < 0.05) && (msh1[1] > 0.05))
+    {
+    msh2[2] = vtkColorTransferFunctionAdjustHue(msh1, msh2[0]);
+    }
+
+  double mshTmp[3];
+  mshTmp[0] = (1-s)*msh1[0] + s*msh2[0];
+  mshTmp[1] = (1-s)*msh1[1] + s*msh2[1];
+  mshTmp[2] = (1-s)*msh1[2] + s*msh2[2];
+
+  // Now convert back to RGB
+  double labTmp[3];
+  vtkColorTransferFunctionMshToLab(mshTmp, labTmp);
+  vtkMath::LabToRGB(labTmp, result);
+}
+
 //----------------------------------------------------------------------------
 // Construct a new vtkColorTransferFunction with default values
 vtkColorTransferFunction::vtkColorTransferFunction()
@@ -140,7 +267,7 @@ vtkColorTransferFunction::~vtkColorTransferFunction()
 // Return the number of points which specify this function
 int vtkColorTransferFunction::GetSize()
 {
-  return this->Internal->Nodes.size();
+  return static_cast<int>(this->Internal->Nodes.size());
 }
 
 // Since we no longer store the data in an array, we must
@@ -148,7 +275,7 @@ int vtkColorTransferFunction::GetSize()
 // could be added if performance is a problem
 double *vtkColorTransferFunction::GetDataPointer()
 {
-  int size = this->Internal->Nodes.size();
+  int size = static_cast<int>(this->Internal->Nodes.size());
   
   if ( this->Function )
     {
@@ -277,7 +404,7 @@ void vtkColorTransferFunction::SortAndUpdateRange()
                 this->Internal->Nodes.end(),
                 this->Internal->CompareNodes );
   
-  int size = this->Internal->Nodes.size();
+  int size = static_cast<int>(this->Internal->Nodes.size());
   if ( size )
     {
     this->Range[0] = this->Internal->Nodes[0]->X;
@@ -438,9 +565,12 @@ unsigned char *vtkColorTransferFunction::MapValue( double x )
   double rgb[3];
   this->GetColor( x, rgb );
   
-  this->UnsignedCharRGBAValue[0] = (unsigned char) (255.0*rgb[0] + 0.5);
-  this->UnsignedCharRGBAValue[1] = (unsigned char) (255.0*rgb[1] + 0.5);
-  this->UnsignedCharRGBAValue[2] = (unsigned char) (255.0*rgb[2] + 0.5);
+  this->UnsignedCharRGBAValue[0] =
+    static_cast<unsigned char>(255.0*rgb[0] + 0.5);
+  this->UnsignedCharRGBAValue[1] =
+    static_cast<unsigned char>(255.0*rgb[1] + 0.5);
+  this->UnsignedCharRGBAValue[2] =
+    static_cast<unsigned char>(255.0*rgb[2] + 0.5);
   this->UnsignedCharRGBAValue[3] = 255;
   return this->UnsignedCharRGBAValue;
 }
@@ -489,7 +619,7 @@ void vtkColorTransferFunction::GetTable( double xStart, double xEnd,
 {
   int i, j;
   int idx = 0;
-  int numNodes = this->Internal->Nodes.size();
+  int numNodes = static_cast<int>(this->Internal->Nodes.size());
   
   // Need to keep track of the last value so that
   // we can fill in table locations past this with
@@ -544,12 +674,15 @@ void vtkColorTransferFunction::GetTable( double xStart, double xEnd,
       {
       if(usingLogScale)
         {
-        logX = logStart + (double(i)/double(size-1))*(logEnd-logStart);
-        x = pow((double)10.0, logX);
+        logX = logStart +
+          (static_cast<double>(i)/static_cast<double>(size-1))
+          *(logEnd-logStart);
+        x = pow(static_cast<double>(10.0), logX);
         }
       else
         {
-        x = xStart + (double(i)/double(size-1))*(xEnd-xStart);
+        x = xStart + (static_cast<double>(i)/static_cast<double>(size-1))
+          *(xEnd-xStart);
         }
       }
     else
@@ -557,7 +690,7 @@ void vtkColorTransferFunction::GetTable( double xStart, double xEnd,
       if(usingLogScale)
         {
         logX = 0.5*(logStart+logEnd);
-        x = pow((double)10.0, logX);
+        x = pow(static_cast<double>(10.0), logX);
         }
       else
         {
@@ -718,7 +851,7 @@ void vtkColorTransferFunction::GetTable( double xStart, double xEnd,
           // Now convert this back to RGB
           vtkMath::HSVToRGB( hsvTmp, tptr );
           }
-        else // this->ColorSpace == VTK_CTF_LAB
+        else if (this->ColorSpace == VTK_CTF_LAB)
           {
           double lab1[3], lab2[3];
           vtkMath::RGBToLab(rgb1, lab1);
@@ -732,6 +865,14 @@ void vtkColorTransferFunction::GetTable( double xStart, double xEnd,
           // Now convert back to RGB
           vtkMath::LabToRGB(labTmp, tptr);
           }
+        else if (this->ColorSpace == VTK_CTF_DIVERGING)
+          {
+          vtkColorTransferFunctionInterpolateDiverging(s, rgb1, rgb2, tptr);
+          }
+        else
+          {
+          vtkErrorMacro("ColorSpace set to invalid value.");
+          }
         continue;
         }    
       
@@ -813,7 +954,7 @@ void vtkColorTransferFunction::GetTable( double xStart, double xEnd,
         // Now convert this back to RGB
         vtkMath::HSVToRGB( hsvTmp, tptr );
         }
-      else // this->ColorSpace == VTK_CTF_LAB
+      else if (this->ColorSpace == VTK_CTF_LAB)
         {
         double lab1[3], lab2[3];
         vtkMath::RGBToLab(rgb1, lab1);
@@ -832,6 +973,17 @@ void vtkColorTransferFunction::GetTable( double xStart, double xEnd,
         // Now convert this back to RGB
         vtkMath::LabToRGB(labTmp, tptr);
         }
+      else if (this->ColorSpace == VTK_CTF_DIVERGING)
+        {
+        // I have not implemented proper interpolation by a hermite curve for
+        // the diverging color map, but I cannot think of a good use case for
+        // that anyway.
+        vtkColorTransferFunctionInterpolateDiverging(s, rgb1, rgb2, tptr);
+        }
+      else
+        {
+        vtkErrorMacro("ColorSpace set to invalid value.");
+        }
       
       // Final error check to make sure we don't go outside [0,1]
       for ( j = 0; j < 3; j++ )
@@ -909,8 +1061,10 @@ const unsigned char *vtkColorTransferFunction::GetTable( double xStart, double x
 }
 
 //----------------------------------------------------------------------------
-void vtkColorTransferFunction::BuildFunctionFromTable( double xStart, double xEnd,
-                                                       int size, double *table)
+void vtkColorTransferFunction::BuildFunctionFromTable(double xStart,
+                                                      double xEnd,
+                                                      int size,
+                                                      double *table)
 {
   double inc = 0.0;
   double *tptr = table;
@@ -919,7 +1073,7 @@ void vtkColorTransferFunction::BuildFunctionFromTable( double xStart, double xEn
   
   if( size > 1 )
     {
-    inc = (xEnd-xStart)/(double)(size-1);
+    inc = (xEnd-xStart)/static_cast<double>(size-1);
     }
   
   int i;
@@ -944,7 +1098,7 @@ void vtkColorTransferFunction::BuildFunctionFromTable( double xStart, double xEn
 // For a specified index value, get the node parameters
 int vtkColorTransferFunction::GetNodeValue( int index, double val[6] )
 {
-  int size = this->Internal->Nodes.size();
+  int size = static_cast<int>(this->Internal->Nodes.size());
   
   if ( index < 0 || index >= size )
     {
@@ -966,8 +1120,8 @@ int vtkColorTransferFunction::GetNodeValue( int index, double val[6] )
 // For a specified index value, get the node parameters
 int vtkColorTransferFunction::SetNodeValue( int index, double val[6] )
 {
-  int size = this->Internal->Nodes.size();
-  
+  int size = static_cast<int>(this->Internal->Nodes.size());
+
   if ( index < 0 || index >= size )
     {
     vtkErrorMacro("Index out of range!");
@@ -1047,7 +1201,7 @@ void vtkColorTransferFunctionMapData(vtkColorTransferFunction* self,
   double          rgb[3];
   unsigned char  *optr = output;
   T              *iptr = input;
-  unsigned char   alpha = (unsigned char)(self->GetAlpha()*255.0);
+  unsigned char   alpha = static_cast<unsigned char>(self->GetAlpha()*255.0);
   
   if(self->GetSize() == 0)
     {
@@ -1057,18 +1211,19 @@ void vtkColorTransferFunctionMapData(vtkColorTransferFunction* self,
 
   while (--i >= 0) 
     {
-    x = (double) *iptr;
+    x = static_cast<double>(*iptr);
     self->GetColor(x, rgb);
     
     if (outFormat == VTK_RGB || outFormat == VTK_RGBA)
       {
-      *(optr++) = (unsigned char)(rgb[0]*255.0 + 0.5);
-      *(optr++) = (unsigned char)(rgb[1]*255.0 + 0.5);
-      *(optr++) = (unsigned char)(rgb[2]*255.0 + 0.5);
+      *(optr++) = static_cast<unsigned char>(rgb[0]*255.0 + 0.5);
+      *(optr++) = static_cast<unsigned char>(rgb[1]*255.0 + 0.5);
+      *(optr++) = static_cast<unsigned char>(rgb[2]*255.0 + 0.5);
       }
     else // LUMINANCE  use coeffs of (0.30  0.59  0.11)*255.0
       {
-      *(optr++) = (unsigned char)(rgb[0]*76.5 + rgb[1]*150.45 + rgb[2]*28.05 + 0.5); 
+      *(optr++) = static_cast<unsigned char>(rgb[0]*76.5 + rgb[1]*150.45 +
+                                             rgb[2]*28.05 + 0.5); 
       }
     
     if (outFormat == VTK_RGBA || outFormat == VTK_LUMINANCE_ALPHA)
@@ -1228,7 +1383,7 @@ void vtkColorTransferFunctionMagMapData(vtkColorTransferFunction* self,
     sum = 0;
     for (j = 0; j < inIncr; ++j)
       {
-      tmp = (double)(*input);  
+      tmp = static_cast<double>(*input);  
       sum += (tmp * tmp);
       ++input;
       }
