@@ -26,6 +26,8 @@ PURPOSE.  See the above copyright notice for more information.
 #include "vtkInformationDataObjectKey.h"
 #include "vtkInformationDoubleKey.h"
 #include "vtkInformationDoubleVectorKey.h"
+#include "vtkInformationExecutivePortKey.h"
+#include "vtkInformationExecutivePortVectorKey.h"
 #include "vtkInformationIntegerKey.h"
 #include "vtkInformationIntegerPointerKey.h"
 #include "vtkInformationIntegerVectorKey.h"
@@ -34,7 +36,7 @@ PURPOSE.  See the above copyright notice for more information.
 #include "vtkInformationVector.h"
 #include "vtkDataSetAttributes.h"
 
-vtkCxxRevisionMacro(vtkDataObject, "$Revision: 1.36 $");
+vtkCxxRevisionMacro(vtkDataObject, "$Revision: 1.42 $");
 vtkStandardNewMacro(vtkDataObject);
 
 vtkCxxSetObjectMacro(vtkDataObject,Information,vtkInformation);
@@ -49,6 +51,8 @@ vtkInformationKeyMacro(vtkDataObject, DATA_NUMBER_OF_GHOST_LEVELS, Integer);
 vtkInformationKeyMacro(vtkDataObject, DATA_TIME_STEPS, DoubleVector);
 vtkInformationKeyMacro(vtkDataObject, POINT_DATA_VECTOR, InformationVector);
 vtkInformationKeyMacro(vtkDataObject, CELL_DATA_VECTOR, InformationVector);
+vtkInformationKeyMacro(vtkDataObject, VERTEX_DATA_VECTOR, InformationVector);
+vtkInformationKeyMacro(vtkDataObject, EDGE_DATA_VECTOR, InformationVector);
 vtkInformationKeyMacro(vtkDataObject, FIELD_ARRAY_TYPE, Integer);
 vtkInformationKeyMacro(vtkDataObject, FIELD_ASSOCIATION, Integer);
 vtkInformationKeyMacro(vtkDataObject, FIELD_ATTRIBUTE_TYPE, Integer);
@@ -62,6 +66,7 @@ vtkInformationKeyRestrictedMacro(vtkDataObject, DATA_EXTENT, IntegerPointer, 6);
 vtkInformationKeyRestrictedMacro(vtkDataObject, ORIGIN, DoubleVector, 3);
 vtkInformationKeyRestrictedMacro(vtkDataObject, SPACING, DoubleVector, 3);
 vtkInformationKeyMacro(vtkDataObject, DATA_GEOMETRY_UNMODIFIED, Integer);
+vtkInformationKeyMacro(vtkDataObject, SIL, DataObject);
 
 class vtkDataObjectToSourceFriendship
 {
@@ -101,7 +106,10 @@ const char vtkDataObject
   "vtkDataObject::FIELD_ASSOCIATION_POINTS",
   "vtkDataObject::FIELD_ASSOCIATION_CELLS",
   "vtkDataObject::FIELD_ASSOCIATION_NONE",
-  "vtkDataObject::FIELD_ASSOCIATION_POINTS_THEN_CELLS"
+  "vtkDataObject::FIELD_ASSOCIATION_POINTS_THEN_CELLS",
+  "vtkDataObject::FIELD_ASSOCIATION_VERTICES",
+  "vtkDataObject::FIELD_ASSOCIATION_EDGES",
+  "vtkDataObject::FIELD_ASSOCIATION_ROWS"
 };
 
 //----------------------------------------------------------------------------
@@ -281,8 +289,8 @@ void vtkDataObject::SetPipelineInformation(vtkInformation* newInfo)
 
       // If the new producer is a vtkSource then setup the backward
       // compatibility link.
-      vtkExecutive* newExec = newInfo->GetExecutive(vtkExecutive::PRODUCER());
-      int newPort = newInfo->GetPort(vtkExecutive::PRODUCER());
+      vtkExecutive* newExec = vtkExecutive::PRODUCER()->GetExecutive(newInfo);
+      int newPort = vtkExecutive::PRODUCER()->GetPort(newInfo);
       if(newExec)
         {
         vtkSource* newSource = vtkSource::SafeDownCast(newExec->GetAlgorithm());
@@ -301,8 +309,8 @@ void vtkDataObject::SetPipelineInformation(vtkInformation* newInfo)
       {
       // If the old producer was a vtkSource then remove the backward
       // compatibility link.
-      vtkExecutive* oldExec = oldInfo->GetExecutive(vtkExecutive::PRODUCER());
-      int oldPort = oldInfo->GetPort(vtkExecutive::PRODUCER());
+      vtkExecutive* oldExec = vtkExecutive::PRODUCER()->GetExecutive(oldInfo);
+      int oldPort = vtkExecutive::PRODUCER()->GetPort(oldInfo);
       if(oldExec)
         {
         vtkSource* oldSource = vtkSource::SafeDownCast(oldExec->GetAlgorithm());
@@ -406,6 +414,16 @@ void vtkDataObject::CopyInformationToPipeline(vtkInformation *request,
         {
         output->CopyEntry(input, CELL_DATA_VECTOR(), 1);
         }
+      // copy vertex data.
+      if (input->Has(VERTEX_DATA_VECTOR()))
+        {
+        output->CopyEntry(input, VERTEX_DATA_VECTOR(), 1);
+        }
+      // copy edge data.
+      if (input && input->Has(EDGE_DATA_VECTOR()))
+        {
+        output->CopyEntry(input, EDGE_DATA_VECTOR(), 1);
+        }
       // copy the actual time
       if (input->Has(DATA_TIME_STEPS()))
         {
@@ -437,6 +455,14 @@ vtkInformation *vtkDataObject::GetActiveFieldInformation(vtkInformation *info,
     {
     fieldDataInfoVector = info->Get(CELL_DATA_VECTOR());
     }
+  else if (fieldAssociation == FIELD_ASSOCIATION_VERTICES)
+    {
+    fieldDataInfoVector = info->Get(VERTEX_DATA_VECTOR());
+    }
+  else if (fieldAssociation == FIELD_ASSOCIATION_EDGES)
+    {
+    fieldDataInfoVector = info->Get(EDGE_DATA_VECTOR());
+    }
   else
     {
     vtkGenericWarningMacro("Unrecognized field association!");
@@ -477,6 +503,14 @@ vtkInformation *vtkDataObject::GetNamedFieldInformation(vtkInformation *info,
     {
     fieldDataInfoVector = info->Get(CELL_DATA_VECTOR());
     }
+  else if (fieldAssociation == FIELD_ASSOCIATION_VERTICES)
+    {
+    fieldDataInfoVector = info->Get(VERTEX_DATA_VECTOR());
+    }
+  else if (fieldAssociation == FIELD_ASSOCIATION_EDGES)
+    {
+    fieldDataInfoVector = info->Get(EDGE_DATA_VECTOR());
+    }
   else
     {
     vtkGenericWarningMacro("Unrecognized field association!");
@@ -517,6 +551,14 @@ void vtkDataObject::RemoveNamedFieldInformation(vtkInformation *info,
     {
     fieldDataInfoVector = info->Get(CELL_DATA_VECTOR());
     }
+  else if (fieldAssociation == FIELD_ASSOCIATION_VERTICES)
+    {
+    fieldDataInfoVector = info->Get(VERTEX_DATA_VECTOR());
+    }
+  else if (fieldAssociation == FIELD_ASSOCIATION_EDGES)
+    {
+    fieldDataInfoVector = info->Get(EDGE_DATA_VECTOR());
+    }
   else
     {
     vtkGenericWarningMacro("Unrecognized field association!");
@@ -559,6 +601,14 @@ vtkInformation *vtkDataObject::SetActiveAttribute(vtkInformation *info,
     {
     fieldDataInfoVector = info->Get(CELL_DATA_VECTOR());
     }
+  else if (fieldAssociation == FIELD_ASSOCIATION_VERTICES)
+    {
+    fieldDataInfoVector = info->Get(VERTEX_DATA_VECTOR());
+    }
+  else if (fieldAssociation == FIELD_ASSOCIATION_EDGES)
+    {
+    fieldDataInfoVector = info->Get(EDGE_DATA_VECTOR());
+    }
   else
     {
     vtkGenericWarningMacro("Unrecognized field association!");
@@ -571,10 +621,18 @@ vtkInformation *vtkDataObject::SetActiveAttribute(vtkInformation *info,
       {
       info->Set(POINT_DATA_VECTOR(), fieldDataInfoVector);
       }
-    else // (fieldAssociation == FIELD_ASSOCIATION_CELLS)
+    else if (fieldAssociation == FIELD_ASSOCIATION_CELLS)
       {
       info->Set(CELL_DATA_VECTOR(), fieldDataInfoVector);
       }
+    else if (fieldAssociation == FIELD_ASSOCIATION_VERTICES)
+      {
+      info->Set(VERTEX_DATA_VECTOR(), fieldDataInfoVector);
+      }
+    else // if (fieldAssociation == FIELD_ASSOCIATION_EDGES)
+      {
+      info->Set(EDGE_DATA_VECTOR(), fieldDataInfoVector);
+      }
     fieldDataInfoVector->Delete();
     }
 
@@ -684,26 +742,6 @@ void vtkDataObject::DataHasBeenGenerated()
 {
   this->DataReleased = 0;
   this->UpdateTime.Modified();
-
-  // Assume that the algorithm produced the required data unless the
-  // algorithm sets otherwise.
-  // NOTE: This is a temporary fix. We should check if the algorithm
-  // produced what was requested and produce an error if it didn't.
-  // However, when such a check is added, all algorithms that do this:
-  // internalAlg->Update();
-  // myOutput->ShallowCopy(internalAlg->GetOutput());
-  // will break when running in parallel because ShallowCopy() will 
-  // copy that piece related keys which will be 0 of 1.
-  if (true || !this->Information->Has(DATA_PIECE_NUMBER()) ||
-      this->Information->Get(DATA_PIECE_NUMBER()) == - 1)
-    {
-    this->Information->Set(DATA_PIECE_NUMBER(), 
-                           this->GetUpdatePiece());
-    this->Information->Set(DATA_NUMBER_OF_PIECES(), 
-                           this->GetUpdateNumberOfPieces());
-    this->Information->Set(DATA_NUMBER_OF_GHOST_LEVELS(), 
-                           this->GetUpdateGhostLevel());
-    }
 }
 
 //----------------------------------------------------------------------------
@@ -781,7 +819,7 @@ vtkExecutive* vtkDataObject::GetExecutive()
 {
   if(this->PipelineInformation)
     {
-    return this->PipelineInformation->GetExecutive(vtkExecutive::PRODUCER());
+    return vtkExecutive::PRODUCER()->GetExecutive(this->PipelineInformation);
     }
   return 0;
 }
@@ -791,7 +829,7 @@ int vtkDataObject::GetPortNumber()
 {
   if(this->PipelineInformation)
     {
-    return this->PipelineInformation->GetPort(vtkExecutive::PRODUCER());
+    return vtkExecutive::PRODUCER()->GetPort(this->PipelineInformation);
     }
   return 0;
 }
@@ -907,6 +945,13 @@ void vtkDataObject::InternalDataObjectCopy(vtkDataObject *src)
     }
 
   this->DataReleased = src->DataReleased;
+  
+  // Do not copy pipeline specific information from data object to
+  // data object. This meta-data is specific to the algorithm and the
+  // what was requested of it when it executed. What looks like a single
+  // piece to an internal algorithm may be a piece to an external
+  // algorithm.
+  /*
   if(src->Information->Has(DATA_PIECE_NUMBER()))
     {
     this->Information->Set(DATA_PIECE_NUMBER(),
@@ -922,10 +967,12 @@ void vtkDataObject::InternalDataObjectCopy(vtkDataObject *src)
     this->Information->Set(DATA_NUMBER_OF_GHOST_LEVELS(),
                            src->Information->Get(DATA_NUMBER_OF_GHOST_LEVELS()));
     }
+  */
   if(src->Information->Has(DATA_TIME_STEPS()))
     {
     this->Information->CopyEntry(src->Information, DATA_TIME_STEPS(), 1);
     }
+  
   vtkInformation* thatPInfo = src->GetPipelineInformation();
   vtkInformation* thisPInfo = this->GetPipelineInformation();
   if(thisPInfo && thatPInfo)
