@@ -424,6 +424,9 @@ public:
     FIELD_ASSOCIATION_CELLS,
     FIELD_ASSOCIATION_NONE,
     FIELD_ASSOCIATION_POINTS_THEN_CELLS,
+    FIELD_ASSOCIATION_VERTICES,
+    FIELD_ASSOCIATION_EDGES,
+    FIELD_ASSOCIATION_ROWS,
     NUMBER_OF_ASSOCIATIONS
   };
   //ETX
@@ -455,6 +458,8 @@ public:
   static vtkInformationDoubleVectorKey* DATA_TIME_STEPS();
   static vtkInformationInformationVectorKey* POINT_DATA_VECTOR();
   static vtkInformationInformationVectorKey* CELL_DATA_VECTOR();
+  static vtkInformationInformationVectorKey* VERTEX_DATA_VECTOR();
+  static vtkInformationInformationVectorKey* EDGE_DATA_VECTOR();
   static vtkInformationIntegerKey* FIELD_ARRAY_TYPE();
   static vtkInformationIntegerKey* FIELD_ASSOCIATION();
   static vtkInformationIntegerKey* FIELD_ATTRIBUTE_TYPE();
@@ -468,6 +473,9 @@ public:
   static vtkInformationDoubleVectorKey* SPACING();
   static vtkInformationIntegerKey* DATA_GEOMETRY_UNMODIFIED();
 
+  // Key used to put SIL information in the output information by readers.
+  static vtkInformationDataObjectKey* SIL();
+
   //BTX
   // Description:
   // Retrieve an instance of this class from an information object.
