@@ -30,7 +30,7 @@
 #include "vtkIdTypeArray.h"
 #include "vtkObjectFactory.h"
 
-vtkCxxRevisionMacro(vtkDataSetAttributes, "$Revision: 1.24 $");
+vtkCxxRevisionMacro(vtkDataSetAttributes, "$Revision: 1.27 $");
 vtkStandardNewMacro(vtkDataSetAttributes);
 
 //--------------------------------------------------------------------------
@@ -143,21 +143,16 @@ void vtkDataSetAttributes::DeepCopy(vtkFieldData *fd)
       newData = data->NewInstance(); //instantiate same type of object
       newData->DeepCopy(data);
       newData->SetName(data->GetName());
-      if ((attributeType=dsa->IsArrayAnAttribute(i)) != -1 )
-        {
-        // If this array is an attribute in the source, make it so
-        // in the target as well.
-        this->SetAttribute(newData, attributeType);
-        }
-      else
-        {
-        this->AddArray(newData);
-        }
+      this->AddArray(newData);
       newData->Delete();
       }
     // Copy the copy flags
     for(attributeType=0; attributeType<NUM_ATTRIBUTES; attributeType++)
       {
+      // If an array is an attribute in the source, then mark it as a attribute
+      // in the clone as well.
+      this->AttributeIndices[attributeType] = dsa->AttributeIndices[attributeType];
+
       this->CopyAttributeFlags[COPYTUPLE][attributeType] = 
         dsa->CopyAttributeFlags[COPYTUPLE][attributeType];
       this->CopyAttributeFlags[INTERPOLATE][attributeType] = 
@@ -195,16 +190,15 @@ void vtkDataSetAttributes::ShallowCopy(vtkFieldData *fd)
       {
       this->NumberOfActiveArrays++;
       this->SetArray(i, fd->GetAbstractArray(i));
-      if ((attributeType=dsa->IsArrayAnAttribute(i)) != -1)
-        {
-        // If this array is an attribute in the source, make it so
-        // in the target as well.
-        this->SetActiveAttribute(i, attributeType);
-        }
       }
+
     // Copy the copy flags
     for(attributeType=0; attributeType<NUM_ATTRIBUTES; attributeType++)
       {
+      // If an array is an attribute in the source, then mark it as a attribute
+      // in the clone as well.
+      this->AttributeIndices[attributeType] = dsa->AttributeIndices[attributeType];
+
       this->CopyAttributeFlags[COPYTUPLE][attributeType] = 
         dsa->CopyAttributeFlags[COPYTUPLE][attributeType];
       this->CopyAttributeFlags[INTERPOLATE][attributeType] = 
@@ -445,14 +439,15 @@ void vtkDataSetAttributesCopyValues(
   unsigned char *outZPtr;
   
   // Get the starting input pointer.
-  inZPtr = (unsigned char*)(srcIter->GetArray()->GetVoidPointer(0));
+  inZPtr = static_cast<unsigned char*>(srcIter->GetArray()->GetVoidPointer(0));
   // Shift to the start of the subextent.
   inZPtr += (outExt[0]-outExt[0])*inIncs[0] * data_type_size +
     (outExt[2] - outExt[2])*inIncs[1] * data_type_size +
     (outExt[4] - outExt[4])*inIncs[2] * data_type_size;
 
   // Get output pointer.
-  outZPtr = (unsigned char*)(destIter->GetArray()->GetVoidPointer(0));
+  outZPtr =
+    static_cast<unsigned char*>(destIter->GetArray()->GetVoidPointer(0));
 
   // Loop over z axis.
   int zIdx, yIdx;
@@ -1453,8 +1448,26 @@ void vtkDataSetAttributes::RemoveArray(const char *name)
 }
 
 //--------------------------------------------------------------------------
-void vtkDataSetAttributes::CopyAllocate(vtkDataSetAttributes::FieldList& list, 
-                                        vtkIdType sze, vtkIdType ext)
+void vtkDataSetAttributes::CopyAllocate(
+  vtkDataSetAttributes::FieldList& list,
+  vtkIdType sze, vtkIdType ext)
+{
+  this->InternalCopyAllocate(list, COPYTUPLE, sze, ext);
+}
+
+//--------------------------------------------------------------------------
+void vtkDataSetAttributes::InterpolateAllocate(
+  vtkDataSetAttributes::FieldList& list, vtkIdType sze,
+  vtkIdType ext)
+{
+  this->InternalCopyAllocate(list, INTERPOLATE, sze, ext);
+}
+
+//--------------------------------------------------------------------------
+void vtkDataSetAttributes::InternalCopyAllocate(
+  vtkDataSetAttributes::FieldList& list,
+  int ctype,
+  vtkIdType sze, vtkIdType ext)
 {
   vtkAbstractArray* newAA=0;
   vtkDataArray* newDA=0;
@@ -1486,7 +1499,7 @@ void vtkDataSetAttributes::CopyAllocate(vtkDataSetAttributes::FieldList& list,
       if ( i < NUM_ATTRIBUTES )
         {
         // since attributes can only be DataArray, newDA must be non-null.
-        if ( this->CopyAttributeFlags[COPYTUPLE][i] && newDA)
+        if ( this->CopyAttributeFlags[ctype][i] && newDA)
           {
           list.FieldIndices[i] = this->AddArray(newDA);
           this->SetActiveAttribute(list.FieldIndices[i], i);
@@ -1538,6 +1551,30 @@ void vtkDataSetAttributes::CopyData(vtkDataSetAttributes::FieldList& list,
     }
 }
 
+//--------------------------------------------------------------------------
+// Interpolate data from points and interpolation weights. Make sure that the 
+// method InterpolateAllocate() has been invoked before using this method.
+void vtkDataSetAttributes::InterpolatePoint(
+  vtkDataSetAttributes::FieldList& list,
+  vtkDataSetAttributes *fromPd, 
+  int idx,
+  vtkIdType toId, vtkIdList *ptIds, 
+  double *weights)
+{
+  vtkAbstractArray *fromArray;
+  vtkAbstractArray *toArray;
+
+  for (int i=0; i < list.NumberOfFields; i++)
+    {
+    if ( list.FieldIndices[i] >= 0 )
+      {
+      toArray = this->GetAbstractArray(list.FieldIndices[i]);
+      fromArray = fromPd->GetAbstractArray(list.DSAIndices[idx][i]);
+      toArray->InterpolateTuple(toId, ptIds, fromArray, weights);
+      }
+    }
+}
+
 // FieldList support ---------------------------------------------------------
 // To perform intersection of attribute data, use IntializeFieldList() to grab
 // an initial vtkDataSetAttributes. Then use IntersectFieldList() to add (and 
