@@ -23,11 +23,9 @@
 #include "vtkDataObjectTypes.h"
 #include "vtkDataSet.h"
 #include "vtkGarbageCollector.h"
-#include "vtkMultiBlockDataSet.h"
-#include "vtkMultiGroupDataSet.h"
-#include "vtkHierarchicalBoxDataSet.h"
-#include "vtkHierarchicalDataSet.h"
 #include "vtkInformation.h"
+#include "vtkInformationExecutivePortKey.h"
+#include "vtkInformationExecutivePortVectorKey.h"
 #include "vtkInformationIntegerKey.h"
 #include "vtkInformationKeyVectorKey.h"
 #include "vtkInformationRequestKey.h"
@@ -39,7 +37,7 @@
 
 #include <vtkstd/vector>
 
-vtkCxxRevisionMacro(vtkDemandDrivenPipeline, "$Revision: 1.53 $");
+vtkCxxRevisionMacro(vtkDemandDrivenPipeline, "$Revision: 1.58 $");
 vtkStandardNewMacro(vtkDemandDrivenPipeline);
 
 vtkInformationKeyMacro(vtkDemandDrivenPipeline, DATA_NOT_GENERATED, Integer);
@@ -135,7 +133,7 @@ vtkDemandDrivenPipeline::ComputePipelineMTime(vtkInformation* request,
         // call ComputePipelineMTime on the input
         vtkExecutive* e;
         int producerPort;
-        info->Get(vtkExecutive::PRODUCER(),e,producerPort);
+        vtkExecutive::PRODUCER()->Get(info,e,producerPort);
         if(e)
           {
           unsigned long pmtime;
@@ -552,10 +550,7 @@ void vtkDemandDrivenPipeline::ExecuteDataStart(vtkInformation* request,
         {
         vtkInformation* outInfo = outputs->GetInformationObject(i);
         vtkDataObject* output = outInfo->Get(vtkDataObject::DATA_OBJECT());
-        // We want to pass the field data unless it is a table.
-        // Since a table's data is stored in the field data, we want
-        // table algorithms to start with an empty field data.
-        if(output && !output->IsA("vtkTable"))
+        if(output)
           {
           output->GetFieldData()->PassData(input->GetFieldData());
           }
@@ -641,7 +636,8 @@ int vtkDemandDrivenPipeline::CheckDataObject(int port,
   if(const char* dt = portInfo->Get(vtkDataObject::DATA_TYPE_NAME()))
     {
     int incorrectdata = data && (!data->IsA(dt) ||
-      (!strcmp(data->GetClassName(),"vtkTemporalDataSet") && strcmp(dt,"vtkTemporalDataSet")) );
+      (!strcmp(data->GetClassName(),"vtkTemporalDataSet") &&
+       strcmp(dt,"vtkTemporalDataSet")) );
     // The output port specifies a data type.  Make sure the data
     // object exists and is of the right type.
     if(!data || incorrectdata)
@@ -787,7 +783,8 @@ int vtkDemandDrivenPipeline::InputTypeIsValid
   vtkDataObject* input = this->GetInputData(port, index, inInfoVec);
 
   // Enforce required type, if any.
-  if(const char* dt = info->Get(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE()))
+  if(info->Has(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE()) 
+     && info->Length(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE()) > 0)
     {
     // The input cannot be NULL unless the port is optional.
     if(!input && !info->Get(vtkAlgorithm::INPUT_IS_OPTIONAL()))
@@ -795,19 +792,33 @@ int vtkDemandDrivenPipeline::InputTypeIsValid
       vtkErrorMacro("Input for connection index " << index
                     << " on input port index " << port
                     << " for algorithm " << this->Algorithm->GetClassName()
-                    << "(" << this->Algorithm << ") is NULL, but a " << dt
+                    << "(" << this->Algorithm << ") is NULL, but a " 
+                    << info->Get(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), 0)
                     << " is required.");
       return 0;
       }
 
-    // The input must be of required type or NULL.
-    if(input && !input->IsA(dt))
+    // The input must be one of the required types or NULL.
+    bool foundMatch = false;
+    if(input)
+      {
+      int size = info->Length(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE());
+      for(int i = 0; i < size; ++i)
+        {
+        if(input->IsA(info->Get(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), i)))
+          {
+          foundMatch = true;
+          }
+        }
+      }
+    if(input && !foundMatch)
       {
       vtkErrorMacro("Input for connection index " << index
                     << " on input port index " << port
                     << " for algorithm " << this->Algorithm->GetClassName()
                     << "(" << this->Algorithm << ") is of type "
-                    << input->GetClassName() << ", but a " << dt
+                    << input->GetClassName() << ", but a " 
+                    << info->Get(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), 0)
                     << " is required.");
       return 0;
       }
