@@ -32,7 +32,7 @@
 #include "vtkObjectFactory.h"
 #include "vtkSmartPointer.h"
 
-vtkCxxRevisionMacro(vtkStreamingDemandDrivenPipeline, "$Revision: 1.53.4.1 $");
+vtkCxxRevisionMacro(vtkStreamingDemandDrivenPipeline, "$Revision: 1.58 $");
 vtkStandardNewMacro(vtkStreamingDemandDrivenPipeline);
 
 vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, CONTINUE_EXECUTING, Integer);
@@ -51,6 +51,7 @@ vtkInformationKeyRestrictedMacro(vtkStreamingDemandDrivenPipeline,
                                  EXTENT_TRANSLATOR, ObjectBase,
                                  "vtkExtentTranslator");
 vtkInformationKeyRestrictedMacro(vtkStreamingDemandDrivenPipeline, WHOLE_BOUNDING_BOX, DoubleVector, 6);
+vtkInformationKeyRestrictedMacro(vtkStreamingDemandDrivenPipeline, PIECE_BOUNDING_BOX, DoubleVector, 6);
 vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, TIME_STEPS, DoubleVector);
 vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, UPDATE_TIME_STEPS, DoubleVector);
 vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, PREVIOUS_UPDATE_TIME_STEPS, DoubleVector);
@@ -137,6 +138,9 @@ int vtkStreamingDemandDrivenPipeline
         return 0;
         }
 
+      // Remove update-related keys from the input information.
+      this->ResetUpdateInformation(request, inInfoVec, outInfoVec);
+
       // Invoke the request on the algorithm.
       this->LastPropogateUpdateExtentShortCircuited = 0;
       result = this->CallAlgorithm(request, vtkExecutive::RequestUpstream,
@@ -321,6 +325,8 @@ vtkStreamingDemandDrivenPipeline
           outInfo->CopyEntry(inInfo, EXTENT_TRANSLATOR());
           outInfo->CopyEntry(inInfo, TIME_STEPS());
           outInfo->CopyEntry(inInfo, TIME_RANGE());
+          outInfo->CopyEntry(inInfo, PIECE_BOUNDING_BOX());
+          outInfo->CopyEntry(inInfo, vtkDataObject::FIELD_RANGE());
           }
         }
       }
@@ -777,6 +783,36 @@ vtkStreamingDemandDrivenPipeline
   // Tell outputs they have been generated.
   this->Superclass::MarkOutputsGenerated(request,inInfoVec,outInfoVec);
 
+  int outputPort = 0;
+  if(request->Has(FROM_OUTPUT_PORT()))
+    {
+    outputPort = request->Get(FROM_OUTPUT_PORT());
+    outputPort = (outputPort >= 0 ? outputPort : 0);
+    }
+
+  // Get the piece request from the update port (port 0 if none)
+  // The defaults are:
+  int piece = 0;
+  int numPieces = 1;
+  int ghostLevel = 0;
+  vtkInformation* fromInfo = 0;
+  if (outputPort < outInfoVec->GetNumberOfInformationObjects())
+    {
+    fromInfo = outInfoVec->GetInformationObject(outputPort);
+    if (fromInfo->Has(UPDATE_PIECE_NUMBER()))
+      {
+      piece = fromInfo->Get(UPDATE_PIECE_NUMBER());
+      }
+    if (fromInfo->Has(UPDATE_NUMBER_OF_PIECES()))
+      {
+      numPieces = fromInfo->Get(UPDATE_NUMBER_OF_PIECES());
+      }
+    if (fromInfo->Has(UPDATE_NUMBER_OF_GHOST_LEVELS()))
+      {
+      ghostLevel = fromInfo->Get(UPDATE_NUMBER_OF_GHOST_LEVELS());
+      }
+    }
+
   for(int i=0; i < outInfoVec->GetNumberOfInformationObjects(); ++i)
     {
     vtkInformation* outInfo = outInfoVec->GetInformationObject(i);
@@ -795,6 +831,18 @@ vtkStreamingDemandDrivenPipeline
           }
         }
       
+      // Copy the update piece information from the update port to
+      // the data piece information of all output ports UNLESS the
+      // algorithm already specified it.
+      vtkInformation* dataInfo = data->GetInformation();
+      if (!dataInfo->Has(vtkDataObject::DATA_PIECE_NUMBER()) ||
+          dataInfo->Get(vtkDataObject::DATA_PIECE_NUMBER()) == - 1)
+        {
+        dataInfo->Set(vtkDataObject::DATA_PIECE_NUMBER(), piece);
+        dataInfo->Set(vtkDataObject::DATA_NUMBER_OF_PIECES(), numPieces);
+        dataInfo->Set(vtkDataObject::DATA_NUMBER_OF_GHOST_LEVELS(), ghostLevel);
+        }
+        
       // In this block, we make sure that DATA_TIME_STEPS() is set if:
       // * There was someone upstream that supports time (TIME_RANGE() key
       //   is present)
@@ -807,7 +855,6 @@ vtkStreamingDemandDrivenPipeline
       // be copied from input to output.
       //
       // Check if the output has DATA_TIME_STEPS().
-      vtkInformation* dataInfo = data->GetInformation();
       if (!dataInfo->Has(vtkDataObject::DATA_TIME_STEPS()) &&
           outInfo->Has(TIME_RANGE()))
         {
@@ -836,11 +883,11 @@ vtkStreamingDemandDrivenPipeline
         }
 
       // We are keeping track of the previous time request.
-      if (outInfo->Has(UPDATE_TIME_STEPS()))
+      if (fromInfo->Has(UPDATE_TIME_STEPS()))
         {
         outInfo->Set(PREVIOUS_UPDATE_TIME_STEPS(),
-                     outInfo->Get(UPDATE_TIME_STEPS()),
-                     outInfo->Length(UPDATE_TIME_STEPS()));
+                     fromInfo->Get(UPDATE_TIME_STEPS()),
+                     fromInfo->Length(UPDATE_TIME_STEPS()));
         }
       else
         {
@@ -1670,3 +1717,76 @@ double* vtkStreamingDemandDrivenPipeline::GetWholeBoundingBox(int port)
     }
   return info->Get(WHOLE_BOUNDING_BOX());
 }
+
+//----------------------------------------------------------------------------
+double vtkStreamingDemandDrivenPipeline::ComputePriority(int port)
+{
+  vtkInformation* rqst;
+
+  vtkInformationVector **inVec = this->GetInputInformation();
+  vtkInformationVector *outVec = this->GetOutputInformation();
+
+  //make sure global information is up to date
+  this->UpdateInformation();
+
+  //tell pipeline what piece to ask about
+  rqst = vtkInformation::New();
+  rqst->Set(REQUEST_UPDATE_EXTENT());
+  rqst->Set(vtkExecutive::FORWARD_DIRECTION(),
+            vtkExecutive::RequestUpstream);
+  rqst->Set(vtkExecutive::ALGORITHM_BEFORE_FORWARD(), 1);
+  rqst->Set(vtkExecutive::FROM_OUTPUT_PORT(), port);
+  this->ProcessRequest(rqst, inVec, outVec);
+  rqst->Delete();
+
+  //ask upstream filters to estimate priority for the piece
+  rqst = vtkInformation::New();
+  rqst->Set(REQUEST_UPDATE_EXTENT_INFORMATION());
+  rqst->Set(vtkExecutive::FORWARD_DIRECTION(),
+            vtkExecutive::RequestUpstream);
+  rqst->Set(vtkExecutive::ALGORITHM_AFTER_FORWARD(), 1);
+  rqst->Set(vtkExecutive::FROM_OUTPUT_PORT(), port);
+  //be sure pipeline propagates
+  //scalar ranges
+  rqst->Append(vtkExecutive::KEYS_TO_COPY(), vtkDataObject::FIELD_RANGE());
+  //geometric extent
+  rqst->Append(vtkExecutive::KEYS_TO_COPY(), PIECE_BOUNDING_BOX());
+  //result
+  rqst->Append(vtkExecutive::KEYS_TO_COPY(), PRIORITY());
+  this->ProcessRequest(rqst, inVec, outVec);
+  rqst->Delete();
+
+  //obtain the priority returned
+  double priority = 1.0;
+  vtkInformation *info = outVec->GetInformationObject(port);
+  if (info && info->Has(PRIORITY()))
+    {
+    priority = info->Get(PRIORITY());
+    }
+
+  return priority;
+}
+
+//----------------------------------------------------------------------------
+void vtkStreamingDemandDrivenPipeline::ResetUpdateInformation(
+  vtkInformation* vtkNotUsed(request),
+  vtkInformationVector** inInfoVec,
+  vtkInformationVector* vtkNotUsed(outInfoVec))
+{
+  int num_ports = this->GetNumberOfInputPorts();
+
+  for (int cc=0; cc < num_ports; cc++)
+    {
+    int num_conns = inInfoVec[cc]->GetNumberOfInformationObjects();
+    for (int kk=0; kk < num_conns; kk++)
+      {
+      vtkInformation* inInfo = inInfoVec[cc]->GetInformationObject(kk);
+      if (inInfo)
+        {
+        inInfo->Remove(vtkStreamingDemandDrivenPipeline::FAST_PATH_OBJECT_ID());
+        inInfo->Remove(vtkStreamingDemandDrivenPipeline::FAST_PATH_OBJECT_TYPE());
+        inInfo->Remove(vtkStreamingDemandDrivenPipeline::FAST_PATH_ID_TYPE());
+        }
+      }
+    }
+}
