@@ -19,10 +19,17 @@
 #include "vtkConvertSelection.h"
 
 #include "vtkCellData.h"
+#include "vtkCommand.h"
+#include "vtkCompositeDataSet.h"
 #include "vtkDataArrayTemplate.h"
 #include "vtkDataSet.h"
+#include "vtkDoubleArray.h"
+#include "vtkExtractSelectedThresholds.h"
 #include "vtkExtractSelection.h"
 #include "vtkFieldData.h"
+#include "vtkGraph.h"
+#include "vtkHierarchicalBoxDataIterator.h"
+#include "vtkHierarchicalBoxDataSet.h"
 #include "vtkIdList.h"
 #include "vtkIdTypeArray.h"
 #include "vtkInformation.h"
@@ -33,18 +40,21 @@
 #include "vtkSmartPointer.h"
 #include "vtkStringArray.h"
 #include "vtkTable.h"
+#include "vtkUnsignedIntArray.h"
 #include "vtkVariantArray.h"
 
-#include <vtksys/stl/algorithm>
-#include <vtksys/stl/iterator>
-#include <vtksys/stl/set>
+#include <vtkstd/algorithm>
+#include <vtkstd/iterator>
+#include <vtkstd/map>
+#include <vtkstd/set>
+#include <vtkstd/vector>
 
 #define VTK_CREATE(type,name) \
   vtkSmartPointer<type> name = vtkSmartPointer<type>::New()
 
 vtkCxxSetObjectMacro(vtkConvertSelection, ArrayNames, vtkStringArray);
 
-vtkCxxRevisionMacro(vtkConvertSelection, "$Revision: 1.5 $");
+vtkCxxRevisionMacro(vtkConvertSelection, "$Revision: 1.21 $");
 vtkStandardNewMacro(vtkConvertSelection);
 //----------------------------------------------------------------------------
 vtkConvertSelection::vtkConvertSelection()
@@ -87,7 +97,7 @@ int vtkConvertSelection::SelectTableFromTable(
   vtkTable* dataTable, 
   vtkIdTypeArray* indices)
 {
-  vtksys_stl::set<vtkIdType> matching;
+  vtkstd::set<vtkIdType> matching;
   VTK_CREATE(vtkIdList, list);
   for (vtkIdType row = 0; row < selTable->GetNumberOfRows(); row++)
     {
@@ -109,23 +119,26 @@ int vtkConvertSelection::SelectTableFromTable(
           }
         else
           {
-          vtksys_stl::set<vtkIdType> curMatch, intersection;
-          curMatch.insert(
-            list->GetPointer(0), 
-            list->GetPointer(0) + list->GetNumberOfIds());
-          vtksys_stl::set_intersection(
+          vtkstd::set<vtkIdType> intersection;
+          vtkstd::sort(list->GetPointer(0), list->GetPointer(0) + list->GetNumberOfIds());
+          vtkstd::set_intersection(
             matching.begin(), matching.end(), 
-            curMatch.begin(), curMatch.end(), 
-            vtksys_stl::inserter(intersection, intersection.begin()));
+            list->GetPointer(0), list->GetPointer(0) + list->GetNumberOfIds(), 
+            vtkstd::inserter(intersection, intersection.begin()));
           matching = intersection;
           }
         }
       }
-    vtksys_stl::set<vtkIdType>::iterator it, itEnd = matching.end();
+    vtkstd::set<vtkIdType>::iterator it, itEnd = matching.end();
     for (it = matching.begin(); it != itEnd; ++it)
       {
       indices->InsertNextValue(*it);
       }
+    if (row % 100 == 0)
+      {
+      double progress = 0.8 * row / selTable->GetNumberOfRows();
+      this->InvokeEvent(vtkCommand::ProgressEvent, &progress);
+      }
     }
   return 1;
 }
@@ -136,18 +149,13 @@ int vtkConvertSelection::ConvertToIndexSelection(
   vtkDataSet* data,
   vtkSelection* output)
 {
-  // Change the input to preserve topology
-  vtkSelection* selTemp = vtkSelection::New();
-  selTemp->ShallowCopy(input);
-  selTemp->GetProperties()->Set(vtkSelection::PRESERVE_TOPOLOGY(), true);
-  
   // Use the extraction filter to create an insidedness array.
   vtkExtractSelection* const extract = vtkExtractSelection::New();
+  extract->PreserveTopologyOn();
   extract->SetInput(0, data);
-  extract->SetInput(1, selTemp);
+  extract->SetInput(1, input);
   extract->Update();
-  vtkDataSet* const extracted = extract->GetOutput();
-  selTemp->Delete();
+  vtkDataSet* const extracted = vtkDataSet::SafeDownCast(extract->GetOutput());
   
   output->GetProperties()->Set(vtkSelection::CONTENT_TYPE(), vtkSelection::INDICES);
   int type = input->GetProperties()->Get(vtkSelection::FIELD_TYPE());
@@ -172,7 +180,7 @@ int vtkConvertSelection::ConvertToIndexSelection(
   
   if (!insidedness)
     {
-    vtkErrorMacro("Did not find expected vtkInsidedness array.");
+    // Empty selection
     extract->Delete();
     return 0;
     }
@@ -193,23 +201,224 @@ int vtkConvertSelection::ConvertToIndexSelection(
 }
 
 //----------------------------------------------------------------------------
-template <class T>
-void vtkConvertSelectionLookup(
-  T* selArr, 
-  T* dataArr, 
-  vtkIdTypeArray* indices)
+int vtkConvertSelection::ConvertToBlockSelection(
+  vtkSelection* input, vtkCompositeDataSet* data, vtkSelection* output)
 {
-  vtkIdType numTuples = selArr->GetNumberOfTuples();
-  VTK_CREATE(vtkIdList, list);
-  for (vtkIdType i = 0; i < numTuples; i++)
+  output->SetContentType(vtkSelection::BLOCKS);
+
+  vtkstd::vector<vtkSelection*> inputSelections;
+
+  // If input selection is a composite selection consisting of other selections,
+  // then iterate over each of the constituent selection instances.
+  if (input->GetContentType() == vtkSelection::SELECTIONS)
     {
-    dataArr->LookupValue(selArr->GetValue(i), list);
-    vtkIdType numIds = list->GetNumberOfIds();
-    for (vtkIdType j = 0; j < numIds; j++)
+    for (unsigned int i = 0; i < input->GetNumberOfChildren(); ++i)
       {
-      indices->InsertNextValue(list->GetId(j));
+      vtkSelection* inputChild = input->GetChild(i);
+      if (inputChild && inputChild->GetContentType() != vtkSelection::SELECTIONS)
+        {
+        inputSelections.push_back(inputChild);
+        }
       }
     }
+  else
+    {
+    inputSelections.push_back(input);
+    }
+
+  vtkstd::set<unsigned int> indices;
+
+  vtkstd::vector<vtkSelection*>::iterator iter;
+  for (iter = inputSelections.begin(); iter != inputSelections.end(); ++iter)
+    {
+    vtkSmartPointer<vtkSelection> curSel = (*iter);
+    if (curSel->GetContentType() == vtkSelection::GLOBALIDS)
+      {
+      // global id selection does not have COMPOSITE_INDEX() key, so we convert
+      // it to an index base selection, so that we can determine the composite
+      // indices.
+      vtkSelection* newSel = vtkConvertSelection::ToIndexSelection(curSel, data);
+      curSel.TakeReference(newSel);
+      }
+
+    vtkInformation* properties = curSel->GetProperties();
+    if (properties->Has(vtkSelection::CONTENT_TYPE()) &&
+      properties->Has(vtkSelection::COMPOSITE_INDEX()))
+      {
+      indices.insert(static_cast<unsigned int>(
+          properties->Get(vtkSelection::COMPOSITE_INDEX())));
+      }
+    else if (properties->Has(vtkSelection::CONTENT_TYPE()) &&
+      properties->Has(vtkSelection::HIERARCHICAL_INDEX()) &&
+      properties->Has(vtkSelection::HIERARCHICAL_LEVEL()) && 
+      data->IsA("vtkHierarchicalBoxDataSet"))
+      {
+       // convert hierarchical index to composite index.
+       vtkHierarchicalBoxDataSet* hbox = static_cast<vtkHierarchicalBoxDataSet*>(data);
+       indices.insert(
+         hbox->GetFlatIndex(
+           static_cast<unsigned int>(properties->Get(vtkSelection::HIERARCHICAL_LEVEL())),
+           static_cast<unsigned int>(properties->Get(vtkSelection::HIERARCHICAL_INDEX()))));
+
+      }
+    }
+ 
+  VTK_CREATE(vtkUnsignedIntArray, selectionList);
+  selectionList->SetNumberOfTuples(indices.size());
+  vtkstd::set<unsigned int>::iterator siter;
+  vtkIdType index=0;
+  for (siter = indices.begin(); siter != indices.end(); ++siter, ++index)
+    {
+    selectionList->SetValue(index, *siter);
+    }
+  output->SetSelectionList(selectionList);
+  return 1;
+}
+
+//----------------------------------------------------------------------------
+int vtkConvertSelection::ConvertCompositeDataSet(
+  vtkSelection* input,
+  vtkCompositeDataSet* data,
+  vtkSelection* output)
+{
+  // If this->OutputType == vtkSelection::BLOCKS we just want to create a new
+  // selection with the chosen block indices.
+  if (this->OutputType == vtkSelection::BLOCKS)
+    {
+    return this->ConvertToBlockSelection(input, data, output);
+    }
+
+
+  // If input selection is a composite selection consisting of other selections,
+  // then iterate over each of the constituent selection instances.
+  if (input->GetContentType() == vtkSelection::SELECTIONS)
+    {
+    output->SetContentType(vtkSelection::SELECTIONS);
+    for (unsigned int i = 0; i < input->GetNumberOfChildren(); ++i)
+      {
+      vtkSelection* inputChild = input->GetChild(i);
+      VTK_CREATE(vtkSelection, outputChild);
+      if (!this->ConvertCompositeDataSet(inputChild, data, outputChild))
+        {
+        return 0;
+        }
+
+      if (outputChild->GetContentType() == vtkSelection::SELECTIONS)
+        {
+        unsigned int numChildren = outputChild->GetNumberOfChildren();
+        for (unsigned int cc=0; cc < numChildren; cc++)
+          {
+          output->AddChild(outputChild->GetChild(cc));
+          }
+        }
+      else if (outputChild->GetContentType() != -1)
+        {
+        output->AddChild(outputChild);
+        }
+      }
+    return 1;
+    }
+
+  // *  If input has no composite keys then it implies that it applies to all
+  //    nodes in the data. If input has composite keys, output will have
+  //    composite keys unless outputContentType == GLOBALIDS. 
+  //    If input does not have composite keys, then composite
+  //    keys are only added for outputContentType == INDICES, FRUSTUM and
+  //    PEDIGREEIDS.
+  bool has_composite_key =
+    input->GetProperties()->Has(vtkSelection::COMPOSITE_INDEX()) != 0;
+
+  unsigned int composite_index = has_composite_key?
+    static_cast<unsigned int>(
+      input->GetProperties()->Get(vtkSelection::COMPOSITE_INDEX())) : 0;
+
+  bool has_hieararchical_key = 
+    input->GetProperties()->Has(vtkSelection::HIERARCHICAL_INDEX()) != 0 &&
+    input->GetProperties()->Has(vtkSelection::HIERARCHICAL_LEVEL()) != 0;
+
+  unsigned int hierarchical_level = has_hieararchical_key?
+    static_cast<unsigned int>(
+      input->GetProperties()->Get(vtkSelection::HIERARCHICAL_LEVEL())) : 0;
+  unsigned int hierarchical_index = has_hieararchical_key?
+    static_cast<unsigned int>(
+      input->GetProperties()->Get(vtkSelection::HIERARCHICAL_INDEX())) : 0;
+  
+  vtkSmartPointer<vtkCompositeDataIterator> iter;
+  iter.TakeReference(data->NewIterator());
+
+  vtkHierarchicalBoxDataIterator* hbIter =
+    vtkHierarchicalBoxDataIterator::SafeDownCast(iter);
+
+  vtkstd::vector<vtkSmartPointer<vtkSelection> > selections;
+  for (iter->InitTraversal(); !iter->IsDoneWithTraversal(); iter->GoToNextItem())
+    {
+    if (has_hieararchical_key && hbIter &&
+      (hbIter->GetCurrentLevel() != hierarchical_level ||
+       hbIter->GetCurrentIndex() != hierarchical_index))
+      {
+      continue;
+      }
+
+    if (has_composite_key &&
+      iter->GetCurrentFlatIndex() != composite_index)
+      {
+      continue;
+      }
+
+    VTK_CREATE(vtkSelection, outputChild);
+    if (!this->Convert(input, iter->GetCurrentDataObject(), outputChild))
+      {
+      return 0;
+      }
+
+    if ((has_hieararchical_key || has_composite_key || 
+      this->OutputType == vtkSelection::INDICES ||
+      this->OutputType == vtkSelection::PEDIGREEIDS || 
+      this->OutputType == vtkSelection::FRUSTUM) &&
+      this->OutputType != vtkSelection::GLOBALIDS) 
+      {
+      outputChild->GetProperties()->Set(vtkSelection::COMPOSITE_INDEX(),
+        iter->GetCurrentFlatIndex());
+
+      if (has_hieararchical_key && hbIter)
+        {
+        outputChild->GetProperties()->Set(vtkSelection::HIERARCHICAL_LEVEL(),
+          hierarchical_level);
+        outputChild->GetProperties()->Set(vtkSelection::HIERARCHICAL_INDEX(),
+          hierarchical_index);
+        }
+      selections.push_back(outputChild);
+      }
+    else 
+      {
+      // just merge the selection lists from all composite nodes.
+      if (output->GetContentType() == -1)
+        {
+        output->ShallowCopy(outputChild);
+        }
+      else
+        {
+        vtkAbstractArray* inList = outputChild->GetSelectionList();
+        vtkAbstractArray* outList = output->GetSelectionList();
+        vtkIdType numTuples = inList->GetNumberOfTuples();
+        for (vtkIdType cc=0; cc  < numTuples; cc++)
+          {
+          outList->InsertNextTuple(cc, inList);
+          }
+        }
+      }
+    }
+
+  if (selections.size() > 0)
+    {
+    output->SetContentType(vtkSelection::SELECTIONS);
+    for (unsigned int cc=0; cc < selections.size(); cc++)
+      {
+      output->AddChild(selections[cc]);
+      }
+    }
+
+  return 1;
 }
 
 //----------------------------------------------------------------------------
@@ -328,21 +537,64 @@ int vtkConvertSelection::Convert(
       {
       dsa = vtkDataSet::SafeDownCast(data)->GetPointData();
       }
-    else // vtkSelection::NONE
-      {
-      fd = data->GetFieldData();
-      }
-    }
-  else
-    {
-    if (!input->GetProperties()->Has(vtkSelection::FIELD_TYPE()) ||
-        input->GetFieldType() == vtkSelection::NONE)
+    else if (input->GetFieldType() == vtkSelection::FIELD)
       {
       fd = data->GetFieldData();
       }
     else
       {
-      vtkErrorMacro("Can only select points or cells from a subclass of vtkDataSet");
+      vtkErrorMacro("Inappropriate selection type for a vtkDataSet");
+      return 0;
+      }
+    }
+  else if (vtkGraph::SafeDownCast(data))
+    {
+    if (!input->GetProperties()->Has(vtkSelection::FIELD_TYPE()) ||
+        input->GetFieldType() == vtkSelection::EDGE)
+      {
+      dsa = vtkGraph::SafeDownCast(data)->GetEdgeData();
+      }
+    else if (input->GetFieldType() == vtkSelection::VERTEX)
+      {
+      dsa = vtkGraph::SafeDownCast(data)->GetVertexData();
+      }
+    else if (input->GetFieldType() == vtkSelection::FIELD)
+      {
+      fd = data->GetFieldData();
+      }
+    else
+      {
+      vtkErrorMacro("Inappropriate selection type for a vtkGraph");
+      return 0;
+      }
+    }
+  else if (vtkTable::SafeDownCast(data))
+    {
+    if (!input->GetProperties()->Has(vtkSelection::FIELD_TYPE()) ||
+        input->GetFieldType() == vtkSelection::ROW)
+      {
+      dsa = vtkTable::SafeDownCast(data)->GetRowData();
+      }
+    else if (input->GetFieldType() == vtkSelection::FIELD)
+      {
+      fd = data->GetFieldData();
+      }
+    else
+      {
+      vtkErrorMacro("Inappropriate selection type for a vtkTable");
+      return 0;
+      }
+    }
+  else
+    {
+    if (!input->GetProperties()->Has(vtkSelection::FIELD_TYPE()) ||
+        input->GetFieldType() == vtkSelection::FIELD)
+      {
+      fd = data->GetFieldData();
+      }
+    else
+      {
+      vtkErrorMacro("Inappropriate selection type for a non-dataset, non-graph");
       return 0;
       }
     }
@@ -353,14 +605,12 @@ int vtkConvertSelection::Convert(
   
   VTK_CREATE(vtkIdTypeArray, indices);
   
-  // TODO: Only use vtkExtractSelection filter for frustum and locations selections
   if (input->GetContentType() == vtkSelection::FRUSTUM || 
-      input->GetContentType() == vtkSelection::THRESHOLDS ||
       input->GetContentType() == vtkSelection::LOCATIONS)
     {
     if (!vtkDataSet::SafeDownCast(data))
       {
-      vtkErrorMacro("Can only convert from frustum, thresholds, or locations if the input is a vtkDataSet");
+      vtkErrorMacro("Can only convert from frustum or locations if the input is a vtkDataSet");
       return 0;
       }
     // Use the extract selection filter to create an index selection
@@ -369,6 +619,42 @@ int vtkConvertSelection::Convert(
     // TODO: We should shallow copy this, but the method is not defined.
     indices->DeepCopy(indexSelection->GetSelectionList());
     }
+  else if (input->GetContentType() == vtkSelection::THRESHOLDS)
+    {
+    vtkDoubleArray *lims = vtkDoubleArray::SafeDownCast(input->GetSelectionList());
+    if (!lims)
+      {
+      vtkErrorMacro("Thresholds selection requires vtkDoubleArray selection list.");
+      return 0;
+      }
+    vtkDataArray *dataArr = 0;
+    if (dsa)
+      {
+      dataArr = vtkDataArray::SafeDownCast(dsa->GetAbstractArray(lims->GetName()));
+      }
+    else if (fd)
+      {
+      dataArr = vtkDataArray::SafeDownCast(fd->GetAbstractArray(lims->GetName()));
+      }
+    if (!dataArr)
+      {
+      vtkErrorMacro("Could not find vtkDataArray for thresholds selection.");
+      return 0;
+      }
+    int inverse = 0;
+    if (input->GetProperties()->Has(vtkSelection::INVERSE()))
+      {
+      inverse = input->GetProperties()->Get(vtkSelection::INVERSE());
+      }
+    for (vtkIdType id = 0; id < dataArr->GetNumberOfTuples(); id++)
+      {
+      int keepPoint = vtkExtractSelectedThresholds::EvaluateValue(dataArr, id, lims);
+      if (keepPoint ^ inverse)
+        {
+        indices->InsertNextValue(id);
+        }
+      }
+    }
   else if (input->GetContentType() == vtkSelection::INDICES)
     {
     // TODO: We should shallow copy this, but the method is not defined.
@@ -378,16 +664,16 @@ int vtkConvertSelection::Convert(
     {
     vtkFieldData* selData = input->GetSelectionData();
     VTK_CREATE(vtkTable, selTable);
-    selTable->SetFieldData(selData);
+    selTable->GetRowData()->ShallowCopy(selData);
     VTK_CREATE(vtkTable, dataTable);
     for (vtkIdType col = 0; col < selTable->GetNumberOfColumns(); col++)
       {
       vtkAbstractArray* dataArr = 0;
-      if (dsa && input->GetContentType() == vtkSelection::VALUES)
+      if (dsa)
         {
         dataArr = dsa->GetAbstractArray(selTable->GetColumn(col)->GetName());
         }
-      else if (fd && input->GetContentType() == vtkSelection::VALUES)
+      else if (fd)
         {
         dataArr = fd->GetAbstractArray(selTable->GetColumn(col)->GetName());
         }
@@ -414,10 +700,16 @@ int vtkConvertSelection::Convert(
       {
       dataArr = dsa->GetGlobalIds();
       }
+    else if (fd && selArr->GetName())
+      {
+      // Since data objects only have field data which does not have attributes,
+      // use the array name to try to match the incoming selection's array.
+      dataArr = fd->GetAbstractArray(selArr->GetName());
+      }
     else
       {
-      // TODO: Make this error go away.
-      vtkErrorMacro("BUG: Currently you can only specify pedigree and global ids on a vtkDataSet.");
+      vtkErrorMacro("Tried to use array name to match global or pedigree ids on data object,"
+        << "but name not set on selection array.");
       return 0;
       }
     
@@ -427,35 +719,49 @@ int vtkConvertSelection::Convert(
       vtkErrorMacro("Selection array does not exist in input dataset.");
       return 0;
       }
-    if (dataArr->GetDataType() != selArr->GetDataType())
+
+    // Handle the special case where we have a domain array.
+    vtkStringArray* domainArr = dsa ? vtkStringArray::SafeDownCast(dsa->GetAbstractArray("domain")) : 0;
+    if (input->GetContentType() == vtkSelection::PEDIGREEIDS &&
+        domainArr && selArr->GetName())
       {
-      vtkErrorMacro("Selection array type does not match input dataset array type.");
-      return 0;
+      // Perform the lookup, keeping only those items in the correct domain.
+      vtkStdString domain = selArr->GetName();
+      vtkIdType numTuples = selArr->GetNumberOfTuples();
+      VTK_CREATE(vtkIdList, list);
+      for (vtkIdType i = 0; i < numTuples; i++)
+        {
+        dataArr->LookupValue(selArr->GetVariantValue(i), list);
+        vtkIdType numIds = list->GetNumberOfIds();
+        for (vtkIdType j = 0; j < numIds; j++)
+          {
+          if (domainArr->GetValue(list->GetId(j)) == domain)
+            {
+            indices->InsertNextValue(list->GetId(j));
+            }
+          }
+        }
       }
-    
-    // Perform the lookup
-    switch (selArr->GetDataType())
+    else
       {
-      vtkTemplateMacro(vtkConvertSelectionLookup(
-        static_cast<vtkDataArrayTemplate<VTK_TT>*>(selArr), 
-        static_cast<vtkDataArrayTemplate<VTK_TT>*>(dataArr),
-        indices));
-      case VTK_STRING:
-        vtkConvertSelectionLookup(
-          vtkStringArray::SafeDownCast(selArr), 
-          vtkStringArray::SafeDownCast(dataArr), indices);
-        break;
-      case VTK_VARIANT:
-        vtkConvertSelectionLookup(
-          vtkVariantArray::SafeDownCast(selArr), 
-          vtkVariantArray::SafeDownCast(dataArr), indices);
-        break;
-      default:
-        vtkErrorMacro("Unsupported data type " << selArr->GetDataType());
-        return 0;
+      // Perform the lookup
+      vtkIdType numTuples = selArr->GetNumberOfTuples();
+      VTK_CREATE(vtkIdList, list);
+      for (vtkIdType i = 0; i < numTuples; i++)
+        {
+        dataArr->LookupValue(selArr->GetVariantValue(i), list);
+        vtkIdType numIds = list->GetNumberOfIds();
+        for (vtkIdType j = 0; j < numIds; j++)
+          {
+          indices->InsertNextValue(list->GetId(j));
+          }
+        }
       }
     }
   
+  double progress = 0.8;
+  this->InvokeEvent(vtkCommand::ProgressEvent, &progress);
+  
   //
   // Now that we have the list of indices, convert the selection by indexing
   // values in another array.
@@ -473,6 +779,60 @@ int vtkConvertSelection::Convert(
     {
     numOutputArrays = this->ArrayNames->GetNumberOfValues();
     }
+
+  // Handle the special case where we have a pedigree id selection with a domain array.
+  vtkStringArray* outputDomainArr = vtkStringArray::SafeDownCast(
+    dsa->GetAbstractArray("domain"));
+  if (this->OutputType == vtkSelection::PEDIGREEIDS && outputDomainArr)
+    {
+    vtkAbstractArray* outputDataArr = dsa->GetPedigreeIds();
+    // Check array existence.
+    if (!outputDataArr)
+      {
+      vtkErrorMacro("Output selection array does not exist in input dataset.");
+      return 0;
+      }
+
+    output->SetContentType(vtkSelection::SELECTIONS);
+    vtkstd::map<vtkStdString, vtkSmartPointer<vtkAbstractArray> > domainArrays;
+    vtkIdType numTuples = outputDataArr->GetNumberOfTuples();
+    vtkIdType numIndices = indices->GetNumberOfTuples();
+    for (vtkIdType i = 0; i < numIndices; ++i)
+      {
+      vtkIdType index = indices->GetValue(i);
+      if (index >= numTuples)
+        {
+        continue;
+        }
+      vtkStdString domain = outputDomainArr->GetValue(index);
+      if (domainArrays.count(domain) == 0)
+        {
+        domainArrays[domain].TakeReference(
+          vtkAbstractArray::CreateArray(outputDataArr->GetDataType()));
+        domainArrays[domain]->SetName(domain);
+        }
+      vtkAbstractArray* domainArr = domainArrays[domain];
+      domainArr->InsertNextTuple(index, outputDataArr);
+      if (i % 1000 == 0)
+        {
+        progress = 0.8 + (0.2 * i / numIndices);
+        this->InvokeEvent(vtkCommand::ProgressEvent, &progress);
+        }
+      }
+    output->SetContentType(vtkSelection::SELECTIONS);
+    vtkstd::map<vtkStdString, vtkSmartPointer<vtkAbstractArray> >::iterator it, itEnd;
+    it = domainArrays.begin();
+    itEnd = domainArrays.end();
+    for (; it != itEnd; ++it)
+      {
+      vtkSmartPointer<vtkSelection> child = vtkSmartPointer<vtkSelection>::New();
+      child->SetContentType(vtkSelection::PEDIGREEIDS);
+      child->SetFieldType(input->GetFieldType());
+      child->SetSelectionList(it->second);
+      output->AddChild(child);
+      }
+    return 1;
+    }
   
   VTK_CREATE(vtkFieldData, outputData);
   for (vtkIdType ind = 0; ind < numOutputArrays; ind++)
@@ -508,13 +868,10 @@ int vtkConvertSelection::Convert(
       vtkErrorMacro("Output selection array does not exist in input dataset.");
       return 0;
       }
-    
+
     // Put the array's values into the selection.
     vtkAbstractArray* outputArr = vtkAbstractArray::CreateArray(outputDataArr->GetDataType());
-    if (this->OutputType == vtkSelection::VALUES)
-      {
-      outputArr->SetName(outputDataArr->GetName());
-      }
+    outputArr->SetName(outputDataArr->GetName());
     vtkIdType numTuples = outputDataArr->GetNumberOfTuples();
     vtkIdType numIndices = indices->GetNumberOfTuples();
     for (vtkIdType i = 0; i < numIndices; ++i)
@@ -529,6 +886,11 @@ int vtkConvertSelection::Convert(
         // TODO: Make this a silent warning when we are done debugging
         vtkWarningMacro("Attempting to select an index outside the array range.");
         }
+      if (i % 1000 == 0)
+        {
+        progress = 0.8 + (0.2 * (ind*numIndices + i)) / (numOutputArrays*numIndices);
+        this->InvokeEvent(vtkCommand::ProgressEvent, &progress);
+        }
       }
     outputData->AddArray(outputArr);
     outputArr->Delete();
@@ -554,10 +916,21 @@ int vtkConvertSelection::RequestData(
   vtkInformation* outInfo = outputVector->GetInformationObject(0);
   vtkSelection* output = vtkSelection::SafeDownCast(
     outInfo->Get(vtkDataObject::DATA_OBJECT()));
+  if (data && data->IsA("vtkCompositeDataSet"))
+    {
+    return this->ConvertCompositeDataSet(input,
+      static_cast<vtkCompositeDataSet*>(data), output);
+    }
   
   return this->Convert(input, data, output);
 }
 
+//----------------------------------------------------------------------------
+void vtkConvertSelection::SetDataObjectConnection(vtkAlgorithmOutput* in)
+{
+  this->SetInputConnection(1, in);
+}
+
 //----------------------------------------------------------------------------
 int vtkConvertSelection::FillInputPortInformation(
   int port, vtkInformation* info)
@@ -569,7 +942,12 @@ int vtkConvertSelection::FillInputPortInformation(
     }
   else if (port == 1)
     {
-    info->Set(vtkConvertSelection::INPUT_REQUIRED_DATA_TYPE(), "vtkDataObject");
+    // Can convert from a vtkDataSet, vtkGraph, or vtkTable
+    info->Remove(vtkConvertSelection::INPUT_REQUIRED_DATA_TYPE());
+    info->Append(vtkConvertSelection::INPUT_REQUIRED_DATA_TYPE(), "vtkCompositeDataSet");
+    info->Append(vtkConvertSelection::INPUT_REQUIRED_DATA_TYPE(), "vtkDataSet");
+    info->Append(vtkConvertSelection::INPUT_REQUIRED_DATA_TYPE(), "vtkGraph");
+    info->Append(vtkConvertSelection::INPUT_REQUIRED_DATA_TYPE(), "vtkTable");
     }
   return 1;
 }
