@@ -28,6 +28,7 @@
 #include "vtkInformationVector.h"
 #include "vtkMergePoints.h"
 #include "vtkObjectFactory.h"
+#include "vtkPlane.h"
 #include "vtkPointData.h"
 #include "vtkPolyData.h"
 #include "vtkRectilinearGrid.h"
@@ -37,10 +38,11 @@
 #include "vtkSynchronizedTemplates3D.h"
 #include "vtkSynchronizedTemplatesCutter3D.h"
 #include "vtkUnstructuredGrid.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkCutter, "$Revision: 1.87 $");
+vtkCxxRevisionMacro(vtkCutter, "$Revision: 1.89 $");
 vtkStandardNewMacro(vtkCutter);
 vtkCxxSetObjectMacro(vtkCutter,CutFunction,vtkImplicitFunction);
 vtkCxxSetObjectMacro(vtkCutter,Locator,vtkPointLocator)
@@ -360,7 +362,7 @@ int vtkCutter::RequestData(
     }
   if (input->GetDataObjectType() == VTK_RECTILINEAR_GRID)
     {
-    int dim = ((vtkRectilinearGrid*)input)->GetDataDimension();
+    int dim = static_cast<vtkRectilinearGrid *>(input)->GetDataDimension();
     if ( dim == 3 )
       {
       this->RectilinearGridCutter(input, output);
@@ -438,7 +440,8 @@ void vtkCutter::DataSetCutter(vtkDataSet *input, vtkPolyData *output)
 
   // Create objects to hold output of contour operation
   //
-  estimatedSize = (vtkIdType) pow ((double) numCells, .75) * numContours;
+  estimatedSize = static_cast<vtkIdType>(
+    pow(static_cast<double>(numCells), .75)) * numContours;
   estimatedSize = estimatedSize / 1024 * 1024; //multiple of 1024
   if (estimatedSize < 1024)
     {
@@ -511,7 +514,7 @@ void vtkCutter::DataSetCutter(vtkDataSet *input, vtkPolyData *output)
         if ( !(++cut % progressInterval) )
           {
           vtkDebugMacro(<<"Cutting #" << cut);
-          this->UpdateProgress ((double)cut/numCuts);
+          this->UpdateProgress (static_cast<double>(cut)/numCuts);
           abortExecute = this->GetAbortExecute();
           }
 
@@ -592,7 +595,7 @@ void vtkCutter::DataSetCutter(vtkDataSet *input, vtkPolyData *output)
           if (dimensionality == 3 && !(++cut % progressInterval) )
             {
             vtkDebugMacro(<<"Cutting #" << cut);
-            this->UpdateProgress ((double)cut/numCuts);
+            this->UpdateProgress (static_cast<double>(cut)/numCuts);
             abortExecute = this->GetAbortExecute();
             }
           value = this->ContourValues->GetValue(iter);
@@ -666,7 +669,8 @@ void vtkCutter::UnstructuredGridCutter(vtkDataSet *input, vtkPolyData *output)
 
   // Create objects to hold output of contour operation
   //
-  estimatedSize = (vtkIdType) pow ((double) numCells, .75) * numContours;
+  estimatedSize = static_cast<vtkIdType>(
+    pow(static_cast<double>(numCells),.75)) * numContours;
   estimatedSize = estimatedSize / 1024 * 1024; //multiple of 1024
   if (estimatedSize < 1024)
     {
@@ -720,7 +724,7 @@ void vtkCutter::UnstructuredGridCutter(vtkDataSet *input, vtkPolyData *output)
   vtkIdType progressInterval = numCuts/20 + 1;
   int cut=0;
 
-  vtkUnstructuredGrid *grid = (vtkUnstructuredGrid *)input;
+  vtkUnstructuredGrid *grid = static_cast<vtkUnstructuredGrid *>(input);
   vtkIdType *cellArrayPtr = grid->GetCells()->GetPointer();
   double *scalarArrayPtr = cutScalars->GetPointer(0);
   double tempScalar;
@@ -743,7 +747,7 @@ void vtkCutter::UnstructuredGridCutter(vtkDataSet *input, vtkPolyData *output)
         if ( !(++cut % progressInterval) )
           {
           vtkDebugMacro(<<"Cutting #" << cut);
-          this->UpdateProgress ((double)cut/numCuts);
+          this->UpdateProgress (static_cast<double>(cut)/numCuts);
           abortExecute = this->GetAbortExecute();
           }
 
@@ -787,7 +791,7 @@ void vtkCutter::UnstructuredGridCutter(vtkDataSet *input, vtkPolyData *output)
             if ( !(++cut % progressInterval) )
               {
               vtkDebugMacro(<<"Cutting #" << cut);
-              this->UpdateProgress ((double)cut/numCuts);
+              this->UpdateProgress (static_cast<double>(cut)/numCuts);
               abortExecute = this->GetAbortExecute();
               }
             value = this->ContourValues->GetValue(iter);
@@ -887,7 +891,7 @@ void vtkCutter::UnstructuredGridCutter(vtkDataSet *input, vtkPolyData *output)
             if (dimensionality == 3 && !(++cut % progressInterval) )
               {
               vtkDebugMacro(<<"Cutting #" << cut);
-              this->UpdateProgress ((double)cut/numCuts);
+              this->UpdateProgress (static_cast<double>(cut)/numCuts);
               abortExecute = this->GetAbortExecute();
               }
             value = this->ContourValues->GetValue(iter);
@@ -991,3 +995,130 @@ void vtkCutter::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "Generate Cut Scalars: "
      << (this->GenerateCutScalars ? "On\n" : "Off\n");
 }
+
+//-----------------------------------------------------------------------
+
+int vtkCutter::ProcessRequest(vtkInformation* request,
+                              vtkInformationVector** inputVector,
+                              vtkInformationVector* outputVector)
+{
+  if(request->Has(vtkStreamingDemandDrivenPipeline::
+     REQUEST_UPDATE_EXTENT_INFORMATION()))
+    {
+    // compute the priority for this UpdateExtent
+    vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+    if (!inInfo)
+      {
+      return 1;
+      }
+
+    double inPrior = 1;
+    if (inInfo->Has(vtkStreamingDemandDrivenPipeline::PRIORITY()))
+      {
+      inPrior = inInfo->Get(vtkStreamingDemandDrivenPipeline::
+                            PRIORITY());
+      }
+
+    // Get bounds and evaluate implicit function. If all bounds
+    // evaluate to a value smaller than input value, this piece
+    // has priority set to 0.
+
+    static double bounds[] = {-1.0,1.0, -1.0,1.0, -1.0,1.0};
+    double prior = 1;
+
+    // determine geometric bounds of this piece
+    double *wBBox =
+      inInfo->
+        Get(vtkStreamingDemandDrivenPipeline::PIECE_BOUNDING_BOX());
+
+    if (!wBBox)
+      {
+      wBBox =
+        inInfo->
+        Get(vtkStreamingDemandDrivenPipeline::WHOLE_BOUNDING_BOX());      
+      }
+    if (wBBox)
+      {
+      bounds[0] = wBBox[0];
+      bounds[1] = wBBox[1];
+      bounds[2] = wBBox[2];
+      bounds[3] = wBBox[3];
+      bounds[4] = wBBox[4];
+      bounds[5] = wBBox[5];
+      }
+    else
+      {
+      //try to figure out geometric bounds
+      double *origin = inInfo->Get(vtkDataObject::ORIGIN());
+      double *spacing = inInfo->Get(vtkDataObject::SPACING());
+      int *subExtent = inInfo->Get(
+        vtkStreamingDemandDrivenPipeline::UPDATE_EXTENT());
+      if (origin && spacing && subExtent)
+        {
+        bounds[0] = origin[0]+subExtent[0]*spacing[0];
+        bounds[1] = origin[0]+subExtent[1]*spacing[0];
+        bounds[2] = origin[1]+subExtent[2]*spacing[1];
+        bounds[3] = origin[1]+subExtent[3]*spacing[1];
+        bounds[4] = origin[2]+subExtent[4]*spacing[2];
+        bounds[5] = origin[2]+subExtent[5]*spacing[2];
+        }
+      else
+        {
+        //cerr << "Need geometric bounds meta information to evaluate priority" << endl;
+        outputVector->GetInformationObject(0)->
+          Set(vtkStreamingDemandDrivenPipeline::PRIORITY(),inPrior);
+        return 1;
+        }
+      }
+
+    vtkPlane* fPtr = vtkPlane::SafeDownCast(this->GetCutFunction());
+    if (!fPtr)
+      {
+      //cerr << "Can not evaluate priority for that clip type" << endl;
+      outputVector->GetInformationObject(0)->
+        Set(vtkStreamingDemandDrivenPipeline::PRIORITY(),inPrior);
+      return 1;
+      }
+    
+    static double fVal[8];
+    fVal[0] = fPtr->EvaluateFunction(bounds[0],bounds[2],bounds[4]);
+    fVal[1] = fPtr->EvaluateFunction(bounds[0],bounds[2],bounds[5]);
+    fVal[2] = fPtr->EvaluateFunction(bounds[0],bounds[3],bounds[4]);
+    fVal[3] = fPtr->EvaluateFunction(bounds[0],bounds[3],bounds[5]);
+    fVal[4] = fPtr->EvaluateFunction(bounds[1],bounds[2],bounds[4]);
+    fVal[5] = fPtr->EvaluateFunction(bounds[1],bounds[2],bounds[5]);
+    fVal[6] = fPtr->EvaluateFunction(bounds[1],bounds[3],bounds[4]);
+    fVal[7] = fPtr->EvaluateFunction(bounds[1],bounds[3],bounds[5]);
+
+    prior = 0;
+    int numOffsets = this->ContourValues->GetNumberOfContours();
+    for (int c=0; c < numOffsets; c++)
+      {      
+      double value = this->ContourValues->GetValue(c);
+      int i;
+      bool less = fVal[0]<value;
+      for (i=1; i<8;i++)
+        {
+        if ((fVal[i]<=value) != less)
+          {
+          //this corner is on different side than first, piece 
+          //intersects and cannot be rejected
+          prior = inPrior;
+          c = numOffsets;
+          break;
+          }
+        }
+      }
+    if (prior != inPrior)
+      {
+      //cerr << "rejected something!" << endl;
+      }
+    outputVector->GetInformationObject(0)->
+      Set(vtkStreamingDemandDrivenPipeline::PRIORITY(),prior);
+    return 1;
+    }
+
+  //all other requests handled by superclass
+  return this->Superclass::ProcessRequest(request, inputVector,
+                                          outputVector);
+}
