@@ -35,7 +35,7 @@
 #include "vtkLine.h"
 #include "vtkSelection.h"
 
-vtkCxxRevisionMacro(vtkExtractSelectedFrustum, "$Revision: 1.10 $");
+vtkCxxRevisionMacro(vtkExtractSelectedFrustum, "$Revision: 1.16 $");
 vtkStandardNewMacro(vtkExtractSelectedFrustum);
 vtkCxxSetObjectMacro(vtkExtractSelectedFrustum,Frustum,vtkPlanes);
 
@@ -49,7 +49,6 @@ vtkExtractSelectedFrustum::vtkExtractSelectedFrustum(vtkPlanes *f)
 
   this->ShowBounds = 0;
 
-  this->PassThrough = 0;
   this->FieldType = 0;
   this->ContainingCells = 0;
   this->InsideOut = 0;
@@ -175,8 +174,8 @@ void vtkExtractSelectedFrustum::ComputePlane(int idx,
 //and we sometimes want to change it to make an UnstructuredGrid regardless of 
 //input type
 int vtkExtractSelectedFrustum::RequestDataObject(
-  vtkInformation*, 
-  vtkInformationVector** inputVector , 
+  vtkInformation* req,
+  vtkInformationVector** inputVector ,
   vtkInformationVector* outputVector)
 {
   vtkInformation* inInfo = inputVector[0]->GetInformationObject(0);
@@ -187,54 +186,21 @@ int vtkExtractSelectedFrustum::RequestDataObject(
 
   vtkDataSet *input = vtkDataSet::SafeDownCast(
     inInfo->Get(vtkDataObject::DATA_OBJECT()));
-  if (input)
+  if (input && this->ShowBounds)
     {
-    this->PassThrough = 0;
-    vtkInformation* selInfo = inputVector[1]->GetInformationObject(0);
-    if (selInfo)
+    vtkInformation* info = outputVector->GetInformationObject(0);
+    vtkDataSet *output = vtkDataSet::GetData(info);
+    if (!output || !output->IsA("vtkUnstructuredGrid"))
       {
-      vtkSelection *sel = vtkSelection::SafeDownCast(
-        selInfo->Get(vtkDataObject::DATA_OBJECT()));
-      if (sel->GetProperties()->Has(vtkSelection::PRESERVE_TOPOLOGY()) &&
-          sel->GetProperties()->Get(vtkSelection::PRESERVE_TOPOLOGY()) != 0)
-        {
-        this->PassThrough = 1;
-        }
+      vtkUnstructuredGrid* newOutput = vtkUnstructuredGrid::New();
+      newOutput->SetPipelineInformation(info);
+      newOutput->Delete();
+      this->GetOutputPortInformation(0)->Set(
+        vtkDataObject::DATA_EXTENT_TYPE(), newOutput->GetExtentType());
       }
-
-    for(int i=0; i < this->GetNumberOfOutputPorts(); ++i)
-      {
-      vtkInformation* info = outputVector->GetInformationObject(i);
-      vtkDataSet *output = vtkDataSet::SafeDownCast(
-        info->Get(vtkDataObject::DATA_OBJECT()));
-      
-      if (!output || 
-          ((this->ShowBounds || !this->PassThrough) && !output->IsA("vtkUnstructuredGrid")) ||
-          (this->PassThrough && !output->IsA(input->GetClassName()))
-        ) 
-        {
-        if (this->ShowBounds || !this->PassThrough)
-          { 
-          vtkUnstructuredGrid* newOutput = vtkUnstructuredGrid::New();
-          newOutput->SetPipelineInformation(info);
-          newOutput->Delete();
-          this->GetOutputPortInformation(0)->Set(
-            vtkDataObject::DATA_EXTENT_TYPE(), newOutput->GetExtentType());
-          }
-        else
-          {
-          vtkDataSet* newOutput;
-          newOutput = input->NewInstance();
-          newOutput->SetPipelineInformation(info);
-          newOutput->Delete();
-          this->GetOutputPortInformation(0)->Set(
-            vtkDataObject::DATA_EXTENT_TYPE(), newOutput->GetExtentType());
-          }
-        }
-      }
-    return 1;
     }
-  return 0;
+
+  return this->Superclass::RequestDataObject(req, inputVector, outputVector);
 }
 
 //----------------------------------------------------------------------------
@@ -259,12 +225,6 @@ int vtkExtractSelectedFrustum::RequestData(
       vtkDoubleArray *corners = vtkDoubleArray::SafeDownCast(
         sel->GetSelectionList());
       this->CreateFrustum(corners->GetPointer(0));
-      if (sel->GetProperties()->Has(vtkSelection::PRESERVE_TOPOLOGY()))
-        {
-        this->SetPassThrough(
-          sel->GetProperties()->Get(vtkSelection::PRESERVE_TOPOLOGY())
-          );
-        }
       if (sel->GetProperties()->Has(vtkSelection::INVERSE()))
         {
         this->SetInsideOut(
@@ -283,12 +243,6 @@ int vtkExtractSelectedFrustum::RequestData(
           sel->GetProperties()->Get(vtkSelection::CONTAINING_CELLS())
           );
         }
-      if (sel->GetProperties()->Has(vtkSelection::SHOW_BOUNDS()))
-        {
-        this->SetShowBounds(
-          sel->GetProperties()->Get(vtkSelection::SHOW_BOUNDS())
-          );
-        }
       }    
     }
 
@@ -425,7 +379,7 @@ int vtkExtractSelectedFrustum::RequestData(
 
   signed char flag = this->InsideOut ? 1 : -1;
 
-  if (this->PassThrough)
+  if (this->PreserveTopology)
     {
     //the output is a copy of the input, with two new arrays defined
     outputDS->ShallowCopy(input);
@@ -455,15 +409,19 @@ int vtkExtractSelectedFrustum::RequestData(
     //the output is a new unstructured grid
     outputUG->Allocate(numCells/4); //allocate storage for geometry/topology
     newPts->Allocate(numPts/4,numPts);
+    outputPD->SetCopyGlobalIds(1);
+    outputPD->CopyFieldOff("vtkOriginalPointIds");
     outputPD->CopyAllocate(pd);
 
     if ((this->FieldType == vtkSelection::CELL)
         ||
-        this->PassThrough 
+        this->PreserveTopology 
         ||
         this->ContainingCells
       )
       {
+      outputCD->SetCopyGlobalIds(1);
+      outputCD->CopyFieldOff("vtkOriginalCellIds");
       outputCD->CopyAllocate(cd);
 
       originalCellIds = vtkIdTypeArray::New();
@@ -508,7 +466,7 @@ int vtkExtractSelectedFrustum::RequestData(
       {
       if ( ! (cellId % updateInterval) ) //manage progress reports 
           {
-          this->UpdateProgress ((float)cellId / numCells);
+          this->UpdateProgress (static_cast<double>(cellId) / numCells);
           }
 
       input->GetCellBounds(cellId, bounds);
@@ -537,7 +495,7 @@ int vtkExtractSelectedFrustum::RequestData(
             */
 
             input->GetPoint(ptId, x);      
-            if (this->PassThrough)
+            if (this->PreserveTopology)
               {
               pointInArray->SetValue(ptId, flag);
               newPointId = ptId;
@@ -553,7 +511,7 @@ int vtkExtractSelectedFrustum::RequestData(
           newCellPts->InsertId(i,newPointId);
           }
 
-        if (this->PassThrough)
+        if (this->PreserveTopology)
           {
           cellInArray->SetValue(cellId, flag);
           }
@@ -593,7 +551,7 @@ int vtkExtractSelectedFrustum::RequestData(
           /*
           NUMPTS++;
           */
-          if (this->PassThrough)
+          if (this->PreserveTopology)
             {
             pointInArray->SetValue(ptId, flag);
             }
@@ -620,7 +578,7 @@ int vtkExtractSelectedFrustum::RequestData(
       
       if ( ! (ptId % updateInterval) ) //manage progress reports
           {
-          this->UpdateProgress ((float)ptId / numPts);
+          this->UpdateProgress (static_cast<double>(ptId) / numPts);
           }
 
       input->GetPoint(ptId,x);      
@@ -630,7 +588,7 @@ int vtkExtractSelectedFrustum::RequestData(
         /*
         NUMPTS++;
         */
-        if (this->PassThrough)
+        if (this->PreserveTopology)
           {
           newPointId = ptId;
           pointInArray->SetValue(ptId,flag);
@@ -651,7 +609,7 @@ int vtkExtractSelectedFrustum::RequestData(
     timer->StartTimer();
     */
 
-    if (this->PassThrough)
+    if (this->PreserveTopology)
       {
       //we have already created a copy of the input and marked points as 
       //being in or not
@@ -766,7 +724,7 @@ int vtkExtractSelectedFrustum::RequestData(
   pointInArray->Delete();
   cellInArray->Delete();
 
-  if (!this->PassThrough)
+  if (!this->PreserveTopology)
     {
     outputUG->SetPoints(newPts);
     }
@@ -894,7 +852,7 @@ int vtkExtractSelectedFrustum::ABoxFrustumIsect(double *bounds, vtkCell *cell)
   */
   vtkCell *face;
   vtkCell *edge;
-  vtkPoints *pts;
+  vtkPoints *pts=0;
   double *vertbuffer;
   int maxedges = 16;
   //be ready to resize if we hit a polygon with many vertices
@@ -910,8 +868,42 @@ int vtkExtractSelectedFrustum::ABoxFrustumIsect(double *bounds, vtkCell *cell)
     int nedges = cell->GetNumberOfEdges();
     if (nedges < 1)
       {
-      delete[] vertbuffer;
-      return this->IsectDegenerateCell(cell);
+      // VTK_LINE and VTK_POLY_LINE have no "edges" -- the cells
+      // themselves are edges.  We catch them here and assemble the
+      // list of vertices by hand because the code below assumes that
+      // GetNumberOfEdges()==0 means a degenerate cell containing only
+      // points.
+      if (cell->GetCellType() == VTK_LINE)
+        {
+        nedges = 2;
+        vtkPoints *points = cell->GetPoints();
+        points->GetPoint(0, &vlist[0*3]);
+        points->GetPoint(1, &vlist[1*3]);
+        }
+      else if (cell->GetCellType() == VTK_POLY_LINE)
+        {
+        nedges = cell->GetPointIds()->GetNumberOfIds();
+        vtkPoints *points = cell->GetPoints();
+        if (nedges + 4 > maxedges)
+          {
+          delete [] vertbuffer;
+          maxedges = (nedges + 4) * 2;
+          vertbuffer = new double[3*maxedges + 3];
+          vlist = &vertbuffer[0*maxedges*3];
+          wvlist = &vertbuffer[1*maxedges*3];
+          ovlist = &vertbuffer[2*maxedges*3];
+          }
+        for (int i = 0; i < cell->GetPointIds()->GetNumberOfIds(); ++i)
+          {
+          points->GetPoint(cell->GetPointIds()->GetId(i),
+                           &vlist[i*3]);
+          }
+        }
+      else
+        {
+        delete[] vertbuffer;
+        return this->IsectDegenerateCell(cell);
+        }
       }
     if (nedges+4 > maxedges)
       {
@@ -923,9 +915,12 @@ int vtkExtractSelectedFrustum::ABoxFrustumIsect(double *bounds, vtkCell *cell)
       ovlist = &vertbuffer[2*maxedges*3];
       }
     edge = cell->GetEdge(0);
-    pts = edge->GetPoints();
-    pts->GetPoint(0, &vlist[0*3]);
-    pts->GetPoint(1, &vlist[1*3]);
+    if (edge)
+      {
+      pts = edge->GetPoints();
+      pts->GetPoint(0, &vlist[0*3]);
+      pts->GetPoint(1, &vlist[1*3]);
+      }
     switch (cell->GetCellType())
       {
       case VTK_PIXEL:
@@ -944,9 +939,10 @@ int vtkExtractSelectedFrustum::ABoxFrustumIsect(double *bounds, vtkCell *cell)
         break;
         }
       case VTK_LINE:
-        {
-        break;
-        }
+      case VTK_POLY_LINE:
+      {
+      break;
+      }
       default:
         {
         for (int e = 1; e < nedges-1; e++)
@@ -1069,7 +1065,7 @@ int vtkExtractSelectedFrustum::FrustumClipPolygon(int nverts,
                                             double *ovlist)
 {
   int nwverts = nverts;
-  memcpy((void*)wvlist, (void*)ivlist, nverts*sizeof(double)*3);
+  memcpy(wvlist,ivlist, nverts*sizeof(double)*3);
 
   int noverts = 0;
   int pid;
@@ -1081,7 +1077,7 @@ int vtkExtractSelectedFrustum::FrustumClipPolygon(int nverts,
       {
       return 0;
       }
-    memcpy((void*)wvlist, (void*)ovlist, noverts*sizeof(double)*3);
+    memcpy(wvlist,ovlist, noverts*sizeof(double)*3);
     nwverts = noverts;
     }
 
@@ -1140,35 +1136,16 @@ void vtkExtractSelectedFrustum::PlaneClipEdge(double *V0, double *V1, int pid,
   return;
 }
 
-//----------------------------------------------------------------------------
-int vtkExtractSelectedFrustum::FillInputPortInformation(
-  int port, vtkInformation* info)
-{
-  if (port==0)
-    {
-    info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkDataSet");    
-    }
-  else
-    {
-    info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkSelection");
-    info->Set(vtkAlgorithm::INPUT_IS_OPTIONAL(), 1);
-    }
-  return 1;
-}
-
 //----------------------------------------------------------------------------
 void vtkExtractSelectedFrustum::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
 
   os << indent << "Frustum: " 
-     << (void *)this->Frustum << "\n";
+     << static_cast<void *>(this->Frustum) << "\n";
 
   os << indent << "ClipPoints: " << this->ClipPoints << "\n";
 
-  os << indent << "PassThrough: " 
-     << (this->PassThrough ? "On\n" : "Off\n");
-
   os << indent << "FieldType: "
      << (this->FieldType ? "On\n" : "Off\n");
 
