@@ -11,7 +11,7 @@
      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
      PURPOSE.  See the above copyright notice for more information.
 
-  Copyright 2003-2006 Sandia Corporation.
+  Copyright 2003-2008 Sandia Corporation.
   Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
   license for use of this work by or on behalf of the
   U.S. Government. Redistribution and use in source and binary forms, with
@@ -38,7 +38,7 @@
 
 #include "verdict.h"
 
-vtkCxxRevisionMacro(vtkMeshQuality,"$Revision: 1.51 $");
+vtkCxxRevisionMacro(vtkMeshQuality,"$Revision: 1.54 $");
 vtkStandardNewMacro(vtkMeshQuality);
 
 typedef double (*CellQualityType)( vtkCell*  );
@@ -660,7 +660,7 @@ int vtkMeshQuality::RequestData(
           }
         Eqtri += q;
         Eqtri2 += q * q;
-        ntri++;
+        ++ ntri;
         break;
       case VTK_QUAD:
         q = QuadQuality( cell );
@@ -678,7 +678,7 @@ int vtkMeshQuality::RequestData(
           }
         Eqqua += q;
         Eqqua2 += q * q;
-        nqua++;
+        ++ nqua;
         break;
       case VTK_TETRA:
         q = TetQuality( cell );
@@ -696,7 +696,7 @@ int vtkMeshQuality::RequestData(
           }
         Eqtet += q;
         Eqtet2 += q * q;
-        ntet++;
+        ++ ntet;
         if ( this->Volume )
           {
           V = TetVolume( cell );
@@ -722,7 +722,7 @@ int vtkMeshQuality::RequestData(
           }
         Eqhex += q;
         Eqhex2 += q * q;
-        nhex++;
+        ++ nhex;
         break;
       default:
         q = 0.;
@@ -745,8 +745,9 @@ int vtkMeshQuality::RequestData(
 
   if ( ntri )
     {
-    Eqtri  /= (double) ntri;
-    Eqtri2 /= (double) ntri;
+    Eqtri  /= static_cast<double>( ntri );
+    double multFactor = 1. / static_cast<double>( ntri > 1 ? ntri - 1 : ntri );
+    Eqtri2 = multFactor * ( Eqtri2 - static_cast<double>( ntri ) * Eqtri * Eqtri );
     }
   else
     {
@@ -755,8 +756,9 @@ int vtkMeshQuality::RequestData(
   
   if ( nqua )
     {
-    Eqqua  /= (double) nqua;
-    Eqqua2 /= (double) nqua;
+    Eqqua  /= static_cast<double>( nqua );
+    double multFactor = 1. / static_cast<double>( nqua > 1 ? nqua - 1 : nqua );
+    Eqqua2 = multFactor * ( Eqqua2 - static_cast<double>( nqua ) * Eqqua * Eqqua );
     }
   else
     {
@@ -765,8 +767,9 @@ int vtkMeshQuality::RequestData(
   
   if ( ntet )
     {
-    Eqtet  /= (double) ntet;
-    Eqtet2 /= (double) ntet;
+    Eqtet  /= static_cast<double>( ntet );
+    double multFactor = 1. / static_cast<double>( ntet > 1 ? ntet - 1 : ntet );
+    Eqtet2 = multFactor * ( Eqtet2 - static_cast<double>( ntet ) * Eqtet * Eqtet );
     }
   else
     {
@@ -775,8 +778,9 @@ int vtkMeshQuality::RequestData(
 
   if ( nhex )
     {
-    Eqhex  /= (double) nhex;
-    Eqhex2 /= (double) nhex;
+    Eqhex  /= static_cast<double>( nhex );
+    double multFactor = 1. / static_cast<double>( nhex > 1 ? nhex - 1 : nhex );
+    Eqhex2 = multFactor * ( Eqhex2 - static_cast<double>( nhex ) * Eqhex * Eqhex );
     }
   else
     {
