@@ -36,28 +36,32 @@
 #ifndef __vtkExodusIIReader_h
 #define __vtkExodusIIReader_h
 
-#include "vtkUnstructuredGridAlgorithm.h"
+#include "vtkMultiBlockDataSetAlgorithm.h"
 
-class vtkIntArray;
-class vtkFloatArray;
 class vtkDataArray;
 class vtkDataSet;
-class vtkPoints;
+class vtkExodusIICache;
 class vtkExodusIIReaderPrivate;
 class vtkExodusModel;
-class vtkExodusIICache;
+class vtkFloatArray;
+class vtkGraph;
+class vtkIntArray;
+class vtkPoints;
+class vtkUnstructuredGrid;
 
-class VTK_HYBRID_EXPORT vtkExodusIIReader : public vtkUnstructuredGridAlgorithm 
+class VTK_HYBRID_EXPORT vtkExodusIIReader : public vtkMultiBlockDataSetAlgorithm 
 {
 public:
   static vtkExodusIIReader *New();
-  vtkTypeRevisionMacro(vtkExodusIIReader,vtkUnstructuredGridAlgorithm);
+  vtkTypeRevisionMacro(vtkExodusIIReader,vtkMultiBlockDataSetAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
   // Determine if the file can be readed with this reader.
   int CanReadFile(const char* fname);
 
+  //virtual void Modified();
+
   // Description:
   // Return the object's MTime. This is overridden to include the timestamp of its internal class.
   virtual unsigned long GetMTime();
@@ -111,6 +115,12 @@ public:
   int GetGenerateGlobalNodeIdArray();
   vtkBooleanMacro(GenerateGlobalNodeIdArray, int);
 
+  virtual void SetGenerateFileIdArray( int f );
+  int GetGenerateFileIdArray();
+  vtkBooleanMacro(GenerateFileIdArray, int);
+  virtual void SetFileId( int f );
+  int GetFileId();
+
   // Description:
   // Extra cell data array that can be generated.  By default, this array
   // is off.  The value of the array is the integer global id of the cell.
@@ -147,6 +157,8 @@ public:
     MATERIAL = 62,
     HIERARCHY = 63,
     // extended values (not in Exodus headers) for use in cache keys:
+    QA_RECORDS = 103,          //!< Exodus II Quality Assurance (QA) string metadata
+    INFO_RECORDS = 104,        //!< Exodus II Information Records string metadata
     GLOBAL_TEMPORAL = 102,  //!< global data across timesteps
     NODAL_TEMPORAL = 101,      //!< nodal data across timesteps
     ELEM_BLOCK_TEMPORAL = 100,  //!< element data across timesteps
@@ -162,7 +174,7 @@ public:
     EDGE_SET_CONN = 90,        //!< edge set connectivity
     NODE_SET_CONN = 89,        //!< node set connectivity
     NODAL_COORDS = 88,         //!< raw nodal coordinates (not the "squeezed" version)
-    GLOBAL_OBJECT_ID = 87,     //!< assembled object id (old BlockId) array
+    OBJECT_ID = 87,            //!< object id (old BlockId) array
     GLOBAL_ELEMENT_ID = 86,    //!< assembled, zero-padded element id array
     GLOBAL_NODE_ID = 85,       //!< assembled, zero-padded nodal id array
     ELEMENT_ID = 84,           //!< element id map (old-style elem_num_map or first new-style elem map) array
@@ -312,10 +324,10 @@ public:
     { this->SetObjectAttributeStatus( objectType, objectIndex,
       this->GetObjectAttributeIndex( objectType, objectIndex, attribName ), status ); }
 
-  virtual int GetTotalNumberOfNodes();
-  virtual int GetTotalNumberOfEdges();
-  virtual int GetTotalNumberOfFaces();
-  virtual int GetTotalNumberOfElements();
+  virtual vtkIdType GetTotalNumberOfNodes();
+  virtual vtkIdType GetTotalNumberOfEdges();
+  virtual vtkIdType GetTotalNumberOfFaces();
+  virtual vtkIdType GetTotalNumberOfElements();
 
   // Descriptions:
   // By default all parts are loaded. These methods allow the user to select
@@ -377,7 +389,7 @@ public:
 
   // Description:
   //   There is a great deal of model information lost when an Exodus II
-  //   file is read in to a vtkUnstructuredGrid.  Turn this option ON 
+  //   file is read in to a vtkMultiBlockDataSet.  Turn this option ON 
   //   if you want this metadata to be read in to a vtkExodusModel object.
   //   The default is OFF.
 
@@ -621,25 +633,30 @@ public:
   void SetElementSetResultArrayStatus(const char* name, int flag)
     { this->SetObjectArrayStatus(ELEM_SET, name, flag); }
 
+  /**!\brief Fast path 
+    *
+    * The following are set using the fast-path keys found in 
+    * vtkPExodusIIReader's input information. 
+    * Fast-path keys are meant to be used by an filter that 
+    * works with temporal data. Rather than re-executing the pipeline
+    * for each timestep, since the exodus reader, as part of its API, contains
+    * a faster way to read temporal data, algorithms may use these
+    * keys to request temporal data.
+    * See also: vtkExtractArraysOverTime. 
+    */
+  //@{
   // Description:
-  // The following are set using the fast-path keys found in 
-  // vtkPExodusIIReader's input information. 
-  // Fast-path keys are meant to be used by an filter that 
-  // works with temporal data. Rather than re-executing the pipeline
-  // for each timestep, since the exodus reader, as part of its API, contains
-  // a faster way to read temporal data, algorithms may use these
-  // keys to request temporal data.
-  // See also: vtkExtractArraysOverTime. 
-
   // Set the fast-path keys. All three must be set for the fast-path
   // option to work.
   // Possible argument values: "POINT","CELL","EDGE","FACE"
   void SetFastPathObjectType(const char *type);
+  // Description:
   // Possible argument values: "INDEX","GLOBAL"
   // "GLOBAL" means the id refers to a global id
   // "INDEX" means the id refers to an index into the VTK array
   void SetFastPathIdType(const char *type);
   void SetFastPathObjectId(vtkIdType id);
+  //@}
 
   // Description:
   // Reset the user-specified parameters and flush internal arrays
@@ -659,11 +676,26 @@ public:
   // keep the active cache but return to initial array selections, etc.
   void ResetSettings();
 
+  // Description:
+  // Clears out the cache entries.
+  void ResetCache();
+
   // Description:
   // Re-reads time information from the exodus file and updates
   // TimeStepRange accordingly.
   virtual void UpdateTimeInformation();
 
+  virtual void Dump();
+
+  // Description:
+  // SIL describes organization of/relationships between classifications 
+  // eg. blocks/materials/hierarchies.
+  vtkGraph* GetSIL();
+
+  // Description:
+  // Every time the SIL is updated a this will return a different value.
+  vtkGetMacro(SILUpdateStamp, int);
+
 protected:
   vtkExodusIIReader();
   ~vtkExodusIIReader();
@@ -679,8 +711,6 @@ protected:
   virtual void SetMetadata( vtkExodusIIReaderPrivate* );
   vtkGetObjectMacro(Metadata,vtkExodusIIReaderPrivate);
 
-  virtual void Dump();
-
   // Description: 
   // Returns true if XMLFileName has already been set. Otherwise, look for the XML
   // metadata file in the same directory as the data file(s) using the following
@@ -691,6 +721,21 @@ protected:
   //  Return true if found, false otherwise
   bool FindXMLFile();
 
+  // Time query function. Called by ExecuteInformation().
+  // Fills the TimestepValues array.
+  void GetAllTimes(vtkInformationVector*); 
+
+  // Description:
+  // Populates the TIME_STEPS and TIME_RANGE keys based on file metadata.
+  void AdvertiseTimeSteps( vtkInformation* outputInfo );
+
+  virtual void SetExodusModel( vtkExodusModel* em );
+
+  int ProcessRequest( vtkInformation *, vtkInformationVector **, vtkInformationVector *);
+  int RequestInformation( vtkInformation *, vtkInformationVector **, vtkInformationVector *);
+  int RequestData( vtkInformation *, vtkInformationVector **, vtkInformationVector *);
+  //int RequestDataOverTime( vtkInformation *, vtkInformationVector **, vtkInformationVector *);
+
   // Parameters for controlling what is read in.
   char* FileName;
   char* XMLFileName;
@@ -709,20 +754,11 @@ protected:
   // Metadata containing a description of the currently open file.
   vtkExodusIIReaderPrivate* Metadata;
 
-  // Time query function. Called by ExecuteInformation().
-  // Fills the TimestepValues array.
-  void GetAllTimes(vtkInformationVector*); 
-
   vtkExodusModel *ExodusModel;
   int PackExodusModelOntoOutput;
   int ExodusModelMetadata;
 
-  int ProcessRequest( vtkInformation *, vtkInformationVector **, vtkInformationVector *);
-  int RequestInformation( vtkInformation *, vtkInformationVector **, vtkInformationVector *);
-  int RequestData( vtkInformation *, vtkInformationVector **, vtkInformationVector *);
-  //int RequestDataOverTime( vtkInformation *, vtkInformationVector **, vtkInformationVector *);
-
-  virtual void SetExodusModel( vtkExodusModel* em );
+  int SILUpdateStamp;
 
 private:
   vtkExodusIIReader(const vtkExodusIIReader&); // Not implemented
