@@ -22,6 +22,7 @@
 #include "vtkErrorCode.h"
 #include "vtkFieldData.h"
 #include "vtkFloatArray.h"
+#include "vtkGraph.h"
 #include "vtkIdTypeArray.h"
 #include "vtkInformation.h"
 #include "vtkInformationVector.h"
@@ -35,11 +36,13 @@
 #include "vtkShortArray.h"
 #include "vtkStreamingDemandDrivenPipeline.h"
 #include "vtkStringArray.h"
+#include "vtkTable.h"
 #include "vtkTypeInt64Array.h"
 #include "vtkUnsignedCharArray.h"
 #include "vtkUnsignedIntArray.h"
 #include "vtkUnsignedLongArray.h"
 #include "vtkUnsignedShortArray.h"
+#include "vtkVariantArray.h"
 #include <vtksys/ios/sstream>
 
 // We only have vtkTypeUInt64Array if we have long long
@@ -58,7 +61,7 @@
 // so it would be nice to put this in a common file.
 static int my_getline(istream& stream, vtkStdString &output, char delim='\n');
 
-vtkCxxRevisionMacro(vtkDataReader, "$Revision: 1.148 $");
+vtkCxxRevisionMacro(vtkDataReader, "$Revision: 1.157 $");
 vtkStandardNewMacro(vtkDataReader);
 
 vtkCxxSetObjectMacro(vtkDataReader, InputArray, vtkCharArray);
@@ -179,55 +182,57 @@ vtkDataReader::~vtkDataReader()
 }
 
 void vtkDataReader::SetInputString(const char *in)
-{ 
+{
+  int len = 0;
   if (in != NULL)
     {
-    this->SetInputString(in, static_cast<int>(strlen(in)));
-    }
-  else
-    {
-    if (this->InputString)
-      {
-      delete [] this->InputString; 
-      }
-    this->InputString = NULL;
+    len = static_cast<int>(strlen(in));
     }
+  this->SetInputString(in, len);
 }
 
 void vtkDataReader::SetBinaryInputString(const char *in, int len)
 {
-    this->SetInputString(in,len);
+  this->SetInputString(in, len);
 }
 
 void vtkDataReader::SetInputString(const char *in, int len)
-{ 
+{
   if (this->Debug)
     {
-    vtkDebugMacro(<< "setting InputString to " << in );
+    vtkDebugMacro(<< "SetInputString len: " << len
+      << " in: " << (in ? in : "(null)"));
     }
 
   if (this->InputString && in && strncmp(in, this->InputString, len) == 0)
     {
     return;
     }
-  
+
   if (this->InputString)
     {
-    delete [] this->InputString; 
+    delete [] this->InputString;
     }
-  if (in) 
-    { 
-    this->InputString = new char[len]; 
-    memcpy(this->InputString,in,len); 
+
+  if (in && len>0)
+    {
+    // Add a NULL terminator so that GetInputString
+    // callers (from wrapped languages) get a valid
+    // C string in *ALL* cases...
+    //
+    this->InputString = new char[len+1];
+    memcpy(this->InputString,in,len);
+    this->InputString[len] = 0;
     this->InputStringLength = len;
-    } 
-   else 
-    { 
-    this->InputString = NULL; 
+    }
+   else
+    {
+    this->InputString = NULL;
     this->InputStringLength = 0;
-    } 
-  this->Modified(); 
-} 
+    }
+
+  this->Modified();
+}
 
 // Internal function to read in a line up to 256 characters.
 // Returns zero if there was an error.
@@ -736,7 +741,7 @@ int vtkDataReader::ReadCellData(vtkDataSet *ds, int numCells)
         }
       for(int i=0; i<f->GetNumberOfArrays(); i++)
         {
-        a->AddArray(f->GetArray(i));
+        a->AddArray(f->GetAbstractArray(i));
         }
       f->Delete();
       }
@@ -914,6 +919,431 @@ int vtkDataReader::ReadPointData(vtkDataSet *ds, int numPts)
   return 1;
 }
 
+
+// Read the vertex data of a vtk data file. The number of vertices (from the 
+// graph) must match the number of vertices defined in vertex attributes (unless
+// no geometry was defined).
+int vtkDataReader::ReadVertexData(vtkGraph *g, int numVertices)
+{
+  char line[256];
+  vtkDataSetAttributes *a=g->GetVertexData();
+
+  vtkDebugMacro(<< "Reading vtk vertex data");
+    
+  //
+  // Read keywords until end-of-file
+  //
+  while (this->ReadString(line))
+    {
+    //
+    // read scalar data
+    //
+    if ( ! strncmp(this->LowerCase(line), "scalars", 7) )
+      {
+      if ( ! this->ReadScalarData(a, numVertices) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read vector data
+    //
+    else if ( ! strncmp(line, "vectors", 7) )
+      {
+      if ( ! this->ReadVectorData(a, numVertices) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read 3x3 tensor data
+    //
+    else if ( ! strncmp(line, "tensors", 7) )
+      {
+      if ( ! this->ReadTensorData(a, numVertices) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read normals data
+    //
+    else if ( ! strncmp(line, "normals", 7) )
+      {
+      if ( ! this->ReadNormalData(a, numVertices) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read texture coordinates data
+    //
+    else if ( ! strncmp(line, "texture_coordinates", 19) )
+      {
+      if ( ! this->ReadTCoordsData(a, numVertices) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read the global id data
+    //
+    else if ( ! strncmp(line, "global_ids", 10) )
+      {
+      if ( ! this->ReadGlobalIds(a, numVertices) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read the pedigree id data
+    //
+    else if ( ! strncmp(line, "pedigree_ids", 10) )
+      {
+      if ( ! this->ReadPedigreeIds(a, numVertices) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read color scalars data
+    //
+    else if ( ! strncmp(line, "color_scalars", 13) )
+      {
+      if ( ! this->ReadCoScalarData(a, numVertices) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read lookup table. Associate with scalar data.
+    //
+    else if ( ! strncmp(line, "lookup_table", 12) )
+      {
+      if ( ! this->ReadLutData(a) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read field of data
+    //
+    else if ( ! strncmp(line, "field", 5) )
+      {
+      vtkFieldData *f;
+      if ( ! (f=this->ReadFieldData()) )
+        {
+        return 0;
+        }
+      for(int i=0; i<f->GetNumberOfArrays(); i++)
+        {
+        a->AddArray(f->GetAbstractArray(i));
+        }
+      f->Delete();
+      }
+    //
+    // maybe bumped into edge data
+    //
+    else if ( ! strncmp(line, "edge_data", 10) )
+      {
+      int npts;
+      if (!this->Read(&npts))
+        {
+        vtkErrorMacro(<<"Cannot read point data!");
+        return 0;
+        }
+
+      this->ReadEdgeData(g, npts);
+      }
+
+    else
+      {
+      vtkErrorMacro(<< "Unsupported vertex attribute type: " << line 
+                    << " for file: " << (this->FileName?this->FileName:"(Null FileName)"));
+      return 0;
+      }
+    }
+
+  return 1;
+}
+
+// Read the edge data of a vtk data file. The number of edges (from the 
+// graph) must match the number of edges defined in edge attributes (unless
+// no geometry was defined).
+int vtkDataReader::ReadEdgeData(vtkGraph *g, int numEdges)
+{
+  char line[256];
+  vtkDataSetAttributes *a=g->GetEdgeData();
+
+  vtkDebugMacro(<< "Reading vtk edge data");
+    
+  //
+  // Read keywords until end-of-file
+  //
+  while (this->ReadString(line))
+    {
+    //
+    // read scalar data
+    //
+    if ( ! strncmp(this->LowerCase(line), "scalars", 7) )
+      {
+      if ( ! this->ReadScalarData(a, numEdges) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read vector data
+    //
+    else if ( ! strncmp(line, "vectors", 7) )
+      {
+      if ( ! this->ReadVectorData(a, numEdges) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read 3x3 tensor data
+    //
+    else if ( ! strncmp(line, "tensors", 7) )
+      {
+      if ( ! this->ReadTensorData(a, numEdges) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read normals data
+    //
+    else if ( ! strncmp(line, "normals", 7) )
+      {
+      if ( ! this->ReadNormalData(a, numEdges) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read texture coordinates data
+    //
+    else if ( ! strncmp(line, "texture_coordinates", 19) )
+      {
+      if ( ! this->ReadTCoordsData(a, numEdges) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read the global id data
+    //
+    else if ( ! strncmp(line, "global_ids", 10) )
+      {
+      if ( ! this->ReadGlobalIds(a, numEdges) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read the pedigree id data
+    //
+    else if ( ! strncmp(line, "pedigree_ids", 10) )
+      {
+      if ( ! this->ReadPedigreeIds(a, numEdges) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read color scalars data
+    //
+    else if ( ! strncmp(line, "color_scalars", 13) )
+      {
+      if ( ! this->ReadCoScalarData(a, numEdges) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read lookup table. Associate with scalar data.
+    //
+    else if ( ! strncmp(line, "lookup_table", 12) )
+      {
+      if ( ! this->ReadLutData(a) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read field of data
+    //
+    else if ( ! strncmp(line, "field", 5) )
+      {
+      vtkFieldData *f;
+      if ( ! (f=this->ReadFieldData()) )
+        {
+        return 0;
+        }
+      for(int i=0; i<f->GetNumberOfArrays(); i++)
+        {
+        a->AddArray(f->GetAbstractArray(i));
+        }
+      f->Delete();
+      }
+    //
+    // maybe bumped into vertex data
+    //
+    else if ( ! strncmp(line, "vertex_data", 10) )
+      {
+      int npts;
+      if (!this->Read(&npts))
+        {
+        vtkErrorMacro(<<"Cannot read vertex data!");
+        return 0;
+        }
+
+      this->ReadVertexData(g, npts);
+      }
+
+    else
+      {
+      vtkErrorMacro(<< "Unsupported vertex attribute type: " << line 
+                    << " for file: " << (this->FileName?this->FileName:"(Null FileName)"));
+      return 0;
+      }
+    }
+
+  return 1;
+}
+
+// Read the row data of a vtk data file.
+int vtkDataReader::ReadRowData(vtkTable *t, int numEdges)
+{
+  char line[256];
+  vtkDataSetAttributes *a=t->GetRowData();
+
+  vtkDebugMacro(<< "Reading vtk row data");
+    
+  //
+  // Read keywords until end-of-file
+  //
+  while (this->ReadString(line))
+    {
+    //
+    // read scalar data
+    //
+    if ( ! strncmp(this->LowerCase(line), "scalars", 7) )
+      {
+      if ( ! this->ReadScalarData(a, numEdges) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read vector data
+    //
+    else if ( ! strncmp(line, "vectors", 7) )
+      {
+      if ( ! this->ReadVectorData(a, numEdges) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read 3x3 tensor data
+    //
+    else if ( ! strncmp(line, "tensors", 7) )
+      {
+      if ( ! this->ReadTensorData(a, numEdges) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read normals data
+    //
+    else if ( ! strncmp(line, "normals", 7) )
+      {
+      if ( ! this->ReadNormalData(a, numEdges) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read texture coordinates data
+    //
+    else if ( ! strncmp(line, "texture_coordinates", 19) )
+      {
+      if ( ! this->ReadTCoordsData(a, numEdges) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read the global id data
+    //
+    else if ( ! strncmp(line, "global_ids", 10) )
+      {
+      if ( ! this->ReadGlobalIds(a, numEdges) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read the pedigree id data
+    //
+    else if ( ! strncmp(line, "pedigree_ids", 10) )
+      {
+      if ( ! this->ReadPedigreeIds(a, numEdges) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read color scalars data
+    //
+    else if ( ! strncmp(line, "color_scalars", 13) )
+      {
+      if ( ! this->ReadCoScalarData(a, numEdges) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read lookup table. Associate with scalar data.
+    //
+    else if ( ! strncmp(line, "lookup_table", 12) )
+      {
+      if ( ! this->ReadLutData(a) )
+        {
+        return 0;
+        }
+      }
+    //
+    // read field of data
+    //
+    else if ( ! strncmp(line, "field", 5) )
+      {
+      vtkFieldData *f;
+      if ( ! (f=this->ReadFieldData()) )
+        {
+        return 0;
+        }
+      for(int i=0; i<f->GetNumberOfArrays(); i++)
+        {
+        a->AddArray(f->GetAbstractArray(i));
+        }
+      f->Delete();
+      }
+    else
+      {
+      vtkErrorMacro(<< "Unsupported row attribute type: " << line 
+                    << " for file: " << (this->FileName?this->FileName:"(Null FileName)"));
+      return 0;
+      }
+    }
+
+  return 1;
+}
+
 // General templated function to read data of various types.
 template <class T>
 int vtkReadBinaryData(istream *IS, T *data, int numTuples, int numComp)
@@ -1308,12 +1738,7 @@ vtkAbstractArray *vtkDataReader::ReadArray(const char *dataType, int numTuples,
         for (int j=0; j<numComp; j++)
           {
           my_getline(*(this->IS), s);
-          vtkIdType idx = i*numComp + j;
-          if (idx % 100 == 0 || idx > 4900)
-            {
-            cerr << "reading index " << idx << "=" << s.substr(0, 10) << endl;
-            }
-          int length = s.length();
+          int length = static_cast<int>(s.length());
           char* decoded = new char[length + 1];
           int decodedLength = this->DecodeString(decoded, s.c_str());
           vtkStdString decodedStr(decoded, decodedLength);
@@ -1322,6 +1747,83 @@ vtkAbstractArray *vtkDataReader::ReadArray(const char *dataType, int numTuples,
         }
       }
     }
+  else if ( ! strncmp(type, "variant", 7) )
+    {
+    array = vtkVariantArray::New();
+    array->SetNumberOfComponents(numComp);
+    for (int i=0; i<numTuples; i++)
+      {
+      for (int j=0; j<numComp; j++)
+        {
+        int t;
+        vtkStdString str;
+        *(this->IS) >> t >> str;
+        char* decoded = new char[str.length() + 1];
+        int decodedLength = this->DecodeString(decoded, str.c_str());
+        vtkStdString decodedStr(decoded, decodedLength);
+        vtkVariant sv(decodedStr);
+        vtkVariant v;
+        switch (t)
+          {
+          case VTK_CHAR:
+            v = sv.ToChar();
+            break;
+          case VTK_SIGNED_CHAR:
+            v = sv.ToSignedChar();
+            break;
+          case VTK_UNSIGNED_CHAR:
+            v = sv.ToUnsignedChar();
+            break;
+          case VTK_SHORT:
+            v = sv.ToShort();
+            break;
+          case VTK_UNSIGNED_SHORT:
+            v = sv.ToUnsignedShort();
+            break;
+          case VTK_INT:
+            v = sv.ToInt();
+            break;
+          case VTK_UNSIGNED_INT:
+            v = sv.ToUnsignedInt();
+            break;
+          case VTK_LONG:
+            v = sv.ToLong();
+            break;
+          case VTK_UNSIGNED_LONG:
+            v = sv.ToUnsignedLong();
+            break;
+          case VTK_FLOAT:
+            v = sv.ToFloat();
+            break;
+          case VTK_DOUBLE:
+            v = sv.ToDouble();
+            break;
+#ifdef VTK_TYPE_USE_LONG_LONG
+          case VTK_LONG_LONG:
+            v = sv.ToLongLong();
+            break;
+          case VTK_UNSIGNED_LONG_LONG:
+            v = sv.ToUnsignedLongLong();
+            break;
+#endif
+#ifdef VTK_TYPE_USE___INT64
+          case VTK___INT64:
+            v = sv.To__Int64();
+            break;
+          case VTK_UNSIGNED___INT64:
+            v = sv.ToUnsigned__Int64();
+            break;
+#endif
+          case VTK_STRING:
+            v = sv.ToString();
+            break;
+          default:
+            vtkErrorMacro("Unknown variant type " << t);
+          }
+        ((vtkVariantArray*)array)->InsertNextValue(v);
+        }
+      }
+    }
   else 
     {
     vtkErrorMacro(<< "Unsupported data type: " << type);
@@ -1367,6 +1869,40 @@ int vtkDataReader::ReadPoints(vtkPointSet *ps, int numPts)
   return 1;
 }
 
+// Read point coordinates. Return 0 if error.
+int vtkDataReader::ReadPoints(vtkGraph *g, int numPts)
+{
+  char line[256];
+  vtkDataArray *data;
+
+  if (!this->ReadString(line)) 
+    {
+    vtkErrorMacro(<<"Cannot read points type!" << " for file: " << (this->FileName?this->FileName:"(Null FileName)"));
+    return 0;
+    }
+
+  data = vtkDataArray::SafeDownCast(
+    this->ReadArray(line, numPts, 3));
+  if ( data != NULL )
+    {
+    vtkPoints *points=vtkPoints::New();
+    points->SetData(data);
+    data->Delete();
+    g->SetPoints(points);
+    points->Delete();
+    }
+  else
+    {
+    return 0;
+    }
+
+  vtkDebugMacro(<<"Read " << g->GetNumberOfVertices() << " points");
+  float progress = this->GetProgress();
+  this->UpdateProgress(progress + 0.5*(1.0 - progress));
+
+  return 1;
+}
+
 // Read the coordinates for a rectilinear grid. The axes parameter specifies
 // which coordinate axes (0,1,2) is being read.
 int vtkDataReader::ReadCoordinates(vtkRectilinearGrid *rg, int axes, 
@@ -2628,7 +3164,7 @@ int vtkDataReader::DecodeString(char *resname, const char* name)
     }
   strncpy(resname, str.str().c_str(), reslen+1);
   resname[reslen+1] = 0;
-  return reslen;
+  return static_cast<int>(reslen);
 }
 
 static int
