@@ -24,6 +24,7 @@
 #include "vtkErrorCode.h"
 #include "vtkFieldData.h"
 #include "vtkFloatArray.h"
+#include "vtkGraph.h"
 #include "vtkIntArray.h"
 #include "vtkLongArray.h"
 #include "vtkLookupTable.h"
@@ -32,15 +33,17 @@
 #include "vtkPoints.h"
 #include "vtkShortArray.h"
 #include "vtkStringArray.h"
+#include "vtkTable.h"
 #include "vtkTypeTraits.h"
 #include "vtkUnsignedCharArray.h"
 #include "vtkUnsignedIntArray.h"
 #include "vtkUnsignedLongArray.h"
 #include "vtkUnsignedShortArray.h"
+#include "vtkVariantArray.h"
 #include <vtksys/ios/sstream>
 
 
-vtkCxxRevisionMacro(vtkDataWriter, "$Revision: 1.123 $");
+vtkCxxRevisionMacro(vtkDataWriter, "$Revision: 1.135 $");
 vtkStandardNewMacro(vtkDataWriter);
 
 // this undef is required on the hp. vtkMutexLock ends up including
@@ -76,7 +79,6 @@ vtkDataWriter::vtkDataWriter()
 
   this->WriteToOutputString = 0;
   this->OutputString = NULL;
-  this->OutputStringAllocatedLength = 0;
   this->OutputStringLength = 0;
 }
 
@@ -132,7 +134,6 @@ vtkDataWriter::~vtkDataWriter()
     delete [] this->OutputString;
     this->OutputString = NULL;
     this->OutputStringLength = 0;
-    this->OutputStringAllocatedLength = 0;
     }
 }
 
@@ -160,7 +161,6 @@ ostream *vtkDataWriter::OpenVTKFile()
       delete [] this->OutputString;
       this->OutputString = NULL;
       this->OutputStringLength = 0;
-      this->OutputStringAllocatedLength = 0;
       }
     // Allocate the new output string. (Note: this will only work with binary).
     if (input == NULL)
@@ -169,9 +169,10 @@ ostream *vtkDataWriter::OpenVTKFile()
       return NULL;    
       }
     input->Update();
-    this->OutputStringAllocatedLength = (int) (500 
-      + 1024 * input->GetActualMemorySize());
-    this->OutputString = new char[this->OutputStringAllocatedLength];
+    /// OutputString will be allocated on CloseVTKFile().
+    /// this->OutputStringAllocatedLength =
+    ///   static_cast<int> (500+ 1024 * input->GetActualMemorySize());
+    /// this->OutputString = new char[this->OutputStringAllocatedLength];
 
     fptr = new vtksys_ios::ostringstream;
     }
@@ -523,6 +524,442 @@ int vtkDataWriter::WritePointData(ostream *fp, vtkDataSet *ds)
   return 1;
 }
 
+// Write the vertex data (e.g., scalars, vectors, ...) of a vtk graph.
+// Returns 0 if error.
+int vtkDataWriter::WriteVertexData(ostream *fp, vtkGraph *ds)
+{
+  int numVertices;
+  vtkDataArray *scalars;
+  vtkDataArray *vectors;
+  vtkDataArray *normals;
+  vtkDataArray *tcoords;
+  vtkDataArray *tensors;
+  vtkDataArray *globalIds;
+  vtkAbstractArray *pedigreeIds;
+  vtkFieldData *field;
+  vtkDataSetAttributes *cd=ds->GetVertexData();
+
+  vtkDebugMacro(<<"Writing vertex data...");
+
+  numVertices = ds->GetNumberOfVertices();
+  if(numVertices <= 0)
+    {
+    vtkDebugMacro(<<"No vertex data to write!");
+    return 1;
+    }
+    
+  scalars = cd->GetScalars();
+  if(scalars && scalars->GetNumberOfTuples() <= 0)
+    scalars = 0;
+    
+  vectors = cd->GetVectors();
+  if(vectors && vectors->GetNumberOfTuples() <= 0)
+    vectors = 0;
+    
+  normals = cd->GetNormals();
+  if(normals && normals->GetNumberOfTuples() <= 0)
+    normals = 0;
+    
+  tcoords = cd->GetTCoords();
+  if(tcoords && tcoords->GetNumberOfTuples() <= 0)
+    tcoords = 0;
+    
+  tensors = cd->GetTensors();
+  if(tensors && tensors->GetNumberOfTuples() <= 0)
+    tensors = 0;
+    
+  globalIds = cd->GetGlobalIds();
+  if(globalIds && globalIds->GetNumberOfTuples() <= 0)
+    globalIds = 0;
+    
+  pedigreeIds = cd->GetPedigreeIds();
+  if(pedigreeIds && pedigreeIds->GetNumberOfTuples() <= 0)
+    pedigreeIds = 0;
+    
+  field = cd;
+  if(field && field->GetNumberOfTuples() <= 0)
+    field = 0;
+
+  if(!(scalars || vectors || normals || tcoords || tensors || globalIds || pedigreeIds || field))
+    {
+    vtkDebugMacro(<<"No cell data to write!");
+    return 1;
+    }
+
+  *fp << "VERTEX_DATA " << numVertices << "\n";
+  //
+  // Write scalar data
+  //
+  if( scalars )
+    {
+    if ( ! this->WriteScalarData(fp, scalars, numVertices) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write vector data
+  //
+  if( vectors )
+    {
+    if ( ! this->WriteVectorData(fp, vectors, numVertices) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write normals
+  //
+  if ( normals )
+    {
+    if ( ! this->WriteNormalData(fp, normals, numVertices) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write texture coords
+  //
+  if ( tcoords )
+    {
+    if ( ! this->WriteTCoordData(fp, tcoords, numVertices) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write tensors
+  //
+  if ( tensors )
+    {
+    if ( ! this->WriteTensorData(fp, tensors, numVertices) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write global ids
+  //
+  if ( globalIds )
+    {
+    if ( ! this->WriteGlobalIdData(fp, globalIds, numVertices) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write pedigree ids
+  //
+  if ( pedigreeIds )
+    {
+    if ( ! this->WritePedigreeIdData(fp, pedigreeIds, numVertices) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write field
+  //
+  if ( field )
+    {
+    if ( ! this->WriteFieldData(fp, field) )
+      {
+      return 0;
+      }
+    }
+
+  return 1;
+}
+
+// Write the edge data (e.g., scalars, vectors, ...) of a vtk graph.
+// Returns 0 if error.
+int vtkDataWriter::WriteEdgeData(ostream *fp, vtkGraph *g)
+{
+  int numEdges;
+  vtkDataArray *scalars;
+  vtkDataArray *vectors;
+  vtkDataArray *normals;
+  vtkDataArray *tcoords;
+  vtkDataArray *tensors;
+  vtkDataArray *globalIds;
+  vtkAbstractArray *pedigreeIds;
+  vtkFieldData *field;
+  vtkDataSetAttributes *cd=g->GetEdgeData();
+
+  vtkDebugMacro(<<"Writing edge data...");
+
+  numEdges = g->GetNumberOfEdges();
+  if(numEdges <= 0)
+    {
+    vtkDebugMacro(<<"No edge data to write!");
+    return 1;
+    }
+    
+  scalars = cd->GetScalars();
+  if(scalars && scalars->GetNumberOfTuples() <= 0)
+    scalars = 0;
+    
+  vectors = cd->GetVectors();
+  if(vectors && vectors->GetNumberOfTuples() <= 0)
+    vectors = 0;
+    
+  normals = cd->GetNormals();
+  if(normals && normals->GetNumberOfTuples() <= 0)
+    normals = 0;
+    
+  tcoords = cd->GetTCoords();
+  if(tcoords && tcoords->GetNumberOfTuples() <= 0)
+    tcoords = 0;
+    
+  tensors = cd->GetTensors();
+  if(tensors && tensors->GetNumberOfTuples() <= 0)
+    tensors = 0;
+    
+  globalIds = cd->GetGlobalIds();
+  if(globalIds && globalIds->GetNumberOfTuples() <= 0)
+    globalIds = 0;
+    
+  pedigreeIds = cd->GetPedigreeIds();
+  if(pedigreeIds && pedigreeIds->GetNumberOfTuples() <= 0)
+    pedigreeIds = 0;
+    
+  field = cd;
+  if(field && field->GetNumberOfTuples() <= 0)
+    field = 0;
+
+  if(!(scalars || vectors || normals || tcoords || tensors || globalIds || pedigreeIds || field))
+    {
+    vtkDebugMacro(<<"No edge data to write!");
+    return 1;
+    }
+
+  *fp << "EDGE_DATA " << numEdges << "\n";
+  //
+  // Write scalar data
+  //
+  if( scalars )
+    {
+    if ( ! this->WriteScalarData(fp, scalars, numEdges) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write vector data
+  //
+  if( vectors )
+    {
+    if ( ! this->WriteVectorData(fp, vectors, numEdges) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write normals
+  //
+  if ( normals )
+    {
+    if ( ! this->WriteNormalData(fp, normals, numEdges) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write texture coords
+  //
+  if ( tcoords )
+    {
+    if ( ! this->WriteTCoordData(fp, tcoords, numEdges) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write tensors
+  //
+  if ( tensors )
+    {
+    if ( ! this->WriteTensorData(fp, tensors, numEdges) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write global ids
+  //
+  if ( globalIds )
+    {
+    if ( ! this->WriteGlobalIdData(fp, globalIds, numEdges) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write pedigree ids
+  //
+  if ( pedigreeIds )
+    {
+    if ( ! this->WritePedigreeIdData(fp, pedigreeIds, numEdges) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write field
+  //
+  if ( field )
+    {
+    if ( ! this->WriteFieldData(fp, field) )
+      {
+      return 0;
+      }
+    }
+
+  return 1;
+}
+
+// Write the row data (e.g., scalars, vectors, ...) of a vtk table.
+// Returns 0 if error.
+int vtkDataWriter::WriteRowData(ostream *fp, vtkTable *t)
+{
+  int numRows;
+  vtkDataArray *scalars;
+  vtkDataArray *vectors;
+  vtkDataArray *normals;
+  vtkDataArray *tcoords;
+  vtkDataArray *tensors;
+  vtkDataArray *globalIds;
+  vtkAbstractArray *pedigreeIds;
+  vtkFieldData *field;
+  vtkDataSetAttributes *cd=t->GetRowData();
+
+  numRows = t->GetNumberOfRows();
+
+  vtkDebugMacro(<<"Writing row data...");
+
+  scalars = cd->GetScalars();
+  if(scalars && scalars->GetNumberOfTuples() <= 0)
+    scalars = 0;
+    
+  vectors = cd->GetVectors();
+  if(vectors && vectors->GetNumberOfTuples() <= 0)
+    vectors = 0;
+    
+  normals = cd->GetNormals();
+  if(normals && normals->GetNumberOfTuples() <= 0)
+    normals = 0;
+    
+  tcoords = cd->GetTCoords();
+  if(tcoords && tcoords->GetNumberOfTuples() <= 0)
+    tcoords = 0;
+    
+  tensors = cd->GetTensors();
+  if(tensors && tensors->GetNumberOfTuples() <= 0)
+    tensors = 0;
+    
+  globalIds = cd->GetGlobalIds();
+  if(globalIds && globalIds->GetNumberOfTuples() <= 0)
+    globalIds = 0;
+    
+  pedigreeIds = cd->GetPedigreeIds();
+  if(pedigreeIds && pedigreeIds->GetNumberOfTuples() <= 0)
+    pedigreeIds = 0;
+    
+  field = cd;
+  if(field && field->GetNumberOfTuples() <= 0)
+    field = 0;
+
+  if(!(scalars || vectors || normals || tcoords || tensors || globalIds || pedigreeIds || field))
+    {
+    vtkDebugMacro(<<"No row data to write!");
+    return 1;
+    }
+
+  *fp << "ROW_DATA " << numRows << "\n";
+  //
+  // Write scalar data
+  //
+  if( scalars )
+    {
+    if ( ! this->WriteScalarData(fp, scalars, numRows) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write vector data
+  //
+  if( vectors )
+    {
+    if ( ! this->WriteVectorData(fp, vectors, numRows) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write normals
+  //
+  if ( normals )
+    {
+    if ( ! this->WriteNormalData(fp, normals, numRows) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write texture coords
+  //
+  if ( tcoords )
+    {
+    if ( ! this->WriteTCoordData(fp, tcoords, numRows) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write tensors
+  //
+  if ( tensors )
+    {
+    if ( ! this->WriteTensorData(fp, tensors, numRows) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write global ids
+  //
+  if ( globalIds )
+    {
+    if ( ! this->WriteGlobalIdData(fp, globalIds, numRows) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write pedigree ids
+  //
+  if ( pedigreeIds )
+    {
+    if ( ! this->WritePedigreeIdData(fp, pedigreeIds, numRows) )
+      {
+      return 0;
+      }
+    }
+  //
+  // Write field
+  //
+  if ( field )
+    {
+    if ( ! this->WriteFieldData(fp, field) )
+      {
+      return 0;
+      }
+    }
+
+  return 1;
+}
+
 // Template to handle writing data in ascii or binary
 // We could change the format into C++ io standard ...
 template <class T>
@@ -557,20 +994,20 @@ void vtkWriteDataArray(ostream *fp, T *data, int fileType,
       switch (sizeT)
         {
         case 2:
-          // typecast doesn't have to be valid here
-          vtkByteSwap::SwapWrite2BERange((short *)data,num*numComp, fp);
+          // no typecast needed here; method call takes void* data
+          vtkByteSwap::SwapWrite2BERange(data, num*numComp, fp);
           break;
         case 4:
-          // typecast doesn't have to be valid here
-          vtkByteSwap::SwapWrite4BERange((float *)data,num*numComp, fp);
+          // no typecast needed here; method call takes void* data
+          vtkByteSwap::SwapWrite4BERange(data, num*numComp, fp);
           break;
         case 8:
-          // typecast doesn't have to be valid here
-          vtkByteSwap::SwapWrite8BERange((double *)data,num*numComp, fp);
+          // no typecast needed here; method call takes void* data
+          vtkByteSwap::SwapWrite8BERange(data, num*numComp, fp);
           break;
         default:
-          fp->write((char *)data, ( sizeof(T))*( num*numComp));
-
+          fp->write(reinterpret_cast<char*>(data), sizeof(T) * (num*numComp));
+          break;
         }
       }
     }
@@ -598,7 +1035,7 @@ int vtkDataWriter::WriteArray(ostream *fp, int dataType, vtkAbstractArray *data,
           for (i=0; i<numComp; i++)
             {
             idx = i + j*numComp;
-            s = ((vtkBitArray *)data)->GetValue(idx);
+            s = static_cast<vtkBitArray *>(data)->GetValue(idx);
             *fp << (s!=0.0?1:0); 
             if ( !((idx+1)%8) )
               {
@@ -613,8 +1050,10 @@ int vtkDataWriter::WriteArray(ostream *fp, int dataType, vtkAbstractArray *data,
         }
       else
         {
-        unsigned char *cptr=((vtkUnsignedCharArray *)data)->GetPointer(0);
-        fp->write((char *)cptr, (sizeof(unsigned char))*((num-1)/8+1));
+        unsigned char *cptr=
+          static_cast<vtkUnsignedCharArray *>(data)->GetPointer(0);
+        fp->write(reinterpret_cast<char *>(cptr),
+                  (sizeof(unsigned char))*((num-1)/8+1));
 
         }
       *fp << "\n";
@@ -624,7 +1063,7 @@ int vtkDataWriter::WriteArray(ostream *fp, int dataType, vtkAbstractArray *data,
     case VTK_CHAR:
       {
       sprintf (str, format, "char"); *fp << str; 
-      char *s=((vtkCharArray *)data)->GetPointer(0);
+      char *s=static_cast<vtkCharArray *>(data)->GetPointer(0);
       vtkWriteDataArray(fp, s, this->FileType, "%hhd ", num, numComp);
       }
     break;
@@ -632,7 +1071,8 @@ int vtkDataWriter::WriteArray(ostream *fp, int dataType, vtkAbstractArray *data,
     case VTK_UNSIGNED_CHAR:
       {
       sprintf (str, format, "unsigned_char"); *fp << str; 
-      unsigned char *s=((vtkUnsignedCharArray *)data)->GetPointer(0);
+      unsigned char *s=
+        static_cast<vtkUnsignedCharArray *>(data)->GetPointer(0);
       vtkWriteDataArray(fp, s, this->FileType, "%hhu ", num, numComp);
       }
     break;
@@ -640,7 +1080,7 @@ int vtkDataWriter::WriteArray(ostream *fp, int dataType, vtkAbstractArray *data,
     case VTK_SHORT:
       {
       sprintf (str, format, "short"); *fp << str; 
-      short *s=((vtkShortArray *)data)->GetPointer(0);
+      short *s=static_cast<vtkShortArray *>(data)->GetPointer(0);
       vtkWriteDataArray(fp, s, this->FileType, "%hd ", num, numComp);
       }
     break;
@@ -648,7 +1088,8 @@ int vtkDataWriter::WriteArray(ostream *fp, int dataType, vtkAbstractArray *data,
     case VTK_UNSIGNED_SHORT:
       {
       sprintf (str, format, "unsigned_short"); *fp << str; 
-      unsigned short *s=((vtkUnsignedShortArray *)data)->GetPointer(0);
+      unsigned short *s=
+        static_cast<vtkUnsignedShortArray *>(data)->GetPointer(0);
       vtkWriteDataArray(fp, s, this->FileType, "%hu ", num, numComp);
       }
     break;
@@ -656,7 +1097,7 @@ int vtkDataWriter::WriteArray(ostream *fp, int dataType, vtkAbstractArray *data,
     case VTK_INT:
       {
       sprintf (str, format, "int"); *fp << str; 
-      int *s=((vtkIntArray *)data)->GetPointer(0);
+      int *s=static_cast<vtkIntArray *>(data)->GetPointer(0);
       vtkWriteDataArray(fp, s, this->FileType, "%d ", num, numComp);
       }
     break;
@@ -664,7 +1105,7 @@ int vtkDataWriter::WriteArray(ostream *fp, int dataType, vtkAbstractArray *data,
     case VTK_UNSIGNED_INT:
       {
       sprintf (str, format, "unsigned_int"); *fp << str; 
-      unsigned int *s=((vtkUnsignedIntArray *)data)->GetPointer(0);
+      unsigned int *s=static_cast<vtkUnsignedIntArray *>(data)->GetPointer(0);
       vtkWriteDataArray(fp, s, this->FileType, "%u ", num, numComp);
       }
     break;
@@ -672,7 +1113,7 @@ int vtkDataWriter::WriteArray(ostream *fp, int dataType, vtkAbstractArray *data,
     case VTK_LONG:
       {
       sprintf (str, format, "long"); *fp << str; 
-      long *s=((vtkLongArray *)data)->GetPointer(0);
+      long *s=static_cast<vtkLongArray *>(data)->GetPointer(0);
       vtkWriteDataArray(fp, s, this->FileType, "%ld ", num, numComp);
       }
     break;
@@ -680,7 +1121,8 @@ int vtkDataWriter::WriteArray(ostream *fp, int dataType, vtkAbstractArray *data,
     case VTK_UNSIGNED_LONG:
       {
       sprintf (str, format, "unsigned_long"); *fp << str; 
-      unsigned long *s=((vtkUnsignedLongArray *)data)->GetPointer(0);
+      unsigned long *s=
+        static_cast<vtkUnsignedLongArray *>(data)->GetPointer(0);
       vtkWriteDataArray(fp, s, this->FileType, "%lu ", num, numComp);
       }
     break;
@@ -699,7 +1141,8 @@ int vtkDataWriter::WriteArray(ostream *fp, int dataType, vtkAbstractArray *data,
     case VTK_UNSIGNED___INT64:
       {
       sprintf (str, format, "vtktypeuint64"); *fp << str; 
-      unsigned __int64 *s= static_cast<unsigned __int64*>(data->GetVoidPointer(0));
+      unsigned __int64 *s=
+        static_cast<unsigned __int64*>(data->GetVoidPointer(0));
       strcpy(outputFormat, vtkTypeTraits<unsigned __int64>::ParseFormat());
       strcat(outputFormat, " ");
       vtkWriteDataArray(fp, s, this->FileType, outputFormat, num, numComp);
@@ -721,7 +1164,8 @@ int vtkDataWriter::WriteArray(ostream *fp, int dataType, vtkAbstractArray *data,
     case VTK_UNSIGNED_LONG_LONG:
       {
       sprintf (str, format, "vtktypeuint64"); *fp << str; 
-      unsigned long long *s= static_cast<unsigned long long*>(data->GetVoidPointer(0));
+      unsigned long long *s=
+        static_cast<unsigned long long*>(data->GetVoidPointer(0));
       strcpy(outputFormat, vtkTypeTraits<unsigned long long>::ParseFormat());
       strcat(outputFormat, " ");
       vtkWriteDataArray(fp, s, this->FileType, outputFormat, num, numComp);
@@ -732,7 +1176,7 @@ int vtkDataWriter::WriteArray(ostream *fp, int dataType, vtkAbstractArray *data,
     case VTK_FLOAT:
       {
       sprintf (str, format, "float"); *fp << str; 
-      float *s=((vtkFloatArray *)data)->GetPointer(0);
+      float *s=static_cast<vtkFloatArray *>(data)->GetPointer(0);
       vtkWriteDataArray(fp, s, this->FileType, "%g ", num, numComp);
       }
     break;
@@ -740,7 +1184,7 @@ int vtkDataWriter::WriteArray(ostream *fp, int dataType, vtkAbstractArray *data,
     case VTK_DOUBLE:
       {
       sprintf (str, format, "double"); *fp << str; 
-      double *s=((vtkDoubleArray *)data)->GetPointer(0);
+      double *s=static_cast<vtkDoubleArray *>(data)->GetPointer(0);
       vtkWriteDataArray(fp, s, this->FileType, "%lg ", num, numComp);
       }
     break;
@@ -751,7 +1195,7 @@ int vtkDataWriter::WriteArray(ostream *fp, int dataType, vtkAbstractArray *data,
       int size = data->GetNumberOfTuples();
       int *intArray = new int[size*numComp];
       sprintf (str, format, "vtkIdType"); *fp << str; 
-      vtkIdType *s=((vtkIdTypeArray *)data)->GetPointer(0);
+      vtkIdType *s=static_cast<vtkIdTypeArray *>(data)->GetPointer(0);
       for (i = 0; i < size*numComp; i++)
         {
         intArray[i] = s[i];
@@ -772,7 +1216,7 @@ int vtkDataWriter::WriteArray(ostream *fp, int dataType, vtkAbstractArray *data,
           for (i=0; i<numComp; i++)
             {
             idx = i + j*numComp;
-            s = ((vtkStringArray *)data)->GetValue(idx);
+            s = static_cast<vtkStringArray *>(data)->GetValue(idx);
             this->EncodeWriteString(fp, s.c_str(), false);
             *fp << "\n";
             }
@@ -786,21 +1230,24 @@ int vtkDataWriter::WriteArray(ostream *fp, int dataType, vtkAbstractArray *data,
           for (i=0; i<numComp; i++)
             {
             idx = i + j*numComp;
-            s = ((vtkStringArray *)data)->GetValue(idx);
+            s = static_cast<vtkStringArray *>(data)->GetValue(idx);
             vtkTypeUInt64 length = s.length();
             if (length < (static_cast<vtkTypeUInt64>(1) << 6))
               {
-              vtkTypeUInt8 len = (static_cast<vtkTypeUInt8>(3) << 6) | static_cast<vtkTypeUInt8>(length);
-              fp->write((char*)(&len), 1);
+              vtkTypeUInt8 len = (static_cast<vtkTypeUInt8>(3) << 6)
+                | static_cast<vtkTypeUInt8>(length);
+              fp->write(reinterpret_cast<char*>(&len), 1);
               }
             else if (length < (static_cast<vtkTypeUInt64>(1) << 14))
               {
-              vtkTypeUInt16 len = (static_cast<vtkTypeUInt16>(2) << 14) | static_cast<vtkTypeUInt16>(length);
+              vtkTypeUInt16 len = (static_cast<vtkTypeUInt16>(2) << 14)
+                | static_cast<vtkTypeUInt16>(length);
               vtkByteSwap::SwapWrite2BERange(&len, 1, fp);
               }
             else if (length < (static_cast<vtkTypeUInt64>(1) << 30))
               {
-              vtkTypeUInt32 len = (static_cast<vtkTypeUInt32>(1) << 30) | static_cast<vtkTypeUInt32>(length);
+              vtkTypeUInt32 len = (static_cast<vtkTypeUInt32>(1) << 30)
+                | static_cast<vtkTypeUInt32>(length);
               vtkByteSwap::SwapWrite4BERange(&len, 1, fp);
               }
             else
@@ -815,6 +1262,19 @@ int vtkDataWriter::WriteArray(ostream *fp, int dataType, vtkAbstractArray *data,
       }
     break;
 
+    case VTK_VARIANT:
+      {
+      sprintf (str, format, "variant"); *fp << str; 
+      vtkVariant *v=static_cast<vtkVariantArray *>(data)->GetPointer(0);
+      for (j = 0; j < num*numComp; j++)
+        {
+        *fp << v[j].GetType() << " ";
+        this->EncodeWriteString(fp, v[j].ToString().c_str(), false);
+        *fp << endl;
+        }
+      }
+    break;
+
     default:
       {
       vtkErrorMacro(<<"Type currently not supported");
@@ -880,7 +1340,7 @@ int vtkDataWriter::WriteCoordinates(ostream *fp, vtkDataArray *coords,
 int vtkDataWriter::WriteScalarData(ostream *fp, vtkDataArray *scalars, int num)
 {
   int i, j, size=0;
-  char *name;
+  const char *name;
   vtkLookupTable *lut;
   int dataType = scalars->GetDataType();
   int numComp = scalars->GetNumberOfComponents();
@@ -888,7 +1348,7 @@ int vtkDataWriter::WriteScalarData(ostream *fp, vtkDataArray *scalars, int num)
   if ( (lut=scalars->GetLookupTable()) == NULL || 
        (size = lut->GetNumberOfColors()) <= 0 )
     {
-    name = (char *) "default";
+    name = "default";
     }
   else 
     {
@@ -944,7 +1404,8 @@ int vtkDataWriter::WriteScalarData(ostream *fp, vtkDataArray *scalars, int num)
   else //color scalars
     {
     int nvs = scalars->GetNumberOfComponents();
-    unsigned char *data=((vtkUnsignedCharArray *)scalars)->GetPointer(0);
+    unsigned char *data=
+      static_cast<vtkUnsignedCharArray *>(scalars)->GetPointer(0);
     *fp << "COLOR_SCALARS " << scalarsName << " " << nvs << "\n";
 
     if ( this->FileType == VTK_ASCII )
@@ -953,7 +1414,7 @@ int vtkDataWriter::WriteScalarData(ostream *fp, vtkDataArray *scalars, int num)
         {
         for (j=0; j<nvs; j++)
           {
-          *fp << ((float)data[nvs*i+j]/255.0) << " ";
+          *fp << (static_cast<float>(data[nvs*i+j])/255.0) << " ";
           }
         if ( i != 0 && !(i%2) )
           {
@@ -963,8 +1424,8 @@ int vtkDataWriter::WriteScalarData(ostream *fp, vtkDataArray *scalars, int num)
       }
     else // binary type
       {
-      fp->write((char *)data, (sizeof(unsigned char))*(nvs*num));
-
+      fp->write(reinterpret_cast<char *>(data),
+                (sizeof(unsigned char))*(nvs*num));
       }
 
     *fp << "\n";
@@ -988,7 +1449,8 @@ int vtkDataWriter::WriteScalarData(ostream *fp, vtkDataArray *scalars, int num)
     else
       {
       unsigned char *colors=lut->GetPointer(0);
-      fp->write((char *)colors, (sizeof(unsigned char)*4*size));
+      fp->write(reinterpret_cast<char *>(colors),
+                (sizeof(unsigned char)*4*size));
       }
     *fp << "\n";
     }
@@ -1328,11 +1790,11 @@ int vtkDataWriter::WriteCells(ostream *fp, vtkCellArray *cells, const char *labe
     for (cells->InitTraversal(); cells->GetNextCell(npts,pts); )
       {
       // currently writing vtkIdType as int
-      *fp << (int)npts << " ";
+      *fp << static_cast<int>(npts) << " ";
       for (j=0; j<npts; j++)
         {
         // currently writing vtkIdType as int
-        *fp << (int)pts[j] << " ";
+        *fp << static_cast<int>(pts[j]) << " ";
         }
       *fp << "\n";
       }
@@ -1376,30 +1838,19 @@ void vtkDataWriter::WriteData()
 void vtkDataWriter::CloseVTKFile(ostream *fp)
 {
   vtkDebugMacro(<<"Closing vtk file\n");
- 
+
   if ( fp != NULL )
     {
     if (this->WriteToOutputString)
       {
-      vtksys_ios::ostringstream *ostr = (vtksys_ios::ostringstream*)(fp);
+      vtksys_ios::ostringstream *ostr =
+        static_cast<vtksys_ios::ostringstream*>(fp);
 
-      if (this->OutputString &&
-          static_cast<unsigned int>(this->OutputStringAllocatedLength) > ostr->str().size())
-        {
-        this->OutputStringLength = ostr->str().size();
-        memcpy(this->OutputString, ostr->str().c_str(), 
-          this->OutputStringLength);
-        }
-      else
-        {
-        if (this->OutputString)
-          {
-          this->OutputString[0] = 0;
-          }
-        this->OutputStringLength = 0;
-
-        vtkErrorMacro("OutputString allocated buffer is not large enough.");
-        }
+      delete [] this->OutputString;
+      this->OutputStringLength = static_cast<int>(ostr->str().size());
+      this->OutputString = new char[ostr->str().size()];
+      memcpy(this->OutputString, ostr->str().c_str(), 
+        this->OutputStringLength);
       }
     delete fp;
     }
@@ -1411,7 +1862,6 @@ char *vtkDataWriter::RegisterAndGetOutputString()
   
   this->OutputString = NULL;
   this->OutputStringLength = 0;
-  this->OutputStringAllocatedLength = 0;
   
   return tmp;
 }
@@ -1443,7 +1893,7 @@ void vtkDataWriter::PrintSelf(ostream& os, vtkIndent indent)
     }
   
   os << indent << "Output String Length: " << this->OutputStringLength << "\n";
-  os << indent << "Output String (addr): " << ((void *)this->OutputString) << "\n";
+  os << indent << "Output String (addr): " << static_cast<void *>(this->OutputString) << "\n";
   os << indent << "WriteToOutputString: " << (this->WriteToOutputString ? "On\n" : "Off\n");
 
   if ( this->ScalarsName )
