@@ -33,7 +33,7 @@
 #include <vtkstd/string>
 #include <vtkstd/vector>
 
-vtkCxxRevisionMacro(vtkEnSightReader, "$Revision: 1.71 $");
+vtkCxxRevisionMacro(vtkEnSightReader, "$Revision: 1.82 $");
 
 //----------------------------------------------------------------------------
 typedef vtkstd::vector< vtkSmartPointer<vtkIdList> > vtkEnSightReaderCellIdsTypeBase;
@@ -228,7 +228,7 @@ int vtkEnSightReader::RequestData(
     {
     timeStep = timeStepInFile = 1;
     fileNum = 1;
-    fileName = new char[strlen(this->GeometryFileName) + 1];
+    fileName = new char[strlen(this->GeometryFileName) + 10];
     strcpy(fileName, this->GeometryFileName);
     
     if (this->UseTimeSets)
@@ -266,8 +266,8 @@ int vtkEnSightReader::RequestData(
         if (this->UseFileSets)
           {
           fileSet = this->FileSets->IsId(this->GeometryFileSet);
-          numStepsList = (vtkIdList*)this->FileSetNumberOfSteps->
-            GetItemAsObject(fileSet);
+          numStepsList = static_cast<vtkIdList*>(this->FileSetNumberOfSteps->
+                                                 GetItemAsObject(fileSet));
           
           if (timeStep > numStepsList->GetId(0))
             {
@@ -312,7 +312,7 @@ int vtkEnSightReader::RequestData(
     {
     timeStep = timeStepInFile = 1;
     fileNum = 1;
-    fileName = new char[strlen(this->MeasuredFileName) + 1];
+    fileName = new char[strlen(this->MeasuredFileName) + 10];
     strcpy(fileName, this->MeasuredFileName);
     
     if (this->UseTimeSets)
@@ -350,8 +350,8 @@ int vtkEnSightReader::RequestData(
         if (this->UseFileSets)
           {
           fileSet = this->FileSets->IsId(this->MeasuredFileSet);
-          numStepsList = (vtkIdList*)this->FileSetNumberOfSteps->
-            GetItemAsObject(fileSet);
+          numStepsList = static_cast<vtkIdList*>(this->FileSetNumberOfSteps->
+                                                 GetItemAsObject(fileSet));
           
           if (timeStep > numStepsList->GetId(0))
             {
@@ -436,7 +436,7 @@ int vtkEnSightReader::RequestInformation(
     vtkstd::vector<double> uniqueTimeValues(
       timeValues.begin(),
       vtkstd::unique(timeValues.begin(), timeValues.end()));
-    int numTimeValues = uniqueTimeValues.size();
+    int numTimeValues = static_cast<int>(uniqueTimeValues.size());
     if (numTimeValues > 0)
       {
       vtkInformation* outInfo = outputVector->GetInformationObject(0);
@@ -1220,7 +1220,7 @@ int vtkEnSightReader::ReadVariableFiles(vtkMultiBlockDataSet *output)
   vtkDataArray *times;
   float newTime;
   vtkIdList *numStepsList, *filenameNumbers;
-  //int fileNum;
+  int fileNum;
   int validTime, filenameNum;
   char* fileName, *fileName2;
   
@@ -1250,9 +1250,9 @@ int vtkEnSightReader::ReadVariableFiles(vtkMultiBlockDataSet *output)
     
     timeStep = 0;
     timeStepInFile = 1;
-    //fileNum = 1;
+    fileNum = 0;
     validTime = 1;
-    fileName = new char[strlen(this->VariableFileNames[i]) + 1];
+    fileName = new char[strlen(this->VariableFileNames[i]) + 10];
     strcpy(fileName, this->VariableFileNames[i]);
     if (this->UseTimeSets)
       {
@@ -1299,20 +1299,21 @@ int vtkEnSightReader::ReadVariableFiles(vtkMultiBlockDataSet *output)
         {
         timeStepInFile = timeStep;
         fileSet = this->VariableFileSetIds->GetId(i);
-        numStepsList = (vtkIdList*)this->FileSetNumberOfSteps->
-          GetItemAsObject(this->FileSets->IsId(fileSet));
+        numStepsList = static_cast<vtkIdList*>(this->FileSetNumberOfSteps->
+                               GetItemAsObject(this->FileSets->IsId(fileSet)));
 
         if (timeStep > numStepsList->GetId(0))
           {
           numSteps = numStepsList->GetId(0);
           timeStepInFile -= numSteps;
-          for (i = 1; i < numStepsList->GetNumberOfIds(); i++)
+          fileNum = 1;
+          for (j = 1; j < numStepsList->GetNumberOfIds(); j++)
             {
-            numSteps += numStepsList->GetId(i);
+            numSteps += numStepsList->GetId(j);
             if (timeStep > numSteps)
               {
-              //fileNum++;
-              timeStepInFile -= numStepsList->GetId(i);
+              fileNum++;
+              timeStepInFile -= numStepsList->GetId(j);
               }
             }
           }
@@ -1324,7 +1325,7 @@ int vtkEnSightReader::ReadVariableFiles(vtkMultiBlockDataSet *output)
             {
             filenameNumbers = this->FileSetFileNameNumbers->
               GetItem(collectionNum);
-            filenameNum = filenameNumbers->GetId(timeStep-1);
+            filenameNum = filenameNumbers->GetId(fileNum);
             this->ReplaceWildcards(fileName, filenameNum);
             }
           }
@@ -1392,11 +1393,11 @@ int vtkEnSightReader::ReadVariableFiles(vtkMultiBlockDataSet *output)
       }
     timeStep = 0;
     timeStepInFile = 1;
-    //fileNum = 1;
+    fileNum = 0;
     validTime = 1;
-    fileName = new char[strlen(this->ComplexVariableFileNames[2*i]) + 1];
+    fileName = new char[strlen(this->ComplexVariableFileNames[2*i]) + 10];
     strcpy(fileName, this->ComplexVariableFileNames[2*i]);
-    fileName2 = new char[strlen(this->ComplexVariableFileNames[2*i+1]) + 1];
+    fileName2 = new char[strlen(this->ComplexVariableFileNames[2*i+1]) + 10];
     strcpy(fileName2, this->ComplexVariableFileNames[2*i+1]);
     if (this->UseTimeSets)
       {
@@ -1442,20 +1443,21 @@ int vtkEnSightReader::ReadVariableFiles(vtkMultiBlockDataSet *output)
         {
         timeStepInFile = timeStep;
         fileSet = this->VariableFileSetIds->GetId(i);
-        numStepsList = (vtkIdList*)this->FileSetNumberOfSteps->
-          GetItemAsObject(this->FileSets->IsId(fileSet));
+        numStepsList = static_cast<vtkIdList*>(this->FileSetNumberOfSteps->
+                                               GetItemAsObject(this->FileSets->IsId(fileSet)));
 
         if (timeStep > numStepsList->GetId(0))
           {
           numSteps = numStepsList->GetId(0);
           timeStepInFile -= numSteps;
-          for (i = 1; i < numStepsList->GetNumberOfIds(); i++)
+          fileNum = 1;
+          for (j = 1; j < numStepsList->GetNumberOfIds(); j++)
             {
-            numSteps += numStepsList->GetId(i);
+            numSteps += numStepsList->GetId(j);
             if (timeStep > numSteps)
               {
-              //fileNum++;
-              timeStepInFile -= numStepsList->GetId(i);
+              fileNum++;
+              timeStepInFile -= numStepsList->GetId(j);
               }
             }
           }
@@ -1797,6 +1799,10 @@ int vtkEnSightReader::GetElementType(const char* line)
     {
     return vtkEnSightReader::QUAD8;
     }
+  else if (strncmp(line, "nfaced", 6) == 0)
+    {
+    return vtkEnSightReader::NFACED;
+    }
   else if (strncmp(line, "tetra4", 6) == 0)
     {
     return vtkEnSightReader::TETRA4;
@@ -1837,84 +1843,49 @@ int vtkEnSightReader::GetElementType(const char* line)
 
 void vtkEnSightReader::ReplaceWildcards(char* filename, int num)
 {
-  int wildcardPos, numWildcards, numDigits = 1, i;
-  int tmpNum = num, multTen = 1;
-  char newChar;
-  int newNum;
-  
+  int wildcardPos, numWildcards, i, j;
+  char pattern[32];
+  char numStr[32];
+  char filenameTmp[2048];
+  int len, cnt, numStrLen;
+  int foundWildcard = 0;
+
   wildcardPos = static_cast<int>(strcspn(filename, "*"));
   numWildcards = static_cast<int>(strspn(filename + wildcardPos, "*"));
-  
-  if (numWildcards == 0)
+
+  if (numWildcards < 1)
+     return;
+  else if (numWildcards == 1)
+    strcpy(pattern, "%d");
+  else
+    sprintf(pattern, "%%0%dd", numWildcards);
+  sprintf(numStr, pattern, num);
+  numStrLen = static_cast<int>(strlen(numStr));
+  len = static_cast<int>(strlen(filename));
+  cnt = 0;
+  for (i = 0 ; i < len ; i++)
     {
-    return;
-    }
-  
-  tmpNum /= 10;
-  while (tmpNum >= 1)
+    if (filename[i] == '*')
     {
-    numDigits++;
-    multTen *= 10;
-    tmpNum /= 10;
+      if (foundWildcard == 0)
+        {
+        for (j = 0 ; j < numStrLen ; j++)
+          filenameTmp[cnt++] = numStr[j];
+        foundWildcard = 1;
+        }
     }
-  
-  for (i = 0; i < numWildcards - numDigits; i++)
-    {
-    filename[i + wildcardPos] = '0';
-    }
-  
-  tmpNum = num;
-  for (i = numWildcards - numDigits; i < numWildcards; i++)
-    {
-    newNum = tmpNum / multTen;
-    switch (newNum)
-      {
-      case 0:
-        newChar = '0';
-        break;
-      case 1:
-        newChar = '1';
-        break;
-      case 2:
-        newChar = '2';
-        break;
-      case 3:
-        newChar = '3';
-        break;
-      case 4:
-        newChar = '4';
-        break;
-      case 5:
-        newChar = '5';
-        break;
-      case 6:
-        newChar = '6';
-        break;
-      case 7:
-        newChar = '7';
-        break;
-      case 8:
-        newChar = '8';
-        break;
-      case 9:
-        newChar = '9';
-        break;
-      default:
-        // This case should never be reached.
-        return;
-      }
-    
-    filename[i + wildcardPos] = newChar;
-    tmpNum -= multTen * newNum;
-    multTen /= 10;
+    else
+      filenameTmp[cnt++] = filename[i];
     }
+  filenameTmp[cnt] = '\0';
+  strcpy(filename, filenameTmp);
 }
 
 //----------------------------------------------------------------------------
 void vtkEnSightReader::RemoveLeadingBlanks(char *line)
 {
   int count = 0;
-  int len = strlen(line);
+  int len = static_cast<int>(strlen(line));
   while (line[count] == ' ')
     {
     count++;
@@ -1926,10 +1897,11 @@ void vtkEnSightReader::RemoveLeadingBlanks(char *line)
 vtkIdList* vtkEnSightReader::GetCellIds(int index, int cellType)
 {
   // Check argument range.
-  if(cellType < 0 || cellType > 15)
+  if(cellType < POINT || cellType >= NUMBER_OF_ELEMENT_TYPES)
     {
     vtkErrorMacro("Cell type " << cellType
-                  << " out of range.  Only 16 types exist.");
+                  << " out of range.  Only "
+                  << NUMBER_OF_ELEMENT_TYPES-1 << " types exist.");
     return 0;
     }
   if(index < 0 || index > this->UnstructuredPartIds->GetNumberOfIds())
@@ -1939,7 +1911,7 @@ vtkIdList* vtkEnSightReader::GetCellIds(int index, int cellType)
                   << " IDs exist.");
     return 0;
     }
-  
+
   // Create the container if necessary.
   if(!this->CellIds)
     {
@@ -1947,7 +1919,7 @@ vtkIdList* vtkEnSightReader::GetCellIds(int index, int cellType)
     }
   
   // Get the index of the actual vtkIdList requested.
-  unsigned int cellIdsIndex = index*16 + cellType;
+  unsigned int cellIdsIndex = index*NUMBER_OF_ELEMENT_TYPES + cellType;
   
   // Make sure the container is large enough for this index.
   if(cellIdsIndex+1 > this->CellIds->size())
@@ -1967,6 +1939,37 @@ vtkIdList* vtkEnSightReader::GetCellIds(int index, int cellType)
   return (*this->CellIds)[cellIdsIndex].GetPointer();
 }
 
+//----------------------------------------------------------------------------
+void vtkEnSightReader::AddToBlock(vtkMultiBlockDataSet* output, 
+                                  unsigned int blockNo,
+                                  vtkDataSet* dataset)
+{
+  vtkDataObject* blockDO = output->GetBlock(blockNo);
+  if (blockDO)
+    {
+    vtkErrorMacro("Block already has a vtkDataSet assigned to it.");
+    return;
+    }
+
+  output->SetBlock(blockNo, dataset);
+}
+
+
+//----------------------------------------------------------------------------
+vtkDataSet* vtkEnSightReader::GetDataSetFromBlock(
+  vtkMultiBlockDataSet* output,
+  unsigned int blockno)
+{
+  return vtkDataSet::SafeDownCast(output->GetBlock(blockno));
+}
+
+//----------------------------------------------------------------------------
+void vtkEnSightReader::SetBlockName(vtkMultiBlockDataSet* output,
+  unsigned int blockNo, const char* name)
+{
+  output->GetMetaData(blockNo)->Set(vtkCompositeDataSet::NAME(), name);
+}
+
 //----------------------------------------------------------------------------
 void vtkEnSightReader::PrintSelf(ostream& os, vtkIndent indent)
 {
