@@ -25,7 +25,7 @@
 #include "vtkUnsignedCharArray.h"
 #include "vtkDataArrayCollection.h"
 
-vtkCxxRevisionMacro(vtkPLOT3DReader, "$Revision: 1.86 $");
+vtkCxxRevisionMacro(vtkPLOT3DReader, "$Revision: 1.88 $");
 vtkStandardNewMacro(vtkPLOT3DReader);
 
 #define VTK_RHOINF 1.0
@@ -318,22 +318,6 @@ int vtkPLOT3DReader::GetNumberOfOutputs()
   return 1;
 }
 
-//----------------------------------------------------------------------------
-int vtkPLOT3DReader::GenerateDefaultConfiguration()
-{
-  FILE* xyzFp;
-  
-  if ( this->CheckGeometryFile(xyzFp) != VTK_OK)
-    {
-    return 0;
-    }
-  char buf[1024];
-  fread(buf, 1, 1024, xyzFp);
-  int retVal = this->VerifySettings(buf, 1024);
-  fclose(xyzFp);
-  return retVal;
-}
-
 //----------------------------------------------------------------------------
 void vtkPLOT3DReader::ReadIntBlockV(char** buf, int n, int* block)
 {
@@ -359,70 +343,6 @@ void vtkPLOT3DReader::SkipByteCountV(char** buf)
     }
 }
 
-//----------------------------------------------------------------------------
-int vtkPLOT3DReader::VerifySettings(char* buf, int vtkNotUsed(bufSize))
-{
-  int numGrid=0;
-
-  if ( this->MultiGrid )
-    {
-    this->SkipByteCountV(&buf);
-    this->ReadIntBlockV(&buf, 1, &numGrid);
-    this->SkipByteCountV(&buf);
-    }
-  else
-    {
-    numGrid=1;
-    }
-
-  int retVal=1;
-
-  long fileSize = 0;
-  // Size of number of grids information.
-  if ( this->MultiGrid )
-    {
-    fileSize += 4; // numGrids
-    if (this->HasByteCount)
-      {
-      fileSize += 4*4; // byte counts for the header
-      }
-    }
-
-  // Add the size of each grid.
-  this->SkipByteCountV(&buf);
-  for(int i=0; i<numGrid; i++)
-    {
-    int ni, nj, nk;
-    this->ReadIntBlockV(&buf, 1, &ni);
-    this->ReadIntBlockV(&buf, 1, &nj);
-    if (!this->TwoDimensionalGeometry)
-      {
-      this->ReadIntBlockV(&buf, 1, &nk);
-      }
-    else
-      {
-      nk = 1;
-      }
-    fileSize += this->EstimateSize(ni, nj, nk);
-    // If this number is larger than the file size, there
-    // is something wrong.
-    if ( fileSize > this->FileSize )
-      {
-      retVal = 0;
-      break;
-      }
-    }
-  this->SkipByteCountV(&buf);
-  // If this number is different than the file size, there
-  // is something wrong.
-  if ( fileSize != this->FileSize )
-    {
-    retVal = 0;
-    }
-
-  return retVal;
-}
-
 //----------------------------------------------------------------------------
 // Read the header and return the number of grids.
 int vtkPLOT3DReader::GetNumberOfOutputsInternal(FILE* xyzFp, int verify)
@@ -558,17 +478,13 @@ int vtkPLOT3DReader::ReadGeometryHeader(FILE* fp)
   this->SkipByteCount(fp);
   for(i=0; i<numGrid; i++)
     {
-    int ni, nj, nk;
+    int ni, nj, nk=1;
     this->ReadIntBlock(fp, 1, &ni);
     this->ReadIntBlock(fp, 1, &nj);
     if (!this->TwoDimensionalGeometry)
       {
       this->ReadIntBlock(fp, 1, &nk);
       }
-    else
-      {
-      nk = 1;
-      }
     vtkDebugMacro("Geometry, block " << i << " dimensions: "
                   << ni << " " << nj << " " << nk);
     this->GetOutput(i)->SetWholeExtent(0, ni-1, 0, nj-1, 0, nk-1);
@@ -600,10 +516,13 @@ int vtkPLOT3DReader::ReadQHeader(FILE* fp)
   this->SkipByteCount(fp);
   for(int i=0; i<numGrid; i++)
     {
-    int ni, nj, nk;
+    int ni, nj, nk=1;
     this->ReadIntBlock(fp, 1, &ni);
     this->ReadIntBlock(fp, 1, &nj);
-    this->ReadIntBlock(fp, 1, &nk);
+    if (!this->TwoDimensionalGeometry)
+      {
+      this->ReadIntBlock(fp, 1, &nk);
+      }
     vtkDebugMacro("Q, block " << i << " dimensions: "
                   << ni << " " << nj << " " << nk);
 
@@ -1034,7 +953,8 @@ void vtkPLOT3DReader::Execute()
       nthOutput->GetPointData()->AddArray(se);
       se->Delete();
 
-      
+      this->SkipByteCount(qFp);
+ 
       if ( this->FunctionList->GetNumberOfTuples() > 0 )
         {
         int fnum;
