@@ -21,7 +21,7 @@
 #include <ctype.h>
 #include <vtksys/ios/sstream>
 
-vtkCxxRevisionMacro(vtkXMLDataElement, "$Revision: 1.28 $");
+vtkCxxRevisionMacro(vtkXMLDataElement, "$Revision: 1.32 $");
 vtkStandardNewMacro(vtkXMLDataElement);
 
 //----------------------------------------------------------------------------
@@ -62,6 +62,37 @@ vtkXMLDataElement::~vtkXMLDataElement()
   this->SetCharacterData(0, 0);
 }
 
+//----------------------------------------------------------------------------
+void vtkXMLDataElement::RemoveAttribute(const char *name)
+{
+  if (!name || !name[0])
+    {
+    return;
+    }
+
+  // Find the attribute
+  
+  int i, j;
+  for (i = 0; i < this->NumberOfAttributes; ++i)
+    {
+    if(!strcmp(this->AttributeNames[i], name))
+      {
+      // Shift the other attributes
+      for (j = i; j < this->NumberOfAttributes - 1; ++j)
+        {
+        this->AttributeNames[j] = this->AttributeNames[j + 1];
+        this->AttributeValues[j] = this->AttributeValues[j + 1];
+        }
+      // Delete the last one
+      delete [] this->AttributeNames[this->NumberOfAttributes - 1];
+      delete [] this->AttributeValues[this->NumberOfAttributes - 1];
+      --this->NumberOfAttributes;
+      // this->AttributesSize is unchanged
+      return;
+      }
+    }
+}
+
 //----------------------------------------------------------------------------
 void vtkXMLDataElement::RemoveAllAttributes()
 {
@@ -142,9 +173,9 @@ void vtkXMLDataElement::AddCharacterData(const char* c, int length)
     {
     return;
     }
-  
+
   char* old_data = this->CharacterData;
-  int old_length = (old_data)? strlen(old_data): 0;
+  int old_length = (old_data) ? static_cast<int>(strlen(old_data)) : 0;
   int total_length = old_length + length;
   
   this->CharacterData = new char[total_length + 1];
@@ -527,6 +558,32 @@ vtkXMLDataElement* vtkXMLDataElement::LookupElementUpScope(const char* id)
   return start;
 }
 
+//----------------------------------------------------------------------------
+vtkXMLDataElement* vtkXMLDataElement::LookupElementWithName(const char* name)
+{
+  if (!name)
+    {
+    return NULL;
+    }
+
+  int i;
+  for (i = 0; i < this->NumberOfNestedElements; ++i)
+    {
+    const char *nname = this->NestedElements[i]->GetName();
+    if (nname && !strcmp(nname, name))
+      {
+      return this->NestedElements[i];
+      }
+    vtkXMLDataElement *found = 
+      this->NestedElements[i]->LookupElementWithName(name);
+    if (found)
+      {
+      return found;
+      }
+    }
+  return NULL;
+}
+
 //----------------------------------------------------------------------------
 int vtkXMLDataElement::GetScalarAttribute(const char* name, int& value)
 {
@@ -898,8 +955,14 @@ void vtkXMLDataElement::SeekInlineDataPosition(vtkXMLDataParser* parser)
     stream->clear(stream->rdstate() & ~ios::eofbit);
     stream->clear(stream->rdstate() & ~ios::failbit);
     parser->SeekG(this->GetXMLByteIndex());
-    while(stream->get(c) && (c != '>'));
-    while(stream->get(c) && this->IsSpace(c));
+    while(stream->get(c) && (c != '>'))
+      {
+      ;
+      }
+    while(stream->get(c) && this->IsSpace(c))
+      {
+      ;
+      }
     unsigned long pos = parser->TellG();
     this->InlineDataPosition = pos-1;
     }
@@ -977,7 +1040,8 @@ void vtkXMLDataElement::DeepCopy(vtkXMLDataElement *elem)
   this->SetXMLByteIndex(elem->GetXMLByteIndex());
   this->SetAttributeEncoding(elem->GetAttributeEncoding());
   const char *elem_cdata = elem->GetCharacterData();
-  this->SetCharacterData(elem_cdata, elem_cdata ? strlen(elem_cdata) : 0);
+  this->SetCharacterData(elem_cdata,
+    elem_cdata ? static_cast<int>(strlen(elem_cdata)) : 0);
 
   // Copy attributes
 
