@@ -22,7 +22,7 @@
 #include "vtkObjectFactory.h"
 #include "vtkXMLDataElement.h"
 
-vtkCxxRevisionMacro(vtkXMLDataParser, "$Revision: 1.31 $");
+vtkCxxRevisionMacro(vtkXMLDataParser, "$Revision: 1.36 $");
 vtkStandardNewMacro(vtkXMLDataParser);
 vtkCxxSetObjectMacro(vtkXMLDataParser, Compressor, vtkDataCompressor);
 
@@ -58,6 +58,9 @@ vtkXMLDataParser::vtkXMLDataParser()
 #endif
 
   this->AttributesEncoding = VTK_ENCODING_NONE;
+
+  //change default because vtk file formats store this information elsewhere
+  this->IgnoreCharacterData = 1;
 }
 
 //----------------------------------------------------------------------------
@@ -218,8 +221,14 @@ void vtkXMLDataParser::FindAppendedDataPosition()
   char c=0;
   long returnPosition = this->TellG();
   this->SeekG(this->GetXMLByteIndex());
-  while(this->Stream->get(c) && (c != '>'));
-  while(this->Stream->get(c) && this->IsSpace(c));
+  while(this->Stream->get(c) && (c != '>'))
+    {
+    ;
+    }
+  while(this->Stream->get(c) && this->IsSpace(c))
+    {
+    ;
+    }
 
   // Store the start of the appended data.  We skip the first
   // character because it is always a "_".
@@ -248,8 +257,14 @@ vtkXMLDataParser::FindInlineDataPosition(OffsetType start)
   // Scan for the start of the actual inline data.
   char c=0;
   this->SeekG(start);
-  while(this->Stream->get(c) && (c != '>'));
-  while(this->Stream->get(c) && this->IsSpace(c));
+  while(this->Stream->get(c) && (c != '>'))
+    {
+    ;
+    }
+  while(this->Stream->get(c) && this->IsSpace(c))
+    {
+    ;
+    }
 
   // Make sure some data were found.
   if(c == '<') { return 0; }
@@ -577,8 +592,8 @@ vtkXMLDataParser::ReadUncompressedData(unsigned char* data,
     return 0;
     }
 
-  // Read data in 32KB blocks and report progress.
-  const long blockSize = 32768;
+  // Read data in 2MB blocks and report progress.
+  const long blockSize = 2097152;
   long left = length;
   p = data;
   this->UpdateProgress(0);
@@ -1055,5 +1070,17 @@ void vtkXMLDataParser::FreeAsciiBuffer()
 void vtkXMLDataParser::UpdateProgress(float progress)
 {
   this->Progress = progress;
-  this->InvokeEvent(vtkCommand::ProgressEvent, &progress);
+  double dProgress=progress;
+  this->InvokeEvent(vtkCommand::ProgressEvent, &dProgress);
+}
+
+//----------------------------------------------------------------------------
+void vtkXMLDataParser::CharacterDataHandler( 
+  const char* data, int length )
+{  
+  unsigned int numOpen = this->NumberOfOpenElements;
+  if(numOpen > 0)
+    {
+    this->OpenElements[numOpen-1]->AddCharacterData(data, length);
+    }
 }
