@@ -29,7 +29,7 @@
 
 #include "assert.h"
 
-vtkCxxRevisionMacro(vtkXMLDataReader, "$Revision: 1.34 $");
+vtkCxxRevisionMacro(vtkXMLDataReader, "$Revision: 1.38 $");
 
 //----------------------------------------------------------------------------
 vtkXMLDataReader::vtkXMLDataReader()
@@ -204,10 +204,12 @@ int vtkXMLDataReader::SetUpdateExtentInfo(vtkXMLDataElement *eDSA,
 }
 
 //----------------------------------------------------------------------------
-void vtkXMLDataReader::CopyOutputInformation(vtkInformation *outInfo, int port)
-  {
+void vtkXMLDataReader::CopyOutputInformation(vtkInformation *outInfo,
+                                             int port)
+{
   vtkInformation *localInfo = 
     this->GetExecutive()->GetOutputInformation( port );
+
   if ( localInfo->Has(vtkDataObject::POINT_DATA_VECTOR()) )
     {
     outInfo->CopyEntry( localInfo, vtkDataObject::POINT_DATA_VECTOR() );
@@ -216,7 +218,7 @@ void vtkXMLDataReader::CopyOutputInformation(vtkInformation *outInfo, int port)
     {
     outInfo->CopyEntry( localInfo, vtkDataObject::CELL_DATA_VECTOR() );
     }
-  }
+}
 
 
 //----------------------------------------------------------------------------
@@ -306,8 +308,9 @@ void vtkXMLDataReader::SetupOutputData()
 {
   this->Superclass::SetupOutputData();
   
-  vtkPointData* pointData = this->GetOutputAsDataSet(0)->GetPointData();
-  vtkCellData* cellData = this->GetOutputAsDataSet(0)->GetCellData();
+  vtkDataSet* output = vtkDataSet::SafeDownCast(this->GetCurrentOutput());
+  vtkPointData* pointData = output->GetPointData();
+  vtkCellData* cellData = output->GetCellData();
   
   // Get the size of the output arrays.
   unsigned long pointTuples = this->GetNumberOfPoints();
@@ -391,7 +394,7 @@ void vtkXMLDataReader::SetupOutputData()
     for(int i=0; i<this->NumberOfPointArrays;i++)
       {
       this->PointDataTimeStep[i] = -1;
-      this->PointDataOffset[i] = (unsigned long)-1;
+      this->PointDataOffset[i] = static_cast<unsigned long>(-1);
       }
     }
   if( this->NumberOfCellArrays )
@@ -409,7 +412,7 @@ void vtkXMLDataReader::SetupOutputData()
     for(int i=0; i<this->NumberOfCellArrays;i++)
       {
       this->CellDataTimeStep[i] = -1;
-      this->CellDataOffset[i]   = (unsigned long)-1;
+      this->CellDataOffset[i]   = static_cast<unsigned long>(-1);
       }
     }
 }
@@ -451,8 +454,10 @@ int vtkXMLDataReader::ReadPieceData(int piece)
 //----------------------------------------------------------------------------
 int vtkXMLDataReader::ReadPieceData()
 {
-  vtkPointData* pointData = this->GetOutputAsDataSet(0)->GetPointData();
-  vtkCellData* cellData = this->GetOutputAsDataSet(0)->GetCellData();
+  vtkDataSet* output = vtkDataSet::SafeDownCast(this->GetCurrentOutput());
+
+  vtkPointData* pointData = output->GetPointData();
+  vtkCellData* cellData = output->GetCellData();
   vtkXMLDataElement* ePointData = this->PointDataElements[this->Piece];
   vtkXMLDataElement* eCellData = this->CellDataElements[this->Piece];
   
@@ -527,7 +532,7 @@ int vtkXMLDataReader::ReadPieceData()
           if(!this->ReadArrayForCells(eNested, cellData->GetAbstractArray(a++)))
             {
             vtkErrorMacro("Cannot read cell data array \""
-              << cellData->GetArray(a-1)->GetName() << "\" from "
+              << cellData->GetAbstractArray(a-1)->GetName() << "\" from "
               << ePointData->GetName() << " in piece " << this->Piece
               << ".  The data array in the element may be too short.");
             return 0;
@@ -554,7 +559,7 @@ void vtkXMLDataReader::ReadXMLData()
   if (this->FieldDataElement) // read the field data information
     {
     int i, numTuples;
-    vtkFieldData *fieldData = this->GetOutputDataObject(0)->GetFieldData();
+    vtkFieldData *fieldData = this->GetCurrentOutput()->GetFieldData();
     for(i=0; i < this->FieldDataElement->GetNumberOfNestedElements() &&
              !this->AbortExecute; i++)
       {
