@@ -33,8 +33,48 @@
 #include <sys/stat.h>
 #include <assert.h>
 
-vtkCxxRevisionMacro(vtkXMLReader, "$Revision: 1.47 $");
+vtkCxxRevisionMacro(vtkXMLReader, "$Revision: 1.54 $");
+//-----------------------------------------------------------------------------
+static void ReadStringVersion(const char* version, int& major, int& minor)
+{
+  if (!version)
+    {
+    major = -1;
+    minor = -1;
+    return;
+    }
+  // Extract the major and minor version numbers.
+  size_t length = strlen(version);
+  const char* begin = version;
+  const char* end = version + length;
+  const char* s;
 
+  for(s=begin; (s != end) && (*s != '.'); ++s)
+    {
+    ;
+    }
+
+  if(s > begin)
+    {
+    vtksys_ios::stringstream str;
+    str.write(begin, s-begin);
+    str >> major;
+    if(!str)
+      {
+      major = 0;
+      }
+    }
+  if(++s < end)
+    {
+    vtksys_ios::stringstream str;
+    str.write(s, end-s);
+    str >> minor;
+    if(!str)
+      {
+      minor = 0;
+      }
+    }
+}
 //----------------------------------------------------------------------------
 vtkXMLReader::vtkXMLReader()
 {
@@ -48,7 +88,6 @@ vtkXMLReader::vtkXMLReader()
   this->InformationError = 0;
   this->DataError = 0;
   this->ReadError = 0;
-  this->CurrentOutput = -1;
   this->ProgressRange[0] = 0;
   this->ProgressRange[1] = 1;
   
@@ -77,6 +116,11 @@ vtkXMLReader::vtkXMLReader()
   this->TimeSteps = 0;
   this->CurrentTimeStep = 0;
   this->TimeStepWasReadOnce = 0;
+
+  this->FileMinorVersion = -1;
+  this->FileMajorVersion = -1;
+  
+  this->CurrentOutput = 0;
 }
 
 //----------------------------------------------------------------------------
@@ -288,7 +332,6 @@ int vtkXMLReader::ReadXMLInformation()
     // reported by OpenVTKFile.
     if(!this->OpenVTKFile())
       {
-      this->SetupEmptyOutput();
       return 0;
       }
 
@@ -318,7 +361,6 @@ int vtkXMLReader::ReadXMLInformation()
       // The output should be empty to prevent the rest of the pipeline
       // from executing.
       this->ReadError = 1;
-      this->SetupEmptyOutput();
       }
 
     // Close the file to prevent resource leaks.
@@ -393,24 +435,16 @@ int vtkXMLReader
 }
 
 //----------------------------------------------------------------------------
-int vtkXMLReader::RequestData(vtkInformation *request, 
+int vtkXMLReader::RequestData(vtkInformation *vtkNotUsed(request),
                               vtkInformationVector **vtkNotUsed(inputVector),
                               vtkInformationVector *outputVector)
 {
-  // Set which output we are updating.  If the given object is not one
-  // of our outputs, just initialize it to empty and return.
-  this->CurrentOutput = request->Get( vtkDemandDrivenPipeline::FROM_OUTPUT_PORT() );
-  if(this->CurrentOutput < 0)
-    {
-    this->GetExecutive()->GetOutputData(0)->Initialize();
-    return 0;
-    }
-
   this->CurrentTimeStep = this->TimeStep;
 
   // Get the output pipeline information and data object.
   vtkInformation* outInfo = outputVector->GetInformationObject(0);
   vtkDataObject* output = outInfo->Get(vtkDataObject::DATA_OBJECT());
+  this->CurrentOutput = output;
 
   // Save the time value in the output data information.
   double* steps =
@@ -456,6 +490,7 @@ int vtkXMLReader::RequestData(vtkInformation *request,
   if(!this->OpenVTKFile())
     {
     this->SetupEmptyOutput();
+    this->CurrentOutput = 0;
     return 0;
     }
   if(!this->XMLParser)
@@ -487,13 +522,13 @@ int vtkXMLReader::RequestData(vtkInformation *request,
     // If we aborted or there was an error, provide empty output.
     if(this->DataError || this->AbortExecute)
       {
-      this->GetOutputAsDataSet(this->CurrentOutput)->Initialize();
+      this->SetupEmptyOutput();
       }
     }
   else
     {
     // There was an error reading the file.  Provide empty output.
-    this->GetOutputAsDataSet(this->CurrentOutput)->Initialize();
+    this->SetupEmptyOutput();
     }
   
   // We have finished reading.
@@ -507,6 +542,7 @@ int vtkXMLReader::RequestData(vtkInformation *request,
     this->TimeStepWasReadOnce = 1; 
     }
 
+  this->CurrentOutput = 0;
   return 1;
 }
 
@@ -533,6 +569,8 @@ int vtkXMLReader::ReadVTKFile(vtkXMLDataElement* eVTKFile)
     return 0;
     }
 
+  ::ReadStringVersion(version, this->FileMajorVersion, this->FileMinorVersion);
+
   // Setup the compressor if there is one.
   const char* compressor = eVTKFile->GetAttribute("compressor");
   if(compressor)
@@ -599,11 +637,7 @@ int vtkXMLReader::ReadPrimaryElement(vtkXMLDataElement* ePrimary)
 void vtkXMLReader::SetupOutputData()
 {
   // Initialize the output.
-  int i;
-  for(i=0; i < this->GetNumberOfOutputPorts(); ++i)
-    {
-    this->GetExecutive()->GetOutputData(i)->Initialize();
-    }
+  this->CurrentOutput->Initialize();
 }
 
 //----------------------------------------------------------------------------
@@ -671,37 +705,9 @@ int vtkXMLReader::CanReadFile(const char* name)
 //----------------------------------------------------------------------------
 int vtkXMLReader::CanReadFileVersionString(const char* version)
 {
-  // Extract the major and minor version numbers.
-  size_t length = strlen(version);
   int major = 0;
   int minor = 0;
-  const char* begin = version;
-  const char* end = version + length;
-  const char* s;
-  
-  for(s=begin; (s != end) && (*s != '.'); ++s);
-  
-  if(s > begin)
-    {
-    vtksys_ios::stringstream str;
-    str.write(begin, s-begin);
-    str >> major;
-    if(!str)
-      {
-      major = 0;
-      }
-    }
-  if(++s < end)
-    {
-    vtksys_ios::stringstream str;
-    str.write(s, end-s);
-    str >> minor;
-    if(!str)
-      {
-      minor = 0;
-      }
-    }
-  
+  ::ReadStringVersion(version, major, minor);
   return this->CanReadFileVersion(major, minor);
 }
 
@@ -855,6 +861,7 @@ void vtkXMLReader::SetDataArraySelections(vtkXMLDataElement* eDSA,
     sel->SetArrays(0, 0);
     return;
     }
+
   int numArrays = eDSA->GetNumberOfNestedElements();
   if(!numArrays)
     {
@@ -1121,6 +1128,8 @@ int vtkXMLReader::ProcessRequest(vtkInformation* request,
                                  vtkInformationVector** inputVector,
                                  vtkInformationVector* outputVector)
 {
+  this->CurrentOutputInformation =
+    outputVector->GetInformationObject(0);
   // FIXME This piece of code should be rewritten to handle at the same
   // time Pieces and TimeSteps. The REQUEST_DATA_NOT_GENERATED should
   // ideally be changed during execution, so that allocation still
@@ -1130,10 +1139,8 @@ int vtkXMLReader::ProcessRequest(vtkInformation* request,
     request->Has(vtkDemandDrivenPipeline::REQUEST_DATA_NOT_GENERATED()))
     {
     vtkInformation* outInfo = outputVector->GetInformationObject(0);
-    if ( this->CurrentOutput == 0)
-      {
-      outInfo->Set(vtkDemandDrivenPipeline::DATA_NOT_GENERATED(), 1);
-      }
+    outInfo->Set(vtkDemandDrivenPipeline::DATA_NOT_GENERATED(), 1);
+    this->CurrentOutputInformation = 0;
     return 1;
     }
   // END FIXME 
@@ -1141,30 +1148,41 @@ int vtkXMLReader::ProcessRequest(vtkInformation* request,
   // generate the data
   if(request->Has(vtkDemandDrivenPipeline::REQUEST_DATA()))
     {
-    return this->RequestData(request, inputVector, outputVector);
+    int retVal = this->RequestData(request, inputVector, outputVector);
+    this->CurrentOutputInformation = 0;
+    return retVal;
     }
 
   // create the output
   if(request->Has(vtkDemandDrivenPipeline::REQUEST_DATA_OBJECT()))
     {
-    return this->RequestDataObject(request, inputVector, outputVector);
+    int retVal = this->RequestDataObject(request, inputVector, outputVector);
+    this->CurrentOutputInformation = 0;
+    return retVal;
     }
 
   // execute information
   if(request->Has(vtkDemandDrivenPipeline::REQUEST_INFORMATION()))
     {
-    return this->RequestInformation(request, inputVector, outputVector);
+    int retVal = this->RequestInformation(request, inputVector, outputVector);
+    this->CurrentOutputInformation = 0;
+    return retVal;
     }
 
   // return UE info
   if(request->Has
      (vtkStreamingDemandDrivenPipeline::REQUEST_UPDATE_EXTENT_INFORMATION()))
     {
-    return this->RequestUpdateExtentInformation(request, 
-                                                inputVector, outputVector);
+    int retVal = this->RequestUpdateExtentInformation(request, 
+        inputVector, outputVector);
+    this->CurrentOutputInformation = 0;
+    return retVal;
     }
 
-  return this->Superclass::ProcessRequest(request, inputVector, outputVector);
+  int retVal = 
+    this->Superclass::ProcessRequest(request, inputVector, outputVector);
+  this->CurrentOutputInformation = 0;
+  return retVal;
 }
 
 //----------------------------------------------------------------------------
@@ -1193,3 +1211,14 @@ int vtkXMLReader::IsTimeStepInArray(int timestep, int* timesteps, int length)
   return 0;
 }
 
+//----------------------------------------------------------------------------
+vtkDataObject* vtkXMLReader::GetCurrentOutput()
+{
+  return this->CurrentOutput;
+}
+
+//----------------------------------------------------------------------------
+vtkInformation* vtkXMLReader::GetCurrentOutputInformation()
+{
+  return this->CurrentOutputInformation;
+}
