@@ -208,7 +208,7 @@ int vtkXMLWriterWriteBinaryDataBlocks(vtkXMLWriter* writer,
 }
 //*****************************************************************************
 
-vtkCxxRevisionMacro(vtkXMLWriter, "$Revision: 1.71 $");
+vtkCxxRevisionMacro(vtkXMLWriter, "$Revision: 1.73 $");
 vtkCxxSetObjectMacro(vtkXMLWriter, Compressor, vtkDataCompressor);
 //----------------------------------------------------------------------------
 vtkXMLWriter::vtkXMLWriter()
@@ -553,7 +553,7 @@ int vtkXMLWriter::OpenFile()
   else
     {
     // Strip trailing whitespace from the filename.
-    int len = strlen(this->FileName);
+    int len = static_cast<int>(strlen(this->FileName));
     for (int i = len-1; i >= 0; i--)
       {      
       if (isalnum(this->FileName[i])) 
@@ -2301,7 +2301,7 @@ void vtkXMLWriter::WritePointsAppendedData(vtkPoints* points, int timestep,
     // since points->Data is a vtkDataArray.
     vtkDataArray* outPoints = vtkDataArray::SafeDownCast(
       this->CreateArrayForPoints(points->GetData()));
-    if( pointsMTime != mtime )
+    if( pointsMTime != mtime || timestep == 0 )
       {
       pointsMTime = mtime;
       this->WriteArrayAppendedData(outPoints, 
