@@ -17,33 +17,37 @@
 #include "vtkImageStencilSource.h"
 #include "vtkInformation.h"
 #include "vtkInformationVector.h"
+#include "vtkDemandDrivenPipeline.h"
+#include "vtkDataSetAttributes.h"
+#include "vtkDataArray.h"
 #include "vtkObjectFactory.h"
 #include "vtkMath.h"
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkImageStencilData, "$Revision: 1.21 $");
+vtkCxxRevisionMacro(vtkImageStencilData, "$Revision: 1.30 $");
 vtkStandardNewMacro(vtkImageStencilData);
 
 //----------------------------------------------------------------------------
 vtkImageStencilData::vtkImageStencilData()
 {
-  this->Spacing[0] = this->OldSpacing[0] = 1;
-  this->Spacing[1] = this->OldSpacing[1] = 1;
-  this->Spacing[2] = this->OldSpacing[2] = 1;
+  this->Spacing[0] = 1;
+  this->Spacing[1] = 1;
+  this->Spacing[2] = 1;
 
-  this->Origin[0] = this->OldOrigin[0] = 0;
-  this->Origin[1] = this->OldOrigin[1] = 0;
-  this->Origin[2] = this->OldOrigin[2] = 0;
+  this->Origin[0] = 0;
+  this->Origin[1] = 0;
+  this->Origin[2] = 0;
 
   this->NumberOfExtentEntries = 0;
   this->ExtentLists = NULL;
   this->ExtentListLengths = NULL;
 
-  int extent[6] = {0, -1, 0, -1, 0, -1};
-  memcpy(this->Extent, extent, 6*sizeof(int));
   this->Information->Set(vtkDataObject::DATA_EXTENT_TYPE(), VTK_3D_EXTENT);
   this->Information->Set(vtkDataObject::DATA_EXTENT(), this->Extent, 6);
+
+  int extent[6] = {0, -1, 0, -1, 0, -1};
+  memcpy(this->Extent, extent, 6*sizeof(int));
 }
 
 //----------------------------------------------------------------------------
@@ -77,15 +81,6 @@ void vtkImageStencilData::PrintSelf(ostream& os, vtkIndent indent)
      << this->Origin[1] << ", "
      << this->Origin[2] << ")\n";
 
-  os << indent << "OldSpacing: (" 
-     << this->OldSpacing[0] << ", "
-     << this->OldSpacing[1] << ", "
-     << this->OldSpacing[2] << ")\n";
-
-  os << indent << "OldOrigin: (" 
-     << this->OldOrigin[0] << ", "
-     << this->OldOrigin[1] << ", "
-     << this->OldOrigin[2] << ")\n";
 }
 
 //----------------------------------------------------------------------------
@@ -96,7 +91,10 @@ void vtkImageStencilData::Initialize()
     int n = this->NumberOfExtentEntries;
     for (int i = 0; i < n; i++)
       {
-      delete [] this->ExtentLists[i];
+      if (this->ExtentLists[i]) 
+        { 
+        delete [] this->ExtentLists[i]; 
+        }
       }
     delete [] this->ExtentLists;
     }
@@ -116,6 +114,73 @@ void vtkImageStencilData::Initialize()
     }
 }
 
+//----------------------------------------------------------------------------
+void vtkImageStencilData::CopyInformationToPipeline(vtkInformation* request,
+                                                    vtkInformation* input,
+                                                    vtkInformation* output,
+                                                    int forceCopy)
+{
+  // Let the superclass copy whatever it wants.
+  this->Superclass::CopyInformationToPipeline(request, input, output, forceCopy);
+
+  // Set default pipeline information during a request for information.
+  if(request->Has(vtkDemandDrivenPipeline::REQUEST_INFORMATION()))
+    {
+    // Copy settings from the input if available.  Otherwise use our
+    // current settings.
+
+    if(input && input->Has(ORIGIN()))
+      {
+      output->CopyEntry(input, ORIGIN());
+      }
+    else if (!output->Has(ORIGIN()) || forceCopy)
+      {
+      // Set origin (only if it is not set).
+      output->Set(ORIGIN(), this->GetOrigin(), 3);
+      }
+
+    if(input && input->Has(SPACING()))
+      {
+      output->CopyEntry(input, SPACING());
+      }
+    else if (!output->Has(SPACING()) || forceCopy)
+      {
+      // Set spacing (only if it is not set).
+      output->Set(SPACING(), this->GetSpacing(), 3);
+      }
+    }
+}
+
+//----------------------------------------------------------------------------
+void vtkImageStencilData::CopyInformationFromPipeline(vtkInformation* request)
+{
+  // Let the superclass copy whatever it wants.
+  this->Superclass::CopyInformationFromPipeline(request);
+
+  // Copy pipeline information to data information before the producer
+  // executes.
+  if(request->Has(vtkDemandDrivenPipeline::REQUEST_DATA()))
+    {
+    this->CopyOriginAndSpacingFromPipeline();
+    }
+}
+
+//----------------------------------------------------------------------------
+void vtkImageStencilData::CopyOriginAndSpacingFromPipeline()
+{
+  // Copy origin and spacing from pipeline information to the internal
+  // copies.
+  vtkInformation* info = this->PipelineInformation;
+  if(info->Has(SPACING()))
+    {
+    this->SetSpacing(info->Get(SPACING()));
+    }
+  if(info->Has(ORIGIN()))
+    {
+    this->SetOrigin(info->Get(ORIGIN()));
+    }
+}
+
 //----------------------------------------------------------------------------
 void vtkImageStencilData::SetExtent(int* extent)
 {
@@ -174,7 +239,10 @@ void vtkImageStencilData::InternalImageStencilDataCopy(vtkImageStencilData *s)
     int n = this->NumberOfExtentEntries;
     for (int i = 0; i < n; i++)
       {
-      delete [] this->ExtentLists[i];
+      if (this->ExtentLists[i])
+        {
+        delete [] this->ExtentLists[i];
+        }
       }
     delete [] this->ExtentLists;
     }
@@ -197,102 +265,30 @@ void vtkImageStencilData::InternalImageStencilDataCopy(vtkImageStencilData *s)
     for (int i = 0; i < n; i++)
       {
       this->ExtentListLengths[i] = s->ExtentListLengths[i];
-      int m = this->ExtentListLengths[i];
-      this->ExtentLists[i] = new int[m];
-      for (int j = 0; j < m; j++)
+      if (this->ExtentListLengths[i])
         {
-        this->ExtentLists[i][j] = s->ExtentLists[i][j];
+        int m = this->ExtentListLengths[i];
+
+        int clistmaxlen = 2;
+        while (m > clistmaxlen)
+          {
+          clistmaxlen *= 2;
+          }
+        this->ExtentLists[i] = new int[clistmaxlen];
+        for (int j = 0; j < m; j++)
+          {
+          this->ExtentLists[i][j] = s->ExtentLists[i][j];
+          }
+        }
+      else
+        {
+        this->ExtentLists[i] = 0;
         }
       }
     }
   memcpy(this->Extent, s->GetExtent(), 6*sizeof(int));
 }
 
-//----------------------------------------------------------------------------
-// Override from vtkDataObject because we have to handle the Spacing
-// and Origin as well as the UpdateExtent.
-void vtkImageStencilData::PropagateUpdateExtent()
-{
-#if 0
-  // If we need to update due to PipelineMTime, or the fact that our
-  // data was released, then propagate the update extent to the source 
-  // if there is one.
-  if ( this->UpdateTime < this->PipelineMTime || this->DataReleased ||
-       this->UpdateExtentIsOutsideOfTheExtent() || 
-       this->SpacingOrOriginHasChanged() || 
-       this->LastUpdateExtentWasOutsideOfTheExtent)
-    {
-    if (this->Source)
-      {
-      this->Source->PropagateUpdateExtent(this);
-      }
-    }
-  
-  // update the value of this ivar
-  this->LastUpdateExtentWasOutsideOfTheExtent = 
-    this->UpdateExtentIsOutsideOfTheExtent();
-#else
-  this->Superclass::PropagateUpdateExtent();
-#endif
-}
-
-//----------------------------------------------------------------------------
-// Override from vtkDataObject because we have to handle the Spacing
-// and Origin as well as the UpdateExtent.
-void vtkImageStencilData::TriggerAsynchronousUpdate()
-{
-#if 0
-  // If we need to update due to PipelineMTime, or the fact that our
-  // data was released, then propagate the trigger to the source
-  // if there is one.
-  if ( this->UpdateTime < this->PipelineMTime || this->DataReleased ||
-       this->UpdateExtentIsOutsideOfTheExtent() || 
-       this->SpacingOrOriginHasChanged())
-    {
-    if (this->Source)
-      {
-      this->Source->TriggerAsynchronousUpdate();
-      }
-    }
-#else
-  this->Superclass::TriggerAsynchronousUpdate();
-#endif
-}
-
-//----------------------------------------------------------------------------
-void vtkImageStencilData::UpdateData()
-{
-#if 0
-  // If we need to update due to PipelineMTime, or the fact that our
-  // data was released, then propagate the UpdateData to the source
-  // if there is one.
-  if (this->UpdateTime < this->PipelineMTime || this->DataReleased ||
-      this->UpdateExtentIsOutsideOfTheExtent() ||
-      this->SpacingOrOriginHasChanged())
-    {
-    if (this->Source)
-      {
-      this->Source->UpdateData(this);
-      }
-    }
-#else
-  this->Superclass::UpdateData();
-#endif
-}
-
-//----------------------------------------------------------------------------
-int vtkImageStencilData::SpacingOrOriginHasChanged()
-{
-  double *spacing = this->Spacing;
-  double *origin = this->Origin;
-  double *ospacing = this->OldSpacing;
-  double *oorigin = this->OldOrigin;
-
-  return (spacing[0] != ospacing[0] || origin[0] != oorigin[0] ||
-          spacing[1] != ospacing[1] || origin[1] != oorigin[1] ||
-          spacing[2] != ospacing[2] || origin[2] != oorigin[2]);
-}
-
 //----------------------------------------------------------------------------
 void vtkImageStencilData::AllocateExtents()
 {
@@ -307,7 +303,10 @@ void vtkImageStencilData::AllocateExtents()
       int n = this->NumberOfExtentEntries;
       for (int i = 0; i < n; i++)
         {
-        delete [] this->ExtentLists[i];
+        if (this->ExtentLists[i] != 0)
+          {
+          delete [] this->ExtentLists[i];
+          }
         }
       delete [] this->ExtentLists;
       delete [] this->ExtentListLengths;
@@ -335,7 +334,7 @@ void vtkImageStencilData::AllocateExtents()
       if (this->ExtentListLengths[i] != 0)
         {
         this->ExtentListLengths[i] = 0;
-        delete this->ExtentLists[i];
+        delete [] this->ExtentLists[i];
         this->ExtentLists[i] = NULL;
         }
       }
@@ -343,11 +342,13 @@ void vtkImageStencilData::AllocateExtents()
 }
 
 //----------------------------------------------------------------------------
-// Given the output x extent [rmin,rmax] and the current y, z indices,
-// return the sub extents [r1,r2] for which transformation/interpolation
-// are to be done.  The variable 'iter' should be initialized to zero
-// before the first call.  The return value is zero if there are no
-// extents remaining.
+// Given the total output x extent [xMin,xMax] and the current y, z indices,
+// return each sub-extent [r1,r2] that lies within within the unclipped
+// region in sequence.  A value of '0' is returned if no more sub-extents
+// are available.  The variable 'iter' must be initialized to zero before
+// the first call, unless you want the complementary sub-extents in which
+// case you must initialize 'iter' to -1.  The variable 'iter' is used
+// internally to keep track of which sub-extent should be returned next.
 int vtkImageStencilData::GetNextExtent(int &r1, int &r2,
                                        int rmin, int rmax,
                                        int yIdx, int zIdx, int &iter)
@@ -365,6 +366,14 @@ int vtkImageStencilData::GetNextExtent(int &r1, int &r2,
 
   if (yIdx < 0 || yIdx >= yExt || zIdx < 0 || zIdx >= zExt)
     { // out-of-bounds in y or z, use null extent
+    // if iter was set to negative, reverse the result
+    if (iter < 0)
+      {
+      iter = 0;
+      r1 = rmin;
+      r2 = rmax;
+      return 1;
+      }
     return 0;
     }
 
@@ -411,6 +420,10 @@ int vtkImageStencilData::GetNextExtent(int &r1, int &r2,
       return 0;
       }
     r1 = clist[iter++];
+    if (r1 < rmin)
+      {
+      r1 = rmin;
+      }
     }
 
   if (r1 > rmax)
@@ -479,7 +492,8 @@ void vtkImageStencilData::InsertNextExtent(int r1, int r2, int yIdx, int zIdx)
 }
 
 //----------------------------------------------------------------------------
-void vtkImageStencilData::InsertAndMergeExtent(int r1, int r2, int yIdx, int zIdx)
+void vtkImageStencilData::InsertAndMergeExtent(int r1, int r2,
+                                               int yIdx, int zIdx)
 {
   // calculate the index into the extent array
   int extent[6];
@@ -497,57 +511,118 @@ void vtkImageStencilData::InsertAndMergeExtent(int r1, int r2, int yIdx, int zId
     clist[clistlen++] = r2 + 1;
     return;
     }
-  else
-    { 
-    for (int k = 0; k < clistlen; k+=2)
+  
+  for (int k = 0; k < clistlen; k+=2)
+    {
+    if ((r1 >= clist[k] && r1 < clist[k+1]) || 
+      (r2 >= clist[k] && r2 < clist[k+1]))
       {
-      if ((r1 >= clist[k] && r1 < clist[k+1]) || 
-          (r2 >= clist[k] && r2 < clist[k+1]))
+      // An intersecting extent is already present. Merge with that one.
+      if (r1 < clist[k])
         {
-        // An intersecting extent is already present. Merge with that one.
-        if (r1 < clist[k])
-          {
-          clist[k]   = r1;
-          }
-        if (r2 >= clist[k+1])
-          {
-          clist[k+1] = r2+1;
-          }
-        return;
+        clist[k]   = r1;
         }
+      else if (r2 >= clist[k+1])
+        {
+        clist[k+1] = r2+1;
+        this->CollapseAdditionalIntersections(r2, k+2, clist, clistlen);
+        }
+      return;
       }
-
-    // We will be inserting a unique extent...
-      
-    // check whether more space is needed
-    // the allocated space is always the smallest power of two
-    // that is not less than the number of stored items, therefore
-    // we need to allocate space when clistlen is a power of two
-    int clistmaxlen = 2;
-    while (clistlen > clistmaxlen)
+    else if (r1 < clist[k] && r2 >= clist[k+1])
       {
-      clistmaxlen *= 2;
-      }
-    if (clistmaxlen == clistlen)
-      { // need to allocate more space
-      clistmaxlen *= 2;
-      int *newclist = new int[clistmaxlen];
-      for (int k = 0; k < clistlen; k++)
-        {
-        newclist[k] = clist[k];
-        }
-      delete [] clist;
-      clist = newclist;
+      clist[k]   = r1;
+      clist[k+1] = r2+1;
+      this->CollapseAdditionalIntersections(r2, k+2, clist, clistlen);
+      return;
       }
     }
 
-  clist[clistlen++] = r1;
-  clist[clistlen++] = r2 + 1;
+  // We will be inserting a unique extent...
+
+  // check whether more space is needed
+  // the allocated space is always the smallest power of two
+  // that is not less than the number of stored items, therefore
+  // we need to allocate space when clistlen is a power of two
+  int clistmaxlen = 2;
+  while (clistlen > clistmaxlen)
+    {
+    clistmaxlen *= 2;
+    }
+  int insertIndex = clistlen, offset = 0;
+  if (clistmaxlen == clistlen || r1 < clist[clistlen-1])
+    { // need to allocate more space or rearrange
+    if (clistmaxlen == clistlen)
+      {
+      clistmaxlen *= 2;
+      }
+    int *newclist = new int[clistmaxlen];
+    for (int k = 0; k < clistlen; k+=2)
+      {
+      if (offset == 0 && r1 < clist[k])
+        {
+        insertIndex = k;
+        offset = 2;
+        }
+      newclist[k+offset] = clist[k];
+      newclist[k+1+offset] = clist[k+1];
+      }
+    delete [] clist;
+    clist = newclist;
+    }
+
+  clist[insertIndex] = r1;
+  clist[insertIndex+1] = r2 + 1;
+  clistlen += 2;
+}
+
+
+//----------------------------------------------------------------------------
+void vtkImageStencilData::CollapseAdditionalIntersections(int r2, int idx,
+                                                          int *clist,
+                                                          int &clistlen)
+{
+  if (idx >= clistlen)
+    {
+    return;
+    }
+
+  int removeExtentStart = idx, removeExtentEnd = idx;
+  // overlap with any of the remainder of the list?
+  for (; idx < clistlen; idx+=2, removeExtentEnd+=2)
+    {
+    if (r2 < clist[idx])
+      {
+      if (idx == removeExtentStart)
+        {
+        // no additional overlap... thus no collapse
+        return;
+        }
+      break;
+      }
+    else if (r2 < clist[idx+1])
+      {
+      clist[removeExtentStart - 1] = clist[idx+1];
+      }
+    }
+
+  // collapse the list?
+  int i;
+  for (i = removeExtentEnd, idx = removeExtentStart; i < clistlen; i++, idx++)
+    {
+    clist[idx] = clist[i];
+    }
+  clistlen = idx;
 }
 
 //----------------------------------------------------------------------------
 void vtkImageStencilData::RemoveExtent(int r1, int r2, int yIdx, int zIdx)
 {
+  if (zIdx < this->Extent[4] || zIdx > this->Extent[5] || 
+      yIdx < this->Extent[2] || yIdx > this->Extent[3] )
+    {
+    return;
+    }
   
   // calculate the index into the extent array
   int extent[6];
@@ -568,6 +643,7 @@ void vtkImageStencilData::RemoveExtent(int r1, int r2, int yIdx, int zIdx)
     // remove the whole row.
     clistlen = 0;
     delete [] clist;
+    clist = NULL;
     return;
     }
   
@@ -575,16 +651,17 @@ void vtkImageStencilData::RemoveExtent(int r1, int r2, int yIdx, int zIdx)
   for (int k = 0; k < length; k += 2)
     {
     if (r1 <=  clist[k] && r2 >= (clist[k+1]-1))
-      {
+      {  
       // Remove this entry;
       clistlen -= 2;
-  
+
       if (clistlen == 0)
         {
         delete [] clist;
+        clist = NULL;
         return;
         }
-      
+
       int clistmaxlen = 2;
       while (clistlen > clistmaxlen)
         {
@@ -724,6 +801,13 @@ void vtkImageStencilData::Add( vtkImageStencilData * stencil1 )
   stencil1->GetExtent(extent1);
   this->GetExtent(extent2);
 
+  if (extent1[0] > extent1[1] || 
+      extent1[2] > extent1[3] || 
+      extent1[4] > extent1[5])
+    {
+    return;
+    }
+
   if (vtkMath::ExtentIsWithinOtherExtent(extent1,extent2))
     {
 
@@ -839,3 +923,52 @@ void vtkImageStencilData::Subtract( vtkImageStencilData * stencil1 )
   this->Modified();
 } 
 
+//----------------------------------------------------------------------------
+int vtkImageStencilData::Clip( int extent[6] )
+{
+  int currentExtent[6], idy, idz;
+  this->Update();
+  this->GetExtent( currentExtent );
+
+  if (vtkMath::ExtentIsWithinOtherExtent( currentExtent, extent ))
+    {
+    // Nothing to do, we are already within the clipping extents.
+    return 0;
+    }
+
+  bool removeXLeft  = (extent[0] > currentExtent[0]);
+  bool removeXRight = (extent[1] < currentExtent[1]);  
+  bool remove = false, removed = false;
+
+  for (idz=currentExtent[4]; idz<=currentExtent[5]; idz++)
+    {
+    remove = (idz < extent[4] || idz > extent[5]);
+    for (idy = currentExtent[2]; idy <= currentExtent[3]; idy++)
+      {
+      if (remove || idy < extent[2] || idy > extent[3])
+        {
+        // Remove everything at Y = idy, Z = idz.
+        this->RemoveExtent( currentExtent[0],currentExtent[1], idy, idz );
+        removed |= true;
+        }
+      else
+        {
+        if (removeXLeft)
+          {
+          // Clip on the left at Y = idy, Z = idz.
+          this->RemoveExtent( currentExtent[0], extent[0]-1, idy, idz );
+          removed |= true;
+          }
+        if (removeXRight)
+          {
+          // Clip on the right at Y = idy, Z = idz.
+          this->RemoveExtent( extent[1]+1, currentExtent[1], idy, idz );
+          removed |= true;
+          }
+        }
+      }
+    }
+
+  return (removed ? 1 : 0);
+}
+
