@@ -12,32 +12,36 @@
      PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
-/*----------------------------------------------------------------------------
- Copyright (c) Sandia Corporation
- See Copyright.txt or http://www.paraview.org/HTML/Copyright.html for details.
-----------------------------------------------------------------------------*/
+/*-------------------------------------------------------------------------
+  Copyright 2008 Sandia Corporation.
+  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+  the U.S. Government retains certain rights in this software.
+-------------------------------------------------------------------------*/
 
 #include "vtkClustering2DLayoutStrategy.h"
 
+#include "vtkBitArray.h"
 #include "vtkCellArray.h"
 #include "vtkCellData.h"
 #include "vtkCommand.h"
 #include "vtkDataArray.h"
-#include "vtkBitArray.h"
-#include "vtkIntArray.h"
-#include "vtkFloatArray.h"
 #include "vtkDoubleArray.h"
+#include "vtkEdgeListIterator.h"
+#include "vtkFastSplatter.h"
+#include "vtkFloatArray.h"
+#include "vtkGraph.h"
+#include "vtkImageData.h"
 #include "vtkInformation.h"
 #include "vtkInformationVector.h"
+#include "vtkIntArray.h"
 #include "vtkMath.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
-#include "vtkAbstractGraph.h"
-#include "vtkGraph.h"
-#include "vtkFastSplatter.h"
-#include "vtkImageData.h"
+#include "vtkPoints.h"
+#include "vtkSmartPointer.h"
+#include "vtkTree.h"
 
-vtkCxxRevisionMacro(vtkClustering2DLayoutStrategy, "$Revision: 1.8 $");
+vtkCxxRevisionMacro(vtkClustering2DLayoutStrategy, "$Revision: 1.15 $");
 vtkStandardNewMacro(vtkClustering2DLayoutStrategy);
 
 // This is just a convenient macro for smart pointers
@@ -159,7 +163,7 @@ void vtkClustering2DLayoutStrategy::GenerateGaussianSplat(vtkImageData *splat, i
 // Set the graph that will be laid out
 void vtkClustering2DLayoutStrategy::Initialize()
 {
-  srand(this->RandomSeed);
+  vtkMath::RandomSeed(this->RandomSeed);
 
   // Set up some quick access variables
   vtkPoints* pts = this->Graph->GetPoints();
@@ -227,8 +231,8 @@ void vtkClustering2DLayoutStrategy::Initialize()
   // Jitter x and y, skip z
   for (vtkIdType i=0; i<numVertices*3; i+=3)
     {
-    rawPointData[i] += this->RestDistance*(static_cast<float>(rand())/RAND_MAX - .5);
-    rawPointData[i+1] += this->RestDistance*(static_cast<float>(rand())/RAND_MAX - .5);
+    rawPointData[i] += this->RestDistance*(vtkMath::Random() - .5);
+    rawPointData[i+1] += this->RestDistance*(vtkMath::Random() - .5);
     }
 
   // Get the weight array
@@ -251,20 +255,30 @@ void vtkClustering2DLayoutStrategy::Initialize()
     }
     
   // Load up the edge data structures
-  for (vtkIdType i=0; i<numEdges; ++i)
+  vtkSmartPointer<vtkEdgeListIterator> edges =
+    vtkSmartPointer<vtkEdgeListIterator>::New();
+  this->Graph->GetEdges(edges);
+  while (edges->HasNext())
     {
-    this->EdgeArray[i].from = this->Graph->GetSourceVertex(i);
-    this->EdgeArray[i].to = this->Graph->GetTargetVertex(i);
-    this->EdgeArray[i].dead_edge = 0;
+    vtkEdgeType e = edges->Next();
+    this->EdgeArray[e.Id].from = e.Source;
+    this->EdgeArray[e.Id].to = e.Target;
+    this->EdgeArray[e.Id].dead_edge = 0;
     
     if (weightArray != NULL)
       {
-      weight = weightArray->GetTuple1(i);
-      this->EdgeArray[i].weight = weight / maxWeight;
+      weight = weightArray->GetTuple1(e.Id);
+      float normalized_weight = weight / maxWeight;
+      
+      // Now increase the effect of high weight edges
+      // Note: This is full of magic goodness
+      normalized_weight = pow(normalized_weight, 4);
+      
+      this->EdgeArray[e.Id].weight = normalized_weight;
       }
     else
       {
-      this->EdgeArray[i].weight = 1.0;
+      this->EdgeArray[e.Id].weight = 1.0;
       }
     }
     
@@ -524,8 +538,10 @@ void vtkClustering2DLayoutStrategy::ResolveCoincidentVertices()
   // Place the vertices into a giant grid (100xNumVertices)
   // and see if you have any collisions
   vtkBitArray *giantGrid = vtkBitArray::New();
-  vtkIdType xDim = static_cast<int>(sqrt((float)numVertices) * 10);
-  vtkIdType yDim = static_cast<int>(sqrt((float)numVertices) * 10);
+  vtkIdType xDim =
+    static_cast<int>(sqrt(static_cast<double>(numVertices)) * 10);
+  vtkIdType yDim =
+    static_cast<int>(sqrt(static_cast<double>(numVertices)) * 10);
   vtkIdType gridSize = xDim * yDim;
   giantGrid->SetNumberOfValues(gridSize);
   
@@ -535,8 +551,16 @@ void vtkClustering2DLayoutStrategy::ResolveCoincidentVertices()
     giantGrid->SetValue(i, 0);
     }
   
-  double bounds[6];
+  double bounds[6], paddedBounds[6];
   this->Graph->GetBounds(bounds);
+  
+  // Give bounds a 10% padding
+  paddedBounds[0] = bounds[0] - (bounds[1]-bounds[0])*.1;
+  paddedBounds[1] = bounds[1] + (bounds[1]-bounds[0])*.1;
+  paddedBounds[2] = bounds[2] - (bounds[3]-bounds[2])*.1;
+  paddedBounds[3] = bounds[3] + (bounds[3]-bounds[2])*.1;
+  paddedBounds[4] = paddedBounds[5] = 0;
+  
   int totalCollisionOps = 0;
   
   for(vtkIdType i=0; i<numVertices; ++i)
@@ -545,11 +569,11 @@ void vtkClustering2DLayoutStrategy::ResolveCoincidentVertices()
       
     // Compute indices into the buckets
     int indexX = static_cast<int>(
-                 (rawPointData[rawIndex]-bounds[0]) /
-                 (bounds[1]-bounds[0]) * (xDim-1) + .5);
+                 (rawPointData[rawIndex]-paddedBounds[0]) /
+                 (paddedBounds[1]-paddedBounds[0]) * (xDim-1) + .5);
     int indexY = static_cast<int>(
-                 (rawPointData[rawIndex+1]-bounds[2]) /
-                 (bounds[3]-bounds[2]) * (yDim-1) + .5);
+                 (rawPointData[rawIndex+1]-paddedBounds[2]) /
+                 (paddedBounds[3]-paddedBounds[2]) * (yDim-1) + .5);
                  
     // See if you collide with another vertex
     if (giantGrid->GetValue(indexX + indexY*xDim))
@@ -559,7 +583,7 @@ void vtkClustering2DLayoutStrategy::ResolveCoincidentVertices()
       // by randomly jumping to a place that doesn't
       // have another vertex
       bool collision = true;
-      float jumpDistance = 5.0*(bounds[1]-bounds[0])/xDim; // 2.5 grid spaces max
+      float jumpDistance = 5.0*(paddedBounds[1]-paddedBounds[0])/xDim; // 2.5 grid spaces max
       int collisionOps = 0;
       
       // You get 10 trys and then we have to punt
@@ -568,16 +592,16 @@ void vtkClustering2DLayoutStrategy::ResolveCoincidentVertices()
         collisionOps++;
         
         // Move
-        rawPointData[rawIndex] += jumpDistance*(static_cast<float>(rand())/RAND_MAX - .5);
-        rawPointData[rawIndex+1] += jumpDistance*(static_cast<float>(rand())/RAND_MAX - .5);
+        rawPointData[rawIndex] += jumpDistance*(vtkMath::Random() - .5);
+        rawPointData[rawIndex+1] += jumpDistance*(vtkMath::Random() - .5);
         
         // Test
         indexX = static_cast<int>(
-                 (rawPointData[rawIndex]-bounds[0]) /
-                 (bounds[1]-bounds[0]) * (xDim-1) + .5);
+                 (rawPointData[rawIndex]-paddedBounds[0]) /
+                 (paddedBounds[1]-paddedBounds[0]) * (xDim-1) + .5);
         indexY = static_cast<int>(
-                     (rawPointData[rawIndex+1]-bounds[2]) /
-                     (bounds[3]-bounds[2]) * (yDim-1) + .5);
+                     (rawPointData[rawIndex+1]-paddedBounds[2]) /
+                     (paddedBounds[3]-paddedBounds[2]) * (yDim-1) + .5);
         if (!giantGrid->GetValue(indexX + indexY*xDim))
           {
           collision = false; // yea
@@ -591,6 +615,7 @@ void vtkClustering2DLayoutStrategy::ResolveCoincidentVertices()
     }
   
   // Delete giantGrid
+  giantGrid->Initialize();
   giantGrid->Delete();
   
   // Report number of collision operations just for sanity check  
