@@ -12,33 +12,36 @@
      PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
-/*----------------------------------------------------------------------------
- Copyright (c) Sandia Corporation
- See Copyright.txt or http://www.paraview.org/HTML/Copyright.html for details.
-----------------------------------------------------------------------------*/
+/*-------------------------------------------------------------------------
+  Copyright 2008 Sandia Corporation.
+  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+  the U.S. Government retains certain rights in this software.
+-------------------------------------------------------------------------*/
 
 #include "vtkCommunity2DLayoutStrategy.h"
 
+#include "vtkBitArray.h"
 #include "vtkCellArray.h"
 #include "vtkCellData.h"
 #include "vtkCommand.h"
-#include "vtkBitArray.h"
 #include "vtkDataArray.h"
-#include "vtkIntArray.h"
-#include "vtkFloatArray.h"
 #include "vtkDoubleArray.h"
+#include "vtkEdgeListIterator.h"
+#include "vtkFastSplatter.h"
+#include "vtkFloatArray.h"
+#include "vtkGraph.h"
+#include "vtkImageData.h"
 #include "vtkInformation.h"
 #include "vtkInformationVector.h"
+#include "vtkIdTypeArray.h"
 #include "vtkMath.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
-#include "vtkAbstractGraph.h"
-#include "vtkGraph.h"
+#include "vtkPoints.h"
+#include "vtkSmartPointer.h"
 #include "vtkTree.h"
-#include "vtkFastSplatter.h"
-#include "vtkImageData.h"
 
-vtkCxxRevisionMacro(vtkCommunity2DLayoutStrategy, "$Revision: 1.3 $");
+vtkCxxRevisionMacro(vtkCommunity2DLayoutStrategy, "$Revision: 1.16 $");
 vtkStandardNewMacro(vtkCommunity2DLayoutStrategy);
 
 // This is just a convenient macro for smart pointers
@@ -69,14 +72,16 @@ vtkCommunity2DLayoutStrategy::vtkCommunity2DLayoutStrategy()
     
   this->RandomSeed = 123;
   this->MaxNumberOfIterations = 200;
-  this->IterationsPerLayout = 1;
+  this->IterationsPerLayout = 200;
   this->InitialTemperature = 5;
   this->CoolDownRate = 50.0;
   this->LayoutComplete = 0;
   this->EdgeWeightField = 0;
   this->SetEdgeWeightField("weight");
   this->RestDistance = 0;
-  this->EdgeArray = NULL;
+  this->EdgeArray = 0;
+  this->CommunityArrayName = 0;
+  this->SetCommunityArrayName("community");
 }
 
 // ----------------------------------------------------------------------
@@ -84,6 +89,7 @@ vtkCommunity2DLayoutStrategy::vtkCommunity2DLayoutStrategy()
 vtkCommunity2DLayoutStrategy::~vtkCommunity2DLayoutStrategy()
 {
   this->SetEdgeWeightField(0);
+  this->SetCommunityArrayName(0);
 }
 
 
@@ -159,7 +165,7 @@ void vtkCommunity2DLayoutStrategy::GenerateGaussianSplat(vtkImageData *splat, in
 // Set the graph that will be laid out
 void vtkCommunity2DLayoutStrategy::Initialize()
 {
-  srand(this->RandomSeed);
+  vtkMath::RandomSeed(this->RandomSeed);
 
   // Set up some quick access variables
   vtkPoints* pts = this->Graph->GetPoints();
@@ -218,8 +224,8 @@ void vtkCommunity2DLayoutStrategy::Initialize()
   // Jitter x and y, skip z
   for (vtkIdType i=0; i<numVertices*3; i+=3)
     {
-    rawPointData[i] += this->RestDistance*(static_cast<float>(rand())/RAND_MAX - .5);
-    rawPointData[i+1] += this->RestDistance*(static_cast<float>(rand())/RAND_MAX - .5);
+    rawPointData[i] += this->RestDistance*(vtkMath::Random() - .5);
+    rawPointData[i+1] += this->RestDistance*(vtkMath::Random() - .5);
     }
 
   // Get the weight array
@@ -242,20 +248,23 @@ void vtkCommunity2DLayoutStrategy::Initialize()
     }
     
   // Load up the edge data structures
-  for (vtkIdType i=0; i<numEdges; ++i)
+  vtkSmartPointer<vtkEdgeListIterator> edges =
+    vtkSmartPointer<vtkEdgeListIterator>::New();
+  this->Graph->GetEdges(edges);
+  while (edges->HasNext())
     {
-    this->EdgeArray[i].from = this->Graph->GetSourceVertex(i);
-    this->EdgeArray[i].to = this->Graph->GetTargetVertex(i);
-    this->EdgeArray[i].dead_edge = 0;
+    vtkEdgeType e = edges->Next();
+    this->EdgeArray[e.Id].from = e.Source;
+    this->EdgeArray[e.Id].to = e.Target;
     
     if (weightArray != NULL)
       {
-      weight = weightArray->GetTuple1(i);
-      this->EdgeArray[i].weight = weight / maxWeight;
+      weight = weightArray->GetTuple1(e.Id);
+      this->EdgeArray[e.Id].weight = weight / maxWeight;
       }
     else
       {
-      this->EdgeArray[i].weight = 1.0;
+      this->EdgeArray[e.Id].weight = 1.0;
       }
     }
     
@@ -292,15 +301,13 @@ void vtkCommunity2DLayoutStrategy::Layout()
   vtkIdType numVertices = this->Graph->GetNumberOfVertices();
   vtkIdType numEdges = this->Graph->GetNumberOfEdges();
   
-  // Get a quick pointer to the biconnected array
-  //vtkIdTypeArray *biConn = vtkIdTypeArray::SafeDownCast(
-  //  this->Graph->GetVertexData()->GetArray("biconnected component"));
-  vtkIntArray *community = vtkIntArray::SafeDownCast(
-    this->Graph->GetVertexData()->GetArray("community"));
+  // Get a quick pointer to the community array
+  vtkDataArray *community =
+    this->Graph->GetVertexData()->GetArray(this->CommunityArrayName);
   if (community == NULL)
     {
-    vtkErrorMacro("vtkCommunity2DLayoutStrategy did not find a \"community\" array." <<
-                  "\n so the layout will not pull communities together like it should");
+    vtkWarningMacro("vtkCommunity2DLayoutStrategy did not find a \"community\" array." <<
+                    "\n so the layout will not pull communities together like it should");
     }
   
   // Get a quick pointer to the point data
@@ -406,32 +413,44 @@ void vtkCommunity2DLayoutStrategy::Layout()
       int sourceIndex = this->EdgeArray[j].from;
       int targetIndex = this->EdgeArray[j].to;
       
-      // Perform weight adjustment
-      attractValue = this->EdgeArray[j].weight*disSquared-this->RestDistance;
-
       // Clustering: Get close to other nodes that are 
       // part of your community
+      float communityWeight = 1;
       if (community)
         {
-        int sourceComm = community->GetValue(sourceIndex);
-        int targetComm = community->GetValue(targetIndex);
-        if ((sourceComm != -1) &&
-            (sourceComm == targetComm))
+        int sourceComm = static_cast<int> (community->GetTuple1(sourceIndex));
+        int targetComm = static_cast<int> (community->GetTuple1(targetIndex));
+        
+        // Often -1 is used for no/unspecified community
+        // if either node is marked as such then just skip
+        if ((sourceComm == -1) || (targetComm == -1))
           {
-          attractValue *= 1e3;
+          continue;
+          } 
+               
+        // If the source and target are the same
+        // then increase the weight between them
+        if (sourceComm == targetComm)
+          {
+          communityWeight = 10;
           }
-        else if ((sourceComm != -1) && 
-                 (sourceComm != -1))
+          
+        // If source and target are different
+        // then decrease the weight between them
+        else 
           {
-          attractValue *= 1e-3;
-          this->EdgeArray[j].dead_edge = 1;
+          communityWeight = .01;
           }     
-
-        rawAttractArray[rawSourceIndex]   -= delta[0] * attractValue;
-        rawAttractArray[rawSourceIndex+1] -= delta[1] * attractValue;
-        rawAttractArray[rawTargetIndex]   += delta[0] * attractValue;
-        rawAttractArray[rawTargetIndex+1] += delta[1] * attractValue;
         } // if community
+      
+      // Perform weight adjustment
+      attractValue = this->EdgeArray[j].weight*communityWeight*disSquared -
+                     this->RestDistance;
+
+      rawAttractArray[rawSourceIndex]   -= delta[0] * attractValue;
+      rawAttractArray[rawSourceIndex+1] -= delta[1] * attractValue;
+      rawAttractArray[rawTargetIndex]   += delta[0] * attractValue;
+      rawAttractArray[rawTargetIndex+1] += delta[1] * attractValue;
       } // for each edge
       
     // Okay now set new positions based on replusion 
@@ -509,8 +528,10 @@ void vtkCommunity2DLayoutStrategy::ResolveCoincidentVertices()
   // Place the vertices into a giant grid (100xNumVertices)
   // and see if you have any collisions
   vtkBitArray *giantGrid = vtkBitArray::New();
-  vtkIdType xDim = static_cast<int>(sqrt((float)numVertices) * 10);
-  vtkIdType yDim = static_cast<int>(sqrt((float)numVertices) * 10);
+  vtkIdType xDim =
+    static_cast<int>(sqrt(static_cast<double>(numVertices)) * 10);
+  vtkIdType yDim =
+    static_cast<int>(sqrt(static_cast<double>(numVertices)) * 10);
   vtkIdType gridSize = xDim * yDim;
   giantGrid->SetNumberOfValues(gridSize);
   
@@ -520,8 +541,16 @@ void vtkCommunity2DLayoutStrategy::ResolveCoincidentVertices()
     giantGrid->SetValue(i, 0);
     }
   
-  double bounds[6];
+  double bounds[6], paddedBounds[6];
   this->Graph->GetBounds(bounds);
+  
+  // Give bounds a 10% padding
+  paddedBounds[0] = bounds[0] - (bounds[1]-bounds[0])*.1;
+  paddedBounds[1] = bounds[1] + (bounds[1]-bounds[0])*.1;
+  paddedBounds[2] = bounds[2] - (bounds[3]-bounds[2])*.1;
+  paddedBounds[3] = bounds[3] + (bounds[3]-bounds[2])*.1;
+  paddedBounds[4] = paddedBounds[5] = 0;
+  
   int totalCollisionOps = 0;
   
   for(vtkIdType i=0; i<numVertices; ++i)
@@ -530,11 +559,11 @@ void vtkCommunity2DLayoutStrategy::ResolveCoincidentVertices()
       
     // Compute indices into the buckets
     int indexX = static_cast<int>(
-                 (rawPointData[rawIndex]-bounds[0]) /
-                 (bounds[1]-bounds[0]) * (xDim-1) + .5);
+                 (rawPointData[rawIndex]-paddedBounds[0]) /
+                 (paddedBounds[1]-paddedBounds[0]) * (xDim-1) + .5);
     int indexY = static_cast<int>(
-                 (rawPointData[rawIndex+1]-bounds[2]) /
-                 (bounds[3]-bounds[2]) * (yDim-1) + .5);
+                 (rawPointData[rawIndex+1]-paddedBounds[2]) /
+                 (paddedBounds[3]-paddedBounds[2]) * (yDim-1) + .5);
                  
     // See if you collide with another vertex
     if (giantGrid->GetValue(indexX + indexY*xDim))
@@ -544,7 +573,7 @@ void vtkCommunity2DLayoutStrategy::ResolveCoincidentVertices()
       // by randomly jumping to a place that doesn't
       // have another vertex
       bool collision = true;
-      float jumpDistance = 5.0*(bounds[1]-bounds[0])/xDim; // 2.5 grid spaces max
+      float jumpDistance = 5.0*(paddedBounds[1]-paddedBounds[0])/xDim; // 2.5 grid spaces max
       int collisionOps = 0;
       
       // You get 10 trys and then we have to punt
@@ -553,16 +582,16 @@ void vtkCommunity2DLayoutStrategy::ResolveCoincidentVertices()
         collisionOps++;
         
         // Move
-        rawPointData[rawIndex] += jumpDistance*(static_cast<float>(rand())/RAND_MAX - .5);
-        rawPointData[rawIndex+1] += jumpDistance*(static_cast<float>(rand())/RAND_MAX - .5);
+        rawPointData[rawIndex] += jumpDistance*(vtkMath::Random() - .5);
+        rawPointData[rawIndex+1] += jumpDistance*(vtkMath::Random() - .5);
         
         // Test
         indexX = static_cast<int>(
-                 (rawPointData[rawIndex]-bounds[0]) /
-                 (bounds[1]-bounds[0]) * (xDim-1) + .5);
+                 (rawPointData[rawIndex]-paddedBounds[0]) /
+                 (paddedBounds[1]-paddedBounds[0]) * (xDim-1) + .5);
         indexY = static_cast<int>(
-                     (rawPointData[rawIndex+1]-bounds[2]) /
-                     (bounds[3]-bounds[2]) * (yDim-1) + .5);
+                     (rawPointData[rawIndex+1]-paddedBounds[2]) /
+                     (paddedBounds[3]-paddedBounds[2]) * (yDim-1) + .5);
         if (!giantGrid->GetValue(indexX + indexY*xDim))
           {
           collision = false; // yea
@@ -593,4 +622,5 @@ void vtkCommunity2DLayoutStrategy::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "CoolDownRate: " << this->CoolDownRate << endl;
   os << indent << "RestDistance: " << this->RestDistance << endl;
   os << indent << "EdgeWeightField: " << (this->EdgeWeightField ? this->EdgeWeightField : "(none)") << endl;
+  os << indent << "CommunityArrayName: " << (this->CommunityArrayName ? this->CommunityArrayName : "(none)") << endl;
 }
