@@ -31,12 +31,14 @@
 
 #include "vtkObject.h"
 
-class vtkDataSet;
-class vtkImageData;
-class vtkDataObject;
-class vtkDataArray;
 class vtkBoundingBox;
 class vtkCharArray;
+class vtkDataArray;
+class vtkDataObject;
+class vtkDataSet;
+class vtkImageData;
+class vtkMultiBlockDataSet;
+class vtkTemporalDataSet;
 
 class VTK_PARALLEL_EXPORT vtkCommunicator : public vtkObject
 {
@@ -137,6 +139,9 @@ public:
   int Send(const int* data, vtkIdType length, int remoteHandle, int tag) {
     return this->SendVoidArray(data, length, VTK_INT, remoteHandle, tag);
   }
+  int Send(const unsigned int* data, vtkIdType length, int remoteHandle, int tag) {
+    return this->SendVoidArray(data, length, VTK_INT, remoteHandle, tag);
+  }
   int Send(const unsigned long* data, vtkIdType length,
            int remoteHandle, int tag) {
     return this->SendVoidArray(data, length,VTK_UNSIGNED_LONG,remoteHandle,tag);
@@ -189,6 +194,9 @@ public:
   int Receive(int* data, vtkIdType length, int remoteHandle, int tag) {
     return this->ReceiveVoidArray(data, length, VTK_INT, remoteHandle, tag);
   }
+  int Receive(unsigned int* data, vtkIdType length, int remoteHandle, int tag) {
+    return this->ReceiveVoidArray(data, length, VTK_INT, remoteHandle, tag);
+  }
   int Receive(unsigned long* data, vtkIdType length, int remoteHandle, int tag){
     return this->ReceiveVoidArray(data, length, VTK_UNSIGNED_LONG, remoteHandle,
                                   tag);
@@ -797,10 +805,16 @@ protected:
 
   // Internal methods called by Send/Receive(vtkDataObject *... ) above.
   int SendElementalDataObject(vtkDataObject* data, int remoteHandle, int tag);
+  int SendMultiBlockDataSet(vtkMultiBlockDataSet* data, int remoteHandle, int tag);
+  int SendTemporalDataSet(vtkTemporalDataSet* data, int remoteHandle, int tag);
   int ReceiveDataObject(vtkDataObject* data, 
                         int remoteHandle, int tag, int type=-1);
   int ReceiveElementalDataObject(vtkDataObject* data, 
                                  int remoteHandle, int tag);
+  int ReceiveMultiBlockDataSet(
+    vtkMultiBlockDataSet* data, int remoteHandle, int tag);
+  int ReceiveTemporalDataSet(
+    vtkTemporalDataSet* data, int remoteHandle, int tag);
 
   int MaximumNumberOfProcesses;
   int NumberOfProcesses;
