@@ -100,7 +100,7 @@
   x = NULL;      \
 }
 
-vtkCxxRevisionMacro(vtkExodusIIWriter, "$Revision: 1.21.4.1 $");
+vtkCxxRevisionMacro(vtkExodusIIWriter, "$Revision: 1.23 $");
 vtkStandardNewMacro(vtkExodusIIWriter);
 vtkCxxSetObjectMacro(vtkExodusIIWriter, ModelMetadata, vtkModelMetadata);
 
@@ -388,8 +388,7 @@ int vtkExodusIIWriter::ProcessRequest(vtkInformation *request,
   else if(request->Has(vtkStreamingDemandDrivenPipeline::REQUEST_UPDATE_EXTENT()))
     {
     // get the requested update extent
-    if(!this->TimeValues && inputVector[0]->GetInformationObject(0)
-          ->Has(vtkStreamingDemandDrivenPipeline::TIME_STEPS()))
+    if(!this->TimeValues)
       {
       this->TimeValues = vtkDoubleArray::New();
       this->TimeValues->SetArray(
@@ -398,7 +397,7 @@ int vtkExodusIIWriter::ProcessRequest(vtkInformation *request,
         inputVector[0]->GetInformationObject(0)
           ->Length(vtkStreamingDemandDrivenPipeline::TIME_STEPS()),1);
       }
-    if (this->WriteAllTimeSteps && this->TimeValues)
+    if (this->WriteAllTimeSteps)
       {
       if(this->TimeValues->GetPointer(0))
         {
@@ -473,9 +472,7 @@ int vtkExodusIIWriter::RequestData(
     }
 
   // is this the first request
-  if (this->CurrentTimeIndex==0 && 
-      this->WriteAllTimeSteps && 
-      this->NumberOfTimeSteps>0)
+  if (this->CurrentTimeIndex==0 && this->WriteAllTimeSteps)
     {
     // Tell the pipeline to start looping.
     request->Set(vtkStreamingDemandDrivenPipeline::CONTINUE_EXECUTING(), 1);
@@ -881,7 +878,7 @@ int vtkExodusIIWriter::CreateExodusModel()
             {
             blockIds->InsertNextValue(*iter);
             }
-          this->SetAllBlockIds(blockIdSet.size(), blockIds->GetPointer(0));
+          this->SetAllBlockIds( static_cast<int>( blockIdSet.size() ), blockIds->GetPointer(0));
           blockIds->Delete();
           }
         }
@@ -1120,7 +1117,7 @@ int vtkExodusIIWriter::CreateBlockIdInformation(vtkModelMetadata *em)
       {
       blockIds->InsertNextValue(*iter);
       }
-    this->SetAllBlockIds(blockIdSet.size(), blockIds->GetPointer(0));
+    this->SetAllBlockIds( static_cast<int>( blockIdSet.size() ), blockIds->GetPointer(0));
     blockIds->Delete();
     }
 
@@ -2239,12 +2236,10 @@ int vtkExodusIIWriter::WriteNextTimeStep()
   int rc = 0;
 
   int ts = this->CurrentTimeIndex;
-  float tsv = 0;
-  if(this->TimeValues)
-    {
-    tsv = this->TimeValues->GetValue(this->CurrentTimeIndex);
-    }
- 
+  float tsv = (this->TimeValues->GetNumberOfTuples() > 0 ? 
+               this->TimeValues->GetValue(this->CurrentTimeIndex):
+               0.0);
+
   if (this->PassDoubles)
     {
     double dtsv = (double)tsv;
@@ -3535,7 +3530,7 @@ char *vtkExodusIIWriter::StrDupWithNew(const char *s)
 
   if (s)
     {
-    int len = strlen(s);
+    size_t len = strlen(s);
     if (len == 0)
       {
       newstr = new char [1];
@@ -3554,11 +3549,11 @@ char *vtkExodusIIWriter::StrDupWithNew(const char *s)
 
 void vtkExodusIIWriter::StringUppercase(const char* str, char* upperstr)
 { 
-  int len = 0;
+  size_t len = 0;
   if ( str )
     {
     len = strlen(str);
-    for (int i=0; i<len; i++)
+    for (size_t i=0; i<len; i++)
       upperstr[i] = toupper(str[i]);
     }
   else
