@@ -23,16 +23,14 @@
 #include "vtkRectilinearGrid.h"
 #include "vtkStructuredGrid.h"
 #include "vtkToolkits.h"
-
 #include "vtkMPI.h"
-
 #include "vtkSmartPointer.h"
 #define VTK_CREATE(type, name) \
   vtkSmartPointer<type> name = vtkSmartPointer<type>::New()
 
 #include <vtkstd/vector>
 
-vtkCxxRevisionMacro(vtkMPICommunicator, "$Revision: 1.45 $");
+vtkCxxRevisionMacro(vtkMPICommunicator, "$Revision: 1.48 $");
 vtkStandardNewMacro(vtkMPICommunicator);
 
 vtkMPICommunicator* vtkMPICommunicator::WorldCommunicator = 0;
@@ -202,23 +200,40 @@ template <class T>
 int vtkMPICommunicatorSendData(const T* data, int length, int sizeoftype, 
                                int remoteProcessId, int tag, 
                                MPI_Datatype datatype, MPI_Comm *Handle, 
-                               int useCopy) 
+                               int useCopy,
+                               int useSsend)
 {
   if (useCopy)
     {
     int retVal;
-    
+
     char* tmpData = vtkMPICommunicator::Allocate(length*sizeoftype);
     memcpy(tmpData, data, length*sizeoftype);
-    retVal = MPI_Send(tmpData, length, datatype, remoteProcessId, tag, 
-                      *(Handle));
+    if (useSsend)
+      {
+      retVal = MPI_Ssend(tmpData, length, datatype, remoteProcessId, tag, 
+        *(Handle));
+      }
+    else
+      {
+      retVal = MPI_Send(tmpData, length, datatype, remoteProcessId, tag, 
+        *(Handle));
+      }
     vtkMPICommunicator::Free(tmpData);
     return retVal;
     }
   else
     {
-    return MPI_Send(const_cast<T *>(data), length, datatype,
-                    remoteProcessId, tag, *(Handle));
+    if (useSsend)
+      {
+      return MPI_Ssend(const_cast<T *>(data), length, datatype,
+        remoteProcessId, tag, *(Handle));
+      }
+    else
+      {
+      return MPI_Send(const_cast<T *>(data), length, datatype,
+        remoteProcessId, tag, *(Handle));
+      }
     }
 }
 //----------------------------------------------------------------------------
@@ -367,6 +382,7 @@ void vtkMPICommunicator::PrintSelf(ostream& os, vtkIndent indent)
     {
     os << "(none)\n";
     }
+  os << indent << "UseSsend: " << (this->UseSsend? "On" : " Off") << endl;
   os << indent << "Initialized: " << (this->Initialized ? "On\n" : "Off\n");
   os << indent << "Keep handle: " << (this->KeepHandle ? "On\n" : "Off\n");
   if ( this != vtkMPICommunicator::WorldCommunicator )
@@ -393,6 +409,7 @@ vtkMPICommunicator::vtkMPICommunicator()
   this->Initialized = 0;
   this->KeepHandle = 0;
   this->LastSenderId = -1;
+  this->UseSsend = 0;
 }
 
 //----------------------------------------------------------------------------
@@ -693,14 +710,15 @@ int vtkMPICommunicator::SendVoidArray(const void *data, vtkIdType length,
     {
     vtkMPICommunicatorSendData(byteData, maxSend, sizeOfType, remoteProcessId,
                                tag, mpiType, this->MPIComm->Handle,
-                               vtkCommunicator::UseCopy);
+                               vtkCommunicator::UseCopy,
+                               this->UseSsend);
     byteData += maxSend*sizeOfType;
     length -= maxSend;
     }
   return CheckForMPIError
     (vtkMPICommunicatorSendData(byteData, length, sizeOfType, remoteProcessId,
                                 tag, mpiType, this->MPIComm->Handle,
-                                vtkCommunicator::UseCopy));
+                                vtkCommunicator::UseCopy, this->UseSsend));
 }
 
 //-----------------------------------------------------------------------------
@@ -996,6 +1014,22 @@ int vtkMPICommunicator::GatherVVoidArray(const void *sendBuffer,
     {
     int numProc;
     MPI_Comm_size(*this->MPIComm->Handle, &numProc);
+#if defined(OPEN_MPI) && (OMPI_MAJOR_VERSION <= 1) && (OMPI_MINOR_VERSION <= 2)
+    // I found a bug in OpenMPI 1.2 and earlier where MPI_Gatherv sometimes
+    // fails with only one process.  I am told that they will fix it in a later
+    // version, but here is a workaround for now.
+    if (numProc == 1)
+      {
+      switch(type)
+        {
+        vtkTemplateMacro(vtkstd::copy(
+                       reinterpret_cast<const VTK_TT*>(sendBuffer),
+                       reinterpret_cast<const VTK_TT*>(sendBuffer) + sendLength,
+                       reinterpret_cast<VTK_TT*>(recvBuffer) + offsets[0]));
+        }
+      return 1;
+      }
+#endif //OPEN_MPI
     vtkstd::vector<int> mpiRecvLengths, mpiOffsets;
     mpiRecvLengths.resize(numProc);  mpiOffsets.resize(numProc);
     for (int i = 0; i < numProc; i++)
@@ -1054,6 +1088,23 @@ int vtkMPICommunicator::ScatterVVoidArray(const void *sendBuffer,
     {
     int numProc;
     MPI_Comm_size(*this->MPIComm->Handle, &numProc);
+#if defined(OPEN_MPI) && (OMPI_MAJOR_VERSION <= 1) && (OMPI_MINOR_VERSION <= 2)
+    // I found a bug in OpenMPI 1.2 and earlier where MPI_Scatterv sometimes
+    // fails with only one process.  I am told that they will fix it in a later
+    // version, but here is a workaround for now.
+    if (numProc == 1)
+      {
+      switch(type)
+        {
+        vtkTemplateMacro(vtkstd::copy(
+                   reinterpret_cast<const VTK_TT*>(sendBuffer) + offsets[0],
+                   reinterpret_cast<const VTK_TT*>(sendBuffer) + offsets[0]
+                                                               + sendLengths[0],
+                   reinterpret_cast<VTK_TT*>(recvBuffer)));
+        }
+      return 1;
+      }
+#endif //OPEN_MPI
     vtkstd::vector<int> mpiSendLengths, mpiOffsets;
     mpiSendLengths.resize(numProc);  mpiOffsets.resize(numProc);
     for (int i = 0; i < numProc; i++)
@@ -1107,6 +1158,37 @@ int vtkMPICommunicator::AllGatherVVoidArray(const void *sendBuffer,
   // become int arrays.
   int numProc;
   MPI_Comm_size(*this->MPIComm->Handle, &numProc);
+#if defined(OPEN_MPI) && (OMPI_MAJOR_VERSION <= 1) && (OMPI_MINOR_VERSION <= 2)
+  // I found a bug in OpenMPI 1.2 and earlier where MPI_Allgatherv sometimes
+  // fails with only one process.  I am told that they will fix it in a later
+  // version, but here is a workaround for now.
+  if (numProc == 1)
+    {
+    switch(type)
+      {
+      vtkTemplateMacro(vtkstd::copy(
+                       reinterpret_cast<const VTK_TT*>(sendBuffer),
+                       reinterpret_cast<const VTK_TT*>(sendBuffer) + sendLength,
+                       reinterpret_cast<VTK_TT*>(recvBuffer) + offsets[0]));
+      }
+    return 1;
+    }
+
+  // I found another bug with Allgatherv where it sometimes fails to send data
+  // to all processes when one process is sending nothing.  That is not
+  // sufficient and I have not figured out what else needs to occur (although
+  // the TestMPIController test's random tries eventually catches it if you
+  // run it a bunch).  I sent a report to the OpenMPI mailing list.  Hopefully
+  // they will have it fixed in a future version.
+  for (int i = 0; i < this->NumberOfProcesses; i++)
+    {
+    if (recvLengths[i] < 1)
+      {
+      this->Superclass::AllGatherVVoidArray(sendBuffer, recvBuffer, sendLength,
+                                            recvLengths, offsets, type);
+      }
+    }
+#endif //OPEN_MPI
   vtkstd::vector<int> mpiRecvLengths, mpiOffsets;
   mpiRecvLengths.resize(numProc);  mpiOffsets.resize(numProc);
   for (int i = 0; i < numProc; i++)
