@@ -29,12 +29,12 @@ PURPOSE.  See the above copyright notice for more information.
 #include <math.h>
 
 //----------------------------------------------------------------------------
-vtkCxxRevisionMacro(vtkCarbonRenderWindow, "$Revision: 1.65 $");
+vtkCxxRevisionMacro(vtkCarbonRenderWindow, "$Revision: 1.70 $");
 vtkStandardNewMacro(vtkCarbonRenderWindow);
 
 //----------------------------------------------------------------------------
 #include <AvailabilityMacros.h>
-#if MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_3
+#if MAC_OS_X_VERSION_MAX_ALLOWED < 1030
 enum
 {
   kEventControlVisibilityChanged = 157
@@ -265,6 +265,15 @@ vtkCarbonRenderWindow::vtkCarbonRenderWindow()
 vtkCarbonRenderWindow::~vtkCarbonRenderWindow()
 {
   this->Finalize();
+  
+  vtkRenderer *ren;
+  vtkCollectionSimpleIterator rit;
+  this->Renderers->InitTraversal(rit);
+  while ( (ren = this->Renderers->GetNextRenderer(rit)) )
+    {
+    ren->SetRenderWindow(NULL);
+    }
+
   delete this->Internal;
 }
 
@@ -284,6 +293,7 @@ void vtkCarbonRenderWindow::DestroyWindow()
         (ren = this->Renderers->GetNextRenderer(rsit));)
     {
     ren->SetRenderWindow(NULL);
+    ren->SetRenderWindow(this);
     }
 
   /* finish OpenGL rendering */
@@ -513,7 +523,9 @@ void vtkCarbonRenderWindow::SetSize(int x, int y)
     this->Size[0] = x;
     this->Size[1] = y;
 
-    if(this->OffScreenRendering && this->Internal->OffScreenWindow)
+    if(this->OffScreenRendering && 
+       (this->Internal->OffScreenWindow 
+        || this->OffScreenUseFrameBuffer))
       {
       this->ResizeOffScreenWindow(x,y);
       }
@@ -1131,11 +1143,13 @@ WindowPtr vtkCarbonRenderWindow::GetRootWindow()
   // if not, then WindowId is set and we're using HIViews.
   // Instead of storing the RootWindow, we ask for it in case of a dynamic 
   // GUI where the root window can change
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_3
-  return this->RootWindow ? this->RootWindow : HIViewGetWindow(this->WindowId);
-#else
-  return this->RootWindow;
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= 1030
+  if(HIViewGetWindow != NULL && !this->RootWindow)
+    {
+    return HIViewGetWindow(this->WindowId);
+    }
 #endif
+  return this->RootWindow;
 }
 
 //----------------------------------------------------------------------------
