@@ -13,6 +13,7 @@ PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
 
+#import "vtkCocoaMacOSXSDKCompatibility.h" // Needed to support old SDKs
 #import "vtkCocoaRenderWindow.h"
 #import "vtkIdList.h"
 #import "vtkObjectFactory.h"
@@ -21,7 +22,7 @@ PURPOSE.  See the above copyright notice for more information.
 
 #include <vtksys/ios/sstream>
 
-vtkCxxRevisionMacro(vtkCocoaRenderWindow, "$Revision: 1.53 $");
+vtkCxxRevisionMacro(vtkCocoaRenderWindow, "$Revision: 1.59 $");
 vtkStandardNewMacro(vtkCocoaRenderWindow);
 
 
@@ -30,7 +31,8 @@ vtkCocoaRenderWindow::vtkCocoaRenderWindow()
 {
   // First, create the cocoa objects manager. The dictionary is empty so
   // essentially all objects are initialized to NULL.
-  this->SetCocoaManager(reinterpret_cast<void *>([NSMutableDictionary dictionary]));
+  NSMutableDictionary * cocoaManager = [NSMutableDictionary dictionary];
+  this->SetCocoaManager(reinterpret_cast<void *>(cocoaManager));
   
   this->WindowCreated = 0;
   this->ViewCreated = 0;
@@ -52,6 +54,15 @@ vtkCocoaRenderWindow::~vtkCocoaRenderWindow()
     this->ShowCursor();
     }
   this->Finalize();
+  
+  vtkRenderer *renderer;
+  vtkCollectionSimpleIterator rsit;
+
+  for ( this->Renderers->InitTraversal(rsit);
+        (renderer = this->Renderers->GetNextRenderer(rsit));)
+    {
+    renderer->SetRenderWindow(0);
+    }
 
   if (this->Capabilities)
     {
@@ -115,6 +126,7 @@ void vtkCocoaRenderWindow::DestroyWindow()
           (ren = this->Renderers->GetNextRenderer(rsit));)
       {
       ren->SetRenderWindow(NULL);
+      ren->SetRenderWindow(this);
       }
     
     this->SetContextId(NULL);
@@ -133,7 +145,7 @@ void vtkCocoaRenderWindow::SetWindowName( const char * _arg )
     {
     NSString* winTitleStr;
 
-#if defined(MAC_OS_X_VERSION_10_4) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4)
+#if MAC_OS_X_VERSION_MIN_REQUIRED >= 1040
     winTitleStr = [NSString stringWithCString:_arg encoding:NSASCIIStringEncoding];
 #else
     winTitleStr = [NSString stringWithCString:_arg];
@@ -299,7 +311,7 @@ void vtkCocoaRenderWindow::SetSize(int x, int y)
         {
         resizing = 1;
         // VTK measures in pixels, but NSWindow/NSView measure in points; convert.
-        NSSize theSize = NSMakeSize((double)x / this->ScaleFactor, (double)y / this->ScaleFactor);
+        NSSize theSize = NSMakeSize((CGFloat)x / this->ScaleFactor, (CGFloat)y / this->ScaleFactor);
         [(NSWindow*)this->GetWindowId() setContentSize:theSize];
         resizing = 0;
         }
@@ -334,7 +346,7 @@ void vtkCocoaRenderWindow::SetPosition(int x, int y)
       if (!resizing)
         {
         resizing = 1;
-        NSPoint origin = NSMakePoint((float)x, (float)y);
+        NSPoint origin = NSMakePoint((CGFloat)x, (CGFloat)y);
         [(NSWindow*)this->GetWindowId() setFrameOrigin:origin];
         resizing = 0;
         }
@@ -417,13 +429,13 @@ void vtkCocoaRenderWindow::WindowConfigure()
 //----------------------------------------------------------------------------
 void vtkCocoaRenderWindow::SetupPixelFormat(void*, void*, int, int, int)
 {
-  vtkErrorMacro(<< "vtkCocoaRenderWindow::SetupPixelFormat - IMPLEMENT\n");
+  vtkErrorMacro(<< "vtkCocoaRenderWindow::SetupPixelFormat - IMPLEMENT");
 }
 
 //----------------------------------------------------------------------------
 void vtkCocoaRenderWindow::SetupPalette(void*)
 {
-  vtkErrorMacro(<< "vtkCocoaRenderWindow::SetupPalette - IMPLEMENT\n");
+  vtkErrorMacro(<< "vtkCocoaRenderWindow::SetupPalette - IMPLEMENT");
 }
 
 //----------------------------------------------------------------------------
@@ -434,7 +446,7 @@ void vtkCocoaRenderWindow::CreateAWindow()
   
   // Get the screen's scale factor.
   // It will be used to create the window if not created yet.
-#if defined(MAC_OS_X_VERSION_10_4) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4)
+#if MAC_OS_X_VERSION_MIN_REQUIRED >= 1040
   this->ScaleFactor = [[NSScreen mainScreen] userSpaceScaleFactor];
 #endif
 
@@ -472,10 +484,10 @@ void vtkCocoaRenderWindow::CreateAWindow()
       }
 
     // VTK measures in pixels, but NSWindow/NSView measure in points; convert.
-    NSRect ctRect = NSMakeRect((float)this->Position[0],
-                               (float)this->Position[1],
-                               (float)this->Size[0] / this->ScaleFactor,
-                               (float)this->Size[1] / this->ScaleFactor);
+    NSRect ctRect = NSMakeRect((CGFloat)this->Position[0],
+                               (CGFloat)this->Position[1],
+                               (CGFloat)this->Size[0] / this->ScaleFactor,
+                               (CGFloat)this->Size[1] / this->ScaleFactor);
 
     NSWindow* theWindow = [[NSWindow alloc]
                            initWithContentRect:ctRect
@@ -498,7 +510,7 @@ void vtkCocoaRenderWindow::CreateAWindow()
   
   // Always use the scaling factor from the window once it is created.
   // The screen and the window might possibly have different scaling factors, though unlikely.
-#if defined(MAC_OS_X_VERSION_10_4) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4)
+#if MAC_OS_X_VERSION_MIN_REQUIRED >= 1040
   if (this->GetWindowId())
     {
     this->ScaleFactor = [(NSWindow*)this->GetWindowId() userSpaceScaleFactor];
@@ -510,8 +522,8 @@ void vtkCocoaRenderWindow::CreateAWindow()
     {
     // VTK measures in pixels, but NSWindow/NSView measure in points; convert.
     NSRect glRect = NSMakeRect(0.0, 0.0,
-                               (float)this->Size[0] / this->ScaleFactor,
-                               (float)this->Size[1] / this->ScaleFactor);
+                               (CGFloat)this->Size[0] / this->ScaleFactor,
+                               (CGFloat)this->Size[1] / this->ScaleFactor);
     vtkCocoaGLView *glView = [[[vtkCocoaGLView alloc] initWithFrame:glRect] autorelease];
     [(NSWindow*)this->GetWindowId() setContentView:glView];
     
@@ -534,7 +546,7 @@ void vtkCocoaRenderWindow::CreateAWindow()
   if (this->WindowCreated)
     {
     NSString * winName = [NSString stringWithFormat:@"Visualization Toolkit - Cocoa #%i", count++];
-#if defined(MAC_OS_X_VERSION_10_4) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4)
+#if MAC_OS_X_VERSION_MIN_REQUIRED >= 1040
     this->SetWindowName([winName cStringUsingEncoding:NSASCIIStringEncoding]);
 #else
     this->SetWindowName([winName cString]);
@@ -629,11 +641,11 @@ void vtkCocoaRenderWindow::Initialize ()
     // create on screen
     if(!this->OnScreenInitialized)
       {
-      this->CreateAWindow();
       this->OnScreenInitialized = 1;
+      this->CreateAWindow();
       }
     }
-  if(this->OnScreenInitialized)
+  if((this->OnScreenInitialized) && (this->Mapped))
     {
     // the error "invalid drawable" in the console from this call can appear
     // but only early in the app's lifetime (ie sometime during launch)
@@ -691,7 +703,7 @@ int *vtkCocoaRenderWindow::GetScreenSize()
   NSRect screenRect = [screen frame];
   
   // VTK measures in pixels, but NSWindow/NSView measure in points; convert.
-#if defined(MAC_OS_X_VERSION_10_4) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4)
+#if MAC_OS_X_VERSION_MIN_REQUIRED >= 1040
   this->Size[0] = (int)round(NSWidth(screenRect) * [screen userSpaceScaleFactor]);
   this->Size[1] = (int)round(NSHeight(screenRect) * [screen userSpaceScaleFactor]);
 #else
@@ -954,20 +966,20 @@ void vtkCocoaRenderWindow::SetCocoaManager(void *manager)
     NSMutableDictionary* cocoaManager = 
       reinterpret_cast<NSMutableDictionary *>(manager);
     #ifdef __OBJC_GC__
-      #error VTK does not yet support garbage collection
-    #else
-      [cocoaManager release];
+      [[NSGarbageCollector defaultCollector]
+        enableCollectorForPointer:cocoaManager];
     #endif
+    [cocoaManager release];
     }
   else
     {
     NSMutableDictionary* cocoaManager = 
       reinterpret_cast<NSMutableDictionary *>(manager);
     #ifdef __OBJC_GC__
-      #error VTK does not yet support garbage collection
-    #else
-      [cocoaManager retain];
+      [[NSGarbageCollector defaultCollector]
+        disableCollectorForPointer:cocoaManager];
     #endif
+    [cocoaManager retain];
     }
   this->CocoaManager = manager;
 }
