@@ -12,10 +12,11 @@
      PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
-/*----------------------------------------------------------------------------
- Copyright (c) Sandia Corporation
- See Copyright.txt or http://www.paraview.org/HTML/Copyright.html for details.
-----------------------------------------------------------------------------*/
+/*-------------------------------------------------------------------------
+  Copyright 2008 Sandia Corporation.
+  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+  the U.S. Government retains certain rights in this software.
+-------------------------------------------------------------------------*/
 
 #include "vtkDynamic2DLabelMapper.h"
 
@@ -25,6 +26,7 @@
 #include "vtkDataArray.h"
 #include "vtkDataSet.h"
 #include "vtkExecutive.h"
+#include "vtkGraph.h"
 #include "vtkIdTypeArray.h"
 #include "vtkInformation.h"
 #include "vtkKdTree.h"
@@ -49,7 +51,7 @@ using vtksys_ios::ofstream;
 # define SNPRINTF snprintf
 #endif
 
-vtkCxxRevisionMacro(vtkDynamic2DLabelMapper, "$Revision: 1.5 $");
+vtkCxxRevisionMacro(vtkDynamic2DLabelMapper, "$Revision: 1.11 $");
 vtkStandardNewMacro(vtkDynamic2DLabelMapper);
 
 //----------------------------------------------------------------------------
@@ -122,7 +124,7 @@ void vtkDynamic2DLabelMapper::RenderOpaqueGeometry(vtkViewport *viewport,
   vtkAbstractArray *abstractData;
   vtkDataArray *numericData;
   vtkStringArray *stringData;
-  vtkDataSet *input=this->GetInput();
+  vtkDataObject *input = this->GetExecutive()->GetInputData(0, 0);
 
   if ( ! input )
     {
@@ -140,8 +142,16 @@ void vtkDynamic2DLabelMapper::RenderOpaqueGeometry(vtkViewport *viewport,
   input->Update();
 
   // Input might have changed
-  input = this->GetInput();
-  vtkPointData *pd=input->GetPointData();
+  input = this->GetExecutive()->GetInputData(0, 0);
+  vtkDataSet *dsInput = vtkDataSet::SafeDownCast(input);
+  vtkGraph *gInput = vtkGraph::SafeDownCast(input);
+  if (!dsInput && !gInput)
+    {
+    vtkErrorMacro(<<"Input must be vtkDataSet or vtkGraph.");
+    return;
+    }
+  vtkDataSetAttributes *pd = 
+    dsInput ? dsInput->GetPointData() : gInput->GetVertexData();
 
   // Check to see whether we have to rebuild everything
   if ( this->GetMTime() > this->BuildTime || 
@@ -227,7 +237,19 @@ void vtkDynamic2DLabelMapper::RenderOpaqueGeometry(vtkViewport *viewport,
       }
     else if ( !stringData )
       {
-      vtkErrorMacro(<<"Need input data to render labels (3)");
+      if (this->FieldDataName)
+        {
+        vtkWarningMacro(<< "Could not find label array ("
+                        << this->FieldDataName << ") "
+                        << "in input.");
+        }
+      else 
+        {
+        vtkWarningMacro(<< "Could not find label array ("
+                        << "index " << this->FieldDataArray << ") "
+                        << "in input.");
+        }
+
       return;
       }
 
@@ -296,7 +318,8 @@ void vtkDynamic2DLabelMapper::RenderOpaqueGeometry(vtkViewport *viewport,
       vtkDebugMacro(<<"Using default format string " << FormatString.c_str());
       } // Done building default format string
 
-    this->NumberOfLabels = input->GetNumberOfPoints();
+    this->NumberOfLabels = 
+      dsInput ? dsInput->GetNumberOfPoints() : gInput->GetNumberOfVertices();
     if ( this->NumberOfLabels > this->NumberOfLabelsAllocated )
       {
       // delete old stuff
@@ -449,7 +472,14 @@ void vtkDynamic2DLabelMapper::RenderOpaqueGeometry(vtkViewport *viewport,
       {
       double* dc;
       double pti[3];
-      input->GetPoint(i, pti);
+      if (dsInput)
+        {
+        dsInput->GetPoint(i, pti);
+        }
+      else
+        {
+        gInput->GetPoint(i, pti);
+        }
       coord->SetValue(pti);
       dc = coord->GetComputedDoubleDisplayValue(0);
       pts->InsertNextPoint(dc[0], dc[1], 0);
@@ -475,10 +505,14 @@ void vtkDynamic2DLabelMapper::RenderOpaqueGeometry(vtkViewport *viewport,
       }
     
     // If the array is found, sort it and rearrange the corresponding index array.
-    vtkAbstractArray* arr = this->GetInputAbstractArrayToProcess(0, input);
-    if (arr)
+    vtkAbstractArray* inputArr = this->GetInputAbstractArrayToProcess(0, input);
+    if (inputArr)
       {
+      // Don't sort the original array, instead make a copy.
+      vtkAbstractArray* arr = vtkAbstractArray::CreateArray(inputArr->GetDataType());
+      arr->DeepCopy(inputArr);
       vtkSortDataArray::Sort(arr, index);
+      arr->Delete();
       }
     
     // We normally go from highest (at the end) to lowest (at the beginning).
@@ -487,7 +521,7 @@ void vtkDynamic2DLabelMapper::RenderOpaqueGeometry(vtkViewport *viewport,
     vtkIdType begin = this->NumberOfLabels - 1;
     vtkIdType end = -1;
     vtkIdType step = -1;
-    if ((this->ReversePriority && arr) || (!this->ReversePriority && !arr))
+    if ((this->ReversePriority && inputArr) || (!this->ReversePriority && !inputArr))
       {
       begin = 0;
       end = this->NumberOfLabels;
@@ -504,8 +538,8 @@ void vtkDynamic2DLabelMapper::RenderOpaqueGeometry(vtkViewport *viewport,
         float* ptj = reinterpret_cast<float*>(pts->GetVoidPointer(3*indexJ));
         float absX = (pti[0] - ptj[0]) > 0 ? (pti[0] - ptj[0]) : -(pti[0] - ptj[0]);
         float absY = (pti[1] - ptj[1]) > 0 ? (pti[1] - ptj[1]) : -(pti[1] - ptj[1]);
-        float xScale = 2*absX/(float)(this->LabelWidth[indexI] + this->LabelWidth[indexJ]);
-        float yScale = 2*absY/(float)(this->LabelHeight[indexI] + this->LabelHeight[indexJ]);
+        float xScale = 2*absX/(this->LabelWidth[indexI] + this->LabelWidth[indexJ]);
+        float yScale = 2*absY/(this->LabelHeight[indexI] + this->LabelHeight[indexJ]);
         float maxScale = xScale < yScale ? yScale : xScale;
         if (maxScale < this->Cutoff[indexJ] && maxScale < this->Cutoff[indexI])
           {
@@ -542,7 +576,14 @@ void vtkDynamic2DLabelMapper::RenderOpaqueGeometry(vtkViewport *viewport,
 
   for (i = 0; i < this->NumberOfLabels; i++)
     {
-    input->GetPoint(i,x);
+    if (dsInput)
+      {
+      dsInput->GetPoint(i,x);
+      }
+    else
+      {
+      gInput->GetPoint(i,x);
+      }
     if ((1.0 / scale) < this->Cutoff[i])
       {
       actor->GetPositionCoordinate()->SetCoordinateSystemToWorld();
@@ -587,8 +628,10 @@ void vtkDynamic2DLabelMapper::RenderOverlay(vtkViewport *viewport,
 {
   int i;
   double x[3];
-  vtkDataSet *input=this->GetInput();
-  vtkIdType numPts = input->GetNumberOfPoints();
+  vtkDataObject *input = this->GetExecutive()->GetInputData(0, 0);
+  vtkGraph *gInput = vtkGraph::SafeDownCast(input);
+  vtkDataSet *dsInput = vtkDataSet::SafeDownCast(input);
+  vtkIdType numPts = dsInput ? dsInput->GetNumberOfPoints() : gInput->GetNumberOfVertices();
 
   // Determine the current scale
   double scale = this->GetCurrentScale(viewport) / this->ReferenceScale;
@@ -603,7 +646,14 @@ void vtkDynamic2DLabelMapper::RenderOverlay(vtkViewport *viewport,
     }
   for (i=0; i<this->NumberOfLabels && i<numPts; i++)
     {
-    input->GetPoint(i,x);
+    if (dsInput)
+      {
+      dsInput->GetPoint(i, x);
+      }
+    else
+      {
+      gInput->GetPoint(i, x);
+      }
     actor->SetPosition(x);
     double* display = actor->GetPositionCoordinate()->GetComputedDoubleDisplayValue(viewport);
     double screenX = display[0];
