@@ -22,7 +22,7 @@
 #include "vtkPointData.h"
 #include "vtkMath.h"
 
-vtkCxxRevisionMacro(vtkMapper, "$Revision: 1.121 $");
+vtkCxxRevisionMacro(vtkMapper, "$Revision: 1.124 $");
 
 // Initialize static member that controls global immediate mode rendering
 static int vtkMapperGlobalImmediateModeRendering = 0;
@@ -64,6 +64,8 @@ vtkMapper::vtkMapper()
   this->InterpolateScalarsBeforeMapping = 0;
   this->ColorCoordinates = 0;
   this->ColorTextureMap = 0;
+
+  this->ForceCompileOnly=0;
 }
 
 vtkMapper::~vtkMapper()
@@ -90,12 +92,10 @@ vtkMapper::~vtkMapper()
 // (Xmin,Xmax,Ymin,Ymax,Zmin,Zmax).
 double *vtkMapper::GetBounds()
 {
-  static double bounds[] = {-1.0,1.0, -1.0,1.0, -1.0,1.0};
-
   vtkDataSet *input = this->GetInput();
   if ( ! input ) 
     {
-    return bounds;
+      vtkMath::UninitializeBounds(this->Bounds);
     }
   else
     {
@@ -104,8 +104,8 @@ double *vtkMapper::GetBounds()
       this->Update();
       }
     input->GetBounds(this->Bounds);
-    return this->Bounds;
     }
+  return this->Bounds;
 }
 
 vtkDataSet *vtkMapper::GetInput()
@@ -118,6 +118,16 @@ vtkDataSet *vtkMapper::GetInput()
     this->GetExecutive()->GetInputData(0, 0));
 }
 
+void vtkMapper::SetForceCompileOnly(int value)
+{
+  if(this->ForceCompileOnly!=value)
+    {
+      this->ForceCompileOnly=value;
+      // make sure we don't call this->Modified();
+      //      this->Modified();
+    }
+}
+
 void vtkMapper::SetGlobalImmediateModeRendering(int val)
 {
   if (val == vtkMapperGlobalImmediateModeRendering)
@@ -516,7 +526,7 @@ void vtkMapperCreateColorTextureCoordinates(T* input, float* output,
       sum = 0;
       for (j = 0; j < numComps; ++j)
         {
-        tmp = (double)(*input);  
+        tmp = static_cast<double>(*input);
         sum += (tmp * tmp);
         ++input;
         }
@@ -536,7 +546,7 @@ void vtkMapperCreateColorTextureCoordinates(T* input, float* output,
     input += component;
     for (i = 0; i < num; ++i)
       {
-      output[i] = k * ((float)(*input) - range[0]);
+      output[i] = k * (static_cast<double>(*input) - range[0]);
       if (output[i] > 1.0)
         {
         output[i] = 1.0;
@@ -583,13 +593,13 @@ void vtkMapper::MapScalarsToTexture(vtkDataArray* scalars, double alpha)
     // Get the texture map from the lookup table.
     // Create a dummy ramp of scalars.
     // In the future, we could extend vtkScalarsToColors.
-    double k = (range[1]-range[0]) / (double)(ColorTextureMapSize-1);
+    double k = (range[1]-range[0]) / (ColorTextureMapSize-1);
     vtkFloatArray* tmp = vtkFloatArray::New();
     tmp->SetNumberOfTuples(ColorTextureMapSize);
     float* ptr = tmp->GetPointer(0);
     for (int i = 0; i < ColorTextureMapSize; ++i)
       {
-      *ptr = range[0] + ((float)(i)) * k;
+      *ptr = range[0] + i * k;
       ++ptr;
       }
     this->ColorTextureMap = vtkImageData::New();
@@ -673,6 +683,10 @@ void vtkMapper::PrintSelf(ostream& os, vtkIndent indent)
 
   os << indent << "Immediate Mode Rendering: " 
     << (this->ImmediateModeRendering ? "On\n" : "Off\n");
+
+   os << indent << "Force compile only for display lists: " 
+    << (this->ForceCompileOnly ? "On\n" : "Off\n");
+
   os << indent << "Global Immediate Mode Rendering: " << 
     (vtkMapperGlobalImmediateModeRendering ? "On\n" : "Off\n");
 
