@@ -1,38 +1,40 @@
 /*=========================================================================
 
-Program:   Visualization Toolkit
-Module:    $RCSfile: vtkOpenGLRenderWindow.cxx,v $
+  Program:   Visualization Toolkit
+  Module:    $RCSfile: vtkOpenGLRenderWindow.cxx,v $
 
-Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-All rights reserved.
-See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
+  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+  All rights reserved.
+  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
 
-This software is distributed WITHOUT ANY WARRANTY; without even
-the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-PURPOSE.  See the above copyright notice for more information.
+     This software is distributed WITHOUT ANY WARRANTY; without even
+     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
+     PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
 #include "vtkOpenGLRenderWindow.h"
-#include "vtkOpenGLRenderer.h"
-#include "vtkOpenGLProperty.h"
-#include "vtkOpenGLTexture.h"
-#include "vtkOpenGLCamera.h"
-#include "vtkOpenGLLight.h"
-#include "vtkOpenGLActor.h"
-#include "vtkOpenGLPolyDataMapper.h"
+#include "assert.h"
+#include "vtkFloatArray.h"
+#include "vtkgl.h"
 #include "vtkIdList.h"
 #include "vtkObjectFactory.h"
-#include "vtkFloatArray.h"
-#include "vtkUnsignedCharArray.h"
-#include "assert.h"
+#include "vtkOpenGLActor.h"
+#include "vtkOpenGLCamera.h"
 #include "vtkOpenGLExtensionManager.h"
-#include "vtkgl.h"
+#include "vtkOpenGLLight.h"
+#include "vtkOpenGLPolyDataMapper.h"
+#include "vtkOpenGLProperty.h"
+#include "vtkOpenGLRenderer.h"
+#include "vtkOpenGLTexture.h"
+#include "vtkUnsignedCharArray.h"
 
 #ifndef VTK_IMPLEMENT_MESA_CXX
 
-vtkCxxRevisionMacro(vtkOpenGLRenderWindow, "$Revision: 1.87.10.1 $");
+vtkCxxRevisionMacro(vtkOpenGLRenderWindow, "$Revision: 1.99 $");
 #endif
 
+vtkCxxSetObjectMacro(vtkOpenGLRenderWindow, ExtensionManager, vtkOpenGLExtensionManager);
+
 #define MAX_LIGHTS 8
 
 // Initialize static member that controls global maximum number of multisamples
@@ -51,6 +53,7 @@ int vtkOpenGLRenderWindow::GetGlobalMaximumNumberOfMultiSamples()
 
 vtkOpenGLRenderWindow::vtkOpenGLRenderWindow()
 {
+  this->ExtensionManager = 0;
   this->MultiSamples = vtkOpenGLRenderWindowGlobalMaximumNumberOfMultiSamples;
   this->TextureResourceIds = vtkIdList::New();
   if ( this->WindowName ) 
@@ -63,43 +66,53 @@ vtkOpenGLRenderWindow::vtkOpenGLRenderWindow()
   this->BackLeftBuffer=static_cast<unsigned int>(GL_BACK_LEFT);
   this->BackRightBuffer=static_cast<unsigned int>(GL_BACK_RIGHT);
   this->FrontLeftBuffer=static_cast<unsigned int>(GL_FRONT_LEFT);
+  this->FrontRightBuffer=static_cast<unsigned int>(GL_FRONT_RIGHT);
+  this->BackBuffer=static_cast<unsigned int>(GL_BACK);
+  this->FrontBuffer=static_cast<unsigned int>(GL_FRONT);
+  
+  this->LastGraphicError=static_cast<unsigned int>(GL_NO_ERROR);
 }
 
 // free up memory & close the window
 vtkOpenGLRenderWindow::~vtkOpenGLRenderWindow()
 {
   this->TextureResourceIds->Delete();
+  if (this->ExtensionManager)
+    {
+    this->ExtensionManager->SetRenderWindow(0);
+    }
+  this->SetExtensionManager(0);
 }
 
 // ----------------------------------------------------------------------------
 // Description:
 // Return the OpenGL name of the back left buffer.
 // It is GL_BACK_LEFT if GL is bound to the window-system-provided
-// framebuffer. It is vtkgl::COLOR_ATTACHMENT0_EXT+i if GL is bound to an
+// framebuffer. It is vtkgl::COLOR_ATTACHMENT0_EXT if GL is bound to an
 // application-created framebuffer object (GPU-based offscreen rendering)
 // It is used by vtkOpenGLCamera.
 unsigned int vtkOpenGLRenderWindow::GetBackLeftBuffer()
 {
   return this->BackLeftBuffer;
 }
-  
+
 // ----------------------------------------------------------------------------
 // Description:
 // Return the OpenGL name of the back right buffer.
 // It is GL_BACK_RIGHT if GL is bound to the window-system-provided
-// framebuffer. It is vtkgl::COLOR_ATTACHMENT0_EXT+i if GL is bound to an
+// framebuffer. It is vtkgl::COLOR_ATTACHMENT0_EXT+1 if GL is bound to an
 // application-created framebuffer object (GPU-based offscreen rendering)
 // It is used by vtkOpenGLCamera.
 unsigned int vtkOpenGLRenderWindow::GetBackRightBuffer()
 {
   return this->BackRightBuffer;
 }
-  
+
 // ----------------------------------------------------------------------------
 // Description:
 // Return the OpenGL name of the front left buffer.
-// It is GL_FRONT if GL is bound to the window-system-provided
-// framebuffer. It is vtkgl::COLOR_ATTACHMENT0_EXT+i if GL is bound to an
+// It is GL_FRONT_LEFT if GL is bound to the window-system-provided
+// framebuffer. It is vtkgl::COLOR_ATTACHMENT0_EXT if GL is bound to an
 // application-created framebuffer object (GPU-based offscreen rendering)
 // It is used by vtkOpenGLCamera.
 unsigned int vtkOpenGLRenderWindow::GetFrontLeftBuffer()
@@ -107,6 +120,42 @@ unsigned int vtkOpenGLRenderWindow::GetFrontLeftBuffer()
   return this->FrontLeftBuffer;
 }
 
+// ----------------------------------------------------------------------------
+// Description:
+// Return the OpenGL name of the front right buffer.
+// It is GL_FRONT_RIGHT if GL is bound to the window-system-provided
+// framebuffer. It is vtkgl::COLOR_ATTACHMENT0_EXT+1 if GL is bound to an
+// application-created framebuffer object (GPU-based offscreen rendering)
+// It is used by vtkOpenGLCamera.
+unsigned int vtkOpenGLRenderWindow::GetFrontRightBuffer()
+{
+  return this->FrontRightBuffer;
+}
+
+// ----------------------------------------------------------------------------
+// Description:
+// Return the OpenGL name of the back left buffer.
+// It is GL_BACK if GL is bound to the window-system-provided
+// framebuffer. It is vtkgl::COLOR_ATTACHMENT0_EXT if GL is bound to an
+// application-created framebuffer object (GPU-based offscreen rendering)
+// It is used by vtkOpenGLCamera.
+unsigned int vtkOpenGLRenderWindow::GetBackBuffer()
+{
+  return this->BackBuffer;
+}
+
+// ----------------------------------------------------------------------------
+// Description:
+// Return the OpenGL name of the front left buffer.
+// It is GL_FRONT if GL is bound to the window-system-provided
+// framebuffer. It is vtkgl::COLOR_ATTACHMENT0_EXT if GL is bound to an
+// application-created framebuffer object (GPU-based offscreen rendering)
+// It is used by vtkOpenGLCamera.
+unsigned int vtkOpenGLRenderWindow::GetFrontBuffer()
+{
+  return this->FrontBuffer;
+}
+
 // Update system if needed due to stereo rendering.
 void vtkOpenGLRenderWindow::StereoUpdate(void)
 {
@@ -158,13 +207,47 @@ void vtkOpenGLRenderWindow::StereoUpdate(void)
 
 void vtkOpenGLRenderWindow::OpenGLInit()
 {
+  // When a new OpenGL context is created, we want to get rid of the old OpenGL
+  // extension manager, if any.
+  this->SetExtensionManager(0);
+
+  this->ContextCreationTime.Modified();
   glMatrixMode( GL_MODELVIEW );
   glDepthFunc( GL_LEQUAL );
   glEnable( GL_DEPTH_TEST );
   glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 
   // initialize blending for transparency
-  glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
+  
+  // We have to set the function pointer to null, otherwise the following
+  // scenario would fail on Windows (and maybe other kind of configurations):
+  // 1. Render onscreen on GPU that supports OpenGL 1.4
+  // 2. Switch to offscreen with GDI Windows implementation (1.1)
+  vtkgl::BlendFuncSeparate=0;
+  
+  // Try to initialize vtkgl::BlendFuncSeparate() if available.
+  vtkOpenGLExtensionManager *extensions = this->GetExtensionManager();
+  if (extensions->ExtensionSupported("GL_VERSION_1_4"))
+    {
+    extensions->LoadExtension("GL_VERSION_1_4");
+    }
+  else
+    {
+    if (extensions->ExtensionSupported("GL_EXT_blend_func_separate"))
+      {
+      extensions->LoadCorePromotedExtension("GL_EXT_blend_func_separate");
+      }
+    }
+  
+  if(vtkgl::BlendFuncSeparate!=0)
+    {
+    vtkgl::BlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA,
+                             GL_ONE,GL_ONE_MINUS_SRC_ALPHA);
+    }
+  else
+    {
+    glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
+    }
   glEnable(GL_BLEND);
 
   if (this->PointSmoothing)
@@ -214,7 +297,7 @@ int vtkOpenGLRenderWindow::GetDepthBufferSize()
     this->MakeCurrent();
     size = 0;
     glGetIntegerv( GL_DEPTH_BITS, &size );
-    return (int) size;
+    return static_cast<int>(size);
     }
   else
     {
@@ -240,13 +323,13 @@ int vtkOpenGLRenderWindow::GetColorBufferSizes(int *rgba)
     {
     this->MakeCurrent();
     glGetIntegerv( GL_RED_BITS, &size );
-    rgba[0] = (int)size;
+    rgba[0] = static_cast<int>(size);
     glGetIntegerv( GL_GREEN_BITS, &size  );
-    rgba[1] = (int)size;
+    rgba[1] = static_cast<int>(size);
     glGetIntegerv( GL_BLUE_BITS, &size );
-    rgba[2] = (int)size;
+    rgba[2] = static_cast<int>(size);
     glGetIntegerv( GL_ALPHA_BITS, &size );
-    rgba[3] = (int)size;
+    rgba[3] = static_cast<int>(size);
     return rgba[0]+rgba[1]+rgba[2]+rgba[3];
     }
   else
@@ -372,7 +455,10 @@ int vtkOpenGLRenderWindow::GetPixelData(int x1, int y1,
     }
 
   // Must clear previous errors first.
-  while(glGetError() != GL_NO_ERROR);
+  while(glGetError() != GL_NO_ERROR)
+    {
+    ;
+    }
 
   if (front)
     {
@@ -486,7 +572,10 @@ int vtkOpenGLRenderWindow::SetPixelData(int x1, int y1, int x2, int y2,
 
   // Error checking
   // Must clear previous errors first.
-  while(glGetError() != GL_NO_ERROR);
+  while(glGetError() != GL_NO_ERROR)
+    {
+    ;
+    }
 
   if (front)
     {
@@ -551,8 +640,8 @@ int vtkOpenGLRenderWindow::SetPixelData(int x1, int y1, int x2, int y2,
     glMatrixMode( GL_PROJECTION );
     glPushMatrix();
     glLoadIdentity();
-    glRasterPos3f( (2.0 * (GLfloat)(x_low) / this->Size[0] - 1),
-                   (2.0 * (GLfloat)(yloop) / this->Size[1] - 1),
+    glRasterPos3f( (2.0 * static_cast<GLfloat>(x_low) / this->Size[0] - 1),
+                   (2.0 * static_cast<GLfloat>(yloop) / this->Size[1] - 1),
                    -1.0 );
     glMatrixMode( GL_PROJECTION );
     glPopMatrix();
@@ -581,8 +670,8 @@ int vtkOpenGLRenderWindow::SetPixelData(int x1, int y1, int x2, int y2,
   glMatrixMode( GL_PROJECTION );
   glPushMatrix();
   glLoadIdentity();
-  glRasterPos3f( (2.0 * (GLfloat)(x_low) / this->Size[0] - 1), 
-                 (2.0 * (GLfloat)(y_low) / this->Size[1] - 1),
+  glRasterPos3f( (2.0 * static_cast<GLfloat>(x_low) / this->Size[0] - 1), 
+                 (2.0 * static_cast<GLfloat>(y_low) / this->Size[1] - 1),
                  -1.0 );
   glMatrixMode( GL_PROJECTION );
   glPopMatrix();
@@ -727,7 +816,10 @@ int vtkOpenGLRenderWindow::GetRGBAPixelData(int x1, int y1, int x2, int y2,
 
   // Error checking
   // Must clear previous errors first.
-  while(glGetError() != GL_NO_ERROR);
+  while(glGetError() != GL_NO_ERROR)
+    {
+    ;
+    }
 
   if (front)
     {
@@ -820,7 +912,10 @@ int vtkOpenGLRenderWindow::SetRGBAPixelData(int x1, int y1, int x2, int y2,
 
   // Error checking
   // Must clear previous errors first.
-  while(glGetError() != GL_NO_ERROR);
+  while(glGetError() != GL_NO_ERROR)
+    {
+    ;
+    }  
 
   if (front)
     {
@@ -865,8 +960,8 @@ int vtkOpenGLRenderWindow::SetRGBAPixelData(int x1, int y1, int x2, int y2,
   glMatrixMode( GL_PROJECTION );
   glPushMatrix();
   glLoadIdentity();
-  glRasterPos3f( (2.0 * (GLfloat)(x_low) / this->Size[0] - 1), 
-                 (2.0 * (GLfloat)(y_low) / this->Size[1] - 1),
+  glRasterPos3f( (2.0 * static_cast<GLfloat>(x_low) / this->Size[0] - 1), 
+                 (2.0 * static_cast<GLfloat>(y_low) / this->Size[1] - 1),
                  -1.0 );
   glMatrixMode( GL_PROJECTION );
   glPopMatrix();
@@ -1022,7 +1117,10 @@ int vtkOpenGLRenderWindow::GetRGBACharPixelData(int x1, int y1,
 
 
   // Must clear previous errors first.
-  while(glGetError() != GL_NO_ERROR);
+  while(glGetError() != GL_NO_ERROR)
+    {
+    ;
+    }
 
   if (front)
     {
@@ -1118,7 +1216,10 @@ int vtkOpenGLRenderWindow::SetRGBACharPixelData(int x1, int y1, int x2,
 
   // Error checking
   // Must clear previous errors first.
-  while(glGetError() != GL_NO_ERROR);
+  while(glGetError() != GL_NO_ERROR)
+    {
+    ;
+    }
 
   if (front)
     {
@@ -1166,14 +1267,17 @@ int vtkOpenGLRenderWindow::SetRGBACharPixelData(int x1, int y1, int x2,
   glMatrixMode( GL_PROJECTION );
   glPushMatrix();
   glLoadIdentity();
-  glRasterPos3f( (2.0 * (GLfloat)(x_low) / this->Size[0] - 1),
-                 (2.0 * (GLfloat)(y_low) / this->Size[1] - 1),
+  glRasterPos3f( (2.0 * static_cast<GLfloat>(x_low) / this->Size[0] - 1),
+                 (2.0 * static_cast<GLfloat>(y_low) / this->Size[1] - 1),
                  -1.0 );
   glMatrixMode( GL_PROJECTION );
   glPopMatrix();
   glMatrixMode( GL_MODELVIEW );
   glPopMatrix();
 
+  // Disable writing on the z-buffer.
+  glDepthMask(GL_FALSE);
+  glDisable(GL_DEPTH_TEST);
 
   glDisable( GL_SCISSOR_TEST );
   
@@ -1194,6 +1298,10 @@ int vtkOpenGLRenderWindow::SetRGBACharPixelData(int x1, int y1, int x2,
                   data);
     }
 
+  // Renenable writing on the z-buffer.
+  glDepthMask(GL_TRUE);
+  glEnable(GL_DEPTH_TEST);
+
   // This seems to be necessary for the image to show up
   glFlush();  
 
@@ -1241,7 +1349,10 @@ int vtkOpenGLRenderWindow::GetZbufferData( int x1, int y1, int x2, int y2,
 
   // Error checking
   // Must clear previous errors first.
-  while(glGetError() != GL_NO_ERROR);
+  while(glGetError() != GL_NO_ERROR)
+    {
+    ;
+    }
 
   // Turn of texturing in case it is on - some drivers have a problem
   // getting / setting pixels with texturing enabled.
@@ -1342,7 +1453,10 @@ int vtkOpenGLRenderWindow::SetZbufferData( int x1, int y1, int x2, int y2,
 
   // Error checking
   // Must clear previous errors first.
-  while(glGetError() != GL_NO_ERROR);
+  while(glGetError() != GL_NO_ERROR)
+    {
+    ;
+    }
 
   glViewport(0, 0, this->Size[0], this->Size[1]);
   glMatrixMode( GL_MODELVIEW );
@@ -1351,8 +1465,8 @@ int vtkOpenGLRenderWindow::SetZbufferData( int x1, int y1, int x2, int y2,
   glMatrixMode( GL_PROJECTION );
   glPushMatrix();
   glLoadIdentity();
-  glRasterPos2f( 2.0 * (GLfloat)(x_low) / this->Size[0] - 1, 
-                 2.0 * (GLfloat)(y_low) / this->Size[1] - 1);
+  glRasterPos2f( 2.0 * static_cast<GLfloat>(x_low) / this->Size[0] - 1, 
+                 2.0 * static_cast<GLfloat>(y_low) / this->Size[1] - 1);
   glMatrixMode( GL_PROJECTION );
   glPopMatrix();
   glMatrixMode( GL_MODELVIEW );
@@ -1382,7 +1496,7 @@ int vtkOpenGLRenderWindow::SetZbufferData( int x1, int y1, int x2, int y2,
 
 void vtkOpenGLRenderWindow::RegisterTextureResource (GLuint id)
 {
-  this->TextureResourceIds->InsertNextId ((int) id);
+  this->TextureResourceIds->InsertNextId (static_cast<int>(id));
 }
 
 // ----------------------------------------------------------------------------
@@ -1406,8 +1520,7 @@ int vtkOpenGLRenderWindow::CreateHardwareOffScreenWindow(int width, int height)
   
   // 2. check for OpenGL extensions GL_EXT_framebuffer_object and
   // GL_ARB_texture_non_power_of_two (core-promoted feature in OpenGL 2.0)
-  vtkOpenGLExtensionManager *extensions=vtkOpenGLExtensionManager::New();
-  extensions->SetRenderWindow(this);
+  vtkOpenGLExtensionManager *extensions = this->GetExtensionManager();
   
   int supports_GL_EXT_framebuffer_object=
     extensions->ExtensionSupported("GL_EXT_framebuffer_object");
@@ -1425,19 +1538,27 @@ int vtkOpenGLRenderWindow::CreateHardwareOffScreenWindow(int width, int height)
   int supports_texture_rectangle=
     extensions->ExtensionSupported("GL_ARB_texture_rectangle");
   
+  // The following extension does not exist on ATI. There will be no HW
+  // Offscreen on ATI if a stencil buffer is required.
+  int supports_packed_depth_stencil=
+    extensions->ExtensionSupported("GL_EXT_packed_depth_stencil");
+    
   int result=0;
   
   if(!(supports_GL_EXT_framebuffer_object &&
        (supports_texture_non_power_of_two || supports_texture_rectangle) &&
-       !isMesa))
+       !isMesa && (!this->StencilCapable || supports_packed_depth_stencil)))
     {
     if(!supports_GL_EXT_framebuffer_object)
       {
-      vtkDebugMacro(<<" extension GL_EXT_framebuffer_object is not supported. Hardware accelerated offscreen rendering is not available");
+      vtkDebugMacro( << " extension GL_EXT_framebuffer_object is not supported. "
+        "Hardware accelerated offscreen rendering is not available" );
       }
     if(!supports_texture_non_power_of_two)
       {
-      vtkDebugMacro(<<" extension texture_non_power_of_two is not supported because neither OpenGL 2.0 nor GL_ARB_texture_non_power_of_two extension is supported. Hardware accelerated offscreen rendering is not available");
+      vtkDebugMacro( << " extension texture_non_power_of_two is not supported "
+        "because neither OpenGL 2.0 nor GL_ARB_texture_non_power_of_two extension "
+        "is supported. Hardware accelerated offscreen rendering is not available");
       }
     if(!supports_texture_rectangle)
       {
@@ -1445,7 +1566,13 @@ int vtkOpenGLRenderWindow::CreateHardwareOffScreenWindow(int width, int height)
       }
     if(isMesa)
       {
-      vtkDebugMacro(<<" Renderer is Mesa. Hardware accelerated offscreen rendering is not available");
+      vtkDebugMacro(<<" Renderer is Mesa. Hardware accelerated offscreen "
+        "rendering is not available");
+      }
+    if(this->StencilCapable && !supports_packed_depth_stencil)
+      {
+      vtkDebugMacro(<<" a stencil buffer is required but extension "
+        "GL_EXT_packed_depth_stencil is not supported");
       }
     this->DestroyWindow();
     }
@@ -1549,47 +1676,79 @@ int vtkOpenGLRenderWindow::CreateHardwareOffScreenWindow(int width, int height)
       }
     else
       {
-      result=1;
-
       // Set up the depth (and stencil), render buffer
-      vtkgl::BindRenderbufferEXT(
-        vtkgl::RENDERBUFFER_EXT, depthRenderBufferObject);
-      vtkgl::RenderbufferStorageEXT(
-        vtkgl::RENDERBUFFER_EXT, vtkgl::DEPTH_STENCIL_EXT, width,height);
-      vtkgl::FramebufferRenderbufferEXT(
-        vtkgl::FRAMEBUFFER_EXT,
-        vtkgl::DEPTH_ATTACHMENT_EXT, vtkgl::RENDERBUFFER_EXT, depthRenderBufferObject);
-      vtkgl::FramebufferRenderbufferEXT(
-        vtkgl::FRAMEBUFFER_EXT, 
-        vtkgl::STENCIL_ATTACHMENT_EXT, vtkgl::RENDERBUFFER_EXT, depthRenderBufferObject);
-
-      this->BackLeftBuffer=
-        static_cast<unsigned int>(vtkgl::COLOR_ATTACHMENT0_EXT);
-      this->FrontLeftBuffer=
-        static_cast<unsigned int>(vtkgl::COLOR_ATTACHMENT0_EXT);
-    
-      if(this->NumberOfFrameBuffers==2)
+      vtkgl::BindRenderbufferEXT(vtkgl::RENDERBUFFER_EXT,
+                                 depthRenderBufferObject);
+      if(this->StencilCapable)
         {
-        this->BackRightBuffer=
-          static_cast<unsigned int>(vtkgl::COLOR_ATTACHMENT1_EXT);
+        vtkgl::RenderbufferStorageEXT(vtkgl::RENDERBUFFER_EXT,
+                                      vtkgl::DEPTH_STENCIL_EXT, width,height);
         }
-    
-      // Save GL objects by static casting to standard C types. GL* types
-      // are not allowed in VTK header files.
-      this->FrameBufferObject=static_cast<unsigned int>(frameBufferObject);
-      this->DepthRenderBufferObject=
-        static_cast<unsigned int>(depthRenderBufferObject);
-      i=0;
-      while(i<this->NumberOfFrameBuffers)
+      else
         {
-        this->TextureObjects[i]=static_cast<unsigned int>(textureObjects[i]);
-        ++i;
+        vtkgl::RenderbufferStorageEXT(vtkgl::RENDERBUFFER_EXT,
+                                      vtkgl::DEPTH_COMPONENT24,width,height);
+        }
+      vtkgl::FramebufferRenderbufferEXT(vtkgl::FRAMEBUFFER_EXT,
+                                        vtkgl::DEPTH_ATTACHMENT_EXT,
+                                        vtkgl::RENDERBUFFER_EXT,
+                                        depthRenderBufferObject);
+      if(this->StencilCapable)
+        {
+        vtkgl::FramebufferRenderbufferEXT(vtkgl::FRAMEBUFFER_EXT, 
+                                          vtkgl::STENCIL_ATTACHMENT_EXT,
+                                          vtkgl::RENDERBUFFER_EXT,
+                                          depthRenderBufferObject);
+        }
+      
+      // Last check to see if the FBO is supported or not.
+      status=vtkgl::CheckFramebufferStatusEXT(vtkgl::FRAMEBUFFER_EXT);
+      if(status!=vtkgl::FRAMEBUFFER_COMPLETE_EXT)
+        {
+        vtkDebugMacro(<<"Hardware does not support GPU Offscreen rendering withthis depth/stencil configuration.");
+        glBindTexture(target,0);
+        vtkgl::BindFramebufferEXT(vtkgl::FRAMEBUFFER_EXT,0);
+        vtkgl::DeleteFramebuffersEXT(1,&frameBufferObject);
+        vtkgl::DeleteRenderbuffersEXT(1,&depthRenderBufferObject);
+        glDeleteTextures(this->NumberOfFrameBuffers,textureObjects);
+        this->DestroyWindow();
+        }
+      else
+        {
+        result=1;
+        this->BackLeftBuffer=
+          static_cast<unsigned int>(vtkgl::COLOR_ATTACHMENT0_EXT);
+        this->FrontLeftBuffer=
+          static_cast<unsigned int>(vtkgl::COLOR_ATTACHMENT0_EXT);
+        
+        this->BackBuffer=static_cast<unsigned int>(
+          vtkgl::COLOR_ATTACHMENT0_EXT);
+        this->FrontBuffer=
+          static_cast<unsigned int>(vtkgl::COLOR_ATTACHMENT0_EXT);
+        
+        if(this->NumberOfFrameBuffers==2)
+          {
+          this->BackRightBuffer=
+            static_cast<unsigned int>(vtkgl::COLOR_ATTACHMENT1_EXT);
+          this->FrontRightBuffer=
+            static_cast<unsigned int>(vtkgl::COLOR_ATTACHMENT1_EXT);
+          }
+        
+        // Save GL objects by static casting to standard C types. GL* types
+        // are not allowed in VTK header files.
+        this->FrameBufferObject=static_cast<unsigned int>(frameBufferObject);
+        this->DepthRenderBufferObject=
+          static_cast<unsigned int>(depthRenderBufferObject);
+        i=0;
+        while(i<this->NumberOfFrameBuffers)
+          {
+          this->TextureObjects[i]=static_cast<unsigned int>(textureObjects[i]);
+          ++i;
+          }
+        this->OffScreenUseFrameBuffer=1;
         }
-    
-      this->OffScreenUseFrameBuffer=1;
       }
     }
-  extensions->Delete();
   
   // A=>B = !A || B
   assert("post: valid_result" && (result==0 || result==1)
@@ -1613,6 +1772,9 @@ void vtkOpenGLRenderWindow::DestroyHardwareOffScreenWindow()
   this->BackLeftBuffer=static_cast<unsigned int>(GL_BACK_LEFT);
   this->BackRightBuffer=static_cast<unsigned int>(GL_BACK_RIGHT);
   this->FrontLeftBuffer=static_cast<unsigned int>(GL_FRONT_LEFT);
+  this->FrontRightBuffer=static_cast<unsigned int>(GL_FRONT_RIGHT);
+  this->BackBuffer=static_cast<unsigned int>(GL_BACK);
+  this->FrontBuffer=static_cast<unsigned int>(GL_FRONT);
   
   GLuint frameBufferObject=static_cast<GLuint>(this->FrameBufferObject);
   vtkgl::DeleteFramebuffersEXT(1,&frameBufferObject);
@@ -1635,3 +1797,88 @@ void vtkOpenGLRenderWindow::DestroyHardwareOffScreenWindow()
   
   assert("post: destroyed" && !this->OffScreenUseFrameBuffer);
 }
+
+// ----------------------------------------------------------------------------
+// Description:
+// Update graphic error status, regardless of ReportGraphicErrors flag.
+// It means this method can be used in any context and is not restricted to
+// debug mode.
+void vtkOpenGLRenderWindow::CheckGraphicError()
+{
+  this->LastGraphicError=static_cast<unsigned int>(glGetError());
+}
+  
+// ----------------------------------------------------------------------------
+// Description:
+// Return the last graphic error status. Initial value is false.
+int vtkOpenGLRenderWindow::HasGraphicError()
+{
+  return static_cast<GLenum>(this->LastGraphicError)!=GL_NO_ERROR;
+}
+
+// ----------------------------------------------------------------------------
+// Description:
+// Return a string matching the last graphic error status.
+const char *vtkOpenGLRenderWindow::GetLastGraphicErrorString()
+{
+  const char *result;
+  switch(static_cast<GLenum>(this->LastGraphicError))
+    {
+    case GL_NO_ERROR:
+      result="No error";
+      break;
+    case GL_INVALID_ENUM:
+      result="Invalid enum";
+      break;
+    case GL_INVALID_VALUE:
+      result="Invalid value";
+      break;
+    case GL_INVALID_OPERATION:
+      result="Invalid operation";
+      break;
+    case GL_STACK_OVERFLOW:
+      result="Stack overflow";
+      break;
+    case GL_STACK_UNDERFLOW:
+      result="Stack underflow";
+      break;
+    case GL_OUT_OF_MEMORY:
+      result="Out of memory";
+      break;
+    case vtkgl::TABLE_TOO_LARGE:
+      // GL_ARB_imaging
+      result="Table too large";
+      break;
+    case vtkgl::INVALID_FRAMEBUFFER_OPERATION_EXT:
+      // GL_EXT_framebuffer_object
+      result="Invalid framebuffer operation";
+      break;
+    case vtkgl::TEXTURE_TOO_LARGE_EXT:
+      // GL_EXT_texture
+      result="Texture too large";
+      break;
+    default:
+      result="Unknown error";
+      break;
+    }
+  return result;
+}
+
+
+// ----------------------------------------------------------------------------
+// Description:
+// Returns the extension manager. A new one will be created if one hasn't
+// already been set up.
+vtkOpenGLExtensionManager* vtkOpenGLRenderWindow::GetExtensionManager()
+{
+  if (!this->ExtensionManager)
+    {
+    vtkOpenGLExtensionManager* mgr = vtkOpenGLExtensionManager::New();
+    // This does not form a reference loop since vtkOpenGLExtensionManager does
+    // not keep a reference to the render window.
+    mgr->SetRenderWindow(this);
+    this->SetExtensionManager(mgr);
+    mgr->Delete();
+    }
+  return this->ExtensionManager;
+}
