@@ -43,7 +43,7 @@ public:
 };
 
 #ifndef VTK_IMPLEMENT_MESA_CXX
-vtkCxxRevisionMacro(vtkOpenGLRenderer, "$Revision: 1.73 $");
+vtkCxxRevisionMacro(vtkOpenGLRenderer, "$Revision: 1.90 $");
 vtkStandardNewMacro(vtkOpenGLRenderer);
 #endif
 
@@ -139,7 +139,7 @@ void vtkOpenGLRenderer::ClearLights (void)
   // now delete all the old lights 
   for (curLight = GL_LIGHT0; curLight < GL_LIGHT0 + VTK_MAX_LIGHTS; curLight++)
     {
-    glDisable((GLenum)curLight);
+    glDisable(static_cast<GLenum>(curLight));
     }
 
   this->NumberOfLightsBound = 0;
@@ -192,8 +192,8 @@ int vtkOpenGLRenderer::UpdateLights ()
     // also make sure we still have room.             
     if ((status > 0.0)&& (curLight < (GL_LIGHT0+VTK_MAX_LIGHTS)))
       {
-      light->Render((vtkRenderer *)this,curLight);
-      glEnable((GLenum)curLight);
+      light->Render(this,curLight);
+      glEnable(static_cast<GLenum>(curLight));
       // increment the current light by one 
       curLight++;
       count++;
@@ -325,15 +325,18 @@ void vtkOpenGLRenderer::DeviceRenderTranslucentPolygonalGeometry()
       
       int supports_depth_texture;
       int supports_shadow;
+      int supports_blend_func_separate;
       if(supports_GL_1_4)
         {
         supports_depth_texture=1;
+        supports_blend_func_separate=1;
         supports_shadow=1;
         }
       else
         {
         supports_depth_texture=extensions->ExtensionSupported("GL_ARB_depth_texture");
         supports_shadow=extensions->ExtensionSupported("GL_ARB_shadow");
+        supports_blend_func_separate=extensions->ExtensionSupported("GL_EXT_blend_func_separate");
         }
       
       int supports_GL_ARB_texture_rectangle=extensions->ExtensionSupported("GL_ARB_texture_rectangle");
@@ -361,6 +364,7 @@ void vtkOpenGLRenderer::DeviceRenderTranslucentPolygonalGeometry()
       this->DepthPeelingIsSupported =
         supports_depth_texture &&
         supports_shadow &&
+        supports_blend_func_separate &&
         supports_shadow_funcs &&
         supports_vertex_shader &&
         supports_fragment_shader &&
@@ -384,6 +388,15 @@ void vtkOpenGLRenderer::DeviceRenderTranslucentPolygonalGeometry()
           }
         // GL_ARB_depth_texture, GL_ARB_shadow and GL_EXT_shadow_funcs
         // don't introduce new functions.
+        if(supports_GL_1_4)
+          {
+          extensions->LoadExtension("GL_VERSION_1_4");
+          }
+        else
+          {
+          extensions->LoadCorePromotedExtension("GL_EXT_blend_func_separate");
+          }
+        
         if(supports_GL_2_0)
           {
           extensions->LoadExtension("GL_VERSION_2_0");
@@ -515,8 +528,8 @@ void vtkOpenGLRenderer::DeviceRenderTranslucentPolygonalGeometry()
                     &mesa_major, &mesa_minor, &mesa_patch) >= 2)
             {
             if(mesa_major  < 6 ||
-               mesa_major == 6 && mesa_major  < 5 ||
-               mesa_major == 6 && mesa_minor == 5 && mesa_patch < 3)
+               (mesa_major == 6 && mesa_major  < 5) ||
+               (mesa_major == 6 && mesa_minor == 5 && mesa_patch < 3))
               {
               this->DepthPeelingIsSupported = 0;
               }
@@ -524,30 +537,24 @@ void vtkOpenGLRenderer::DeviceRenderTranslucentPolygonalGeometry()
           }
         else if(isATIRadeon9600XT)
           {
-          // The Mac OS X 10.4.9 and 10.4.10 versions of the ATI driver, known not to work
-          if(strstr(gl_version, "1.5 ATI-1.4.18"))
-            {
-            this->DepthPeelingIsSupported = 0;
-            }
+          // The Mac OS X 10.4.9->10.4.10, 10.5.0->10.5.1
+          // versions of the ATI driver, known not to work
+          // 1.5 ATI-1.4.18, 2.0 ATI-1.5.16, 2.0 ATI-1.5.18
+          this->DepthPeelingIsSupported = 0;
           }
         else if(isATIFireGLV3300)
           {
-          if(strstr(gl_version, "2.0.6237"))
-            {
+            // so far, 2.0.6237 and 2.0.6672 don't work
             this->DepthPeelingIsSupported = 0;
-            }
           }
         else if(isATIRadeonX1600 || isATIRadeonX1900)
           {
-          // The Mac OS X 10.4.8 version of the ATI driver, known not to work
-          // The Mac OS X 10.4.9 version of the ATI driver, known not to work
-          // The Mac OS X 10.4.10 version of the ATI driver, known not to work
-          if(strstr(gl_version, "2.0 ATI-1.4.40") ||
-             strstr(gl_version, "2.0 ATI-1.4.52") ||
-             strstr(gl_version, "2.0 ATI-1.4.56"))
-            {
-            this->DepthPeelingIsSupported = 0;
-            }
+          // The Mac OS X 10.4.8->10.4.11, 10.5.0->10.5.1
+          // versions of the ATI driver, known not to work
+          // 2.0 ATI-1.4.40, 2.0 ATI-1.4.52, 2.0 ATI-1.4.56,
+          // 2.0 ATI-1.4.58
+          // 2.0 ATI-1.5.16, 2.0 ATI-1.5.18,
+          this->DepthPeelingIsSupported = 0;
           }
         else if(isATIRadeonX300X550)
           {
@@ -681,6 +688,9 @@ void vtkOpenGLRenderer::DeviceRenderTranslucentPolygonalGeometry()
     unsigned int threshold=static_cast<unsigned int>(this->ViewportWidth*this->ViewportHeight*OcclusionRatio);
     this->LayerList=new vtkOpenGLRendererLayerList;
     
+    // save the default blend function.
+    glPushAttrib(GL_COLOR_BUFFER_BIT);
+    
     int multiSampleStatus=glIsEnabled(vtkgl::MULTISAMPLE);
     
     if(multiSampleStatus)
@@ -707,7 +717,7 @@ void vtkOpenGLRenderer::DeviceRenderTranslucentPolygonalGeometry()
           ++l;
           if(!stop && !infiniteLoop)
             {
-            stop=l>this->MaximumNumberOfPeels;
+            stop=l>=this->MaximumNumberOfPeels;
             }
           }
         }
@@ -737,15 +747,11 @@ void vtkOpenGLRenderer::DeviceRenderTranslucentPolygonalGeometry()
     glMatrixMode(GL_MODELVIEW);
     glPushMatrix();
     glLoadIdentity();
-    glClearColor( ((GLclampf)(0)),
-                  ((GLclampf)(0)),
-                  ((GLclampf)(0)),
-                  ((GLclampf)(0.0)) );
+    glClearColor( static_cast<GLclampf>(0),static_cast<GLclampf>(0),
+                  static_cast<GLclampf>(0),static_cast<GLclampf>(0));
     
     glClear(GL_COLOR_BUFFER_BIT);
     glDisable(GL_DEPTH_TEST);
-    glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
-    glEnable(GL_BLEND);
     
     vtkgl::ActiveTexture(vtkgl::TEXTURE0 );
     glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
@@ -754,6 +760,7 @@ void vtkOpenGLRenderer::DeviceRenderTranslucentPolygonalGeometry()
     // actor in wireframe may have change that
     glPolygonMode(GL_FRONT, GL_FILL);
     
+    glDisable(GL_BLEND);
     // First the opaque layer
     glBindTexture(vtkgl::TEXTURE_RECTANGLE_ARB,opaqueLayerRgba);
     glBegin(GL_QUADS);
@@ -767,6 +774,9 @@ void vtkOpenGLRenderer::DeviceRenderTranslucentPolygonalGeometry()
     glVertex2f(0, this->ViewportHeight);
     glEnd();
     
+    vtkgl::BlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA,
+                             GL_ONE,GL_ONE_MINUS_SRC_ALPHA);
+    glEnable(GL_BLEND);
     // the transparent layers
     vtkstd::list<GLuint>::reverse_iterator it=this->LayerList->List.rbegin();
     vtkstd::list<GLuint>::reverse_iterator itEnd=this->LayerList->List.rend();
@@ -786,10 +796,11 @@ void vtkOpenGLRenderer::DeviceRenderTranslucentPolygonalGeometry()
       glEnd();
       ++it;
       }
-    
+    // Restore the default blend function for the next stage (overlay)
+    glPopAttrib();
+  
     glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
     glDisable(vtkgl::TEXTURE_RECTANGLE_ARB);
-    glDisable(GL_BLEND);
     glEnable(GL_DEPTH_TEST);
     glMatrixMode(GL_PROJECTION);
     glPopMatrix();
@@ -821,10 +832,6 @@ void vtkOpenGLRenderer::DeviceRenderTranslucentPolygonalGeometry()
     glDeleteTextures(1,&opaqueLayerRgba);
     glDeleteTextures(1,&opaqueLayerZ);
     }
-  
-  // Restore default alpha blending for the next stage (overlay)
-  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-  glEnable(GL_BLEND);
 }
 
 // ----------------------------------------------------------------------------
@@ -1089,18 +1096,58 @@ void vtkOpenGLRenderer::Clear(void)
 
   if (! this->Transparent())
     {
-    glClearColor( ((GLclampf)(this->Background[0])),
-                  ((GLclampf)(this->Background[1])),
-                  ((GLclampf)(this->Background[2])),
-                  ((GLclampf)(0.0)) );
+    glClearColor( static_cast<GLclampf>(this->Background[0]),
+                  static_cast<GLclampf>(this->Background[1]),
+                  static_cast<GLclampf>(this->Background[2]),
+                  static_cast<GLclampf>(0.0));
     clear_mask |= GL_COLOR_BUFFER_BIT;
     }
 
-  glClearDepth( (GLclampd)( 1.0 ) );
+  glClearDepth(static_cast<GLclampf>(1.0));
   clear_mask |= GL_DEPTH_BUFFER_BIT;
 
   vtkDebugMacro(<< "glClear\n");
   glClear(clear_mask);
+
+  // If gradient background is turned on, draw it now.
+  if (!this->Transparent() && this->GradientBackground)
+    {
+    glPushAttrib(GL_ENABLE_BIT);
+    glDisable(GL_DEPTH_TEST);
+    glDisable(GL_LIGHTING);
+    glDisable(GL_TEXTURE_1D);
+    glDisable(GL_TEXTURE_2D);
+    glShadeModel(GL_SMOOTH); // color interpolation
+
+    glMatrixMode(GL_MODELVIEW);
+    glPushMatrix();
+    {
+      glLoadIdentity();
+      glMatrixMode(GL_PROJECTION);
+      glPushMatrix();
+      {
+        glLoadIdentity();
+        glOrtho(-1.0,1.0,-1.0,1.0,-1.0,1.0);
+
+        glBegin(GL_QUADS);
+
+        //top vertices
+        glColor3dv(this->Background);
+        glVertex2f(-1.0, -1.0);
+        glVertex2f(1.0, -1.0);
+
+        //bottom vertices
+        glColor3dv(this->Background2);
+        glVertex2f(1.0, 1.0);
+        glVertex2f(-1.0, 1.0);
+        glEnd();
+      }
+      glPopMatrix();
+      glMatrixMode(GL_MODELVIEW);
+    }
+    glPopMatrix();
+    glPopAttrib();
+    }
 }
 
 void vtkOpenGLRenderer::StartPick(unsigned int pickFromSize)
@@ -1168,7 +1215,7 @@ void vtkOpenGLRenderer::DonePick()
   GLuint hits = glRenderMode(GL_RENDER); 
   this->PickInfo->NumPicked = hits;
 
-  unsigned int depth = (unsigned int)-1;
+  unsigned int depth = static_cast<unsigned int>(-1);
   GLuint* ptr = this->PickInfo->PickBuffer;
   this->PickInfo->PickedId = 0;
   for(unsigned int k =0; k < hits; k++)
@@ -1204,7 +1251,7 @@ void vtkOpenGLRenderer::DonePick()
     // integer value corresponds to window coordinate depth 1.0, 
     // and zero corresponds to window coordinate depth 0.0.
     
-    this->PickedZ = ((double)depth / (double)VTK_UNSIGNED_INT_MAX);
+    this->PickedZ = depth/static_cast<double>(VTK_UNSIGNED_INT_MAX);
 
     // Clamp to range [0,1]
     this->PickedZ = (this->PickedZ < 0.0) ? 0.0 : this->PickedZ;
@@ -1226,7 +1273,7 @@ double vtkOpenGLRenderer::GetPickedZ()
 
 unsigned int vtkOpenGLRenderer::GetPickedId()
 {
-  return (unsigned int)this->PickInfo->PickedId;
+  return static_cast<unsigned int>(this->PickInfo->PickedId);
 }
 
 vtkOpenGLRenderer::~vtkOpenGLRenderer()
@@ -1241,7 +1288,7 @@ vtkOpenGLRenderer::~vtkOpenGLRenderer()
 
 unsigned int vtkOpenGLRenderer::GetNumPickedIds()
 {
-  return (unsigned int)this->PickInfo->NumPicked;
+  return static_cast<unsigned int>(this->PickInfo->NumPicked);
 }
 
 int vtkOpenGLRenderer::GetPickedIds(unsigned int atMost,
@@ -1262,7 +1309,7 @@ int vtkOpenGLRenderer::GetPickedIds(unsigned int atMost,
     iptr++; // move to first depth value
     iptr++; // move to next depth value
     iptr++; // move to first name picked
-    *optr = (unsigned int)*iptr;
+    *optr = static_cast<unsigned int>(*iptr);
     optr++;
     // skip additonal names
     iptr += num_names;
