@@ -35,6 +35,7 @@
 #include "vtkXMLShader.h"
 
 #include <stdlib.h>
+#include <vtksys/ios/sstream>
 
 #include <vtkstd/map>
 #include <vtksys/SystemTools.hxx>
@@ -47,7 +48,7 @@ public:
   MapOfTextures Textures;
 };
 
-vtkCxxRevisionMacro(vtkProperty, "$Revision: 1.68 $");
+vtkCxxRevisionMacro(vtkProperty, "$Revision: 1.73 $");
 vtkCxxSetObjectMacro(vtkProperty, ShaderProgram, vtkShaderProgram);
 //----------------------------------------------------------------------------
 // Needed when we don't use the vtkStandardNewMacro.
@@ -270,7 +271,7 @@ vtkProperty *vtkProperty::New()
 { 
   // First try to create the object from the vtkObjectFactory
   vtkObject* ret = vtkGraphicsFactory::CreateInstance("vtkProperty");
-  return (vtkProperty*)ret;
+  return static_cast<vtkProperty *>(ret);
 }
 
 //----------------------------------------------------------------------------
@@ -355,6 +356,38 @@ vtkTexture* vtkProperty::GetTexture(const char* name)
   return NULL;
 }
 
+//----------------------------------------------------------------------------
+void vtkProperty::SetTexture(int unit, vtkTexture* tex)
+{
+  vtksys_ios::ostringstream name;
+  name << unit;
+
+  vtkPropertyInternals::MapOfTextures::iterator iter = 
+    this->Internals->Textures.find(name.str().c_str());
+  if (iter != this->Internals->Textures.end())
+    {
+    vtkWarningMacro("Texture with name " << name 
+      << " exists. It will be replaced.");
+    }
+  this->Internals->Textures[name.str().c_str()] = tex;
+}
+
+//----------------------------------------------------------------------------
+vtkTexture* vtkProperty::GetTexture(int unit)
+{
+  vtksys_ios::ostringstream name;
+  name << unit;
+
+  vtkPropertyInternals::MapOfTextures::iterator iter = 
+    this->Internals->Textures.find(name.str().c_str());
+  if (iter != this->Internals->Textures.end())
+    {
+    return iter->second.GetPointer();
+    }
+  vtkErrorMacro("No texture with name " << name << " exists.");
+  return NULL;
+}
+
 //----------------------------------------------------------------------------
 int vtkProperty::GetNumberOfTextures()
 {
@@ -414,12 +447,12 @@ int vtkProperty::GetTextureIndex(const char* name)
 //----------------------------------------------------------------------------
 void vtkProperty::LoadMaterial(const char* name)
 {
+  this->SetMaterialName(0);
   if( !name || strlen(name) == 0)
     {
     this->LoadMaterial(static_cast<vtkXMLMaterial*>(0));
     return;
     }
-  this->SetMaterialName(name);
 
   // vtkXMLMaterial::CreateInstance using library/absolute path/repository
   // in that order.
@@ -439,6 +472,7 @@ void vtkProperty::LoadMaterial(const char* name)
 //----------------------------------------------------------------------------
 void vtkProperty::LoadMaterialFromString(const char* materialxml)
 {
+  this->SetMaterialName(0);
   if (!materialxml)
     {
     this->LoadMaterial(static_cast<vtkXMLMaterial*>(0));
@@ -456,9 +490,11 @@ void vtkProperty::LoadMaterialFromString(const char* materialxml)
 //----------------------------------------------------------------------------
 void vtkProperty::LoadMaterial(vtkXMLMaterial* material)
 {
+  this->SetMaterialName(0);
   vtkSetObjectBodyMacro(Material, vtkXMLMaterial, material);
   if (this->Material)
     {
+    this->SetMaterialName(this->Material->GetRootElement()->GetAttribute("name"));
     this->LoadProperty();
     this->LoadTextures();
     int lang = this->Material->GetShaderLanguage();
@@ -945,6 +981,8 @@ void vtkProperty::PrintSelf(ostream& os, vtkIndent indent)
     {
     os << "(none)" << endl;
     }
+ os << indent << "MaterialName: " << 
+   (this->MaterialName? this->MaterialName:"(none)") << endl;
 
   os << indent << "ShaderProgram: ";
   if (this->ShaderProgram)
