@@ -14,33 +14,34 @@
 =========================================================================*/
 #include "vtkRenderer.h"
 
+#include "vtkAreaPicker.h"
 #include "vtkAssemblyNode.h"
 #include "vtkAssemblyPath.h"
 #include "vtkCamera.h"
 #include "vtkCommand.h"
-#include "vtkCuller.h"
 #include "vtkCullerCollection.h"
+#include "vtkCuller.h"
 #include "vtkFrustumCoverageCuller.h"
 #include "vtkGraphicsFactory.h"
-#include "vtkLight.h"
+#include "vtkIdentColoredPainter.h"
 #include "vtkLightCollection.h"
+#include "vtkLight.h"
 #include "vtkMath.h"
 #include "vtkMatrix4x4.h"
 #include "vtkOutputWindow.h"
+#include "vtkPainterPolyDataMapper.h"
 #include "vtkPicker.h"
-#include "vtkAreaPicker.h"
 #include "vtkProp3DCollection.h"
+#include "vtkPropCollection.h"
 #include "vtkRenderWindow.h"
 #include "vtkTimerLog.h"
 #include "vtkVolume.h"
-#include "vtkPropCollection.h"
-#include "vtkIdentColoredPainter.h"
-#include "vtkPainterPolyDataMapper.h"
-#include "vtkPolyDataPainter.h"
+#include "vtkRendererDelegate.h"
 
-vtkCxxRevisionMacro(vtkRenderer, "$Revision: 1.235 $");
+vtkCxxRevisionMacro(vtkRenderer, "$Revision: 1.241 $");
 
 vtkCxxSetObjectMacro(vtkRenderer, IdentPainter, vtkIdentColoredPainter);
+vtkCxxSetObjectMacro(vtkRenderer, Delegate, vtkRendererDelegate);
 
 //----------------------------------------------------------------------------
 // Needed when we don't use the vtkStandardNewMacro.
@@ -118,6 +119,8 @@ vtkRenderer::vtkRenderer()
   this->OcclusionRatio=0.0;
   this->MaximumNumberOfPeels=4;
   this->LastRenderingUsedDepthPeeling=0;
+
+  this->Delegate=0;
 }
 
 vtkRenderer::~vtkRenderer()
@@ -169,12 +172,18 @@ vtkRenderer *vtkRenderer::New()
 { 
   // First try to create the object from the vtkObjectFactory
   vtkObject* ret = vtkGraphicsFactory::CreateInstance("vtkRenderer");
-  return (vtkRenderer *)ret;
+  return static_cast<vtkRenderer *>(ret);
 }
 
 // Concrete render method.
 void vtkRenderer::Render(void)
 {
+  if(this->Delegate!=0 && this->Delegate->GetUsed())
+    {
+      this->Delegate->Render(this);
+      return;
+    }
+
   double   t1, t2;
   int      i;
   vtkProp  *aProp;
@@ -239,10 +248,14 @@ void vtkRenderer::Render(void)
       
       // backing store should be OK, lets use it
       // calc the pixel range for the renderer
-      rx1 = (int)(this->Viewport[0]*(this->RenderWindow->GetSize()[0] - 1));
-      ry1 = (int)(this->Viewport[1]*(this->RenderWindow->GetSize()[1] - 1));
-      rx2 = (int)(this->Viewport[2]*(this->RenderWindow->GetSize()[0] - 1));
-      ry2 = (int)(this->Viewport[3]*(this->RenderWindow->GetSize()[1] - 1));
+      rx1 = static_cast<int>(this->Viewport[0]*
+                             (this->RenderWindow->GetSize()[0] - 1));
+      ry1 = static_cast<int>(this->Viewport[1]*
+                             (this->RenderWindow->GetSize()[1] - 1));
+      rx2 = static_cast<int>(this->Viewport[2]*
+                             (this->RenderWindow->GetSize()[0] - 1));
+      ry2 = static_cast<int>(this->Viewport[3]*
+                             (this->RenderWindow->GetSize()[1] - 1));
       this->RenderWindow->SetPixelData(rx1,ry1,rx2,ry2,this->BackingImage,0);
       this->InvokeEvent(vtkCommand::EndEvent,NULL);
       return;
@@ -323,10 +336,10 @@ void vtkRenderer::Render(void)
     
     // backing store should be OK, lets use it
     // calc the pixel range for the renderer
-    rx1 = (int)(this->Viewport[0]*(size[0] - 1));
-    ry1 = (int)(this->Viewport[1]*(size[1] - 1));
-    rx2 = (int)(this->Viewport[2]*(size[0] - 1));
-    ry2 = (int)(this->Viewport[3]*(size[1] - 1));
+    rx1 = static_cast<int>(this->Viewport[0]*(size[0] - 1));
+    ry1 = static_cast<int>(this->Viewport[1]*(size[1] - 1));
+    rx2 = static_cast<int>(this->Viewport[2]*(size[0] - 1));
+    ry2 = static_cast<int>(this->Viewport[3]*(size[1] - 1));
     this->BackingImage = this->RenderWindow->GetPixelData(rx1,ry1,rx2,ry2,0);
     this->BackingStoreSize[0] = size[0];
     this->BackingStoreSize[1] = size[1];
@@ -341,7 +354,7 @@ void vtkRenderer::Render(void)
     {
     // Measure the actual RenderTime
     t2 = vtkTimerLog::GetUniversalTime();
-    this->LastRenderTimeInSeconds = (double) (t2 - t1);
+    this->LastRenderTimeInSeconds = static_cast<double>(t2 - t1);
 
     if (this->LastRenderTimeInSeconds == 0.0)
       {
@@ -394,7 +407,7 @@ int vtkRenderer::UpdateCamera ()
     }
 
   // update the viewing transformation
-  this->ActiveCamera->Render((vtkRenderer *)this); 
+  this->ActiveCamera->Render(this); 
   
   return 1;
 }
@@ -500,9 +513,7 @@ void vtkRenderer::AllocateTime()
        (aCuller=this->Cullers->GetNextCuller(sit));)
     {
     totalTime =
-      aCuller->Cull((vtkRenderer *)this, 
-                    this->PropArray, this->PropArrayCount,
-                    initialized );
+      aCuller->Cull(this,this->PropArray, this->PropArrayCount,initialized );
     }
 
   // loop through all props and set the AllocatedRenderTime
@@ -1321,6 +1332,16 @@ void vtkRenderer::PrintSelf(ostream& os, vtkIndent indent)
   
   // I don't want to print this since it is used just internally
   // os << indent << this->NumberOfPropsRendered;
+  
+  os << indent << "Delegate:";
+  if(this->Delegate!=0)
+    {
+      os << "exists" << endl;
+    }
+  else
+    {
+      os << "null" << endl;
+    }
 }
 
 int vtkRenderer::VisibleActorCount()
@@ -1711,6 +1732,8 @@ int vtkRenderer::UpdateGeometryForSelection()
   double origBG[3];
   this->GetBackground(origBG);
   this->SetBackground(0.0,0.0,0.0);
+  bool origGrad = this->GetGradientBackground();
+  this->GradientBackgroundOff();
   this->Clear();
 
   //todo: save off and swap in other renderer/renderwindow settings that
@@ -1718,7 +1741,7 @@ int vtkRenderer::UpdateGeometryForSelection()
 
   //create holders for prop's original rendering settings
   int orig_visibility;
-  vtkPolyDataPainter *orig_painter = NULL;
+  vtkPainter*orig_painter = NULL;
 
   //create a painter that will color each cell with an index
   if (this->IdentPainter==NULL)
@@ -1791,16 +1814,17 @@ int vtkRenderer::UpdateGeometryForSelection()
 
   //restore original background
   this->SetBackground(origBG);
+  this->SetGradientBackground(origGrad);
 
   return this->NumberOfPropsRendered;
 }
 
 //----------------------------------------------------------------------------
-vtkPolyDataPainter* vtkRenderer::SwapInSelectablePainter(
+vtkPainter* vtkRenderer::SwapInSelectablePainter(
   vtkProp *prop,
   int &orig_visibility)
 {
-  vtkPolyDataPainter* orig_painter = NULL;
+  vtkPainter* orig_painter = NULL;
   vtkPainterPolyDataMapper *orig_mapper = NULL;
   
   //try to find a polydatapainter that we can swap out
@@ -1840,7 +1864,7 @@ vtkPolyDataPainter* vtkRenderer::SwapInSelectablePainter(
 //----------------------------------------------------------------------------
 void vtkRenderer::SwapOutSelectablePainter(
   vtkProp *prop,
-  vtkPolyDataPainter* orig_painter,
+  vtkPainter* orig_painter,
   int orig_visibility) 
 {
   vtkPainterPolyDataMapper *orig_mapper = NULL;
