@@ -43,9 +43,10 @@ class vtkCamera;
 class vtkLightCollection;
 class vtkCullerCollection;
 class vtkLight;
-class vtkPolyDataPainter;
+class vtkPainter;
 class vtkIdentColoredPainter;
 class vtkVisibleCellSelector;
+class vtkRendererDelegate;
 
 class VTK_RENDERING_EXPORT vtkRenderer : public vtkViewport
 {
@@ -392,7 +393,8 @@ public:
   
   // Description:
   // Turn on/off rendering of translucent material with depth peeling
-  // technique.
+  // technique. The render window must have alpha bits (ie call
+  // SetAlphaBitPlanes(1)) to support depth peeling.
   // If UseDepthPeeling is on and the GPU supports it, depth peeling is used
   // for rendering translucent materials.
   // If UseDepthPeeling is off, alpha blending is used.
@@ -425,6 +427,14 @@ public:
   // Initial value is false.
   vtkGetMacro(LastRenderingUsedDepthPeeling,int);
   
+  // Description:
+  // Set/Get a custom Render call. Allows to hook a Render call from an
+  // external project.It will be used in place of vtkRenderer::Render() if it
+  // is not NULL and its Used ivar is set to true.
+  // Initial value is NULL.
+  void SetDelegate(vtkRendererDelegate *d);
+  vtkGetObjectMacro(Delegate,vtkRendererDelegate);
+
 protected:
   vtkRenderer();
   ~vtkRenderer();
@@ -574,6 +584,7 @@ protected:
   // VISIBLE CELL SELECTION ----------------------------------------
   //BTX  
   friend class vtkVisibleCellSelector;
+  friend class vtkRendererDelegate;
 
   //Description:
   // Call to put the Renderer into a mode in which it will color visible 
@@ -599,13 +610,13 @@ protected:
   // Description:
   // Called by UpdateGeometryForSelection to temporarily swap in a mapper to 
   // render a prop in selection mode.
-  vtkPolyDataPainter* SwapInSelectablePainter(vtkProp *, 
+  vtkPainter* SwapInSelectablePainter(vtkProp *, 
                                               int &);
 
   // Description:
   // Called by UpdateGeometryForSelection to restore a prop's original mapper.
   void SwapOutSelectablePainter(vtkProp *,
-                                vtkPolyDataPainter*, 
+                                vtkPainter*, 
                                 int );
 
   // Description:
@@ -621,6 +632,8 @@ protected:
 
   //---------------------------------------------------------------
 
+  vtkRendererDelegate *Delegate;
+
 private:
   vtkRenderer(const vtkRenderer&);  // Not implemented.
   void operator=(const vtkRenderer&);  // Not implemented.
