@@ -32,6 +32,14 @@
 
 #include <stdlib.h>
 
+// Silent warning like
+// "dereferencing type-punned pointer will break strict-aliasing rules"
+// it happens because this kind of expression: (long *)&ptr
+// pragma GCC diagnostic is available since gcc>=4.2
+#if defined(__GNUG__) && (__GNUC__>4) || (__GNUC__==4 && __GNUC_MINOR__>=2)
+#pragma GCC diagnostic ignored "-Wstrict-aliasing"
+#endif
+
 #define VTK_ALL_EVENTS_MASK \
     KeyPressMask|KeyReleaseMask|ButtonPressMask|ButtonReleaseMask|      \
     EnterWindowMask|LeaveWindowMask|PointerMotionMask|ExposureMask|     \
@@ -516,7 +524,7 @@ extern "C"
     Tk_SetClass(tkwin, (char *) "vtkTkRenderWidget");
     
     // Create vtkTkRenderWidget data structure 
-    self = (struct vtkTkRenderWidget *)ckalloc(sizeof(struct vtkTkRenderWidget));
+    self=(struct vtkTkRenderWidget *)ckalloc(sizeof(struct vtkTkRenderWidget));
     self->TkWin = tkwin;
     self->Interp = interp;
     self->Width = 0;
@@ -894,7 +902,8 @@ static int vtkTkRenderWidget_MakeRenderWindow(struct vtkTkRenderWidget *self)
     vtkTclGetObjectFromPointer(self->Interp, self->RenderWindow,
                                "vtkRenderWindow");
 #endif
-    self->RW = ckalloc(strlen(self->Interp->result) + 1);
+    self->RW = ckalloc(
+                  static_cast<unsigned int>(strlen(self->Interp->result) + 1));
     strcpy(self->RW, self->Interp->result);
     self->Interp->result[0] = '\0';
     }
@@ -1070,7 +1079,8 @@ vtkTkRenderWidget_MakeRenderWindow(struct vtkTkRenderWidget *self)
     vtkTclGetObjectFromPointer(self->Interp, self->RenderWindow,
           "vtkRenderWindow");
 #endif
-    self->RW = ckalloc(strlen(self->Interp->result) + 1);
+    self->RW = ckalloc(
+                  static_cast<unsigned int>(strlen(self->Interp->result) + 1));
     strcpy(self->RW, self->Interp->result);
     self->Interp->result[0] = '\0';
     }
@@ -1242,7 +1252,8 @@ vtkTkRenderWidget_MakeRenderWindow(struct vtkTkRenderWidget *self)
     vtkTclGetObjectFromPointer(self->Interp, self->RenderWindow,
           "vtkRenderWindow");
 #endif
-    self->RW = ckalloc(strlen(self->Interp->result) + 1);
+    self->RW = ckalloc(
+                  static_cast<unsigned int>(strlen(self->Interp->result) + 1));
     strcpy(self->RW, self->Interp->result);
     self->Interp->result[0] = '\0';
     }
