@@ -30,14 +30,9 @@ PURPOSE.  See the above copyright notice for more information.
 #include <math.h>
 #include <vtksys/ios/sstream>
 
-#if defined(_MSC_VER) || defined (__BORLANDC__)
-# include "vtkWindows.h"
-# include <GL/glaux.h>
-#else
-# include "vtkOpenGL.h"
-#endif
+#include "vtkOpenGL.h"
 
-vtkCxxRevisionMacro(vtkWin32OpenGLRenderWindow, "$Revision: 1.151 $");
+vtkCxxRevisionMacro(vtkWin32OpenGLRenderWindow, "$Revision: 1.157 $");
 vtkStandardNewMacro(vtkWin32OpenGLRenderWindow);
 
 #define VTK_MAX_LIGHTS 8
@@ -66,7 +61,15 @@ vtkWin32OpenGLRenderWindow::vtkWin32OpenGLRenderWindow()
 vtkWin32OpenGLRenderWindow::~vtkWin32OpenGLRenderWindow()
 {
   this->Finalize();
-  this->CleanUpRenderers();
+  
+  vtkRenderer *ren;
+  vtkCollectionSimpleIterator rsit;
+  for (this->Renderers->InitTraversal(rsit); 
+       (ren = this->Renderers->GetNextRenderer(rsit));)
+    {
+    ren->SetRenderWindow(NULL);
+    }
+
   delete[] this->Capabilities;
 }
 
@@ -134,6 +137,7 @@ void vtkWin32OpenGLRenderWindow::CleanUpRenderers()
        (ren = this->Renderers->GetNextRenderer(rsit));)
     {
     ren->SetRenderWindow(NULL);
+    ren->SetRenderWindow(this);
     }
 }
 
@@ -1233,14 +1237,7 @@ void vtkWin32OpenGLRenderWindow::SetOffScreenRendering(int offscreen)
     this->CleanUpOffScreenRendering();
     if (!this->WindowId)
       {
-      vtkRenderer* ren;
       this->WindowInitialize();
-      vtkCollectionSimpleIterator rsit;
-      for (this->Renderers->InitTraversal(rsit); 
-           (ren = this->Renderers->GetNextRenderer(rsit));)
-        {
-        ren->SetRenderWindow(this);
-        }
       this->OpenGLInit();
       if (this->Interactor)
         {
@@ -1342,16 +1339,6 @@ void vtkWin32OpenGLRenderWindow::CreateOffScreenDC(HBITMAP hbmp, HDC aHdc)
     vtkErrorMacro("wglCreateContext failed in CreateOffScreenDC(), error: " << GetLastError());
     }
   this->MakeCurrent();
-  
-  // Renderers will need to redraw anything cached in display lists
-  vtkCollectionSimpleIterator rsit;
-  vtkRenderer *ren;
-  for (this->Renderers->InitTraversal(rsit); 
-       (ren = this->Renderers->GetNextRenderer(rsit));)
-    {
-    ren->SetRenderWindow(this);
-    }
-  
   this->OpenGLInit();
 }
 
@@ -1418,11 +1405,19 @@ void vtkWin32OpenGLRenderWindow::CleanUpOffScreenRendering(void)
       {
       vtkErrorMacro("wglDeleteContext failed in CleanUpOffScreenRendering(), error: " << GetLastError());
       }
+    this->ContextId=0;
     }
 }
 
 void vtkWin32OpenGLRenderWindow::ResumeScreenRendering(void)
-{  
+{
+  // release OpenGL graphics resources before switch back to on-screen. 
+  if(this->ContextId!=0)
+    {
+      this->MakeCurrent();
+      this->CleanUpRenderers();
+    }
+
   this->Mapped = this->ScreenMapped;
   this->Size[0] = this->ScreenWindowSize[0];
   this->Size[1] = this->ScreenWindowSize[1];
@@ -1430,14 +1425,6 @@ void vtkWin32OpenGLRenderWindow::ResumeScreenRendering(void)
   this->DoubleBuffer = this->ScreenDoubleBuffer;
   this->ContextId = this->ScreenContextId;
   this->MakeCurrent();
-
-  vtkRenderer* ren;
-  vtkCollectionSimpleIterator rsit;
-  for (this->Renderers->InitTraversal(rsit);
-       (ren = this->Renderers->GetNextRenderer(rsit));)
-    {
-    ren->SetRenderWindow(this);
-    }
 }
 
 //----------------------------------------------------------------------------
@@ -1517,6 +1504,9 @@ void vtkWin32OpenGLRenderWindow::SetCurrentCursor(int shape)
       cursorName = IDC_ARROW;
 #endif
       break;
+    case VTK_CURSOR_CROSSHAIR:
+      cursorName = IDC_CROSS;
+      break;
     }
   
   if (cursorName)
