@@ -1,15 +1,15 @@
 /*=========================================================================
 
-Program:   Visualization Toolkit
-Module:    $RCSfile: vtkXOpenGLRenderWindow.cxx,v $
+  Program:   Visualization Toolkit
+  Module:    $RCSfile: vtkXOpenGLRenderWindow.cxx,v $
 
-Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-All rights reserved.
-See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
+  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+  All rights reserved.
+  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
 
-This software is distributed WITHOUT ANY WARRANTY; without even
-the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-PURPOSE.  See the above copyright notice for more information.
+     This software is distributed WITHOUT ANY WARRANTY; without even
+     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
+     PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
 #ifndef VTK_IMPLEMENT_MESA_CXX
@@ -110,7 +110,7 @@ vtkXOpenGLRenderWindowInternal::vtkXOpenGLRenderWindowInternal(
 
 
 #ifndef VTK_IMPLEMENT_MESA_CXX
-vtkCxxRevisionMacro(vtkXOpenGLRenderWindow, "$Revision: 1.87 $");
+vtkCxxRevisionMacro(vtkXOpenGLRenderWindow, "$Revision: 1.93 $");
 vtkStandardNewMacro(vtkXOpenGLRenderWindow);
 #endif
 
@@ -329,7 +329,7 @@ XVisualInfo *vtkXOpenGLRenderWindow::GetDesiredVisualInfo()
   // get the default display connection 
   if (!this->DisplayId)
     {
-    this->DisplayId = XOpenDisplay((char *)NULL); 
+    this->DisplayId = XOpenDisplay(static_cast<char *>(NULL)); 
 
     if (this->DisplayId == NULL)
       {
@@ -402,17 +402,17 @@ XVisualInfo *vtkXOpenGLRenderWindow::GetDesiredVisualInfo()
 
 vtkXOpenGLRenderWindow::vtkXOpenGLRenderWindow()
 {
-  this->ParentId = (Window)NULL;
+  this->ParentId = static_cast<Window>(NULL);
   this->ScreenSize[0] = 0;
   this->ScreenSize[1] = 0;
   this->OwnDisplay = 0;
   this->CursorHidden = 0;
   this->ForceMakeCurrent = 0;
   this->UsingHardware = 0;
-  this->DisplayId = (Display *)NULL;
-  this->WindowId = (Window)NULL;
-  this->NextWindowId = (Window)NULL;
-  this->ColorMap = (Colormap)0;
+  this->DisplayId = static_cast<Display *>(NULL);
+  this->WindowId = static_cast<Window>(NULL);
+  this->NextWindowId = static_cast<Window>(NULL);
+  this->ColorMap = static_cast<Colormap>(0);
   this->OwnWindow = 0;
  
   this->Internal = new vtkXOpenGLRenderWindowInternal(this);
@@ -436,6 +436,15 @@ vtkXOpenGLRenderWindow::~vtkXOpenGLRenderWindow()
 {
   // close-down all system-specific drawing resources
   this->Finalize();
+  
+  vtkRenderer* ren;
+  this->Renderers->InitTraversal();
+  for ( ren = vtkOpenGLRenderer::SafeDownCast(this->Renderers->GetNextItemAsObject());
+        ren != NULL;
+        ren = vtkOpenGLRenderer::SafeDownCast(this->Renderers->GetNextItemAsObject())  )
+    {
+    ren->SetRenderWindow(NULL);
+    }
 
   delete this->Internal;
 
@@ -504,8 +513,8 @@ void vtkXOpenGLRenderWindow::CreateAWindow()
   if ((this->Position[0] >= 0)&&(this->Position[1] >= 0))
     {
     xsh.flags |= USPosition;
-    xsh.x =  (int)(this->Position[0]);
-    xsh.y =  (int)(this->Position[1]);
+    xsh.x =  static_cast<int>(this->Position[0]);
+    xsh.y =  static_cast<int>(this->Position[1]);
     }
   
   x = ((this->Position[0] >= 0) ? this->Position[0] : 5);
@@ -519,7 +528,7 @@ void vtkXOpenGLRenderWindow::CreateAWindow()
   // get the default display connection 
   if (!this->DisplayId)
     {
-    this->DisplayId = XOpenDisplay((char *)NULL); 
+    this->DisplayId = XOpenDisplay(static_cast<char *>(NULL));
     if (this->DisplayId == NULL) 
       {
       vtkErrorMacro(<< "bad X server connection. DISPLAY="
@@ -582,9 +591,9 @@ void vtkXOpenGLRenderWindow::CreateAWindow()
     vtkDebugMacro(<< "Resizing the xwindow\n");
     XResizeWindow(this->DisplayId,this->WindowId,
                   ((this->Size[0] > 0) ? 
-                   (int)(this->Size[0]) : 300),
+                   static_cast<int>(this->Size[0]) : 300),
                   ((this->Size[1] > 0) ? 
-                   (int)(this->Size[1]) : 300));
+                   static_cast<int>(this->Size[1]) : 300));
     XSync(this->DisplayId,False);
     }
 
@@ -630,11 +639,13 @@ void vtkXOpenGLRenderWindow::CreateAWindow()
     XSync(this->DisplayId,False);
     XGetWindowAttributes(this->DisplayId,
                          this->WindowId,&winattr);
+    // guarantee that the window is mapped before the program continues 
+    // on to do the OpenGL rendering.
     while (winattr.map_state == IsUnmapped)
       {
       XGetWindowAttributes(this->DisplayId,
                            this->WindowId,&winattr);
-      };
+      }
     }
   // free the visual info
   if (v)
@@ -722,6 +733,7 @@ void vtkXOpenGLRenderWindow::DestroyWindow()
         ren = vtkOpenGLRenderer::SafeDownCast(this->Renderers->GetNextItemAsObject())  )
     {
     ren->SetRenderWindow(NULL);
+    ren->SetRenderWindow(this);
     }
 
 
@@ -732,14 +744,14 @@ void vtkXOpenGLRenderWindow::DestroyWindow()
     /* first delete all the old lights */
     for (short cur_light = GL_LIGHT0; cur_light < GL_LIGHT0+MAX_LIGHTS; cur_light++)
       {
-      glDisable((GLenum)cur_light);
+      glDisable(static_cast<GLenum>(cur_light));
       }
 
     /* now delete all textures */
     glDisable(GL_TEXTURE_2D);
     for (int i = 1; i < this->TextureResourceIds->GetNumberOfIds(); i++)
       {
-      GLuint txId = (GLuint) this->TextureResourceIds->GetId(i);
+      GLuint txId = static_cast<GLuint>(this->TextureResourceIds->GetId(i));
 #ifdef GL_VERSION_1_1
       if (glIsTexture(txId))
         {
@@ -765,7 +777,7 @@ void vtkXOpenGLRenderWindow::DestroyWindow()
   if (this->OwnWindow && this->DisplayId && this->WindowId)
     {
     XDestroyWindow(this->DisplayId,this->WindowId);
-    this->WindowId = (Window)NULL;
+    this->WindowId = static_cast<Window>(NULL);
     }
 
   // if we create the display, we'll delete it
@@ -818,7 +830,7 @@ void vtkXOpenGLRenderWindow::CreateOffScreenWindow(int width, int height)
       // get the default display connection 
       if (!this->DisplayId)
         {
-        this->DisplayId = XOpenDisplay((char *)NULL); 
+        this->DisplayId = XOpenDisplay(static_cast<char *>(NULL));
         if (this->DisplayId == NULL) 
           {
           vtkErrorMacro(<< "bad X server connection. DISPLAY="
@@ -991,7 +1003,8 @@ void vtkXOpenGLRenderWindow::ResizeOffScreenWindow(int width, int height)
     }
 
   if(this->Internal->PixmapContextId ||
-     this->Internal->PbufferContextId 
+     this->Internal->PbufferContextId || 
+     this->OffScreenUseFrameBuffer 
 #ifdef VTK_OPENGL_HAS_OSMESA
      || this->Internal->OffScreenContextId
 #endif
@@ -1152,7 +1165,7 @@ void vtkXOpenGLRenderWindow::WindowRemap()
 
   // set the default windowid 
   this->WindowId = this->NextWindowId;
-  this->NextWindowId = (Window)NULL;
+  this->NextWindowId = static_cast<Window>(NULL);
 
   // set everything up again 
   this->Initialize();
@@ -1369,17 +1382,20 @@ void *vtkXOpenGLRenderWindow::GetGenericContext()
 #endif
     if(this->OffScreenRendering && this->Internal->PbufferContextId)
       {
-      return (void *)this->Internal->PbufferContextId;
+      return static_cast<void *>(this->Internal->PbufferContextId);
       }
     else if(this->OffScreenRendering && this->Internal->PixmapContextId)
       {
-      return (void *)this->Internal->PixmapContextId;
+      return static_cast<void *>(this->Internal->PixmapContextId);
       }
     else
       {
-      static GC gc = (GC) NULL;
-      if (!gc) gc = XCreateGC(this->DisplayId, this->WindowId, 0, 0);
-      return (void *) gc;
+      static GC gc = static_cast<GC>(NULL);
+      if (!gc)
+        {
+        gc = XCreateGC(this->DisplayId, this->WindowId, 0, 0);
+        }
+      return static_cast<void *>(gc);
       }
 
 }
@@ -1394,7 +1410,7 @@ int vtkXOpenGLRenderWindow::GetEventPending()
     return vtkXOpenGLRenderWindowFoundMatch;
     }
   XCheckIfEvent(this->DisplayId, &report, vtkXOpenGLRenderWindowPredProc, 
-                (char *)this->WindowId);
+                reinterpret_cast<char *>(this->WindowId));
   return vtkXOpenGLRenderWindowFoundMatch;
 }
 
@@ -1404,7 +1420,7 @@ int *vtkXOpenGLRenderWindow::GetScreenSize()
   // get the default display connection 
   if (!this->DisplayId)
     {
-    this->DisplayId = XOpenDisplay((char *)NULL); 
+    this->DisplayId = XOpenDisplay(static_cast<char *>(NULL)); 
     if (this->DisplayId == NULL) 
       {
       vtkErrorMacro(<< "bad X server connection. DISPLAY="
@@ -1455,7 +1471,7 @@ Display *vtkXOpenGLRenderWindow::GetDisplayId()
   // get the default display connection 
   if (!this->DisplayId)
     {
-    this->DisplayId = XOpenDisplay((char *)NULL); 
+    this->DisplayId = XOpenDisplay(static_cast<char *>(NULL));
     if (this->DisplayId == NULL) 
       {
       vtkErrorMacro(<< "bad X server connection. DISPLAY="
@@ -1463,7 +1479,7 @@ Display *vtkXOpenGLRenderWindow::GetDisplayId()
       }
     this->OwnDisplay = 1;
     }
-  vtkDebugMacro(<< "Returning DisplayId of " << (void *)this->DisplayId << "\n"); 
+  vtkDebugMacro(<< "Returning DisplayId of " << static_cast<void *>(this->DisplayId) << "\n"); 
 
   return this->DisplayId;
 }
@@ -1471,14 +1487,14 @@ Display *vtkXOpenGLRenderWindow::GetDisplayId()
 // Get this RenderWindow's parent X window id.
 Window vtkXOpenGLRenderWindow::GetParentId()
 {
-  vtkDebugMacro(<< "Returning ParentId of " << (void *)this->ParentId << "\n");
+  vtkDebugMacro(<< "Returning ParentId of " << reinterpret_cast<void *>(this->ParentId) << "\n");
   return this->ParentId;
 }
 
 // Get this RenderWindow's X window id.
 Window vtkXOpenGLRenderWindow::GetWindowId()
 {
-  vtkDebugMacro(<< "Returning WindowId of " << (void *)this->WindowId << "\n");
+  vtkDebugMacro(<< "Returning WindowId of " << reinterpret_cast<void *>(this->WindowId) << "\n");
   return this->WindowId;
 }
 
@@ -1510,7 +1526,7 @@ void vtkXOpenGLRenderWindow::SetParentId(Window arg)
 //     return;
 //     }
   
-  vtkDebugMacro(<< "Setting ParentId to " << (void *)arg << "\n"); 
+  vtkDebugMacro(<< "Setting ParentId to " << reinterpret_cast<void *>(arg) << "\n"); 
 
   this->ParentId = arg;
 }
@@ -1518,7 +1534,7 @@ void vtkXOpenGLRenderWindow::SetParentId(Window arg)
 // Set this RenderWindow's X window id to a pre-existing window.
 void vtkXOpenGLRenderWindow::SetWindowId(Window arg)
 {
-  vtkDebugMacro(<< "Setting WindowId to " << (void *)arg << "\n"); 
+  vtkDebugMacro(<< "Setting WindowId to " << reinterpret_cast<void *>(arg) << "\n"); 
 
   this->WindowId = arg;
 
@@ -1537,7 +1553,7 @@ void vtkXOpenGLRenderWindow::SetWindowInfo(char *info)
   // get the default display connection 
   if (!this->DisplayId)
     {
-    this->DisplayId = XOpenDisplay((char *)NULL); 
+    this->DisplayId = XOpenDisplay(static_cast<char *>(NULL));
     if (this->DisplayId == NULL) 
       {
       vtkErrorMacro(<< "bad X server connection. DISPLAY="
@@ -1560,7 +1576,7 @@ void vtkXOpenGLRenderWindow::SetNextWindowInfo(char *info)
   int tmp;
   sscanf(info,"%i",&tmp);
  
-  this->SetNextWindowId((Window)tmp);
+  this->SetNextWindowId(static_cast<Window>(tmp));
 }
 
 // Sets the X window id of the window that WILL BE created.
@@ -1571,7 +1587,7 @@ void vtkXOpenGLRenderWindow::SetParentInfo(char *info)
   // get the default display connection 
   if (!this->DisplayId)
     {
-    this->DisplayId = XOpenDisplay((char *)NULL); 
+    this->DisplayId = XOpenDisplay(static_cast<char *>(NULL));
     if (this->DisplayId == NULL) 
       {
       vtkErrorMacro(<< "bad X server connection. DISPLAY="
@@ -1590,11 +1606,11 @@ void vtkXOpenGLRenderWindow::SetParentInfo(char *info)
 
 void vtkXOpenGLRenderWindow::SetWindowId(void *arg)
 {
-  this->SetWindowId((Window)arg);
+  this->SetWindowId(reinterpret_cast<Window>(arg));
 }
 void vtkXOpenGLRenderWindow::SetParentId(void *arg)
 {
-  this->SetParentId((Window)arg);
+  this->SetParentId(reinterpret_cast<Window>(arg));
 }
 
 const char* vtkXOpenGLRenderWindow::ReportCapabilities()
@@ -1618,10 +1634,10 @@ const char* vtkXOpenGLRenderWindow::ReportCapabilities()
   const char *clientExtensions = glXGetClientString(this->DisplayId,
                                                     GLX_EXTENSIONS);
   const char *glxExtensions = glXQueryExtensionsString(this->DisplayId,scrnum);
-  const char *glVendor = (const char *) glGetString(GL_VENDOR);
-  const char *glRenderer = (const char *) glGetString(GL_RENDERER);
-  const char *glVersion = (const char *) glGetString(GL_VERSION);
-  const char *glExtensions = (const char *) glGetString(GL_EXTENSIONS);
+  const char *glVendor = reinterpret_cast<const char *>(glGetString(GL_VENDOR));
+  const char *glRenderer = reinterpret_cast<const char *>(glGetString(GL_RENDERER));
+  const char *glVersion = reinterpret_cast<const char *>(glGetString(GL_VERSION));
+  const char *glExtensions = reinterpret_cast<const char *>(glGetString(GL_EXTENSIONS));
 
   vtksys_ios::ostringstream strm;
   strm << "server glx vendor string:  " << serverVendor << endl;
@@ -1642,11 +1658,14 @@ const char* vtkXOpenGLRenderWindow::ReportCapabilities()
 
   for (int i = 0; i < n; i++) 
     {
-    if (i != n-1) {
-    strm << extlist[i] << ", ";
-    } else {
-    strm << extlist[i] << endl;
-    }
+    if (i != n-1)
+      {
+      strm << extlist[i] << ", ";
+      }
+    else
+      {
+      strm << extlist[i] << endl;
+      }
     }
 
   delete[] this->Capabilities;
@@ -1661,7 +1680,7 @@ const char* vtkXOpenGLRenderWindow::ReportCapabilities()
 
 int vtkXOpenGLRenderWindow::SupportsOpenGL()
 {
-  MakeCurrent();
+  this->MakeCurrent();
   if (!this->DisplayId) 
     {
     return 0;
@@ -1681,7 +1700,7 @@ int vtkXOpenGLRenderWindow::SupportsOpenGL()
 
 int vtkXOpenGLRenderWindow::IsDirect()
 {
-  MakeCurrent();
+  this->MakeCurrent();
   this->UsingHardware = 0;
   if (this->OffScreenRendering && this->Internal->PbufferContextId)
     {
@@ -1708,7 +1727,7 @@ void vtkXOpenGLRenderWindow::SetWindowName(const char * cname)
   strcpy(name, cname);
   XTextProperty win_name_text_prop;
 
-  vtkOpenGLRenderWindow::SetWindowName( name );
+  this->vtkOpenGLRenderWindow::SetWindowName( name );
 
   if (this->Mapped)
     {
@@ -1731,14 +1750,14 @@ void vtkXOpenGLRenderWindow::SetWindowName(const char * cname)
 // Specify the X window id to use if a WindowRemap is done.
 void vtkXOpenGLRenderWindow::SetNextWindowId(Window arg)
 {
-  vtkDebugMacro(<< "Setting NextWindowId to " << (void *)arg << "\n"); 
+  vtkDebugMacro(<< "Setting NextWindowId to " << reinterpret_cast<void *>(arg) << "\n"); 
 
   this->NextWindowId = arg;
 }
 
 void vtkXOpenGLRenderWindow::SetNextWindowId(void *arg)
 {
-  this->SetNextWindowId((Window)arg);
+  this->SetNextWindowId(reinterpret_cast<Window>(arg));
 }
 
 
@@ -1746,7 +1765,7 @@ void vtkXOpenGLRenderWindow::SetNextWindowId(void *arg)
 // X display id.
 void vtkXOpenGLRenderWindow::SetDisplayId(Display  *arg)
 {
-  vtkDebugMacro(<< "Setting DisplayId to " << (void *)arg << "\n"); 
+  vtkDebugMacro(<< "Setting DisplayId to " << static_cast<void *>(arg) << "\n"); 
 
   this->DisplayId = arg;
   this->OwnDisplay = 0;
@@ -1754,7 +1773,7 @@ void vtkXOpenGLRenderWindow::SetDisplayId(Display  *arg)
 }
 void vtkXOpenGLRenderWindow::SetDisplayId(void *arg)
 {
-  this->SetDisplayId((Display *)arg);
+  this->SetDisplayId(static_cast<Display *>(arg));
   this->OwnDisplay = 0;
 }
 
@@ -1881,15 +1900,15 @@ void *vtkXOpenGLRenderWindow::GetGenericWindowId()
 #ifdef VTK_OPENGL_HAS_OSMESA
   if (this->OffScreenRendering && this->Internal->OffScreenWindow)
     {
-    return (void *)this->Internal->OffScreenWindow;
+    return reinterpret_cast<void*>(this->Internal->OffScreenWindow);
     }
   else
 #endif
     if(this->OffScreenRendering)
       {
-      return (void*)this->Internal->PixmapWindowId;
+      return reinterpret_cast<void*>(this->Internal->PixmapWindowId);
       }
-  return (void *)this->WindowId;
+  return reinterpret_cast<void*>(this->WindowId);
 }
 
 void vtkXOpenGLRenderWindow::SetCurrentCursor(int shape)
@@ -1912,6 +1931,7 @@ void vtkXOpenGLRenderWindow::SetCurrentCursor(int shape)
       
   switch (shape)
     {
+    case VTK_CURSOR_CROSSHAIR: // XC_crosshair sucks on linux, default to arrow
     case VTK_CURSOR_ARROW:
       if (!this->XCArrow)
         {
