@@ -29,7 +29,7 @@
 
 #include <vtkstd/map>
 
-vtkCxxRevisionMacro(vtkXRenderWindowInteractor, "$Revision: 1.131 $");
+vtkCxxRevisionMacro(vtkXRenderWindowInteractor, "$Revision: 1.137 $");
 vtkStandardNewMacro(vtkXRenderWindowInteractor);
 
 // Map between the X native id to our own integer count id.  Note this
@@ -43,6 +43,7 @@ public:
     {
     this->TimerIdCount = 1;
     }
+
   int CreateLocalId(XtIntervalId xid)
     {
     int id = this->TimerIdCount++;
@@ -50,10 +51,12 @@ public:
     this->XToLocal[xid] = id;
     return id;
     }
+
   int GetLocalId(XtIntervalId xid)
     {
     return this->XToLocal[xid];
     }
+
   XtIntervalId DestroyLocalId(int id)
     {
     XtIntervalId xid = this->LocalToX[id];
@@ -61,6 +64,7 @@ public:
     this->XToLocal.erase(xid);
     return xid;
     }
+
 private:
   int TimerIdCount;
   vtkstd::map<int, XtIntervalId> LocalToX;
@@ -81,21 +85,35 @@ OptionsRec      Options;
 
 XtResource resources[] =
 {
-        {(char *) "visual", (char *) "Visual", XtRVisual, sizeof (Visual *),
-        XtOffsetOf (OptionsRec, visual), XtRImmediate, NULL},
-        {(char *) "depth", (char *) "Depth", XtRInt, sizeof (int),
-        XtOffsetOf (OptionsRec, depth), XtRImmediate, NULL},
+  {(char *)("visual"),
+   (char *)("Visual"),
+   XtRVisual,
+   sizeof (Visual *),
+   XtOffsetOf (OptionsRec, visual),
+   XtRImmediate,
+   NULL},
+  {(char *)("depth"),
+   (char *)("Depth"),
+   XtRInt,
+   sizeof (int),
+   XtOffsetOf (OptionsRec, depth),
+   XtRImmediate,
+   NULL},
 };
 
 XrmOptionDescRec Desc[] =
 {
-        {(char *) "-visual", (char *) "*visual", XrmoptionSepArg, NULL},
-        {(char *) "-depth", (char *) "*depth", XrmoptionSepArg, NULL}
+  {(char *)("-visual"),
+   (char *)("*visual"),
+   XrmoptionSepArg,
+   NULL},
+  {(char *)("-depth"),
+   (char *)("*depth"),
+   XrmoptionSepArg, NULL}
 };
 
 
 //-------------------------------------------------------------------------
-// Construct an instance so that the light follows the camera motion.
 vtkXRenderWindowInteractor::vtkXRenderWindowInteractor()
 {
   this->Internal = new vtkXRenderWindowInteractorInternals;
@@ -114,11 +132,14 @@ vtkXRenderWindowInteractor::vtkXRenderWindowInteractor()
 vtkXRenderWindowInteractor::~vtkXRenderWindowInteractor()
 {
   this->Disable();
+
   if(this->OwnTop)
     {
     XtDestroyWidget(this->Top);
     }
+
   this->BreakXtLoopCallback->Delete();
+
   if (vtkXRenderWindowInteractor::App)
     {
     if(vtkXRenderWindowInteractor::NumAppInitialized == 1)
@@ -131,6 +152,7 @@ vtkXRenderWindowInteractor::~vtkXRenderWindowInteractor()
       }
     vtkXRenderWindowInteractor::NumAppInitialized--;
     }
+
   delete this->Internal;
 }
 
@@ -240,21 +262,21 @@ void vtkXRenderWindowInteractor::Start()
     {
     this->Initialize();
     }
-  if (! this->Initialized )
+  if (!this->Initialized)
     {
     return;
     }
-  
-  this->AddObserver(vtkCommand::ExitEvent, this->BreakXtLoopCallback);
+
+  unsigned long ExitTag = this->AddObserver(vtkCommand::ExitEvent, this->BreakXtLoopCallback);
   this->BreakLoopFlag = 0;
   do 
     {
     XEvent event;
     XtAppNextEvent(vtkXRenderWindowInteractor::App, &event);
     XtDispatchEvent(&event);
-    } 
+    }
   while (this->BreakLoopFlag == 0);
-  this->RemoveObserver(this->BreakXtLoopCallback);
+  this->RemoveObserver(ExitTag);
 }
 
 //-------------------------------------------------------------------------
@@ -273,6 +295,11 @@ void vtkXRenderWindowInteractor::Initialize(XtAppContext app)
 // want to have mouse interaction.
 void vtkXRenderWindowInteractor::Initialize()
 {
+  if (this->Initialized)
+    {
+    return;
+    }
+
   vtkXOpenGLRenderWindow *ren;
   int depth;
   Colormap cmap;
@@ -281,7 +308,7 @@ void vtkXRenderWindowInteractor::Initialize()
   int *position;
   int argc = 0;
 
-  
+
   // make sure we have a RenderWindow and camera
   if ( ! this->RenderWindow)
     {
@@ -290,7 +317,7 @@ void vtkXRenderWindowInteractor::Initialize()
     }
 
   this->Initialized = 1;
-  ren = (vtkXOpenGLRenderWindow *)(this->RenderWindow);
+  ren = static_cast<vtkXOpenGLRenderWindow *>(this->RenderWindow);
 
   // do initialization stuff if not initialized yet
   if (vtkXRenderWindowInteractor::App)
@@ -362,7 +389,7 @@ void vtkXRenderWindowInteractor::Initialize()
   else
     {
     XWindowAttributes attribs;
-    
+
     XtRealizeWidget(this->Top);
     XSync(this->DisplayId,False);
     ren->SetWindowId(XtWindow(this->Top));
@@ -394,6 +421,11 @@ void vtkXRenderWindowInteractor::Enable()
     return;
     }
 
+  if (!this->Top)
+    {
+    vtkErrorMacro(<< "Calling Enable too early, before Top is set...");
+    }
+
   // Add the event handler to the system.
   // If we change the types of events processed by this handler, then
   // we need to change the Disable() routine to match.  In order for Disable()
@@ -401,15 +433,16 @@ void vtkXRenderWindowInteractor::Enable()
   // passed to XtAddEventHandler and XtRemoveEventHandler must MATCH
   // PERFECTLY
   XtAddEventHandler(this->Top,
-                    KeyPressMask | KeyReleaseMask | 
+                    KeyPressMask | KeyReleaseMask |
                     ButtonPressMask | ButtonReleaseMask |
-                    ExposureMask | StructureNotifyMask | 
-                    EnterWindowMask | LeaveWindowMask | 
+                    ExposureMask | StructureNotifyMask |
+                    EnterWindowMask | LeaveWindowMask |
                     PointerMotionHintMask | PointerMotionMask,
                     True,  // True means we also observe ClientMessage
-                    vtkXRenderWindowInteractorCallback,(XtPointer)this);
+                    vtkXRenderWindowInteractorCallback,
+                    static_cast<XtPointer>(this));
 
-   // Setup for capturing the window deletion
+  // Setup for capturing the window deletion
   this->KillAtom = XInternAtom(this->DisplayId,"WM_DELETE_WINDOW",False);
   XSetWMProtocols(this->DisplayId,this->WindowId,&this->KillAtom,1);
 
@@ -425,7 +458,9 @@ void vtkXRenderWindowInteractor::Disable()
     {
     return;
     }
-  
+
+  this->Enabled = 0;
+
   // Remove the event handler to the system.
   // If we change the types of events processed by this handler, then
   // we need to change the Disable() routine to match.  In order for Disable()
@@ -437,14 +472,17 @@ void vtkXRenderWindowInteractor::Disable()
   // keep track of the window size (we will not render if we are disabled,
   // we simply track the window size changes for a possible Enable()).
   // Expose events are disabled.
-  XtRemoveEventHandler(this->Top,
-                    KeyPressMask | KeyReleaseMask | ButtonPressMask | 
-                    ExposureMask | ButtonReleaseMask |
-                    EnterWindowMask | LeaveWindowMask | 
-                    PointerMotionHintMask | PointerMotionMask,
-                    True,vtkXRenderWindowInteractorCallback,(XtPointer)this);
+  if (this->Top)
+    {
+    XtRemoveEventHandler(this->Top,
+                        KeyPressMask | KeyReleaseMask | ButtonPressMask | 
+                        ExposureMask | ButtonReleaseMask |
+                        EnterWindowMask | LeaveWindowMask | 
+                        PointerMotionHintMask | PointerMotionMask,
+                        True,vtkXRenderWindowInteractorCallback,
+                        static_cast<XtPointer>(this));
+    }
 
-  this->Enabled = 0;
   this->Modified();
 }
 
@@ -452,6 +490,7 @@ void vtkXRenderWindowInteractor::Disable()
 void vtkXRenderWindowInteractor::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
+
   if (vtkXRenderWindowInteractor::App)
     {
     os << indent << "App: " << this->App << "\n";
@@ -460,22 +499,23 @@ void vtkXRenderWindowInteractor::PrintSelf(ostream& os, vtkIndent indent)
     {
     os << indent << "App: (none)\n";
     }
+
   os << indent << "BreakLoopFlag: " 
      << (this->BreakLoopFlag ? "On\n" : "Off\n");
 }
 
-//void vtkXRenderWindowInteractor::UpdateSize(int x,int y)
-//{
-//  // if the size changed send this on to the RenderWindow
-//  if ((x != this->Size[0])||(y != this->Size[1]))
-//    {
-//    this->Size[0] = x;
-//    this->Size[1] = y;
-//    this->RenderWindow->SetSize(x,y);
-//    }
-//
-//}
- 
+//-------------------------------------------------------------------------
+void vtkXRenderWindowInteractor::UpdateSize(int x,int y)
+{
+  // if the size changed send this on to the RenderWindow
+  if ((x != this->Size[0])||(y != this->Size[1]))
+    {
+    this->Size[0] = x;
+    this->Size[1] = y;
+    this->RenderWindow->SetSize(x,y);
+    }
+}
+
 //-------------------------------------------------------------------------
 void vtkXRenderWindowInteractorTimer(XtPointer client_data,
                                      XtIntervalId *id)
@@ -483,14 +523,15 @@ void vtkXRenderWindowInteractorTimer(XtPointer client_data,
   vtkXRenderWindowInteractor *me = 
     static_cast<vtkXRenderWindowInteractor*>(client_data);
   XtIntervalId xid = *id;
-  int platformTimerId = me->Internal->GetLocalId(xid);
 
+  int platformTimerId = me->Internal->GetLocalId(xid);
   int timerId = me->GetVTKTimerId(platformTimerId);
-  if (me->Enabled) 
+
+  if (me->GetEnabled())
     {
-    me->InvokeEvent(vtkCommand::TimerEvent,(void*)&timerId);
+    me->InvokeEvent(vtkCommand::TimerEvent,&timerId);
     }
-  
+
   if ( ! me->IsOneShotTimer(timerId) )
     {
     me->ResetTimer(timerId);
@@ -505,10 +546,12 @@ int vtkXRenderWindowInteractor::InternalCreateTimer(int vtkNotUsed(timerId),
 {
   duration = (duration > 0 ? duration : this->TimerDuration);
   XtIntervalId xid =
-    this->AddTimeOut(vtkXRenderWindowInteractor::App, duration,
+    this->AddTimeOut(vtkXRenderWindowInteractor::App,
+                     duration,
                      vtkXRenderWindowInteractorTimer,
-                     (XtPointer)this);
-  return this->Internal->CreateLocalId(xid);
+                     static_cast<XtPointer>(this));
+  int platformTimerId = this->Internal->CreateLocalId(xid);
+  return platformTimerId;
 }
 
 //-------------------------------------------------------------------------
@@ -527,16 +570,16 @@ void vtkXRenderWindowInteractorCallback(Widget vtkNotUsed(w),
   vtkXRenderWindowInteractor *me;
   int xp, yp;
 
-  me = (vtkXRenderWindowInteractor *)client_data;
-  
+  me = static_cast<vtkXRenderWindowInteractor *>(client_data);
+
   switch (event->type) 
     {
     case Expose:
       {
       if (!me->Enabled) 
-       {
-       return;
-       }
+        {
+        return;
+        }
       XEvent result;
       while (XCheckTypedWindowEvent(me->DisplayId, 
                                     me->WindowId,
@@ -546,12 +589,11 @@ void vtkXRenderWindowInteractorCallback(Widget vtkNotUsed(w),
         // just getting the expose configure event
         event = &result;
         }
-      int width = (reinterpret_cast<XConfigureEvent *>(event))->width;
-      int height = (reinterpret_cast<XConfigureEvent *>(event))->height;
-      me->SetEventSize(width, height);
-      xp = (reinterpret_cast<XButtonEvent*>(event))->x;
-      yp = (reinterpret_cast<XButtonEvent*>(event))->y;
-      yp = me->Size[1] - xp - 1;
+      XExposeEvent* exposeEvent = reinterpret_cast<XExposeEvent *>(event);
+      me->SetEventSize(exposeEvent->width, exposeEvent->height);
+      xp = exposeEvent->x;
+      yp = exposeEvent->y;
+      yp = me->Size[1] - yp - 1;
       me->SetEventPosition(xp, yp);
       // only render if we are currently accepting events
       if (me->Enabled)
@@ -575,8 +617,10 @@ void vtkXRenderWindowInteractorCallback(Widget vtkNotUsed(w),
     case ConfigureNotify: 
       {
       XEvent result;
-      while (XCheckTypedWindowEvent(me->DisplayId, me->WindowId,
-                                    ConfigureNotify, &result))
+      while (XCheckTypedWindowEvent(me->DisplayId,
+                                    me->WindowId,
+                                    ConfigureNotify,
+                                    &result))
         {
         // just getting the last configure event
         event = &result;
@@ -625,7 +669,7 @@ void vtkXRenderWindowInteractorCallback(Widget vtkNotUsed(w),
         }
       else
         {
-          MousePressTime = reinterpret_cast<XButtonEvent*>(event)->time;
+        MousePressTime = reinterpret_cast<XButtonEvent*>(event)->time;
         }
 
       me->SetEventInformationFlipY(xp, 
@@ -864,4 +908,3 @@ void vtkXRenderWindowInteractor::Callback(Widget w,
 {
   vtkXRenderWindowInteractorCallback(w, client_data, event, ctd);
 }
-
