@@ -28,25 +28,23 @@
 
 #include <math.h>
 
-double verdictSqrt2 = sqrt( (double)2.0 );
-
-int numberGaussPoints;
-int numberNodes;
-int numberDims;
-double gaussPointY[maxNumberGaussPoints];
-double gaussWeight[maxNumberGaussPoints];
-double shapeFunction[maxTotalNumberGaussPoints][maxNumberNodes];
-double dndy1GaussPts[maxTotalNumberGaussPoints][maxNumberNodes];
-double dndy2GaussPts[maxTotalNumberGaussPoints][maxNumberNodes];
-double dndy3GaussPts[maxTotalNumberGaussPoints][maxNumberNodes];
-double totalGaussWeight[maxTotalNumberGaussPoints];
-int totalNumberGaussPts;
-double y1Area[maxNumberGaussPointsTri];
-double y2Area[maxNumberGaussPointsTri];
-double y1Volume[maxNumberGaussPointsTet];
-double y2Volume[maxNumberGaussPointsTet];
-double y3Volume[maxNumberGaussPointsTet];
-double y4Volume[maxNumberGaussPointsTet];  
+static int numberGaussPoints;
+static int numberNodes;
+static int numberDims;
+static double gaussPointY[maxNumberGaussPoints];
+static double gaussWeight[maxNumberGaussPoints];
+static double shapeFunction[maxTotalNumberGaussPoints][maxNumberNodes];
+static double dndy1GaussPts[maxTotalNumberGaussPoints][maxNumberNodes];
+static double dndy2GaussPts[maxTotalNumberGaussPoints][maxNumberNodes];
+static double dndy3GaussPts[maxTotalNumberGaussPoints][maxNumberNodes];
+static double totalGaussWeight[maxTotalNumberGaussPoints];
+static int totalNumberGaussPts;
+static double y1Area[maxNumberGaussPointsTri];
+static double y2Area[maxNumberGaussPointsTri];
+static double y1Volume[maxNumberGaussPointsTet];
+static double y2Volume[maxNumberGaussPointsTet];
+static double y3Volume[maxNumberGaussPointsTet];
+static double y4Volume[maxNumberGaussPointsTet];  
 
 void GaussIntegration::initialize(int n, int m, int dim, int tri)
 {
@@ -661,6 +659,12 @@ void GaussIntegration::get_signs_for_node_local_coord_hex(int node_id, double &s
       sign_node_y2 = 0.;
       sign_node_y3 = 1.;
       break;
+   default:
+      // Should not be possible to get here, but if we do, at least results will be consistent, not random
+      sign_node_y1 = 0.;
+      sign_node_y2 = 0.;
+      sign_node_y3 = 0.;
+      break;
    }
 }
 
@@ -1097,5 +1101,12 @@ void GaussIntegration::get_node_local_coord_tet(int node_id, double &y1, double
       y3 = 0.5;
       y4 = 0.;
       break;
+   default:
+      // Should not be possible to get here, but if we do, at least results will be consistent, not random
+      y1 = 0.;
+      y2 = 0.;
+      y3 = 0.;
+      y4 = 0.;
+      break;
    }
 }
