@@ -12,11 +12,11 @@
      PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
-/*----------------------------------------------------------------------------
- Copyright (c) Sandia Corporation
- See Copyright.txt or http://www.paraview.org/HTML/Copyright.html for details.
-----------------------------------------------------------------------------*/
-
+/*-------------------------------------------------------------------------
+  Copyright 2008 Sandia Corporation.
+  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
+  the U.S. Government retains certain rights in this software.
+-------------------------------------------------------------------------*/
 #include "vtkTreeLayoutView.h"
 
 #include "vtkActor.h"
@@ -25,8 +25,10 @@
 #include "vtkCamera.h"
 #include "vtkCellData.h"
 #include "vtkCommand.h"
+#include "vtkConvertSelection.h"
 #include "vtkDataRepresentation.h"
 #include "vtkDynamic2DLabelMapper.h"
+#include "vtkEdgeListIterator.h"
 #include "vtkExtractSelectedGraph.h"
 #include "vtkGraphLayout.h"
 #include "vtkGraphToPolyData.h"
@@ -44,13 +46,18 @@
 #include "vtkRenderWindowInteractor.h"
 #include "vtkSelection.h"
 #include "vtkSelectionLink.h"
+#include "vtkSmartPointer.h"
 #include "vtkTextProperty.h"
 #include "vtkTreeLayoutStrategy.h"
 #include "vtkVertexGlyphFilter.h"
 #include "vtkViewTheme.h"
 #include "vtkVisibleCellSelector.h"
 
-vtkCxxRevisionMacro(vtkTreeLayoutView, "$Revision: 1.3 $");
+#include <vtksys/stl/set>
+
+using vtksys_stl::set;
+
+vtkCxxRevisionMacro(vtkTreeLayoutView, "$Revision: 1.12 $");
 vtkStandardNewMacro(vtkTreeLayoutView);
 //----------------------------------------------------------------------------
 vtkTreeLayoutView::vtkTreeLayoutView()
@@ -101,6 +108,7 @@ vtkTreeLayoutView::vtkTreeLayoutView()
   this->OutlineActor->PickableOff();
   this->OutlineActor->GetProperty()->SetPointSize(7);
   this->OutlineActor->SetPosition(0, 0, -0.001);
+  this->OutlineMapper->SetScalarVisibility(false);
   this->EdgeMapper->SetScalarModeToUseCellData();
   this->EdgeMapper->SetLookupTable(this->EdgeColorLUT);
   this->EdgeActor->SetPosition(0, 0, -0.003);
@@ -112,11 +120,13 @@ vtkTreeLayoutView::vtkTreeLayoutView()
   this->LabelMapper->GetLabelTextProperty()->SetItalic(0);
   this->LabelMapper->GetLabelTextProperty()->SetLineOffset(10);
   this->LabelActor->PickableOff();
-  this->SelectionVertexActor->GetProperty()->SetPointSize(5);
+  this->SelectionVertexActor->GetProperty()->SetPointSize(11);
   this->SelectionVertexActor->PickableOff();
-  this->SelectionVertexActor->SetPosition(0, 0, 0.001);
+  this->SelectionVertexActor->SetPosition(0, 0, -0.002);
+  this->SelectionVertexMapper->SetScalarVisibility(false);
   this->SelectionEdgeActor->PickableOff();
   this->SelectionEdgeActor->SetPosition(0, 0, -0.002);
+  this->SelectionEdgeMapper->SetScalarVisibility(false);
   
   // Set default parameters
   this->SetLabelArrayName("label");
@@ -146,9 +156,9 @@ vtkTreeLayoutView::vtkTreeLayoutView()
 
   this->KdTreeSelector->SetInputConnection(this->GraphLayout->GetOutputPort());
   this->ExtractSelectedGraph->SetInputConnection(0, this->GraphLayout->GetOutputPort());
-  vtkSelection* empty = vtkSelection::New();
-  empty->GetProperties()->Set(vtkSelection::CONTENT_TYPE(), vtkSelection::INDICES);
-  vtkIdTypeArray* arr = vtkIdTypeArray::New();
+  vtkSelection *empty = vtkSelection::New();
+  empty->SetContentType(vtkSelection::INDICES);
+  vtkIdTypeArray *arr = vtkIdTypeArray::New();
   empty->SetSelectionList(arr);
   arr->Delete();
   this->ExtractSelectedGraph->SetInput(1, empty);
@@ -195,7 +205,7 @@ vtkTreeLayoutView::~vtkTreeLayoutView()
 }
 
 //----------------------------------------------------------------------------
-void vtkTreeLayoutView::SetLabelArrayName(const char* name)
+void vtkTreeLayoutView::SetLabelArrayName(const char *name)
 {
   this->LabelMapper->SetFieldDataName(name);
 }
@@ -231,7 +241,7 @@ void vtkTreeLayoutView::LabelVisibilityOff()
 }
 
 //----------------------------------------------------------------------------
-void vtkTreeLayoutView::SetVertexColorArrayName(const char* name)
+void vtkTreeLayoutView::SetVertexColorArrayName(const char *name)
 {
   this->SetVertexColorArrayNameInternal(name);
   this->VertexMapper->SetScalarModeToUsePointFieldData();
@@ -269,7 +279,7 @@ void vtkTreeLayoutView::ColorVerticesOff()
 }
 
 //----------------------------------------------------------------------------
-void vtkTreeLayoutView::SetEdgeColorArrayName(const char* name)
+void vtkTreeLayoutView::SetEdgeColorArrayName(const char *name)
 {
   this->SetEdgeColorArrayNameInternal(name);
   this->EdgeMapper->SetScalarModeToUseCellFieldData();
@@ -373,24 +383,31 @@ const char* vtkTreeLayoutView::GetDistanceArrayName()
 }
 
 //----------------------------------------------------------------------------
-void vtkTreeLayoutView::SetDistanceArrayName(const char* name)
+void vtkTreeLayoutView::SetDistanceArrayName(const char *name)
 {
   this->TreeStrategy->SetDistanceArrayName(name);
 }
 
 //----------------------------------------------------------------------------
-void vtkTreeLayoutView::SetupRenderWindow(vtkRenderWindow* win)
+void vtkTreeLayoutView::SetupRenderWindow(vtkRenderWindow *win)
 {
   this->Superclass::SetupRenderWindow(win);
   win->GetInteractor()->SetInteractorStyle(this->InteractorStyle);
 }
 
 //----------------------------------------------------------------------------
-void vtkTreeLayoutView::AddInputConnection(vtkAlgorithmOutput* conn)
+void vtkTreeLayoutView::AddInputConnection( int port, int item,
+  vtkAlgorithmOutput *conn, vtkAlgorithmOutput *selectionConn)
 {
-  if (this->GraphLayout->GetNumberOfInputConnections(0) == 0)
+  if( port != 0 || item != 0 )
+    {
+    vtkErrorMacro("This view only supports one representation.");
+    return;
+    }
+  else if (this->GraphLayout->GetNumberOfInputConnections(0) == 0)
     {
     this->GraphLayout->SetInputConnection(conn);
+    this->ExtractSelectedGraph->SetInputConnection(1, selectionConn);
   
     this->Renderer->AddActor(this->VertexActor);
     this->Renderer->AddActor(this->OutlineActor);
@@ -408,12 +425,19 @@ void vtkTreeLayoutView::AddInputConnection(vtkAlgorithmOutput* conn)
 }
 
 //----------------------------------------------------------------------------
-void vtkTreeLayoutView::RemoveInputConnection(vtkAlgorithmOutput* conn)
+void vtkTreeLayoutView::RemoveInputConnection( int port, int item,
+  vtkAlgorithmOutput *conn, vtkAlgorithmOutput *selectionConn)
 {
+  if( port != 0 || item != 0 )
+    {
+    vtkErrorMacro("This view only supports one representation.");
+    }
+
   if (this->GraphLayout->GetNumberOfInputConnections(0) > 0 &&
       this->GraphLayout->GetInputConnection(0, 0) == conn)
     {
     this->GraphLayout->RemoveInputConnection(0, conn);
+    this->ExtractSelectedGraph->RemoveInputConnection(1, selectionConn);
   
     this->Renderer->RemoveActor(this->VertexActor);
     this->Renderer->RemoveActor(this->OutlineActor);
@@ -424,12 +448,6 @@ void vtkTreeLayoutView::RemoveInputConnection(vtkAlgorithmOutput* conn)
     }
 }
 
-//----------------------------------------------------------------------------
-void vtkTreeLayoutView::SetSelectionLink(vtkSelectionLink* link)
-{
-  this->ExtractSelectedGraph->SetInputConnection(1, link->GetOutputPort());
-}
-
 //----------------------------------------------------------------------------
 void vtkTreeLayoutView::MapToXYPlane(
   double displayX, double displayY, 
@@ -437,9 +455,9 @@ void vtkTreeLayoutView::MapToXYPlane(
 {
   this->Coordinate->SetViewport(this->Renderer);
   this->Coordinate->SetValue(displayX, displayY);
-  double* pt = this->Coordinate->GetComputedWorldValue(0);
+  double *pt = this->Coordinate->GetComputedWorldValue(0);
 
-  vtkCamera* camera = this->Renderer->GetActiveCamera();
+  vtkCamera *camera = this->Renderer->GetActiveCamera();
   double cameraPos[3];
   camera->GetPosition(cameraPos);
 
@@ -455,15 +473,15 @@ void vtkTreeLayoutView::MapToXYPlane(
 
 //----------------------------------------------------------------------------
 void vtkTreeLayoutView::ProcessEvents(
-  vtkObject* caller, 
+  vtkObject *caller, 
   unsigned long eventId, 
-  void* callData)
+  void *callData)
 {
   if (caller == this->InteractorStyle && eventId == vtkCommand::SelectionChangedEvent
       && this->GraphLayout->GetNumberOfInputConnections(0) > 0)
     {
     // Create the selection
-    unsigned int* rect = reinterpret_cast<unsigned int*>(callData);
+    unsigned int *rect = reinterpret_cast<unsigned int*>(callData);
     bool singleSelectMode = false;
     unsigned int pos1X = rect[0];
     unsigned int pos1Y = rect[1];
@@ -494,12 +512,17 @@ void vtkTreeLayoutView::ProcessEvents(
     double dist2 = radiusX*radiusX + radiusY*radiusY;
     this->KdTreeSelector->SetSingleSelectionThreshold(dist2);
     this->KdTreeSelector->Update();
-    vtkSelection* selection = this->KdTreeSelector->GetOutput();
-    selection->Register(0);
+    vtkSelection *kdSelection = this->KdTreeSelector->GetOutput();
+    this->GraphLayout->Update();
+    vtkDataObject *data = this->GraphLayout->GetOutput();
+    vtkSmartPointer<vtkSelection> selection;
+    selection.TakeReference(vtkConvertSelection::ToSelectionType(
+      kdSelection, data, this->SelectionType, this->SelectionArrayNames));
     
+#if 0
     // If the selection is empty, do a visible cell selection
     // to attempt to pick up an edge.
-    vtkAbstractArray* list = selection->GetSelectionList();
+    vtkAbstractArray *list = selection->GetSelectionList();
     if (list->GetNumberOfTuples() == 0)
       {
       // The edge actor must be opaque for visible cell selector.
@@ -516,35 +539,57 @@ void vtkTreeLayoutView::ProcessEvents(
       this->VisibleCellSelector->SetProcessorId(0);
       this->VisibleCellSelector->SetRenderPasses(0, 0, 0, 0, 1);
       this->VisibleCellSelector->Select();  
-      vtkIdTypeArray* ids = vtkIdTypeArray::New();
+      vtkSmartPointer<vtkIdTypeArray> ids = 
+        vtkSmartPointer<vtkIdTypeArray>::New();
       this->VisibleCellSelector->GetSelectedIds(ids);
       
       // Set the opacity back to the original value.
       this->EdgeActor->GetProperty()->SetOpacity(opacity);
       
-      vtkIdTypeArray* selectedIds = vtkIdTypeArray::New();
-      vtkAbstractGraph* graph = this->GraphLayout->GetOutput();
+      vtkGraph *graph = this->GraphLayout->GetOutput();
+
+      // Extract edge ids from the list given by the visible cell selector.
+      vtkSmartPointer<vtkIdTypeArray> selectedEdges = 
+        vtkSmartPointer<vtkIdTypeArray>::New();
       for (vtkIdType i = 0; i < ids->GetNumberOfTuples(); i++)
         {
-        vtkIdType edge = ids->GetValue(4*i+3);
-        vtkIdType source = graph->GetSourceVertex(edge);
-        vtkIdType target = graph->GetTargetVertex(edge);
-        selectedIds->InsertNextValue(source);
-        selectedIds->InsertNextValue(target);
-        if (singleSelectMode)
+        selectedEdges->InsertNextValue(ids->GetValue(4*i+3));
+        }
+
+      // Convert edge ids to vertex ids.
+      set<vtkIdType> selectedVertexSet;
+      vtkSmartPointer<vtkEdgeListIterator> edges = 
+        vtkSmartPointer<vtkEdgeListIterator>::New();
+      graph->GetEdges(edges);
+      while (edges->HasNext())
+        {
+        vtkEdgeType e = edges->Next();
+        if (selectedEdges->LookupValue(e.Id) >= 0)
           {
-          break;
+          selectedVertexSet.insert(e.Source);
+          if (singleSelectMode)
+            {
+            break;
+            }
           }
         }
+
+      vtkSmartPointer<vtkIdTypeArray> selectedVertices = 
+        vtkSmartPointer<vtkIdTypeArray>::New();
+      vtksys_stl::set<vtkIdType>::iterator it = selectedVertexSet.begin();
+      vtksys_stl::set<vtkIdType>::iterator itEnd = selectedVertexSet.end();
+      for (; it != itEnd; ++it)
+        {
+        selectedVertices->InsertNextValue(*it);
+        }
   
       selection->Delete();
       selection = vtkSelection::New();
       selection->GetProperties()->Set(vtkSelection::CONTENT_TYPE(), vtkSelection::INDICES);
       selection->GetProperties()->Set(vtkSelection::FIELD_TYPE(), vtkSelection::POINT);
-      selection->SetSelectionList(selectedIds);
-      ids->Delete();
-      selectedIds->Delete();      
+      selection->SetSelectionList(selectedVertices);
       }
+#endif
     
     // If this is a union selection, append the selection
     if (rect[4] == vtkInteractorStyleRubberBand2D::SELECT_UNION)
@@ -555,12 +600,6 @@ void vtkTreeLayoutView::ProcessEvents(
     
     // Call select on the representation
     this->GetRepresentation()->Select(this, selection);
-    
-    selection->Delete();
-    }
-  else if(eventId == vtkCommand::SelectionChangedEvent)
-    {
-    this->Update();
     }
   else
     {
@@ -581,17 +620,12 @@ void vtkTreeLayoutView::PrepareForRendering()
   vtkAlgorithmOutput* conn = rep->GetInputConnection();
   if (this->GraphLayout->GetInputConnection(0, 0) != conn)
     {
-    this->RemoveInputConnection(this->GraphLayout->GetInputConnection(0, 0));
-    this->AddInputConnection(conn);
+      this->RemoveInputConnection( 0, 0,
+      this->GraphLayout->GetInputConnection(0, 0),
+      this->ExtractSelectedGraph->GetInputConnection(1, 0));
+    this->AddInputConnection(0, 0, conn, rep->GetSelectionConnection());
     }
 
-  // Make sure the selection link is up to date.
-  vtkSelectionLink* link = rep->GetSelectionLink();
-  if (this->ExtractSelectedGraph->GetInputConnection(1, 0)->GetProducer() != link)
-    {
-    this->SetSelectionLink(link);
-    }
-  
   // Update the pipeline up until the graph to polydata
   this->GraphToPolyData->Update();
   vtkPolyData* pd = this->GraphToPolyData->GetOutput();
