@@ -33,15 +33,15 @@ PURPOSE.  See the above copyright notice for more information.
 #include "vtkUnstructuredGridPartialPreIntegration.h"
 #include "vtkVolumeProperty.h"
 
-extern const char *vtkHAVSVolumeMapper_kbufferVP;
-extern const char *vtkHAVSVolumeMapper_k2BeginFP;
-extern const char *vtkHAVSVolumeMapper_k2FP;
-extern const char *vtkHAVSVolumeMapper_k2EndFP;
-extern const char *vtkHAVSVolumeMapper_k6BeginFP;
-extern const char *vtkHAVSVolumeMapper_k6FP;
-extern const char *vtkHAVSVolumeMapper_k6EndFP;
+#include "vtkHAVSVolumeMapper_kbufferVP.h"
+#include "vtkHAVSVolumeMapper_k2BeginFP.h"
+#include "vtkHAVSVolumeMapper_k2FP.h"
+#include "vtkHAVSVolumeMapper_k2EndFP.h"
+#include "vtkHAVSVolumeMapper_k6BeginFP.h"
+#include "vtkHAVSVolumeMapper_k6FP.h"
+#include "vtkHAVSVolumeMapper_k6EndFP.h"
 
-vtkCxxRevisionMacro(vtkOpenGLHAVSVolumeMapper, "$Revision: 1.11 $");
+vtkCxxRevisionMacro(vtkOpenGLHAVSVolumeMapper, "$Revision: 1.13 $");
 vtkStandardNewMacro(vtkOpenGLHAVSVolumeMapper);
 
 //----------------------------------------------------------------------------
@@ -319,7 +319,7 @@ void vtkOpenGLHAVSVolumeMapper::InitializeShaders()
   vtkgl::BindProgramARB(vtkgl::VERTEX_PROGRAM_ARB, this->VertexProgram);
   vtkgl::ProgramStringARB(vtkgl::VERTEX_PROGRAM_ARB,
                           vtkgl::PROGRAM_FORMAT_ASCII_ARB,
-                          strlen(vtkHAVSVolumeMapper_kbufferVP),
+                          static_cast<GLsizei>(strlen(vtkHAVSVolumeMapper_kbufferVP)),
                           vtkHAVSVolumeMapper_kbufferVP);
   
   // Create fragment shaders
@@ -332,20 +332,20 @@ void vtkOpenGLHAVSVolumeMapper::InitializeShaders()
                           this->FragmentProgramBegin);
     vtkgl::ProgramStringARB(vtkgl::FRAGMENT_PROGRAM_ARB,
                             vtkgl::PROGRAM_FORMAT_ASCII_ARB,
-                            strlen(vtkHAVSVolumeMapper_k2BeginFP),
+                            static_cast<GLsizei>(strlen(vtkHAVSVolumeMapper_k2BeginFP)),
                             vtkHAVSVolumeMapper_k2BeginFP);
     vtkgl::GenProgramsARB(1, reinterpret_cast<GLuint *>(&this->FragmentProgram));
     vtkgl::BindProgramARB(vtkgl::FRAGMENT_PROGRAM_ARB, this->FragmentProgram);
     vtkgl::ProgramStringARB(vtkgl::FRAGMENT_PROGRAM_ARB,
                             vtkgl::PROGRAM_FORMAT_ASCII_ARB,
-                            strlen(vtkHAVSVolumeMapper_k2FP),
+                            static_cast<GLsizei>(strlen(vtkHAVSVolumeMapper_k2FP)),
                             vtkHAVSVolumeMapper_k2FP);
     vtkgl::GenProgramsARB(1, reinterpret_cast<GLuint *>(&this->FragmentProgramEnd));
     vtkgl::BindProgramARB(vtkgl::FRAGMENT_PROGRAM_ARB,
                           this->FragmentProgramEnd);
     vtkgl::ProgramStringARB(vtkgl::FRAGMENT_PROGRAM_ARB,
                             vtkgl::PROGRAM_FORMAT_ASCII_ARB,
-                            strlen(vtkHAVSVolumeMapper_k2EndFP),
+                            static_cast<GLsizei>(strlen(vtkHAVSVolumeMapper_k2EndFP)),
                             vtkHAVSVolumeMapper_k2EndFP);
     }
   else
@@ -356,20 +356,20 @@ void vtkOpenGLHAVSVolumeMapper::InitializeShaders()
                           this->FragmentProgramBegin);
     vtkgl::ProgramStringARB(vtkgl::FRAGMENT_PROGRAM_ARB,
                             vtkgl::PROGRAM_FORMAT_ASCII_ARB,
-                            strlen(vtkHAVSVolumeMapper_k6BeginFP),
+                            static_cast<GLsizei>(strlen(vtkHAVSVolumeMapper_k6BeginFP)),
                             vtkHAVSVolumeMapper_k6BeginFP);
     vtkgl::GenProgramsARB(1, reinterpret_cast<GLuint *>(&this->FragmentProgram));
     vtkgl::BindProgramARB(vtkgl::FRAGMENT_PROGRAM_ARB, this->FragmentProgram);
     vtkgl::ProgramStringARB(vtkgl::FRAGMENT_PROGRAM_ARB,
                             vtkgl::PROGRAM_FORMAT_ASCII_ARB,
-                            strlen(vtkHAVSVolumeMapper_k6FP),
+                            static_cast<GLsizei>(strlen(vtkHAVSVolumeMapper_k6FP)),
                             vtkHAVSVolumeMapper_k6FP);
     vtkgl::GenProgramsARB(1, reinterpret_cast<GLuint *>(&this->FragmentProgramEnd));
     vtkgl::BindProgramARB(vtkgl::FRAGMENT_PROGRAM_ARB,
                           this->FragmentProgramEnd);
     vtkgl::ProgramStringARB(vtkgl::FRAGMENT_PROGRAM_ARB,
                             vtkgl::PROGRAM_FORMAT_ASCII_ARB,
-                            strlen(vtkHAVSVolumeMapper_k6EndFP),
+                            static_cast<GLsizei>(strlen(vtkHAVSVolumeMapper_k6EndFP)),
                             vtkHAVSVolumeMapper_k6EndFP);
     }
 
