@@ -1,3 +1,18 @@
+/*=========================================================================
+
+  Program:   Visualization Toolkit
+  Module:    $RCSfile: TerrainPolylineEditor.cxx,v $
+
+  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+  All rights reserved.
+  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
+
+     This software is distributed WITHOUT ANY WARRANTY; without even
+     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
+     PURPOSE.  See the above copyright notice for more information.
+
+=========================================================================*/
+
 #include "vtkDEMReader.h"
 #include "vtkPolyDataReader.h"
 #include "vtkPolyData.h"
@@ -834,6 +849,8 @@ int TerrainPolylineEditor(int argc, char * argv[])
 
   recorder->Play();
   
+  recorder->Off();
+
   int retVal = vtkRegressionTestImage( renWin );
   if ( retVal == vtkRegressionTester::DO_INTERACTOR)
     {
@@ -849,7 +866,6 @@ int TerrainPolylineEditor(int argc, char * argv[])
   iren->Delete();
   renWin->Delete();
   ren1->Delete();
-  recorder->Off();
   recorder->Delete();
   
   return EXIT_SUCCESS;
