@@ -36,7 +36,7 @@
 #include "vtkWindow.h"
 
 
-vtkCxxRevisionMacro(vtkAffineRepresentation2D, "$Revision: 1.6 $");
+vtkCxxRevisionMacro(vtkAffineRepresentation2D, "$Revision: 1.8 $");
 vtkStandardNewMacro(vtkAffineRepresentation2D);
 
 vtkCxxSetObjectMacro(vtkAffineRepresentation2D,Property,vtkProperty2D);
@@ -491,9 +491,9 @@ void vtkAffineRepresentation2D::WidgetInteraction(double eventPos[2])
 }
 
 //----------------------------------------------------------------------
-void vtkAffineRepresentation2D::EndWidgetInteraction(double*)
+void vtkAffineRepresentation2D::EndWidgetInteraction(double vtkNotUsed(eventPos) [2])
 {
-  // Have to play games here because of the "pipelined" nature of the 
+  // Have to play games here because of the "pipelined" nature of the
   // transformations.
   this->GetTransform(this->TempTransform);
   this->TotalTransform->SetMatrix(this->TempTransform->GetMatrix());
@@ -1065,6 +1065,8 @@ void vtkAffineRepresentation2D::PrintSelf(ostream& os, vtkIndent indent)
   
   os << indent << "Display Text: " << (this->DisplayText ? "On\n" : "Off\n");
   
+  os << indent << "Origin: (" << this->Origin[0] << "," 
+     << this->Origin[1] << "," << this->Origin[2] << ")\n";
   os << indent << "Box Width: " << this->BoxWidth << "\n";
   os << indent << "Circle Width: " << this->CircleWidth << "\n";
   os << indent << "Axes Width: " << this->AxesWidth << "\n";
