@@ -34,7 +34,7 @@
 
 #include <vtksys/ios/sstream>
 
-vtkCxxRevisionMacro(vtkBiDimensionalRepresentation2D, "$Revision: 1.22 $");
+vtkCxxRevisionMacro(vtkBiDimensionalRepresentation2D, "$Revision: 1.26 $");
 vtkStandardNewMacro(vtkBiDimensionalRepresentation2D);
 
 
@@ -95,7 +95,7 @@ vtkBiDimensionalRepresentation2D::vtkBiDimensionalRepresentation2D()
   this->LabelFormat = new char[6];
   sprintf(this->LabelFormat,"%s","%0.3g");
 
-  this->ID = 0;
+  this->ID = VTK_UNSIGNED_LONG_MAX;
   this->IDInitialized = 0;
 
   this->ShowLabelAboveWidget = 1;
@@ -884,44 +884,44 @@ void vtkBiDimensionalRepresentation2D::BuildRepresentation()
     int maxX = VTK_INT_MIN, maxY = VTK_INT_MIN;
     if (p1[1] > maxY)
       {
-      maxX = (int)p1[0];
-      maxY = (int)p1[1];
+      maxX = static_cast<int>(p1[0]);
+      maxY = static_cast<int>(p1[1]);
       }
     if (p2[1] > maxY)
       {
-      maxX = (int)p2[0];
-      maxY = (int)p2[1];
+      maxX = static_cast<int>(p2[0]);
+      maxY = static_cast<int>(p2[1]);
       }
     if (p3[1] > maxY)
       {
-      maxX = (int)p3[0];
-      maxY = (int)p3[1];
+      maxX = static_cast<int>(p3[0]);
+      maxY = static_cast<int>(p3[1]);
       }
     if (p4[1] > maxY)
       {
-      maxX = (int)p4[0];
-      maxY = (int)p4[1];
+      maxX = static_cast<int>(p4[0]);
+      maxY = static_cast<int>(p4[1]);
       }
     int minX = VTK_INT_MAX, minY = VTK_INT_MAX;
     if (p1[1] < minY)
       {
-      minX = (int)p1[0];
-      minY = (int)p1[1];
+      minX = static_cast<int>(p1[0]);
+      minY = static_cast<int>(p1[1]);
       }
     if (p2[1] < minY)
       {
-      minX = (int)p2[0];
-      minY = (int)p2[1];
+      minX = static_cast<int>(p2[0]);
+      minY = static_cast<int>(p2[1]);
       }
     if (p3[1] < minY)
       {
-      minX = (int)p3[0];
-      minY = (int)p3[1];
+      minX = static_cast<int>(p3[0]);
+      minY = static_cast<int>(p3[1]);
       }
     if (p4[1] < minY)
       {
-      minX = (int)p4[0];
-      minY = (int)p4[1];
+      minX = static_cast<int>(p4[0]);
+      minY = static_cast<int>(p4[1]);
       }
     int textSize[2];
     this->TextMapper->GetSize(this->Renderer, textSize);
@@ -956,6 +956,22 @@ void vtkBiDimensionalRepresentation2D::GetLabelPosition(double pos[3])
   this->TextActor->GetPositionCoordinate()->GetValue(pos);
 }
 
+//----------------------------------------------------------------------
+void vtkBiDimensionalRepresentation2D::GetWorldLabelPosition(double pos[3])
+{
+  double viewportPos[3], worldPos[4];
+  this->TextActor->GetPositionCoordinate()->GetValue(viewportPos);
+  this->Renderer->ViewportToNormalizedViewport(viewportPos[0], viewportPos[1]);
+  this->Renderer->NormalizedViewportToView(viewportPos[0], viewportPos[1], viewportPos[2]);
+  this->Renderer->SetViewPoint(viewportPos);
+  this->Renderer->ViewToWorld();
+  this->Renderer->GetWorldPoint(worldPos);
+
+  pos[0] = worldPos[0]/worldPos[3];
+  pos[1] = worldPos[1]/worldPos[3];
+  pos[2] = worldPos[2]/worldPos[3];
+}
+
 //----------------------------------------------------------------------
 double vtkBiDimensionalRepresentation2D::GetLength1()
 {
@@ -1018,6 +1034,11 @@ void vtkBiDimensionalRepresentation2D::Highlight(int highlightOn)
 //----------------------------------------------------------------------
 void vtkBiDimensionalRepresentation2D::SetID(unsigned long id)
 {
+  if (id == this->ID)
+    {
+    return;
+    }
+
   this->ID = id;
   this->IDInitialized = 1;
   this->Modified();
@@ -1068,5 +1089,32 @@ void vtkBiDimensionalRepresentation2D::PrintSelf(ostream& os, vtkIndent indent)
     }
 
   os << indent << "Handle Representation: " << this->HandleRepresentation << "\n";
+  
+  os << indent << "ID: " << this->ID << "\n";
+
+  os << indent << "Label Format: ";
+  if ( this->LabelFormat )
+    {
+    os << this->LabelFormat << "\n";
+    }
+  else
+    {
+    os << "(null))\n";
+    }
+  
+  os << indent << "Point1 Representation\n";
+  this->Point1Representation->PrintSelf(os,indent.GetNextIndent());
+
+  os << indent << "Point2 Representation\n";
+  this->Point2Representation->PrintSelf(os,indent.GetNextIndent());
+
+  os << indent << "Point3 Representation\n";
+  this->Point3Representation->PrintSelf(os,indent.GetNextIndent());
+
+  os << indent << "Point4 Representation\n";
+  this->Point4Representation->PrintSelf(os,indent.GetNextIndent());
+  
+  os << indent << "Show Label Above Widget: "
+     << (this->ShowLabelAboveWidget ? "On\n" : "Off\n");
 }
 
