@@ -40,7 +40,7 @@
 #include "vtkTexture.h"
 #include "vtkTransform.h"
 
-vtkCxxRevisionMacro(vtkImagePlaneWidget, "$Revision: 1.11 $");
+vtkCxxRevisionMacro(vtkImagePlaneWidget, "$Revision: 1.18 $");
 vtkStandardNewMacro(vtkImagePlaneWidget);
 
 vtkCxxSetObjectMacro(vtkImagePlaneWidget, PlaneProperty, vtkProperty);
@@ -135,12 +135,11 @@ vtkImagePlaneWidget::vtkImagePlaneWidget() : vtkPolyDataSourceWidget()
 
   // Manage the picking stuff
   //
-  this->PlanePicker = vtkCellPicker::New();
-  this->PlanePicker->SetTolerance(0.005); //need some fluff
-  this->PlanePicker->AddPickList(this->TexturePlaneActor);
-  this->PlanePicker->PickFromListOn();
-  this->PlanePicker->Register(this);
-  this->PlanePicker->Delete();
+  this->PlanePicker = NULL;
+  vtkCellPicker* picker = vtkCellPicker::New();
+  picker->SetTolerance(0.005); //need some fluff
+  this->SetPicker(picker);
+  picker->Delete();
 
   // Set up the initial properties
   //
@@ -319,10 +318,7 @@ void vtkImagePlaneWidget::SetEnabled(int enabling)
     // Add the image data annotation
     this->CurrentRenderer->AddViewProp(this->TextActor);
 
-    if ( this->PlanePicker )
-      {
-      this->TexturePlaneActor->PickableOn();
-      }
+    this->TexturePlaneActor->PickableOn();
 
     this->InvokeEvent(vtkCommand::EnableEvent,0);
 
@@ -357,10 +353,7 @@ void vtkImagePlaneWidget::SetEnabled(int enabling)
     //turn off the image data annotation
     this->CurrentRenderer->RemoveViewProp(this->TextActor);
 
-    if ( this->PlanePicker )
-      {
-      this->TexturePlaneActor->PickableOff();
-      }
+    this->TexturePlaneActor->PickableOff();
 
     this->InvokeEvent(vtkCommand::DisableEvent,0);
     this->SetCurrentRenderer(NULL);
@@ -410,28 +403,60 @@ void vtkImagePlaneWidget::ProcessEvents(vtkObject* vtkNotUsed(object),
     case vtkCommand::MouseMoveEvent:
       self->OnMouseMove();
       break;
+    case vtkCommand::CharEvent:
+      self->OnChar();
+      break;
     }
 }
 
+//----------------------------------------------------------------------------
+void vtkImagePlaneWidget::OnChar()
+{
+  vtkRenderWindowInteractor *i = this->Interactor;
+
+  if ( i->GetKeyCode() == 'r' || i->GetKeyCode() == 'R' )
+    {
+    if ( i->GetShiftKey() || i->GetControlKey() )
+      {
+      this->SetWindowLevel( this->OriginalWindow, this->OriginalLevel );
+      double wl[2] = { this->CurrentWindow, this->CurrentLevel };
+
+      this->EventCallbackCommand->SetAbortFlag(1);
+      this->InvokeEvent(vtkCommand::ResetWindowLevelEvent, wl);
+      }
+    else
+      {
+      this->Interactor->GetInteractorStyle()->OnChar();
+      }
+    }
+  else
+    {
+    this->Interactor->GetInteractorStyle()->OnChar();
+    }
+}
+
+//----------------------------------------------------------------------------
 void vtkImagePlaneWidget::AddObservers(void)
 {
-    // listen for the following events
-    vtkRenderWindowInteractor *i = this->Interactor;
-    if (i)
+  // listen for the following events
+  vtkRenderWindowInteractor *i = this->Interactor;
+  if (i)
     {
-        i->AddObserver(vtkCommand::MouseMoveEvent, this->EventCallbackCommand,
+    i->AddObserver(vtkCommand::MouseMoveEvent, this->EventCallbackCommand,
                        this->Priority);
-        i->AddObserver(vtkCommand::LeftButtonPressEvent,
+    i->AddObserver(vtkCommand::LeftButtonPressEvent,
                        this->EventCallbackCommand, this->Priority);
-        i->AddObserver(vtkCommand::LeftButtonReleaseEvent,
+    i->AddObserver(vtkCommand::LeftButtonReleaseEvent,
                        this->EventCallbackCommand, this->Priority);
-        i->AddObserver(vtkCommand::MiddleButtonPressEvent,
+    i->AddObserver(vtkCommand::MiddleButtonPressEvent,
                        this->EventCallbackCommand, this->Priority);
-        i->AddObserver(vtkCommand::MiddleButtonReleaseEvent,
+    i->AddObserver(vtkCommand::MiddleButtonReleaseEvent,
                        this->EventCallbackCommand, this->Priority);
-        i->AddObserver(vtkCommand::RightButtonPressEvent,
+    i->AddObserver(vtkCommand::RightButtonPressEvent,
                        this->EventCallbackCommand, this->Priority);
-        i->AddObserver(vtkCommand::RightButtonReleaseEvent,
+    i->AddObserver(vtkCommand::RightButtonReleaseEvent,
+                       this->EventCallbackCommand, this->Priority);
+    i->AddObserver(vtkCommand::CharEvent,
                        this->EventCallbackCommand, this->Priority);
     }
 }
@@ -924,6 +949,9 @@ void vtkImagePlaneWidget::StartWindowLevel()
       }
     }
 
+  this->InitialWindow = this->CurrentWindow;
+  this->InitialLevel = this->CurrentLevel;
+
   if( ! found || path == 0 )
     {
     this->State = vtkImagePlaneWidget::Outside;
@@ -936,13 +964,17 @@ void vtkImagePlaneWidget::StartWindowLevel()
     this->State = vtkImagePlaneWidget::WindowLevelling;
     this->HighlightPlane(1);
     this->ActivateText(1);
-    this->WindowLevel(X,Y);
+    this->StartWindowLevelPositionX = X;
+    this->StartWindowLevelPositionY = Y;
     this->ManageTextDisplay();
     }
 
   this->EventCallbackCommand->SetAbortFlag(1);
   this->StartInteraction();
-  this->InvokeEvent(vtkCommand::StartInteractionEvent,0);
+
+  double wl[2] = { this->CurrentWindow, this->CurrentLevel };
+  this->InvokeEvent(vtkCommand::StartWindowLevelEvent,wl);
+
   this->Interactor->Render();
 }
 
@@ -961,7 +993,10 @@ void vtkImagePlaneWidget::StopWindowLevel()
 
   this->EventCallbackCommand->SetAbortFlag(1);
   this->EndInteraction();
-  this->InvokeEvent(vtkCommand::EndInteractionEvent,0);
+
+  double wl[2] = { this->CurrentWindow, this->CurrentLevel };
+  this->InvokeEvent(vtkCommand::EndWindowLevelEvent,wl);
+
   this->Interactor->Render();
 }
 
@@ -1055,7 +1090,16 @@ void vtkImagePlaneWidget::OnMouseMove()
   // Interact, if desired
   //
   this->EventCallbackCommand->SetAbortFlag(1);
-  this->InvokeEvent(vtkCommand::InteractionEvent,0);
+
+  if ( this->State == vtkImagePlaneWidget::WindowLevelling )
+    {
+    double wl[2] = { this->CurrentWindow, this->CurrentLevel };
+    this->InvokeEvent(vtkCommand::WindowLevelEvent,wl);
+    }
+  else
+    {
+    this->InvokeEvent(vtkCommand::InteractionEvent,0);
+    }
 
   this->Interactor->Render();
 }
@@ -1063,48 +1107,131 @@ void vtkImagePlaneWidget::OnMouseMove()
 //----------------------------------------------------------------------------
 void vtkImagePlaneWidget::WindowLevel(int X, int Y)
 {
-  double range[2];
-  this->LookupTable->GetTableRange(range);
-  double window = range[1] - range[0];
-  double level = 0.5*(range[0] + range[1]);
+  int *size = this->CurrentRenderer->GetSize();
+  double window = this->InitialWindow;
+  double level = this->InitialLevel;
 
-  double owin = this->OriginalWindow;
+  // Compute normalized delta
 
-  level = level + (X - this->Interactor->GetLastEventPosition()[0])*owin/500.0;
-  window = 
-    window + (this->Interactor->GetLastEventPosition()[1] - Y)*owin/250.0;
+  double dx = 4.0 * ( X - this->StartWindowLevelPositionX ) / size[0];
+  double dy = 4.0 *( this->StartWindowLevelPositionY - Y ) / size[1];
 
-  if ( window == 0.0 )
+  // Scale by current values
+
+  if ( fabs( window ) > 0.01 )
     {
-    window = 0.001;
+    dx = dx * window;
+    }
+  else
+    {
+    dx = dx * ( window < 0 ? -0.01 : 0.01 );
+    }
+  if ( fabs( level ) > 0.01 )
+    {
+    dy = dy * level;
+    }
+  else
+    {
+    dy = dy * ( level < 0 ? -0.01 : 0.01 );
     }
 
-  double rmin = level - window*0.5;
-  double rmax = level + window*0.5;
+  // Abs so that direction does not flip
 
-  if( rmin < rmax )
+  if ( window < 0.0 )
     {
-    this->CurrentWindow = window;
-    this->CurrentLevel = level;
-    if (!this->UserControlledLookupTable)
+    dx = -1 * dx;
+    }
+  if ( level < 0.0 )
+    {
+    dy = -1 * dy;
+    }
+
+  // Compute new window level
+
+  double newWindow = dx + window;
+  double newLevel = level - dy;
+
+  if ( fabs( newWindow ) < 0.01 )
+    {
+    newWindow = 0.01 * ( newWindow < 0 ? -1 : 1 );
+    }
+  if ( fabs( newLevel ) < 0.01 )
+    {
+    newLevel = 0.01 * ( newLevel < 0 ? -1 : 1 );
+    }
+  
+  if ( !this->UserControlledLookupTable )
+    {
+    if (( newWindow < 0 && this->CurrentWindow > 0 ) || \
+        ( newWindow > 0 && this->CurrentWindow < 0 ))
       {
-      this->LookupTable->SetTableRange(rmin,rmax);
+      this->InvertTable();
       }
+
+    double rmin = newLevel - 0.5*fabs( newWindow );
+    double rmax = rmin + fabs( newWindow );
+    this->LookupTable->SetTableRange( rmin, rmax );
     }
+
+  this->CurrentWindow = newWindow;
+  this->CurrentLevel = newLevel;
 }
 
 //----------------------------------------------------------------------------
-void vtkImagePlaneWidget::SetWindowLevel(double window, double level)
+void vtkImagePlaneWidget::InvertTable()
 {
-  double rmin = level - window*0.5;
-  double rmax = level + window*0.5;
-  this->CurrentWindow = window;
-  this->OriginalWindow  = window;
-  this->CurrentLevel = level;
-  this->OriginalLevel  = level;
-  this->LookupTable->SetTableRange(rmin,rmax);
+  int index = this->LookupTable->GetNumberOfTableValues();
+  unsigned char swap[4];
+  size_t num = 4*sizeof(unsigned char);
+  vtkUnsignedCharArray* table = this->LookupTable->GetTable();
+  for ( int count = 0; count < --index; count++ )
+    {
+    unsigned char *rgba1 = table->GetPointer(4*count);
+    unsigned char *rgba2 = table->GetPointer(4*index);
+    memcpy( swap,  rgba1, num );
+    memcpy( rgba1, rgba2, num );
+    memcpy( rgba2, swap,  num );
+    }
 
-  if(this->Enabled)
+  // force the lookuptable to update its InsertTime to avoid
+  // rebuilding the array
+  this->LookupTable->SetTableValue( 0, this->LookupTable->GetTableValue( 0 ) );
+}
+
+//----------------------------------------------------------------------------
+void vtkImagePlaneWidget::SetWindowLevel(double window, double level, int copy)
+{
+  if ( copy )
+    {
+    this->CurrentWindow = window;
+    this->CurrentLevel = level;
+    return;
+    }
+    
+  if ( this->CurrentWindow == window && this->CurrentLevel == level )
+    {
+    return;
+    }
+
+  // if the new window is negative and the old window was positive invert table
+  if ( (( window < 0 && this->CurrentWindow > 0 ) ||
+        ( window > 0 && this->CurrentWindow < 0 )) &&
+        !this->UserControlledLookupTable )
+    {
+    this->InvertTable();
+    }
+
+  this->CurrentWindow = window;
+  this->CurrentLevel = level;
+
+  if ( !this->UserControlledLookupTable )
+    {
+    double rmin = this->CurrentLevel - 0.5*fabs( this->CurrentWindow );
+    double rmax = rmin + fabs( this->CurrentWindow );
+    this->LookupTable->SetTableRange( rmin, rmax );
+    }
+
+  if ( this->Enabled )
     {
     this->Interactor->Render();
     }
@@ -1113,10 +1240,8 @@ void vtkImagePlaneWidget::SetWindowLevel(double window, double level)
 //----------------------------------------------------------------------------
 void vtkImagePlaneWidget::GetWindowLevel(double wl[2])
 {
-  double range[2];
-  this->LookupTable->GetTableRange(range);
-  wl[0] = range[1] - range[0];
-  wl[1] = 0.5*(range[0]+range[1]);
+  wl[0] = this->CurrentWindow;
+  wl[1] = this->CurrentLevel;
 }
 
 //----------------------------------------------------------------------------
@@ -1374,6 +1499,17 @@ void vtkImagePlaneWidget::SetInput(vtkDataSet* input)
 
   this->OriginalWindow = range[1] - range[0];
   this->OriginalLevel = 0.5*(range[0] + range[1]);
+  
+  if( fabs( this->OriginalWindow ) < 0.001 )
+    {
+    this->OriginalWindow = 0.001 * ( this->OriginalWindow < 0.0 ? -1 : 1 );
+    }
+  if( fabs( this->OriginalLevel ) < 0.001 )
+   {
+   this->OriginalLevel = 0.001 * ( this->OriginalLevel < 0.0 ? -1 : 1 );
+   }
+
+  this->SetWindowLevel(this->OriginalWindow,this->OriginalLevel);
 
   this->Reslice->SetInput(this->ImageData);
   int interpolate = this->ResliceInterpolate;
@@ -1610,23 +1746,34 @@ void vtkImagePlaneWidget::SetResliceInterpolate(int i)
 }
 
 //----------------------------------------------------------------------------
-void vtkImagePlaneWidget::SetPicker(vtkCellPicker* picker)
+void vtkImagePlaneWidget::SetPicker(vtkAbstractPropPicker* picker)
 {
+  // we have to have a picker for slice motion, window level and cursor to work
   if (this->PlanePicker != picker)
     {
     // to avoid destructor recursion
-    vtkCellPicker *temp = this->PlanePicker;
+    vtkAbstractPropPicker *temp = this->PlanePicker;
     this->PlanePicker = picker;
     if (temp != 0)
       {
       temp->UnRegister(this);
       }
-    if (this->PlanePicker != 0)
+
+    int delPicker = 0;
+    if (this->PlanePicker == 0)
       {
-      this->PlanePicker->Register(this);
-      this->PlanePicker->SetTolerance(0.005); //need some fluff
-      this->PlanePicker->AddPickList(this->TexturePlaneActor);
-      this->PlanePicker->PickFromListOn();
+      this->PlanePicker = vtkCellPicker::New();
+      vtkCellPicker::SafeDownCast(this->PlanePicker)->SetTolerance(0.005);
+      delPicker = 1;
+      }
+
+    this->PlanePicker->Register(this);
+    this->PlanePicker->AddPickList(this->TexturePlaneActor);
+    this->PlanePicker->PickFromListOn();
+
+    if ( delPicker )
+      {
+      this->PlanePicker->Delete();
       }
     }
 }
@@ -1681,6 +1828,17 @@ void vtkImagePlaneWidget::SetLookupTable(vtkLookupTable* table)
 
     this->OriginalWindow = range[1] - range[0];
     this->OriginalLevel = 0.5*(range[0] + range[1]);
+
+    if( fabs( this->OriginalWindow ) < 0.001 )
+      {
+      this->OriginalWindow = 0.001 * ( this->OriginalWindow < 0.0 ? -1 : 1 );
+      }
+    if( fabs( this->OriginalLevel ) < 0.001 )
+      {
+      this->OriginalLevel = 0.001 * ( this->OriginalLevel < 0.0 ? -1 : 1 );
+      }
+
+    this->SetWindowLevel(this->OriginalWindow,this->OriginalLevel);
     }
 }
 
@@ -2651,7 +2809,7 @@ void vtkImagePlaneWidget::GenerateText()
 {
   sprintf(this->TextBuff,"NA");
   this->TextActor->SetInput(this->TextBuff);
-  this->TextActor->ScaledTextOff();
+  this->TextActor->SetTextScaleModeToNone();
 
   vtkTextProperty* textprop = this->TextActor->GetTextProperty();
   textprop->SetColor(1,1,1);
