@@ -32,7 +32,7 @@
 #include "vtkObjectFactory.h"
 
 
-vtkCxxRevisionMacro(vtkLineRepresentation, "$Revision: 1.8 $");
+vtkCxxRevisionMacro(vtkLineRepresentation, "$Revision: 1.13 $");
 vtkStandardNewMacro(vtkLineRepresentation);
 
 vtkCxxSetObjectMacro(vtkLineRepresentation,HandleRepresentation,vtkPointHandleRepresentation3D);
@@ -84,6 +84,14 @@ vtkLineRepresentation::vtkLineRepresentation()
   // Set up the initial properties
   this->CreateDefaultProperties();
 
+  // Pass the initial properties to the actors.
+  this->Handle[0]->SetProperty(this->EndPointProperty);
+  this->Point1Representation->SetProperty(this->EndPointProperty);
+  this->Handle[1]->SetProperty(this->EndPointProperty);
+  this->Point2Representation->SetProperty(this->EndPointProperty);
+  this->LineHandleRepresentation->SetProperty(this->EndPointProperty);
+  this->LineActor->SetProperty(this->LineProperty);
+
   // Define the point coordinates
   double bounds[6];
   bounds[0] = -0.5;
@@ -276,6 +284,16 @@ void vtkLineRepresentation::SetPoint2DisplayPosition(double x[3])
   this->Point2Representation->SetWorldPosition(p);
 }
 
+//----------------------------------------------------------------------
+void vtkLineRepresentation::SetRenderer(vtkRenderer* ren)
+{
+  this->HandleRepresentation->SetRenderer(ren);
+  this->Point1Representation->SetRenderer(ren);
+  this->Point2Representation->SetRenderer(ren);
+  this->LineHandleRepresentation->SetRenderer(ren);
+  this->Superclass::SetRenderer(ren);
+}
+
 //----------------------------------------------------------------------
 void vtkLineRepresentation::StartWidgetInteraction(double e[2])
 {
@@ -444,6 +462,7 @@ void vtkLineRepresentation::PlaceWidget(double bds[6])
 
   // Position the handles at the end of the lines
   this->Placed = 1;
+  this->ValidPick = 1;
   this->BuildRepresentation();
 }
 
@@ -794,6 +813,25 @@ void vtkLineRepresentation::PrintSelf(ostream& os, vtkIndent indent)
     os << indent << "Selected Line Property: (none)\n";
     }
 
+  if ( this->EndPointProperty )
+    {
+    os << indent << "End Point Property: " << this->EndPointProperty << "\n";
+    }
+  else
+    {
+    os << indent << "End Point Property: (none)\n";
+    }
+  if ( this->SelectedEndPointProperty )
+    {
+    os << indent << "Selected End Point Property: " << this->SelectedEndPointProperty << "\n";
+    }
+  else
+    {
+    os << indent << "Selected End Point Property: (none)\n";
+    }
+
+  os << indent << "Tolerance: " << this->Tolerance << "\n";
+
   os << indent << "Constrain To Bounds: "
      << (this->ClampToBounds ? "On\n" : "Off\n");
 
@@ -808,6 +846,21 @@ void vtkLineRepresentation::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "Point 2: (" << pt2[0] << ", "
                                << pt2[1] << ", "
                                << pt2[2] << ")\n";
+
+  os << indent << "Point1 Representation: ";
+  this->Point1Representation->PrintSelf(os,indent.GetNextIndent());
+  
+  os << indent << "Point2 Representation: ";
+  this->Point2Representation->PrintSelf(os,indent.GetNextIndent());
+  
+  os << indent << "Line Handle Representation: ";
+  this->LineHandleRepresentation->PrintSelf(os,indent.GetNextIndent());
+  
+  os << indent << "Representation State: " << this->RepresentationState << "\n";
+  
+  // this->InteractionState is printed in superclass
+  // this is commented to avoid PrintSelf errors
 }
 
 
+
