@@ -41,7 +41,7 @@
 #include "vtkBezierContourLineInterpolator.h"
 #include "vtkOpenGL.h"
 
-vtkCxxRevisionMacro(vtkOrientedGlyphContourRepresentation, "$Revision: 1.15 $");
+vtkCxxRevisionMacro(vtkOrientedGlyphContourRepresentation, "$Revision: 1.18 $");
 vtkStandardNewMacro(vtkOrientedGlyphContourRepresentation);
 
 //----------------------------------------------------------------------
@@ -373,7 +373,9 @@ void vtkOrientedGlyphContourRepresentation::Translate(double eventPos[2])
   displayPos[1] = eventPos[1] + this->InteractionOffset[1];
   
   double worldPos[3];
-  double worldOrient[9];
+  double worldOrient[9] = {1.0,0.0,0.0,
+                           0.0,1.0,0.0,
+                           0.0,0.0,1.0};
   if ( this->PointPlacer->ComputeWorldPosition(this->Renderer,
                                                displayPos, ref, worldPos,
                                                worldOrient ) )
@@ -615,6 +617,7 @@ void vtkOrientedGlyphContourRepresentation::GetActors(vtkPropCollection *pc)
 {
   this->Actor->GetActors(pc);
   this->ActiveActor->GetActors(pc);
+  this->LinesActor->GetActors(pc);
 }
 
 //----------------------------------------------------------------------
@@ -622,6 +625,7 @@ void vtkOrientedGlyphContourRepresentation::ReleaseGraphicsResources(vtkWindow *
 {
   this->Actor->ReleaseGraphicsResources(win);
   this->ActiveActor->ReleaseGraphicsResources(win);
+  this->LinesActor->ReleaseGraphicsResources(win);
 }
 
 //----------------------------------------------------------------------
@@ -721,4 +725,34 @@ void vtkOrientedGlyphContourRepresentation::PrintSelf(ostream& os,
   //Superclass typedef defined in vtkTypeMacro() found in vtkSetGet.h
   this->Superclass::PrintSelf(os,indent);
   
+  os << indent << "Always On Top: " 
+     << (this->AlwaysOnTop ? "On\n" : "Off\n");
+
+  if ( this->Property )
+    {
+    os << indent << "Property: " << this->Property << "\n";
+    }
+  else
+    {
+    os << indent << "Property: (none)\n";
+    }
+
+  if ( this->ActiveProperty )
+    {
+    os << indent << "Active Property: " << this->ActiveProperty << "\n";
+    }
+  else
+    {
+    os << indent << "Active Property: (none)\n";
+    }
+
+  if ( this->LinesProperty )
+    {
+    os << indent << "Lines Property: " << this->LinesProperty << "\n";
+    }
+  else
+    {
+    os << indent << "Lines Property: (none)\n";
+    }
+
 }
