@@ -40,7 +40,7 @@
 #include "vtkFocalPlanePointPlacer.h"
 #include "vtkBezierContourLineInterpolator.h"
 
-vtkCxxRevisionMacro(vtkOrientedGlyphFocalPlaneContourRepresentation, "$Revision: 1.5 $");
+vtkCxxRevisionMacro(vtkOrientedGlyphFocalPlaneContourRepresentation, "$Revision: 1.7 $");
 vtkStandardNewMacro(vtkOrientedGlyphFocalPlaneContourRepresentation);
 
 //----------------------------------------------------------------------
@@ -739,6 +739,7 @@ void vtkOrientedGlyphFocalPlaneContourRepresentation::GetActors2D(vtkPropCollect
 {
   this->Actor->GetActors2D(pc);
   this->ActiveActor->GetActors2D(pc);
+  this->LinesActor->GetActors2D(pc);
 }
 
 //----------------------------------------------------------------------
@@ -746,6 +747,7 @@ void vtkOrientedGlyphFocalPlaneContourRepresentation::ReleaseGraphicsResources(v
 {
   this->Actor->ReleaseGraphicsResources(win);
   this->ActiveActor->ReleaseGraphicsResources(win);
+  this->LinesActor->ReleaseGraphicsResources(win);
 }
 
 //----------------------------------------------------------------------
@@ -823,5 +825,32 @@ void vtkOrientedGlyphFocalPlaneContourRepresentation::PrintSelf(
   
   os << indent << "InteractionOffset: (" << this->InteractionOffset[0] 
     << "," << this->InteractionOffset[1] << ")" << endl;
+
+  if ( this->Property )
+    {
+    os << indent << "Property: " << this->Property << "\n";
+    }
+  else
+    {
+    os << indent << "Property: (none)\n";
+    }
+
+  if ( this->ActiveProperty )
+    {
+    os << indent << "Active Property: " << this->ActiveProperty << "\n";
+    }
+  else
+    {
+    os << indent << "Active Property: (none)\n";
+    }
+
+  if ( this->LinesProperty )
+    {
+    os << indent << "Lines Property: " << this->LinesProperty << "\n";
+    }
+  else
+    {
+    os << indent << "Lines Property: (none)\n";
+    }
 }
 
