@@ -31,7 +31,10 @@ static int class_has_new = 0;
 void use_hints(FILE *fp)
 {
   int  i;
-  
+
+  fprintf(fp,"    if(temp%i)\n",MAX_ARGS);
+  fprintf(fp,"      {\n");
+
   switch (currentFunction->ReturnType % 0x1000)
     {
     case 0x301:
@@ -111,6 +114,13 @@ void use_hints(FILE *fp)
     case 0x31A: case 0x31B: case 0x31C: case 0x315: case 0x316:
       break;
     }
+
+  fprintf(fp,"      }\n");
+  fprintf(fp,"    else\n");
+  fprintf(fp,"      {\n");
+  fprintf(fp,"      return Py_BuildValue((char*)\"\");\n");
+  fprintf(fp,"      }\n");
+
   return;
 }
 
@@ -775,6 +785,7 @@ static const char *quote_string(const char *comment, int maxlen)
 void outputFunction2(FILE *fp, FileInfo *data)
 {
   int i, j, k, is_static, is_vtkobject, fnum, occ, backnum, goto_used;
+  int all_legacy;
   FunctionInfo *theFunc;
   FunctionInfo *backFunc;
 
@@ -825,8 +836,9 @@ void outputFunction2(FILE *fp, FileInfo *data)
       {
       fprintf(fp,"\n");
 
-      /* check whether all signatures are static methods */
+      /* check whether all signatures are static methods or legacy */
       is_static = 1;
+      all_legacy = 1;
       for (occ = fnum; occ < numberOfWrappedFunctions; occ++)
         {
         /* is it the same name */
@@ -838,9 +850,19 @@ void outputFunction2(FILE *fp, FileInfo *data)
             {
             is_static = 0;
             }
+
+          /* check for legacy */
+          if (!wrappedFunctions[occ]->IsLegacy)
+            {
+            all_legacy = 0;
+            }
           }
         }
-        
+
+      if(all_legacy)
+        {
+        fprintf(fp,"#if !defined(VTK_LEGACY_REMOVE)\n");
+        }
       fprintf(fp,"static PyObject *Py%s_%s(PyObject *%s, PyObject *args)\n",
               data->ClassName,currentFunction->Name,
               (is_static ? "" : "self"));
@@ -864,7 +886,7 @@ void outputFunction2(FILE *fp, FileInfo *data)
 
           currentFunction = wrappedFunctions[occ];
 
-          if(currentFunction->IsLegacy)
+          if(currentFunction->IsLegacy && !all_legacy)
             {
             fprintf(fp,"#if defined(VTK_LEGACY_REMOVE)\n");
 
@@ -1097,13 +1119,17 @@ void outputFunction2(FILE *fp, FileInfo *data)
             {
             fprintf(fp," break%d:\n",occ);
             }
-          if(currentFunction->IsLegacy)
+          if(currentFunction->IsLegacy && !all_legacy)
             {
             fprintf(fp,"#endif\n");
             }
           }
         }
       fprintf(fp,"  return NULL;\n}\n");
+      if(all_legacy)
+        {
+        fprintf(fp,"#endif\n");
+        }
       fprintf(fp,"\n");
 
       /* clear all occurances of this method from further consideration */
@@ -1316,17 +1342,17 @@ static void create_class_doc(FILE *fp, FileInfo *data)
 
   if (data->SeeAlso)
     {
-    char *dup, *tok;
+    char *sdup, *tok;
     
     fprintf(fp,"  \"See Also:\\n\\n");
-    dup = strdup(data->SeeAlso);
-    tok = strtok(dup," ");
+    sdup = strdup(data->SeeAlso);
+    tok = strtok(sdup," ");
     while (tok)
       {
       fprintf(fp," %s",quote_string(tok,120));
       tok = strtok(NULL," ");
       }
-    free(dup);
+    free(sdup);
     fprintf(fp,"\\n\",\n");
     }
 
